(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    863271,      12419]*)
(*NotebookOutlinePosition[    863988,      12444]*)
(*  CellTagsIndexPosition[    863944,      12440]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  "Reading FITS images into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "Title"],

Cell["Roberto G. Abraham", "Author"],

Cell["\<\
Institute of Astronomy
University of Cambridge
Madingley Road, Cambridge
CB3 OHA  United Kingdom\
\>", "Address"],

Cell[TextData[{
  "Practical image processing now seems possible with Version 4 of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " (which now includes packed arrays, and has vastly improved numerical \
performance and storage characteristics). At least one commercial quality \
software add-on package for image processing in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will soon become available.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " may well prove useful for analysing astronomical CCD data.  However the \
vast majority of professional astronomical images are distributed in FITS (",
  StyleBox["Flexible Image Transport System",
    FontSlant->"Italic"],
  ") format, as opposed to more common image formats such as GIF or JPEG.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " cannot directly handle FITS images with the Import[] function.    I have \
therefore written a package which allows FITS data to be read into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".   This package uses ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " to call the publicly available NASA CFITSIO library. "
}], "Abstract"],

Cell[CellGroupData[{

Cell["Instructions", "SectionFirst"],

Cell[TextData[{
  StyleBox["FITS ",
    FontSlant->"Italic"],
  StyleBox["is a data format based upon the notion of a \"header data unit\" \
(HDU).  Each set of data (which can be a one-dimensional list, a two \
dimensional image, a multi-dimensional data cube, or a table of data in \
binary or ascii format) is preceeded by a number of ASCII header lines giving \
useful information (such as the dimensionality of the data, the origin of the \
data, format for the data, etc.). A single FITS file may contain several \
HDUs. The present version of the ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Mathematica",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" FITS reader is designed to work only with two-dimensional \
images.  More information on the FITS standard is given in Appendix II.",
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[CellGroupData[{

Cell["Setup", "Subsection"],

Cell[TextData[{
  "I am using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " on Windows 98/NT systems, and have thus pre-compiled the FITS reader for \
use with Windows. If you are using a non-Windows  operating system you will \
need to compile the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " source files and link these against the freely available CFITSIO library. \
 The CFITSIO library exists for Windows, many flavours of Unix, VMS, and \
Macintosh.  I expect building the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " code and linking it against CFITSIO is straightforward for almost any \
architecture. The source code for the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " and glue routines is given in Appendix I.  Information on downloading the \
CFITSIO library is given in Appendix II.\n\nAssuming you want to use the \
pre-compiled Windows binary, all you need to do is modify one line in the \
\"Fits.m\" package file to indicate the location of the executible file on \
your system. ",
  StyleBox[" ",
    FontSlant->"Italic"],
  StyleBox["Open tbe \"Fits.m\" package file, and change the following line \
to correspond to the correct location of the fitsread.exe binary on your \
machine:\n\n",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Install[\"C:\\\\Mathematica FITS Reader\\\\fitsread.exe\"];",
    FontFamily->"Courier"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Usage", "Subsection"],

Cell[CellGroupData[{

Cell["Functions", "Subsubsection"],

Cell[TextData[{
  "The Fits.m package exports 3 functions:\n\n",
  StyleBox["FitsRead[] - Reads a FITS file into a ",
    FontFamily->"Courier"],
  StyleBox["Mathematica",
    FontFamily->"Courier",
    FontSlant->"Italic"],
  StyleBox[" Real array.\nFitsHeader[] - Extracts a FITS header as a \
newline-delimited string.\nFitsHeaderToRules[] - Converts a FITS header \
string into a set of rules for convenient manipulation.",
    FontFamily->"Courier"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Reading images", "Subsubsection"],

Cell[TextData[{
  "Fits files are read as follows:\n\n",
  StyleBox["image = FitsRead[filename]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\n\nAn interesting thing to note is that the filename specification can \
include a template specifying a subset of the image. This is very useful for \
quickly reading small subsets of enormous images (the ",
  StyleBox["FitsRead[]",
    FontFamily->"Courier"],
  " routine jumps to the subset in the data file and does not waste memory). \
The template format is as follows:\n\n",
  StyleBox["<filename>[xlow:xhigh,ylow:yhigh]",
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  "\n\nIn other words, to read a 100x100 \"postage stamp\" subset from the \
files \"foo.fits\" beginning at pixel x=50,y=100 you would use the following \
syntax:\n\n",
  StyleBox["im = FitsRead[\"foo.fits[50:150,100:200\"];",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\n\nMultiple header data units can also be handled. The syntax for this \
is:\n\n",
  StyleBox["<filename>[hdu_number][xlow:xhigh,ylow:yhigh]",
    FontFamily->"Courier",
    FontSlant->"Italic"],
  "\n\nFor more information on this capability (and other features) see the \
documentation for the CFITSIO library (linked to in Appendix II)."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examining and manipulating image headers", "Subsubsection"],

Cell[TextData[{
  "The header from a FITS file can be examined as follows:\n\n",
  StyleBox["header = FitsHeader[filename]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\n\nThis extracts the header as a long string, broken into lines known as \
FITS \"cards\". To extract individual header entries or to manipulate the \
header information, it's more convenient to convert this into a set of rules:\
\n\n",
  StyleBox["rules = FitsHeader[filename] // FitsHeaderToRules",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\n\nNote that this routine will change some of FITS keywords from \
\"illegal\" ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " symbols to ones that are acceptable to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". For example, keywords with underscores and minus signs will be \
transformed into corresponding symbols with the illegal characters omitted.  \
In very pathological cases this may lead to problems. For example, if your \
FITS file has keywords \"FOO_BAR\", \"FOO-BAR\" and \"FOOBAR\", then you will \
end up with multiple rules for the FOOBAR symbol. \n\nThis routine also \
attempts to convert keyword values from the Fortran-esque notation often used \
in FITS files to a valid ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " number. For example, the routine will convert from -1.2E11 to  ",
  Cell[BoxData[
      \(TraditionalForm\`\(-1.2\)\ \[Times]10\^11\)]],
  ".  I have tested this conversion carefully and I believe the conversion \
works for all valid Fortran-formatted numbers, but please check this on your \
own data as I would not be surprised if very pathological cases do not \
convert properly.\n"
}], "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["An example", "Section"],

Cell[BoxData[
    \(<< "\<C:\\Mathematica FITS Reader\\Fits.m\>"\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
Read in and display an entire FITS image. Reading a 313x313 image takes \
around 7 seconds on a rather slow P166 machine.\
\>", "Text"],

Cell[BoxData[
    \(\(im = FitsRead["\<C:\\Windows\\Desktop\\N2715_lr.fit\>"];\)\)], "Input",\

  CellLabel->"In[2]:="],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListDensityPlot[im, Mesh \[Rule] False, 
      PlotRange \[Rule] {1000, 3000}]\)], "Input",
  CellLabel->"In[3]:="],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% DensityGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0192308 0.00307201 0.0192308 0.00307201 [
[.01923 -0.0125 -3 -9 ]
[.01923 -0.0125 3 0 ]
[.17283 -0.0125 -6 -9 ]
[.17283 -0.0125 6 0 ]
[.32643 -0.0125 -9 -9 ]
[.32643 -0.0125 9 0 ]
[.48003 -0.0125 -9 -9 ]
[.48003 -0.0125 9 0 ]
[.63363 -0.0125 -9 -9 ]
[.63363 -0.0125 9 0 ]
[.78723 -0.0125 -9 -9 ]
[.78723 -0.0125 9 0 ]
[.94083 -0.0125 -9 -9 ]
[.94083 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01923 -6 -4.5 ]
[-0.0125 .01923 0 4.5 ]
[-0.0125 .17283 -12 -4.5 ]
[-0.0125 .17283 0 4.5 ]
[-0.0125 .32643 -18 -4.5 ]
[-0.0125 .32643 0 4.5 ]
[-0.0125 .48003 -18 -4.5 ]
[-0.0125 .48003 0 4.5 ]
[-0.0125 .63363 -18 -4.5 ]
[-0.0125 .63363 0 4.5 ]
[-0.0125 .78723 -18 -4.5 ]
[-0.0125 .78723 0 4.5 ]
[-0.0125 .94083 -18 -4.5 ]
[-0.0125 .94083 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 1 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01923 0 m
.01923 .00625 L
s
[(0)] .01923 -0.0125 0 1 Mshowa
.17283 0 m
.17283 .00625 L
s
[(50)] .17283 -0.0125 0 1 Mshowa
.32643 0 m
.32643 .00625 L
s
[(100)] .32643 -0.0125 0 1 Mshowa
.48003 0 m
.48003 .00625 L
s
[(150)] .48003 -0.0125 0 1 Mshowa
.63363 0 m
.63363 .00625 L
s
[(200)] .63363 -0.0125 0 1 Mshowa
.78723 0 m
.78723 .00625 L
s
[(250)] .78723 -0.0125 0 1 Mshowa
.94083 0 m
.94083 .00625 L
s
[(300)] .94083 -0.0125 0 1 Mshowa
.125 Mabswid
.04995 0 m
.04995 .00375 L
s
.08067 0 m
.08067 .00375 L
s
.11139 0 m
.11139 .00375 L
s
.14211 0 m
.14211 .00375 L
s
.20355 0 m
.20355 .00375 L
s
.23427 0 m
.23427 .00375 L
s
.26499 0 m
.26499 .00375 L
s
.29571 0 m
.29571 .00375 L
s
.35715 0 m
.35715 .00375 L
s
.38787 0 m
.38787 .00375 L
s
.41859 0 m
.41859 .00375 L
s
.44931 0 m
.44931 .00375 L
s
.51075 0 m
.51075 .00375 L
s
.54147 0 m
.54147 .00375 L
s
.57219 0 m
.57219 .00375 L
s
.60291 0 m
.60291 .00375 L
s
.66435 0 m
.66435 .00375 L
s
.69507 0 m
.69507 .00375 L
s
.72579 0 m
.72579 .00375 L
s
.75651 0 m
.75651 .00375 L
s
.81795 0 m
.81795 .00375 L
s
.84867 0 m
.84867 .00375 L
s
.87939 0 m
.87939 .00375 L
s
.91011 0 m
.91011 .00375 L
s
.97155 0 m
.97155 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01923 m
.00625 .01923 L
s
[(0)] -0.0125 .01923 1 0 Mshowa
0 .17283 m
.00625 .17283 L
s
[(50)] -0.0125 .17283 1 0 Mshowa
0 .32643 m
.00625 .32643 L
s
[(100)] -0.0125 .32643 1 0 Mshowa
0 .48003 m
.00625 .48003 L
s
[(150)] -0.0125 .48003 1 0 Mshowa
0 .63363 m
.00625 .63363 L
s
[(200)] -0.0125 .63363 1 0 Mshowa
0 .78723 m
.00625 .78723 L
s
[(250)] -0.0125 .78723 1 0 Mshowa
0 .94083 m
.00625 .94083 L
s
[(300)] -0.0125 .94083 1 0 Mshowa
.125 Mabswid
0 .04995 m
.00375 .04995 L
s
0 .08067 m
.00375 .08067 L
s
0 .11139 m
.00375 .11139 L
s
0 .14211 m
.00375 .14211 L
s
0 .20355 m
.00375 .20355 L
s
0 .23427 m
.00375 .23427 L
s
0 .26499 m
.00375 .26499 L
s
0 .29571 m
.00375 .29571 L
s
0 .35715 m
.00375 .35715 L
s
0 .38787 m
.00375 .38787 L
s
0 .41859 m
.00375 .41859 L
s
0 .44931 m
.00375 .44931 L
s
0 .51075 m
.00375 .51075 L
s
0 .54147 m
.00375 .54147 L
s
0 .57219 m
.00375 .57219 L
s
0 .60291 m
.00375 .60291 L
s
0 .66435 m
.00375 .66435 L
s
0 .69507 m
.00375 .69507 L
s
0 .72579 m
.00375 .72579 L
s
0 .75651 m
.00375 .75651 L
s
0 .81795 m
.00375 .81795 L
s
0 .84867 m
.00375 .84867 L
s
0 .87939 m
.00375 .87939 L
s
0 .91011 m
.00375 .91011 L
s
0 .97155 m
.00375 .97155 L
s
.25 Mabswid
0 0 m
0 1 L
s
.01923 .99375 m
.01923 1 L
s
.17283 .99375 m
.17283 1 L
s
.32643 .99375 m
.32643 1 L
s
.48003 .99375 m
.48003 1 L
s
.63363 .99375 m
.63363 1 L
s
.78723 .99375 m
.78723 1 L
s
.94083 .99375 m
.94083 1 L
s
.125 Mabswid
.04995 .99625 m
.04995 1 L
s
.08067 .99625 m
.08067 1 L
s
.11139 .99625 m
.11139 1 L
s
.14211 .99625 m
.14211 1 L
s
.20355 .99625 m
.20355 1 L
s
.23427 .99625 m
.23427 1 L
s
.26499 .99625 m
.26499 1 L
s
.29571 .99625 m
.29571 1 L
s
.35715 .99625 m
.35715 1 L
s
.38787 .99625 m
.38787 1 L
s
.41859 .99625 m
.41859 1 L
s
.44931 .99625 m
.44931 1 L
s
.51075 .99625 m
.51075 1 L
s
.54147 .99625 m
.54147 1 L
s
.57219 .99625 m
.57219 1 L
s
.60291 .99625 m
.60291 1 L
s
.66435 .99625 m
.66435 1 L
s
.69507 .99625 m
.69507 1 L
s
.72579 .99625 m
.72579 1 L
s
.75651 .99625 m
.75651 1 L
s
.81795 .99625 m
.81795 1 L
s
.84867 .99625 m
.84867 1 L
s
.87939 .99625 m
.87939 1 L
s
.91011 .99625 m
.91011 1 L
s
.97155 .99625 m
.97155 1 L
s
.25 Mabswid
0 1 m
1 1 L
s
.99375 .01923 m
1 .01923 L
s
.99375 .17283 m
1 .17283 L
s
.99375 .32643 m
1 .32643 L
s
.99375 .48003 m
1 .48003 L
s
.99375 .63363 m
1 .63363 L
s
.99375 .78723 m
1 .78723 L
s
.99375 .94083 m
1 .94083 L
s
.125 Mabswid
.99625 .04995 m
1 .04995 L
s
.99625 .08067 m
1 .08067 L
s
.99625 .11139 m
1 .11139 L
s
.99625 .14211 m
1 .14211 L
s
.99625 .20355 m
1 .20355 L
s
.99625 .23427 m
1 .23427 L
s
.99625 .26499 m
1 .26499 L
s
.99625 .29571 m
1 .29571 L
s
.99625 .35715 m
1 .35715 L
s
.99625 .38787 m
1 .38787 L
s
.99625 .41859 m
1 .41859 L
s
.99625 .44931 m
1 .44931 L
s
.99625 .51075 m
1 .51075 L
s
.99625 .54147 m
1 .54147 L
s
.99625 .57219 m
1 .57219 L
s
.99625 .60291 m
1 .60291 L
s
.99625 .66435 m
1 .66435 L
s
.99625 .69507 m
1 .69507 L
s
.99625 .72579 m
1 .72579 L
s
.99625 .75651 m
1 .75651 L
s
.99625 .81795 m
1 .81795 L
s
.99625 .84867 m
1 .84867 L
s
.99625 .87939 m
1 .87939 L
s
.99625 .91011 m
1 .91011 L
s
.99625 .97155 m
1 .97155 L
s
.25 Mabswid
1 0 m
1 1 L
s
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
.01923 .01923 translate
.96154 .96154 scale
313 string
313 313 8 [313 0 0 313 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2E2C322C372C3033312E2F32313231333030342D362E303032332F3330312E322D2F312F
30312F312C2E3031312E2F2F312F312E3230313135312D2E2D2E322F303131312D2D2F2E
2E322E302F3331312D2C34342F2D302E2D31302D333232312A2F3030322E2D2C2E312D2E
302D2C2E2E2D2F2C3034312F3430363330332D312F362F2D302F312E2C2D322F3330312C
302E30313030312D2F2E302F2E312D322F2F312F3233333232332E2C2D312F34332C3132
3031352F3633312F30332F2E302D3031312C312F2F2B2E2F2F2E2F312C2F302F302E2F30
342F31332D2C2F33322F32302D2F2D2E30322D302E2E2D2C313132332D35312F302F3031
332C302E2D3630303232302F36312F2E312D3031313131312C3032302F302E2E2F313032
2F2F2E302F2E2D2F312F2B2E313230312E2C2D31322F323132
332D2E30332F30342F3135333031343130303231312F30302E2F322F3033312F2F2C3430
33303030323231312C3030312F322F322E312D35332A2F2F302D2D2D302A2F2E3131332E
2D3132312F2F302F312E31352F313430302E32302F313234312D302E31303230342E2E2D
323030362F31352F2B3033322D2F312F302D33332A2F3033322D2B33322D30302F2D3131
303131322C332F302A30322F302F32372E302E332E2F32312D2C362E3534333132302D2E
31343132323231363131313032312E302E2C312F302F2F2F2F2E302D31323235322F2F30
302F33302E2D2B313132352E302E2E2E302F2D302E302F303233333333302D2B2C312E2E
2D2F30322E312F322F3131312D3233312E343031302F2A33302F322E2D2E323131302E30
31302F2E30343035302F2B3131312E2C2D2F32312C2F2C3431
312D322F332A3031323731302F3232302F3135312D32302C2F3433322B2E3031312F3133
3030352F2A322F2E2D2D2C312B2D2E3130302E2F322E312E372E2F30312E2F322F333433
2C31312E302D30302B2D2F2C2E30322D32302C2D3130302E302C2C2F3130272F2F332D2F
312F2E2F332D2D332D332F2E302B302C343131313130323332303030322C302E2E302D30
31323232302C2C2E2E31302F31313230333032332D3531302F30322F2E312F3330333031
332F313332333030303031333132312B3132312F3430313030302E34302D302A2D2C3131
322A30343131293132322F302E322D322D322C2D2D3030332D2E323230312D302B35312B
303230312F2E312D303130302F3034302F362F302D2F2E322C2E312F302D2E312F2F2E34
313135332F342F32303231303033302F2F30332D312E2E342D
3029302E322D3234332E35352D302D32342D30313234312D2D302E312F30313132302E30
302C332E2B312D30303031302C2C2F3230333130312E2E332D2F3330312F2F2D34342F2D
31302E33303230312F2F312E313033322F312F30302D312E30322F323030312D322D3230
32332F32342F2C303230312C2D34312E2F2E2E2E313131313433332E2E343032312F3032
2C312E323133302E3330302E2F2F2C3431312D322B2E30312F302D3631312F3230313231
31373232362E332F2F2E312E2F322D332B3233312D30322D3130302E2F312C30312C3130
2E2F2F312F32342E322E2F2E322C32292E332C30322E302F2E332E3032312C312D303133
32322D2C2C2F362E323330302F3032322F322E2F30312D2F2F312F302D3332312E303031
312D322F30312E312F3330302E3030312B302C302D2D322F2F
2E333035332E3230353236303030312E3531342F2D2F31333031302E2D342F30302F3030
2F2D2F2F2D2C322C342D312E302F30332C3430312D2D2D313334322D3030313030353332
33312E2F3330332F322E2B322E303130332F2E32323131313231322E2F2D31312E322F2F
2C2E3232312F312D2B28312C332F332E2E2F2A312F342C302F2C35302B312F332E2F2E30
2E2F2E3036312F2E32302C2F2E2D302E2F302F3230332E363230303534332B2C2E303536
3136353034312C3333303131303130302E2E2B302C2E313033302E332E302E322E323234
2F30312E312F312F313534312E2F2C2D2F2E302C322F2D2E2B2C31303230322D312C2D31
2F2F302D2E31303231332E2B2B302C312D2C2E34302D312B2F2D30302E32303431303430
31322E302E2E302D30313132312E30323331323332332D3030
2E31323031303330333135313235322D2F303235342E312F31303033333030342F2B2E31
2F2E2E34302E2F302F2E31312C2E312C2D312F3331322B302E332D3131322E3036323131
33323031322F312A2D3131312E312F312F2E2F2E322F2F2D30312F2E2E2F2F2E2C2E2C33
2C343130322D302C322B302F2F2D302E2E303030312F3032312F302F2E2F2C31322D2F2C
2D2D2F2F312F312A2D2E2F312C312D302E3232302D323132322D2F302E2E2E2D31312F32
32333530322F322D2D2E312E302E31322E302F323333323034342B322C2E2D312D2E3430
342E2A2D3030302E2F313030313330352F302E312E32312E30302B322D34342B3131322F
2F30372F2E2D3032303331313231332C322F30303130302F3130312F2E33323130312C32
313030302E2D323334322F2F32302E32322C30312E302F332F
3231362F35312E3036312F33332D31332F302D322F33332E3431323133302F2F3330312C
342F2D2E2F312F312F34312E31312F2D343230302D33322D2F2F3231303132362F312C2F
2A32342F2E312F302F312F302D332D302C312F32302E2D2E2D322F2D2F333033302E332D
2C302D2C2C2C2E302E2F2E312C342F302F2D2F2D33312F2D312E2F2E3132332F34322E31
2E2F2F2F2F2F2D3032322F2B342E31302E32303133313031322E313231313231332D3231
302D303133342D3130312E3230322F2E2F323236353030313132302D302C322E33303634
332E2F30332F2F34342F2E31313031312E2E32302E312E302E2D303131322F312E2E2C2C
302F2C2E30342E31302F322F322E302F3231322D2E30323132332D3031322E3132322E32
2C2F3130312F342F2F342E2C302F3036343530313131312C2E
2C3333302C3332303230342D303430303132302F32313230332F32302D2F333131332F2F
322E2E352F342A302C2D2F2D333133302E3129312E2E302F302F2C33303331332F2E2F33
30302E342D31322E312E2D2E332B2E3033303231302B2D302E2E2E30302B2C313033322F
2F302D2E342C2E2D2F302D2E322E312E2D2E30312C30312F2E2E2D2F323230333632312E
2E2B332F2E322E312C32302E3631323231332D2E32313030303231323133313431332E2D
302E3131312D332B2F2E342C2F2D30302F322F303031342F30332F2D33343130332C3233
333332322F31303131302F2A313033332D313132302F312C2F30312F302D312E2E2E2A2E
2C312D2F312D30312D3132352E3230332E302F31303130302E302D30302E2F2E30303233
322E342D2D2F2F3132322F34303135302E2E2F2D2E2E312E2C
3133302F31312F31322E2F2F302F302E333332333135322E3430302D2F323032322E3334
3331302E322F2F2E312F34322F2F3231312E2C332E322D2D31302E332C303032332F3232
2E2F31342E2F2F3130312C292F2E2E2F2F332D322F2E2D343031342F2F3331302D2C2D2E
312F3032342E2F2F342E302C322C2B302B2E302D30302F2F30362E342F3333332D302E2C
332F2F302F312F312F312F3233312F3132322E32332F30312D30302F312B33322E2C302F
3031313030303033322D2D2F302E2F312F332E2D2D30352C30342F2E2E302D2E30313232
302F2F3530322F3230302D2E31302D2D34342F2E302E302F2E312B2D2D31342E2E302E2E
2F31312E2D2F2E2F2D2F2F3131312E323030302F332D302E322D3530322D2E312F313132
3230312F2D2F2C332E313133312F31002E2E2D2C2C352F2E2F
332D3030313233302F2E312F302F3233303234322F3034332F31342D2D32323233303331
3030312E31302B2F3231302E332E30302E332E322F322D302F2E2E2D2D302E2D2F312C2E
2F312D342E3330302F2E302C2E2E2D2F2D2E323132303335312E30302E3234312D322E30
343236302E2F2D32322E2C2E32302F31302C2F2F2F31302F322F31342C323031302F2F31
303634323033312C322D2E2B302B2D30312B2E2F313031302E322E31312D322D342E332F
303131313330322E2F302F302F2E313132302D2E322B332E2D2E2F313333322D312E322E
3134322E2E302D3133302C302E2C3031323133332E2E2F2E302F2D322F34312C322D2C32
2E2F2E33312F2F3133302F323134312F2F302E30322D2F302D2E313333312F30312D302D
30312E30312E33312D333332302F302F2E2C2F2F2E302E3231
3632302E31322E323034322D2F2F302F313230323033332F2D332E2F31302D33302F2F2E
2F2B2D313030302B2C2F312E302E322E332A302E322E2E3132322D302E30312D2F2B3133
31342D2E31302E30352C302F2E322F323030302930312B2F2D31302E2D34323230323130
302F2F33302E33352F353631312F322D2E303032302F37352C33322F302F2E2E322E3130
3330322C3432312E2E2D302E31332D2F31302D352C30322E323532302F2F32302D323332
2F3232332E32352F2F2C32312C302F2E2D2F322E303132352F30302D2E302D34322E3131
33332E34312F2D312E322E2F31322E2F3330303631322E31312C2E31312D32312E313232
3032312D342E2C2F2F302F2D322C2F3233322E2E2B2E2D30333333362B322E3230323130
2D302E3330312F3135312E3134302D2F31323032322B302E2E
30332E30313430322E322F31333235302E3231302F2F2F2E302B30322E32352E2F312F2F
2D30312C322D2F2E2E312E323434302E2F302C2D303031332F30312E2C30342F2F313031
322F30312F3031332E2F2E3232312F342E353331332C323132322A3131362E322C332D33
3132302D2A2F302D30323031332B34302E32322B312D3232332F2D2F322C3133342E312E
3032313333313230332F30332E3131312F2E2C3030332E2D322E322F30332E3130362D31
303431312D313335333132302E2E2E2F302D312C302F2D323230332E30342E3230323033
2D322E2C30302D332C2E312F30342E3033322D2D2D302F2D302E2E2E2C2E322F2F2E2F31
3131322C2D30303132303332323130342F2D30312F2E2E302E323130302F2F302D32302D
2F3030302D31322D31302E2E2F3331362D30313031302E2C30
30322F2E3430332E2D30312C2F303031313134302D34342F312C3333342E3132332E3337
2C2F312C312D32332C322D2E2E2E2C2C322E2B2F302D3331312B33322F312D2F322B322F
302C3632302F3232303131302D2C312C342F303330332B30312E2F322E2E303430323333
30332C31302C312D323032312D342E333130312F302B303135382E312F2E312F31342F2F
312E312D31312B2E312F2D3231342B31313230332E2E312E2C323330333234352F35332F
3435313032353131313236332C2F2F2F2E302A2F302F2F322F302F2D30302F362F2E2F30
32342C2D2E2F2F36313032343231342E3232332F312C2E2E2B2F2C2E3130322F352F3032
303230302F332D302D2D313435303031322F313031302D2F2E2E323131302F332D2E3031
31302C2C302A323131322E353233303733312F2F312C2F2F32
302F2F32333031323030362C35332C2F2E313032303034313033312E302E2D2E2F332D2C
2E3431302A302C2F31302F352E322D2E2D2E2F343230342F32312E302F2E2B2E32343130
302F2F2F30322F3532302F2F302C2F2E322B312E2D34322D32322C312F2E2E30342F302F
2C342F322B30302F33332D2B2D3132312F2D2F31352E31323032312F302F2E322E302E2E
332E2C312E2C323130303437313131302F2F33362F30302C303133322F2F2F2E3132302C
31303232322F3330303135303132302E31302F303132322E2C2E33302F31312B3130312D
302D3030312C312F2F2B2E2F2F2E2F312C2F302F302E2F3034302D35302F2D3037313532
332E302E2E2F3030302C352F342D302D3033302F312F2E3031302E312F2E33302D333031
2C2D2E2C2F2F312F32322E2E35312F2F2E312B3430322D302E
303232323130313230302D3534333036342E2F2C332A322D312B2F322F332D2C32313232
322F322C322D302F322D2E312D3030302F31323030312F2F2F30322E333231342F33302E
302F3330303031313031322D302D2D2C2F34342E302E322F3332322D2C2F332F2F343131
3131322E343131332B2D3133332C2D30312E2B3330302F2F3230303031313232312C2F32
31302F2E2E332A332E3133332E2F3231342E2F322F313131322E2F332E33303130302F31
3235312E3031313034302F2F2D2C2F2E2F2E2E31352F2E30302F302F2D2D2F302D323130
32312E302E2C312F302F2F2F2F2E302D31323235322F2F30302F30362E32313330312F32
2F2F2E312F2E2D2E3131332F312E2D2F332D34322A2E322F3130302F303333332D2F3132
302E2F342E2D2B3432323132302E312F2E2F30303031302E30
30313232312F31372F2D2F33312E3033332F3130302E352F30302F2D312F32352C302E33
323031303333302F302F3032322E2C2F34332D332E312E2F322C2F31302F2F2E302D332F
332C2E302F2E2F312E322F322D3533323330302D2E2E343334313133302E2F322E2B332F
33313032302F30352E2E2D2B2F33342D322B312D2F313334322F302F2E2F313030312E2E
2E30313333333232332E2C2D312F34332C31323031352F3633312F353335313334303034
2F30322D3133312E2F35312E2F2D323131302F2F31322F313230302D2E2C2F3331303033
3132312B3132312F3430313030302E34302D302A2D2C313132312E3232312F342E2F3130
33312E2F2F3131312E312E302F2D2F30303133313231322F312D312C30313332332D2C31
2E3031322E2F332C2E31353230302F322F3235342C34333434
353331343234322E32302F3230303232322F2F31332C2E2F312E34363132303431323333
3233302F302F2E2E2F322D2F312F2D2F2E302F322D3331352D33312F352D30302E323030
333331302E2D2F352F34312C33363033363030303231312F3031322E2E34313032303034
32302F30343032322F312F2E313130302F2C2F312E333233322E312D332F2F2F332F2931
2E302F2D2F32312D2C362E3534333132302D2E313431323232313631313230352D343135
2E32352E2F332E343130332E3131312F3230312F3131313030312E312F31322F2E2E312E
2F322D332B3233312D30322D3130302E2F312C30312C31302E30312E333032342F323030
302A302F2E3131302F312E2F2F2D313234322D2F2F302C2F2F2D2B2E31302F312F333130
302E312F322E2E2F2C3231322E2D2D3130332D3134302E2E31
302F302D322F37342D33342E35303331363336362E2E2D303233323132323731312E3332
3231332F312C2E2F332F302B2F31322F2E322D2F2F31322C33323130312D2D3431313331
2E342F2F313033302E322D3231343231322F303132342E303135302F312E303432312D33
312C342F31322E32312F2E302F2B303333332C352F352E323832302F35303129322D302F
2D302F2D3531302F30322F2E312F3330333031332F31333233303033343030302F32312F
323533322E2D31323231322E2F2C2D312F313132313233323131302F312F2D33322F3130
303030302E2E2B302C2E313033302E332E302E322E3232342F3033313132303131312F30
2F3130303230312D2C32302E31312C2E332E322F312D31312F2C2E303231322D32322D31
32302C303031302D2F2F2F302D322E32343130313333313231
322E3233312F2E32363132333130312E3231372F34312E2F352F3231312F2D3130302F30
32303333323032312B322F312E33302F30312D312C2C322C2F323231342E2F302E322D32
312D302E303031312E2F342F2E32322F2F2F2D322E32322B34303130342D322E332E2E31
312F3230333031313131303131302F2E2F2F302D31353230353031342E3330332F2F2F33
3030332B2E30312F302D3631312F323031323131373232362E332F312E32342F30302D34
3434353133332D2F3133312E33322F2F32332E3032343030303330312E332C302D2E312E
302E31322E302F323333323034342B322C2E2D312D2E34303434323534323230322C2C32
3331332F2E342C2F323030302F3030322E30322E3031302C292C2D333030322F32313334
2F323331313033303231332E302E313033342F332E2F30302F
2D30303533363133343C32313333303130302F2F333336303431323033373131342F342F
3230303432332C2F2D2F342F322E3230312F3532302F2C332B2E2F302E32333231303132
312B2F3233332F3030303230312D363031322F302E2F332D312E30322E2F33302E2F312F
2D2D3331312E3030343130302E3331313432312D2D3031312E30312D2E2F2F2C3132352F
352F3330332E363230303534332B2C2E3035363136353034312C332D312C3235332F2E2E
303331322F3231303231312E3434303030322D302F322E3130363134303031322F312E32
30322F2E2F323236353030313132302D302C322E33303634333230323434353533343131
2D332F323230302E2F30332E31302E322E2E2F2F2F323234342F2E2D2D332F2E3133302E
333332332F2E2D2D3134303333322F302D2E2F30312E312D2E
32342E32303030303733342F2D332B2F30342F3331313133323131303134353533302F34
3135323130302E3130312F332D30312F302F31302D302D312D2E3032332D30332F322E33
2F322F312D31303035312F342F302F322C2F3133322F302F33332F303132323033313030
2E2D302D2D3030332F2E312F31322D322B312F2C312D2D2F36322E31302F2F332E32332F
3130332D323132322D2F302E2E2E2D31312F3232333530322F322D302F302E2E32303131
3033312F3334323132333134323031312F2E2C3131313033322D302E322E2C312F2E342C
2F2D30302F322F303031342F30332F2D33343130332C32333333312E34352F2D34302E31
30302F34332E3030322E2C2E3031312F2E2D2E2E352D3030322E2D322E312E2E31313030
342F3031302F2E2E2D32312F2F2F322E2E302F2E322D2F2F32
33313432332D32323131313433333236313531332F33312F2D3233302F2E333231313730
33362E2F3233312F2E2E332D2F302D3130342D2E3330312E34312D2F31302F302F313131
313031313031302A30302F2D303131313230312B313332322F2F32322F302E2E312F3430
2E2B302C312E32312E30312F312F2F332F3031302E2E32332C302E3131332E312E302E32
2F2F3133313031322E313231313231332D3231302D303133342D31323231333030323131
31313232313330333332323433342E2F32332E2E2C2E2D302F303133312C312E322D2D2F
302E2F312F332E2D2D31352C30342F2E2E302D2E30313232303034303031332F34333031
3232302F3033313730312E2D3230302E333132332B322F2C2F322D30322E312C35333431
33302F2D302F2D2D303030332F2E3233302F332D2B3033312E
31303432342F303632352E332F31353130322F333136313331323233332E323132323034
2E2D2F32313233313333332D313333302F312D2E2E30302F322E2E312F2D2C2C2E313032
313132302F2E2E312E2F2B30302E32302C3134312E31303334322B31302F362F2F33312E
31302F2F342E2C2E32302E3232302B312F2E32322E30312F3032332E2F2E3332302F3230
2F313432313030303231323133313431332E2D302E3131312D332B32312F303032312D2D
2E2D322F31312E322F3130313034322F312F313230302F2D30333231302E33312F2F2F30
2F2E313132302D2E322B332E2E2F2F313333322D312E322E313432343730363035313233
2C332D32322F30302D2E3132322D2E31323032312D32302E322C2D2F3030332E2F2E3131
31333431312A2E312F333232332F32332E32302D302F33302F
2F2F33323330343031323238323233313232343632323730302E3034312E302F342D3034
342B333432303034333331312F30312F2A2F2F30342F292A352D2B2F2D322E303130302F
2F332D332B2E32313132322D313130323233312F31302C31323234332E303133322E2F2F
32302F2E30342F3430322F312E3535312E302E2E2E34333135312D353333323233313032
2C322E332F30312D30302F312B33322E2C302F303131303030303330332D303030302D32
323130322F30313530332C313034323030312E31312F3130312E32322E2E2E322F2C3231
2C302F2E2E2F322E303132352F30312D2E302D34322E313133343231312D2F3231333033
2E312D2D2F302D302F2E3031322F31342F34332D343532302F31312E2D30312F2F333131
2E30303332302D31312F322C362D302E2E2E2E30322C2E2F31
2F2E33322C343235323233312F37332D362F3231362E322E3530312D332E2E302E342C2F
33343132332F313332313030303433322E332F312C2E32332D332F332F332D2D2F2E2E30
322D312E302D2F2D2E2E3032323032322D372D31312E342D3430332D2D352F2D362F322E
3132302F303132303030322D31342E33333235302B302D312930373031322C2F32333133
33302D313031302E322E31312D322D342E332F303131313330322E303132323234342E32
322D30332E322D2E2F2C3033333434302E3234303230333331302F302D33332F32313230
2E2E2E2F302D312D302F2D333230332E30342F323032303330323532322F323336312F31
2B312F2D30313034342E32302E2F302F2E2E32323230332E2E3130303033312E33303031
2D303231332F2D2F2F2F3333362F30312F303130303131362F
31343432332E32383133312F2F3235353234333236363031302F34312E312F2F33313433
302F2C3131322B323230302F3432302E302F2D302E303330312D2E2F312C2C30312C2F32
2B2C2F332F2E2B2C31352F31312E2F332E302E2E2F2E323035302F302F3130372F2F2C32
2F2F2F2E3232382D303331312F30333332332F2F2E302E332D31333431332E33342F2F2E
3130342C30322E323532302F2F32302D3233322F3232332E32352F2F323232322F323231
3031302E32322F3232332F342D33373131312E352E30303332312F302C35353230313633
2C2F2F2F2E312A2F302F2F322F30302D30302F362F2E2F31312F2E3033343232332F2D30
312E2D2F2E323130302E3031312E2E31303432342D302E2E2E312D30302E2E2E31303030
31352F3031292D2D2B31383334322E302C2F2F362D2E2F312C
323333373030332E363231323031332C3130322F2F32322F2F2F3235322F362F31323233
2D2D2D32313332332F2F3133362B31322F2F312E313031343031312D30302E322E2F2B32
2C342D302F323134322D33312F3032302F31313432352F32332F332F3430343334332D2F
3130313031302E3132332E313235312E313038302E2F31373332312E302F3430322E3031
2E302E30332E2D322E322F30332E3130362D31303431312D313335332C31323533303232
313133332E2E2F2F3332293330323232343034323031342F3131302F30312F3130333530
3132302E31302F303132322E2C2E33302F31312C313131303033333331303230322E2F2C
2A2E2E2F2B31312D32332F2B312C2C3332322D31303231312E312E2F343033312E33322E
313136332F312E2F3032323132303131322F32312F2E312E32
36322D303333323033323230323332322D303030343132333431303032322E32312E2E30
3333323531303230343230332D3434302F342F352E312E3134342E33343231323231352F
2D2D312F322C2931302E302F3530342C33313131342D2C30322F2D30312E332F332F3335
3132333032302F2D3033312F2D333132323230323332323532302F2E302E32372F2F3030
312E322E2E312E2C323330333233352F35332F34353130323531313131362E37322E322F
2F3030303331302B32303232302E312F303030323330313134312F2F2C2E32313030302F
2D2C2F2E2F2E2E31352F2E303030302F2D2D2F302D32302F342F34312D2E3031302D2E30
2D2B2C2E2E32303131313032302B31312F32342D30333130303132302E302F3233322B32
3532302F3430302D343234373034352F2F34312C2F30302E30
3434333233342D2E30312D303437353035312E32332D33323131303238303331302F3330
303133312F3438312D332E313033323231352E332E2F32312B33352E2B32302E3433302D
322D352E31312D2E2C302F2D3432323330332F342D323432352E31313130313234323230
333130332E302D312D2D31313233322D3135302B2F2F313031313130323438363D393130
3132303130302C303133322F2E2F2E31322F2C31303231322F33303031312F3130372F32
372C3533302E2E31323030332F3333322F3532322F34322C312D312E2E2C312E342F312E
2F2D323131312F2F31332F313330302E2E2D2F33323030342F3631332B2F33322F322E30
302D2D30342F302F30312D2F2C3034382E2F3131322F2F2E3030312F3036312D2E2E322E
303532302F35303431303531302E33302F312E30322F343035
3431303030313332303034303532302F35312E31383132302F3333303130353130333130
2D312B323235302F3030362F30312D3135322F33332D332E30323034312F2D33342E2D34
322F31342B3030302E2F2E31322F302E36303030332F312F302F2E34312D322F322E3130
2E2E3532332F322D3131322E2E363132323431312F2F2F2F31313035303334384A423333
31312E2C313131322E2F332E333030302F2F313235312E3031313034302F353131332E34
333333302F30332F30353130333133362E3030322E332F36313035322B302C332F322F31
3131312F3231312F3231313130312E312F31322F2E30342F3337312F2E32312F31302E31
2C34302F2D2F2B2F2F33312C2E2E2F3231372B2E2D30302E312F302E3231333132323232
322F3133302F2F3031312F3030302D2F323330312A302F2F30
3534322C2F34333232303332342F302C2E3030343130322E3433352B302F303134303631
2F313230313230312F312F302E302F2F302F2F2E31302F372E2F2E3230312F3134313130
303331312F2F2E2E312C2E303234322F312F313233313131322D32333331343632313334
31323432322E322E303032363432312F34302D35312F3031312E372E342D3535423B3330
33313330342F3633312F3533353033343030342E30322D3133312D2F3531323130303130
2F323232313133302E312E3132352C342D31312E323537372F302C2E2F2C323033313433
2E302D31303231323132333231323030312F2D30302C332D322F2E31322C2E2E2F2E3135
2F2F2E2D34343430302F3333293131323330312E3330322C2F2D2F32302E31312F312D2E
323132352E2F302F2E2C2F32313531302F302D32312D2F2D32
3034343034332D34313130333036312E2E2E302F332D30352F2E31323233313530332F31
2F3630313130313031312E2A3332302C2A2D342F31342F312D312E302C372F34302F3230
312E2F312B2F302F31303030313330322E2F322D30332C312E3430343431332F332D2D31
3035302F31303331303332323334303430302E31343330333032342C3131323035302E32
332E2D2E302E3232313631313230352D3431352E32352E2F332D34313031313032353031
30353232322F3234312E3132342F342D31303331312E36343330302D2E2E333032323133
322F342F32332E3032353130303330312E3331302E312F322F302F313331312E32303531
2E322D2D2E2F2E323230312F302E30342D3230312F33322D2B3130312F2E313132303035
2F303433332E30322F3232322E30302F32302D2C312E332D2F
33352E353331342F3033313331322E3030303332312F303231323033363231323231312F
2E342F322F33332F32302E302F2C3331312B30342F2E342D3129302F32302B2E302A2E2E
302F302F34353132312F2D33343030313130383636362E3232322E2D33362E2E342E3434
2F2F332F2E3031313036302F2F2E312C2E34302F2F2F343131343234303134323631302F
2E302D3333333233303033343030302F32302F323533322E2D31323231312C2E302C2F2F
313432323133312F2A33313234323031352E322F3430323134333231332D313133312C30
3231342D30322D302F322F3131373234302F2F3035303032323031303130322D312D2F35
2E3231302C34312D303031333031332F302F322E323030343131302F322E2D3135302E33
2E2F333132323032323131302D31332C313232373031313133
312E3630322E3437333133363330303230323537362E302D322D313030312F3330343232
343231332F31302F322E302F2C2E322C312A2D2D2D2F2D31302D2F352C2E322F2E2D332E
33322F31302F2E322D3331322F2F3030343131343A3832312F2F2E323332352D34332E2D
2F3435312F353231342E312E30312D35302F312F313330323830313033322F312F353533
2F322F312E38322E332F312E32332F30302D333434353133332D2E31313030303230302D
302D332F2E322F2F2D3431303032312D34302D3432322F30323532333033332F30323234
312F313031333330332F30313632313033312E2B31322E3034332F3332312E322E303332
2E32332A3031313330302D312D312B303232312E342E303032303134352D312E332E3032
3330333130353630333330312F3032312F2E3130322F303031
37383532332D313736332E313231332E3430383233343031342C342F3030352D3330332E
332E323632312C332E2F2D2F312F2F2B33292D2F312D302D2A2F332C2C2E2D31312F2E32
342F333032322E2E3332342E33303330303136333C362D3430343232303032312B333133
3133323032312C32303333322D312E362F2F2F2E31303336373331313231313035322F33
30303430332F333330332D312C3234322F2E2E303331322F323133322E323031322E2D34
2E382F2F3330323530333233323131322E31322F312F2E34342D32333130332C30302F31
31322E322D3232332F33342F333032303235302F31303438302F3430322F2C302D2F3330
30322F342F31342F332F32313037302E2A312E363136332E2E342F313230322C31332D33
312F34323334323433342E35303531332D312F33312F313132
30303032323131352B333234312E3436333436333631332E32312D343137312D29302E33
312F332F32342F2F3332312D3530332C2C2E32302C34352F32302D2D323131312F312C33
30322E323231313437332F3335353133313330333132342E2E33303231302E3132343331
3335312E2E2E322F32303231312E30353431322F2C35323732322D3130352F3230312F34
2D332F30313133313532302F302E2E313030312F33312F323431313533322D2F322F3230
312F303030323432333033313130313034302F34322F31333235332F31373033322E3230
342E2F2D3132343531312F2E312F2B322F2E3130342D2D2F2E2B31353034323133302D32
30302E2C323234352D333031322F352E30302F32312F2E31333132322C2D2F2D31312C32
2E2E303135323333313031302D34322D3130342F3230322F2D
37353131362F2F2F343634323432333230323234312F2E2E31302F3031322F2F28313031
323132332B31312C2B322C30362F302D2A3130342A34332E3332323431303131322E3231
323133323430342F302D2F2D2F2F2F2F312F3132332F2C333130302E2F3330302F303032
3133352F303331313130332F32342E342D313434303432373034302E35302F34342E3136
313034303131363133352F3130322F2F323131313032323134352F3232332F30302E3030
343330333131302F363132313130312F32333134323232332F31362F2E34323132322E31
2E323131303131333633312D322E3030322D2F33312F2E302F33332B342D302F312D3231
2B322F3333342E2E3231302F342A33313431332E30313334303438342D332F312F2F2F31
3031332D332C333033363331302F2F3131332F343534333030
3434333031342F2D33302F37312E31323532312B3031362D2F303430362E302E2F343231
302F2F2E2D2E3331332F3134312F3931322E32313135302F2C302F2E2D2E303033333033
2F322F3136332E302F2E2F3430322D373333313431333030303030343331333132343032
2D2F342F3231323436313232332D2F35342F333231313134302D332F30332F342F2D3036
312F36303333323730373430312E31332A30343131342E342F392E32363233313237302E
2D32322F2F33332F322F303632322F38332E2F322F30313434332E31352F2E35322E3130
3230313031313030372D342E302A303032312F2F30332D32332B3033333332353031302E
2D342F2F3231322F3034303230322E2E323030303030302F323434362F332D31302F2F33
303132312F2E3335333031313036342F31322F31332E333130
313134363831302C352F37312F2E30322C352F3530332F322F313035312F322E31323133
2F31312E2D3330312F30322D2E2B30322E312A322B353132322F2D2C2E30332E3431312E
30303032352F33322F322F2F31343031353232333435322F37302F363030333030332E31
2F3231323131322F2F32302E343235303131312E2F2C30322F30352E3333362E2F332F35
2E3031353337352E34323330363134322B30342F302F303331323131323230353236302F
2F2E3132343131322F312E2F31322E302E2F2F332E332F312F313030332F353432303034
31302E302F302D343130303430302F30323131312E3331302F2F2C31313338342F2E3232
2F2D332F342E2F2F312C2E3132313632322E302E31323130343630333132342E312F2E33
332E31312E31323033312F2F30373033312D303131322D3231
323236373131332E313633322F3032323235313132312F31313433303233313132353131
313130322C312F32322E312E2C2F2B2E30322F353130302D2E30302E302D2F3230322F30
332F31312F34372D32322D30322D322C33303132312F2B332D3030322F2E343030323230
2D323432312F32363B33313231332F3430333334322D3135323136353333343132323030
332F3531302E30302E313533332E2D3031322E35332F2E3233323333323130322F333231
312D2C33303333343031332C32313130322C32322F33332F3332312E302E363534303234
303329332C30322F3137323035333130302D302E2E302D312F2F322D322F2E36332E2F32
2E342C32342E31303233302D3133332E3134302E352F2F323133343234332E322E302E2F
2E3532312E3333312C31303335343434332D302C2C30312E30
3033313133322E2F3033333030323431342D323234333630342F3431312D312F2F32312E
2F343031302E2D2E31312D312D2E312C302E2D312E34302E30342F2E302D2D2F2E342D2F
3130312D2E313437313128302D332F332D2D2E32342D303432312B332F2D323332312E30
313330342E33322C2E2F302C31342E3331353230322E2E2F30303230312E33312D313033
2F32323135313332312E332C302F3132323231323334362E2F3032333333333033343234
2F2C2E30302F33332F3131303230323531332E332E2D333134302E33323131312F31322E
3131332E312E2D322D31323131312E302D313232302F2C2D2D312F2E33322F3230322D30
2B2F3230322D33313031312E322D32322E3130342C312F323134333133322F3333333236
302B3330322E3737322E313335303033343130292B33303031
342E34303034333532333132312F322F3231342E32312D323234303435323130302D3031
3132352E2E342E2E3334312D3134302F2F3032302F2E322E302E32312C34303130313132
322F312E2E312F32333430322E312F2E2D3231322E333133312E33312D3130312C2F322D
3531303533342E343133353530322F3134303231333035302E3337363232333231303432
3032342E2E312F2F343030332E332E2E332E363435363332323134333335343332323133
3533313533312F3333312C3230333132302E302E2E313131313230323532323032323332
333034302E2D2C3231322F3130313130322F322E2E312E2E2F2F2D342F2E32322F31322F
303531342E3132302F33322E3531342F31332F322E302E3430383033382E2D312E303433
32342F30332D32362F323131332F2F2F333232302A2E2E322F
342D3034322E3231332F3231302F31353330323132312D312E323132313134322E2D2F30
3230323135323333322E2F2D2D2E37302F35302F312F302C3231323333302F323331322E
2E302E2F2F3232302F2F3130323337332F3031302E31302E2C32332E3435313231313532
30323332322C32343633342F2F2F2F34352F2D3430303234353536343336363333343233
2F31303332313033353333312F3135323531383D413732312F3330303132353333312E32
2F2D30322F36363230312F2F2F352E332D2D302E3132323134303333322F313838323133
303331322B2F31332F322F31342E3430302D312D2B3030302F302E2E2B313233332D302D
312F323035323130312E33312E2F2D2D3333332F3133312F3132333032323131342F2D30
31313136312F32313230322E33362E2F2E2F342E2F31322E34
34303233353231302F312D3131343030312C2E342E33343131342F34333433312E303330
32303231392C343232322F31332E32332F32323030302F32323136312D332F2D2B322E2D
33313130313234362D32302F313133322E34363231322F302F3233302F3530323333342E
3433333431322F2E33312F2F2F2F3136303530353130343031323330353334322F353332
302E2F313430302E312F3030332F302F312E363439373531333133332F332E332E31322F
3131332E32352F34333336312C3336312F322E343030303332302C323430353631323231
2E32322D32332D2F34333432312F32332F31313031302C2E302F302E2F332F3331323231
2D322F303032313532343431302D31303032323031302E332D3530313334312E312F342F
2D2E302E313130333135303234302E323631322F323131322B
3338322F3133333131343131372E3231353031333235322E3630303031362F3231323035
3031313534362F35323132352D32303133313130303133323331312E332E332F2D2B3133
2F3130332D323732322F33332E2C33323536323331342C34352D3536362C2E3132303432
303133322E313233322C323132323433313431312E3233353332312D3032323533343631
343034302F31313233303130322D3231312C333231363331333231303435313330303236
323236322F323132303134333333303233313533303528303231342F342D343236353434
302E3131302F352E323330372D3237322F3030303131322B302D322E2D2F2D3131343230
322F312E32352B312E302C302D30303132342D3231312D32302F3330312F30302F2A322F
322F322C33303136322F2B2F312B313131322F32302E30342F
3333343132313235303331342B2F333731313038323232343331313431323030322C302F
3335323136313133312E30332F2F2E31363532322E3432322E31322F30302E2C31322E34
32322E31313230362D362D313131332D343334313032303430302E31303036343033332D
2F362D3031333033333030322E333031312F373331362C33303532312E2E323434323434
323332362D32333030323231352D343431332E2F2E303333313331313431323330323030
35312F30302F3132313334302E30323133353433302D2D31373333313130303332312F34
3332322B2E2F2F313434313432332F30312F3132312D2B2F32332D312F2F2F2E2F333133
3430312D3332313233382F3132302E3233372C323333343030302F2E2F2C2E3032352F30
322F2D3033322F2E2D2F31322F3330302F35323231312C3130
353131353432333135373231332F34333130363433323134333235323032322E3332302F
30342F323134362E31322E3334363331332E34323234312D32302F2F3030312E352E342B
3130333133332E3035323030362D31362E2F352D3233313035303033303332303232322F
313132333433313633322D313232312B30332F312F2E2D3033302D323631303035323334
3334323136323330322F322E343131312F302F2F31313130333530323233313531333330
3333302D33303330322D333031353030332F37313032312F3233303330322F353132352C
31333330302C2F30302D303631302B323330323030312E3231322E2E33302F30352E3131
34312E2E312E3331322F2D3034322E3134303433312E31313131312E352E302F2E313634
2D2C2F312E2F312F333233312F302F312D3030343134322E30
2D3131343634342F303233303334342F333132353331323632322F332F33322D31342D32
313330353331332E3235323233373330313133313431332E2F313131342F3334322F2F30
312F3331303132353230303334323332332E33333031313133353137323233302E2F3632
342E372F2D33332F322D31302E33322D2F3130323433333034313434353233302F2E3033
36313631333031343531323233322E333033312F31332E34302F33313134333734343333
303033312E2E30322F303132302E303A313232322D352C2F30302F35313534322D34312F
33342E3231312F2F2F323033342C3331333234302E2E313030302F2D2F303430312E3330
2F2D2F2D2A2E33303334323230322D303034312D333431312C322E31313132322D312F31
322F313030343635312F313030312E2F2E2D312F34332E322F
303432302F2F36323434303130322C2F31323031353131362C30302E2E352F312E322E31
3031302C32333134322E2F322D323232322F2D303333342F3133302B322F30322F303033
302C33302E333030353233332C323431302F363031343230363232302F35302F33303430
32342F31313435362F322F3034322B3531302C312D3235302E3234303230333132333234
32353033343531342E35333532322C312B33342F3431303131332F333030343031343434
30312E343030312F3432333233332F30333135322E3431313434313332333131332E3131
33333232322E302F30302E303230312D312C2E302C342F342E31322F312E3132342F322D
2C312E2B2E2E323134332D2F322D2F323235322E3131342F3236323035333132322F3230
2F2F30303335302F2E323230322D302D2B2F32323033303130
343234313132353530353132353934322F2E33322D2F313230342E2D303036322F33312F
2D2F2F342F3130332D322F342F313330302F3231323535322E3432332E302D3335313330
2F302C302F3031313333313033303230313035382E3130343231313031303034322E3333
3331332C303230363333302E2D32302E352E342D2E313232363331313235313334373332
34322B312F30323332333231322E3330322F3234332F353432333030343232333533332F
2F3333322F31303134363531303133302C32342E2F34343131363233383334302D333331
362F323433312E2F30313233312D3331302E302C2F2E302F2C3032323332313135313132
31352F302E312F322E3233312E2D31322F312E322F302E2E2D312F33303031303033322F
312C30333031332E2F3033322F3031312E323131302F2E302F
303035332E312E303A2F33383738353232353033332F322E2F2F34312E36313032313032
313434362F30323032313230313332312A302F33303430303133302E313230312F342D30
3133322D3132352F35363134302E332F3035322E35313231303033353431333230333330
353132333633323031312C3432323130353734313232322F30303432342F323331303333
33323137332D2F35323334323432332D332E363432302F2D35323131303430312B313636
3132342F3030342F2F303032343030303B373835322D2E3230353837343332322E303130
3334342B2E342E30302D30303030323132313030322D313432303231343035323432332F
30362F2E2E2E31292F303130303733342F3131312E322F3131323531333031322E312D31
2E2E2F3131302F312E2E3031312F33312F3032333332313032
3135313433352F31373136393A38323436342E31312C302D2F2F312F322F2F2F332A2D2D
302C2F2E2F31312B3238333530302E30322F3031343532313034323032353131322E2D30
3233302F32322F32312F35322E2F3433303635333230323434322E33342F2F3033323330
2F33322E3230322F342F35352F3231313430313230322E32342F31323330313233332D31
323231322E31312F333136323231342F32313831342D32312F2E33303130303130313235
312E2F31313030322F2F302C3033333331343331333230333139333236343431312E3332
2E333231353433322B33302C313430312C3730302D3130302E302F313232343331312E32
3335332D34303033333331363233323033343230303031312F2F3030332D2F3433342E31
302E3232303135302D2E2E343230302D313131353231332C30
31323132323034303236323A403B313330352E30313030353033313333372D323133312D
2E30303232353430322E3030322F2E2D30312C30333133353430342B32362F2F3133352C
322F30302F3132303632332E3030342D3530302E2E31322F323133343131303333353036
3731312D363334313231323434313431323030332F3333302F3636343234303332313234
303130352F353231362F343335313234333133343437322E332D362E323230322F333231
3331313534323232303135312E3133322F333431312F3131302E2F36343535322F2D3333
322F33323133322D312F303333323832323333322D312F313031332F2D30343037303231
302F313134303330312F313031333234303030322D322D303133332F312C30332F2D3133
332F312D30323136333132313134332E302E34333133323230
312D343530343431303530343737322E3033322B332D352E343430313531353332303430
32313034322D2F302E322F3134302E31332E2C2F2F313335363331302D302D322B342F2E
3231353034323134303236302F2F34322F3133333131343233322C3231303130392E3331
343434332F3133313431323331353332343337322F323135303034333734323632323233
303130303330313431342F392F32313433303031323037333137312F31313031322E3331
303232303230312D2D3134313433322D30302F33343032313334313635313431372F3030
373230353531322F313032352F3431303330352F332D3332303334373333303232302E31
3333333231332E31332B2F3231333132312F3333322D2E2F31302F3333312D3132303231
303133343032332F3234333030312E3132313632322F302F2E
312F2F2F323034332E303133323133303332333332323130323034303032333233303133
2D312D352D3231302E393032332F30322E3030333231323430323132362C323232322F35
2D35332E31343037312F312E302E302E2E2E3434322F30313230353032352C3633313330
39343932343032323432333333302E363733302E2F31302F3232352F3030302F36303032
323730342F3430353430303035312E352C30323736353230333434353331372F2D32332F
353631323230322F2F2F30312E3339343631313333353434313430323532313234333031
33353036323231323437332F30312F342B303034313033322D2C37323034323232313133
3331313532312C322C352E303132322E30303233342F332F37302A2B31302F322D343031
3232322F312F33322F2E3130323431313030303233312E3131
33342F3532313330333033333331343030343335333133312F332E3330372F3336363034
2F2F313030312D322C31312E312F33312D2E3131322C303333303030302E2D3534363033
2F33323534372A33323234313534303232353230332E32312F3531362F312F2E2F313431
353333333334352E30323132343132303133332E342E3032303230343230333334373230
2F3034372E2F322E3333333133332E31332E3733323032342E2E3130312B333630323031
31333533343231322E3130303431333330333132312F323236343234323232302F2F3435
3333312D323435303634373734322D303030303031342F3234332E3234352F2D2F313230
322F313032332F3034302F32332E2F2E2C33312F2F30312F303132313030302D302D3230
37322D34303232303430303033322C2E30312F2E333132352D
33323231313230312E34332C353532303732312C373131302A3032303136303233323037
312C342B3133302F2B2F342C322E322F2E2D3333302C323231303132313230312E323234
35313431322E322E31302F2E32312D3131303235343135342F3032363431343433302F33
363335323333302D34343332323332323030322C2E32362D342F34343135373036342D31
2E313330333736333536322E313030313030353534323236373131333430333236303732
323131322E30323031313033303537303431313333343330373535353736353436313233
322F333436313633343B3F36332E3231313130312F302F31373630323132313031353135
3233333534302D2E3132313231312E30323232323033372D322B312F3630302F302E3732
33302E2E36362E312E313233323232333230323130362F3131
39303034342F31313034342F333331322F3233352D342C353332332F2C312F2E31303430
3630312E32303130302E2C2E2F3132322B2D33312E332F2D33352F32302E2E3130323336
363134302F30303030362F363333323330302F34342E31313632372E3532332E34333637
34343131302F333132303132322F313231312E3335333333323038333031383330343235
3034322F322F2F2E352B3135362F30343530313134322E32333432313533323032323134
323032352D352E313334383234343430323230333734373035363636333632342F323135
35333434343334353638343134342F30353030373334332F362F31332D31353034333033
3330332F2B302F2F313230312F2D2D313230353233352F3232322E2F3130302D32322F31
32333131362F33323333343130313231343033302D3332342B
332D3330343535342F3336313230343331303632352E3230323233302D302E3430302F2F
333233323632353130332F2F30302E2E3231352D352E3634342E322D312F2D3232373533
373135333032302F2F3133302D3635312E2E2F3430343131343234333031343733343633
3432323535333331302D352F3331343031333430322E3233313131323633313531313136
362F35313936323130312D302F34332F3434333335323231383231303431323232342E36
34333332353230343433352F313331323237363834323632383535353335333635343335
363433373332353633363331343232342E2F33303235313034313135332F343534333233
3233342F2C3231302E3231342F2F3230322E322C32303332322F303132322C2C32322C31
2F2D33313532302D312D32343231342C2E3331313530313032
3530342C3332353031333634322E343331333131333135353334323434312D3137323234
2F333333343130302E323132322F31302F30332F2C2F30342F2E332C30323032362E3536
2F3033333332303635302F2F3831323134333431302C2F2E342A37302F34323031343334
303337382F3430382E2F2F30343131322F352F2C2C322D332F382F343632323131383731
2F3635322E333537333030373333303234303536363334363731333130312F3032353035
3132342E332F353533373432333335343233333232313037343336373734323630333134
3636332F3635323832333031313536322E3235342F31322F35323232322D323630313430
33302B3332353234343236353433312E333331333033342D2F3332333233323130313133
2D2E2E2F32343130312F3231333232303034342E3030323230
313530302F313730353236342C35363432313132312E31342C3134363730302F2F323334
343037363232303233323030302E333430363033333030322F353031332F333630323933
33312B32302F3233333232343436342F32302D32302B36323630313031302E3131303237
3235343836363330313136373431323333323332333433362E353132343337363A362F34
2E322F323232322F30313333323133312F312E3432333334323434353330313135333132
2D34313131333834363135323231363432363030333436343636383635363436363A3734
383136323336323636343533342F3033313232342E34323332302D3331353233322F3136
313331313334303332302E36342F34302D2E353433322F302C32352F2E322E313532332F
2C362F33322F332E3232302D33302E2D302D302D322F312F32
33322D30323435312F3034333332353132313531343032353431323337313332332E2E34
2C2F2D32333133352E3430322F3131313330303432342D33322F3234332D35373133312E
3330312F34342F332E30333031343034333432322E2E36333231333135312E2E32302E35
30343830323132353335323332323331332F31313031342E323432353637323333303631
3332342F342D3333313330312F2F32363631333131343332313230353135333234313033
322F3433323235333633323C33343232303634353434303435353436383633352D353235
343337393436363836373032322F332C333131303131322F32343632342F353433363432
2F30323431353431302F302F2F2F33312D333630332F313031312C2E2E32353033342E31
2E2E2F2E33312E3030302E2F33373834333232333031333431
372E36333533323430343034323032323131353133363433303633332F313235312D2F34
313031322F2D322D332F2E2E32312E3632323036302F303230363233312C2F2E302C2F36
2F313331352E33323231343033332E3432303333323130342E3230333333313331313532
33333134302F2F3032333431322F2E313330312F33343134313233353530333836352D33
3232363330352F3033373432303330313134333536353533323334343534322F34353135
32323336343034333334323332323C3630343632333635323439363D3337393232333632
333732373632363933353631323134343134323432332E3236303432312F2E2E31322F33
2E2E3331353131342F302F32332E3233343332323330302F31312F303131333235363334
33323130332E303630302E2F342F3136353131322F31343133
3032312F3435333334323334303031333234303032343338303130342D362F31342F302F
33313031352E322F2D2E322D3130303330302E2E35323531312F2F3038312D312F343231
312E31332C322D332F303438343235322D3433303434313035333534352D2D2F312F3031
313035323231313134332F33363032343232313331302E323335323431362F3234362E37
3133313432332F3433343231303434333334303633323432313134343834363436333237
333533323336313532333739323B34353537363535373531353D3A38383D343836383635
38393433343E3333363332303333342F323130313631343233313330343433333032362F
30353632332E32322F312F312F31312F2E373032313230343131302E312E333039393731
323031303332302F3231332F3331323234322F332F2E2F2E32
32323434313036322E30362F332F2F2E343031352E33333730333833322E34313435332F
33313231322F30342D302F302F342E2D31312D313231323532302E32303432313033322C
333533332E352F2E322E323031323334323230322F2D342F2E362F323033333032313734
33333132312F30313633353035323234303635323134302D353232333637363530353532
30333131323231333230313532322E393431323334352F2F353330353234333138342F33
3234323130332F2E2E3237313135333736333A383C353635333B3D383538383534333936
373D363834363639323132333132312E333237333335303632303630342E333332332D34
31332F322D322F3633313129342D30313130322E2F303231303230313330333238363530
2F2E303131323231302D303136313033343031322E2F2F312F
2F3332363030363330333133322D312F3033343333342E3234343233312E323231313431
3230363133303130343333332F31323232312D352D332E2D312E2F2F3030332F30323632
3133313730302D312F33313631302F2F3631333430362F342D2E32343430313435343034
3330332E31313032312F3633352F3331313232353834302D2E302F2F2E352D3336332F33
3230313132342D3231303332303D3436333932333630363135353336323632343236372F
373133333233343534323233312F34363437383D3D393C333538383C3838393A3C373A38
3639343B3637323836353736303736323035333034333537323031343134303432323133
303232302E322E30302D303036312F30322F2F323230323432323132323130303531342E
333230333633312E2F2D2F303030333335343030332D313130
30343335383434363133312E31362F2F322E343332353131313530313033323433323438
31312E32302F2F2F30303530312F2E2E2E303530333232322B2E352F2C322F3030343138
2E3333312D3337313130342E342F303135363232302F33312F2C3132323132332E343232
30332F2D34312F35303630322F2E3330322E34323331312E2E3038312B37323335353032
3234343132343430313433353236333332372F3133313436333234333430353333333431
35343433343230303435333434333334363839383C39373836383740383E393839373937
33363B37393737383431323331303133303634313533363B3432333331313330312D3435
32363231332D343230342F323432303334303532332E32332E2F3333333331322F343032
313032312F343232302E2E322D36353131342E312D31312E2F
34313131332E3031343534302F3130302E32343430323633302F2E32332F303131323034
332E32313133342F31302C302E3330322F2E2C363130312F312E342D35302E2E322E3331
3330312E2E312D3134323136302D36342E3030333031312C2F353432313430333634352E
30353332323433332D32322D342D3031312F343333342C3031373033302F35352F313130
343234323934333332323431362E3434323130312F323132353435353331313131313537
38303836353736373035313535353C38393438383840373A383B3C3E3A3C3B3938393B38
343638333639373339323532343238363332323634323632373432323130332F2E322F30
2F3230322E2D2E3132322D313030312F322E31352F3433322E333333313031312F333132
312D3230313037342C3232322E33313335332E35332F313230
2E32333338343533322F31332D3332352F3032342B30312F303233303533302E33303332
2F353030313031312D322D302D31313434312B30313435342F31303334343230322E2D35
2F3433353233312F2F34303630362E35343231342D322F2F323232313032313332322F33
2F33352B303034322E3532342D313431363230312E362D352D3234343633332F352F3734
32333133323133323036343333302D353131313234333330343637333732333532313536
33323033373330373535363336373737353837363B3B39383736363B3B3B3B3E37363639
333A3837363438373338343233323331353531363431363632303635302D3332312E3230
323331313330302F2E30313631352F30333031332E33343530323433312F333333333230
313230323433332F312D34323131363231333230323531302F
31312F342F363431313431302E3131303130323132322E31302F2D32322B3531322E3434
3434302F3034302F302E2F332F34302F3331313131312D332F3032352B2E32332F2C2F32
30313134323330342D34323431353233312F32312F312E30323034333230323332323430
373132303233323333333135303632303432313535302E3232322E302E31303132353130
3231363531322F3231333133333335353332343436313432343E443C3531313832333533
35353533333539353437353738343B3435373D37383A3B393E3C393C3C3E3C393B3B3736
3835363237373534343834323835353832353232333334313032362F3332313036303235
332D313230312C312C34322F3532343034313332323230363533333735332F302F342E34
322F312E332F323632322F3233313333323533353133313230
2F303434312F352F302D3334342F30333233343430312E30322E31313231343633312F2D
34333431322F322D2D2E2D3233313132333731323331332F31323431332F2E3431323133
303434363234313531323430323032303035333732333231333030323030363632313333
3130303332342F3430312F30313432333734343835323330363233343433313232323631
362F3033313730312F33363634332E323533323134333531333236353134323336383436
333335373739343436393738383939393B38393B3E373D413C3A3D3E393B413F3B393939
3B3537363433363833353536363432353734313432353334343337313132312E34323331
2E31302F31333032312D2F3531322E2F30323332312E3131312F3234322F3230322E2E32
313533343232302F3032332E34313434302F332D2D2D32302A
3436343333343734343332352F312F3032353132303530312C312D313231323132302D30
302F31303333312E2F2E2C2F333133332D322B333033363632323431303431302E313033
3632313830333630353231352F3532313233303131333131333333333130312F31313233
312F312F3130333134343334332F33352E34303031332F3332302D2F2E35332D32362D31
2E31302F3333372F34323431343535333233353130323633342F34373331333236333437
373437363739343539353B38373536383C3E3D383F3D3F3D3B3D3D3E3B3D3D3E3C393C37
373731343638363338373634343533383236303134353633303230343033333434323130
32342F3132313432323032303234373033332E2F32322F303336323236342D2F2E34312F
343132303130302F333330322F2E3033323231302D3334302F
30343236352E332E303030343130322C2D3031352E32302E322D30312E302F352D303032
312E31313431342B313030302E343332302F3330302C2F3231343438372F3334382D2E33
33363331333331322E35322E3135363232303731323733332F2E33303032313333303430
2F2D343434333034323535342F333431332E3336322D30313433333430332F333033302F
2D2F333238323238363735303333333431333137332F3532383134333834353635313434
3832353835383A383737383B3A3A3A383C3F393B40403E4139403B3D3D3D3E3C3D39343D
373B373834303633343435343333343530353130303331363335353433362E3938302E33
3033312F3331302E2C3035333233312D33303031332F323234322D312F322F333233312E
352E302E35303130363133362E322F32333132323232313235
3230343236363330313135323037322F312E30312D32302F32333034312D2F3132343430
31312F323230312E35362C302F3235313133332E30312E353231322D2F3733363333302E
332F30303332323234322F3132362F343233322E323031322F333232312F2E2D32303434
32383230353230333431323634323132333433332F3133313431342F312D313031323133
2D312D2D33343338382D353035323632303533313932343532323334353A333231353634
3A38393938393B3C373B393A363E393D3D433D3E3D3B403D3D453B3C3B3C39393E383A37
393734353434363835333531323834373531313233373232323432362F30363A38333330
35322E3230323030322F373333323332313330302D333030313230312F3132322E30302E
2D313332312E30333434343232312F3330303331312E323431
312C2F3538353331353134302F3432353237312C2F312F343031313231313130312A3232
353030302E31342D322F35332F313334323336323531322F363231323334312F32333233
31353531303230382E322D2C2D3232303031322F3233353034303431303330313034302E
2F303236322E2D33333230343339313233333331312E32312F3131362F30303236313132
30313232313330312F332F30313333333534333535353733333832343334373234333536
36343E3F3C3A3A3D3C3E3E3C3E3D3C403C3E3E403E403E3F4143433F403C3B3F3B3C3C34
38353634343336333134313434313332343331332E2F2F333337352F3331323833353030
3131323031362F312E2D2C2D38313534363435303132322E322F303132322E3332333032
3131323331323631352E333531302F312F333232363230302E
3630353133343234323A382D322E3033303335332F2D303631302F332F2F303332322B2F
31362F2E313335312F302E33312F2F333137343133352F342D3430333034313538333233
33363432313231313434322F332D3334323134323232343231312E2F31313330332C3330
2F3033303631312F31332F32312F342F303031323534342E303033323532353933342F33
3135383332312D3234323235323137353531363235313236323635353136363233353736
3B39363E393D3F393D3F3940423F413F423F413F40403E423B42423E3D393B393A3B363A
3536333334303036333236333035343632353132352F352D3535333230333037322F3332
343134302C3130302F35323532353630373331342F2D322E31313233342D31323634342F
313333342F323131382E312F30322F2E302F2E343531333330
34322D3231323130343636353634312E3235353231302B3230312E33323231342C353131
2D332F3230303233313231313034323434353235303534302D3030323131303533342F33
30362E342F3032312F2F31332E2F2F2F34303130313333313233313331312F3330323331
2F332E32302D2E362F313133323331343331353331342C30313332363435342F30323337
30323533322F2F312F312F30373436313337323234343235313337323839373439353A37
36393637393A3A3D42443F3E3E404044414144414440403B3D434240413E3D3B383C3636
3435373530353636383330323633363736352F343234302F32313732323030342E362E30
33343530342F322E2F312E35333531343331313132312F31302D2F2F2F2D303130313031
2E2F312F34323132302F3131312F34302F30313132332E332F
36343034303633352D343334363C30312F2F302F3733302E2E33303234333237312E352F
2C3236302E31333033352E322E3032313335343433363133322E3534302D353333333032
363132303034343137312F3333312D332E3333343331352E32303431312E2E322E303030
31342F333534333432312F3332373034322F36372D37353532343732312D31313433302E
313434343333353533363432333039383430313433353930353635343034363735373635
393A3A3A383C3F404242414142424242464A4743403D403D3B40433C393D3B3E3A3C3937
37353437353634343734353333333033323335333036333133302F2D3531302F33303033
31313330353433342E3331363332323631322F2F2E342E352F3230313532333233303131
3131332E3231323130303332333232312F2F35303232343230
35333330313236333032333432333838322F302F36362F2F30313432342F323433332E2F
3231302F3134302E312F3330323032313232313334333136322E2C332F302F3135333533
32313334323331303331312D312D2E313431312F313030333132313131303530342D332C
322E2E33332E3230363231342E32313533353133333435303531302F3331313432303131
32303731323034332E35353335343232353632303833363635333533363736353936393B
3D383A3B3B3D424244494744484445454748444340423E3D3E403D3A4039393935313A37
3C3630343532313138393638353532303535313039303336342D2F2D3235343036313631
2E3133332F302E3532353133312F302D3131313231302F313035313033303131332D2D31
312F33302E323432363132343133332F323331342F2F2D3130
37303733313133322F343432313534373735313432333130342D3034343136332E313530
362F3234312F2D302E322C3531302D322E3236312F332D36373337313433333233333632
34322E332D33313431302C2F332C3131343232302C30302F3032312E3234302E2C31312F
33312F353231353333333535333330312E34373633323231313232323131313230333235
312F33303431343136393032323233333832353135333531383830323535383538393A38
3C3F3F3B3C3C41444B4F4E4F484A4945484449494341443D3D3C3B3E393E3A3438373633
343938353437363537393433333133363536373333333534322F31313332343533323636
2C3334313431303230313630323331313234333131313131332D2F323236302F2D333431
3333303032343031343238373231312C32332F2E3132313034
2E313031363134372E33323135332C323537333431313234312E30352E34313031323233
333231303231332F313131353231313234313334363332323337313234362E332F383733
31312D342F33363034332C32322E332F30313131313132322E3133303131302E30342B30
3231352F2E30312B3031303332323432313133303333342F342F3431333231323233322F
33333333313334323335343337333133343434313235353633353136373534383C39373F
3B3F39413C444549535D61585651534E4949424243403E3B403C3F403D39373C34383838
39333533353432333636353336363336323131373533353534333130343331323332342B
303333322F342D2F3733362F3031322E342C302E30333032323532323331342F3131302D
3433313230393336323235363232333130352F3331332B302E
302F35363631302D33313231313334343537353933332F33322F2D313130323233333032
32323132343234302E3132302F323133322F362F352F302F323533332D34303134333336
33363131343230363233363030322D30313130313332303330302F3032353333312D3234
3032312E32333330313030343131302E2C34323134303331333234343430353234343531
34352B3232323433322F34373533333634343334343632333535383639383439393C3F40
3C3F3C4144404657575C63645E555349484B4644413D413E3A3D373A3B3939383A3A3936
3533333335333233333231363534333038363031343630353530353431302F3734322F35
302F32302E333334312F2E333034312D3632303531343132302E32323433303033323234
3230312F313030343233333133303334332D323633352F3130
3230312E353535333330322F34312E303631303A35323530302F302E3530343235332F32
3434323335353136313034363331323232303030343334332E32303033312D3133313236
332F313332353533343132332F31313133313132323033333336343431302F3431343032
313334313031333133303632353032322F353332373331333331353233342D3132323533
33353433303332303337303436343134343735333232373738363B3739363B3A3D3B3C42
403E424345494F5757585D60645F5955534E464241403B3F383E39353B3B343339353833
35393337353033323335363237333333353436313630323531332F323031342C312F3530
3132342F322D2E3430333032332D312F353133323034302E343331302F302F3530333236
2E30302E3131322C303232313134303232312F2F2F31303434
3033313730353133342E2D303435343030313336343933302F3233303130303536352F32
32353335373236342E31332E3031322F313132332D303332333131363230322B33313436
323035323032303332313132313031303031313231323433353537333330342D31303331
302E30333032313533303131353432313330353733353536313332353532323437333533
343035333634363236343331352C35353232363539373735353735383939383C383D4242
42443F45454D555D5E595A605D62685E514B4342433C3E393B3A36373838383632363839
3637363734332F32343337312D35343335353432323635323137352D3334343034323133
2F34302E3130302D33302F2D3232312B302E3331332F30353534312F33393234342F3034
362E2C302E323234313334332E31333132313333302E2E3233
3332352F30303337303631353435313130313336353531302E38302E3131312F34302E34
32303437303431352F323030313232323234313233313236343333333535323033333032
2E333432302F33333033312F312F3031323132303330342D333532332F302F3735303831
3434332D323234313134342E3432322D332E353434313033303433353535333337303636
33372F332E2E3232333531332E363335343337333538343834353436373D3C3E423F4042
4542494C505A797C5C5455606162635B4F4B433F3C403C3E393D3A3C3B34363236353735
3737343832313532332E303230342E323333313335363233343033302D30303530343335
312F303235343132313330332F3031312E3134333434333032332E2E323032302D323330
2D2F32322D3034332C3332302E353230323631303031313632
34323030332F34333431323035333335323430313435353530312F312D30303136322E32
343134333232342E31313030322F3332302F35363432343234393632312F36302F333130
3034322F30312F333234342E3435353630313033323431332F313532362F343134303632
3530323135332F31323530303134313131373130322E332F343534343433343733353736
3333322E3531332F3832363237393135362E3336363636333A373639393E3D3D44423F46
484A4C4E506083825F51545C59575B534947453F3D3C3C3A3C383A353833383534363836
373636353435343332312E35322F343433353137373432352E3033333331333234303030
323131352C312F3030322E3233302E33332F2D3531352F34303033322D3331352F34302E
322E3432313532323131303133312E2F332F32333234323030
34323135323634333231353436323037312F303335322C3431362F31312F353033363333
3031373331313733332F2F33323533313431313132323436313431333334353432313736
342C352D2D3337323330353130362F312F2E3234323032313332302F3231333133343031
313034342F3436313331372F312E332D3031353333323335352D3333322F323134383535
2C313531313034313434313530373138323835373736383C3734373B3B3B3F3E464E4747
474D4D52505E6C6655505052504E4C484543403F413B3D383B3C333B3B3B363531333230
3334343435342E36382F34323432342E3436333133333136323730353231343033363135
34312E36312E2E2E33302F312E2D332F322F2E33333331303231313230332F36322E3332
2D2F2F2F33323031323032342F322F323331312B32302D3031
313231333232383531302E323333302E313530313039342E3533312C3330332F322F3534
363235333333323131332F2F343332333336323032313631333133313132323130323437
333332322F2E3034322E2E2E2F2F3335303537333632343233323231342E34322E312F35
2F3531323330342F342F35332C3032323133343032323235332F2F3A342F353134342E31
30333134333032373432363731352F3A33353938373337353538363A3B3A3E414A50504F
4E4D535453575A5F524E4949474A48444142413D433C3A393838393D3736383633353339
383734363438363333313332333137343732323537333534342F34313437363130323237
39302E30302E31332F332E2E2E2E2E2E2F31313231353430363432323030303232322F33
2F32342E3030313130303335332E35332F31322B302E30322F
3332322C352F3233323133303734372F333032343330333335362E2F303333322E323435
2E37353C31312E30302C30302E3132342F2F2F302E313132323033352F3031343232332F
332D2E2F3332313432352E302F2F2E34322E3132323334303030392F342E3231312E3430
313431333432313131322F3232333433352F2F3231313131303330303734323334343532
3536323136353332333231333439332E3537343238343637373A383A3E3E3F4253535250
4E54555B525557524D4C4744474A434142433F3C3D413E3C3B3737343638343637363234
313336353934333A343334342F33312F303534343432363437323134313533322F323335
36323135312F3634323431332F312E31313032342E322E313031343632303133302B322E
30322F33303334342F363231333230323531302F32312D322D
3130332E33343431323430323238343430352F2F3031313433313630313234302E322D34
333435343433322C302F3134302E353632312E3030343233363B3E323531312F32353531
362F3131352F3234312D34302E332F3430323536313535303033332E30372E2E32303531
343530343038303132333137323431343335333735332B35312E34333533383334353032
333234333334353131313632333436343737363439393A3A383A3D3A3E413E464F585654
565B51595352534F4A4C4446424341434040413E413D3C3C393738343635373633373634
33333835393834363332363032323330342D3232313833362D30323432363231362F3333
34333335352E3033313432373234322D303132322E31312F352D3233323231353231302F
342F312E31322F333133333235333531333132303031323334
313131323231343031323231373430332E2E352B3032363436343333302F32332E313135
343434323333372D3130302F2F32312F323233342B333632353B3B3530312E3331323736
32312A363133343132333632362B3230312E312F302D3233343235313433353130333434
30332E2F302F313233323332332B34312F3334343832343431313232312F333232313733
36323132323333343634393131353236343236333834373B3C3B3D393D42444A50596061
5F59585352534C504D4748464544423E4243403F413E3E3D3A39373A3634393832343434
3639323A373932383130373237342F33313036323732333234312F2E2F32343431303334
31303130313036362E343235333136323632323433343233353230313531323132383033
3132303133303434313434302F332D2D2D32302A2D32323230
322E2D332B3432323334323433333333303230322F3333312E3031363134372E33323135
33323731343131343030302D2E2E2D3130333034333032323532352F3237303135323233
3432312E2F303434323236323134323335302E3430342E3732343531333131332F333330
2E30333334322E313330303430363132382F3232303131302E2F37313434303332363333
2F303035323431303433333536303336343438313C3A3C393B373F4242484954545E7273
6A60554E524E4F504A484447474A45424043403A3E3E3D3E3C383A38373A343436363833
36353938353433333732323130323533372D313737313532363531323332313436303336
303235313534342F303531343033312B312F2F3131342E352E32352F3133342F33373331
3533332D3334322F2E3033323231302D3334302F34332D3033
303232342F313131303134342E3730322E323635322C32302F35353631302D3331323031
323333353333343032313032302F2E303231313130313433323138323030323330303731
3331322D3231302D3632343033322D33332E342F312E353235342F35323431362B323133
31303035313136323832302F302F3232303432322F36343635312F373334353534323334
3532313531373638343337353336383937393A363C3F3B3A3B4041424C484A55555B7276
685E53544C524A4B4A4A45464A4747404240414243433F3A3F3B36393736323434323130
31333B3533373330333532333331342F322E362D37373132313335323130313331333235
3131333232323234313535333031302E303332323033322F353232303331333230313231
3031312F3533362E322F323331323232323132352E31323433
32332E31332F303132313131363130302F303330332F323230312E353535323330322F34
312E3036333235353331322C3032312F312F34333233302E303231352E32363533333031
3333313135332F323231373331302D33313227343236313634322E3132312F3130313332
3130322F2F2F333231343134302E32333433353032323333343535323732323232393434
2F353234353634323433382F3132343536363438393D3A3D3F434546504D4F56555C6067
6058534F534C494A4D4A494447434747434242423F3C3F413A3B37383237353637333432
333132363530343435343436352E2F343533333732383330333533342F34333431353032
34333036333331313133333635312F302E3330313336302F2D3131303032303231323033
302E312F34343232312F3330303331312E32343133362D3630
313232332E34312F333431303332323234322E2F2F33303033313630353132342E2D3033
3534303030322F373631342F302F2F2F312F2E3030352E3233352E322C32322F32302C33
303131353133313137322F30333136302D332F342F37322F3435312E3232303033362D30
2E3531333330373532373132303132343334323232333430303034363132383334343432
3438323034333235323737373233373639363333393D403B3F4349515252565158595E61
5B50524F50494A49494B494847464342474341443F41414039393B333933373736363335
3032343137383333343133323433333434343430343430313433303432332E36312D3034
303731333033323331343733353531352D333031303432332E2F32342F31302F332E3333
3031343331333531302F312F333232363230302E302D322F2D
3033322F343034323132312D34333030313130303330323332352F2F3033372F36313534
343131303133363334363236302D33312C3130352D3131332D3533323231333433343031
3432323330313330313034323331332F3234332E32303432313430313330302C34333131
32322D2E3431353333323432373932373531303430363033343636313235353731343639
363132312E3636363533343632363C36363B39383F3A424340434D555752515B58575756
57544E50514A4B4F4647464A4B494248494A4D473E3F3E3A3A3A3A343637333635353532
34332D363338342D35313433352F31322F342F30323734353A3031343434323134302E31
2F3436333134343531322E31352D323132332E343132323134302E303031323435313134
3535322F34312F30322F2E302F2E3435313333303131303432
302E30323232312F303233362E313433333031302F323234323030332F34333431323035
3232353234303032323336342D313232333133302F322F333132302E3031303235323531
3530332F31343132313132303331333530333132303235323231343335312F3131323132
37323231313434353431322E3736312F3337323730333136353034323135333337303838
34313333333834333336353137343438343A3C3A3C413E3F444B52545251515957565552
5151534E504D4E4950484C4D4D48484C4A505B55473D3D3D3D3B37383535383738373834
343134373137363432343237323130362F31323433323230323030322F2F353131363637
34303430332E313334343132313331312F312E343434343130332C302E33303133333233
32332F312F3131312F34302F30313132332E332F303232342E
2F303132302E3331313134323431303133302F3434323234323135323634333131353436
322F37312E2F333532343436323332322C2F302C303131303338352E3432363235313831
302E2F3030312F34353432342F34333733363534302E2D333433322D30353334342F332E
323130302F313331343534333233342F3436323433313432323034343032353338343430
3131353436323233343732363935363A3B3C3D3D414142414A4F50595257555958565450
4C4F514C4B4C504B4D4F525453504B4E4B525A50433C3D3E3E3735343637343634383832
362E34353432332F303631323432353236343134323532343032312E3234312E33323436
323133343532333132313430323532322D35302F32303034302B2F2F333333322C323033
36312F332E3332333232312F2F353032323432303732313431
332E322F3132332E2F3233343334352E3436312C333134313231333232383530302D3233
33302D3135303130393334363532342C2F33312D313137312E30313530332E3433342E32
312F362F33332F34303532303432323531342F34323231313431322E30303034342E3032
30312F36322E342B3333303336313131333332313332343032342E323334333236333733
343432323339383433323437373538323935393F424547444D5358595454545657555153
50514F4C4C4B464F4E5666654F4B474D49474A433F3F433E383A373A38323B3632333436
3233363132313731372D30363533313134343331333235353334352F32363233312F3131
36343738373536303332312F343232343331323032302F313333372F2F3334323330302F
313632333632343133332F323331342F2F2D313035312D3030
3033333330302F2E3133302D32333433373130323331303332322C342F32333231333037
34372F33303234333036363330392C2E2B3330342F3332323632313131322F3237323034
302F352B2F3330303035342F32332F343333323030353133313634312F3130323132332F
33322D32323034332E333431333336363133333135353435343037373537363430352F2F
3533353832313637353736393A343639373D40404642494E505A5C5A54525353594F5254
524E4E4C4D4949444D505A5E4D434A4346463F3E3C3B3D3A3A3C36353735393534343434
3333343133333531313430313933393432332D2F2F303435333531333130303132313132
31322C383D39383434322E2F333334323133303333333432323133323233343230312F2E
303332333638373231312C32332F2E3132313034342E373231
32313032323131312F3133323330333036303231352F333130332E333434313234303232
38343430352F2F303131323136353531343432352D2E2F30332E34373234313333333032
312D313132303131303234333337303231303032343132353334343534332F363132322F
32303331363131313630332E31343331313231332F333235343334323836333835323336
323232313738363735333A3338383A383B453D48484B5055595C5F5F585A535151505355
52505250474D484C4E4E4B4F4748474743464346403B3D413E3835383934373637363735
2F362F3333333134323735313636392C30312F3230312E313432333134323331302F2F32
30302F393935333036313134343231343231333230323034333030352C3033343230352E
283332343735363232333130352F3331332B302E3134303334
3433342F353135303333343335303330352F2E312F2F2E31313132323134303132323137
3430332E2E352B30323630322F33322F312F37313132332F333233303332303334333030
3032303436313032322F32323331323331352E3133312F2E3233342E3334383433343630
31303533302E2E3035312D32342D38333735313030353635373434333632333434343334
333636383A3435363237373437373C3B423F4444494F595E645F5D5B5F5C5C5357565553
50524F4F50504E504B4A484245424346434245453E3E403D413A3B37343A323737363735
3633313136373634323135323134363731322E33323131333135353633323333302F3331
30313531302F35343034343331332F343032313634303433333334333131323232352F2E
3031323635313133303334332D323633352F31303134323130
33313332313032312E30302E30312F30353233302F3432322E2D332B3432323334323433
333333303230322F3333282F3134323232312E37343532352F322B3236363430342E3433
312E2E34313331323134332E31343330342F33372F34303031303431302F2E3030342E35
3233322D31323531303234342E3434343334303132303230333336343431323833343737
3332333437363636373738383C3D363F434342474F505963665C5A585D605B5553565153
585455594C494B4C464E494744444444434143433D413C383C423A39383C373434333634
3136313433343432373034312E333033353031333233303333323032343433352B36332F
3633332F3635343433323431343430322E30333035332F31322D2F2D3232313230333030
2F2B31343234323134303232312F2F2F3130343431332F3536
34303131322F32353132342E313031312F35313637342E303232342F313131303134342E
3730322E323635322C323033303532303637313634342D2F323031313231323630302F30
332F2E2F32323234323134313332333232302F342C313035313135333231323032343331
322D3130352E2F3132323334332F343331302F3331343137323533323435323432333333
3937372E36343532363838393B3C3E434543474E4F555B65645B575A585B5A5953545255
545350514E4B4A4B4A4A4743444244424042403E423E413E3C3E3B383A38303633343836
3536363632362F323631313335343537373430343231333333313730313636342F313231
3131303031313131323232302C322F3332343033322F2E352F2C2E302F2F323233343432
323034353034312E31333132313333302E2E32333330302F2E
3232333531332F2D342E3131342D31322E34313135323832332E31332F30313231313136
3130302F303330332F3234303133343B34343032352F333530343033343232313033362C
33323332343233333231332F33333332313534342D313031313032313131353433352E30
30343335333030332F353036313130372E38343331363330333733343634333631343430
35393A35353531373838353E3C404244484A494F56585C6363555755545C585959585653
57564E514C4F4B47484A45464447413C413B3F413F413B373C393B3B3A36373333303236
363435353731343633363032343338383631343831342F32333031343431313231353230
33322F303134303333342F332F2F35322D3031302E333233323234313330333031322F31
302F312F353434323432303236313030313136323132312E33
3130303332313031373235323635373830313431323030313232332E34312F3334313033
32323234322E2F2F33303632363131353034362F3433312F342E312F333133322F313136
322E2F3432353533323032343531343236302F353032323232303233363631353334322F
3133313031332D3132323330342B303032332F3232383535323035353437343334383632
333836353736373B35373837363E41494B4E545A5F5B60635F58575C5A5756575A545453
5852574F4A4C4E4D4746443F483C3E423D3A3E403F413A3E3D3D3B3B3938353635333635
3234303134313331363031343735363332353231373332332D312F33303131312D343034
2E33313132323338333131313132333136323431332F2E313031342D322E323233313032
3331302F332D332E32322F332F323332343230302F3433312D
333130353032343232362F323130352F323531303230353033322F343034323132312D34
33303031313030333032313031323331313131363236342F2D323234302F2C2E30313433
2E2E2E2F342E33343234312D31332F3334343335312F342D2E3133333333393730333230
3032332F32312F313332323433343431313135303332353637363433323233332F353637
35373331373433333A3F3C383A4042484D4D586163615E5C565757575C595C57555A5654
51544C4F4E484A4C4746443D41423E443E413D3F3F423C3B3C403B353A38383735363732
303333383431323030342D32323636312F33313331333435342F34333433382E34313330
32342F322E31303233313335323131343233323130352E313332312E31322F2F33343330
303232322D332C30363235342F3231352E3230303330343332
2F2C32332F332F32303430333733323434333032343233302E30323232312F303233362E
313433333031302F32323630313134333431343431313433302F333132322B3232333430
30313136333135312E342D2F323030333635312E323433302C3232333230343133363231
3336322F313034323533323133302E3335332F3433323033363631363634393634343B31
38363735363637353936383E3C42474E51575C616E685C585558585C57575E575B5B5754
5954534E4C484B484946424246423E413E3C3F3F3D3D3F3B3F3B38383837373735333233
32382F3231373535323031312F322E36332F343431333333302E38333432322D33303233
3330322E2F312F30322F33312E3434352F2F352D2F34312E31313430302E2E3335343330
3333312E323130323231342E3430333230323130322D313131
3634313332312F313430302F2F3234302F3032323030332F303132302E33313131343234
31303133302F343432323231312F33313234312F33363336312F332C3033322F33343034
313229343230323133323034313030333234313036322F2F333133353333323336303835
312F35362E35323232323735333636333431313433323632343736343532363433333432
37343A383934383A3E37393E414650515B5D5D61706359535C57565855595F595B575254
5251524D484A4749424743414440444040403B3F3D3D3C3E3B3F3D383B3E363831353430
3338323233332F38353231343033323434303434362F34353731303034333136342F3134
3232313231323430352E3232322F3031343330312D2D3132332F30333131303331303331
3030332E3331332C3232342D312D34302E35302E3136302F2E
3233332E342F30323433302D333433322F32323533342F332E322F3132332E2F32333433
34352E3436312C3331343230333431323234302F33312F36353332313033333132313133
2F2E3230343236312F33312E3634333232343432352F3031363131373130353436353534
34322E3631302D3131343333323435353431323036363233303133332F36313937383837
3335363B3537393435383E42484D515A5C615E5B615A59585B5C5A5D5D5D655B59575853
52585253524D48444846474643424442413E423F3D3C3B39383A383B3A3A38393735372E
34333435373733383432303332343335333433322C3133332F2E2F333131323434353134
3130363232332D302D3130332F3133362F3433332E33313131303130323533333332302D
2F2D323031333131332E33303132313431322D313331333134
3436312E3034303534363834333332312D3531353232323033333330302F2E3133302D32
333433373130323331302D323337362D323135302F2C3131323636362E3330343434342E
303032313333342F3334312C332E3134313332332E313533323233303136333133343231
342E32312F3233373134323534353432323432362D343634343336333737353135363636
3836373637383A383A41404648525A60625D515957595A575D5C56555D5A5D5C57565857
55544E4E4F4B4D484646434642414145404140403D3D3D3B3B3D3E383437393132323635
313236313731323132353333312F3337323633313530362B34323238323032353234342A
312E332F30302F30313031323133333231313034302E313236302F2E3130303130343230
34332E31342E3231343032333232363331322F303331353432
30332F3335342F35333439363433323333333231312F3032313032323131312F31333233
30333036303231352F32363330322F30313432343232322F33363432352D2F2F32333431
31313131313233312F31313233323232332E332E35313234323137333133303135303133
313130313138313432353433312F3532313232313332343431343232363235343A33363D
353738393D39383D3E3D404B4C59606B665C555B5B5F5B5657575B565C605F56555A565C
5854565153514C4748494843474142413C3E3E3C413A3C3F3C403F373A3E373732343734
35353330313433332E333132333432323232313031323331313233322F333134322E3334
2F3335302F2D35303032333233323331313231322E3131322E3130313231333136353330
2F333331323130352B32303033313333312D34312F3132312F
312B343132343533373133312F333435303433333236313433342F353135303333343335
303330352F2E312F2F2E343332312E32313235362F2F2F30353432333133343336343232
2F2D34312D333631303435323133312D303230333532332E31342932352F343331353436
3032313336313731332F3335353133343436343432323033313431343A34353734363436
3138373838393A3D3C42414A555C7277685D5C5B5D5F5C5A565A5C61645E5C585B5E575B
57575756524E514C494649464A4640423D3C3D3D3C3B3D3C3C3838393B38373835373334
323633362E3534343432323437313335343231302F302E3333323232303431302E332F30
30322F3333313132302F3234343034382E333231313331342F2F32362D323133332E3333
2F332E2F2E3230342D2D2E302F2D302E30353430302D313335
353134342F35363031333031323338313232313032323733313332313032312E30302E30
312F30353233302F3132343334302C30323035312D33312E35352E2E3032313334313331
2F2E31303431353432332F352F313333332E313134343432342D30313030333334343133
362F303030373533323931323235323235353933343236352F3533343335333639363534
3932393A373C3D3C453E474A58627E7C665F605F5A5E62605E5A616467605F5C5F5C6060
5A5D6057505456524D51444A47423E40413E40433F3C393F3A38393C3736363635333335
302F34353433333235333233333235303735312E33303031302F31322F33303130312E33
34312E2F303033323234333734312F3534303231302F2C312F2E3432312F2E3532322F33
312E2F302F31342F2F2F3534363232303135333230312E3232
302F31343734323033363331353032312D36362F31323334303131322F32353132342E31
3031312F353136363230303032373236342F3534312E312C353533313130322E3431322E
3431303332323232313330313532363532323232303434393331382B3536313233363433
372D3731343434303233313234342F33342E35343330323231363334363637323633343A
3633393B383C3F423B46444E5A63736D5E5E59605E5C60615D5D656C6560605B5B5E5F5A
5F5C5957545658564F484848423C44403C3C3D3F3E3E3A3A3D383B3B383B343837353534
36363833363136343332353336333434363435322F2F302E30322E322F33342F33303132
2F3332332D312C322F312F313333333633312D2E2F312E333131302F303431312E313231
3830322F3131342F31302F34323030312E2D3030303230352F
312F2E3130363332333531352F363031323030343034343232333531332F2D342E313134
2D31322E343131303231322F343330343332332E3233313132342E32352D372E35322D33
3332322E352E31323131302E303332303233333331342F312F352C313234313734323234
30343332323733353035333330373A313232333731323632352F3131393432353237353E
353A35373B3A3F3F40454B575C67706A5F575B606065605C616263686B6361615C5F5D5B
5B585755575752524D4C4B4E45424142413F433D403E3C3C3B3B3B3A37363C3438333537
33313436333433343335322F36342B393235312D2B2D3230303432312D312F3133322F32
32332F3237382D3632333133333431323132322F3232342F303130302F30313332312E33
352E322E32323137312E2F31313734322F3330323030333131
31323035313335313035323031343032323333352F2E3231303033323130313732353236
3537383031342D2C3330333431313235303132322E32363432343431303332302F323233
313030332E2F332F2C352E2F313236333331322D3536303134312F333233313430363331
32323434342F3537343737333333343632333635333633332F35343032343434393A3839
3737353C3B3A3F424346515F6970726A625D5E5E615C5A5F615E6767666664625E605C5D
6059535958565853524B4C494946424340383D3E403D3C39373A383D383938343332313A
323036353133343035323436322E34342F2D30323132322F32322D30332F30312F333032
3231312E2F303434363235322F2F3232322E2F2D3133332E313231333130322E2E343539
36313434322F332C3432333330343433333331323130302E31
342E383035303231362F3134343632313033342D363436333130353032333232362F3230
30352F312D31322F352E333434313B353030333032313131323233353030323430333732
313033322F30303134323033312F3230303530342F30353230313231322F352F30303433
33333030302D342F303133312D3432313537313530343431323331353333353A32343437
38393D3F414145474649536069706A6C685B606065635B61666569625B60616467625D5C
5F575053575454534F504E4B4343473C413D42433E3F3A3B3B3D3838373433363E343635
35323131323034353130303433323234343032312F3231332F33313131313132342E3431
30302E2D3533303132313031302E303435302F2E312F32302F2F36303331303031333234
3636323033333531323131322E342D33313432373132333332
3330343437323032313731352F2C31312E2F31362E3031342B32332F332F323033303237
3232343134302D3231333232363539373035342E31323233333333343031333134322F33
30322E362F2F322F303136313632343337303435322C32332F302F2E3035303330323132
3234342F3334312E2E3232312F303231303436323638343333313133333630373B363233
3738393C3D454647474B51616F6C6C656464616867656163646D66646363646568615C64
5C5356555455504D4D53464F4744434440403B3D443E3B3B39383837393B383437393436
3534343131313233342E3332322D3630323231323330302E313232302E3031353033322F
38323133342E30313232322F2F2F302F3132313130322E332F2F2F322E33313233313232
313331323231313530313234373231302F2F2E2F3331353131
303031322E2E30343431303330352B3030323431332F313332323232312F303430302F2F
3230313131342F3132313535363832323231303130333233353434313133373135313133
2F373232323332313135333137353738343533302F32333430333035303331323331332F
313432342F363132323330312F3130353133333A32383533323733363337353634313837
3C38373B404444484C5056636F6E6E65626261696768666668676568676A6C636460615C
585B5C595957524F505151524C4A4743423F4243413F3C39383C38373637393536393A31
35343334383132383433313331313432313132313132322D2E31322F2D33313130342F2D
2F3233363230313332323030333034322F2F33312E2E312C2F2F312D31342F2F33353132
3531313032313031303130313232313134323236343434332F
2F372D3533353133323332313031342F35333534322F32343233332F312F2E34342E3131
353234312D322D372F362C39343633303033352E322F3031323634333031333635313132
3031342F2F30352E34302E3731383D3B383334322D353330353436313333342E30353334
2D3334323134333530333234303332303236343534343632353739343136323638383A39
3B39373E4045484951525A6573736A645E62626B6A6865686665676C6B6F686B64616160
5F5E62625D51594C4F51534F4C4A494642443D413C3B393537353A3B3737393439363336
353136342F32313536323331342F3433323032303531302E322D312E3031332D2F323333
31312E342E3033313131313430302E2D2F2D3033322E30293134312D31302F3033323435
372F2C2B312F32313032303131333333332F31362F2F32312F
2F31312E31303531323633343231323336333837363132302C3431323133312E34322E34
313130322F34343533333430303132323230323031332F332F3432353334323137312F34
363131333333353432323232303534353531352E333034372E35323033352F2F302F3230
312F2F34353033343231373135313435323533343430363A333535363534353538393A3A
39353E3C43464D4B4D56636575766E635D606967686C6A6A6D6E6B70736F6E6767675E5F
595B66666059564D4E4F52534D4E4544423F3A4041413B38353A3A3639393A3536363535
36333238303332342D3032373137363233323330322C322F2F3232332F2F2E2D31333437
3232302F3131302E32303031312E2F302F31342E30342D2E2E302F312E322D2E32333638
31322F302B2D313132312E312F322F302F3034333234323030
333133323139323530353531343332333235353033353332343232343434383135323330
32302E30332F333533343131343131302F2E322F2F33343131333334333031312F373330
323033343131303531303331362E31373333352F343131323533303130322E342E303031
3432313533313236313434393236343235333333313633333332323431393837343B3C38
383D3D424A495356515B6370737C6A625E676E696669606E6F7074747372686960636061
5E645D5D5457524F4F574C514D4E484042404043403A3636343C3A3A39393A413A383235
3435323334373334323533353430323332313032332F3232312F2F31313031302E30322D
3130303330313031322F332F2E3230302E322F2D2F3234302D302C31302F2E2F32333234
312E2C322E32332E33313233302F3533312E312F2F33333132
34312F31333B3432312E3232302A34322F3530332E303233332F2E2D3036333334323337
313534303635333236342E312E2C2E3131312F3032303332313435323331303636393334
32363636352F3731323131322E3133373136382F322E2F313331313332343130322F312F
372E2F303530323435373436352F3A3632333934343736303A363637333B3B3C363C3840
3C404044444D51545B5E6C76848A7266676D6E676A64717174756F747974716961665F65
5D635956554D52524D4D5251524A484440434040413D3B383139393D39373A3A34383633
37333333343334313233313430333530333330323433323232332D3430342F3232302E31
323433312E30302F333431312E2C3132333031302E2D2F2E35332F31322E313431313132
332E32313130312C33362D34303035333137303131312E302E
34363035313933322E332F342E2E353138323233312E3334312E2E323333303032323333
312E31363331332C343432322F31312F2F312D32323136323133313134303730362F3337
323431373234363733323133363532313335303032313233372D323137343730322E322D
34303436342F3033353438303137353031353635332F363333313A3534383736373B3C42
3F3C4242494E525C5A5C6C7E9EAE876B7170717068696C717A777673787D7B716E656060
62625B5B5550504F4F534F4E504B49463E41423E3B3E3C383C3C3D3D3538373539353135
3A373637313131352F33313335303436363231333030333530332F302D30333336312F32
3733312C2F313033313233303130302C3033353232312F2D3033303133322F2E3034312E
342F32313433313332332F332E31303132303132362F2D2F2E
30312F30353233302D3536322F312E3335322E32303034332F3533323233312E33323331
3430333231322B2E34352F333431312F322E2D322F2E2B333633332F3734313432312D31
343530313133333435322E2D3132313433333635313038363634383432323230342F3230
2D33322D353631353130333A3233333136342F3533332F3936343738333B383739393C3F
3F424048494F54585B63657C9CAC8E6F716D7675747377787A7E7A787A7B7C6C69666263
66625F5B5756545456545755525146444343413E3B383939393C39383B37343535393632
313437362E343331322F34323433362E3130323232332D30322C30302F32332E32342C38
34383130362D303034302E31302F302F2D313031322E332D313532312F30343331353032
342E2E353435332D302E34323335342E312C312D3531323131
313031312F35313637362F332F323137343134313231332F353433303231323530363135
3232323635363235323435302E323633322F2D3533322D3532362F34333133303333352F
333037363534323331312F33293334333233342E2D2D3533303434323231303534322F33
333135342F353433353433333434333936353233313530353235373634383A3C3D404445
4346464E4F5854585C6C6F7B7F807B6D7674727377787E777B817C7A7B7B77776C707076
7A66655D5D5D5B5D5D5A5D55504F4F45414345403E3A3B36373E38383A3639383A333733
3736333533352F3332303433313633312F302E3431313231333134323030332F312B3035
34333433322F2F333130352F2D2F30303133302E2D31322D2D3331302F2E2E332F34362E
3130332F2F3232302F302E2E31323132312F2E33313032332F
342D31322E3431313532382F2F35303133322E3232352F30323232362E3131302E2F3132
3134323634363230303230352F32323430342E3432322D362F3233363433332C33352E37
34313333363331333333313033343132313133312E2F33333531343335332F3234323231
2F373130332F3536303534343337363333323635333531333732393B3433393947504F4A
4544474C55585759666C7276786E756C707972727C7B7D7C838A83807A7677756D74787E
7A6F64676C6767706867665C545050474244423F3C3938393E39383A3835383638313733
3932303233343330353237343336363332313335303136323331332E30332C3334312F30
333431313131322E33323131312E322E2D2E2E343530332F3330343130302F312D323630
2E3432303130323335303332343334342E312F322F2D323130
36353738303134313230302D3232353133313031313030302E3533302C3038302F31322D
31363236313233312D3132312C3337313732313235342F332F342F3231332E3333313336
303430332F2F332F353232352F33363234313433312F3133353230322E303531332F392E
2F30343031313231343336383433383033323632343234333734343934383D373B464C45
4246484E525B5757696D727676717072746B717978798083868983807D736C6A6F757971
71686D7B8377736D6D695E5957544E484242413E3E3D36373B37373A3A39363934323237
323432343030312E33353334322E3234303634313435333230352E31333130322D353135
31353433313030322E302F302E3431312E2D2E312E322F342D3233312E3330313233302F
333231333536332F2D2F33342F2E3433303433322D32323130
3130352F32353130323035303133362C323032342E323130323432342F302D332F333435
34312F35323431322F2B31302F302E303230313334353433343139333334332E34323332
3335333236343331313133333634363332303835373039373537323132322E3433313232
3234313234353335323336373438343237373635353634363434383835373B3B393D4242
4846514E545A5A60676E737773746E777174777F7C7F878886897F77797471716E757263
63646B7E837774726D6C5F5856554F474943413B3D3B3D393C3738383339373635373636
2E36353036362F30343234343232323033342E34363330352F302B302A37302F2E2F3836
34343030302F3133302B31322F313331312F3531333032343133322C3333333333333730
2E332D2E332F2F313230342F2E302F313634322F3232322E32
3733323434333032343233302E313030312D31343033323631303434322E2F3436343433
343432303030333133303030332E3231342F342E34302F372F3533343231312F352F3536
2F3331342F322D34333431323235333333322C3037333234313534303333343136303234
32353436342F34313133333633383237313630372F343430343B313A34363B3B3A3B3D42
474C505453595C646E7479787675767275738389888B8A9089857C7B7B79767473726E65
646A6E7D757E766C67625B5857534D49464044423D3E3A383839383A3736393237353535
313832373230323231312E343234333331302F34383630302F343233302F2F3334303134
34342F2D2F30323637333333313331322E31323434342F3234342D3131322F3130343032
31302D2E3333302E31323131333133353332322F302E313133
2F3234302F3032323030332F30312F32342E3333303530323234312F3430333134333434
3433343437332F322F3531342E34343031313232323232313232313632332E2E2F303133
2F303233342F3137333133333131353833352D2F35322E33313132343235313335332E35
3233313432313436303533323731383832333133323135383638373433383A393B404347
444A51565C5A5F707A8075767C777B787C8286948F8992908C8688867B7C7B7978706A66
666A7A737273736563635C5B5751514B44414340403E3E3B3C3838373A36343A35373333
3530303534343334303330343A2F332F3035343032342F32312E302F3330343031332F38
3532312F323230313637352E2E322C32333232333131323233312F302D3332312D313432
342F313034342F332E3032332E3333323332323030322D3035
333433322F32323533342F332E3333303630342E323430322F303133332E323433343434
333334342E30303132302C322F31333131343332362F3433332F3030312F323433342E37
3233312F332D32303531332D3136313432342E343830303336352F323531313133323036
3030333332362E3535353131303632343132353331343433353833363737393A393C4047
4A4C56555B6062717F7F7D79817C7A7C8689929F99929A9693929190918881847B746C68
706F767B706C6D6462625D5E5451504B464642403E3C3C3A3939363839323537353A3335
343335363034333434343737323335322F32322D312F3030333431312F30353133343130
31322F2C2F332D32363C362F34303030332E2E2E2F352F32312F2D2D32303130312E3132
2F2E2F3034313431323131323234302E362F303230312E3034
333332312D353135323232303335343531302E302F323231333331313330303033333433
33343433322A3430302E322B322D2E302F31303433332F2E353430353331333331343130
2E3633303430303330312C31322E32353134313130333134353233313437343234313634
3433352E3031323132323A30372F3631363334353633333439383637383738413F3E434A
4B54565E636369797E7B78787F7F798C9A9C97A39B9DA1999597999D9898928D7D7A6A6C
686B75716F6A696163635E5854524E4B4943403B3E3D433F3B3C3B39373D393434373537
3736363430363331333238343135343532343533313132332E2D2D31322F302F37333634
2F2C3330302F312F37382D31312F322D303131342E30323230362D35303431322E322F32
30302E322C31333033303234353236363233323132312F312F
3433323333333231312F3032312E3431323530352E312E3332323233342F31312F333333
33333133333131332F2E2D3030312F3230322F312C323335353731333232343137333230
33313433322E3332323132313034323430303333333230333030342D333431332F313233
34322F3631323136303735323236343134323232363635353731363A3A38393C3F414748
4D50555E6071777C7970787C8081839CADA69EA09F978F97989EA0A4A7AE9D938E7C7167
6E696D6C6767655F5E5E5D59554E484A474341433E3B3C3B3839363837363A3430313633
383233343430343536303530333035363232313234322F31352E3230312E2F342E33312F
3230302F2F313232303434323531312C352D322E30302F31322F33313231303032333430
34312F313132302E34312F342F3230342E332D2F2E2E343433
2F3334353034333332363134333430352E3731303232333631362F31302E36302F313232
322F3233323232342E342F2F2F2F2E2F33303133322F2B2D3036362F3032333032303034
2D33382E323435333132323034333233353532343632322F303237303230343132303132
2E3431342E353531333134353433353434303338373138363731363A393A3E423F444149
4D535D6267777774777A77797F828EA2B4A89E9D9D9291979A9EADB9BCC6B1A29383756F
6C6864686566615E62595C58514F4C4B4641423F3F3C3D363935393935373A3A38343534
33373433352D35343436363636342D32323337313032303432352E322F2F343136302C31
32312F2D32313430333232352F322F3330303030333033333432313133332D3130303333
30323432302E3135333533332D303430303331302E31332D36
323338313232313032323733313333312F3732333231313133312F3031302E3530312F30
343234333232322F32312D32323033322D303035303032322F3432312D37333333302E2D
30322A312F3232342D34333631362F3331373230332F322F313835343230333534363334
363433352F3434343332333131343634313532343637372F333937393A3A3A3C42464A4B
4E535C5D696E747978787D7E818D97A1A6A3A1A49F97989CA1A9B7CBDEEBDCB19C908278
6C6869666863635B5F59565450504D4A444241403E39403B3A3939393837373637343335
323531353034323133333135332E2F30323136312F342F3133332D2E2F2E323131303130
31322E3131322F352C31302C302E313531303233362F31322A362E322E32313232323230
31323131303130333431332F31323132343031312D32363131
353032312D36362F313233343034303434332B322E31302D3331322F34322E32352E3432
343232323132303331372B3436322D34322D2F3131333231353330302F2F313231313031
313132313333333535313136353334363831333330343732313637323132343432322E31
3235333230303135363032333238373433353534343B3539373435393B3F394440424C4E
575D595E646D73747980827C8495A2ABACA9A19F9D9DA3A5A9AFBFD7ECFFFFBDA0928778
756A6162665E61665E5E54594F4C4A47413D3D3F3B3C3A383D3637363738383733323635
35333333302F2E3231343235322F2E3032312E32332F32303130343032353635322F3232
2E312F36313430323230322D2F2E333532302E30322F31302D3531333134332F342E3131
342F35302F3130322E31322D31342E2E31303030332D2F3234
2F3630313230303430343432322F2F3533332D353332312F2E342E363232303033323532
33303036363234343032322F33342F323433312F2D30363032323435332C323131313230
35302F3032332C34303A332D3B33343337353336313131323133352E3531353133313231
33323534343732363032353034323A343436323636323536373A383939393B3F4345494C
58545B5E6B717C7B818880808995A6B0B4AC9F9E9BA6AAAEB4AFB6CFDEF1D8B8A18D878C
7E6B62645C60646D6F6D6152524E49454442403D3B3B3C3D363637393432373436313532
322D313633353337323533303331343233353433353031312E323230352F383230313031
2F31322D303236342B3230302E303230322D313230322E343233302F30312E3233333431
32302D3130302F36343132302F2E3032312F2E2E2B302E3133
31343032323333352F2E323133303333332E31342D31333331312F333232343134302F35
332F33323830302C2E30312E303235323231312F3438302F3733342F322F313233333230
333230312E32343232313434313231343333353433303133353132373337333431353135
2F3237342F343437343233343135363236353437343338363638343A3C393F3A454C4952
5058616867787983888883849096A7B6B2AC9E99A3A7ADBDBCB5B6BDC9CFBEACA199938C
8071646262635D6968725D55514C4A46423D3C3C3D403C3A3B39363A3435383535323433
3032323132373232323238333131353333332D383230302D303131342F352F2F32333231
33333230343132303031312F3332333130322F31303234313230303032333431362E3031
2E32322F302B323433333033323433323331322C2C31312F2F
343632313033342D36343635323233323332313035342E322D323135342F32332E34312D
312E352F383230323031352E3131312E31332C302E302F372F353434322F313434303133
312E303533343532313234353031353636323231333631333033323433323237302E3231
322F30373634313732323231343134333330353833363933383639393B3C3C3E4B4B4F51
55575E686C727D8386878A878D9CA6B6B1A49A99A7B0BBC8C0C0BDBFC6C9B5A19E9F9C8F
8E76656866636365665D5752504847464240403C3E3D3F39393A38323034333434323534
353134323932353634332D34312F31313234333630332C2E3132332C2D3330312F2F3132
313332323032302C3132323030332C31302F31303133322F3330313133312F3530333134
2F2E332C312C2E30322F3032313335353230312A2D312D2E30
2F2C31312E2F31362E30312F343131363434332D3130353135342F3130322F3433313432
303134342D31302F34342F343030322F362E30343233332E3033312D382F352D33302E33
2D3130332E3233333535353232323632332F353732343031343234363332333431323132
353234333333343333323434363333343734363439343A373B3A38383D3D41424B4E5254
525C5C646A7583868B8B868B919FADBCACA09BA3ACBEBFC2BFB2B8B4B9C1B6A69F9A9894
88706668696867615D59554D4E49484645403E3C3F3C3B3D383939353637363231333433
313335323434323033312F2F2F3237302E3131333237303133322D323431323133302F30
303631332F32302F303031332E34332F3033322E2F313232322F2F323533322F32323232
303231312F31343133323233323334362F332D33332E2F302E
30352B3030323431333434342E33313233342F32312D35312E33333130302E3336323634
3231333334332E32323332333133323334313630313133323631352E323232313231302F
3034303232332E333133353133333033333234313530313535342F333133333333303133
33333A3135373732393334313433332F3536373737323537393A3B3B3B3B3C404A4A5053
575A5D60727886878F8A8B8A99A5ADB6AC9FA5B5BCB5BCBBB6ACA7A6B2B7B2B2A297968F
846D66686665675D5D59524F4D494643423D423D3D3F3E3F373738333836333933323235
34353336333933322D323331353633322F33363235322F2F33302E323231343230313131
3332322F2E2F2E322F32323136353130332D2D2E312E3134312E2F32312E333035343234
2E31343033313133312F3334313331353232343330312E322F
3031342F3533353430312F33303530333030313232353332343132333433313538353435
3235342F3235322C3036352F3833303231352D3431342E333431373134332E3430353230
322E322E35342E2F333433313435332E343131352E333139333130333534313433323535
3632323031353534343137323534343438343234353939373D3D38364141444345484D55
5559616971828B8D93938F9099A1A3AFA19EB1B7BBBAB4BAADA9ABA3AAADAEADA699938C
80706862686865635F5A524F4A4A4645413F3A3B3C393E40373836363635322F33353435
333235343533312E353333313137342F30322D3531322C2D32333632322F2F3332353333
3734343131303331333034312E353134322F2E32303333343032312F33312E3131333534
2E2F3230322F3133332E332F33363434312E3131313131342F
31322B3132332E34312F34312F2F362F313630313031333234303432342E2F322E313532
382E313434302E2F353830332F332F4E34363131332F323535352E3133343234352F3434
313433323031333333343138383331323131313732303432333437303234313337303530
31343336362F32363333363230323536383336363A3536373B383B3E3E3D4345494A4C52
575A656973818D938F919293A2A4ABA9A1ACC0C8C5BDBEB7B5AEAFADA7AEAFB2ACA29D93
7F7170686869646960524D4B4747454340413E3C3B3E3938393635333638333332333133
353136333636363430303232323533323434313233353130333331312F30343237323333
2D333331323134332F32362E2F322E2F32332F2D32302E322E3234332F30333230323432
31303234313132353234303232323632333030333330313031
2D2F31303330322E31302C31312F35353330333435323134302F32333331303832342D33
2C32323530303231322D3234313532303331342F313430372E3033323231303530363334
30342E3332323132342F33373136353538332E3532363034333430333235323233353430
3335373235343933373933343637392F333235363337373A373A3D3C403A4349464D5054
5B5F636E78858F8E8E949BA7AAB2B0ACA6B7C4C8C0CAC2BCC0B8BBB5B0B1BFBBB4A39F95
86716569696B6E6559514D4A474444463E3D3D3D39373D393C3734353737303438363233
3235353234303232352F313130383335362F342C3130303336312F363233343134323033
30302F3032322F30332F37333130322E2D2F312D30353031333232313031323134313532
3533313131333030322C30313032353231323331312F313333
3332323230313430312E313430323233353232353135333432342E33342F30333031322F
313235323033363031352F3332343036303731333431353232362F303433323131323237
3531342E2E323331333133333331323635343632323134323735303634332F2E30303436
373530383135343333373134323633302F393334373835343A3A3B3E414041454A4C5451
585E696F7E889893909AA1ADACB5B0B1B6BBC7C7C4C5CFC0C3BEBABAB7B0B6BBAFA49F92
8B726A726D6E6B685C4E4B4C49433F3D41393C3A3F3E3D3A3735313639323435372D3538
342D373534342F31362F2E33323332322F323634313035343231323334332F3031302E32
2E323030312E2F32323331323034312F2F31313432312F3131322E3132302E3231323330
3536312F3132312F333232322F30343231323631342F313231
30333232322A2F312D2F3430333234322F33333133352E3432323232343332322C332F33
2F2C2E35333431312D323231333139303232323033313430323230363233313436333933
3337382D35323337303233323436323032333333383634343334303433302F3533363335
3036333830363532363738353634343734323234343636373C393C3E434341444B4C4F54
56666D73808D989AA0A4A9ADAEB0B0B3C5C9CCCDD0D2CFC4C4BBBBBCB8B3B1B1AEAAA292
7E7173757378727367564B484544403D403C403A3E3A3D3A353434323534383235313535
36333533332B2B2E3331353231303936322F2E32333234312F352F36333232342D323531
333431373434312F33343331333332322F302F3132302C323231333232302E313234312F
3234343233302F33303334333236322C322F34333032322C30
2D3332302E2D3230333231323131303234302D3032313134312F36352D32343034303130
3330312F34303233343233302F35323633353632323433323132343233312F3232313336
2D3032343333353131342F343133373137323031323433322F3436343631312F33333732
34313135343232343738343238343533343137333737353837393B3D404447454C4A4E53
5A5D7277808C989FA5AAA9B9ACADB4B9C2D1D6E0DCDBD1C7C5BBB2BEC0AEACA4ADA59B8E
7872737D7C8080796C554F47484041403E3A3C3C3F3D3236373435383936343832333334
3234312F352F32333031353331322E3230322F2D3334312F302F333032292E3731343030
2D2F313332302D303031322E3230322B302E333032332E333032332F3233333132323032
33303535313333313430343230303432303030312D34303030
2F313231322E322F2C3130352F3331303034333131342F3231332F3535302F3331332F33
313531342E342D3132343433313236363234343233333032392D342E3436323832313130
34342C3336323235323234312F323434312E343134323234333735323035303735303635
33333637313831393336383A363934313233353837333737373E3B3D4343454C4951504E
6065747E8A939CA2A8A9AFB2AEB5C2C3CFD3E1E3E5DCD6CBBDB8AFBBC5B6B2A5A59D9182
7A7180878688837D69584E4B484340423B413E393C3737353B3536373532343433363635
36323530303234343333313531323333322E32333432313130303336332E32322F2F3530
3037333230332F2E322E33343231323032352F3233343331323332322F322F32322D3432
332F2E313531333430353334352F332D2D333231322C2F3031
342F2F3032312C32322F3031302F33333134322F323130372F3233313133343032333532
322F3133313331343431313533343B3935303833353432303231313232362C2D3237332F
2F3035343431363138353031313432302F3133302E343233313332323332343734343634
3238343233373534363A3836343432353837393337373638393E414040454D4C4C4C5359
616C787D90969E9FB3B4B4B7BABEC6D0D5E2EAF5F2EBD7C8C9BDB0B2C5BFBEAAA29E8D81
796D78838D8A827362544D4D47443F3C3B3C3C3938383A33383039353733333338343338
35393332313334353032333235323A32333130332F303330352E30322F3332342F313334
333336363131302F34333635303130322E33312C313434333533312F2D332B3331313233
2D31342E33333434323330302F352F30372E2E2F30312C3231
36303033323832323334312F3435323132312F3034312D343133332F343432342D353231
303834323237313033333433363233333031303432333533322E303133312E3330323333
3236313732353430342E35313132303533353334343632333832313333342F3735383537
3739303234333436343238353336343332343236383B373C383B3F3A3E48494F4E515463
737A7D848E97A5A7B1B8C0CAC1C5CADDE4EEFAFFFEFAE7DBCEC8BAB4C1BEBEB0ACA99785
7A6E7179877F736759504A4947423D403B393A3A363A383A32353637363632303330372F
32343432303337343634332D3433343838352F313032353030323033323230343030352F
353332343032332D33333132303431312E2E333131323131302F2E30303033332F2F2F33
31333033303434312E323130323330303331332E3433313032
34312D323032332F303230313032312F32302F323731322E323232323031312F2D333033
31323336323136363433323333333237303433302F333334343532363230333435342F36
33323234343534353731353736333133343132302F32323433343132323532332F342E3B
39393532343435343137363538353833353433343B39373639403C3F45464F4E4F555C69
8F8F8383929FA4B5B8BEC8CACAC7D6E0FAFFFFFFFFFFF8E1CCC6C0B8C0C0B7ADADA79884
7B70737273706A60564D47464341413B3E3B3D3B3935333336343534363234343133332F
30343230302E3433343530323130313935353334342D3334343232333234343131303133
30332E33323131312E31302F342E3130303032342F31343434342E303034322F2E2E3032
2F3330302F343332342E3130363031313431322E2E3434332F
2E312D302F3331333430343232303133322F31302D32313231323330323335303133322B
363231323230342F323432302D3035302F30303230333233303333313133342F342F312C
313632333133323334332E323330313131323231333332333336323134342E3234363433
373C3635362E3432372F3536353936373335343A3537393B413C413F464B4F5556535C6B
85928B8691A5AEB5C0C4D4CDD1D3DCECFFFFFFFFFFFFFBDAC9C7CDBFBAB1A8A6ABA69987
7D726F6D6E63645F574949454343403A3C39353A34383335333037383539323430353230
3035333434343133322D33312F33323331343335323034302E34333130342E352E303332
3032312F3730302F34303330312E31312F333332332E33332E333332303132323233312F
38302F332F2F3231302F302F3435303232322F2E3331322E31
372D312F313032323130322D2F32342E322F323332303132322E31313130332D332F3432
3136343130342F35302A3132313034333037312D303332312E32322E3430303330312F31
332C3234343130332C3332312F303437353236333231333134332D333435373531363334
3134373635343036383834323535373839383434363B393B3D404243464E5454565A5568
788A89868B9EAFB8BEC6D2D7D9DEF2F1FFFFFFFFFFFFFFEED7C7C9BDAFB2ABA7A3A19285
7670766F6E665C565045464248433F3C3C3E39383835343832333536363337363334342E
31362F36333234323232322F3231353132303A3433323135333032323031323130323130
2F2F303431312F302E322F3430332F3031333031323135343431302F3535342F30313130
32312E3231373230322E3534333132313030353330312E3231
2D2F353131312F3233323232313035323030352F2E33313630332D302E33323335373435
34393235303733343332302F3130313136342E3732363332333333332F313533342E3230
32302C3131332E332C2F353230313134313934312F333232353430333230313435363531
342F333533363236343637343537363A373A373B37313C3B3B3E4144494F5B5C58586165
6D7A7F818F93ADBFC4D1DDE7E3E2F1FFFFFFFFFFFFFFFFF0D9D1C4BDA7ABAEA4A49D9988
76716F6D69605A584949494042423C3F403A393A37373438313336373236373433323433
342E33343332303234323431312F323234332E2F33363531342F31323232342D2F2E2E2D
2F333532313030302C352F3331373432343230303133333337332E333038332E32303032
3033343034342D332E302E313130312F3031312E30302F322F
34302B333330322F313231313031343130333032302C3334323330312E3134332E303731
34302D302F303435372C3530302F322E312E30362D3732343332373137323133322F3033
3131322D3234343233322D3132323132323234323330352F322F31313135373335323231
3233363334363033323736353535363936393B383A3A3C3E413C404643535459575B5B67
657774848D9FB3CACACCE2E5DEE4E8FFFFFFFFFFFFFFFFE3DBD9CCC1A8A1A1A19FA19386
79696D6C666255544D4B43414444403E363A3D34333738313633353634323332342F3438
322D362F32303331343232323234363230312D2F32333233313032313032332C322B312F
312F313131313031313130323034323033302F30342F36352E3131352F31322F2D323232
3232333231332F32343337323332313131343131332C2F3431
32312F3234302E2E312E322F352F3334313130312E3034302E32312F3134303337333533
323632342F313033313031322E2F30342F31343534333632332C31332F322D342D2F3230
30312F303331322F2C2D31323230342F3336333335362F3634332D313232343034353231
3132303033323335313635373533323836383435383B3C3B403E42424F50565B57525B62
69707782929EADC5CADDECEADBDFE8FDFFFFFFFFFFFFF9DDDDD7CCB5A39894939B999381
71636B706D5D594F4B47464643423E393938353537333637393337343233323034333735
3632333231323030333331303230303634313333322F2E303132352E30342F352B312D32
2F3030322D342F31302D32332F3132323330303232322E323034342F33322E2F31303234
30312F332F3532333134323231333631302E31333233312E34
2F2E3231332F3133313332323332342E33313133333136302D2E2F30332C303334383233
3333303035312F3430353031343534313234343030353532323631312E2F2B3631302E31
30333131303230332D302F2E3135343230352E343334352F303131333532313532323531
2F3734313332313133353935353437393735393738383C3E414244454B505B595E5C5C5E
676F768A99A5ACC0D8E8F1DFD2E0EBFAFFFFFFFFFFFFE0D8D7C7BDA89C9B989093918D7F
70676F65645D544D4C4445423F3F403B3C383B3936363934363539343434312D32333536
3432333032323136342F2F332F3434323130302D36313132312D3230323036302F323130
2D2E322E33333236323130322F3433313331322F34342E30312E30312E3331342D32322F
2D33323233333433323331343233322F313431303132313331
32362F32333530333234303032313033303330312E3337322F3332303131303433323334
3236332D33303133342F323033362E3033353631303230303034342F32352E332E312F31
35342F31343132362C343432303530323236333333343431333432353432313430353031
323334352F37353932353133363A3634333639393A393E3F414448464E565A66625F5F5F
6B6F8490A2A7ACCCDEE6EBE3E0E5EDF7FFFFFFFFFFF3DBD3CFC1B5A0939E9C938789887E
706F716B5F5855504943473E423B3E3A3836373739373937343935323435313534313237
362F33333231302F3334323333313036303031353331333433303033312F3232322E3032
33322E30313530302E302D333233302E31323036303132323032322E33312C2D332E3234
303532333631313030343332333432323532312E323130332D
332E33343231303130313132333230333232332F2F312F363135322E2E35342F342D3332
302F33353332343030313335323131343639323332333236333032322D2C3536302F3332
30312E35312F2D3334313630312E332F323332363334333133313432333833373335302E
32323330332F343635333A33383538323536383A36383F3C4045494D52585F5F5A5F5D60
68768096A8A5AECAE2E8E7DFDAE8DDE8F7FFFFFBF2E3CECAD6CCB6A6A2A9A7938F888276
71716E6060544F4D484441413D403B3B373C3A3A34373337383435363432343333303333
33362F34333A35343235333535313331313233363130332E302F312E2E312E32312F312D
2E302E323033313034352F32302D32333432342E30332D32322E3333312E312F2E313333
2F2E3034353134322F31333434353330313334303332343031
312E363133333131312F332E31323431313332303332333130313632312D3830322F3230
332E3236312F32322F32332D33303631322F3534323330323035313531332F3233353133
31302F3030343433352D33322F3732342F30353131332F2F313133313230302E34313437
3232332E342E353536343732393435363935383A3A3B3B423F444D4F545D5C5B59545C63
6C70849DB3B0ADC0D7DEEAE1CDD5D2DDE7E8E4E7E1D7C9C8DACDB2ABACB7B09E8F887D73
717774686156544E4B484640403B3D383B3B38373E3B3736362E35393238373531353733
30362F32353334313232312F33322D30363032313335313031322F3230312F2D2F2E2B34
2C3230322F3331322F2E30332E3235332E32312D3132332D353331372E322F312F323134
303430353232343533333135322E2F3331333234323232312D
3031312E3132323333303134333230302F3233373232362F33313232333332372D31312F
2E30302E2E3136302D313430343730322F3636323734372E2F332E333132303235332F2F
3535352C342E2F333537353333313233322E353335313332313336333233333233303233
323434333931353334373732343835393337393C39373A3D45454C54585B5D5E56575B61
697385A5BFAFABC4D3DFE2E3D4D4CFD8D7D7D2D0CFCAC6D7D8C8ACA39EB2B29C8F867C7A
74756A655A55504B464643403E413D3A3839363337393836333535323834373534363531
31313431303237303035332D34322E3232352E3134343032333231343232323132312F31
2E312E3033313633343032323131333132343231342F31342D3235342E32302F2F33332F
2E30303434323131333132342F322D32303232303334363431
32323332322F323233313331332F31323333303335323338333130342F35342F31343432
353329313033343231303237323034323332313733333231332C34323433332E2F353A30
32352F33342D333432322C36303331343133302F36373134353334342E32303034322F38
2D3437383135362E3432313235363335363538403A3C3F42455055585D5C595B57585965
6A79869CBCB1ADB8CED8DEDBDCD1C9CED0C7C3C1BBC1CAD4D9C1A494A2B9B19895897B7C
76786D645B584D47443F3F3E3E3B3D393B3A3B373930383534373335313138383534332F
2F2D2E323134303432362E35303334313231322E323233333231333031322C3331363433
34302B323234303330302F32342E3236313437313032312C30343331333233312E312F32
3132313338342D34323331303132322F343036323335343033
322D31313034332E31343232323434333331353834323030302F312F3531332D33322F2E
2E2E2F2F362F303431312F34323831353134382F31352E333730303235343735352F3033
3333342E3533343134312F2E343136312F3031373A2F323233353034313434302F323430
3131313035303232383935343737343436363739383F3C424752585B5A575A5B565B5969
697687A0B0AAB3BEC2CDD6DBD3C6C3C5C5B5B7B6BFBFC1CBC1AFA099ACB5A9988F8C7E7B
7E716B6056524C4742463F41403C3B39383737373832393431303536373533383434322E
342F3235312E3130343334353231333431332F353230302F323330323130342E3130322D
2D31302E302F31312D322E313032332D2D322D2F3131313133302E303232312F31313131
3031333331333232323331342F34322D323533303235332C33
3232342F2F323535312E34313130302F313331342E33363130342F372F2D33322F302E30
323032313532323033343330373431343030342F3235322E3130322D39312F3032323630
31343134333135333133313334313234322E312F313533323233313332322D323031322E
3534342D2F36323234333437383534363138383A3F3E42414850595957565951595C5A66
6C77839AA4A9ADB5B7C4CDD0C8BFBBBAAFB2B3B6BBBEBDBEBAA7A0A0ADAA9E91958B7F7E
7D736D5C57504944424244423E403B383D3439383A33363B353630373536353337373430
3132333334343532333135322E343431333130343235322E2E2F3231333431313231322F
342E2A3131313531322F3231302F302B323132332F2F30302F303232322E302F30323136
3834352F36313032323536322E31373331343331323336362F
30332E372F32312F322F323332323230322F323131323734323030303434332D34303333
312B3332333230302D323235322E3333323030343333333636302E323231352E33323531
373133313634303730302F31323235312F2F303231323738322F33323233373134303533
3035313334323534333135363A3837343535393C3D3F463F464B585F55565254545B6164
697187939AA7B0ABB1B8C5C5CDC6B3B4B5AFB0B2B9BFC2C6B49D9FA8AFA3998B91877C81
7F6E685B5751484844404143423D3D3D3B3739353336333434333630342E343731343432
33383635312F3231323431302F322E35343133333133323530303237303235342E353230
2F31342E31302E332E2E30332E2E33323232313031312F30323032342F2F343030303330
3137313434313331363132332D32322E36332F31333531372F
33343436312D2F31312F35323330313531323331332F32323030303330342D2F322C2C2E
312D3230342F2E322E32333132342E33342F2E3337383533332E2E323531323133342D30
2C30352E322C313331323332353532332F32323033363432353332312F332F3734303630
36352F39313731353438373438393535363537373F423F44484A515754585456565A5C65
6872848FA0A5A5ACB3BAC1C7C5C7BDBABEB9BDB8B8B2B5B8AE9E96A2A7A2968F8D857C79
7570615C574C4B4442463D3C3C3D3C3639373836393A3634303630323331333034333034
333334313132332E3833302E2F392F3033302E2F302E32323333332E3033312E32343030
3132352E322E2D332E2C2D33332F332E322E31312E2E3132302E312E2E31333334322D34
33323033313432342D2D36302F353532322F322E31312D3132
3030302F3131322D2D3230323331333333332E322E323032343132343231333030312F31
342D333434313131313131313531323132302E33343635313333343430322E3434323132
2E3432312D3130323236342F3334333032313632373536333133332C3632303733323234
343736323336352F3233333433363A34383734383C4243474C4B4B534D5154545A595768
6874848BA4A5A2A8B8BCBEBBBDC3BEC3C1BEB8B7AFB9BDB7A89A959E9D9B999B8D7E7976
756A5F59504B454547413D3E39403D393836373433343531343A34353535303133313233
3332323335343133333334312E3330333131313135313132373132303232303130323032
3033323030312E2C332F332F323430323032323232322F31302F2F302F2F2E312F353036
332C30323032312D3435303233343133302E33332C31343030
353633322F2E312F3333303133333436352D3334323032323332353534332F303232312E
322C2E2F303234363035343131353533343331302D2F312E2E3231313431313034302F34
3130302C343133322D343131303133332F34303433353432332E3330302F333430313230
35383434363332343534323434363732353438393E394143494E484F4E5651545558585E
6B708193A3B1ACB2BBBBB8B3B3B3B7B9C3BBB7B5B8C0BCB4AA9B98A1999489917C7C7977
6C635D55504B484644423E403A3C39373C3737393535372F3332323332303130332B3132
323432303530302F2F3034332F2F2F32322F323030313331323233313330322F302E2F32
2D34322E322E3435322B33362D3236322E3232333130322C32313033312F2E3231313632
3231313132303435342E31312F313130322F2F3231322E3432
2F30312C2C34322E312F3032313532313231322D332F323233312B312B332D3333303131
333231313131362D363231303130312D332F3133333832333130362F322E2A313330352D
2D322C32342F363132352E32343234343031333433313236312E332F3332363134353734
303133373135333733333635363A3E3A3138373E3A40454845474A4B504B5550565D5C60
6B738393A1B7B3B7BCB9AEABB0B3B0B1BAB3B5C1C6C0BAAEA697A1A49D8D8A8274797370
695D58534B47424A433D413E443A383D363B38363536343731352D333432302F34333133
3234312F313432342F33312E3432312F2E2F2E302F32343630342D31303032322E322D30
2F312C31323030302F322F3230322F35302D2F2E302F2F332F36303434303531302C3330
332F3434332F33363031312E30323130322E2F313733342E33
31343435302C312F313431352F2E3130342E2D3732322F2E333131342E31342C2F313034
313232323535333732303231312C313432303032312F3233333233323234333231323034
3A2E302F313133362F352D333233343231313933332F352E35303133343432362F2E382F
3434362F363432323435353137383A383A363539393F404349494A4C4D4E5451535A5E5F
6B737C8FA0B5BFC2BBB5B3B1B9BCB9B1ADABB2C1CEBFAF9E9B8FA1A19D8B86796E73706E
655D574E4B45474341413C3C3C3D3D3739343A363235313234322E31343431342F333231
33312F2F313133302F2F30322E332C3031302F303133312C2E352E343032313232332E30
3032302F33322F2E342E302E2D2F34313135302D2F312D31333330322E32353132323532
302E3033303332313230313232333132322F31343331303631
3A2F2F3030302C3332323434303432303332303632342D303531323032342E312F2D3530
303432372F3035323331372F342F312F2C343330322F31342F2F3330322F2E2D2F363030
2E2F36323331333633313033363431302B3436313034332F303332343230313533363133
3434323434373733333330373637393C3B3B413C3E41434648474A4D4E4E52555656605E
68717A8F9EB3C8C9BFB6B6B3B6B4B1B4ABA8BACCCBB1A5929199A4A5938881756E6B7266
645A574F4E484542404142393A3A37383939383933333332342E33323735353332333332
34333531332E3334323331302E30323230322E33323033312F3133322E2F31312F303230
2F322E30323131312E32313131302F322F313033322C322F3034312E2E31323432333433
363237343336343333302D35323431312D32332E3433302F32
302E342B312E322E32302E333231313136323130302E35313034312D2E362A2D33322E32
3032322E30322D303535323032333231302E2F2E2F2C352F30303532312C323434343233
33353032323034343333312E3430343330353534342C3131303236313531333332323434
3A343433353333353436333936333B3C3A3A3C3F3D424346494A474D4F4F56545458595D
656C7D9199AFC4CFCBC5BBBBBBB9B8B3AAB2C3CCBDA5979B98A4A8988E857E76756A716D
635E544E4C4B42443F443E3B3F3C3936393835343533323637342F323034363232303031
2E2F3831302F32332E2D31332F332E30302F3033312F322E2E312F31312F333631333430
31312D2C312D333131302E2F3030323032312F2F372E32322B31333237322F322F313235
323032353230333031323031333132312D32312E312E303131
3330362F3033303134343333323031313330362D363131332E2E2F312F332F2F30333032
2C2F2F303130303532332F302F2E322A3430312D30322A332D3131313132323434332E34
332F3430363335312E2F3134322F3331322D35372E312E2F313533383130323137303235
34333333333233353634333635313937383B3D3F43434845484C4B4E4F4E4F5156595B5D
636C7D8F9DB3C3CDD3CBBCB8C0C0B6ABACACBDC3B0A192979ECEE8A7857F807873696967
665C56504A4745433F3C3B3A3637373B3630373433343734383033342D35383235343236
333031342F2F30313334312D3432322F32322E2F333032342E3031343131343032333134
2F30303032343331322E302E2D302F3233303032312E2D3331302E2F3030363232353532
31323331352C35342E3035323031302E342E323330302E2D2B
2E32312F3430352E303131332F303134302D2F2E2D353132333231313332323431302F2E
2D2E30323331363232323030353037333330302F343334313433342F32313931332F3230
342F34322E30313334332F34302E303131352E312F2D312F31342F363432323530333436
30323331382F323636383438333B3739383F3E3D4143484B4A494B4C4A50525454565B5E
616E808BA1B0C4D0D2CEC9C5C1BFB7AFAEABB6AE9F908C8F9AE5FFAB84837D7C6C686463
605A5A4E494942403F3F373939383B35353231323631343837393435323337332E323131
30322F30312D30333032342F2D3032332E2E322E2E323230312F323231322F3433303131
2D2F34343432352F30312F35332E2F332F2E31302E362E2F302D302E2E35332F2F2F2C34
33312E32313531363031333231313432332F2D3132322E312E
3433323233353133303133372E2E322C302D3133303132312E3332303333363231323230
3332362F3330303134323231302F2E2F332E323235352E3433312D3132363331312E322E
3232333331333030313431362D323530313432353137313231353132312D332F31333234
32333232353531383238333734343836393D4045434B4B4B484A4A4C524D57515153595D
646F7C8A97A4B9C7D3D5DCE9D7C8B7AFB0B2AFA1928A8C8C8DA6AF907F7A7A7A71666762
6055534D494347413D393E3A373A3433363437323432343736312F35312F31332E2F3135
34333332303235312F2E2F31313232312F322D343134302E2F34323130302F3031333234
3235332E2D332F303331322E332F312F2E2F3232332F322F35312D373132333032343336
323234313530353031313333332E312F2F362F313333312D30
30332F2D2E3131343032322E3433302F31353333322E34323031313234302E2E31323232
31322F3032313130322D32322F343133302F3230303230342E2F34323132303131353230
322F36373131303433303432323231302E34312E3132342E333232323134342F31313330
333536323435333635323639383B393D383C4242444648454A474D4C515253505456605E
6B6B7B84949DADBAC4D5F2FFFCCDB5ADB1AEA99B91838A89848D8D847D7A7678716A6362
5C55514F4944423C3C393B3736373A353136353632343436373235363331313432323033
2F2F3030303230312E2E3134312F2F312F3333302C3230342C2E2D2B32302E33312D3032
31302F313130302E3035342E3035342F34322C2F302F3130323231333031303234312B30
35353131313432302F3130313230312E2F352E31343335312F
34323134363330332F2F37333335342D30332F322F33343330312F323332323034323234
32313134312F312D2E30302E332E2F31312F332F2F2E362F2E322E34312E323531343031
31303330323331322E30313230333330342E31352F343332313230333633322E3630312F
3536353634363436383636383A3337373B3E3E414A484C4E4A4D4E4B504F5157595D5A5D
626D7D818794A1A7B5C6F5FFFFCEB0B0AEA79C95898680828785837E74777775716F6362
5A53514D504842423B3D3C38393637363936333834373335333630313034323632333231
3232333135302E32363432313231312A332E312D322E312F303332302F31303334322F33
3233313033302F34322F31312D313134313235333130322D2F322F313030343232303133
33312F353332362F312D32303131302D2D313331343031312F
2E3136373C3434333132323131312E2F2D31302F30302F352F3533312F302E322E2E3030
3431342E35342D313232362D3233312E303431312E303034342E3232382F363233332F34
323032313135302E313432332E323236312E31353235342C303432333234333734353633
343434353235343237363838373A363B3A3B3B46474B4F4F444D5153505B595B565E5960
636D797E868D94A3A7BDDBF8E5BFBCB2A598938C817E7D7879797877786E71726C6F6060
5B544B494A43433C423D3B3C363837313335383434353A33323430323633302F332F3435
302E343131323434343433323434362D31342E2F32323031333134333330352E2F332E35
2F312F302F2F353332342D2F30312E3030312D332E2B3232313131303231333234323231
3733353532312E2F32302F332D2C32312E3135323435333036
3132323C373B32352E36313132322E30323434303230313232302C363130323132332D32
2B31323030302F3033322E31302F312D2F313030313733342E33312F383334312D303130
3333302F3431312E333430322F30333132323B31343533312F33303632323530302D322F
333134323334333436383536373738383D3D3E404650514E4E5252545659595A5A5A5D5B
646770768B9992959DA8C9DBCCC8BAAFA9928884817E7A79787F7F7573716E686A68635D
55544E484842423D3E3B38363E353536373434363330343534332F33332F333332343731
353035323034313131322F3432323133302F2D2F322E302E332D2F343330303131323034
2E3330312F2F32352E36312D302E30313030302F2C332E31323330353030353233353737
362F34323434333031322D3030303431313131323234322F33
32333433333334363432333232322D2E31302F30302E31353334333531313233312F2E32
332F322D2F33302B32302E31302E2F332F313231323032322E3133313532303232342831
2E353434312F2E333033302F34323532333033353333363132312F3535362F3233323433
30333532303433323335353434393C3C393D3C41474C504C5351565858545B5D5C5E5C5D
6464686B7F8D918E929DACADAFADB0B4AF9C8E807B7F797B7C82817972736F6864625D58
53534F474240423E413E383335343638363333393038323531332F333635323531333332
3432322F3233353231312E2F3131333231322F312D2F2E2F3133322D2E30303634313233
3030302D33303431323030312E33302D2E2B31302F2E2F3030333332363035333232363B
3536333033352D34313230333331302C2D2D31302E34302F30
3232332F32333633342C31323232302F302F303033303232303131312D322F31322F322F
2F3331333635312F302D313435322F3435332E312E3331372D332E312F362F3132323035
32343335333033343032303132313832302E3532323432333333362F31342F3234303232
3330302F333633393534343838373B3A3B3C413F474F585852515154555B5C615D5D5D5F
5D6566677078808490939492959EA2A4A798888783827C7B7E7C7B76716C6A64625F5B5C
5A564D4E4842453E3E3A363536343632333434323433303B333133323132323331323130
363331352D33303431312E2D3133333431312E2F32332B302F312F31313233312F313232
302F2F33312F2E312F31323130302D313032302F312F2B2F332E3032342F30333134373E
3838313433312F2F3332323132313135313232323434332E2E
313131312E3232332F2E303132302F292F302F2E342F2F3430323434323233302F33332E
302F2F3031322D30322F303330383232312C2F312F3134342C353131333231362E353033
30342F2F3237323131323233313230353331313430343236323232333435333234343430
343736352F333836333835353738383F373E3D414449535252555253545A555C58635E5F
5E6063666F74717B8A8D8684898F959C958887887E7B7A7E787B767269696D6458605D58
5E625B4E474742433B393839383632363532323335313131332F302E3231303131343031
302F2E3033302E32343130303331322E2F32322F2F323132302C31332D303032322F3034
2F313033322F322D2F303532323232332C3030322D313130303433322E312F2E34333539
3A3532323131302F302E2E2E312F3033303130333234312F2F
35303232302F3333302E31312E312F3130323133303331332F332F342E34392F3333312C
342D31322F32332E30322C34303231312E2F32322D34313131333132312E363131303134
3637343334322F32333432323232333333313032313133363634313331302F3535303330
3234323436373C3C3F3A36383B393A3F39383E47454E4C514E575756535654595759595E
5C5E6266666970737E837E7B7B848C928B858681797978747877767067645E5B5E5C5460
5E5E5A4C4D4541383B343B36323735343632333230343536343036303034333233343333
30333134333431322F2F2E2F31322C3031322D322F32322F3330303131322E333130342F
30322E2E32303130322F333130363131312F302F2D30312E2F3431333134302E34323130
333133352E312E31302E333336342E34343133312F2E342F2F
323232333030343330322E3230303230313232332E34313231353335303131322F323037
2D313233313132333033303131342E36302F3032342E31323433332D2E332C3736332F34
313231323033372C3134312D363433312D34313330332F36363234333631313035323932
333435333539373D433B343B38393E3A3B3B404243494E4E50555551545754565B575D5E
5E595D6163666770756E716D6F7B8F8B7E81808378717A7873736C67676262595B56595F
5B5757534B4340433E373A3A34373430343235373338363330332E313332323433323430
323032313132313233322E2F32302E3132323133303235303233322E353031333131332F
32302E302D333332312E323031323231303232312F33302E31312C3133342D323130302E
31333133362D33332F2F3234333032323731342E3230332F33
343131313630353C383432312E3130322A3233332C30322E32343134303536363030332D
312F32302D352E303031323030342F322E333130303232302F3033313233303130303232
33312B352E332F2F2B302E332F3233303132303134313030323337332F332F3131333433
2F343631382F39383A3735363C3B3C3C423E414447434A4F555055565556565458585959
585B5D5B586163656E6869696F727879727D7C7D73747471706C6863635F5D57595A595B
58514D4C4A473C3A383A393138343136373835343036363232302E3D2F2F353234323336
3231332D3531302D312E3234302E2E32312D352F322E32372F312F312F30363030302E32
3230302E3130323133343131313132313030302F3230342F312F2E3630333333362F3033
32303230332C2F31312E312C2E32322D313131323031303231
3332342F34313436393531322C313231313231313233343230303331353130323131332D
302F2D30303431362D333030302F323031312F322E35303033332F3033302E332E322B32
2F34332F33303333322F3231343033353332342E313130332F3431312D33333235313232
322F33353333353538353737383E403D403D4041444449494F5252505959535757585757
595D5A585A5E626563676469686A71737774746A71726B6E6865605F60625E5A56595751
4D4E474A4A403F433C39363A36353637342E373532322F302E3131333231312E3336352F
342E2F31323135323030313330323033302F2E2F352E3235312E332E323433313332312F
2F30302E2F2C332F3134302E31343432303133302E31323330322F302F2F2E333330332D
323335333333312F323130352F2F2D33313231352F30322E31
303334363234353739363137333131303237323437363330312F2F32362F2F2B33332F32
30302F3134313333352F3430353032322C3133302E2F2F313330322E3434313135303432
313132323031342B333130303230302F3130333232312E33342E3330332D2F3335323231
34313730333335343A3934373439363D3C3F444143454C4C4B525350545052525A585759
5A5B58565E5F5C5E61646261656C707375716A64696F6A6865615C5D5C5A595656515653
4F4C4D4644423D40323436353337313139353336313038332E322D323332323133303030
3432332F3232302F323132312F322D3231312E332B3231323532382F2B302E2D35322D30
31332B322E3232332E312E30332F32302F3630362E2C2F2C2D322E362E322D302D332F33
313231303135322F2F322C2F323330312D312E2F3032312D33
323130313132333138343538333331333034342C303332303331343530342D2F2E2E2D34
303432342E332E2F313330333331342D36302F33322D2F3132333032333030303032302E
33312F3031322D31322F2F31333134302D31353334343531313432303333323235363833
303232332F3234323639363A37383C3E43403943464849504E5350515352535458595759
62645E5A5C5D5B5D626160626567736E727066636A6868645E5E5D5F5958575B5852504E
494B464341403E3815372C373137333C36343333333532342D30342F2F2F31373436332D
30333231353633313231322F322F322F34363233332C332E3530302F322E322E31323536
3030312A31303032332F2D302C342C33312D30302E353232303132322F332D2F2E343032
3130303034313330313232302F2D2F322F2E303232342F3131
3134342E3431353230312E332D3534333231343532343530322F3132302F2E3530303330
332F332F342F3233312F322F3231333332323231312D2E2E302F31302F332E3033343131
30362D333435313033313331332F342E3332303330303135323030303432313232313030
2F363035313632333637343A39383B3C3C40404444494E4B4E4D51505351525453535456
635E5E5A595D595F5F655F62686A6C6B6C696066646A69635D5C5D595253525352534E48
46434948423D3D3A3B392F36363533343336313230332D323434352E3032303136303131
342F3132322F313131323131353531343031303233312E2D323233323030322E2F2F3031
2E332F2D2F33302F312F3030313332323032302F2F302D2E313031303031322F2F2B2F2F
333232313431323032312D32333433313331302D2E31333032
333031392E3232313032332E352E35333033333432302E34313332302F31292F2A2F2E2F
2E2F333430342F3134343130313232332E2F332F35312E3233323231343031342C323030
2F35332E34303232332E343530313633302E30333330333033313031333330342F2F302E
323337333230373435363338373E3A3A3F4143424649454B494B4C515154525659585B5B
5B5B5C595857575C5E5C60626869666467635E62696B6E66635B5D5855534D4F50514B47
4342443C3F3B36333C3237303930352E343232302E323635343231323031323032353233
34332F30322E3233323233333330352C31332E2E332E302E2D342F313132343032333332
313130303031333331332D31312F323030302B3032313132303332302E30303631323032
3333312E333430312F2C3332342F31302C3132342D34333031
33342E323033322C332F3330323231302F312D3230312E33312F323432303133342F322E
31362D35303431302F35352F2E2F30313133302C33312E312E30353432312F2F30342F31
31333333313230303332333033332F312F32372F3631323230302F333331332F302D3434
382F32323131343A3537363A37383D3D3E3F3F4141424648474D4F4E5658565055535654
585E575C5C595B5C595C5D6164655E5D635E636165686562625B5C51574E4F4E49474A48
45433F3E383A373438363933353633363530333332302F2F3332362F352F343133333231
32343231312F343234322E34342E323031313132323533343030333233333233322F2F30
2E2E323237302D3532333434322D31303132302E312F313031313133322D32302E2F2D31
3331323236322E322E2F312F292F3434323135312E3232332E
332D322F3032302E3332303434303232302D323631352F3532312E33323035333130312F
36343131322E3232353031313031342F2D3135322E312F313233313532302D3430342F2E
302E32302D2F3432353235322E2F3530333031333134323335322E30323533313430342F
3232363435353536373535343B3B433D3D4443454545444A4A4A515159595D5456535454
5B5A5759585A565E5D605C636260635D6363676363646461605857554F50514F504B4941
41403F3E3F4139343E3E3636343437333434313231303331303330363334313233363133
313030312E332E32322C32302E322E32302D2E32363236363030303033372E332B322E34
35312E30352F33343331303433312F332E2F2F2C2D2F312F2C2F32312F33302D3031342F
33313331303031302D2F333231343134312E3230312E2D332F
30312F33312F2E30342E333233322D323434322F302D313032313332322D322E2F303033
32313035363133332F3132323031342E2F30343132313031313234313432323231352D36
2D303030342E2D3033313131343334322F30313034323531313332353332313337323330
3336323435383439343837333C3B4141414341464443494D4B515354585E5F5E59585856
5C59595D585B5A5B5C615E666062605D6563635F64605D605D5B51544D514E4C4C4B4546
3E41393A3C38393539363032343332383734343537323332362E323432332E313530322E
30313032302D302D2F2E32302C332F332F2D2D352D323835352F31312E2E3131312F3130
2D2F2F302D302D343631323131323331313031312F33313032322E302F2D2D33302F2C2F
30332F2F3131322F3334332E2F2F2F303333312E312D353132
322C333130323631333432303436333333302E2F2E34352E302F312F3131303030312E35
30343136333030322E333534342E323432332E3330313331342F2F31322E362B312B3330
312E33302F313333363333313336313332313333303332342F3134343030353234383134
3137323035393633393637393F3E414040434444494C44494A4B5357585B5E5D5E5B5E59
5B5D5D5C5B5B575C5C5F635E5F5C5B576164636060625D575A5A504E4D4D4E4D4B454641
433D433F3D39383B393038393931353636353331373032333234312D3034312F2F303133
3731353432313331313530302E312F332F33312E2F30322F33343030303132313035342F
302E302F353532322D33302F303033362D2F2E2D2F32312E322E2F2F322E392D31313130
37332F312B2D3033353330323131313233302E3130302E2D2D
3235312C36302D2D2D31322D2D34353333342E333432373631333232312F2E33302F3339
3232342C3231312E362C332E30322E2B312F2D30313132332B2F35313132323133312E30
31332D32332F2F34313431343233372F3334313032322E32323133322E32323130323532
332F30343138343136353C3C3E41393F46434549464A4A4B504F4F56565A5F605B58595A
5D63595F5B665B5962605E5E5B595D5F6065625A615D545456564E514C4B4B4D47434140
3F3F3740383D3C373A3C3137373637343130343231343034353531363233323332323131
333230342E2F2F2D342C362D2E302D3133332E32312C34312E32362D31322E2F322D2E31
323131302F2D2F313234302E3336343230312E332D29332D2E3031322F2F353035313232
3432322F31313032323330352F332D2D30322D342E31303231
312F2E31343330343432302E2E322D33353130302F2F3236303332363134313134353231
322D32323130332F2E343134332F33302E312D33342E313632343233302F322F2F35312F
2B343131332F31322D362C30342D323232332E313135302F352E2D313532303130363235
33363336383136383837383D3B3E403E4245484A4E514A4B4A535255545D58615F5B5C59
5A5C5854575959605D6464605D5C5D5C5F625D60585250535555534F49484A4A43463F3F
40393C3B393C3132383C473935353535333431343634333133362F303331353031303031
33322F2E3232323233342E2F2C2F323130333132303233322F32302F3233303231312D2D
31322F2D303030302E36312E3330312F2E33302A31322F312F3033323334363136313131
3534363033353134363130322E2F303033342A32312F312E31
3131323231373231322F31323130302C342F32363232302F2E31323133303331302E3034
33313130312F32302D322F332E31303330302D2F33312F2F30322F303131302F38303131
343130332D2E333232323135302F353434313433332F30302F2E3331353030332D2F3732
30373531353637363C3F3C3B393B40404751504D4F5251505052535556545B5D5D5B575D
54595850575A5E5A6161615B5B60595C585F5E5B555050545550504A4F4C4C4947464138
3C3C3D39443A353A383D32383334393437363235373731303232313332322F3135322E32
303131333230313432343130302C302E302F3030323331323033302E31302E3133342F2F
2E2F302D323232313133302E3431302F32302F322D2F33352D332D353033342F31313433
3133312F35312F30322F30302E34332E2E30313131302E2E2E
3333322D31343334332E2D2F2C313233342C3032323232323032322F3132333232332F2E
342E3131342E313031312F2C302E2D2E323231372D332E302D30313130312F322E313230
35303236302D3030302F312F3131312D3430323133362F36323230332F30353235343433
333135323436383942463D393C3E41464C525157555054514C4F50545553555957585B57
545B5352595C5F5F5E5E5A5D55545D5C575F5E5B5251545055514D4F4A4A4B48433C4241
3D38363637393238393534313130343237313333363433333437332F2D2F353233333130
2E3233322E3333332E342D2D2F37332E2F34302F322E363331333033313433312E323031
2F333231312F312E343133323031302F302F2E31322B2F2E303230302F31303132333533
2F30333030353233343232302E32323233313130302F323230
332F313530342F313033363230302D3230302F3631302F3031322E30332F313130303230
31313131332F3230303033312E3430332F322D332F302F312D303230303132323232302F
34302D3133303332322E2E34303031302A313131373135322F303631333234342F353131
38303533383734393B3B3B3F3B3D4244525859555455535555545354595455565E585854
575B58595D575B5D5D595950565C585E5D5E56595250524D51504E464A4B4743403D3D37
3D393B383B35383836323335343232382F34333333373235343433343133323831323232
302D322C332F3131332D312C34322E322D312F313330332F31303130303233352F333031
332F2F2F30352C322B2D313131312F2E332F3034313136313330332D2F332F332D34342E
3234332E3135303132303233312F2F3331322E2F3232303231
33303132342E2F332F3231312F322E323134332834342D2E2E332C3131312E322C2F3033
322F34332F31343131363232313135302E2B342F302D333030332F2C3230302F2F2E302F
2D3234313633332F312E322E2F362F332F342D312A30312D3635342F3031303330313532
3833333234343737393A3B3C3C40424D525E5B555353545055535756585657585D5D5C55
56585659585458575B5658595D5D545A5F5C57564F4E4F4E4C50514D4A4A4744443D413B
3A3A3C3937393732332D3A36303034352F36322F303131343431343232323532302F3031
3334332D302E342F2F343230363332312E2F3034332C32303031313032332E2D3130312E
2D30322F2E302E2F32312F3130322E313331302F2F323131322F30322F31342F35313231
33312B2E2F3431343230313034302C30313233332F34323235
333232352F342E2E32302F3033312E2D2F3134322E31323234322E2F33323232312F3135
33373231312F33302D2E2D302D312D32312D2F302F2E322E2D312E30303132312E32332F
30323334303135322F312D303234312F3331313233312F3032313334343331322E323332
36323535353B3434383C3B3B3B3E4445555E5C5A5756585356525457534D5862685E5856
54575354535A505853555455585B5B5D5D5656534E4E4F52504C4B4F4B46464240413B39
3A3C3A3938333734363537323331333030332F333032312F312D33363236332F34342F32
363330323230322C32312E2E302D372A2F312D3330302C312F303333342E2E3030333230
32362E32302D34312C343230342D31302B2E2E3032302E3431312C30322F313131323134
32342D2E30302E3430312F2E33312E2E312F3033303130322C
33303131323431323130332F2F313330332E313433312F2F32312F31313133332E342D33
32332F312E2E2E342C302F2E3334302E313233342F2F2D322E332F313031312F31302E32
2E312F333130342E2E2D322D30302C333033332E3434322C2F32303037332D3330363233
3232343134363637383B393E4041454950575C5B555756565554554B504F5C60695D5751
5451534F56545557545456575A59575C585256524E5152524F4F50494947473F3C3E3B3C
3A373A383638343535353133332F2F2F34353230342E2E2D2F332F34312F303432313231
31323232323230332F2B3232322B2D2E31333034333333303330322F30323432322F2F33
2F2F2B332C2F2E3231302E2F303231302E2F2F3331332F30302D33312E31323332313134
323030333331322E312D3231312E333336342E343431333130
3436313231322D3235303032302B34323135312F3131312F322D332E2F31343030313134
3333312F2F332F332F2F332F2D302F332F302E312F302F31312E30303030302F30302F30
3230332F3130352F31322D333332332E31302F33303131312D333130332F31312F303737
363639303436383A3935353A3F433F454B555A565A555352535252514C555C61615C544E
4D525256535251505452585A5A5A5C56565252515155505451514F4A494A44413F3D3A3A
3E34393C3135323435352F3232352F352F313131302E353630332D323332323435313234
2F2F2E3436302D2E3230302934312D312D3132333035353031352F3333312E32302F2F34
2D302F302C333231303236342F31322C302B3031322E30302F30342E2B312D313232322E
342D2F2E2B31322D303636332F2F3234333032323731342E32
3232303430302F34322F2F30312F3231313032313230312D332E32343230323530333431
3530322D302F302F30312E312E2F2F33303230333330312C2F31302F2F3331303032312D
332C3032332F2F332E33373230323034313130312F33303334332F33333032313331302F
343A32313034353C3738383B3F3F42424A505656585E525453504F4D4C4F535B5654504D
50504D5257575053565357575B5A575953504F4B4E54555C59524D49494A45403C403B3D
383737373537333536322E2F332F3234353031343530312D383131333333353332343131
3231313033302E2D33302F31302C33312F302E3135322E34323133333334322E2F312D2F
312F3130333030312E2C322E333031312F302F3233312C332A342D2B322E2E2D30333430
31312E34302F2E2D32322F31312E312C2E32322D3131313230
2E322F32313330322D322D30352E2E383132333234303032302E303532313431302E3530
2F33332C3330322E2E34322D2F2E2F33302F2E30332D332F2E2F30323032303432313232
30312E2D3134332F30302E35302F3534323334312F3032333032322F302F323133343036
36333831323739363A3E38373E424242485058545B5955574F534C4D484D50514F4D474F
4A4D4F5256504F55585B5A5E59565555504C51474B4D575F5C504B49434446423B3A3B39
383A373633343135333633363033343330323231303134343232333534342C2E332F2F33
2F2D322D332D30332F312F2D312F2F322C32372D2F30332E2C313032313232332F312D30
3330313434302F30322E312F32322B322F312D332D322E2A2F2B322F302E31332E2E3133
32313232313334313331312F323130352F2F2D33313231352F
332F33323133313031313030303530332F33332F2E3230333036312F302E2F2F2E313035
323033302F3032323233332F2F332F322B30303235312E2D302E2F302F333231322E3232
30332B2F332E363733312E3031302F3232342F333731313030312D323035343132323331
302B353233313839353A3D3D3A3D414448515C5E59565951544E514C4D4D50484A46464B
4F4D53525653505053565A5B5E565155504E44454A494D5556524C494A41403E3E3A3C38
383838323836353334313133333232343031332C32302F303333303432383530332F2F35
3330332E303333313232302F2F2D3033312F2D313032382D36302E303232303431312E32
31302F33313230303232312D2E2F2E2F2F303131303031322E3031322F2D2E2F342E3034
3135323332332D323335322F2F322C2F323330312D312E2F30
2F31302F2E3130322E3338312C353131313330332F392E34333336312C3330322E362F31
312E2F332F2F352E323132322E2E2D333130332E3331303030322F30322F303431343433
3333312E312E303230323033322F32323335312F302B343530303330332E333030323231
35332E33333337373B3A3B433B3F454548535E625B564F54514B504E4E464B494B48494A
4D53555D555353525652565657535958524B4B47484B4950554E4746453D3B3C3E3E373D
3A383635353134343433363234322E34323132342D3031332D373337393136343233302E
2F34312E3031322F313333312D2D303332302E30332D30312B3032312F2F34312E2F3431
302F2D2E2F313230303231303134303633302E31342E2D312F2E2D312F30323432303131
302F352C3434333030313330313232302F2D2F322F2E303232
303031323030363232312F352F32343230342E32313131343333343030322D31352F3431
313534353132362D33392F3431323036302E2F2E313034302F323031303230343331322F
2D3230322C333331343630353332303133332E33322C32312F2F30363131303234313033
31303433323437363A323A3D3E4042474B515B5C5C5754564F4C4E4B4F4C49424B4A4744
4E525E695B54545251525552594D54554B474542474A4C4E514B4B413F3E3F3F3E3C3738
333734353334323431323433352F3330333530332D3332303331393B3C3335363A353235
32312F372E322E332F34312C35302D30323132303433342E332D2B30302E2F3630303033
302D2B2E2E2F32323030302E34312C2F313030342C30322E32302E332E342D3232333332
30352F323032302D3631323032312D32333433313331302D2E
312D2D31333434363534323430332E303032323030332E3032313136312F3131312F3031
333532342C2F2E333536343032313434322F2C332E30332E2E3132333332313031333031
30303236303037323033323130322C2E32352D332F3532302F332F33322E333731323231
3330353430303738363939393C3C434544494F5955545355504B4C4B4B4647474348484A
4D505A5E5A4E4C5651515153514E504E4D48454346474C4D4C4A4443413F413C41393637
3434373632363334303932322D3533353231322F32303230353435373B3734323634312B
2D3331342E313335313032352A303233342F2F303132312E2F342F333031322C2C33332E
332B2D3034313031312D323331322F32313232352F3033323132302D2E30322F332D352F
302D342F362F2C32343430312F2C3332342F31302C3132342D
36342F2F3031333234372E342C303133322D313330312F35302E2F302E30313033343037
3337302F3035333335332F2E32313130302F3331312E32312D302D34323233312A343230
312F2B32323033303232322E333432333231303632322E312E3434383431303134343234
33392F3231303338363B3E3C3C4142424A484D4E51524D534C504B4D4749444847484948
4C525450535253515256514C4B4B4D4E4A424846454A4E4B504744423E3C3B383B363A33
353334333733302F35323431313333313232313432322F34303036363633343432363232
312E333031372E342E3233313230312F2F333234342F343130352F322C31303033323131
2C2E2E31353131332F332E312F302D3034373331332F303036322E302A3132312F2E3132
2C3333303332303430322E322E2F312F292F3434323135312E
3032332C31333035302D32343236302E342D2D312F3032322F2B322F3230322D332F3135
2F31322E2D2D3031332E312F2E3031302F3230312E30332E323231343431323131303032
343332343531313031333430322E3132313030313531312A32313234303233312E342F30
3034323330343238393C403D3F403E434748464C4E4E49474A4D47474C4946464445454E
4F5458544F4C514D514B504D4B49484943494A424B484C4C4C45433F3C3C343736373A38
36373531343330343137323332323131303430302F2E3130322E333330322F3232323133
2F313336302F31363032322B332F312E332E323432322F2E32312E2E2E3530332F2F2F31
312F312E2D31312E30322E2F332D32333134323331303430303130322D332F322D2D3030
333032313134302F303031302D2F333231343134312E323031
322E31332E302D3630332F2F30342F2F2F38342F312E2F2F32302D2D2F312E303132302D
30302E2F2F3031303537322E332F32312F2B302F2E3034312E2E3030312F332F372E302D
333230302E3431313432352F2D2C312D322F362E332F31312C333133322D342C3332322B
3232343137323235353B393D3B3D40423F44464D434A4845494446494747464D4B47494E
4E5759534F50514A4E4B4C494E484A4546494A49484A444B46423E3F3B393A39373A3837
3333373036313134343133333130302E303131302E313031302C34312F31333234323230
3032332F2E312F3234323130313231312F3032322D3130312C3331322B3034322F2E2F2E
322F3431303030313033313431312F343332323232313430302F313132332E322C322F32
33353332333134323031322F3334332E2F2F2F303333312E31
35323032323130313431312E322F34313432303230332F302C33302F302E333233313233
3432303031332F2F362F39343230343131312E2E3531343034312D2C3134312F2E323631
34303234333034303231332E3133313432323332352C323131303032332C313134323432
3132342E2F35353836393A403E403C3D4146454646484946444844444847434149494A4D
4D53534F4E4E4D4B484C4A4D4B494645414849474944474B4144403D3A37383637343739
323233323231303130333431352F302E333231303332303635322E33303131303232322E
342F30332F3531323031352C31323231333333322E31313231312D2F2F2F33332E2D2D2B
3131362C3232302E342F2F30333330302E3335333133392D313030312C3332302F332E2D
312D2F3330323030312D3033353330323131313233302E3130
2F332F313133302D3231312F2F302E2D30363131342E3132312E302F30333634312F3131
302E32322C32302C31323434332F342E2F312E3132312F302E313334352F342F2F333333
342F2E2F343232332F32343730333234302F323130322D33323032343633343332353232
31313031313532363733373D413E3F40434344424145404744474545474A4645434B494A
4E4C4D4C4A504E4C4E4B4D4F4A4F4C464546494B46474846423C3D3E3A38383835363333
35352D353635322F2E31372F323332312E2B322F342E2F333134303532332F33342D3330
2F2D3437302F332D30312E2D333431303333312F31332F2F3031302F2D302F3031332F33
2F333230342E303230302F32313030332F33313032352E3130313132302F2E342E2E302F
2A3132313035343535313032323330352F332D2D30322D342E
2D313331303430332C2E302F3030332F332D302F362F2E2F2F332E2E2F303234302F2E2F
2F2D313133312E3131303231333532332E322C322E352D312F362E333331333131323233
30312E34353231343336333130312F30313233322F33342E2D35322F312E3432312E3031
2E313532353034343539373A3E3D3F403E433E3F41433F4544414748454647474646474F
494E51524B474D4C5053514F575A534E4B4A504F4B4A4B47453A3D3A3D3735373B343331
3335333030342F3031343434332D313434323231302F3133323435303131322F2F322D30
2C312D332F302C322F32343131313032313230343531303131303131313330302C2F2D30
3830313532332D31313533323331302E302B2E3031313033302E332F2E2D33303430322B
30312E32303531322F313134363130322E2F303033342A3231
31312E322F342C2F303233332F342E2E303233333630312F312E2E2F2B2F332F31332D30
3231302F33323030323234303232342D33332F2D332C2D34342F322E3632322F32333032
2F3231333430302E333433343131312E332D303332332D34312F343133323431332F3234
343232303531343039343B39383E3B3D3C3F41404246453E47414644454146454B484C4A
4B4C4A494A4C48494F4D555A575A4E524B4E4C54524D4844413E403A3939363A38363730
323733373033312E3235302E2F31313231322E32322D382F2F323334302E332D32313133
2F30322F353131342E3630313232332E3130313131332F3130303232302C31312E352F33
3135333030313130333031343232303331302E333131322E2E312D30302D353231323030
302E2F35312F2E3432313330322F30302E34332E2E30313131
3031302E3531362F2D3331313130313230302D34322F33312F3034302E2D2E30302F322F
2E33322E2E2F33312E343132312E31303334312E31313233332E3233312F3330312F332F
323234342E2F3132333431323233343031343332342C3133352F3133322D313031313235
32343033353031353437383B3D3B423D403E40403F4242403E4342454343444747444C48
4B46494D484C4C484D4B5661695B54514E4B5050514E44413C3B3C3A3536373934333533
323735312F2E313036313730362F342F2A2E312F31302F2E36313330322D3033322F322F
2F312F313130322C2F322F362E2E332E30322F2F2F343232313032323132342F2E342C33
3331322D2E31323131313231322E2E3230333031322D2E2E2F302F30332D2E3431303131
2D31303331333234342E2D33343232302E3232323331313030
343230332D2C312D30342F312E372D32313432303134313336333030343331313232332F
302F322C2C31312D32312F32322E2F2D3034342B33313636302F332E3133333233342E33
3533332E353332353534332F323334313330333231343131352F30323431362E2D332D30
312E3231323737333438323C3A403C3F424144404340413E41494845434447474A4B494A
444C4B4D444B454C4E51515C6D6050524D49484B4A48423E3C4136383836343738363730
3238333231332F3233353737352E362E302E312F2E31332F2C31352E342E352E3132342F
2B3231312E332E2E2E2E31302E2E32322E2F3230323133313131313131322E3232303332
323435302E31313232332F3234342E312D2E322F2F2F2E3432322D2E2E332F33312F2F31
302F2F35313331323431313030303233312F2F3331322E2F34
323431362F3331352E2E352F323534302F3233342F312E322F302E2C2C303232312D3032
2B3232313033312F3132343432303231312E302F2F2F3732312E322D2E322A333530342E
312D32322E3031313130302E332D31343232303430333232373231352C332E31312F3132
3633333639373437343636383F403A3F3C3E40423F3E443E41424343464747494A49484B
4746424648484C4C49535456595A4E48484D484746443F3D3B3D3C3C343A373831373535
34333334322F313236383831313331302E30313333312F332F2F2B3230323130312E3530
2F323230312F3232322E2F33312F2D31313132313332323231323133312F2E31312F3030
3332332E2B3232312E3334333331312D303032343234312D3035322E322D2F312C2C2B31
31353039322E3332303034352F31313034302C303132333730
312D3333323533332E352F2E2D30303230362F302F2F312F312E312E2D322E312E2D2D2F
323230322D322E33323331332E3032343333323033303030302D3230302F34312F313135
323131312C2D31323534323236312C3434342E2F2F3036303230313230312D3431322C32
333133363634363036343532353A3B393D3E3E413F403E413F4345434540434848474C49
494546464A4C4F4C4A4A53504F524B49474C3F464241423F37393334353B353633373632
323632302F302F2F2F303031303031313230313432302D2E3232302C3231332E30353130
35333131333131322C2F2F31302C2D302E342D322C34293133323031312D322E302E3136
2E2F3432302D2F2E31333133313131332B322E33312F2F3230323033312F3131302E2E31
2E2F313131343232303630332E30312D31333133312E2F2F32
2E30333032363430303233312F332E342F3231322E3133362E2D2C2F3632342E2F312F32
332E3131323433342D313231332E342F2E3335302F2E313333312F2F3132303034343134
2E302F2F3233313330333135312E323431322F343331303131302F332F31303231342E36
33333034343533343932363637393A393C3C3D3E42413E3D403F40444449484949494948
494A494647474F4C4A4F51534D494A494B4B444242403F363C3B38343437363833353434
3231303435302E343330303234303131332F323230322E3031322C2F32312E303031322F
3431332E312D30322E2D34322D2E3431302F332E2B2F2D2C2F3230312E31332D2F2E2C30
362E312F323134332E32322D323131332D3130302E3230302F2E303433332E302E2E2C30
2F30353232322F323130302F2D30303130323033322D333132
2D3031342F33302E313031312D2F32312F33322F33313030322C3032302E342D2D313333
2D3030302E32352F34312E34342E2F2F322F3433312F353131302F33323230302D2D342F
323430313132302C2D323032303130303331323330322F333231342E2D2D30323337322E
34332F33323230323435373638383D393B3C3C413D43443E3F3C40444346464945444847
484749494A464D4B4F5256504C504C4B4A453F42413E3D3C3A3A32373537343134343533
2F353233303331302E323033312C362D34322A2D312E302D2E312F2F31312D3331322E31
33302C2F30322E342E2F2F2D302F2E302E313131332F313230302F3231322E31312F3332
2B332C35323532313130342A34322D303030312F2E302E30312E3230323133302F2D312F
2E312E322F343030303232302F302F302F30342B2E302F3332
313330333233312F3431332E2E342D3132313030303032342D30332F31302F312C2E3135
3033302F2F352E322F323131322E322E30332F30332F303332302F2D3232313533313132
30312F31322E30302E333031312F31313033342F3230312E2E2C2F2F3032332D33323130
34333132313230363437383739393C39393B3C3E3C403F3C3D4042464544484448494649
464E4A4A464B4D4F555B5754504D4F4843494241413F3E3734373836343735382F34342F
353232332F2F343731363133312F3133322F2E303131313230302D2E313030312D2D3030
312E3330322E2C3130332C2F34313430303330332F32302E31302E2C313033302D323032
32322F30312E372D2C2F2F2F3032323231342F31312E302D3230322E333431302D2F2D32
332E3230312E3331313433342F3532302E312E2E2F332F302E
343231322F31342F2C2F33332E3532322C323131332F332F30312F2E323230322E313233
312F342E2F322F35322E302D3432312F2D3132333033362E2D3132322E32342F34313331
34332F3131313330332E312C323532302E33312F322D303133332E2D35313136332D312E
2E312E323334333B383A3A363A383C3A3C3F3B4141393B3F4042433E434444464A47484E
474E5248514E4D5355595B4E4F5051474B414640413B3938343B3A3736313535312E3434
30323032312F303132302E32343231303131343234322E2E2F2C3331302F33353030312F
30302E31302E303231322E2C2F2C2D312F2D372D2D2F31303133332E2F2E2F313430302E
2D30322F3232312B2F3031332D322E2D333236312D3434302E32303230313130332E2D31
2E312D2B342F2D332C332D2E2E31302E3132322D302E31302E
36352D332E2F34312F3331332E31322F332F2F303231323130312D32332F313231303231
343030312D363133333030312F3330303830353530342C343132332F30303230312F3231
2F332D312E34332F2E333431362F30333131312C2F2F3132313334303030353130363434
2C2C3132393436373637373937393D3B3F403D3B403E3D3E4340494340454547434A4842
4850554D4D515055545A594B4B4B4A44423F403D3F3C3839373A363638373B2C38323233
333133322F30332F32323232343333312F2E2F2E312E322E3132312D2E2C2E3630323231
32312E2F33343230322F2E2E3331343332322D33312F3131312D30313032343133312C35
2F3030302E2D2E2C2C2D2C2F3131322F2F2F31302F2F312F2D2F312F332E30302E312E32
312E352E2B313330322D3233322F3432342F31333133332F2F
3533322F2D2D322F342E32302A303035302F342D33312F3232343233363232303130312F
3131322C3133342E33303131312E2D313033313231323236352F322F3133303031303130
332C33303130352E332E33313030323031332E2E3131343030302E303033383331303031
302F3432343336323839343236373D3C3D3F3D3F3E3A3D42423E414142444848434D4B4D
49555E584D5354555454504B4C4A464442433F3C3C3C383934343636363637312F323431
302F342E362F3230303334333131322D332F2E2D3030342F3232303031322E312B2D3333
312E332D30333031343033312C2F2F312E312C332E302D2D303332312F322E332F2D302F
2E2D302F2F2E2B2D2B30302D2F312F303132302F302E3131312B2D322F2D30302B2E2D31
312E30332C303131343230303132343031333032302F31312E
332F3230302E2D35342E2E312D2D32332C3233353333313330302E323531323331302E30
33302F2E2B313131302F3132302D322E322E322D332E343233322F3430332D342F313031
32313032342E2D322B2F33322F313032333233333334323432312F302F33333434333035
2D32332F312F3435343435303039373D3C3B3B3C3D403C40433E404543464C4D4B514C48
4C545357515D5A585A574E464847484741403D3D3737373834353736333234303333342F
31313131302F303033332F2E2F302F33322E2E312C32302F312D3131302F313031303130
2E2F322E3031333133303132312E3030312F30333332313231313033352D34332E31302E
33312C2D2C302A2C2F333135333431312F2C3032332D2C2F3231333131312D2B30312D2E
2F30322E313032312E342E332E2F313132322C3033312F3231
313434312E302D2F322D2E3133303134332E31312E33312B2F323131333430313232312E
302D2E3131323330302F3231342F31342F302D342E2F2F3031342E2E2F332F3234333234
332D352E30352F3032323336333031313333323135303031372F35333732323133343535
313533323331313737343233343A383B3F3E3E3D3F403D4341414445424849515651504E
504C52504E57585458514E42433E4C43433B3A3E38393633363434363232333030313030
353230313231303231302F312F30302C362B302F2F2F2C2D2F2E2F2F312D303031323130
3031312F30333130322E302F322E2E2C322F3134332C2D302E31322F31302F3230303332
2F2F2F2F2E313030323432312F2F2F322F3133323032322F3131312E3032312C34323033
31313130312E2F2F3331332E3132303131302F3236322F2F32
2D322F322C322F3631342D3233312F322E2F322D3132303033312F30333233322C332B32
32322E32312D313031323230312D2F32302F322F313033332D362E2F322F343032332E31
342E35312F2E312E333032303232312F2F343531312E312D3331322C313331333132322C
31332E3232322E31343533343335343C3D3D3A3C3D403F403F424241494850545451544F
4F5049484D4E5156534F49464340403D42433D3939353736353434363835362A31323331
36333133332C2F322F302D2E2D2C2B2E2E2E30332F2F312E2E2C2F332F2E342F2C333030
2D2F2F32312D2E3031312E312F312A332F2F30302E34312F2F2E33322C3030303331302D
362C302C31302E302F312C302E32312F31333433313230302C2E30303231342D2F30342E
33312E303031312C2F32313034363331323330322D33303132
322F362E32312F32323030312E33322D32282C31312D302F3634312E3131332F2F2E3032
2D302E323533323030312E2E34302B372C31332F302E343432323030332E312C302F3131
2C3031302D35303331313333323232343434303334343032342F3332302F332E3430342E
3236312E322E35343233323632353A3B3C3C3B3F4041424245484446494B535556585A52
4E4C49454B48484D4B4649464241403B3B3A393539372F323235323B353B302E2E353432
322D35302E303132303232302F3130302D30302B30323130302E33302D2C3230352F2E32
3230353030332F2E2F31312D313130302D3230302D30303230342E32312F2A2F332F302E
2D34342E32323030312C333431312C30313031333030322E32323030313033332E32322D
2E313030333030312F312E31332F3031343132312F32303231
3233322F32332F2E32302F30342F3630323031313132313334323131342E3032322D3232
2E3131322E313031302F3032322C2F34312D30312F343630332D323832322E33342E3331
32323231332E2F332F3432342F352D312E2F31342D30303631302C353430322F30303031
343032302F33322F313437343737383B3C3F3E413C4140424544444C49555B5F615B5354
4C514C49484A4747444545433C403D3A383E3A3732373430383333323D31313132303036
31362C322E2F302E2D312E322F2C2D31303130332D35312D2E3030323233303033353333
312F32323230322E30342F34332E302E3031313232332F2D2F322D2D302F332D30312B30
3230302B312D322F2E2F2E31313032312E342E34322F3433302D2F3232322E33332F302F
312E32312E2E33343331322F323030313230332E3130312F2C
313232323231312F2E302F32313033312E302F332F32332C323230312D342E2E2D323332
312F313531333534302F2E2D2C31313231333132342F2E2E342F32362E2F2F3432303234
3130302D34323030312F31302F2F3131332E33303032313331372D363232323533343131
32313433363431312F343136363731384039394140424841414349484F586865645D5754
555153494D49494043413F3F41423F38393E3436372D3B35353436303132332C32303331
36343131332F30332E2E342B3030312F2E2F30303132322F2F34312F32332D2F31313030
3131333230312E3434332F323033322E2D2F312F30322E3133333331332F31312D2C342F
2F2F2E2D30343033332D312F33352F32332F323132302E2F2B32333030312D312F332D31
2E323134323033303030313133313230303236372E2E322E2F
322E33352F3232333130303333302D3133312F3430323332332F2D332E30342F2F2F2F34
2D32323137312E2F2D2F2E312E3132323830343134302D2E342D2E2E31322F3231303132
3432333034322E312F31323432302E30303633322E34322E32332F323030313231363332
303035322E2F303333313632383834393B3C3D40404345444346494C565D6A70625C5451
535452504A464043443E3F3F3F3E3A3C383636343031363733323033332F313030333435
3432312F33313131322F32302D32312D322D312E342D312E3132313130332F3132313132
3130322E2F332E2E302F33312C302F322E2F302F2D302E333331302A3130322F352D3231
2E323431322E32343133322C313233323133313331313134332F302D2C332E313432302F
2E2D332E2E333231323235312F313034302F33302F31302D2F
33312E30342E332C2F30322F3330333330323034323231312F30323232322E33312F3334
2E3233302F3134303330302D342C30352B30322F2D323234323331312E322F2E342E3530
342D2E2F3236363131343937332E353436313433323539313132312E3434353233363235
393035323433333134303437373735393A38393E393D3C4248484E50546069695D53544F
534E504C47413E3E3D3F3C3F3E3F3D3D3C393436333634372D302F3034313130322C3431
34323230342E31303030303430322E2F30333031343030333133302E2F3033313131322D
2F30312C303136322F373232302E31313131323131312C2E312D2F302E2F33302D2E3131
322F2C3033322F2E313230322E303330312E2E2D3030322F2F30302F2C302F2F2E2C3230
302E343431302E2F302F2E3131332F2F2F31303133322F3334
30333233303231322E322F2D312F322F2F323433353132313132313330312E3131303230
31312E2F362E363133332D2F2F31332D35312F302E312F363031323331312E3332333031
3332323030303030303435322E342F322D32313036333332343433303131323133393430
3630332F2E2E362F313336353732393635373736363F424645484C4E53636364584E4B4B
4E494E47424440443F3E3A3D3E39393A3639342F37333534333432333335363532343033
322F32312D332D2F31342F2C3031312F2F3132312C2F34303031302C322F312D352E2E30
34322F2D332F31302E2F352E30323032322D3132312B2E3231322D2E2C333130322F302C
322E2E30303331322D303331312D312B322F2C33333130332F322F2C3033362A32302F33
31312D322D2F312F302F32312F3131323534322D2F32333533
34333134312F302E31303133302C322F32323432303330313031322F322E312F302E2C30
2F32322F362F33313031313230313531323831342B3330322F322D3033343430312F342F
3431342F3730343632303032303632312F352E34303430333132303130332F2F31303530
34323332363133333334343736353732383E3C3B36403C454249494F545D5B57554D4A4C
474F4C494445413E3D383A3A3E3C39383639393234353436333134303333313632303233
2D31322D30312F332F312F322C302E332F30332E31312F342E3034342F3130313331332D
332F32323630322E313232313031312C32312F313331343232302E2E302F302F2E302E30
332D333331332B333032282D2F2E3732343232302E31303231312F2D32312C302F2F302F
2D30332C312E31323032342F3332313031363130332F323233
2E32322E2E303330352E2E2E312E342D2D333134322F31322F322F323232302E332E3230
2E322F322E2F2F323430313432302E2F2E3234332D342F2F3030302F343330322E34302F
30333033333033313034303533343032312F3032323033332F32302D332F332E33353031
2F2E303032343430373237343533343435373737383A3C3F434746484C5655525448454A
474B463F424141433A3D393A373B3637363335373336313537353038333633303530322F
332F2F2F2F313331302E30342D2F332D31302F2E303335332F322D332E2E2F2E322E2E30
312E312E303132313031333331342F322C3031322F32362B2D33332E332F322E33302F33
332F2E302F34302E342E312E2E32303232332C322D3132312F342E2F30302F312C2D2F2D
3131302E34312E2F35302F32313030322C3133302C2F323631
2E2E31362B3033322F2C3035303132332E342F31313432312E312E303131313435322F32
312F32303032313332312E3330303332323433343131322A302F3131303231333332312E
342F3231313034352D2E3434383935322F32323334313133322E3131323135332E372C30
32333034333434323032343832343337373737363A3D3E3F3F4246465151514E4D484848
494246423E413E3E3E3E3A3B393C37353637333337303732363131313134313030353530
34382F3131303130312B3032312D2E34303430302F32352E312E3334302F2D2B2E2E2F2F
3231323232302F34323333332E2D34332E342D33312E31322E31302F2E33333330312F2E
3330332F31322D32302F2C2F2E302E322E322F342C2F30332E3030312F313031302E2E33
2E2F2F3033302F322E3130312E323332313135333231332F2F
2F3030322E3433303131302E2F3330322C3031312D2F30312E312F302E2F3130322C3234
30312D362F31302F3230302E332C2E332D322F302E312B332F33312D2F30312E2F30302F
2D2E32333031313031302E35373F2F3237312F2E2E312E332B302F31332F343130333132
2E3232332E3533333236313032343333353637393C3B3A3B3C3E46474F484F4C4F474741
444444433E3B3D403B383836373A38363436363833323336353333353534323332323330
31302E34332E2D2F3234322A30332D312F3132322F342F3130342F302F2C312E2E2C2D2F
332C312D2D31332E3332322F2A312F3231303331332F2F2F302D2F322F303230312E312E
30322F2D30302E3232313034332F31312F312E30332F2F322E2C2F3130303130312F3133
2E2D3132302F2F3032322F312E3132302E322E2D2F31313232
2F36352C3334352F332E323334302F2F32332D3031312F32312D2F31312D3130372E2D31
2E2E31332D2F2D332F352E353233353031313131302D2F30322D313330322F3432332F32
2E2E2D2D322F34313532302E363533322F303334313234313133313230342F3333303231
352F35343033353630313435323A3534363336353938383F3F41444949504C4B47414041
444244423A3C3B3F3A3A3B3A3736333432333539333532333530343331363136302C3433
31333130302F332F2E3432322F2D31312E3232332B353233302C2D2E2F2D2C3130313132
313232322C3230313238393332303031312F313130302F3234322F342F302F3130313130
30322D342D322C332D302F31312D312F2F2F323330322D30333030303030313232323130
31302F303230332E2C2C302D312E353230332E30302F312E30
2E3131342F3036332F312C3233302F2E32312D3133322E322D3232302E2A2F2F2F2F302F
2D2F30332F3030302E32322F303230322F332E2D3330302E342F323233332E30302D352F
3130322B3032303030342E313130313032322F3536312F3034323430323332322E31322E
3031323230342E333033343630373632353438363A37393C3F41464B4B4D494145444641
403F3F3F403D3E383C3A3B3D383533323733312F313433322F3635353332373035333035
31312C333331342F2F3132323231353334362C323331312F3332343233312D2F332E3234
2D3335333233313332383837312E303033322E3333342E2D312F3133313230302F30302D
312B2D32313236302F3332323231303734352E2D3330332D2D3230313132323131333133
3133333031303231313231332D33302F33303630342D2D3135
3232313332323231322F31303233302D2D332F31313230322F332E30362D2F30312E3232
3031303533313136312E30332C34342E303034312F30312E312D313431313331302F3332
333134313031343233312E3032342E2C3132343130373332322F2E30302F323133353030
2F2F323734353131302F33363835313336333938363B3F3D3D4142484C4A46474346433E
3A423A3A413B373A3B3638434038343530373134353734333434333435323034302D2F33
3134322F313035312E2F313432302F2E3535302C32323233322F32323333332D32302F30
30312E30302E342F323638362E322F343030352E33323232312F322D312E2C312E2F312D
2A2E2F2F302F322F2E2F2D33323133323234312F3131322F2C3130323031323332302E33
323433322F2F322E302F322D2E2F3430292D362C332C313331
343235332F332E2F333B2F2D3236322E322E2F2C3132323231312F3433323331332D2D2D
322D2B2F31323134312D362F35323031312F332F2D3233322E333433312F33323132342D
3333332D302D3734313030323232312E2F2F342F343036333534343130323233332E3132
313434323432323032353236363332343535363A3B3C383E463D484847443F423E454437
403B3D3B3D3C3A373C363B37353936363435343032333533373534303430333635332F31
323134373330303233313031342F2F2F3432313031312E34302E332F2C2F2E303030302E
2D2D2E2E343233323433343432332F2F2F313032323630302D2C302D3035323032332E2D
3032313332323130322F3131312D353133332D31323230342E313232323132322D2F2F32
30322F3031302F3031302E322D2F32312C2E2F312D31323331
34362D35303231332E313135302E2F2D2D3231342E3030313133303134312F2F2F302D2E
313336312E2D2F323332302D2F31372E33322E343330323233352F2D31332F2F2E313530
2E2E302F2B31333137343133353331313132312F2D342F3533333033323033363333322F
32313332313032313434332F353634363238373735383A3F3D3F41424846403F3F41423F
3B3C3B373A393837333537363937383B323335313432362E313035333432303132323032
35352E3333303430332E2E2B353234302E2F31302F332F353033313231302F2F332E3430
3232302E3132302F313134332D302E31323130313032322F2E322F322E35322B2F2E322E
2F2F34342C342C30332E2D2E3133322D302C322C313334332D32312D2E33313331303030
323230332F32313332302F2D2E3133302D2B2E2F322E313333
3233333133312E363130362F2E3334303034302D2D3231313131323230332E2E2D302F31
352F2E3330322E31322E2B34303130342D2A2D312D302F3331322F2F3534312F32303130
2E302D3232302F2F3434352E36302E333232323133343335303031313433303134352F31
333433343135353638383630333335333535353939363C3D41403E4540403D443E3D3E3D
393A373A3A3A373939373A3636333637393534332F333332323238363332323033333235
2E2E32353230312F2F333231333231323032313032332F30303131302F31322F312D3232
32333034343230302F3033352E313130322E303032302E2D332F31302F3131312F2F2E2E
2E30312F313030313231312F2F32333330302D302F3234312E3035313330322D3231302F
322D31332E32303332303034333031322C3231323231332E34
2D323235363130322E322F2B333133302C2F2D312F302E312F342C31312B342F312E3630
312B323031362D332B312C3031342B2D3234342C322E3330312F332F332D332F31323131
2B32303134362E343233313132323236333032363134332E363537363233342F332E3334
2F2E3332352F3734353437312F383436383737363739383B3D404141434339403F3D4040
363A3638363D39363B3138343935343130333233313533323232343432322E3035332C30
3531322D31302C313031322E302F343435323032322D2F3230303034313031312D2E2E33
33312D312D322E302F2D333130312F2E2E322E322E302E312B2F312C2D342D3434313030
30322C302F2E31302D302D313132352E342B322E33312F333132312F332D30303132302F
3430332D31332F2F2F2F342F332C313133302C322D31332B2F
332D302E31313132312F2E343031312E2F322B322D312F2B312F2F2D2F3133312E313033
312F33333431302C2F322E3433342E2E2C2A2F332D312D2C342F313333322D3230313231
333035343031313230352F3133312E303030303035333731362D343032322F34332F3632
3531303436343735333638373735363A37353A393534393B3A3A4240423F403C3C383A3A
3639343336383532373A35343337343533343432363432302F37322F3231313732303334
32302D2E333132302F3331333331322F2F343331312E34302E3031323132313331332F30
323230322E313434313232342D30302F2E313230312B2F31332E3031302F303232312E32
312E332F2F2F2F313131302F31322D332F2F30302E2E3030342F2C32303030302D32322F
30312D30302D332F312D2D2F31322E322F31323033312F3332
2C35302F353336333029332D3230302D2E3433343334333033322F313132353134313032
2F34312B3434342C3730372E33332F322F2E3231302D34312F2F32312E3331352D333232
3530333034322E30303031343030302E322D32312F3732343032352D31303130332F3131
3131303036303537383836363634363737373733393838393C403D423E3A3D3F3D3A3936
3A36393A37383433313336352F33373233313030353331323434323034353231362F3237
3133362D303134303730302F31302B3431333030322F303430342D312E2F3030322E3231
332E32312D333331312F3230323233332F3230302E2C2F2D2E2E352E3133303334313530
3330333430312F2E2A33352C2C2D3030303032312C2E302C312F2D3631332B343031322B
323232322B2F2B343532342F2E2B322E313232333132303430
323032312E3530332F3032313034303032342F333132312F2F312B332D31322E2F2E3230
33322C332E302F312C3237333032322F2E302F2A30302D3231323030313431312F303133
2E31352F342F362B323230332D30352E2F2F353132303430332F322E3231303331312C36
342D332C3433363637353933353432343534363B363832393B3C383E3A39383C3C393938
373037363838373233363436373636323532353031313531343434313531303732332E32
303033343731322D312B312F2F32302F32333331302F3230312F3230343331302E323031
2F34302D332F3132332F2E323132302D2B31302E31302E31303131302E2F31312F353134
2E312F32322E32322D2D2F2D2F302F312F2C322F31322F2F333131312E312E342E313131
392E2F2E31332E312F3334312E2E2D2F302E313130312F3331
313231312E2E302D2F2F2E32343131312E333232302E3030342B2D2F303431302D32312D
31303034323533322C2D303232323231303231302F2F2E312F31322E2E3332333335312F
33313036313031342E2F2E323032302F3430352F2E31333034382E2F33302F33312F2E32
303133323331342F3535373A3736343336363738353238343F3A383B3D3A36383A363536
3634373531363136393331343632343134323238323532352F2E2E312E30313533323330
2D313133312E3432302F2F2F2E313232313130312F32302D3032313130332E31332E2E33
33352C36313330332D3130303133312F2E2D31313332302F2E32303031302E2D33323232
34302E2E322D302E30322F303230332F322E30322E30312F312D303331322F31322F312F
2D2E30302F322F3232303133312D303430342F2F34332D3134
3032342D312D2F2F2F2D3031303035313332302E31342E322C302E3133322F32322E3232
33353232322B322F2C322D2D2F30353130302E3331312E33312C32322B30313234323030
332E2E31312D302E302E2E34333030302C322F3133313130352F3130302F32323236302F
31313232313735343338373335353835363B383A38373936383B373A3C36383A373B3833
343536363A3634343537333435383332332D363031333133313131342E3433322F2E2E34
313132332E312D3033333333312E2E2F2F302C2E313130312D2E30323031332E30333033
2F322E303032323230342E313032302F342D2B2D2E302E302C313335333134332D333333
31343331323031312F2C2D2B2E303330323332302D30322B3130322F31312F312B303035
2F312E343330313231333532342E30322F313333322E2E3233
2E313530303631322F3131322F2D3033303231333232312E3133303332302E312E312D31
2E3336312F34303032302E2D33342F2E312F31322C31302F33303031342F3035322D302E
2E322F31312E30322F312F2F2C302E3132302E31302D333335312E30312E31342F302E2E
3734353236352F303634353436333636373634313534373634363738393E393A35393533
34313832353633323534353330372F352E2F31323336323232362C32373430312E353137
3133312E313630302C3332343032332F2E2F2E30312D322E302F2F312F32302E31302F31
383432312F37332C3431352F312B323130302F2D3331323134342E2E303137312F35312F
2F3031323231332E3032322F2F2D32302F2E30322E2F2F2F312F30302E30303231313031
30323432323433322E3339343634342E32322C32323031332D
2D30323033312E35342E313433333433322F342F302D35312F312C30302D32312F302D31
322F313033312A322F2D33332D31312D30312E312E323231322E31312E2F302D31323133
312F302F312F2E2E30312D2B2C3134312F30332E323437313430342E2E3130322C333332
333738343A33343434323033353832333634383835353737323938383A36353434343737
36353033363534383936333433332F303034312F3233342D2F302E323134343230313033
303530372F2F34302F3132303030302F2F3031302D2F312D312E32332D322F3133313230
2C353730342F2F34343031313233313231302F32302F2F34333532332F31312F30353133
31352F32302F3131342D2C302B3132322E2E2F2E312E302D32332E3033302D30332F3232
303231342D353330363433353234312E3132322F3032303031
31322F30313133323233313230332D343230322E363330332F32313135342D3935352E33
32343333332F322D3737312E302D2E2E3532302D322F312E30302F303230343033323133
302E302F2E2F3033302F32332C342E2D3033312F30322F373030352E3132303333333035
30333833313030353236333938343236343634383638373433353A353933373834343632
34313435353334303436303232322F3131313133312E32323134332E3230313433333033
2F34332F332F3033323330312F31312E2F2F2D312E30303331302C302D2E2F332D313233
322E31333230332E3033332E322F2F2F322F32333430322E3330342E3035342E3031322F
3031302F302E322E302B2D2F322D3031333132302E31332E3432312F2F30312F30303331
3233313431333130343636363533312E343231303133312D31
303131303232303133312E3234343235323334333331312E2F313131312E342F3131302F
313232343333342F2F322C31352E34333236302A30302F302E323431332F30342F302E2E
31372E312F302F2E2F322D2F312D2F302D332D3433332D2D2F3231333033323130313033
3031343033343532323436343634363237323434323432343531353536343B3536313033
363632383936343531343231313332313133302F2E34363131302F38312F2E2F34312E2E
333134342B2F2F322A333333312F322F312E312E31312C2F322D2E3031302E3231362F36
333231302E3834362F3332343632323331332E322D3134333731323232342E312C343330
31302E2E30302F2E32323330312C2F30332F2F332F32322F2D2F2F33322D2E3331303330
302F30313031312F312E363335312E30313235323132353232
3431332F31313034313030342F2B34332F3135323730333632302D323232333332313330
2D2E312C332D3031313136312E3130312E3034332F332E2F2E302E32342D2F3031313232
2E33302D2F3331302E362E313033372F2E2F34343231312F313331342F313635302F2D32
3431333232332F34313234323734383437353437343736353634373933353538363A3536
36323B333334333633333130332F33322E2D2F3033332F333436303431332F2E32343332
3431352F3131302F2E312D33333232313331322C2F3030322C33302F31312F312E2E3132
2E2E36333231332D3336343233332A34313133333635343034352F2F2E3230333037312F
2F33322E2F2C30312E2F303333333434323033302C2C32302E2F322E2E2F2D3030342E33
312C302F30302E323230312F3135312F2C2F313031342E3131
2E2E333330322E303431322E322D2F3634303132323430363031342F3033333030303235
2B353333302D2F352F2D2E2F2D30302F33342F323430312F2E312C2E343431302D323030
302B2E2E352E2E2D302C313232312F342F32353032313330312E2F323432333234342F32
2F32343231322E3630333231303337303433373534383536343533333333313733323432
333033323133323631303433363331332F31302E34353132372E3330343334333231322F
3034372E302F2F2F2F2F323132323432352D342D32303031342E2E312C32332F2D312E32
3130312E322F313232302F3136322D332A32353233352D32302E3235302D32332F2F2E2E
2D312E2E2E31302B31312E3032312E372F302D2C3631352F312F2F2D2D2D2F2E342F3131
32322D31333231313134333231303033302F31332E3531312E
2F2C312D32303030332E313030312E3130322F332F33312C2F322B322F2F2F3031303232
313031302D31322A322E322F312B332F3135312F343032343030362E2D2F2E2D31323330
2E303133313031353032312E2E312E3331342C32313031303330342E3130312E2D35342D
36332E362E353032333633323235343430333736373437373437313333343435333C3534
352E31312F2F36302F35343233323630322C312F2E30323234312D2E342F333031303632
3037323131302E332F322F303630323330313133332B2E352F332E302E302E2E2C303330
3531302E32342F373132332F3232312E3330313331303132332F3235312D2E3431322F2F
34302D303331302F31342D302E2D2C302F2D302E32322F2F2D2E342D322F323433302F32
3234302F2C2E33322D2F2D31312F342F2E302E322E32323233
3231312E3235343332332E342F322F3332303137342E342D332F3230312D302D30303330
333134302E34323131322F2C3032312E303230322E35302F302D2D35322F31323233312F
353232313233342F2F32302D332D2C322D3130352E2E322E34302F3131323433332E2F36
322D362F3534303731333336343832353433383334343232313234333332373435363535
343331343230323232342E2E2E2E34302C2F2A3330323334352F31303032302C2F2E3337
2F363836322E2F352E342F32322F2E313131312F3034352F303030332E2E2C3132333035
333334332E2F2E2C2F302F333332322F3032303330323330322E2D2C332C35332E2D302F
3132322B2F2F31332F2E3034312D302E2C2D2E332D2D322F302E312E2D332D3130303136
2E2E332F322E2D2D302D323034302D2F31333032313233332F
2D2E2F303135313232323032312F2E30302E31333134322E312F32362E352F2C34333330
3330332E31332F312E322D33323031323030322C332B322D2F302E322F2F3232312D2F32
35322F303331322E312C2D2E30332E2C2F2E2E313336323235313132332D343436303133
2A343131303033333232313233353534373234303734333231322F343335352F332E3737
353433362E323336332F2E2F322E333531312C2F352E32313233332E3031343431323230
33373532312C3034313531313133333434313130332D3031312E30302E322E3535313132
342D322B3434323134322E323132312F313433333035322F332E3531312D33342C33302F
2C2F31312F2E3131302E30302D302E2F2D2E312F32312F2F2C2D3030322F2E2E2F313235
3033313130302D342B312E2E2F30323333302E312F2C31302D
31312F3031302F2F35322F36322F33302E342F2F303332322D3032332D2F322E30323532
2F312E2C2D302F2E2F32332F312E2D312C2D2C2F2D322E2D36342F312E322D3131312F2C
312F342E302E2C2E2D30303030302E2E3130312D2F2C2E312F3230322E35373631363233
2E2D303333363331313432302F3730363135343835363838333432333533313530333430
322E34313230352C3531313030303130312E2D2D3030302F2F34343332342D3231343236
353137313030323334313230333231333232313132352F333633302F323230303133302C
34332E34302F303231312F35312D30303432303234322E2F2E30312D2E32313131322F30
322C332F2E362F2E2E302D302C2F2D2D302D342E312F2B302D312F2D312E322E32343333
34313131302F2E2F2F2C302F352F2E32332F302C30332F312F
362E31322F34322F3132302E2E303133312F322E353230303032312F2F30323132313531
303130333031302F30322F30312F353131332B323031322F372E302E2F2C32312F333231
3432362E2E2E333233312D2E3330312B323031312F312E2F3232332F3134323136333032
3134362F302F353334303232333532343431353734343133333438333735313432323230
343032323432322D3231323336302E342E302F2F2E312831333332333334333231333332
333032363130362E302F2D313330312F322E313031312F3330302E322F3633302C322C31
302F2D2F31323032322D34322C353232322F3232363133323033312F2F332F34312E2E2C
2F2D30312F2F3130332F3132332D30302F312C322D2E2E302D30332F342E302F3135353A
32323031302F32302F30322F3033313230302E31333130302F
2E2F2D2B302D2C322F2E2F2E30312F31302E2C2F312E3333302F33312E33302E3330362F
34332D3332312F2C3231322E322F33332E2D302F32323030303035342F2D303132312F2C
2F3234322E342E2D302C302E2F2C2E312F312F3032342D2E303030333332363331313030
2F2E322E30352F3332363135333635333B36303335313535353632343235333531333638
2E302E322C33342D2F302F352F2C323336302F3132312F322F2D36313331343433323131
3432312E3333303430323433332F362D30323236342D2D2F2E35302F3030313137343231
342D312E312F33342D31312E322F32322E332D32303035322D30332F3232303032312F31
30302F302C302F3033332E31312F2A2F2E3130302F31312E32302B2F343231332C343536
32302D2F302E32302D2F33312F313030302B3230302D2F3432
31312A3330323134322F3031312E333034303130333230303231322E2E31333730332E32
302F3030322D3031312F32342E3133303330302E342F2E30312F302E2E35323033333032
2F2F322E33332F33322D352D2F2E31363431322F2E30352E313236323232302B32302F2F
2D32312E3031323234332F30332D35363330323938323332302D362F32353632302E3335
34333132322F2E312F3231332F3130302E33322F2F2E30302F2C2E3632302F2F302E3331
2F3132323231312F3131322B36312F2F343430312F303332312F32342F313235322E3132
302E3531322E31312E2D313030332E2B2F313231322F322F322E35332E322C31322F3136
2E2F2E2E2E2E313131312C2F2E312F3230312F31302E322F312E2D352F2F312D30313334
3330303132302E3232323532313231312E2D32332F2C2E2E31
30313131313131322D31302F2F30302E3131322F3531303034342F34322F312E332F3132
2C312D30312D2E34302E2F313231312D333032322E313230342F2E303031312E37303332
2B30332E32353A2F2F3131313030342E2F32313232342F303132322F3132332E2F303533
2D3831333132313634322E33343333363437323038342F3332342F3433323437352C3132
30303232302F30302E322F3332312B2B34333332332E30323032322B3531332D2F312E30
34333431312E3330312D3235312C2F312F312E35303130322D322E323232302E332E2D32
312F302E3031312E30302F312E322E2E33312D2E2D2F362D3133353334332D342F302E34
302E33312C2F302C3131312F2F3130302F302F302F2C302E2F3531313232312E2E2F3034
2F32302E31313131302C3131313534332F332F2E30322E3031
2E312E313030302E342F2F2E31333231323431303230352E312D33322F33302F332F3231
2E2F2A2C312F332F342E333134312C302E2F3130323033322F2D2F302D2A2C2D312D3634
30332F302D2F31312F303331312D34333032302F342F2B2B2E2E313231342F2F31323131
313234312C3230353433352E302B36363231353137353732322D3631333234332E303432
30332F2E2F2E2D34342F2F30313332362B32323333332E2D322E2F3231312D2F2E33342E
3330333131342E3331303235313230303331312F302E2F33332E3031303330343134322F
312E2F30312E3431302E32322C31313230342F312F2F332F2F3031363032313035302F34
322C2F332F2E2E2E32312D2E2F322E2E2F312D2A3131322E2F3230332F3131302E2D3131
3134302E2F30312D2D33323234323632312F2F2F2D2E2F3233
313031302C313032312E303030302F313034312E31322F2F3032312D30273331382F3632
312F2C2F30313230322F2E31302E323233302F2E343233323132302E302D2E2D2D2F3135
312D312E2E2D342E3131322D2E2F2F3131312F2D332F32343133312F35312F3130303631
332E30323130353034333239353335312E32323231333632333431343232333232323232
32303232363330342D2E3331302D31352E30302E332F332F31313131322E312F3133352D
303A303431312E2F2F32323431343031333133313031312D312D322F33343630332C332E
322C33302B2C2E2D302E33323433323233312F34312E2D302F32302D2F322F3231312F2F
30333232302D372E332D2E2B2E322F30303030322F2F322C3331303031312C342F302F34
31322F2E312E2F332F2F31313432313633342E2F31312F3435
302F33353131303030302F30313131313134333232313232322E30342F323032302D332E
3031302C32342E34303133322F322F3132323035342E312E2F2D3431303231303334322F
3432333131332E31342F2F2F3130343032312D2F2F3230353031342D34303733322E2F31
342F2F31332D2F36322E3130363032343332343132302F3231322E303733313031323031
312E32342E35322D34332F303132303030303233343632323430382E32322E3036303333
3133333230302E30343332303030333133303230353030373033343230363232322D342E
2F2F322F3333313130313231322C32312B313133302F2E3230302F30322D31342E333235
322E30352A2F312F2C2C2D2F2F2F2E312E2F2E303134302E2F30312E2E312F3131332D32
2F31312D312E2C312C3235342F303632353130322E2E33322F
363331302F312C2F2C302C3231312F30303133343030322F3131302D3233323231343031
322E313132332F31342E303332352F2D3230322E2C332D3131332E2E31342F2E34313231
2D312F3031313432352E2D3134313233362E352C332E2D332F3232343133302D30333330
31313135332D3232343232322F2E3532333131302C323031363132313231333330322F30
303130333033312E3131302F2E2D30333031333333313431312F352E322E2D2E33303431
35323332322F30302E2E303136343433312E2F3232322D32342C2F312C302E3031622A2F
2E30302D322C2F3030302F2D343030343134303033313032312E2F312F30333232322F2F
2F3130302C36312C32313030302E30332F31312E302D302D30312C3032332E2F2E332F2E
313233313031322F2F2F2D34313733343431302F2F31303333
2F2E332F2C342D31322A2E3133313231323032322E2D2F31303330313338333533323030
2F2C2F3433332F3133312F32312F30342C34312D3234312E30302E2F2E332F2D372F3431
3132332D302F343331343332333031353134313434343131342F31312F3430323232322F
363032343130322F2D32332F3030332E3231313132312E33322E2E313136303231333430
343331332D2D2E322E303031312E31312F2E30312D2C3432312E323137342F2F30332F30
352D323333332F323235302F3234343130303431353133373034303431302F302C403030
2E342E3530323032302C32302F2F2F312D322F323433312E3231302E3031303034302E31
3033302E2D312A2E2F2D2E2C2C2F32302F2D2B312C2D2C2E33302F2E302E333233303634
32322E322F2E3231312F303633323234352F3334302F32312D
323134292E2D2C31342D313231303331302F32322E313230332E2F312C32322C31312E31
32302E34312D2F3333322F3331342D30332F2F32302E2E312E2F2F3230312E36302C2F30
30342F302C32333731322F303030302F342F303237303134333331332E3231332C2D3034
2F30342F3130333336352F33363132322C353431312F322D2E30282F312F323231362F31
2F3432312C2B2C302F3031313034313031332B313034312F32363031372F32302D333235
3133322E3231302F3330312E33313531342E2D313132342D2E31332E342F302E2F2D302F
303032322D30313331352E3132313036302F2E302F2D2D2E313334302E2F302E30303032
3332312F2F302D2D2F2D2F2D2D2D2E2E2F2A32332C322D3133352F31332F312D35313233
30322E35322E322F322D342D2D31333031312F2C30312B312E
31312E2E2D3232322F34332E3430303132332F333131303232312C2B322F30353033302F
32312E3330312F333434322D302F32302E3531303031302E312D2D2D32312D312D2F2F2D
322F2E3230322F3331302D30352F2F2F33312D30312D2C3233323033362B3331362F3136
343131302F3235322D30332F323234303132303032322F3031333432313133362E312D2C
30323231342D30322C2D302E2C30313332323033312E31312F303137313431332F2E3231
2F34322E322E322E36323332322F3436323131333232363031323032312E31302D2F302E
313433312C32322E2B32332E36322C2E312F332F30302F2C302F32303230312D322F3033
2F2B323131312D302C2F33322F2F302F2F2D2E2F2C30322E332E302F3034373632322A34
302E2B2D302E312E302E2F31312D2B31302B30312E312F302D
3630332D352D303633323530323031323133302D3333342F33332E34322F342F2E323031
3633313632303036342C2A2D2D322E2C2D33303130322F342E2C31353435313031303430
32302E312D3635342F3136303131312F34332E3031313133313033333431353431312F36
2E2D2F30312D342E3B363235353331322E313031303032323230333033353530302C332E
2D333235342E322F302F33322F322E2F31322C32322D2E32312E3431353434322D2E3031
36303331343133313036312E323030342E302E31303134333233302F2B302A30302D332D
303031312D312D2E302A2E2D32302D2E2F2F322E343330312D30312E303131302D303231
2E2C313030302C32312E32312D2C2E2C313132312E2F2D2F312F312F3333323033303330
36332E2E2D312D2D3230303233312F2E3234322E3130312D32
303033332E312C31323132333330302C3333312F302B30342F32322F323131312B2F2F2F
3134352E3330342E30322E2C342D3230332F2F3133333334323134333130332E332F3134
322E3034363334313137323133342D33313233322F313331342C2C2D3436312F2F30302F
3033342F322F2F312F2F2F343134323235353331303233313134342E322E313334332D31
2F302F32332F2F2C312E332F2C323234303031322B312E30313230343234313233333133
33322E322E2F302F352F313031303130322F312D2F30312E322E2E312F3231333132332F
2B322C302E31302F2D2E303233323031312D2D332F2E36313631333132332F2F2F31302D
2D2E2F30303130302D2E302E37302E32323230312D2A322C32303032353131332E2C322A
2F2F2D30323132312F313331332E2F303132343034352F3230
2F32322F342F302D3430332F3231362E3030332F30322F3430322F332E2F313131323431
333133322F2D2B2D312F31312E31332E2F2F30312E333131302D32342E2C332F302F3131
323133353431302F33322F35322D333033342E302E303233333333353133313132342F32
332F3431323332333020332D312F34323538312F2E363031342E32312B31313232303134
2E2C2D30303032302E2E2E332F31312E3232302E31302E2A322E31323431352E352F3133
33303235322D312F33323332323332333031303133323035302E33303034342F31312E2E
2F342E312B2E2F30302F3030302D30302E3332302F2F312F3330332F302D33302F302E32
312E2E31352B312C333530342D2D31342F302C2D2E312A312D322E2D332F3031312E2F30
312E3236322F30323132322D3334352F322F32312D322F2E31
2D3431352D2E2F2E2F2F303132323032343331332E31363333332F2E312F302F34313332
333230342E2E30332E322F2C332C312F2C2E3030323132352F342E34302E332F322E2E32
2E31302F2C32332F3132332E2D332E2E332F322F2F312E33333335333130323235303230
31342B32312E313032323035343131333639363638343234352D322F3233313334353134
2D3131343129302F2F3030343032323330342E2B2E2D30343133322F363332342F33322C
323233303030303131342E30312D32312F362F313232343333302F3130312F3132303030
313033312931312B2E2F30323230342F312C312E2F302F303130312F302F30333434312F
302A312D342D332F30322C31332F2D2E31322E32302F2C2E2B2C2F2E31302E2F312F3230
2F3330302F34312C2E312E31312F322E2E2D2E3134302F3030
332E33312E312F322E30322D3131312B3033342D2C2F2B3133332C313131313530333031
3530302F2C34322E302E31332F30322F32322F3132303330302D30313431302F2E2E2E2E
313332363231312D313233313234343135343131322E3630332D30333333323032333030
2F2E343230323130303231312F31342C34383A3136313233352E32302F2D32333130302D
323130333134313031333235303836342F2F2F3335303130343330333231373332323330
2E2C3031303132342F31302F312E2E2D34302E312D312E322F303032332F3530332E2F30
2C2D2F2B302E2E3131342D2F2E2D2F302E2C2F32322F2D332E333630303233302F2E3131
2C2E32302D34322E302C32302F302F313135303032302E2A322F312E312E32332F333333
3332313534363431312E2F3232312E333431332E30322E3130
31312C31323030302F30302E3031322E2E31312C2F322E3531313133313130343332302D
30332C2E33332F31322F3232313033312F30302E36322F342A312E33302F2C2C2F2A2D31
332F34322E322F312F3136362F362F333333322E323234352E36322F32312F343033302F
2E323530322D2E313032332F31322E303737343032342F31313631333133313331333131
3331302D2D3331302C2F2D2D2E30312E3030323131303132303130313034313132312D32
2F30332F323231323434332F2D322D2F3234313134322F34322F3331352E2D2F2E303330
32312F2D2C2E2E2E2D312D322E302B2F2D322E2D302D313232342E2F332E2F30332E2E30
2F32302E2E322D2F2F322D2E332F2D3031302B34312E31302E383031332F2F313130302E
31303432322E31332C3030342E322F2E2F313231332F2D3332
2F312D332E332E323230342D302F3130353033302C302F2D352E2C2D3531303135303032
303632313333312F31302B30313130322E302E2D2C2E312E2C30342F2E2E3230302E3132
2E2F302F3332313134323432342E3031333736333135342D372F32333030303132303034
343334333030313031312D312F34303031332B3432332E3233322F322C35302F2D2A2C31
2E2D2D2F30332E3134313433312F332D3330312C312E2F2F332E2F312F342F2E3333312D
3130312E2E302C303132333231303336353032312D34313233302D312D342E2D3232302E
302D312D2E31302E30302F2C2E30312B2B2E2E332F2F2F32312E3031342D2C313330312F
32312D2E2E2E302E312C322F333133322D312A2F2D32312E2E3236303131302E35303234
31302F322F312E2D322E2C2E2F322F312E30302C2D302D302F
2F3133343132312F2E3531312B303133302F3132343130322E3331332F30323034303030
322F2C353233342F2F2F3034313030322E323033323330312D2D2C2D30302A2F2D2A2E31
2E302E2E2F2E2F31323131312F33342E32302E3231312F342F3332333233323130312F31
2E2E303331342D3437333332332D332C3332303130303033363830312F323230342D3231
312D2E2E303733332E2F2F32313336342E303131302C332F2E32322F302F2F2C2F2C342E
3731312C2F3134302F2E322F332D3233323032323132332E312B2F2C30322E312D322C2C
342E2E302D3031302F322F2E3330312E2E332D302C313230323132303032352E2C2C302E
3331342E32312C2E3130312C2D31303232343130322F2E2F31322F302B30332E2F2D2E2D
312F303232323030312F322C2E2F2F3232312E31332E2C2E2D
2F2F30352E2B3130323230322F333231343232332F2D312B3132372F2F3131332D2D3131
2F342F3430383631312D322C352F30302E312F30302F3132312C2E302F3133322F2F2D2F
332D33333034312E31362F2F32353430303132333431312F2E3334332D30343133343534
3230323432302E312F342E2E2F2B2E3134323232312F33322E31303334343031332E322F
2D302D2D303031302B32302E2C3332302E2F302E30312F342F34303030332F2F322F2E30
3131333432342F33302F302E352F31303330333133322F323035302E2F2E2E2F302F312E
3332312E31313131302E3130303131322E2F3231322B2F312B332F3132302E322D332F34
322F342C3331332E2E2D2E3031332C2F32322F352F312E2D3230322D3531302C33322C2F
2C2F31302B2F2E2F3533302F31302F2E31333131322E303332
32302E2F2B3230342E322F302F303133332E2C312A3130303133362F363334322F2D3033
2C3331333136353434302E2F2F322F2D312B34302E2B302C30342E2C30322E30342D2F30
2E342D3232302E2F2D313034302E3332302E3233313130332D33312F2F30313434352F33
342E31303035333232313033362E322D302F2F333132343431313232332B36322F2F3131
3030312E30323035322F39342F3035302E312E302F2D3231303030333134323231333131
302E343133322D312E34302D2E2F2D2F30322F312B313132323131332F342F3431313137
2C3033332E312D322E30303030332B2F2D312D30312E32322D323130302C302D2F343131
2E2B2F30312F34322F3432312E2F2D31313131333633332E302E32302E2F30282F2F332D
2F2E2E313134313033322E2D303032333031312E332E2E2D31
2F2D332B342D2D333130332E302F333130322E2F2E312F33303130323330353131313132
312F323035363732303434303231302E30303034312F2F30322F303134302F2E32302D2C
2E312E2E2F2F322E313034302D2D2E3032323133312F2D303130332E2C2F34313032332F
2F31302E33302F322F332E332D3032342F32333834302F2E343231322E352C342E2F2F32
3332302E2A2C302B33343030332E2C302E3333333233302E342F32342F322D302F323031
312E302E322E2F3130302F33312F32322E2F322F373230302D332F33303433362F302D33
2C2F32302C3331332E2F2E2E2F2C2F2E2E302B34312F2D2C2F302F33332F2F322E2D2E30
2F2F322F2F33312F2C2E2D302F372F2F3431302E303030332E322F2E3430312F322F2B32
332D302C31302D2B3332332E302F30332E2F2D302F2F31302F
352E32303230302D2E32322E31302D333035322F302E32332F30302E30312F3430302E32
2D35302F3034303231323131332E323032312F2E313031302D342D332C2D302E33323331
32332D2E2F2F3130302D30312F332E32342D2F302D2F3234323331332F333131312D3132
323333312F2F2E2F332F302E3034303030362F343033313332322F3236302D3333302E31
303032332F2E2F31302C323131332D352F2F2D2D302F333630312E322F34343335302F30
2B3230312F31312D30322C312D312A313032303134312C2E3332302F302E303231312D2C
2D2F3130312F302E2D322C342D302E2E2F31352F2D2F2E3229342E312F2F2D30342F2E30
31302E312C302F30312E2D3031332C2D302E3033332E3331332F302D3433312C30312F30
302D322D2A2A2E332D302C342E31312C322B32312E322E302F
30313130303131322F2E2E352E303132302F353033302C2F2E2E30343238302E2E312F32
32342C2D2E2F2F31303030323130342F2F2D3130312D2E2E3230322D303736312F31332E
302B342F302E312D33313830322F31342E303232333030312F302E302F2E353130303133
2E31332D2D2E2D2D31322D2E323032333136333230332F3332323333323236333131302F
332E34342E2D303533312F3030302D313030302E31302F32303231323332302F2F2E312E
323232342F322E323231312F3031332D3235302D322F323332302F3133333532312F3231
2B3031312E2E2C30302E31312E332A31302F30332D302B2D2F2E2D3030303130302E3231
31312C272F33332D2C2F3132323531312F30302E2E3131302F2C302E3230313232313232
312A312E2D33312D33322D332F3330343133323232312E332D
323231312C322D31303031303034302C2F30302E2C332C2E2D2E342F3134303235313330
30323034302E3334312E2F2F31312E2E312D302B2E31302F3436322F3335302F2D323231
2D2F2E2F3330322F30332F35323230373233343431302E302E33322D30313031332F342C
342F2E2B31332C332B322D3230323332333434332E33312C3333353030312F2D2E2D3030
2E2C2D2B333032342E30302F2F302E2E2E2E343230312F3731313131332E2F332F2E2F32
322B30312C31302E34323330322F323130332E2E2F3130332F3031323334383032332E2B
303132322F2D2F322C2F2F2A2E3130312E302F3132322F2F2D302C332C2D2B3331332E2E
3231322F2F31312D30302F342F2E2B34352E3130332D2D322E2C33323632303131323334
353133312D2B33312E32303230333331312C3330302E303531
302D32302E3333312F312E2D2F33302F3031342E312F342E3034302E342F323033302F35
3229322D323132322F2E3331343034302A2E322E312D2F2F30312F312F332F31332D2E2F
332C2D2E2F33352F2E3229322E30322E2E332E3336302F36302F2F2E31342F2D302E2E32
2E2E312C30302C2F2C302F2E312F3035312F2D2F2E2E323232302C332F2F31332E302F2D
332F322F31302F2E2D332E31323134312F2C2E312F323336343132322F34323131313130
303231302C3331322D2F2F31333230332F2F2B2E3231353130333132313331313230332E
2E322F2F312E2D2D3030322F303332322D302F34302F33312E303032302C303032302B2C
2E2F2F2D352C302D3232313030322E313233363132302D3030302F2E2F2A2D322E2F3131
31323231313132312F2D2E3230303033322F2D2E3132312F30
2F312F322F2E2F2D3031302C302C30302D322D322D32302E2F2F322E32322E322F342B31
3333312B303231332D3230302E32313034322E31302C302C3230323136302D2C2D30302F
2E302C2D2E29322D33322F362D312E2F2E30353434313331342E312A3231312D33303230
2C2D2F3430352A3230302E3030343132333031312E3330322E312E3031332F2E2F302D30
322E2F2F2C2F31313230333230302F31312C2D32312E3230323230313031333232303130
31312F3232322D31303330342E333032302F34312E322F33302E3330312E322E2E302F32
2E2B2E302D2F312E2E33312F30313133322C302F3231302E2E2D302F2E2C2C323332312E
31302E303231322F2F2C2F2B31313232342D2F322C2E322C312E2F2E3431312F2D2F3330
2E33312F2D2C2D352F2E30302E303035303130323131303131
32323333322F32312F30342C31302C2F302C2C2E31322F312C3330332F3234302E2D312D
30302B30332E2D2E3233322E303032312F2E312E30323430303232312F302C2B3034302F
34333131302E302C31333431343331302D2D322F313135303332352D2E32332E2C303136
2D342D302F2C3331332D2E2E2E2D342F322E352D322E3032332C2D3030312D2E33303131
312E2C302D312F2A2D312D303531332E33312F312B30332F2E31313237342B33312F2E32
2E2F3030322F2C3031323031322E2F312E30303130313233313031302E30303530332C2B
312E312F3233342F2C2C332F2D333432312C2D2F302C312C312D2C2E32302E313230342F
2F3431322D333034302E2F3333343230312E353129332E3031312F303632313532313131
3230332D302E302E322D3432342B2F3330333233323432332B
303031343132302E302F2E332E2B302E313231303231312E2D3031332E32302F342E3134
302D302E323630352F322F302F2E322F35332E33333331353430322C323735313032302E
312C2C2D2E2D32342B3133303531302F333331333132323236343433332B313332303532
2F302D342D35322E322E3033333335303130312D2F2C3030302B313030322F332F342F37
302E33302B31342C2D2D2C2D34343530342F2D302E2F2F332F322D39332C2D2E34332E30
3331313134312F342C312F3330302D31302D352E2F32322E342F322D30302D3230312D31
313433312D33352D2F343633322E3331313233313033302B2B302D2E3431312E2D323532
342F2F2E3230322E2D2F33322B30342E2E3332312F312D3031362F3231312D2E312E3030
31322F30312D342B302C312F32303231312F3230332D312F31
2F332C313031302F312F2F2C30323133322C2D2E302F2F3333312D33302F32323032352E
2D2F3333312D3330323730322D30332F30322D30312F33393237302F3332353431312F30
30312E342D2D2D31322F32332F2B2B30302D322E31313335353236343230333232303332
2F31333336332E3130333034312E2C34302E323131342F33302D2F312F34312C2E302F2D
2F2F332F2F2D2B29302E3030333433322E313130303130312E2F3033352D32312C2D2E2E
2C312C2C332F2E2E3230313030302F2F312C312D2E31323230372D332F2E2E3332312E2D
302D302F3334342F2D2A323030302E332F2C3132322F2E2D342E2B2C2A30342F312F3537
36343133302E3030312E2E2D3030312E2F332F2D302E2F3031322E2F2E2F3230322F3330
303230323031342E2E30312E342B2D3130313130313131322D
2E2F2E2D36312E2F332E303230312C332C2E2D342F322E31332F2F2E3134342F2F333032
352E32323033312E2E3131322C342C303131353333303131313435322D33342E2B332E2E
3133333334302D3331323230312F2E302F2D2E332D3032303232362F303235313331342E
2F3030302F31322E322E31312F2F3030332E30332F322D2F332E302C332E2D322D2F3131
2E2E2E2F33312F2D2F2E3032352F3430362F2E2D332F31313531333230303531312E2F2E
2F302C2B2F322E302E2D2C2F312F2D313230313131303132343131313232333231313430
3031352F31322C302F3133362B31322C2F2E2C30323230302C30312D312F2F2D30363337
3B3330312C3031302B2E30312C2C2F322D32343230342F332D32303331322D2E322E3231
2C31312D2B2C312E2D30312B2F33302F322B34312F33322F2F
2E2F302C3232312F3131322C2A2E2E32322E34342F2C33313334373630332F3036313330
3332302D2E312F30322C2E2E2F2D2F2F32322D2E31323035343035302C2F342F2F30312D
3331312F312F2F2E342E322E302E2E30312C2F302C3631343032332F333335333232332D
313030322E31342F2F3330312D322F32322D312F2D312D2C302E31312E2D302F2E302F2D
2C3131322D2D2D2E322B322D2D33303133302F35342D30302F35323131323232312D2D2C
2D312E312F32313230312E2E2C2E3031322E2B2E302F302F30332C2F3031302B30312E2C
31312C3132323130342E32302F2F32322D2E2E2E2F2C2D302A2D2E2F302E2D303233323E
3834302C2D2D34312B2E313031322F3231302E2F312D2C2D332E3130322D2D312E2E302E
30333335322D2C313232302D33322F32303533313032303233
312E2E2C312F322D333431312E34332E3130352D3630313333343331302C332F2D2E3333
2F30303230352E342F2E2F33303432303033303231313531372F31302E3030333132362E
342D332F31302C342E312F31342F302F35312F30332D323330302D312E32303030303036
3431302E2F2F2D332F2E312E322F3035302E332C312D2E332E2E2E2E2E29342B2D2F3432
32302E3231302D2D2E2B2E2E312C323031332E323030302F3135313632333232332F3130
30322F302F3131302D2E302F313232322F2D2F2C31312D2D31302F34332D2C322B2B2D33
2D2A2E31313230312F332D342D3130332F323130352F292B33322E322D2F2E3238363738
3D382F322D332D2D2D2E2D2C30362F342C343035302B2D2E312F33312D302D2F302F3030
312F302E2D2D2E2C2E2E2F3131302F332E342D3231322D3031
2C303134303030323232333130332E33332B2F31312E363231303131343331343134302F
302F33343B33322E2E2E352D2E2D31302C30332F31312C2B2D33302C2D2C2E3234312F2F
2B2D303032323233323032302C2D2F2F312C30342D2E372F313031322E333132332D322F
32302E2D3135322F303133312F2C2F302E3031312E2B302E303132372C31322D312F2930
2F2F2E2F3331322F302D342F2E30302E2F3332312E2B332F31313131313334322D31302F
332E2E2B302C30332E2D30322E302E313032303132302C332C2F2F322D322D2F302E2D30
2B2F31342A322F2E30302C2D2A2B30303234352E31332F2D332D322E31292E2D31303038
36352F303030302E2D322F2D2D2F2D302F3130322F322F302D31302C2F2F2E302F2F302C
2E2E322F322E2F3031323030302E312E303334313132333031
31312D3131313230313132333134322C2C2E2C303036353538303431313432292F31322D
2F31353030313034312E30312F2E2F312E342C32312F30303130302C312E2E2E31302C30
342F302E332F2C2E30302F32322F2D2D2F2E2F33303131323232313030312E302B35312E
33302F2D3130312E332B2F322D2D2D2F31313132312D2E332E33302D2A2F2C2C3432302F
2F2B303231342E312C2D322E2E3332312E322C2F3230312E313431312E30322F31333130
2E31302E2E30312F303133302F312B3330302B2C342E302C322E2E2F342E312B2F2C312E
2E2F31352E312F342D332C2E2E2E2F303131303130312D2F3031303030302F302C302B2D
3334302F312C323330312F313231332D332F30322E2E2D362D30302F2E2E2C30322B2E2D
2B302F2C33342D322F322C312C302D3132313534312E2F2E32
33312E312A333031322E322E3033312B31332F2F31322F2E2E2F32312E303332312D3231
2C31342E312E30302F302D33343230322E30342F352F2F302E3134302F30353134342C30
2D2C2D2F353333312E312F2F322D303030322E3131302E2D2E342D322B333032322B2F32
3330303030322D302C2E2E2E312F302E2C2F322F302F323331312F2B302E302C2F2F2D31
312E3133332E2E32302F322E2E32332F32302F2D2D31302D322D30322F312F322F2E2F30
302E312F302F2D322D322D2D2F303033332F3131313031302E293130322E33352D312E34
3134313228312E333230342E3131333030333033322E2F33312B2C2C3132332F342F3330
2E3631313630312F2E2D313132343334332F30303031302E2E293230323031312F2F2C30
36303032332E312C36302C302E2B312F2D32322F2F2E312F30
2F302D2F2D302F30323030313032303133312E322E2E30342C2F333134312F352D303331
34302F3230322D2E3130302E2E312D32312E2F342F302B2E303337312C2E302F342D2E30
2C2F312D332F2F34303031313232322E31313230342E2E2F302E2F312F2E342F2F31302D
3230333033302D2E2D2E2D302F2C31302B3230322E2D2D313331342F2E33302C2F282E31
2E2D3133302E302F2B352F2E2E3031302A2F2F3030302F2D2F30312E31302C2B32322E35
2E2F31302D2E2E3130312D30312E2E302E312E312E2D2C2E2C322E2D3030302F2E32302C
2D353030332D32342F312F342E2D2D312D2F3238333133313231312E322C2B332B313034
2D3233362E302E3432302B2A343131343232342F2F302E2F2F312C2F332F302E31323431
2D2E2C3031302C31332E2F2F2D312D2F333335313230332D2E
33312D32322D2E342D2E32322F32302F2E2C31302E3331342E302F2D312B2F36312F3139
32342E30332A3431313130332F2F2F30303330302E2F342F30312E312F302C31302E2F36
312F2F31303233312F32323134302E32303030323132313330302F312E302D342F31302F
3132342E3130322F2F302D2E2D2D2F332E3033303030302F3431302B2A31332F2F2F3529
2C2D30302A2F302E2A2F33302E2C302C2F2D302F312D30332D293332312E302E2C2D302F
2E312C2C2B2D302F2F3234322F30303231312E2F2E3131332E2B302C30302C3332312E30
35312E3231322F32342F322E2D302D303031303832303531332E35332F32312E33323132
313531332F31332C2F2B3431323433332F2D302D302C30302F2C2B302C31312C2E303332
2E312D2D312F32332F31312E2E2E3334333132332E312E312C
3031332C2F2F2C33312E2D34322C3831313031302E2E2F31332F312E31302E2F3630322F
2E32312F2D32322C312D302E332E2E3231342F3031342F33322C2B2E2E2F2D323333312F
2E2E2F2F303130312F32342D2C2E2C303233323131322D3032302D302C2E2D2F302C3030
312F352D342F2E2F2C313133313231332F323032333431312F2E34302E30302E3130362D
32302E332D3030302D2C312D312F2D302F322E2F332F343235332E322C302C312C2F2F30
333131332F312D2E2F302D2F2F3030312E2C322C2C322F352E302F2F302C332F3530322F
302D2F2F30322E3032343332322C33342E323335312E2F382F3331313031303133323030
31302E333031323430312E2E342B312F313030302D2E2E31322B2B312C31332C322E2E31
35313032312D312E2F2E2E312E2F2F312F2F30312F3130322F
2F2E2F2F302E2E322F30302C303331323331302D322F2C30312C30322B3134312D313333
31312F33342D342E2E2E2D33323132323531302E2F3133302F2E2E2E2D302F342F35302F
2F30322E303032302F322C2F312F31322F3232342F2E2F2E332E2F3332322E3032343333
332C2E29302E3134312B312D2D30313136342E302F302E2E2F2B3132322C2F2E302C3532
302E332F312E33343033322E3033302D2F2F313031343236322E2F342F2C342F2D332E32
2D2F2C2D2F32312E2F302E2F2D313031302E2E2A2E2E2F2C2F31343130302E2F30313132
2E30303035312E322E323530302E33303431322F31332F3030332F3030322D30312D3330
2E2B30303333303031342E312C30312A322C3331313232302A3231302F312C302D343130
30312F302F31302F2C2D32332E3033302C302C2D30332E312F
2F323331333030332B2E312D312C32302C3330312D2D2F3231303230302D313332312E2F
33343331312F2D322B3030332D2D2E2E2C33312E302F2C312D2E322E2F2F2B2E32322C34
2F302F2C343130352F312F312C323031332D303133332D2F2D302F2C2C352D3130302F2E
312D2F2F2C303030332C3430332D35303B363435302F31302F32332E292E302F2E313230
312E312E31342E2C343130332F2F2F2F30312C31302F342E3230312E30312F3031313131
2E322F2F2B33313130302F2E322F3236302F302C2B312F322E2E2F2D2D2E2F2F2D2E2F32
3431342E37363032353130332C333231303330332F343031332D33302E2F3430332D3033
2A3030302E312F31332D2D2E2E2E2E2C31312F2F30312E313330302F30332F2D302F2E31
2F313432322C3430322F2F33322B2F34322F2F322C30302D32
312F352F3031322D2F302F2D30382E2F31332A2D3036312D322F33313230332E2F2C3435
2D2E2C2F312D2D312D2A312F2D2E302F2E2C323030302E2D30312B2E2F2F30302F2F3131
302F322E332D2E312B2F343130323031303232332C2E31302D342F2B30302C30312F3231
332E312E2E31362C2F30343134353330333333302F3233302F2E332F33312A2E3130312E
2B2E31312E32312C30312E3030302E2E30333432333330322F35313130323231312E2D33
2F302C332D2E2D2D302D35302E2F3433313535322F312F303030302E2E3130312E322F32
2F312D3033343033353130303032303131313030342F2F3130312D30332D2F2F30322A33
2E2F2F312D2D322D322F302F312D302B2C2D31342C2F2F30332E2A303030312C332C342C
2E2F2E31302E2E302E3030322F2D2C302D332F312C2F30302D
2E322F3033312E3232312E33302F33312D31312E313132302F2E2F313230312E32312D32
302F2F31303431332F2F3234302D34362D30342D2C30303032332E34312E2D2C2C30322F
2F312F2E2E312F3031343334322D2E3031303130323031303231323030322D2F2B31312C
332C322E312F322E342E342C3232313536322C3130352E2C312E30303132303431323232
332F2F32302D3030332F322F303330312E342F2F2F332F2F342E34292C2E2F2D2E313332
32302F2F302C2D2F2F2D313332303333302F2E2F2A2E2C2B2B2F322C3333302E31313433
313632303633333035313432322D30353232303031332F2F322F2E312F2E2F2E30302E33
30322F34312E2C2F312C342F2F2E2E2D312E2E2D2B322E2D302D2E2D2E2E2E342D2F3231
322D2C2D3430302E30352B2E2E30322E302F2C2E2C2F2E322E
33312F322D37312D2C2E33302C303033372C302E303231313134313033312E3631303231
2F2F3334342E2E2D2F2D2F2F3133302F2D31312E342F322F2D342D2F302C2E3630313131
2E2E302F302E2D332F342E3131333231303132322F333131332E31303230312E2F322B2F
302E33303131322F32332F2E32313233382E312E2D3132313234312D3131352C33322F31
302D30332E2F2E2C2E2E2E2F31353234342F2F3430332F342F2E32322D312E2B2B323031
352932312C2E2E3333322F30322F302D31352E2F2B2E2E303034302D3230303432343430
32313030303335302D33303331302F313135363031302B2E2B30302F31323231342E352E
2D2F2C2F2C312F2D2F2A2F2B2B332F2D30322E312B332D30302E2F312E322E2F33343131
2D3231333133342D3431302C2D3035313131332E2C2D2D302B
3032302F332C312E302E2E2E2C2E34292B2F302F303031332D332C313432312D30312F2E
2E2F2E342F2A2C2E31322E333332313033343332312D302E2F2E2E3231302F2E382F3130
2F313130312F2E322D323131312D322F3232303334303131352E32322F302D2C302B3331
2E2D302F2B30333132302F2B3030302B2C313030343230333031332E3130313031303331
2E2B2F3230303532302F3231332F343230312D2E302F332E302F32332C3230312F2A2E30
322D2F312D343132302E3130332E30312D322E2C3030323032302C2F3130352A302F3431
2D322D303232302F33302E32322B34322F2F3131312F2B323331303132332D32302F332F
32312F2C2B2E2D2F302F2D2D332F2E2F302B2E2F302F2F30322F312B2F322A3235342D32
33312D3630312E33302F2D2E2C2D303230302E2C2F2F2B322D
2F2F302E2E30312F2B2E31332F2E302E2F2C332E322D2E3131322E2F2D2E3034302E2D30
302D3133322E2D322A312F303032302F352E313130302F312D3030323230322F332F2C2E
2D2F2E3033302E30313432303131313030312D2E31343335323630312A312D2F2E342F2E
2F33312D323032342F3130312E2E303132343032302E312B2C312E2D2D332D32322F2C2E
332F2C3033302F2E333131313230312F2D302E312C2C3333332E2F2C30302F2F2D2F322C
3131342E3230332C303035303030323231322C2E2F2F3131302F30323233302F2D303033
32322E32322F302D34322D30312A313331312F2C30312D2E2E3230302E322F332E302E33
31302E34322E31342E312C2D2D2F302D2C2F2B2B2E2E2F2E303030302E2F29332F2D312E
312D30302D333134313232312E34322F35312D2B2D2E302D2E
2E302C2E322F332E34312E3331332D322E2F302C2E31342D332D2C323231322D302F3130
3032302F2E3132323130302F322F313133302F302C3132312E3234342C322F31342F2F31
2C2E31312D32312B3334302F36302F34312F3034312F302E2E303032322E322D2D302E35
33322E302E30352E2E2F34312E2E2D2F333131313131322F3032302B2C2D2F302F2B2D30
2C322D2D2E303033312E322F33312F2F3631332C2F322F312F2E2F2F2F2F302B2F34312F
2F2F30332C2E2E3230312F322E2F302F323332302E2F2D2E32322F332D32322D30312F2F
2E312E332E32312C2F32322A2F3130302E31302E342E2E302F30312E2F31303130352F31
34333034333232302E332C312E2E2C2D2E302F2B30303131333331302F2F2D2C2B342D32
2F2E32322D3032302E2B2B332C2E312F302D32302E2E2C3031
32332E2F2E3334312A2C342F2F303032332D2E302C2E31342D3032322F302E3432302D2F
32302E3534302E2F312E32303230322E303030342D2F2D2D312F3330342E34342D31322E
31302F2F32332D3232313331323030302E313231313131303430322E3133302E352C2C2F
2F2E302F34332F33332F2F302D312E2C2E2C3033322D2F31313033312F302E2E2C322F2D
30302C302C2F2F2D3331302F31302E2D2D2C333031333031302F3031322E312D332F3033
32312E322F2C3130322F2F2F302E333230332F333331333031312C342F37322F2E303031
302F312E2B2C2D31343331332C36312E31312B2F2F2C30342E3030303235342E3232302F
332D3330302F2A2D2E2E2D30302A2F2D2B2F2C2D302D3030323334322E322F3134333130
30332E31302F2F342E302D2C2E302F332E312F312F2F2F2E32
302F302A303232302D322F352F303131312E2E2F313030303234332F32312C2C30322E30
302F342D343031313230302C2F3334302D2D30322E33322F302D2E31323233362F2E2F33
332F322F332F31302E30322E302F2F3033302F322F3030352E30322F2E302F33312D2E30
312F32322F33302E31302F2E333030303230313231323033322F2E312E2E2E2E33332F2D
2E3132322F30343130303031332D2E2F32332E32343232313233303330312E2E30302D2F
2D2C2F2C2E302B2E302B322E35332C323631322D30312D2E2F33353130313230312E3035
2D352D312F2F322E333130302E2E302E332E312F31312E33312F312F2F34333335322F2E
2E31312E323732332C3130312E31322C33322D2E322D2E313133312E2D322E3032303130
3130332C2F2F312F332D2E2F2E2F302B303032312D2F2E2E2E
2E322F30302F2F2E33332E32322F3132312D30302A30322F2E2E2C343231312E2B2F322F
302F303230322F313035322E34302F2E2F30322A312D312D2D2C35342B312F3432323032
2E3231333235343231312C322F35302E3031302F302C3030323232352E2D2F2F2E2E322F
303235303432312F31312D302E2E3230322D30302E2D2E3030313133302E3132332B3333
2F2D32343434312E312F2F3233342F2C33322C2E2F3234342D2F2E342F2F303130302B30
2F31312C3333323034312E2B3236312F32352E2E32303431353034312D2D2F2F2E2E332C
2D3135302F2E312C33312E2C2E2C2D2D302D2E302E312E2D333431302D2E3234342D342B
30312D2E3130302E32322B2D322D2D2C2C322F2D2E2E2C3033332F32322F2E2B2F313330
32312F2F3230302F2F2F2E2D31332A3033332B332E302F3331
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{397, 397},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006=0001SA000`40O003h00O0?m_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgX008e[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX00Fmk003oJeZ>JeX0003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J005[NP00of]JSV]J0000ofmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00SF]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FP01Kg/00?m[FXi[FP0001a[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP80000eKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_N`02000016]JKg][FV]j0P0002e[FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX0
1@0000Q[FVmk000006]JKg][FV]j0P0002Y[FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX5000026mkJeX0
0000Kg][FV]jJeX20000:f]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk00@00003JeY[NV]J00800004
JgY[FVmkJeX20000:VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP@00003JgY[FVmk00800004JeY_Nf]J
JgX20000:f]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J00800004JgY[FVmkJeX2000016mkJeY[NV]J
0P00011[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP5[NP006f]J00@006]JJeX003=[FP04
001[FV]J0002JeX01000JeY[FP00;V]J00D006]JJeY[FP00009[FP04001[FV]J0002JeX00`00JeY[
FP0YJeX01@00JeY[FV]J00000V]J00@006]JJeX0009[FP03001[FV]J02Q[FP03001[FV]J00=[FP04
001[FV]J0002JeX01000JeY[FP00:F]J00<006]JJeX00f]J00@006]JJeX0009[FP04001[FV]J000Y
JeX01000JeY[FP000V]J00@006]JJeX0009[FP04001[FV]J000@JeX0003oKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j001_Nf]J001[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeX006]JJgX006mkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeX006]JKg/006]jJeX006]JJgX006mkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]j001_Nf]JJgY[FVmkJeX006]JKg/006]jJeX006]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]J001[FV]j002=JeY_N`00JgY[FP00JeY[NP00Kg][FP00JeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmk001[NV]JKg][FV]jJeY_N`00JgY[FP00JeY[NP00Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgX006mkJeY[NP00Kg][FP00JeY_N`00JgY[FP00JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
005_N`006f]J00@006]JJeX003A[FP8000=[FP04001[FV]J000^JeX01@00JeY[FV]J00000V]J00@0
06]JJeX0009[FP03001[FV]J02U[FP03001[FV]J009[FP8000=[FP04001[FV]J000/JeX01@00JeY[
FV]J00000V]J00@006]JJeX0009[FP03001[FV]J02U[FP03001[FV]J009[FP8000=[FP04001[FV]J
000/JeX01000JeY[FP000V]J00@006]JJeX0009[FP03001[FV]J00i[FP000?m[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeX006]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FP00JeY_N`00JgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FP00JeY[NP00Kg][FP00JeY_N`00JgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg/006]jJeY_Nf]J001[FVmkJeY[NP00Kg][FP00JeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmk001[NV]J000008e[FV]j001_Nf]J001[FVmk001[NV]J001[FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]J001[FVmk001[NV]JKg/006]jJeY_Nf]J001[FVmk001[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
001_Nf]J001[FVmk001[NV]J001[FV]j001_Nf]J001[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeX00F]j000LJeX2000eJeX30003JeX2000^JeX20004JeX20004JeX2000[JeX20004JeX30003JeX2
000/JeX20004JeX20004JeX2000[JeX20004JeX30003JeX2000[JeX20004JeX20004JeX2000AJeX0
003oKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j002=JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]J005_N`00of]JSV]J0000of]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_N`00SF]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP01JgX00?m[FXi[
FP000?m_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX008e[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeX00Fmk003oJeZ>JeX0003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmk002=JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J005[NP005V]J
o`00M`000F]J0000ofmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[
FVmkJeY[NV]JKg/006]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeY_Nf]JJgY[FVmkJeY[NV]J001[
FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]JJgY[
FVmkJeY[NP00Kg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][
FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]JJgY[
FP00JeY[NV]JKg][FV]jJeY_Nf]JJgX006mkJeY[NV]JKg][FV]jJeY_Nf]JJgX006mkJeY[NV]JKg][
FV]jJeY_Nf]J001[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg][FV]jJeY_N`00JgY[
FVmkJeY[NV]JKg][FV]jJeY_N`00JgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NV]JKg][
FV]jJeX006]JJgY[FVmkJeY[NV]JKg][FV]j001_Nf]JJgY[FVmkJeY[NP00SF]JKg][FV]j001_Nf]J
JgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmkJeY[NV]J
Kg/006]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]J
JgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]JJgY[FVmkJeX006]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]J
JgY[FVmk001[NV]JKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP03001[FV]J03M[
FP03001[FV]J03I[FP03001[FV]J03I[FP03001[FV]J03M[FP03001[FV]J03I[FP03001[FV]J03I[
FP03001[FV]J01=[FP40005[FP000?m[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FP00JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/008e[FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP3oJeYc
JeX10001JeX0000>Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP80003oKg][FV]jJeY_Nf]J001[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]j001mJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX00001Kg/000e[FP04001[FV]J0005JeX00`00JeY[FP3o
JeYcJeX10001JeX0000FJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX006mkJeX006]JKg][FV]jJeX4
0000ofmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00L6]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]j0`000F]j000=JeX01000JeY[FP001F]J00<006]JJeX016]J0QBU
00DHaQBU5:DHaQBU00<HaP<DY@THaP035:DHaQBU00DHaP045:DHaQS66<H25:D01AS65:DDYAS65:D0
0aS600@DYAS65:DDY@@HaP<DY@036<HDYAS6008DY@LHaP@DY@036<HDYAS600@HaPHDY@046<HDYAS6
5:D46<H25:D26<H35:D00aS65:DDY@026<H25:D46<H25:D46<H45:D26<H25:D00aS65:DDY@065:D3
6<H25:D76<H011BU6<HDYAS60QBU0QS600<DYAS65:D00QBU00<HaQBU5:D00aS600<DYAS65:D01QS6
11BU00DHaQBU5:DHaQBU008HaP8DY@036<HDYAS600HHaP<DY@8HaP045:DHaQS65:D66<H011BU6<HH
aQS60QBU0QS60QBU00<HaQBU5:D00aS600<DYAS65:D021BU0QS60QBU00<HaQBU6<H00aBU0QS600<D
YAS66<H011BU0QS600@DYAS66<HHaP@DY@8HaP8DY@036<HDYABU008DY@DHaP045:DHaQS65:D26<H0
1QBU6<HHaQS65:DHaP<DY@LHaP035:DHaQS6008DY@8HaP035:DHaQS600DHaP065:DHaQS66<HDYAS6
11BU0aS611BU00<HaQBU5:D00aBU00@HaQBU5:DDY@DHaP<DY@8HaP8DY@<HaPM[FP40005[FP0001i_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_N`00JgY[FP00JeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk6<H2
5:D36<H011BU6<HHaQBU3QS600@DYAS66<HHaP8DY@036<HDYAS600<HaP<DY@/HaP8DY@<HaP035:DH
aQBU008HaP035:DHaQBU008HaP@DY@046<HDYABU5:D26<H35:D36<H35:D36<H25:D26<H00aBU6<HD
Y@026<H00aBU6<HHaP036<H01ABU6<HHaQS65:D011S600@DYAS66<HDY@DHaP@DY@@HaP8DY@8HaP8D
Y@@HaP8DY@036<HDYAS6008DY@8HaP8DY@@HaP8DY@8HaP045:DHaQS66<H25:D76<H01ABU6<HDYAS6
5:D00aS600<DYAS65:D00aS600@DYAS65:DHaP8DY@<HaP8DY@036<HDYAS600HHaP8DYA4HaP065:DH
aQBU5:DHaQBU0QS61ABU00<HaQBU5:D01AS60aBU0QS600<DYAS66<H011BU11S60QBU00DHaQBU5:DD
YAS600<DY@046<HDYAS65:D86<H45:D00aS65:DDY@025:D36<H01ABU6<HDYAS65:D011S600@DYAS6
6<HHaP8DY@@HaP8DY@<HaP035:DHaQBU008DY@DHaP035:DHaQS6008HaP8DY@HHaP8DY@<HaPDDY@8H
aP@DY@8HaP08Kg][FV]jJeY_Nf]JJgX0005_N`003V]J0P001V]J00<006]JJeX016]J00<HaQBU6<H0
0QBU00<HaQBU6<H01QS600@DYAS66<HHaP8DY@<HaP035:DHaQS600<DY@<HaP8DY@@HaP035:DHaQS6
00@HaP8DY@036<HDYABU00@DY@046<HDYABU5:D46<H25:D01QS65:DDYAS65:DHaP8DY@<HaP8DY@03
6<HDYAS600<HaP065:DHaQS65:DHaQBU0aS61ABU0aS600<DYAS66<H00aBU0aS600<DYAS65:D00aBU
0QS600<@Q1BU5:D00QS60QBU00@HaQBU5:DDY@8HaP8DY@036<HDYAS600<DY@046<HDYAS65:D?6<H0
0aBU6<HDY@025:D00aS65:DHaP066<H45:D36<H00aBU6<HHaP076<H00aBU6<HHaP026<H00aBU6<HH
aP025:D26<H00aBU6<HHaP056<H00aBU6<HHaP0=6<H01ABU6<HHaQS65:D01aS600HDYAS66<HHaQBU
6<H35:D46<H00aBU6<HHaP036<H01ABU6<HHaQS65:D00QS600HDYAS65:DHaQBU6<H45:D36<H35:D4
6<H011BU6<HDYABU0QS600<DYAS66<H00aS60QBU00<HaQBU6<H011S600@DYAS66<HHaP8DY@036<HD
YAS600<DY@8HaP8DY@036<HDYAS600<DY@046<HDYABU5:D66<H00aBU6<HDY@0:6<H25:D26<H00aBU
6<HDY@025:D00aS65:E[FP06JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQS65:DHaP8DY@036<HDYAS6008HaP075:DHaQS66<HD
YAS65:D00aS600<DYAS66<H00aS60aBU00@HaQBU6<HDY@HHaP035:DHaQS6008DY@056<HDYABU6<HD
Y@066<H35:D76<H35:D00aS65:DDY@046<H35:D26<H35:D26<H00aBU6<HHaP046<H25:D00aS65:DH
aP046<H00aBU6<HDY@036<H00aBU6<HDY@026<H25:D46<H011BU6<HDYABU11S600@DYAS66<HHaP8D
Y@@HaP<DY@8HaPHDY@PHaP8DY@DHaP055:DHaQS65:DHaP025:D46<H00aBU6<HHaP026<H45:D46<H0
11BU6<HDYABU0aS600<DYAS65:D011S600<DYAS66<H02AS600<DYAS65:D00aBU00<HaQBU5:D00QS6
00<DYAS65:D00aS60QBU0QS600@DYAS66<HHaP<DY@056<HDYAS66<HDY@036<H35:D00aS65:DHaP02
6<H00aBU6<HDY@035:D00aS65:DHaP025:D00aS65:DDY@026<H011BU6<HDYABU0QS600HDYAS66<HH
aQBU6<H25:D56<H55:D00aS65:DHaP046<H011BU6<HHaQS60QBU00<HaQBU5:D00QS611BU00@HaQBU
6<HDY@@HaP035:DHaQS6008HaP8DY@056<HDYAS66<HDY@026<H01ABU6<HHaQS65:D011S600@DYAS6
5:DHaP<DY@0;6<HDYABUJgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP036<HDYAS6
008DY@036<HDYAS6008HaP075:DHaQS66<HDYAS65:D00aS600<DYAS66<H00aS60aBU00@HaQBU6<HD
Y@HHaP035:DHaQS6008DY@056<HDYABU6<HDY@066<H35:D76<H35:D00aS65:DDY@046<H35:D26<H3
5:D26<H00aBU6<HHaP046<H25:D00aS65:DHaP046<H00aBU6<HDY@036<H00aBU6<HDY@026<H25:D4
6<H011BU6<HDYABU11S600@DYAS66<HHaP8DY@@HaP<DY@8HaPHDY@PHaP8DY@DHaP055:DHaQS65:DH
aP025:D46<H00aBU6<HHaP026<H45:D46<H011BU6<HDYABU0aS600<DYAS65:D011S600<DYAS66<H0
2AS600<DYAS65:D00aBU00<HaQBU5:D00QS600<DYAS65:D00aS60QBU0QS600@DYAS66<HHaP<DY@05
6<HDYAS66<HDY@036<H35:D00aS65:DHaP026<H00aBU6<HDY@035:D00aS65:DHaP025:D00aS65:DD
Y@026<H011BU6<HDYABU0QS600HDYAS66<HHaQBU6<H25:D56<H55:D00aS65:DHaP046<H011BU6<HH
aQS60QBU00<HaQBU5:D00QS611BU00@HaQBU6<HDY@@HaP035:DHaQS6008HaP8DY@056<HDYAS66<HD
Y@026<H01ABU6<HHaQS65:D011S600@DYAS65:DHaP<DY@036<HDYABU00M[FP40005[FP0001i_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:D56<H0
0aBU6<HHaP086<H00aBU6<HHaP026<H35:D66<H25:D00aS65:DDY@026<H00aBU6<HHaP075:D02AS6
5:DHaQS65:DHaQBU6<HDY@036<H011BU6<HHaQS611BU11S600<DYAS66<H02QS60QBU11S600HDYAS6
5:DDYAS65:D56<H25:D:6<H35:D26<H00aBU6<HHaP045:D46<H00aBU6<HDY@025:D01QS65:DDYAS6
5:DHaPDDY@056<HDYAS65:DHaP035:D26<H00aBU6<HDY@026<H35:D00aS65:DDY@025:D36<H25:D3
6<H45:D01QS65:DDYABU6<HDY@<HaP8DY@PHaP<DY@XHaP035:DHaQS600THaP<DY@046<HDYABU5:D4
6<H021BU6<HHaQBU6<HDYAS65:D46<H01ABU6<HHaQBU6<H00QBU00<HaQBU6<H011S61QBU0QS600<D
YAS65:D01ABU1AS600<DYAS66<H00QBU00@HaQBU5:DHaP<DY@HHaP<DY@036<HDYAS600@DY@8HaP03
5:DHaQBU00<DY@8HaP035:DHaQS600THaP055:DHaQBU5:DHaP025:D56<H00aBU6<HHaP086<H02aBU
6<HHaVmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX00aBU6<HHaP0>6<H35:D46<H0
11BU6<HDYABU2AS60aBU00<HaQBU5:D00QBU0QS60QBU00@HaQBU5:DDY@8HaP8DY@066<HDYABU5:DH
aQBU11S600DDYAS65:DHaQBU00@HaP035:DHaQS600THaP035:DHaQS6008DY@<HaP055:DHaQS65:DH
aP055:D011S65:DDYABU0QS62aBU00<HaQBU6<H011S600HDYAS65:DHaQBU6<H45:D00aS65:DDY@05
6<H00aBU6<HHaP026<H00aBU6<HDY@035:D36<H85:D00aS65:DHaP055:D011S65:DHaQBU0QS600DD
YAS66<HHaQBU00DHaP8DY@036<HDYABU00<DY@8HaP035:DHaQS600DHaP035:DHaQBU00<DY@036<HD
YAS6008DY@8HaP035:DHaQBU00PHaP035:DHaQS600<DY@036<HDYABU00@HaP<DY@@HaP8DY@PHaP06
5:DHaQBU5:DHaQBU0QS600LDYAS66<HHaQBU6<HDY@026<H00aBU6<HHaP026<H25:D26<H45:D:6<H0
11BU6<HHaQBU1AS60QBU0aS60QBU1QS600<DYAS66<H011S60QBU1AS60QBU0QS60QBU0QS600@DYAS6
6<HDY@8HaP035:DHaQBU00M[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00<HaP8DY@8HaP045:DHaQS66<H25:D26<H00aBU6<HH
aP025:D011S65:DHaQBU0QS60QBU1QS60aBU0aS600<DYAS65:D011BU00<HaQBU6<H00QBU0QS600<D
YAS66<H00aBU11S600@DYAS66<HHaP<DY@LHaP035:DHaQBU00<DY@8HaP8DY@096<HDYABU6<HDYAS6
5:DHaQBU008HaP055:DHaQBU6<HDY@036<H45:D00aS65:DDY@025:D46<H25:D011S65:DDYAS61QBU
00@HaQBU5:DDY@8HaP045:DHaQBU6<H55:D26<H25:D26<H35:D36<H25:D26<H00aBU6<HDY@075:D4
6<H25:D00aS65:DHaP026<H00aBU6<HHaP076<H00aBU6<HHaP066<H00aBU6<HHaP026<H00aBU6<HH
aP026<H00aBU6<HHaP026<H00aBU6<HHaP026<H35:D;6<H25:D011S65:DHaQBU1QS60QBU0QS60aBU
0QS60QBU1QS60QBU0aS600@DYAS65:DHaP<DY@@HaP035:DHaQBU00@DY@036<HDYABU008DY@8HaP03
5:DHaQS6008DY@066<HDYAS65:DHaQBU11S60QBU1QS600@DYAS66<HHaP8DY@<HaP035:DHaQBU008D
Y@<HaP035:DHaQBU008DY@056<HDYABU6<HDY@0;6<H25:D026]jJeY_Nf]JJgY[FVmk0001JgX001I[
FP03001[FV]J00A[FP035:DHaQS6008HaP035:DHaQS600DHaP035:DHaQS600HHaP035:DHaQS600@H
aP035:DHaQS6008DY@DHaP8DY@096<HDYABU5:DHaQBU6<HDYAS600DDY@DHaP045:DHaQBU6<H35:D0
0aS65:DHaP025:D66<H35:D46<H00aBU6<HDY@036<H00aBU6<HDY@025:D011S65:DDYABU1aS60QBU
00<HaQBU5:D00QBU0QS60QBU1AS60QBU00<HaQBU5:D00QS611BU00LHaQBU5:DDYAS65:DHaP045:D2
6<H00aBU6<HHaP065:D86<H35:D26<H25:D011S65:DHaQBU0aS600<DYAS66<H01AS60QBU41S60QBU
0QS600HDYAS66<HHaQBU6<H45:D00aS65:DDY@025:D26<H00aBU6<HDY@056<H00aBU6<HHaP035:D5
6<H00aBU6<HHaP056<H00aBU6<HHaP046<H35:D46<H00aBU6<HHaP036<H011BU6<HDYABU0aS600@D
YAS65:DHaPLDY@066<HDYABU5:DHaQBU0QS600<DYAS66<H00QS600@DYAS66<HHaP8DY@036<HDYAS6
00DHaP035:DHaQBU008HaP@DY@DHaP8DY@036<HDYABU008DY@@HaP035:DHaQS600@HaPLDY@036<HD
YABU00M[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FP00JeY_Nf]JJgY[FVmk00<HaP8DY@8HaP035:DHaQS600@DY@046<HDYAS65:D86<H25:D36<H25:D5
6<H00aBU6<HHaP046<H00aBU6<HDY@025:D26<H00aBU6<HHaP025:D46<H25:D011S65:DHaQS60QBU
0QS600@DYAS65:DDY@@HaP045:DHaQS66<H25:D26<H45:D36<H85:D011S65:DHaQS60aBU1AS60QBU
0aS61ABU00<HaQBU6<H00aS60aBU00<HaQBU6<H00QBU00@HaQBU5:DHaP<DY@046<HDYAS66<H25:D3
6<H00aBU6<HDY@046<H01ABU6<HDYABU6<H00aBU00HHaQBU6<HDYAS65:D26<H01ABU6<HHaQS65:D0
11S600@DYAS66<HDY@<HaP055:DHaQS65:DHaP025:D26<H25:D00aS65:DHaP0:6<H35:D01aS65:DD
YABU6<HDYAS600@DY@8HaP035:DHaQS600@DY@036<HDYABU00HHaP8DY@@HaP045:DHaQS66<H35:D2
6<H25:D26<H35:D00aS65:DHaP025:D26<H35:D26<H35:D011S65:DDYABU0QS62QBU0aS600<DYAS6
6<H00aS600@DYAS65:DHaP8DY@036<HDYAS6008HaP<DY@036<HDYAS600HHaP@DY@8HaP035:DHaQS6
008HaP045:DHaQS600055:D26<H35:D026mkJeY[NV]JKg][FV]j0001Kg/001I[FP<000A[FP036<HD
YAS600HHaP8DY@8HaP035:DHaQBU00LHaP035:DHaQS6008HaP035:DHaQS6008DY@`HaP035:DHaQS6
008DY@@HaP035:DHaQBU00<HaP055:DHaQBU6<HDY@026<H00aBU6<HDY@055:D01AS65:DDYABU6<H0
11BU00@HaQBU6<HDY@@HaP8DY@036<HDYABU00HDY@PHaP055:DHaQS66<HDY@036<H011BU6<HHaQBU
1AS611BU0QS611BU0QS600<DYAS66<H01ABU0QS600@DYAS66<HDY@8HaP035:DHaQS600<HaP8DY@TH
aP035:DHaQBU00<DY@036<HDYABU00<HaP<DY@DHaP035:DHaQBU008HaP8DY@066<HDYAS65:DHaQBU
21S60aBU00<HaQBU6<H00aBU11S60QBU0QS600<DYAS65:D00aBU1AS600<DYAS65:D00QS600@DYAS6
6<HHaP8DY@8HaP055:DHaQS66<HDY@026<H25:D76<H45:D01QS65:DDYABU6<HDY@8HaP035:DHaQS6
008DY@046<HDYABU5:D36<H25:D46<H00aBU6<HHaP026<H35:D011S65:DHaQS60QBU0QS60QBU11S6
0QBU0QS600<DYAS65:D00aS600@DYAS66<HDY@<HaP035:DHaQS6008HaP035:DHaQS600HDY@8HaP03
5:DHaQS600I[FP80005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FP00JeY[NV]JKg][FV]j00<HaP8DY@8HaP035:DHaQS600<HaP<DY@036<HDYAS600LHaP<DY@03
6<HDYABU008HaP8DY@8HaPLDY@@HaP@DY@056<HDYAS65:DHaP025:D01AS65:DHaQBU6<H00aBU0aS6
00@DYAS65:DDY@8HaP<DY@DHaP8DY@8HaP8DY@8HaP045:DHaQBU5:D26<H00aBU6<HHaP026<H25:D2
6<H35:D36<H25:D96<H25:D36<H011BU6<HHaQBU1AS600@DYAS65:DDY@DHaP035:DHaQS6008DY@8H
aP035:DHaQBU00<DY@036<HDYAS600DHaP045:DHaQS66<H45:D011S65:DHaQS60aBU0QS600<DYAS6
5:D00aS600<DYAS66<H00QS60aBU0QS600@DYAS66<HHaP8DY@<HaP035:DHaQS600@DY@8HaP035:DH
aQS600@DY@036<HDYAS600@HaP035:DHaQS600<DY@056<HDYAS66<HDY@056<H00aBU6<HHaP035:D0
0aS65:DHaP035:D26<H25:D76<H00aBU6<HHaP035:D26<H00aBU6<HHaP025:D66<H25:D00aS65:DD
Y@025:D26<H25:D00aS65:DDY@046<H55:D66<H011BU6<HDYABU1AS600@DYAS66<HDY@<HaP035:DH
aQS6008HaP045:DHaQS66<H35:D56<H25:D02aS65:DDYF]jJeY_Nf]JJgY[FVmk00000F]j000FJeX0
0`00JeY[FP04JeX36<H25:D26<H00aBU6<HHaP036<H35:D00aS65:DHaP076<H35:D00aS65:DDY@02
6<H25:D26<H75:D46<H45:D01AS65:DHaQBU6<H00QBU00DHaQBU6<HDYAS600<DY@<HaP045:DHaQBU
5:D26<H35:D56<H25:D26<H25:D26<H011BU6<HDYABU0QS600<DYAS66<H00QS60QBU0QS60aBU0aS6
0QBU2AS60QBU0aS600@DYAS66<HDY@DHaP045:DHaQBU5:D56<H00aBU6<HHaP025:D26<H00aBU6<HD
Y@035:D00aS65:DHaP056<H011BU6<HHaQS611BU00@HaQBU6<HHaP<DY@8HaP035:DHaQBU00<HaP03
5:DHaQS6008HaP<DY@8HaP045:DHaQS66<H25:D36<H00aBU6<HHaP045:D26<H00aBU6<HHaP045:D0
0aS65:DHaP046<H00aBU6<HHaP035:D01AS65:DHaQS65:D01AS600<DYAS66<H00aBU00<HaQBU6<H0
0aBU0QS60QBU1aS600<DYAS66<H00aBU0QS600<DYAS66<H00QBU1QS60QBU00<HaQBU5:D00QBU0QS6
0QBU00<HaQBU5:D011S61ABU1QS600@DYAS65:DDY@DHaP045:DHaQS65:D36<H00aBU6<HHaP026<H0
11BU6<HHaQS60aBU1AS60QBU00<HaQBU5:D01f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QS600<DYAS66<H011S600@DYAS6
5:DDY@DHaP8DY@<HaPDDY@056<HDYAS66<HDY@036<H25:D011S65:DDYABU0aS600<DYAS65:D011BU
00<HaQBU6<H00aS60aBU00<HaQBU5:D01AS600<DYAS66<H00aBU0QS60QBU1AS600@DYAS66<HDY@@H
aP<DY@@HaP035:DHaQBU00DHaP035:DHaQS6008HaP8DY@<HaP035:DHaQBU008HaP035:DHaQS6008H
aP@DY@036<HDYAS600DHaP045:DHaQS65:D36<H00aBU6<HDY@046<H35:D00aS65:DHaP036<H00aBU
6<HDY@0:6<H25:D26<H00aBU6<HHaP026<H35:D36<H35:D011S65:DHaQBU0aS600DDYAS66<HHaQBU
00HHaP035:DHaQS600HHaPDDY@046<HDYAS65:D26<H25:D46<H25:D26<H00aBU6<HHaP046<H011BU
6<HDYABU0aS600DDYAS65:DDYAS6008DY@8HaP045:DHaQS66<H45:D011S65:DDYAS61QBU00<HaQBU
5:D00aBU11S60aBU31S60aBU0QS60aBU0QS600<DYAS66<H00QS60aBU00@HaQBU6<HHaP8DY@@HaP03
5:DHaQS6008DY@8HaP<DY@@HaP035:DHaQS600@HaP8DY@096<I_Nf]JJgY[FVmkJeY[NP00005_N`00
5V]J00<006]JJeX016]J0QS60aBU0aS60QBU0aS60QBU21S600<DYAS66<H00QBU00<HaQBU6<H00QS6
00<DYAS66<H00QS600<DYAS66<H00aBU00@HaQBU6<HDY@8HaP8DY@036<HDYABU00DDY@056<HDYABU
5:DHaP025:D36<H00aBU6<HHaP025:D01QS65:DDYAS65:DHaP8DY@036<HDYAS6008HaP035:DHaQS6
00DHaP<DY@046<HDYAS65:D56<H00aBU6<HHaP035:D00aS65:DDY@096<H01aBU6<HHaQS65:DHaQBU
00DHaP035:DHaQBU00DHaP035:DHaQBU00@HaP037>LDYAS6008DY@036<HDYABU008HaP8DY@056<HD
YAS66<HDY@026<H25:D011S65:DDYABU0aS600<DYAS66<H011S60QBU0QS60QBU21S600@DYAS66<HD
Y@hHaPHDY@036<HDYABU008HaP8DY@036<HDYAS600<DY@8HaP035:DHaQBU00<DY@<HaPHDY@THaP03
5:DHaQS6008HaP035:DHaQBU00LDY@<HaP045:DHaQS65:D76<H011BU6<HDYAS60aBU21S600<DYAS6
6<H00QS61ABU11S60QBU00<HaQBU5:D01AS60QBU00<HaQBU6<H011S600<DYAS66<H01QS60QBU00<H
aQBU5:D00QBU00<HaV]JJeX01F]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXHaP8DY@XHaP035:DHaQS600@DY@/HaP035:DHaQBU
00@DY@046<HDYABU5:D46<H011BU6<HDYABU0aS600@DYAS65:DHaPHDY@DHaP055:DHaQS65:DHaP05
5:D66<H35:D26<H25:D36<H25:D00aS65:DDY@025:D00aS65:DHaP035:D26<H00aBU6<HHaP025:D0
11S65:DDYABU0QS60QBU00HHaQBU5:DHaQBU6<H25:D26<H00aBU6<HHaP045:D36<H45:D26<H00aBU
6<HHaP046<H011BU6<HDYABU0QS600DDYAS65:DDYAS600<DY@036<HDYABU00`HaP8DY@8HaP055:DH
aQS66<HDY@046<H55:D36<H00aBU6<HHaP046<H00aBU6<HHaP096<H011BU6<HHaQBU1AS60aBU0aS6
00@DYAS66<HDY@@HaP035:DHaQBU00<HaP8DY@036<HDYABU00LDY@036<HDYABU008HaP045:DHaQBU
5:D46<H00aBU6<HHaP025:D76<H00aBU6<HDY@035:D36<H25:D01QS65:DHaQBU6<HDY@@HaP045:DH
aQBU5:D46<H011BU6<HDYABU0aS600@DYAS66<HHaPLDY@036<HDYABU008HaP8DY@8HaP@DY@036<HD
YAS600<HaP0;5:DHaQBUJgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP/HaP8DY@LH
aP<DY@036<HDYAS6008DY@066<HDYABU6<HDYAS60aBU1AS60QBU00HHaQBU6<HDYAS65:D26<H25:D0
0aS65:DHaP036<H00aBU6<HHaP046<H35:D26<H00aBU6<HHaP036<H00aBU6<HHaP025:D00aS65:DH
aP096<H00aBU6<HDY@045:D26<H25:D011S65:DHaQBU11S60aBU00<HaQBU5:D01aS600<DYAS66<H0
0aS60QBU11S60QBU0QS60aBU0aS60QBU2aS60QBU0aS611BU00@HaQBU6<HDY@@HaP8DY@@HaP8DY@03
6<HDYAS600@HaP8DY@036<HDYAS600DHaP035:DHaQS600<HaP035:DHaQS600DHaPXDY@<HaP8DY@8H
aP035:DHaQBU00@DY@036<HDYAS600DHaP035:DHaQBU008DY@036<HDYAS600HDY@036<HDYAS600DH
aP8DY@8HaP8DY@8HaP035:DHaQS600@HaP035:DHaQBU008DY@8HaP@DY@@HaP035:DHaQBU008DY@8H
aP035:DHaQS6008DY@036<HDYABU00<HaP035:DHaQS600<HaP8DY@<HaP<DY@046<HDYABU5:D76<H0
0aBU6<HHaP035:D66<H00aBU6<I[FP06JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`066<H00aBU6<HHaP045:D26<H00aBU6<HH
aP026<H01QBU6<HHaQS65:DHaP8DY@8HaP8DY@036<HDYAS6008HaP035:DHaQBU00THaP045:DHaQS6
5:D36<H45:D26<H00aBU6<HDY@026<H25:D00aS65:DDY@036<H35:D00aS65:DHaP025:D011S65:DD
YAS611BU00HHaQBU6<HDYAS65:D76<H45:D86<H25:D01QS65:DDYABU6<HDY@HHaP035:DHaQS600HD
Y@8HaP055:DHaQBU5:DHaP025:D56<H00aBU6<HDY@025:D56<H35:D96<H45:D00aS65:DHaP026<H0
0aBU6<HHaP036<H25:D36<H00aBU6<HHaP086<H25:D26<H011BU6<HHaQS60aBU0QS611BU11S60QBU
0aS600<DYAS66<H00QS61ABU2AS600<DYAS66<H00QS600<DYAS66<H01AS600@DYAS66<HDY@8HaP<D
Y@DHaP065:DHaQS66<HDYAS60aBU11S611BU0aS600DDYAS66<HDYAS600<DY@THaP8DY@046<HDYAS6
5:D66<H25:D00aS65:DHaP036<H25:D011S65:DDYABU1AS60QBU00<HaQBU6<H00QS600<DYAS66<H0
0aS600Q_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX86<H00aBU6<HHaP025:D76<H2
5:D26<H45:D00aS65:DHaP0>6<H00aBU6<HDY@045:D011S65:DDYAS61ABU00@HaQBU6<HDY@@HaP04
5:DHaQS65:D26<H011BU6<HHaQBU21S611BU00DHaQBU6<HHaQBU00dHaP035:DHaQS6008HaP8DY@XH
aP035:DHaQS600@HaP035:DHaQBU008DY@@HaP@DY@036<HDYAS600@HaP045:DHaQBU6<H45:D01QS6
5:DDYAS65:DHaP@DY@8HaP8DY@8HaP035:DHaQS600DHaP8DY@lHaP045:DHaQS66<H25:D26<H25:D0
0aS65:DHaP046<H00aBU6<HHaP026<H01ABU6<HHaQS65:D01aS600@DYAS66<HDY@8HaP<DY@8HaP8D
Y@046<HDYAS65:D46<H00aBU6<HHaP025:D36<H25:D00aS65:DDY@026<H011BU6<HHaQBU0aS600<D
YAS66<H00QS60QBU11S600<DYAS65:D00QBU0aS600<DYAS66<H011BU1AS60aBU00<HaQBU5:D01ABU
0QS60QBU00<HaQBU6<H00aS60QBU00<HaQBU6<H01ABU0aS611BU0aS600<DYAS66<H00QS60QBU00<H
aV]JJeX01F]J0@000F]J000086]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]J001[FV]jJeY_Nf]JJgXHaQBU6<H25:D01AS65:DHaQS65:D00aS600<DYAS66<H01aS60aBU31S6
00<DYAS66<H011S600<DYAS65:D00QBU0QS600@DYAS65:DDY@8HaP<DY@046<HDYABU5:D36<H00aBU
6<HHaP046<H25:D66<H011BU6<HDYABU1AS600<DYAS65:D01aS600<DYAS66<H00aS600<DYAS66<H0
0aS600<DYAS65:D01AS600HDYAS66<HDYAS65:D36<H00aBU6<HHaP025:D26<H25:D56<H01QBU6<HD
YAS65:DHaP8Li`8HaP045:DHaQS66<H25:D00aS65:DHaP025:D011S65:DDYABU0aS600@DYAS66<HH
aP8DY@036<HDYAS600HHaP035:DHaQS600/HaP045:DHaQS65:D56<H25:D66<H55:D00aS65:DHaP0:
6<H25:D00aS65:DDY@026<H00aBU6<HHaP056<H45:D00aS65:DDY@056<H011BU6<HDYAS60QBU00<H
aQBU6<H00QS600<DYAS66<H02AS600<DYAS65:D01aS60QBU0aS600<DYAS66<H00QBU0QS600DDYAS6
5:DHaQBU00<HaP<DY@DHaP045:DHaQS65:D46<H00aBU6<HHaP026<H55:D00aS65:DHaP025:D;6<H0
26]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP036<HDYAS6008DY@056<HDYAS66<HD
Y@036<H00aBU6<HHaP076<H35:D<6<H00aBU6<HHaP046<H00aBU6<HDY@025:D26<H011BU6<HDYABU
0QS60aBU00@HaQBU5:DDY@<HaP035:DHaQS600@HaP8DY@HHaP045:DHaQBU5:D56<H00aBU6<HDY@07
6<H00aBU6<HHaP036<H00aBU6<HHaP036<H00aBU6<HDY@056<H01QBU6<HHaQBU6<HDY@<HaP035:DH
aQS6008DY@8HaP8DY@DHaP065:DHaQBU6<HDYAS60QcW0QS600@DYAS66<HHaP8DY@036<HDYAS6008D
Y@046<HDYABU5:D36<H011BU6<HHaQS60QBU00<HaQBU6<H01QS600<DYAS66<H02aS600@DYAS66<HD
Y@DHaP8DY@HHaPDDY@036<HDYAS600XHaP8DY@036<HDYABU008HaP035:DHaQS600DHaP@DY@036<HD
YABU00DHaP045:DHaQBU6<H25:D00aS65:DHaP026<H00aBU6<HHaP096<H00aBU6<HDY@076<H25:D3
6<H00aBU6<HHaP025:D26<H01ABU6<HDYAS65:D00aS60aBU1AS600@DYAS66<HDY@@HaP035:DHaQS6
008HaPDDY@036<HDYAS6008DY@/HaPM[FP40005[FP0001m_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk6<HDY@046<H25:D96<H00aBU6<HHaP026<H0
0aBU6<HHaP035:D00aS65:DHaP026<H35:D36<H00aBU6<HHaP086<H25:D011S65:DHaQBU0QS60QBU
0QS600<DYAS65:D00QBU00<HaQBU5:D01AS60QBU00@HaQBU6<HDY@<HaP035:DHaQBU00DHaP8DY@03
6<HDYABU00<HaP@DY@8HaP045:DHaQS65:D66<H011BU6<HDYAS60aBU0QS600<DYAS66<H031S61ABU
00<HaQBU6<H021S600<DYAS66<H00QS60aBU1AS60QBU0QS600@DYAS65:DDY@<HaP035:DHaQS600XH
aP055:DHaQBU6<HDY@036<H011BU6<HHaQBU21S60aBU0aS600@DYAS66<HHaP8DY@8HaP035:DHaQS6
00THaP045:DHaQBU6<H25:D26<H00aBU6<HDY@026<H011BU6<HHaQBU21S600<DYAS65:D00aBU00<H
aQBU5:D03AS60QBU11S60aBU00<HaQBU5:D01AS600@DYAS66<HHaP8DY@8HaP045:DHaQS66<H55:D5
6<H00aBU6<HHaP026<H00aBU6<HHaP0:6<H00aBU6<HDY@056<H00aBU6<HDY@025:D26<H02ABUKg][
FV]jJeY_Nf]JJgX00001Kg/001I[FP<000A[FP035:DHaQS600LHaP037>LHaQS600PHaP8DY@lHaP03
5:DHaQBU00LHaPDDY@046<HDYAS65:D46<H011BU6<HHaQS60aBU00DHaQBU5:DDYAS6008DY@THaP8D
Y@<HaP8DY@HHaP8DY@@HaP035:DHaQS6008DY@046<HDYAS65:D36<H25:D26<H35:D011S65:DDYABU
11S600<DYAS66<H01AS600<DYAS66<H01AS60QBU0aS60QBU0QS61QBU0aS600<DYAS65:D011S600<D
YAS66<H01QS611BU2QS600@DYAS65:DHaP8DY@<HaP<DY@DHaP035:DHaQS600DHaP035:DHaQS600DH
aP065:DHaQBU5:DHaQBU2QS600@DYAS65:DDY@<HaP<DY@`HaP035:DHaQBU008HaP035:DHaQS6010H
aP035:DHaQBU00DHaP8DY@<HaP055:DHaQS65:DHaP065:D56<H45:D011S65:DDYABU0aS600<DYAS6
6<H00aS61ABU1QS60QBU00@HaQBU5:DDY@8HaP8DY@036<HDYABU00I[FP80005[FP0001e[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j008HaP035:DH
aQS600PHaP8DY@036<HDYABU00<HaP035:DHaQS600lHaP035:DHaQS600HHaP035:DHaQS6008HaP03
5:DHaQBU008HaP8DY@056<HDYAS66<HDY@026<H011BU6<HDYABU11S600HDYAS66<HDYAS65:D26<H0
1ABU6<HDYAS65:D01QS600<DYAS65:D00QS600@DYAS65:DDY@@HaP035:DHaQBU008HaP8DY@XHaP8D
Y@046<HDYABU5:D36<H25:D00aS65:DDY@026<H00aBU6<HDY@026<H25:D011S65:DDYABU0aS600<D
YAS66<H00QBU0QS600@DYAS66<HDY@@HaP035:DHaQS6008HaP<DY@036<HDYABU008DY@<HaP035:DH
aQS600DHaP055:DHaQBU6<HDY@026<H25:D86<H00aBU6<HHaP0<6<H35:D76<H01ABU6<HDYABU6<H0
0QBU00<HaQBU5:D00QS60aBU0QS600@DYAS66<HDY@@HaP8DY@8HaP8DY@HHaP035:DHaQS600@HaP03
5:DHaQS6008DY@<HaP055:DHaQS66<HDY@036<H011BU6<HHaQS611BU0aS61QBU00<HaQBU6<H00aS6
0QBU00<HaQBU6<H00aBU1AS60QBU0aS61ABU0QS60aBU0QS60QBU00@HaQBU5:DHaP@DY@096<I[NV]J
Kg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J1QS600<DYAS66<H03QS600<DYAS66<H00aBU
0aS60QBU2QS60QBU0aS611BU00DHaQBU5:DHaQBU00@HaP8DY@@HaP035:DHaQS6008DY@<HaP035:DH
aQBU00`HaP035:DHaQS6008DY@PHaP8DY@@HaP8DY@<HaP055:DHaQBU5:DHaP025:D26<H25:D01AS6
5:DHaQS65:D00QS600<DYAS66<H00QBU00DHaQBU5:DHaQBU00@HaP8DY@8HaP8DY@036<HDYAS6008H
aP075:DHaQS65:DHaQBU6<H00QBU1aS600<DYAS66<H01aS600DDYAS66<HHaQBU00DHaP035:DHaQS6
01XHaP8DY@8HaPHDY@036<HDYAS600@HaP045:DHaQBU6<H45:D01AS65:DDYAS65:D00QS60aBU0QS6
0QBU0QS60aBU00@HaQBU5:DDY@`HaP035:DHaQS600HHaP8DY@LHaP8DY@<HaP035:DHaQS600<HaP03
5:DHaQBU00<DY@056<HDYAS66<HDY@026<H00aBU6<HHaP056<H25:D00aS65:DDY@025:D46<H25:D4
6<H011BU6<HDYABU11S600<DYF]JJeX01F]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/01QS600<DYAS66<H00QS60QBU00<HaQBU
6<H01AS600<DYAS66<H02AS600<DYAS66<H01AS611BU2AS600<DYAS66<H00aS600<DYAS65:D00QBU
0aS600DDYAS65:DDYAS600HDY@PHaP@DY@046<HDYABU5:D26<H25:D26<H00aBU6<HHaP026<H00aBU
6<HHaP036<H25:D26<H011BU6<HDYABU0aS600<DYAS66<H00QBU0QS60aBU0QS600<DYAS66<H00QS6
00<DYAS65:D00QBU0QS600@DYAS66<HDY@@HaP<DY@@HaP045:DHaQS65:DC6<H25:D00aS65:DHaP03
6<H00aBU6<HDY@026<H25:D46<H55:D01AS65:DHaQS65:D00QS600<DYAS66<H01AS600<DYAS65:D0
1AS60QBU1QS600<DYAS66<H011BU00<HaQBU5:D01AS60QBU00<HaQBU6<H01ABU1AS600DDYAS65:DH
aQBU00hHaP035:DHaQBU00<HaP035:DHaQS600<DY@<HaP8DY@HHaP035:DHaQS6008DY@046<HDYABU
5:D46<H35:D86<H25:D26<H00aBU6<HHaP026<H01ABU6<HHaQS65:D00QS600@DYAS65:DDY@8HaP09
5:E_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0QBU2aS600<LiaS66<H03AS600DD
YAS66<HHaQBU008HaP035:DHaQBU00<HaP8DY@/HaP035:DHaQS600DDY@8HaP@DY@036<HDYABU00<D
Y@036<HDYAS600<HaP<DY@036<HDYAS6008DY@HHaP035:DHaQS600HHaP8DY@8HaP035:DHaQS600@H
aP035:DHaQS6008HaP@DY@8HaP8DY@<HaP055:DHaQS66<HDY@026<H01ABU6<HHaQS65:D00QS60aBU
1QS600<DYAS66<H021S600HDYAS65:DDYAS65:D26<H01aBU6<HHaQS65:DHaQBU008HaP<DY@036<HD
YAS600XHaP8DY@@HaP035:DHaQS600@HaP035:DHaQS600@HaP035:DHaQS600HHaP055:DHaQS66<HD
Y@036<H011BU6<HHaQS60aBU00<HaQBU5:D00aS600<DYAS65:D011BU00<HaQBU6<H011S600<DYAS6
6<H00QBU00<HaQBU5:D00QS600<DYAS66<H01QS60QBU1QS600<DYAS65:D00aBU00<HaQBU6<H00aBU
0aS600<DYAS66<H00QBU0QS600<DYAS66<H00aS600<DYAS66<H00QBU0aS60QBU0aS600<DYAS66<H0
11S600TDYAS66<HHaQBU6<HDYAS65:D00QS611BU0QS611BU00<HaV]JJeX01F]J0@000F]J00007F]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX00QBU
2aS600<LiaS66<H03AS600DDYAS66<HHaQBU008HaP035:DHaQBU00<HaP8DY@/HaP035:DHaQS600DD
Y@8HaP@DY@036<HDYABU00<DY@036<HDYAS600<HaP<DY@036<HDYAS6008DY@HHaP035:DHaQS600HH
aP8DY@8HaP035:DHaQS600@HaP035:DHaQS6008HaP@DY@8HaP8DY@<HaP055:DHaQS66<HDY@026<H0
1ABU6<HHaQS65:D00QS60aBU1QS600<DYAS66<H021S600HDYAS65:DDYAS65:D26<H01aBU6<HHaQS6
5:DHaQBU008HaP<DY@036<HDYAS600XHaP8DY@@HaP035:DHaQS600@HaP035:DHaQS600@HaP035:DH
aQS600HHaP055:DHaQS66<HDY@036<H011BU6<HHaQS60aBU00<HaQBU5:D00aS600<DYAS65:D011BU
00<HaQBU6<H011S600<DYAS66<H00QBU00<HaQBU5:D00QS600<DYAS66<H01QS60QBU1QS600<DYAS6
5:D00aBU00<HaQBU6<H00aBU0aS600<DYAS66<H00QBU0QS600<DYAS66<H00aS600<DYAS66<H00QBU
0aS60QBU0aS600<DYAS66<H011S600TDYAS66<HHaQBU6<HDYAS65:D00QS611BU0QS611BU00THaV]j
JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX25:D36<H00aBU6<HHaP066<H011BU6<HH
aQBU0QS600HDYAS66<HHaQBU6<H25:D36<H00aBU6<HDY@025:D00aS65:DHaP025:D66<H00aBU6<HH
aP0:6<H011BU6<HDYAS60aBU0QS600@DYAS65:DHaP8DY@036<HDYABU00<DY@<HaP045:DHaQBU6<H6
5:D76<H00aBU6<HDY@036<H00aBU6<HDY@046<H25:D011S65:DDYAS60QBU00<HaQBU6<H00QS600<D
YAS66<H01QS60QBU0QS600<DYAS66<H011S600@DYAS65:DHaP8DY@DHaP<DY@HHaP8DY@@HaP045:DH
aQBU5:D26<H00aBU6<HDY@026<H00aBU6<HDY@086<H00aBU6<HHaP066<H01ABU6<HHaQS65:D00QS6
00<DYAS65:D011BU1aS600<DYAS66<H021S600@DYAS66<HDY@8HaP035:DHaQS600<HaP@DY@066<HD
YABU6<HDYAS60QBU1AS600<DYAS66<H00QBU2aS600<DYAS66<H00aS600@DYAS65:DHaP<DY@DHaP8D
Y@8HaP8DY@036<HDYABU008DY@DHaP<DY@LHaP035:DHaQS6008HaP035:DHaQS600<HaPHDY@<HaP05
5:DHaQS66<HDY@086<H00aBUJeY[FP05JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`066<H00aBU6<HLi`046<H25:D>6<H01ABU
6<HHaQBU6<H00aBU1AS60aBU0aS600<DYAS66<H00aS600@DYAS66<HHaP8DY@056<HDYABU6<HDY@05
6<H35:D26<H25:D26<H25:D26<H35:D00aS65:DDY@035:D26<H00aBU6<HHaP035:D00aS65:DHaP05
5:D46<H25:D00aS65:DHaP026<H45:D00aS65:DDY@025:D36<H00acW5:DHaP046<H00aBU6<HHaP04
6<H35:D00aS65:DHaP025:D56<H011BU6<HHaQS60aBU11S600@DYAS66<HDY@DHaP8DY@8HaP8DY@<H
aP035:DHaQS6008HaP035:DHaQS6008DY@DHaP035:DHaQS600DHaP045:DHaQS65:D46<H00aBU6<HD
Y@066<H011BU6<HDYABU1AS600@DYAS66<HDY@PHaPDDY@8HaP8DY@036<HDYABU008HaP045:DHaQS6
5:D26<H25:D011S65:DDYABU0QS60aBU1aS60QBU0QS61ABU11S60QBU0aS60QBU1QS600<DYAS65:D0
0aBU00@HaQBU6<HHaP@DY@LHaP035:DHaQS600DDY@036<HLiaS6008HaP8DY@056<HDYABU5:DHaP04
5:D02QS65:E_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX86<H00aBU6<HHaP066<H0
0aBU6<HHaP026<H25:D26<H45:D36<H011BU6<HHaQBU11S611BU1AS60aBU0aS600@DYAS66<HHaP8D
Y@036<HDYAS600LHaP065:DHaQS66<HDYAS60aBU0QS600DDYAS65:DHaQBU00DHaP045:DHaQS65:D4
6<H00aBU6<HHaP046<H011BU6<HHaQBU0QS600<DYAS66<H01AS60QBU1aS600DDYAS66<HHaQBU00DH
aP065:DHaQS66<HLiaS60QBU1QS600<DYAS65:D011S600DDYAS66<HDYAS6008DY@8HaP8DY@056<HD
YAS66<HDY@026<H00aBU6<HHaP026<H00aBU6<HHaP046<H00aBU6<HHaP026<H00aBU6<HHaP0;6<H5
5:D26<H00aBU6<HHaP0<6<H011BU6<HHaQS60QBU0QS600<DYAS66<H01aS600DDYAS66<HHaQBU00@H
aP@DY@8HaP055:DHaQS66<HDY@0?6<H95:D26<H011BU6<HHaQS60QBU00<HaQBU5:D011S600<DYAS6
6<H011S600@DYAS65:DDY@DHaP045:DHaQS65:D56<H00aBU6<HDY@025:D:6<H00aBU6<HHaP035:D0
0aS65:DHaP07JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX006]JJgY[FVmkJeY[NP026<H00aBU6<HHaP0=6<H25:D>6<H25:D26<H25:D?6<H00aBU6<HH
aP026<H01ABU6<HDYAS65:D00QS600DDYAS66<HHaQBU00THaP@DY@036<HDYAS600<DY@8HaP035:DH
aQBU00@HaP035:DHaQS600@HaP8DY@8HaP<DY@8HaP045:DHaQBU6<H25:D96<H25:D36<H35:D?6<H2
5:D00aS65:DHaP026<H25:D36<H00aBU6<HHaP025:D00aS65:DDY@086<H25:D26<H00aBU6<HHaP0;
6<H01ABU6<HHaQBU6<H00aBU1QS60QBU1AS60QBU00<HaQBU6<H02QS61ABU1QS62ABU0QS60QBU1AS6
1ABU00HHaQBU6<HHaQBU6<H25:D26<H25:D46<H25:D00aS65:DDY@045:D96<H00aBU6<HHaP025:D2
6<H00aBU6<HHaP076<H011BU6<HDYABU0aS600<DYAS66<H00aS600@DYAS66<HHaP8DY@PHaP8DY@8H
aP8DY@<HaP0:5:DHaV]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FPLHaP8DY@8HaP8D
Y@PHaP045:DHaQS65:D76<H00acW6<HHaP036<H00aBU6<HHaP056<H011BU6<HLiaS60QBU00<HaQBU
6<H01aS600<DYAS65:D00QBU0QS600<DYAS66<H00aBU0QS600@DYAS66<HHaP8DY@046<HDYAS65:D2
6<H45:D00aS65:DDY@076<H011BU6<HHaQBU11S600<DYAS66<H03QS60QBU00<HaQBU6<H00QBU1QS6
00<DYAS66<H00QS60aBU2AS600<LiaS66<H00QcW2AS600<DYAS66<H00aS611BU0aS60QBU1QS600<D
YAS66<H011S600DDYAS66<HHaQBU00<HaP045:DHaQS66<H25:D66<H00aBU6<HHaP026<H011BU6<HH
aQS60QBU0QS600@DYAS65:DHaP@DY@046<HDYAS65:D26<H35:D56<H25:D26<H00aBU6<HHaP036<H5
5:D56<H00aBU6<HHaP026<H25:D26<H25:D011S65:DHaQS60QBU0aS600<DYAS65:D00aS60aBU0QS6
00@LiaBU5:DDY@<HaP@DY@<HaP035:DHaQS6008HaP<DY@036<HDYAS600@HaP035:DHaQS600LHaP07
5:DHaQS66<HDYAS65:D00QS60QBU0aS61f]J0@000F]J00005VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J0`0000A[FV]jJeY_Na4HaP075:DHaQS66<HDYAS67>L011S600<D
YAS66<H02QS600@DYAS66<HDY@@HaP035:DHaQS6008HaP045:DHaQS65:D46<H00aBU6<HHaP025:D0
0aS65:DHaP056<H25:D26<H25:D36<H011BU6<HHaQBU11S60aBU0QS60QBU00<HaQBU6<H01AS600@D
YAS65:DHaP<DY@8HaP045:DHaQBU6<H25:D26<H25:D46<H00aBU6<HDY@036<H35:D86<H45:D86<H2
7>L00bDY8@PHaP036<H35:D46<H35:D00aS65:DHaP046<H25:D46<H25:D66<H25:D46<H00aBU6<HH
aP046<H00aBU6<HHaP025:D96<H011BU6<HHaQS60QBU00<HaQBU6<H01AS600HDYAS65:DHaQBU6<H2
5:D46<H00aBU6<HHaP026<H00aBU6<HHaP056<H00aBU6<HDY@036<H25:D26<H00aBU6<HHaP026<H2
5:D26<H01aBU6<HHaQS65:DHaQBU008HaPLDY@<HaP@DY@@HaP<DY@8HaP8DY@046<HDYAS65:D:6<H2
5:D36<H25:D46<H011BU6<HHaQS60aBU11S600<DYAS66<H00QBU00LHaVmkJeY[NV]JKg][FP020001
Kg/001I[FP03001[FV]J00A[FP<HaP<DY@THaP035:DHaQBU008DY@HHaP8DY@<HaP035:DHaQBU00LH
aP035:DHaQS600HHaP8DY@056<HDYAS65:DHaP035:D011S65:DDYABU0aS600<DYAS65:D00QBU11S6
00<DYAS66<H021S61ABU00<HaQBU5:D01AS600<DYAS65:D021S600<DYAS66<H041S600<DYAS65:D0
21S600DDYAS66<HHaQBU008HaP035:DHaQS6008HaP055:DHaQBU6<HDY@036<H00b487>LHaP076<H0
11BU6<HHaQS60QBU2QS600<DYAS66<H00QBU0aS60QBU2AS60QBU21S600<DYAS65:D011S600<DYAS6
5:D00QS600<DYAS66<H00aS600<DYAS65:D011BU1aS600<DYAS65:D041S60aBU0QS600@DYAS65:DH
aP<DY@8HaPHDY@8HaPDDY@DHaP8DY@8HaP035:DHaQS600DHaP045:DHaQS66<H55:D26<H01QBU6<HH
aQS65:DHaP8DY@DHaP8DY@036<HDYABU00<DY@HHaP035:DHaQBU008HaP@DY@036<I[FV]J00E[FP40
005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]J
Kg][FV]j00<HaP<DY@THaP035:DHaQBU008DY@HHaP8DY@<HaP035:DHaQBU00LHaP035:DHaQS600HH
aP8DY@056<HDYAS65:DHaP035:D011S65:DDYABU0aS600<DYAS65:D00QBU11S600<DYAS66<H021S6
1ABU00<HaQBU5:D01AS600<DYAS65:D021S600<DYAS66<H041S600<DYAS65:D021S600DDYAS66<HH
aQBU008HaP035:DHaQS6008HaP055:DHaQBU6<HDY@036<H00b487>LHaP076<H011BU6<HHaQS60QBU
2QS600<DYAS66<H00QBU0aS60QBU2AS60QBU21S600<DYAS65:D011S600<DYAS65:D00QS600<DYAS6
6<H00aS600<DYAS65:D011BU1aS600<DYAS65:D041S60aBU0QS600@DYAS65:DHaP<DY@8HaPHDY@8H
aPDDY@DHaP8DY@8HaP035:DHaQS600DHaP045:DHaQS66<H55:D26<H01QBU6<HHaQS65:DHaP8DY@DH
aP8DY@036<HDYABU00<DY@HHaP035:DHaQBU008HaP@DY@096<I[NV]JKg][FV]jJeY_N`00005[NP00
5V]J00<006]JJeX016]J1aS600<DYAS66<H01aS611BU00@HaQBU6<HDY@<HaP8DY@THaP035:DHaQBU
00/HaP8DY@<HaP@DY@036<HDYAS6008HaP065:DHaQBU6<HDYAS60QBU00DHaQBU6<HHaQBU00@HaP8D
Y@036<HDYABU008HaP035:DHaQS600LHaP8DY@036<HDYABU008HaP035:DHaQBU00LHaP035:DHaQBU
008DY@@HaP035:DHaQS600lHaP035:DHaQS600LHaP035:DHaQS600DHaP035:DHaQS600@DY@036<HD
YAS600THaP8DY@<HaP035:DHaQS600<DY@036<HDYAS6010HaP035:DHaQS6008HaP045:DHaQS66<H2
5:D00aS65:DHaP056<H00aBU6<HHaP036<H45:D86<H00aBU6<HDY@036<H00aBU6<HHaP086<H00aBU
6<HHaP026<H01ABU6<HDYAS65:D00QS600<DYAS66<H00QS600<DYAS66<H00aS600<DYAS65:D01ABU
1AS600<DYAS65:D00QS60QBU0aS600<DYAS66<H00aBU0aS60QBU1aS600<DYAS66<H00aS600DDYAS6
6<HHaQBU00<HaP055:DHaQS66<HDY@026<H25:D011S65:DDYAS60QBU1f]J0@000F]J00007FmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QS600<D
YAS66<H00aS600<DYAS66<H01AS600<DYAS66<H01AS600<DYAS66<H03AS60QBU0QS600<DYAS65:D0
0QS600HDYAS66<HHaQBU6<H25:D46<H00aBU6<HHaP035:D01QS65:DHaQBU6<HDY@<HaP8DY@036<HD
YABU008DY@046<HDYAS65:D66<H25:D86<H00acW6<HHaP026<H011BU6<HHaQS60aBU0QS60QBU00<H
aQBU5:D00QS60QBU00@HaQBU5:DDY@HHaP@DY@036<HDYABU00<HaP<DY@lHaP8DY@036<HDYABU00dH
aP8DY@8HaP035:DHaQS600<HaP8DY@HHaP8DY@046<HDYABU5:D86<H35:D96<H00aBU6<HDY@0;6<H0
0aBU6<HHaP026<H25:D46<H02ABU6<HHaQS65:DHaQBU6<HDY@076<H35:D=6<H00aBU6<HHaP025:D0
0aS65:DHaP036<H011BU6<HHaQBU21S600HDYAS66<HDYAS65:D86<H00aBU6<HDY@025:D36<H00aBU
6<HDY@025:D;6<H01ABU6<HHaQS65:D02QS600Q_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[
FP04JeX00aS65:DHaP036<H00aBU6<HHaP0?6<H01QBU6<HDYABU6<HDY@@HaP8DY@XHaP045:DHaQS6
5:D26<H00aBU6<HDY@025:D011S65:DDYAS61aBU0QS60QBU00DHaQBU5:DDYAS600<DY@036<HDYABU
008HaP035:DHaQS600<DY@036<HDYAS6008HaP<DY@LHaP8Li`8HaP@DY@@HaP045:DHaQS66<H35:D3
6<H25:D46<H011BU6<HDYABU0QS600<DYAS66<H00QBU00<HaQBU6<H00aS60QcW1AS600@DYAS66<HD
Y@<HaP0:5:DHaQBU5:DHaQBU7>LHaQBU6<H25:D01AS65:DHaQS65:D00aS600<DYAS66<H01QS60QBU
2AS600DDYAS66<HDYAS6008DY@036<HDYABU008DY@LHaP045:DHaQS65:D46<H00aBU6<HHaP076<H0
0aBU6<HHaP046<H00aBU6<HHaP066<H00aBU6<HHaP076<H35:D26<H00aBU6<HHaP026<H01QBU6<HH
aQS65:DHaP8DY@<HaP035:DHaQS6008DY@LHaP055:DHaQBU6<HDY@056<H00aBU6<HDY@086<H01QBU
6<HHaQBU6<HDYA0HaP035:DHaQS6008HaP8DY@<HaP8DY@<HaPM[FP40005[FP0001m[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j6<HLi`046<H00aBU
6<HHaP026<H25:D46<H01ABU6<HHaQS67>L01aS600<DYAS65:D011S600HDYAS66<HHaQBU6<H25:D4
6<H00aBU6<HDY@045:D00aS65:DDY@025:D01AS65:DDYABU6<H00QBU00DHaQBU5:DDYAS600DDY@8H
aP8DY@<HaP035:DHaQS6008HaP<DY@<HaP035:DHaQS600PHaP037>LHaQBU00XHaP035:DHaQS600TH
aP035:DHaQS600@HaP045:DHaQBU6<H55:D@6<H00aBU6<HHaP056<H00aBU6<HHaP036<H00aBU6<HD
Y@046<H35:D56<H00aBU6<HHaP026<H00aBU6<HHaP036<H25:D011S65:DDYAcW0QBU3QS600@DYAS6
6<HDY@8HaP8DY@8HaP035:DHaQS600@HaP035:DHaQS6008DY@<HaP035:DHaQBU00@HaP035:DHaQS6
008DY@PHaP065:DHaQS66<HLiaS60QBU0aS60QBU00@HaQBU5:DDY@@HaP045:DHaQS65:D26<H011BU
6<HDYABU1AS60aBU00<HaQBU6<H00aS60aBU00<HaQBU6<H011S600@DYAS66<HDY@8HaP8DY@THaP03
5:DHaQS600@HaP035:DHaQBU008HaP8DY@<HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]J
JeX016]J2AS600<DYAS66<H00aS600<DYAS66<H021S60QBU0QS600<DYAS66<H00aS60QBU00@HaQBU
6<HHaP8DY@046<HDYAS66<H25:D46<H011BU6<HHaQS611BU0QS600DDYAS66<HHaQBU008HaP8DY@DH
aP035:DHaQBU00@HaP035:DHaQS600HHaP035:DHaQS600`HaP8DY@HHaP035:DHaQS600LHaP@DY@03
6<HDYAS600DHaP035:DHaQS600@HaP8DY@HHaP055:DHaQS66<HDY@046<H011BU6<HDYAS60QBU2AS6
00@DYAS65:DDY@DHaP035:DHaQS6008DY@PHaP8DY@036<HDYAS600<HaP035:DHaQS6010HaP055:DH
aQS66<HDY@056<H25:D56<H00aBU6<HHaP026<H35:D76<H35:D011S65:DDYAS60QBU11S61QBU2AS6
00@DYAS66<HHaP<DY@@HaP035:DHaQS600<HaP035:DHaQBU00<HaP035:DHaQS6008DY@HHaPDDY@8H
aP035:DHaQBU008DY@/HaP055:DHaQS66<HDY@036<H00aBU6<HHaP026<H25:D7JeX10001JeX0000M
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`06
6<H35:D?6<H45:D26<H011BU6<HHaQS611BU21S600<DYAS66<H00aBU00HHaQBU6<HHaQBU6<H35:D3
6<H01ABU6<HHaQS65:D02QS600<DYAS66<H02AS600<DYAS65:D01aBU00<HaQBU6<H00aS60QBU1AS6
0QBU0aS600<DYAS66<H01AS600<DYAS66<H01QS60QBU0QS600<DYAS65:D031S600@DYAS66<HDY@<H
aP035:DHaQS600`HaP045:DHaQS66<H35:D;6<H00aBU6<HHaP026<H011BU6<HHaQBU2QS60QBU21S6
00<DYAS66<H01aS600<DYAS66<H00aBU1AS60QBU00<HaQBU6<H02QS60QBU00<HaQBU6<H00QS60aBU
0QS60QBU00<HaQBU6<H00QS600DDYAS65:DHaQBU008HaP065:DHaQS65:DHaQBU11S60QBU11S600<D
YAS65:D01QS600<DYAS66<H01AS600@LiaS65:DHaP8DY@046<HDYABU5:D56<H00aBU6<HDY@086<H3
5:D36<H00aBU6<HHaP056<H026mkJeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FPLHaP8D
Y@8HaP8DY@8HaP035:DHaQS600@HaP045:DHaQS66<H35:D46<H00aBU6<HHaP025:D46<H65:D46<H0
1QBU6<HHaQS65:DLi`<HaP035:DHaQS600@HaP8DY@036<HDYABU00<DY@LHaP035:DHaQBU00<HaP8D
Y@046<HDYABU5:D46<H00aBU6<HHaP0F6<H25:D00aS65:DHaP0:6<H25:D26<H25:D86<H00aBU6<HD
Y@026<H00aBU6<HHaP025:D36<H00aBU6<HHaP0;6<H01ABU6<HHaQS65:D01AS60QBU00@HaQBU7>LD
Y@THaP<DY@8HaP8DY@8HaP8DY@036<HDYAS600@HaP035:DLiaS6008DY@036<HDYABU00DHaP045:DH
aQS66<H25:D26<H00aBU6<HHaP0;6<H01QBU6<HDYABU6<HDY@@HaP<DY@8HaP035:DHaQS6008DY@XH
aP8DY@046<HDYABU5:D36<H00aBU6<HHaP056<H25:D86<H25:D46<H011BU6<HDYABU0QS60QBU1QS6
0QBU2aS600@DYAS66<HDY@8HaP8DY@<HaPM[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00LHaP8DY@8HaP8DY@8HaP035:DHaQS6
00@HaP045:DHaQS66<H35:D46<H00aBU6<HHaP025:D46<H65:D46<H01QBU6<HHaQS65:DLi`<HaP03
5:DHaQS600@HaP8DY@036<HDYABU00<DY@LHaP035:DHaQBU00<HaP8DY@046<HDYABU5:D46<H00aBU
6<HHaP0F6<H25:D00aS65:DHaP0:6<H25:D26<H25:D86<H00aBU6<HDY@026<H00aBU6<HHaP025:D3
6<H00aBU6<HHaP0;6<H01ABU6<HHaQS65:D01AS60QBU00@HaQBU7>LDY@THaP<DY@8HaP8DY@8HaP8D
Y@036<HDYAS600@HaP035:DLiaS6008DY@036<HDYABU00DHaP045:DHaQS66<H25:D26<H00aBU6<HH
aP0;6<H01QBU6<HDYABU6<HDY@@HaP<DY@8HaP035:DHaQS6008DY@XHaP8DY@046<HDYABU5:D36<H0
0aBU6<HHaP056<H25:D86<H25:D46<H011BU6<HDYABU0QS60QBU1QS60QBU2aS600@DYAS66<HDY@8H
aP8DY@<HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J1AS600HLiaS66<HDYAS6
5:D36<H25:D26<H25:D00aS65:DHaP026<H011BU6<HHaQBU11S600@DYAS66<HDY@@HaP045:DHaQS6
6<H25:D36<H25:D26<H35:D36<H01ABU6<HDYAS65:D01AS61ABU0QS600@DYAS66<HHaP8DY@DHaP06
5:DHaQS66<HDYAS60QBU3AS600<DYAS66<H00QBU21S600<DYAS65:D01aS60QBU0QS60QBU21S60aBU
0QS60QBU0QS600@DYAS66<HHaP<DY@046<HDYAS65:D76<H00aBU6<HHaP086<H01QBU6<HHaQBU6<HD
Y@lHaP@DY@LHaP045:DHaQBU5:D36<H00aBU6<HDY@025:D01aS65:DDYAS65:DHaQBU00DHaP035:DH
aQS600LHaP035:DHaQBU008HaP035:DHaQS600HHaP035:DHaQS600@HaP045:DHaQS66<H45:D36<H0
0acW6<HDY@025:D26<H25:D011S65:DDYAS60aBU00@HaQBU5:DDY@LHaP035:DHaQBU00dHaP045:DH
aQBU5:D26<H25:D26<H00aBU6<HHaP046<H25:D66<H00aBU6<HHaP036<H00aBU6<HHaP07JeX10001
JeX0000FKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX3000016]JJgY[
FVmk21S600<DYAS66<H00aS600<DYAS66<H021S600<DYAS66<H04QS600<DYAS65:D00aS600<DYAS6
5:D01ABU0QS600<DYAS66<H00aS60QBU0QS600<DYAS66<H00QBU0aS600DDYAS66<HHaQBU008HaP05
5:DHaQS66<HDY@026<H00aBU6<HHaP025:D00aS65:DHaP056<H25:D00aS65:DHaP036<H25:D76<H0
0aBU6<HHaP036<H011BU6<HHaQcW1QS600<DYAS66<H01AS600<DYAS66<H03aS600<DYAS66<H00QS6
00@DYAS66<HDY@DHaP8DY@@HaP035:DHaQS600<DY@XHaP035:DHaQS600<HaP8DY@THaP035:DHaQS6
00@HaP035:DHaQS6008DY@8HaP045:DHaQS66<H25:D00aS65:DHaP086<H00aBU6<HDY@036<H00aBU
6<HHaP086<H00aBU6<HHaP025:D00aS65:DHaP035:D00aS65:DHaP025:D36<H25:D011S65:DHaQBU
0aS600<DYAS66<H011S600DDYAS66<HHaQBU00@HaP045:DHaQBU5:D86<H25:D00aS65:DHaP035:D4
6<H00aBU6<HHaP026<H00aBU6<HHaP076<H011BU6<HDYABU0aS600PDYAS6Kg][FV]jJeY_Nf]J0P00
0Fmk000FJeX00`00JeY[FP04JeX76<H25:D;6<H00aBU6<HHaP066<H01ABU6<HHaQS65:D00QS60QBU
0QS60QBU1AS60aBU0aS600DDYAS65:DDYAS6008DY@056<HDYABU6<HDY@036<H00aBU6<HHaP025:D2
6<H45:D011S65:DDYABU0aS60QBU1QS600<DYAS65:D00QS600<DYAS65:D00QBU0aS600<DYAS66<H0
0QS60QBU00<HaQBU5:D01AS600<DYAS66<H00aS60QBU0QS60aBU00<HaQBU5:D00QS600<DYAS66<H0
1AS60aBU1QS600DDYAS66<HHaQBU00<HaP035:DHaQS600LHaP8DY@046<HDYAS65:D;6<H25:D<6<H4
5:D26<H01ABU6<HHaQS65:D02QS600<DYAS65:D00QBU11S600<DYAS66<H011S600DDYAS66<HHaQBU
00<HaP035:DHaQBU008DY@036<HDYAS600DHaP035:DHaQBU00DHaP@DY@8HaP8DY@8HaP035:DHaQS6
008HaP045:DHaQBU5:D46<H00aBU6<HHaP046<H00aBU6<HDY@026<H25:D36<H00aBU6<HDY@086<H2
5:D66<H25:D36<H00aBU6<HHaP026<H00aBU6<HHaP086<H25:D56<H7JeX10001JeX0000OJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQS65:D03AS6
00<DYAS65:D011S600@DYAS66<HDY@`HaP035:DHaQS600@HaP8DY@036<HDYABU00@HaP045:DHaQS6
6<H35:D36<H35:D011S65:DHaQBU0QS60QBU2AS600DDYAS65:DDYAS6008DY@DHaP035:DHaQS600<D
Y@@HaP035:DHaQS6008HaP055:DHaQS66<HDY@036<H35:D00aS65:DHaP066<H00aBU6<HHaP066<H0
0aBU6<HHaP086<H25:D@6<H25:D00aS65:DDY@056<H00aBU6<HDY@025:D00aS65:DHaP0J6<H00aBU
6<HHaP026<H00aBU6<HHaP056<H00aBU6<HDY@025:DE6<H45:D36<H00aBU6<HHaP056<H00aBU6<HH
aP025:D00aS65:DDY@045:D00aS65:DDY@026<H25:D26<H00aBU6<HHaP036<H01ABU6<HHaQS65:D0
0aS600DDYAS66<HHaQBU00<HaP055:DHaQBU6<HDY@036<H011cW6<HHaQcW0aBU00<HaQBU6<H01AS6
00@DYAS66<HDY@<HaP035:DHaQS6008HaP@DY@@HaP@DY@0:6<HDYF]jJeY_Nf]JJgY[FVmk0001JgX0
01I[FP03001[FV]J00A[FP036<HDYAS600<HaP055:DHaQS66<HDY@046<H00aBU6<HHaP076<H011BU
6<HHaQBU21S60aBU31S61ABU0aS600DDYAS66<HDYAS6008DY@036<HDYAS600HHaP035:DHaQS6008H
aP<DY@046<HDYABU5:D46<H25:D76<H00aBU6<HHaP026<H00aBU6<HHaP025:D36<H00aBU6<HHaP0=
6<H00aBU6<HHaP036<H55:D26<H35:DC6<H00aBU6<HHaP0;6<H00aBU6<HHaP046<H27>L00b486<HH
aP026<H00aBU6<HHaP086<H00aBU6<HDY@025:D26<H00aBU6<HHaP046<H35:D26<H01QBU6<HDYABU
6<HDY@/HaP045:DHaQcW7>L86<H35:D26<H00aBU6<HDY@036<H01ABU6<HHaQS65:D00QS60QBU0aS6
00<DYAS66<H00aBU1AS600DDYAS65:DHaQBU00PHaP8DY@8HaP@DY@@HaP045:DHaQS66<H25:D:6<H2
5:D76<H00aBU6<HHaP046<H00aBU6<HHaP055:D00aS65:DDY@036<H00aBU6<I[FP06JeX10001JeX0
000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_
N`096<H011BU6<HDYABU1aS60QBU00<HaQBU6<H01AS600<DYAS66<H011S600<DYAS66<H01QS600<L
iaBU6<H011S600@DYAS66<HDY@<HaP035:DHaQS600@HaP035:DHaQS600<HaP8DY@056<HDYABU5:DH
aP035:D96<H011BU6<HHaQBU1AS600<DYAS66<H011S600<DYAS65:D011S600<DYAS66<H01AS600<D
YAS66<H01AS60QBU0QS61ABU5AS600<DYAS66<H00QS60aBU11S600@DYAS65:DDY@<HaP065:DHaQBU
5:DHaQBU0QS600<LiaS66<H01QS600HDYAS66<HDYAS65:D26<H00aBU6<HHaP026<H011BU6<HHaQBU
1QS600<DYAS66<H00QS600<DYAS65:D021S600<DYAS66<H02AS600@DYAS66<HDY@<HaP8DY@HHaP04
5:DHaQS65:D66<H25:D011S65:DDYAS60QBU00<HaQBU6<H01AS600<DYAS65:D031S600<DYAS66<H0
1aS600@DYAS66<HDY@LHaP045:DHaQBU6<H45:D00aS65:DHaP0:6<H25:D46<H00aBU6<HHaP036<H0
2ABUKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP036<HLiaS6008DY@XHaP035:DH
aQS600PHaP8DY@HHaP8DY@`HaP035:DHaQS600@HaP035:DHaQS600hHaP055:DHaQS65:DHaP035:D3
6<H01ABU6<HHaQS65:D01AS600<DYAS66<H00aBU2AS600@DYAS66<HDY@@HaP8DY@XHaP8DY@@HaP03
5:DHaQS600XHaP035:DHaQS600DHaP035:DHaQS600`HaP035:DHaQS600LHaP035:DHaQS6008HaP03
5:DHaQS601LHaP035:DHaQS6018HaP035:DHaQS6008HaP8DY@036<HDYAS600LHaP045:DHaQS66<H2
5:D00aS65:DHaP046<H01ABU6<HHaQS65:D01aS600DDYAS65:DDYAS600@DY@LHaP045:DHaQBU5:D2
6<H00aBU6<HDY@026<H00aBU6<HDY@066<H01ABU6<HHaQS65:D00aS600DDYAS66<HHaQBU00<HaP8D
Y@036<HDYAS6008DY@036<HDYAS600DHaP<DY@036<HDYAS600@HaP035:DHaQS6008DY@8HaP035:E[
FV]J00E[FP40005[FP00021[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FP00JeY[NV]JKg][FV]j6<HLiaS60QBU2QS600<DYAS66<H021S60QBU1QS60QBU31S600<DYAS66<H0
11S600<DYAS66<H03QS600DDYAS66<HDYAS600<DY@<HaP055:DHaQS66<HDY@056<H00aBU6<HHaP03
5:D96<H011BU6<HHaQBU11S60QBU2QS60QBU11S600<DYAS66<H02QS600<DYAS66<H01AS600<DYAS6
6<H031S600<DYAS66<H01aS600<DYAS66<H00QS600<DYAS66<H05aS600<DYAS66<H04QS600<DYAS6
6<H00QS60QBU00<HaQBU6<H01aS600@DYAS66<HHaP8DY@036<HDYAS600@HaP055:DHaQS66<HDY@07
6<H01ABU6<HDYABU6<H011BU1aS600@DYAS65:DDY@8HaP035:DHaQBU008HaP035:DHaQBU00HHaP05
5:DHaQS66<HDY@036<H01ABU6<HHaQS65:D00aS60QBU00<HaQBU6<H00QBU00<HaQBU6<H01AS60aBU
00<HaQBU6<H011S600<DYAS66<H00QBU0QS600TDYF]jJeY_Nf]JJgY[FVmk00000F]j000FJeX00`00
JeY[FP04JeX>6<H25:D66<H00acW6<HHaP0=6<H00aBU6<HDY@0;6<H00aBU6<HHaP035:D66<H00aBU
6<HHaP026<H011BU6<HHaQBU0aS60QBU0QS600<DYAS66<H00QS600<DYAS66<H011S600<DYAS65:D0
1AS600<DYAS66<H02AS60QBU2AS60QBU00<HaQBU6<H02QS600<DYAS66<H00QS60QBU11S600<DYAS6
6<H011S60QBU31S600<DYAS66<H01aS600<DYAS66<H00aS60aBU51S600@DYAS66<HDY@LHaP035:DH
aQS600PHaP8DY@`HaP8DY@@HaPDDY@PHaP045:DHaQS65:D46<H35:D36<H00aBU6<HDY@055:D66<H2
5:D56<H27>L01ABU6<HHaQS65:D011S600<DYAS66<H01QS61QBU0aS600<DYAS66<H00aBU0aS61ABU
0QS600<DYAS66<H00QS600<DYAS66<H011S600<DYAS66<H01f]J0@000F]J00007FmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/03aS600<DYAS66<H0
4QS600DDYAS66<HHaQBU00<HaP035:DHaQS6008HaP055:DHaQS66<HDY@076<H00aBU6<HHaP046<H0
0aBU6<HHaP025:D46<H011BU6<HDYAS60QBU1QS60QBU1QS60QBU0QS60QBU00<HaQBU5:D04AS600<D
YAS66<H02AS600<DYAS66<H00aS600HDYAS66<HHaQBU6<H35:D46<H00aBU6<HHaP0B6<H25:D00aS6
5:DHaP046<H011BU6<HDYABU5QS600<DYAS66<H011S600<DYAS66<H01QS600<DYAS66<H011S600<D
YAS66<H01AS600<DYAS66<H00aS600<DYAS66<H00aS60aBU0QS600<DYAS66<H00aS600<DYAS66<H0
1QS600<DYAS66<H00QS60QBU0QS600DDYAS66<HHaQBU00@HaP<DY@036<HDYAS6008HaP<DY@<HaP03
5:DHaQS600DHaP035:DHaQS600@HaP035:DHaQBU008HaP8DY@<HaP@DY@046<HDYABU6<H25:D46<H0
1QBU6<HDYABU6<HDY@LHaP0:5:DHaVmkJeY[NV]JKg][FV]j0001Kg/000Q[FP8000A[FP8000I[FP03
001[FV]J00A[FP035:DHaQS600DHaP035:DHaQS600HHaP035:DHaQS600XHaP035:DHaQBU00<HaP04
5:DHaQS65:D96<H00aBU6<HHaP0@6<H25:D56<H011BU6<HHaQS60QBU0aS600<DYAS66<H03aS600<D
YAS66<H03QS60aBU0aS600<DYAS65:D00QBU0QS600<DYAS65:D00aS600<DYAS66<H00QBU4QS60QBU
4QS600<DYAS66<H00aS600<DYAS66<H00QBU0QS600<DYAS66<H03QS60QBU0aS600<DYAS66<H00QS6
0QBU00<HaQcW6<H00aS60QBU00<HaQBU5:D00QS600<DYAS66<H011S600DDYAS66<HHaQBU008HaP03
5:DHaQS6008HaP<DY@DHaP035:DHaQS600DHaP8DY@DHaP<DY@DHaP035:DHaQS600LDY@THaP035:DH
aQS600<HaP035:DHaQS600<HaP035:DHaQBU00HHaP035:DHaQBU008HaP8DY@PHaP035:DHaQS6008H
aPDDY@036<HDYAS6008HaP035:DHaQBU00M[FP40005[FP0001e[NV]JKg][FV]jJeY_N`00JgY[FP00
JeY[NP00Kg][FP00JeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00DHaP8DY@THaP8DY@XHaP035:DHaQS6
008DY@076<HDYABU6<HDYAS65:D01AS600<DYAS66<H011S60QBU00<HaQBU6<H011S60QBU1AS600DD
YAS66<HHaQBU008HaP045:DHaQS65:D56<H011BU6<HHaQBU21S600<DYAS66<H00aS600<DYAS66<H0
2QS600@DYAS66<HDY@LHaP035:DHaQS600@HaP035:DHaQBU00@HaP035:DHaQS6008HaP035:DHaQBU
00<HaP8DYADHaP035:DHaQS600@HaP045:DHaQS65:D26<H00aBU6<HHaP056<H00aBU6<HHaP0;6<H0
0aBU6<HHaP026<H25:D76<H00aBU6<HHaP036<H25:D=6<H00aBU6<HHaP066<H25:D01AS65:DHaQS6
5:D01AS600<DYAS65:D00QBU00HHaQBU6<HDYAS65:D36<H00aBU6<HDY@046<H01ABU6<HDYABU6<H0
1ABU1AS60QBU00<HaQBU5:D01AS600@DYAS66<HHaP8DY@XHaP035:DHaQS600<DY@DHaP<DY@@HaP03
5:DHaQS600<DY@PHaP08JgY[FVmkJeY[NV]JKg/0005[NP002V]J00@006]JJeX0009[FP03001[FV]J
00=[FP@000=[FPlHaP037>LHaQS600<DY@8HaP8DY@DHaP8DY@DHaP035:DHaQS600DDY@036<HDYAS6
008HaP8DY@046<HDYAS65:D56<H00aBU6<HHaP056<H011BU6<HHaQS60QBU00<HaQBU6<H01AS600DD
YAS65:DHaQBU010HaP8Li`035:DHaQS600/HaP8DY@DHaP035:DHaQS600HHaP<DY@8HaP055:DHaQBU
5:DHaP025:DC6<H25:D00aS65:DHaP086<H00aBU6<HHaP026<H01ABU6<HHaQS65:D03aS60aBU0aS6
00<DYAS66<H02aS600<DYAS66<H00aBU21S600DLiaS66<HHaQBU00DHaP035:DHaQS600<HaP8DY@HH
aP055:DHaQS66<HDY@026<H35:D00aS65:DDY@0?6<H021BU6<HHaQBU6<HDYAS65:D46<H25:D26<H2
5:D01AS65:DHaQBU6<H011BU00<HaQBU6<H021S600@DYAS65:DDY@DHaP<DY@<HaP035:DHaQS6008H
aP035:DHaQS6008HaP<DY@036<HDYF]J00A[FP<0005[FP0000Q_Nf]JJgY[FVmkJeY[NV]J0P0001=[
NV]JKg/006]jJeX006]JJgY[FVmkJeX006]JKg][FV]jJeY_N`056<H01QBU6<HDYAS67>LDY@8HaP03
7>LHaQcW00PHaP035:DHaQBU00<DY@8HaP035:DHaQS600/HaP035:DHaQS600/HaP035:DHaQBU00TH
aP035:DHaQS600<HaP035:DHaQBU00DHaP035:DHaQS6008HaP035:DHaQS600<HaP8DY@036<HDYAS6
008HaP8DYAdHaP035:DHaQS600`HaP035:DHaQS600@HaP035:DHaQS600/HaP8DY@THaP8DY@036<HD
YAS600<HaP<DY@THaP035:DHaQS600DHaP045:DHaQS66<H35:D86<H00acW6<HLi`036<H25:D36<H0
0acW6<HHaP046<H00aBU6<HHaP056<H25:D26<H011BU6<HHaQBU3AS600<DYAS66<H03AS600<DYAS6
6<H01ABU00<HaQBU5:D021S60QBU0aS600<DYAS65:D02aS600@DYAS65:DHaP@DY@<HaP035:DHaQS6
008DY@<HaP055:DHaQS66<HDY@096<H026mkJeY[NV]JKg][FV]j0001Kg/000Q[FP03001[FV]J009[
FP04001[FV]J0005JeX00`00JeY[FP04JeX76<H00aBU6<HHaP036<H37>L56<H01ABU6<HHaQBU6<H0
11BU00<HaQBU6<H011BU00DHaQBU5:DDYAS600DDY@8HaP045:DHaQS67>L46<H01ABU6<HHaQS65:D0
4AS60QBU1AS600<DYAS66<H00QBU0QS600<DYAS66<H00aBU3AS60QBU0QS60QBU1QS600@DYAS66<HD
Y@@HaP035:DHaQBU00<HaP035:DHaQS600PHaP055:DHaQS66<HDY@096<H00aBU6<HHaP046<H011BU
6<HHaQBU21S60QBU0QS600@LiaS66<HDY@<HaP8DY@dHaP8DY@HHaP8DY@036<HDYAS600hHaP037>LH
aQS600HHaP055:DHaQS66<HDY@086<H00aBU6<HHaP025:D46<H00aBU6<HHaP026<H01ABU6<HHaQS6
5:D00QS600<DYAS66<H01QS600<DYAS66<H00QS60QBU5aS60QBU0aS60aBU0aS600<DYAS66<H00QBU
1aS60aBU1AS600<DYAS66<H01QS600<DYAS6JeX01V]J0@000F]J000026]jJeY_Nf]JJgY[FVmkJeX3
00000f]JJgY[FP0200003F]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00LHaP035:DHaQS600<HaP<L
i`DHaP055:DHaQS65:DHaP045:D00aS65:DHaP045:D01AS65:DDYABU6<H01ABU0QS600@DYAS66<HL
i`@HaP055:DHaQS66<HDY@0A6<H25:D56<H00aBU6<HHaP025:D26<H00aBU6<HHaP035:D=6<H25:D2
6<H25:D66<H011BU6<HHaQBU11S600<DYAS65:D00aS600<DYAS66<H021S600DDYAS66<HHaQBU00TH
aP035:DHaQS600@HaP045:DHaQS65:D86<H25:D26<H011cW6<HHaQBU0aS60QBU3AS60QBU1QS60QBU
00<HaQBU6<H03QS600<LiaS66<H01QS600DDYAS66<HHaQBU00PHaP035:DHaQS6008DY@@HaP035:DH
aQS6008HaP055:DHaQS66<HDY@026<H00aBU6<HHaP066<H00aBU6<HHaP026<H25:DG6<H25:D36<H3
5:D36<H00aBU6<HHaP025:D76<H35:D56<H00aBU6<HHaP066<H02QBU6<I[NV]JKg][FV]jJeY_N`00
0F]j000FJeX00`00JeY[FP04JeX=6<H00acW8@PLi`056<H00aBU6<HHaP0:6<H25:D46<H25:D:6<H0
11BU6<HHaQS611BU0QS600<DYAS66<H01aS600@DYAS66<HHaP8DY@<HaP8DY@056<HDYAS66<HDY@07
6<H00aBU6<HHaP026<H011BU6<HHaQS60aBU0QS600<DYAS66<H03aS600<DYAS66<H04AS600@DYAS6
6<HHaP8DYA4HaP035:DHaQS6008HaP8DY@lHaP055:DHaQBU6<HDY@056<H00aBU6<HHaP0?6<H00aBU
6<HHaP026<H00aBU6<HHaP036<H011BU6<HHaQS60QBU1QS60QBU11S600<DYAS66<H011S600<DYAS6
5:D01AS600<LiaS66<H011S600<DYAS65:D01AS60QBU2AS600<DYAS66<H01QS60QBU2aS600<DYAS6
5:D01AS600<DYAS65:D00aS60QBU0aS60QBU00<HaQBU6<H031S600<DYAS65:D021S61f]J0@000F]J
00007fmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]J
Kg/HaQBU00lHaP035:DHaQS6008HaP045:DHaQBU6<H25:DC6<H25:D01AS65:DDYAS65:D00aS600@D
YAS66<HHaP@DY@PHaP8DY@056<HDYAS65:DHaP035:D=6<H35:D26<H00aBU6<HHaP086<H25:D56<H0
0acW5:DDY@066<H25:DA6<H00aBU6<HHaP0K6<H00aBU7>LDY@0@6<H00aBU6<HHaP046<H00aBU6<HH
aP086<H35:D76<H011BU6<HHaQBU41S600<DYAS66<H02AS600<DYAS66<H00aS600<DYAS66<H01AS6
00<DYAS65:D03QS600<DYAS66<H01QS600<DYAS66<H00aBU1aS600@DYAS66<HHaP@DY@8HaP045:DH
aQS66<H25:D=6<H25:D66<H25:D66<H25:D02aS65:DDYFmkJeY[NV]JKg][FV]j00000Fmk000FJeX0
0`00JeY[FP04JeX00aS65:DDY@025:D46<H00aBU6<HHaP0M6<H01QBU6<HHaQBU6<HDY@<HaP8DY@05
7>LHaQS66<HDY@036<H00aBU6<HHaP0<6<H00aBU6<HHaP026<H011BU6<HHaQBU0QS600<DYAS66<H0
11S600@DYAS65:DHaP8DY@046<HDYABU5:D46<H00aBU6<HHaP076<H00aBU6<HHaP046<H00acW6<HL
i`0=6<H00aBU6<HHaP036<H25:D26<H00aBU6<HHaP026<H011BU6<HHaQS60QBU2AS600<DYAS66<H0
21S600@DYAS66<HDYA0HaP8DY@8HaP035:DHaQS600HHaP@DY@8HaP045:DHaQS67>LT6<H25:D26<H0
0aBU6<HDY@086<H35:DB6<H00aBU6<HDY@026<H00aBU6<HHaP026<H00aBU6<HHaP046<H011BU6<HD
YABU0QS60QBU0QS60QBU00<HaQBU6<H01QS600<DYAS65:D00aS60QBU3QS600<DYAS66<H01f]J0@00
0F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_
Nf]JJgX00QS600<DYAS66<H05aS600<DYAS65:D00aS60QBU1AS60aBU11S600@DYAS65:DDY@8HaP03
5:DHaQBU00<HaP8DY@<HaP8DY@PHaP8DY@HHaP035:DHaQS600<HaP8DYA0HaP045:DHaQS65:D46<H0
0aBU6<HDY@025:D<6<H00aBU6<HHaP0;6<H00aBU6<HDY@0@6<H00aBU6<HHaP026<H25:D00aS65:DH
aP066<H01ABU6<HHaQS65:D01aS60QBU0aS60QBU41S600<DYAS66<H031S600<DYAS66<H02AS60QBU
1QS600<DYAS66<H02QS600<DYAS66<H01QS600<DYAS66<H00QS600@DYAS66<HHaP@DY@@HaP035:DH
aQS600<HaP035:DHaQS6008HaP035:DHaQS600DDY@8HaP8DY@8HaP8DY@PHaP035:DHaQBU00DHaP03
5:DHaQS600XHaP<DY@8HaP8DY@DHaP095:E[NV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX0
16]J2AS600DDYAS66<HHaQBU00PHaP035:DHaQS600<HaP035:DHaQS600/HaP8DY@036<HDYAS6008H
aP@DY@056<HDYAS65:DHaP045:D36<H25:D;6<H00aBU6<HHaP076<H011BU6<HHaQBU0QS60QBU0aS6
00<DYAS66<H021S600<DYAS66<H02AS600<DYAS66<H01aS600<DYAS66<H02aS60QBU0QS600@DYAS6
6<HDY@THaP035:DHaQBU00`HaP035:DHaQS601hHaP035:DHaQS602<HaP035:DHaQS600HHaP8Li`<H
aP035:DHaQS600DHaP035:DHaQBU01HHaP8DY@LHaP035:DHaQS600LHaP8DY@046<HDYAS65:D46<H0
0aBU6<HDY@056<H25:D26<H011BU6<HHaQBU3aS600<DYAS66<H01f]J0@000F]J00007VmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/Li`DHaP035:DH
aQS600@HaP035:DHaQS6008HaP8DY@<HaP035:DHaQBU00DHaP8DY@046<HDYABU5:D86<H00aBU6<HH
aP046<H45:D46<H25:D26<H00aBU6<HHaP025:D26<H011BU6<HHaQS60QBU2QS600<DYAS66<H011S6
00<DYAS66<H01QS600DDYAS66<HHaQBU00HHaP055:DHaQS66<HDY@0;6<H00aBU6<HHaP066<H00aBU
6<HHaP036<H00aBU6<HHaP066<H01AcW6<HHaQS67>L02QS600<DYAS65:D00QBU00<HaQBU5:D00aS6
00<DYAS66<H01aS60QBU4aS600<DYAS65:D011S600<LiaS66<H05QS600<DYAS66<H031S600<LiaS6
6<H00aS600<DYAS66<H01aS600@DYAS65:DDY@8HaP035:DHaQS600THaP<DY@036<HDYABU00DHaP<D
Y@PHaP035:DHaQS6008HaP8DY@@HaP045:DHaQS65:D76<H00aBU6<HHaP0>6<H00aBU6<HHaP026<H0
2ABUKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP037>LHaQS600<HaP035:DHaQS6
00@HaP035:DHaQS6008HaP8DY@<HaP035:DHaQBU00DHaP8DY@046<HDYABU5:D86<H00aBU6<HHaP04
6<H45:D46<H25:D26<H00aBU6<HHaP025:D26<H011BU6<HHaQS60QBU2QS600<DYAS66<H011S600<D
YAS66<H01QS600DDYAS66<HHaQBU00HHaP055:DHaQS66<HDY@0;6<H00aBU6<HHaP066<H00aBU6<HH
aP036<H00aBU6<HHaP066<H01AcW6<HHaQS67>L02QS600<DYAS65:D00QBU00<HaQBU5:D00aS600<D
YAS66<H01aS60QBU4aS600<DYAS65:D011S600<LiaS66<H05QS600<DYAS66<H031S600<LiaS66<H0
0aS600<DYAS66<H01aS600@DYAS65:DDY@8HaP035:DHaQS600THaP<DY@036<HDYABU00DHaP<DY@PH
aP035:DHaQS6008HaP8DY@@HaP045:DHaQS65:D76<H00aBU6<HHaP0>6<H00aBU6<HHaP026<H00aBU
JeY[FP05JeX10001JeX0000OJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeX006]JJgY[FVmkJeY[NQS65:D01aS600<DYAS66<H031S600<DYAS66<H01AS600@DYAS65:DDY@<H
aP8DY@`HaP8DY@8HaP<DY@<HaP045:DHaQBU5:D36<H00aBU6<HDY@026<H25:D>6<H25:D36<H00aBU
6<HHaP026<H35:DN6<H011BU6<HDYABU2QS600<DYAS66<H03aS600DDYAS66<HHaQcW00HHaP035:DH
aQBU008HaP8DY@THaP037>LHaQS600PHaP035:DHaQS600`HaP035:DHaQS600HHaP037>LHaQS600<H
aP037>LHaQS601dHaP8DY@dHaP035:DHaQS600PHaP<DY@<HaP035:DHaQS6008HaP8DY@@HaP035:DH
aQBU00HHaP035:DHaQS6008HaP<DY@8HaP035:DHaQBU008DY@DHaP8DY@036<HDYAS600DHaP8DY@TH
aP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J0`0016]J0aS60QBU2QS600<DYAS66<H041S60QBU1AS6
00<DYAS66<H01QS60QBU11S600DDYAS66<HHaQBU008HaP@DY@8HaP8DY@036<HDYAS600DHaP055:DH
aQS66<HDY@0:6<H25:D00acW6<HHaP086<H35:D00aS65:DHaP026<H00aBU6<HHaP096<H00acW5:DD
Y@026<H011cW5:DDYABU1QS600<DYAS66<H00aBU00<HaQBU6<H00QBU00<LiaBU6<H01QS600@LiaS6
6<HDY@@HaP035:DHaQS601/HaP035:DHaQS600LHaP035:DHaQBU02HHaP035:DHaQS6008HaP037>LH
aQS600LHaP035:DHaQS6008HaP045:DHaQS65:D66<H00aBU6<HHaP076<H00aBU6<HHaP0<6<H00aBU
6<HHaP066<H35:D<6<H55:D66<H00aBU6<HHaP086<H00aBU6<HHaP046<H6JeX20001JeX0000MKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`056<H0
0aBU6<HHaP066<H00aBU6<HHaP076<H01ABU6<HHaQS65:D01aS60QBU4QS600<DYAS66<H02QS600<D
YAS66<H00aS600<DYAS66<H00QS600<LiaS66<H00QS600@DYAS66<HDY@/HaP045:DHaQS65:D36<H0
0aBU6<HHaP076<H00aBU6<HHaP076<H00acW6<HHaP0E6<H25:D86<H01QcW6<HDYAS65:DHaP8DY@@H
aP035:DHaQS600PHaP035:DHaQBU01<HaP8DY@DHaP037>LHaQS601@HaP037>LHaQS600DHaP057>LH
aQS66<HLi`0>6<H00aBU6<HHaP056<H00aBU6<HHaP046<H00aBU6<HHaP056<H00aBU6<HHaP0<6<H0
11BU6<HHaQBU0QS60aBU11S600@DYAS66<HDY@8HaP8DY@036<HDYABU00@HaP8DY@8HaP055:DHaQS6
5:DHaP025:D36<H00aBU6<HHaP035:D01AS65:DHaQBU6<H00QBU00/HaQBU6<I_Nf]JJgY[FVmkJeY[
NP00005_N`005V]J00<006]JJeX016]J0QS600<DYAS66<H011S600<DYAS66<H06QS60QBU00<HaQBU
5:D00QBU1AS60QBU0aS600<DYAS66<H02AS600@DYAS66<HDY@@HaP035:DHaQS600<HaP8DY@<HaP03
5:DHaQS6008DY@036<HDYAS600`HaP8DY@THaP8DY@<HaP055:DHaQS66<HLi`0@6<H25:D56<H00aBU
6<HHaP0@6<H00aBU6<HDY@076<H25:DK6<H00aBU6<HHaP086<H00acW6<HHaP0A6<H01AcW6<HHaQS6
5:D01aS600<LiaS66<H00QS600<LiaS66<H011S600<DYAS65:D021S600<DYAS66<H011S600<DYAS6
6<H01AS600<DYAS66<H01aS60QBU00@HaQBU5:DDY@8HaP8DY@@HaP035:DHaQS600<HaP<DY@HHaP03
5:DHaQBU00@DY@8HaP035:DHaQS6008HaP8DY@8HaP8Li`/HaPM[FP40005[FP0001m[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j6<HDY@0N6<H00aBU
6<HHaP036<H25:D66<H25:D011S65:DHaQS60aBU0QS600<DYAS66<H01AS60QBU21S60aBU00<HaQBU
5:D00QS600<DYAS66<H00QS600<DYAS66<H021S600<DYAS66<H021S600<DYAS66<H04AS60QBU1aS6
0QBU1AS600<DYAS66<H02QS60QcW0QS600<DYAS66<H01QS600<DYAS66<H06QS600<DYAS66<H04aS6
00<LiaS66<H02AS600@LiaS67>LLi`8HaP037>LHaQS600`HaP037>LHaQS600hHaP035:DHaQS600DH
aP<DY@<HaP045:DHaQBU5:D76<H011BU6<HHaQBU0QS600<DYAS66<H021S60QBU0QS600<DYAS66<H0
3QS600<DYAS66<H00aS60QBU00<HaQBU6<H01QS600<DYAS66<H00aS600Q[NV]JKg][FV]jJeY_N`00
0F]j000FJeX00`00JeY[FP04JeX36<H25:DE6<H27>L46<H011BU6<HDYABU0QS600<DYAS65:D01QS6
00<DYAS65:D00aBU00<HaQBU6<H01QS60QBU1QS60QBU00LHaQcW7>LHaQBU6<HDY@056<H01ABU6<HH
aQBU6<H00QBU00DHaQBU6<HHaQcW00DHaP035:DHaQS600`HaP<DY@036<HDYABU00dHaP035:DHaQS6
00/HaP035:DHaQS600HHaP035:DHaQS6008HaP035:DHaQS600HHaP035:DHaQS601HHaP037>LHaQS6
01<HaP047>LHaQcW7>L<6<H67>L011S67>LHaQcW0aS60QcW0aS60QcW2QS600<DYAS66<H051S600<D
YAS66<H011S600@DYAS66<HDY@8HaP035:DHaQBU008HaP<DY@/HaP035:DHaQBU00<HaP8Li`XHaP8D
Y@<HaP035:DHaQS600DHaP035:DHaQBU00@DY@036<I[FV]J00E[FP40005[FP0001e_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00THaP065:DHaQS6
6<HDYAS60aBU1AS600<DYAS66<H011S600@LiaS66<HDY@HHaP035:DHaQS600@HaP035:DHaQS6008D
Y@056<HDYAS65:DHaP035:D26<H00aBU6<HHaP066<H00aBU6<HHaP076<H25:D46<H00aBU6<HDY@02
5:D00aS65:DHaP0;6<H25:D011S65:DDYAS60QBU41S600<DYAS66<H04AS600<DYAS66<H071S600<D
YAcW7>L01QS60aBU2AS600<LiaS65:D021S611BU2QS611cW11S611cW00<HaQcW7>L00aS60QcW0QS6
00<LiaS67>L011S600<LiaS66<H01QS600<DYAS66<H03AS600<DYAS66<H00aS600DDYAS66<HHaQBU
008HaP035:DHaQBU00DHaP035:DHaQBU00HHaP8DY@dHaP047>LHaQS66<H35:D86<H00aBU6<HHaP09
6<H45:D02QS65:E_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX96<H01QBU6<HHaQS6
5:DHaP<DY@DHaP035:DHaQS600@HaP047>LHaQS65:D66<H00aBU6<HHaP046<H00aBU6<HHaP025:D0
1AS65:DHaQBU6<H00aBU0QS600<DYAS66<H01QS600<DYAS66<H01aS60QBU11S600<DYAS65:D00QBU
00<HaQBU6<H02aS60QBU00@HaQBU5:DHaP8DYA0HaP035:DHaQS6014HaP035:DHaQS601`HaP035:DL
iacW00HHaP<DY@THaP037>LHaQBU00PHaP@DY@XHaP@Li`@HaP@Li`036<HLiacW00<HaP8Li`8HaP03
7>LHaQcW00@HaP037>LHaQS600HHaP035:DHaQS600dHaP035:DHaQS600<HaP055:DHaQS66<HDY@02
6<H00aBU6<HDY@056<H00aBU6<HDY@066<H25:D=6<H011cW6<HHaQS60aBU21S600<DYAS66<H02AS6
11BU00<HaQBUJeX01V]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDY@hHaP035:DHaQBU00LHaP035:DHaQS600DHaP035:DHaQS6
01<HaP035:DHaQS600@HaP035:DHaQBU008HaP8DY@046<HDYABU5:D46<H00aBU6<HHaP096<H25:D0
0aS65:DHaP056<H25:D76<H011BU6<HDYABU3aS600<DYAS66<H011S600<DYAS66<H00QS600<DYAS6
6<H01AS600<LiaS66<H00aBU00HHaQBU5:DDYAS65:D46<H00aBU6<HHaP066<H00aBU6<HHaP056<H0
0acW6<HHaP026<H00acW6<HHaP0A6<H00aBU6<HHaP0=6<H25:D46<H67>L26<H97>L021S67>LLiacW
6<HLiaS67>L46<H00acW6<HHaP0S6<H00aBU6<HDY@026<H25:D46<H011BU6<HHaQS60QBU4QS600<D
YAS66<H01QS61ABU2aS60QBU0aS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04JeX5
6<H00acW6<HHaP056<H00aBU6<HHaP025:D26<H00aBU6<HHaP0A6<H01AcW6<HHaQS65:D00QS60aBU
1QS61ABU21S60QBU00HHaQBU5:DDYAS65:D46<H27>L01ABU6<HHaQS65:D01aS600@DYAS65:DDY@PH
aP035:DHaQS6008DY@LHaP035:DHaQS600@HaP8DY@<HaP035:DHaQS600@HaP8DY@<HaP8DY@DHaP<D
Y@046<HLiaS65:DM6<H00aBU6<HHaP0U6<H67>L01AS67>LHaQcW6<H00R481AcW00<HaQcW7>L00aS6
00<LiaS67>L00aS600<LiaS66<H03aS600<LiaS66<H02AS600<DYAS66<H01QS600<DYAS66<H00aS6
00<DYAS66<H02AS600@DYAS66<HHaP8DY@LHaP035:DHaQS600HHaP035:DHaQS600<HaP8DY@036<HD
YAS600DHaP035:DHaQBU00<HaP8DY@M[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00HHaP035:DHaQS600DHaP045:DHaQS66<H2
5:D96<H25:D26<H00aBU6<HHaP066<H25:D56<H01aBU6<HHaQS65:DHaQBU00@HaP<DY@DHaP055:DH
aQBU6<HDY@036<H25:D00aS65:DHaP056<H25:D00aS65:DDY@066<H25:D26<H00aBU6<HHaP056<H2
5:D<6<H00aBU6<HHaP076<H00aBU6<HHaP025:D00aS65:DHaP026<H25:D46<H00aBU6<HHaP056<H0
1ABU6<HHaQS65:D021S600<LiaS66<H01aS600<DYAS66<H01AS600<DYAS66<H03aS60QcW00<HaQcW
6<H031S60acW00<HaQcW7>L00QcW00<Q21S67>L03QcW0QS600<LiaS66<H00QcW0QS600<LiaS66<H0
11S600<LiaS66<H041S60aBU00@HaQBU6<HDY@@HaP<DY@@HaP035:DHaQS6008HaP045:DHaQS65:D2
6<H00aBU6<HHaP026<H00aBU6<HHaP066<H00aBU6<HHaP026<H25:D76<H01ABU6<HHaQS65:D01QS6
00<DYAS66<H00QBU0aS600Q_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX00aBU6<HH
aP026<H00acW6<HHaP026<H01ABU6<HHaQS65:D00aS60QBU0aS600<DYAS66<H00QBU21S600<DYAS6
6<H00QS600<DYAS66<H01QS60QBU00@HaQBU6<HDY@HHaP035:DHaQS600@HaP035:DHaQS600LHaP8D
Y@8HaP035:DHaQS600DHaP8DY@HHaP035:DHaQS600@HaP045:DHaQBU5:D<6<H00aBU6<HDY@026<H0
0aBU6<HHaP036<H00aBU6<HHaP026<H00aBU6<HHaP066<H011BU6<HDYAS60QBU1QS60QBU00<HaQBU
6<H04AS600<DYAS66<H0:QS600<LiaS66<H01AcW0aS61QcW11S600@LiaS67>LLi`@HaP047>LHaQS6
7>LF6<H00aBU6<HHaP026<H00aBU6<HHaP086<H25:D66<H00aBU6<HHaP046<H00aBU6<HHaP076<H0
0aBU6<HHaP0=6<H25:D00aS65:DHaP0?6<H00aBUJeY[FP05JeX10001JeX0000FJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX3000016]JKg][FV]j0QS600@DYAS66<HDY@PH
aP035:DHaQS600PHaP045:DHaQS66<H25:D26<H00aBU6<HHaP026<H00aBU6<HHaP046<H01ABU6<HH
aQS65:D00QS60QBU00@HaQBU6<HHaP8DY@LHaP035:DHaQBU00<HaP<DY@8HaP<DY@/HaP035:DHaQS6
00LHaP075:DHaQS66<HDYAS65:D0:1S600<DYAS66<H00QS600<DYAS65:D03aS600<DYAS66<H04aS6
00<Lib487>L011S600<LiaS66<H02AS600<LiaS66<H011S600@LiaS66<HLi`<HaP037>LHaQcW00lL
i`<HaP037>LHaQS600LHaP057>LHaQS66<HLi`026<H00acW6<HHaP0;6<H00aBU6<HHaP086<H00aBU
6<HHaP036<H00aBU6<HDY@036<H00aBU6<HHaP0B6<H25:D011S65:DHaQBU0QS60QBU00@HaQBU6<HD
Y@DHaP035:DHaQS600hHaP06JgY[FVmkJeY[NV]J0P000F]j000FJeX00`00JeY[FP04JeX00aBU6<HH
aP036<H01ABU6<HDYAS65:D011S600<DYAS66<H01aS600DDYAS66<HHaQBU00THaP8DY@HHaP035:DH
aQBU00@DY@hHaP035:DHaQS600@HaP8DYCHHaP055:DHaQS66<HDY@0:6<H00acW6<HHaP0A6<H00aBU
6<HHaP056<H00aBU6<HHaP046<H00aBU6<HHaP0C6<H00acW6<HHaP066<H00acW6<HHaP026<H00acW
6<HLi`0;7>L00aS67>LQ20077>L00b487>LLi`057>L76<H00acW6<HHaP0H6<H00aBU6<HHaP036<H0
0aBU6<HHaP025:D56<H35:D36<H25:D66<H00aBU6<HHaP026<H01ABU6<HHaQS65:D011S60QBU2AS6
0QBU0aS600<DYAS66<H011S600<DYAS65:D00aBU0QS600<DYF]JJeX01F]J0@000F]J00007FmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/03QS600<D
YAS65:D02aS600<DYAS65:D021S600<DYAS66<H00QBU1AS61QBU1AS600<DYAS65:D03aS600<DYAS6
6<H01AS600<LiaS66<H01aS600<DYAS66<H04aS600<DYAS66<H011S600<DYAS65:D02aS600@DYAS6
6<HDY@HHaP035:DHaQS6008HaP<DY@8HaP8DY@8HaP035:DHaQBU00<HaP055:DHaQS66<HDY@0D6<H0
0aBU6<HHaP0?6<H01QcW6<HHaQS67>LHaP8Li`@HaQLLi`HHaP8Li`8HaP037>LHaQS600DHaP037>LH
aQS601LHaP035:DHaQS6010HaP8DY@8HaP035:DHaQS600HHaP@DY@8HaP035:DHaQS600HHaP8DY@@H
aP8DY@LHaP035:DHaQS6008HaP095:E_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J
3QS600<DYAS65:D02aS600<DYAS65:D021S600<DYAS66<H00QBU1AS61QBU1AS600<DYAS65:D03aS6
00<DYAS66<H01AS600<LiaS66<H01aS600<DYAS66<H04aS600<DYAS66<H011S600<DYAS65:D02aS6
00@DYAS66<HDY@HHaP035:DHaQS6008HaP<DY@8HaP8DY@8HaP035:DHaQBU00<HaP055:DHaQS66<HD
Y@0D6<H00aBU6<HHaP0?6<H01QcW6<HHaQS67>LHaP8Li`@HaQLLi`HHaP8Li`8HaP037>LHaQS600DH
aP037>LHaQS601LHaP035:DHaQS6010HaP8DY@8HaP035:DHaQS600HHaP@DY@8HaP035:DHaQS600HH
aP8DY@@HaP8DY@LHaP035:DHaQS6008HaP035:E[FV]J00E[FP40005[FP0001e[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00HHaP035:DHaQBU00PH
aP<DY@<HaP035:DHaQS6008DY@066<HDYAS66<HDYAS60QBU00<HaQBU6<H00aS60QBU1AS600<DYAS6
6<H00aS600<DYAS66<H00aS600@DYAS66<HHaP<DY@@HaP047>LHaQS65:D26<H00acW5:DDY@0;6<H0
11BU6<HHaQBU3QS60QBU31S60QBU31S600<DYAS66<H00QS60QBU0aS600<DYAS66<H01aS600<DYAS6
6<H00aS60aBU0aS600@LiaS66<HLi`lHaP055:DHaQS66<HLi`036<H00acW6<HHaP066<H27>L26<H0
0acW6<HLi`037>L26<H;7>L38@P011cW8@PLib482AcW0QS600DLiaS67>LHaQcW01hHaP065:DLiacW
7>LHaQBU11S60QBU0aS60QBU1aS600<DYAS66<H011S600<DYAS66<H00aS600@DYAS65:DHaP8DY@@H
aP065:DHaQBU5:DHaQBU2AS600@DYAS65:DDY@/HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<0
06]JJeX016]J4AS600@DYAS66<HDY@8HaP035:DHaQS6008DY@DHaP<DY@PHaP035:DHaQS6008HaP07
5:DHaQS66<HDYAS65:D01aS600DDYAS66<HHaQBU00@HaP<DY@HHaP8DY@036<HDYAS600PHaP035:DH
aQS6008HaP035:DHaQS6008HaP8DY@@HaP035:DHaQS600<HaP<DY@HHaP037>LHaQS601<HaP035:DH
aQS600DHaP035:DHaQBU00LHaP035:DHaQBU008DY@<HaP8Li`8DY@/HaP037>LHaQS600LHaP8Li`HH
aPXLi`056<HLiacW7>LHaP057>L00b487>LLi`037>L012487>LLib482acW00<HaQcW6<H01QS600<L
iaS66<H011S600<LiaS66<H03QS600<DYAS66<H00acW1AS600<DYAS66<H01AS60QBU2aS600<DYAS6
6<H01QS600<DYAS66<H00QS600<DYAS66<H00QBU1AS600<DYAS66<H01aS60QBU1QS60QBU0aS61f]J
0@000F]J00007VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmk
JeY[NV]JKg/HaP8DY@8HaP037>LHaQS600HHaP035:DHaQS600DHaP8DY@036<HDYAS600/HaP035:DH
aQS600DHaP045:DHaQS65:D26<H011BU6<HHaQBU31S600<DYAS66<H01QS600<DYAS66<H02aS600<L
iaBU6<H00aBU1aS60QBU4AS60QBU1AS60QBU1aS600<LiaS66<H01QS600<DYAS66<H00QBU0aS600<D
YAS66<H03aS600@DYAS66<HDYA0HaP037>LHaQS600`HaPlLi`058@PLiacW7>LQ20027>L00b487>LL
i`038@P27>L00b487>LLi`057>L00aS67>LHaP0E6<H35:D56<H01ABU6<HHaQS67>L031S600<DYAS6
5:D011BU00<LiaS66<H02AS60QBU00<HaQBU6<H00aS60QBU41S600<DYAS66<H00QS60QBU00<HaQBU
6<H01aS600TDYFmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX:6<H37>L00aBU6<HD
Y@076<H25:D56<H011BU6<HDYABU1AS60QBU0aS60QBU11S60QBU00@HaQBU6<HHaP8DY@THaP035:DH
aQBU00PHaP037>LHaQS600hHaP045:DHaQBU5:D=6<H35:D56<H25:D26<H00aBU6<HHaP056<H011BU
6<HHaQBU0aS600<DYAS65:D021S600<DYAS66<H01QS60QcW0QS600@DYAS66<HHaP8Li`<HaP035:DH
aQS601lHaP<Li`036<HLiacW00LLi`8Q208Li`068@PLib487>LQ21cW0b4800<Lib487>L00R4821cW
0QS600<LiaS66<H05QS600<DYAS65:D02aS600<DYAS66<H01AS600DDYAS66<HHaQBU00dHaP8DY@03
6<HDYABU00DHaP035:DHaQS600@HaP035:DHaQS600<HaP035:DHaQS6008HaP047>LDYAS65:D26<H3
5:D00aS65:DDY@076<H7JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP026<H00aBU6<HHaP0<6<H00aBU6<HHaP056<H00aBU6<HH
aP026<H00aBU6<HHaP046<H01ABU6<HHaQS65:D00QS600<DYAS66<H05QS600<DYAS66<H021S600<D
YAS66<H00QS600<DYAS65:D011S60QBU0QS61ABU41S600<DYAS66<H011S600<DYAS65:D00aS60QBU
00<HaQBU6<H03QS600<DYAS66<H01aS60QBU2QS60QBU00DHaQBU6<HHaQBU01<HaP<Li`8HaP037>LH
aQcW00<HaP037>LHaQS600DLi`8Q20<Li``Q208Li`DQ20DLi``HaP037>LHaQS600PHaP8DY@@HaP03
5:DHaQS600LHaP8DY@036<HDYAS600HHaP045:DHaQBU5:D26<H00aBU6<HHaP0:6<H011BU6<HHaQS6
1ABU1aS60aBU00<HaQBU6<H01AS600DDYAS66<HHaQBU00<HaP035:DHaQS600@HaP0;5:DHaQBUJgY[
FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FPTHaP035:DHaQS600<HaP037>LHaQS600<D
Y@036<HDYAS6008HaP<DY@THaP045:DHaQBU5:D46<H00aBU6<HHaP046<H00aBU6<HDY@0>6<H00aBU
6<HHaP026<H00aBU6<HHaP0@6<H00aBU6<HHaP026<H00aBU6<HDY@076<H00aBU6<HHaP046<H25:D0
0aS65:DHaP056<H00aBU6<HHaP066<H00aBU6<HHaP056<H25:D26<H00aBU6<HHaP076<H00aBU6<HH
aP046<H00aBU6<HHaP0>6<H37>L66<H27>L>6<H87>L<8@P00bDY8@PQ20028@P011cW8@PLiacW0R48
2QcW7aS60QBU0aS600<DYAS66<H031S600<DYAS66<H02AS60aBU00DHaQBU6<HHaQBU010HaP035:DH
aQS600`HaP8DY@PHaPM[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk010HaP8Li`8HaP035:DHaQBU008HaP<DY@DHaP035:DHaQS6
00<HaP8DY@@HaP055:DHaQS66<HDY@026<H00aBU6<HHaP0?6<H25:D01AS65:DDYAS65:D04QS600<D
YAS65:D00QBU11S60QBU3QS60QBU00<HaQBU6<H00QBU0aS600<DYAS66<H01AS600<DYAS66<H03QS6
00<DYAS66<H041S600<DYAS66<H02aS600<LiaS66<H02aS600<LiaS67>L021cW124800@U:B488@PU
:@DQ20039BTQ224800<Q20<Li`058@PLiacW7>LQ20037>L26<H27>L00aS67>LHaP076<H27>L00aS6
7>LLi`096<H00acW6<HHaP026<H45:D96<H00aBU6<HHaP026<H00aBU6<HDY@076<H00aBU6<HDY@07
6<H00aBU6<HHaP0:6<H25:D26<H25:D26<H00aBU6<HHaP0:6<H00aBU6<HHaP026<H45:D26<H026mk
JeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FQ0HaP8Li`8HaP035:DHaQBU008HaP<DY@DH
aP035:DHaQS600<HaP8DY@@HaP055:DHaQS66<HDY@026<H00aBU6<HHaP0?6<H25:D01AS65:DDYAS6
5:D04QS600<DYAS65:D00QBU11S60QBU3QS60QBU00<HaQBU6<H00QBU0aS600<DYAS66<H01AS600<D
YAS66<H03QS600<DYAS66<H041S600<DYAS66<H02aS600<LiaS66<H02aS600<LiaS67>L021cW1248
00@U:B488@PU:@DQ20039BTQ224800<Q20<Li`058@PLiacW7>LQ20037>L26<H27>L00aS67>LHaP07
6<H27>L00aS67>LLi`096<H00acW6<HHaP026<H45:D96<H00aBU6<HHaP026<H00aBU6<HDY@076<H0
0aBU6<HDY@076<H00aBU6<HHaP0:6<H25:D26<H25:D26<H00aBU6<HHaP0:6<H00aBU6<HHaP026<H4
5:D26<H7JeX10001JeX0000FJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeX3000016]JKg][FV]j2AS600<DYAS66<H04AS600<DYAS66<H01AS600<DYAS66<H00QS60QBU0aS6
0QBU00DHaQBU5:DHaQBU00<HaP045:DHaQS65:D36<H011BU6<HHaQBU4AS600<DYAS65:D01QS60QBU
00<HaQBU5:D01aS600@DYAS66<HDY@@HaP045:DHaQS66<H25:D36<H011BU6<HHaQBU3aS600<DYAS6
6<H05AS600<DYAS66<H01QS60QcW1aS600<LiaS66<H01QS60QcW1AS600<LiaS67>L02QcW0b4822DY
00<Q22DY8@P00RDY12482AcW00<HaQcW6<H011S60QcW1aS600<LiaS66<H03aS60QBU2aS600<DYAS6
6<H06QS60QBU1AS60QBU3aS600<LiaS66<H00aS600<DYAS66<H00QBU1QS600I[NV]JKg][FV]jJeX2
0001JgX001I[FP03001[FV]J00A[FP035:DHaQS600HHaP035:DHaQS600@HaP035:DHaQS600XHaP04
5:DHaQS65:D@6<H00aBU6<HHaP0H6<H011BU6<HDYAcW1AS600<DYAS65:D01QS600<DYAS66<H00QBU
00<HaQBU6<H021S600<DYAS66<H01AS600DDYAS66<HHaQBU00@HaP<DY@8HaP035:DHaQS600lHaP04
5:DHaQS65:D:6<H00aBU6<HHaP0R6<H37>L00aS67>LLi`037>L012487>LQ22480RDY00@YBRe[<H`]
J`@YBP<U:@DQ208Li`038@PLiacW008Q208Li`036<HLiaS600DLibLHaP035:DHaQS600<HaP045:DH
aQBU5:D46<H01ABU6<HHaQS65:D00QS600<DYAS65:D03AS600DDYAS66<HHaQBU00HHaP037>LHaQS6
00`HaP035:DHaQS6008HaP035:DHaQBU00M[FP40005[FP0001m_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk6<HDY@066<H00aBU6<HHaP0;6<H011cW
6<HHaQBU0aS60QBU4aS60QBU0aS600<DYAS66<H00aS600@DYAS65:DHaP8DY@036<HDYAS600<HaP8D
YAPHaP035:DHaQS600XHaP8DY@HHaP8DY@DHaP035:DHaQS600/HaP8DYAPHaP8DY@HHaP8DYA8HaP03
7>LHaQcW008Li`036<HLiacW008Li`038@PLiacW008Li`@Q20<YBP04;F/aS36<;F/3:DX39BT48@P0
0acW8@PLi`027>L00aS67>LLi`087>LC6<H00acW6<HHaP0=6<H00aBU6<HHaP026<H011BU6<HHaQBU
0aS600<DYAS66<H00QS60aBU11S600<DYAS66<H02AS600<DYAS66<H03AS600<DYAS66<H03AS600<D
YAS66<H00aS600/DYAS66<I_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0aS60QBU
21S600@DYAS66<HDY@DHaP037>LHaQS600@HaP035:DHaQBU00LHaP035:DHaQS601/HaP035:DHaQS6
00@HaP035:DHaQS600DHaP035:DHaQS600THaP035:DHaQS6018HaP035:DHaQS601HHaP035:DHaQS6
00dHaP035:DHaQS6020HaP057>LHaQcW6<HLi`026<H57>L38@P011cW8@PQ22480RDY0bU:0Re[0S6<
0be[0RU:00<U:B488@P00b481AcW0QS60QcW0QS600@LiaS67>LLi`8HaP037>LHaQS601XHaP035:DH
aQS6008HaP045:DHaQS65:D56<H25:D00aS65:DDY@076<H00aBU6<HDY@086<H25:D46<H011BU6<HD
YABU1AS600DDYAS66<HHaQBU00<HaP8DY@/HaP<DY@DHaPM[FP40005[FP0001e[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00XHaP<DY@/HaP057>LH
aQS66<HDY@0:6<H00aBU6<HHaP086<H25:D26<H011BU6<HHaQS60QBU11S600<DYAS66<H02QS600<D
YAS66<H09QS600<DYAS66<H011S600<DYAS66<H0=1S600<DYAS66<H01AS600<LiaS66<H01AS621cW
1B4800DLib488@PU:BU:00D]J`08<H`]Jc6<<H`e[Be[:DXU:@@Q20DLi`<HaP<Li`<HaP<Li`LHaP03
5:DHaQS600@HaP035:DHaQS600hHaP035:DHaQS600LHaP035:DHaQS6008DY@<HaP045:DHaQS66<H2
5:D36<H011BU6<HHaQBU0aS600<DYAS66<H011S600<DYAS67>L011S600@DYAS66<HHaP<DY@036<HD
YAS600LHaP035:DHaQS600LHaP<DY@8HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX0
16]J0aS60QBU5aS600@DYAcW6<HDY@@HaP045:DHaQS65:D:6<H25:DO6<H00aBU6<HHaP026<H00aBU
6<HHaP046<H011BU6<HDYABU2QS600<DYAS66<H00aS600<DYAS65:D011S600<LiaS66<H00QS600<D
YAS66<H01QS60QBU0aS600@DYAS65:DDYAHHaP8DY@036<HDYABU00LHaP035:DHaQS600LHaP047>LH
aQS67>L56<H47>L00b487>LQ20048@P29BT00bU:;F/mk`02?Nl00be[:DXYBP05<H`012e[9BTU:B48
0acW00<Q21cW7>L01QcW2aS600<LiaS66<H011S600<DYAS66<H00aS600<DYAS66<H03AS600<DYAS6
6<H01aS600<DYAS66<H02QS600@DYAS66<HHaP8DY@XHaP8DY@DHaP045:DHaQS66<H35:D26<H00aBU
6<HHaP026<H01ABU6<HHaQS65:D03QS61f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/01QS600<DYAS66<H05AS600<DYAS65:D0
1QS600<DYAS66<H01aS600<DYAS66<H011S600<DYAS66<H00QS600<DYAS66<H01QS600<LiaS66<H0
0QS600@DYAS66<HDY@LHaP035:DHaQS6008DY@@HaP035:DHaQS600`HaP035:DHaQS600<HaP035:DH
aQS600`HaP035:DHaQS600hHaP035:DHaQBU018HaP065:DHaQS66<HDYAcW1AS600<LiaS66<H00QS6
00<DYAS66<H01AS600<LiaS66<H01QcW0R4800<Lib489BT012DY00<YBS6<@Q000T8@00<]JbU::DX0
0Re[0RU:00<]JbU:9BT00R482AcW0QS600<LiaS67>L00aS60QcW31S600DDYAS66<HHaQBU00/HaP03
5:DHaQS6010HaP035:DHaQBU00@HaP055:DHaQS66<HDY@036<H25:D36<H00aBU6<HHaP046<H00aBU
6<HHaP026<H01ABU6<HHaQBU6<H00QBU3AS60aBU00<HaQBU6<H01aS600Q_Nf]JJgY[FVmkJeY[NP00
0Fmk000FJeX00`00JeY[FP04JeX66<H00aBU6<HHaP0E6<H00aBU6<HDY@066<H00aBU6<HHaP076<H0
0aBU6<HHaP046<H00aBU6<HHaP026<H00aBU6<HHaP066<H00acW6<HHaP026<H011BU6<HHaQBU1aS6
00<DYAS66<H00QBU11S600<DYAS66<H031S600<DYAS66<H00aS600<DYAS66<H031S600<DYAS66<H0
3QS600<DYAS65:D04QS600HDYAS66<HHaQBU7>L56<H00acW6<HHaP026<H00aBU6<HHaP056<H00acW
6<HHaP067>L28@P00acW8@PU:@049BT00bU:<Ha24002@Q000be[:DXYBP02;F/2:DX00be[:DXU:@02
8@P97>L26<H00acW6<HLi`036<H27>L<6<H01ABU6<HHaQS65:D02aS600<DYAS66<H041S600<DYAS6
5:D011S600DDYAS66<HHaQBU00<HaP8DY@<HaP035:DHaQS600@HaP035:DHaQS6008HaP055:DHaQS6
5:DHaP025:D=6<H35:D00aS65:DHaP076<H7JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP0D6<H00aBU6<HHaP026<H00aBU6<HH
aP026<H011BU6<HHaQBU4QS60QBU7QS600@DYAS65:DDY@/HaP045:DHaQBU5:D;6<H00aBU6<HHaP0;
6<H25:D66<H25:D011S65:DHaQBU2aS600DDYAS66<HHaQBU00@HaP047>LHaQS65:D@6<H27>L00aS6
7>LHaP036<H37>L36<H67>L00b489BTQ20038@P29BT2:DX012e[=JdaS36<1BU:12DY0b4800<Lib48
8@P01AcW00<HaQcW7>L00QcW3QS600@DYAS67>LDY@HHaP035:DHaQS601LHaP045:DHaQS66<H35:D3
6<H01QBU6<HDYABU6<HDY@8HaP8DY@`HaP8DY@8HaP035:DHaQS600DDY@THaP045:DHaQBU5:D46<H0
1ABU6<HHaQS65:D00QS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04JeX76<H011cW
6<HHaQS60QBU11S600<DYAS66<H011S600@LiaS65:DDY@<HaP035:DHaQS6008HaP035:DHaQBU00hH
aP8DYB0HaP8DY@@HaPHDYA4HaP045:DHaQS65:D26<H00aBU6<HDY@076<H00aBU6<HDY@036<H00aBU
6<HHaP0<6<H25:D00acW6<HDY@056<H00aBU6<HHaP0?6<H00aBU7>LLi`026<H27>L66<H00acW6<HL
i`047>L012489BTYBRU:12DY12U:0be[00DYBRDY9BTU:B4800<U:@@Q20037>LQ224800PLi`8HaP03
7>LHaQS600<HaP8Li`@HaP8LiaDHaP8DY@/HaP037>LHaQBU00<HaP045:DHaQS65:D26<H85:DC6<H0
0aBU6<HDY@036<H00aBU6<HHaP086<H01ABU6<HHaQS65:D00QS600@DYAS65:DDY@8HaP035:E[FV]J
00E[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00
JeY_Nf]JJgY[FVmk00<HaP8DY@036<HDYAS600THaP035:DHaQS600XHaP8DY@DHaP055:DHaQS66<HD
Y@036<H011cW6<HHaQBU0aS600@DYAS66<HDY@@HaP<DY@036<HDYAS600LHaP8DY@HHaP8DY@046<HD
YABU5:D76<H00aBU6<HDY@035:D26<H00aBU6<HHaP076<H27>L00aBU6<HDY@036<H25:D=6<H00aBU
6<HHaP056<H35:DR6<H011cW6<HDYABU11S600<LiaS66<H00aS61acW00<Q22U::DX00RU:0RDY0RU:
00<]JbU::DX00bU:0RDY0b480RDY124811cW00@Q21cW7>LLi`DHaP037>LHaQS600THaP057>LHaQS6
6<HLi`056<H011BU6<HHaQBU4QS60QBU2AS600<DYAS66<H01AS600<DYAS65:D01QS600<DYAS65:D0
31S600<DYAS65:D00aS600<DYAS66<H011S600<DYAS66<H02AS600lDYAS66<HHaQBU6<HDYFmkJeY[
NV]JKg][FV]j00000Fmk000FJeX30004JeX36<H25:D:6<H00acW6<HHaP036<H25:D>6<H00aBU6<HD
Y@096<H011BU6<HHaQBU0aS600<DYAS66<H00aS600<DYAS66<H01AS60QcW1AS600<DYAS66<H011S6
00DDYAS66<HHaQBU00@HaP045:DHaQS65:D26<H00aBU6<HHaP0<6<H00aBU6<HHaP025:D;6<H00acW
6<HHaP0@6<H01ABU6<HHaQS65:D011S600<LiaS66<H06aS62QcW0R4800@Lib489BT]J`@YBP03;F/Y
BRe[00<YBP@U:@XQ20037>LQ224800@Li`036<HLiaS600`HaP037>LHaQcW008Li``HaP035:DHaQS6
008HaP047>LHaQS65:D:6<H00aBU6<HHaP066<H00aBU6<HHaP086<H00aBU6<HHaP036<H00aBU6<HH
aP025:D00aS65:DHaP096<H01QBU6<HDYABU6<HDY@8HaP035:DHaQS601<HaPI[FP80005[FP0001e[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j018H
aP<DY@036<HDYAS600THaP055:DHaQS66<HDY@0;6<H00aBU6<HHaP026<H25:D26<H25:D46<H00aBU
6<HHaP036<H27>L46<H00aBU6<HHaP066<H00aBU6<HHaP096<H25:D36<H01QBU6<HDYABU6<HDYA8H
aP8DY@8HaP035:DHaQS600HHaP045:DHaQS65:D46<H00acW6<HHaP076<H00aBU6<HHaP0A6<H00acW
6<HHaP086<H011cW6<HHaQS61QcW0b4800<U:BU:;F/00S6<12e[0bU:00HU:BU::DXU:B489BT58@P0
1AcW8@PQ22487>L00R481AcW00<HaQcW7>L00QS60QcW1QS600@LiaS67>LHaP8Li`036<HLiaS600HH
aP035:DHaQS600/HaP<DYA4HaP035:DHaQS601XHaP037>LHaQS600hHaP035:DHaQBU00<DY@8HaP8D
Y@DHaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00<HaQBU5:D00QS600<DYAS6
6<H03aS600<DYAS66<H00QS600<DYAS66<H01AS600<DYAS66<H03aS611BU3QS600<DYAS66<H02aS6
0QBU3aS600<DYAS66<H00QS600<DYAS66<H021S600<DYAS66<H00QS600<DYAS66<H011S600<DYAS6
6<H021S600<LiaBU5:D01QS60aBU2aS600<DYAS66<H04QS600<LiaS67>L01AcW00@HaQcW8@PQ20<U
:@8YBP03;F/icSW>008e[@04<H`YBRDY:DX29BT2:DX29BT38@P29BT58@P;7>L00aS67>LHaP036<H2
7>L36<H27>L?6<H00aBU6<HHaP0K6<H00aBU6<HHaP066<H00aBU6<HHaP025:D36<H011BU6<HHaQBU
0QS600<DYAS66<H00QS600<DYAS66<H01QS600@DYAS66<HHaP<DY@LHaP055:DHaQS66<HDY@036<H0
0aBU6<HHaP07JeX10001JeX0000NKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeX006]JKg][FV]jJeY_NaS60QBU0QS600<DYAS66<H03aS600<DYAS66<H00QS600<DYAS66<H0
1AS600<DYAS66<H03aS611BU3QS600<DYAS66<H02aS60QBU3aS600<DYAS66<H00QS600<DYAS66<H0
21S600<DYAS66<H00QS600<DYAS66<H011S600<DYAS66<H021S600<LiaBU5:D01QS60aBU2aS600<D
YAS66<H04QS600<LiaS67>L01AcW00@HaQcW8@PQ20<U:@8YBP03;F/icSW>008e[@04<H`YBRDY:DX2
9BT2:DX29BT38@P29BT58@P;7>L00aS67>LHaP036<H27>L36<H27>L?6<H00aBU6<HHaP0K6<H00aBU
6<HHaP066<H00aBU6<HHaP025:D36<H011BU6<HHaQBU0QS600<DYAS66<H00QS600<DYAS66<H01QS6
00@DYAS66<HHaP<DY@LHaP055:DHaQS66<HDY@036<H02aBU6<HHaVmkJeY[NV]JKg][FV]j00000Fmk
000FJeX00`00JeY[FP04JeX56<H00aBU6<HHaP066<H011BU6<HHaQS60QBU11S600DDYAS66<HHaQBU
00DHaP8DYA@HaP8DY@dHaP037>LHaQS600`HaP035:DHaQS6008HaP035:DHaQS600DHaP055:DHaQS6
5:DHaP025:D00aS65:DHaP036<H25:D56<H00aBU6<HHaP0;6<H00acW6<HHaP025:D00aS65:DHaP06
6<H00aBU6<HHaP046<H00aBU6<HHaP0@6<H00acW6<HHaP056<H37>L011S67>LLiaS61QcW0b4812DY
0RU:00<]JcW>>Lh00SF]00D]JbU::DXU:BU:00DU:@8Q20039BTQ224800TQ20@Li`036<HLiacW00/H
aP037>LHaQS600`HaP035:DHaQBU008HaP035:DHaQS6024HaP035:DHaQS600HHaP8DYA4HaP045:DH
aQS66<H25:D00aS65:DHaP0:6<H00aBU6<HHaP036<H7JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP026<H01ABU6<HHaQS65:D0
2aS60QBU11S600<DYAS66<H00aS600<DYAS66<H01QS600<DYAS66<H00QBU2QS600@DYAS66<HHaP8D
Y@036<HDYAS600DHaP035:DHaQS6008HaP8DY@hHaP8DY@PHaP055:DHaQS66<H@Q0086<H25:D36<H0
0aBU6<HHaP076<H45:D86<H00aBU6<HHaP0D6<H011cW6<HHaQBU2aS600<LiaBU6<H01aS61acW0b48
00@YBRDY9BTU:@8YBP03;F/aS36<008aS004;F/YBRDY:DX79BT:8@P47>L012487>LLiaS60QcW61S6
0QBU1aS600<LiaS66<H01AS600<DYAS66<H05QS600@DYAS65:DDY@HHaP<DY@dHaP8DY@036<HDYAS6
00DHaP035:DHaQS600DHaP035:DHaQS600@HaP0;5:DHaQS6JgY[FVmkJeY[NV]JKg/00001JgX001I[
FP03001[FV]J00A[FPDHaP045:DHaQS65:D<6<H35:D=6<H25:D96<H00aBU6<HHaP036<H00aBU6<HD
Y@035:D00aS65:DDY@046<H01QBU6<HHaQS65:DHaP8DY@8HaP045:DHaQS65:D=6<H00aBU6<HHaP04
6<H00aBU6<HDY@026<H011BU6<HHaQBU11S600<DYAS66<H01AS600<DYAS65:D08AS600<LiaS66<H0
1AS600<LiaS66<H03aS600@LiaS66<HHaP8Li`8Q208Li`038@PU:BU:00DYBP<]J`03<H`]Jbe[008Y
BP039BTYBRDY00LU:@TQ208Li`<Q20<Li`8HaP037>LHaQS600/HaP8Lia/HaP055:DHaQS66<HDY@0D
6<H25:D86<H25:D26<H01aBU6<HHaQS65:DHaQBU00dHaP035:DHaQBU00PHaP055:DHaQBU5:DHaP02
5:D7JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX0
06]JKg][FV]jJeY_N`036<H25:D86<H00aBU6<HHaP0>6<H25:D46<H00aBU6<HHaP0A6<H01ABU6<HH
aQS65:D00aS600<DYAS66<H00QS600<DYAS66<H071S600@DYAS66<HHaP8DY@`HaP035:DHaQS600DH
aP8DY@/HaP037>LHaQS601HHaP037>LHaQS600<HaP035:DHaQS600PHaP047>LHaQS66<H57>L58@P0
0bDY:DXYBP03:DX2;F/6:DX00bDY:DXYBP049BT38@P49BT00b489BTU:@029BT00b487>LLi`067>L=
6<H00aBU6<HHaP027>L00aS65:DHaP056<H01ABU6<HHaQBU6<H00QBU1AS600<LiaS66<H02AS600<D
YAS65:D02aS600@DYAS66<HDY@DHaP035:DHaQS600HHaP035:DHaQS600`HaP045:DHaQS65:D36<H2
5:D00aS65:DDY@0=6<H026mkJeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FP036<HDYAS6
00DHaP035:DHaQS600<HaP035:DHaQS600HHaP035:DHaQS600HHaP035:DHaQS601@HaP035:DHaQS6
00HHaP045:DHaQS65:D46<H00aBU6<HHaP0;6<H00aBU6<HHaP0L6<H00aBU6<HHaP0@6<H25:D36<H0
0aBU6<HHaP0C6<H27>L66<H00acW6<HHaP086<H27>L00aS67>LLi`027>L01R487>LLiacW8@PU:@HY
BP03;F/YBRU:00HYBP039BTYBRDY00<U:@03:DXU:BDY00LU:@03:DX]JbU:008Q20DLi`036<HLiacW
008HaP047>LHaQcW6<H27>LC6<H00aBU6<HHaP0;6<H25:D=6<H00aBU6<HHaP096<H25:D00aS65:DH
aP076<H00aBU6<HDY@0;6<H00aBU6<HDY@046<H011BU6<HHaQBU1QS600<DYAS65:D01AS600<DYF]J
JeX01F]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
001[FV]jJeY_Nf]JJgXDY@DHaP035:DHaQS600`HaP035:DHaQS6018HaP8DY@8HaP8DY@`HaP8DY@03
6<HDYAS600DHaP037>LHaQBU00LHaP047>LHaQS66<H25:D36<H25:D56<H00aBU6<HHaP086<H25:D5
6<H00aBU6<HHaP036<H25:D00aS65:DHaP036<H25:D;6<H00aBU6<HHaP0A6<H00acW6<HHaP0?6<H0
0acW6<HHaP067>L58@P29BT00bU:;F/YBP03:DX2;F/3:DX39BT01BU:9BTU:BDY:DX012DY1BU:0bDY
00<YBRe[:DX00R4811cW2AS60acW0QS600<DYAS66<H011S600<DYAS66<H051S600DDYAS66<HHaQBU
01LHaP8DY@8HaP035:DHaQS600@HaP<DY@DHaP035:DHaQS600@HaP035:DHaQBU00LHaP8DY@hHaP08
JgY[FVmkJeY[NV]JKg/0005[NP0000=[FP0000000`000V]J0P0016]J0P001V]J00<006]JJeX016]J
00<HaQBU6<H00QBU0aS60QBU1aS600<DYAS66<H00QS600<DYAS66<H021S600<LiaS66<H00QS600<D
YAS66<H00aS600<DYAS66<H00aS60QcW1QS60aBU0QS600<DYAS66<H00aS600<DYAS66<H011S600DD
YAS66<HHaQBU00<HaP035:DHaQBU008HaP8DY@`HaP035:DHaQS600PHaP035:DHaQS600<HaP8DY@@H
aP075:DHaQS66<HDYAS65:D05AS600<DYAS66<H03QS60QcW21S600DLiaS66<HLiaS6008Li`DQ2004
9BTYBRe[;F/<:DX49BT01B489BTU:BDY:DX00S6<0bDY00HQ22DY9BTQ22DY8@P37>L01B487>LLiacW
6<H00acW00<HaQcW6<H03aS600<DYAS66<H04QS600<DYAS66<H011S600<DYAS66<H00aS60QcW21S6
00<DYAS66<H02AS600<DYAS66<H00aS60QBU1aS600<DYAS66<H02QS600<DYAS66<H00aS60aBU1AS6
00<DYAS66<H01f]J0@000F]J000086mkJeY[NP00Kg][FV]j001_Nf]J001[FVmk001[NV]J001[FV]j
JeY_Nf]J001[FVmkJeY[NV]JKg/HaQBU6<H25:D36<H25:D76<H00aBU6<HHaP026<H00aBU6<HHaP08
6<H00acW6<HHaP026<H00aBU6<HHaP036<H00aBU6<HHaP036<H27>L66<H35:D26<H00aBU6<HHaP03
6<H00aBU6<HHaP046<H01ABU6<HHaQS65:D00aS600<DYAS65:D00QS60QBU31S600<DYAS66<H021S6
00<DYAS66<H00aS60QBU11S600LDYAS66<HHaQBU6<HDY@0E6<H00aBU6<HHaP0>6<H27>L86<H01AcW
6<HHaQcW6<H00QcW1B4800@U:BU:;F/]J``YBP@U:@058@PU:BDY9BTYBP02<H`39BT01R489BTU:B48
9BTQ20<Li`058@PLiacW7>LHaP037>L00aS67>LHaP0?6<H00aBU6<HHaP0B6<H00aBU6<HHaP046<H0
0aBU6<HHaP036<H27>L86<H00aBU6<HHaP096<H00aBU6<HHaP036<H25:D76<H00aBU6<HHaP0:6<H0
0aBU6<HHaP036<H35:D56<H02aBU6<HHaVmkJeY[NV]JKg][FV]j00000Fmk0003JeX01@00JeY[FV]J
00000V]J00@006]JJeX0009[FP03001[FV]J00=[FP@000=[FPLHaP8DY@@HaP035:DHaQS600hHaP03
5:DHaQBU00/HaP035:DHaQS600THaP037>LDYABU008DY@<HaP035:DHaQS600THaP035:DHaQS600DH
aP045:DHaQBU5:D76<H01ABU6<HHaQS65:D03aS600<DYAS66<H01AS600@DYAS66<HDY@HHaP035:DH
aQS601/HaP8DY@@HaP037>LHaQS600HHaP8Li`8HaP8Li`036<HLib4800<Q20<U:@04:DX]Jbe[;F/5
:DX00be[9BTYBP03:DX69BT28@P01BDY:DX]Jbe[9BT00R4800@U:B488@PQ20TLi`DHaP037>LHaQS6
018HaP037>LHaQcW00<HaP@DYA<HaP035:DLiacW00<Li`@HaP8DYATHaP8DY@HHaP037>LHaQS600<H
aP035:DHaQS600<DY@HHaP8DY@<HaPE[FP<0005[FP0001e[NV]JKg/006]jJeY_N`00JgY[FP00JeY[
NP00Kg][FP00JeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00THaP035:DHaQS600`HaP035:DHaQS600<H
aP035:DHaQS600PHaP037>LHaQS600<HaP8DY@lHaP@DY@8HaP035:DHaQS600/HaP035:DHaQS601hH
aP035:DHaQS600<HaP035:DHaQS600XHaP8DY@THaP035:DHaQS600HHaP037>LHaQS6008Li`XHaP03
7>LHaQS600<HaP037>LHaQcW00DLi`038@PLibDY008U:@<YBPL]J`/YBP038@PU:BDY00HU:@038@PU
:BDY00PQ208Li`048@PLiacW6<H37>L86<H00aBU6<HDY@0=6<H011cW5:DHaQS60QBU0aS600<DYAS6
6<H02QS60QBU0aS600@DYAcW7>LLiaXHaP035:DHaQS600DHaP8DY@hHaP035:DHaQS6008HaP035:DH
aQBU00HHaP08JgY[FVmkJeY[NV]JKg/0005[NP000f]J00D006]JJeY[FP00009[FP04001[FV]J0002
JeX00`00JeY[FP03JeX00`00JeY[FP04JeX36<H25:D?6<H25:D011S65:DDYABU4aS60QBU00<HaQBU
6<H01AS600<DYAS66<H00QBU00<HaQBU6<H01AS600<DYAS66<H05QS600<DYAS66<H01aS600<DYAS6
6<H00QS60QBU11S600@DYAS66<HLi``HaP8DY@<HaP065:DHaQS66<HDYAcW71S60QcW2QS60acW00@Q
21cW8@PQ208U:@<]J`03<H`]Jbe[00D]J`PYBP8U:@8YBP039BTYBRU:00<U:@XQ20<Li`038@PLib48
008Li`<HaP037>LHaQS601THaP8DYA0HaP035:DHaQS600HHaP035:DHaQS600LHaP035:DHaQS601@H
aP8DY@hHaP035:DHaQS600<HaP035:DHaQS600HHaPM[FP40005[FP0000Q_Nf]J000006mkJeY[NV]J
0P0000A[NV]JKg][FP80000=Kg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/02AS600DDYAS66<HHaQBU
008HaP035:DHaQS600@HaP035:DHaQS6008HaP8DY@036<HDYAS6010HaP045:DHaQS66<H25:D76<H0
0aBU6<HHaP046<H00aBU6<HDY@076<H00aBU6<HHaP026<H25:D96<H00aBU6<HHaP046<H01ABU6<HH
aQS65:D02AS60QBU11S600<DYAS66<H00QS600<DYAS66<H021S600<DYAS66<H04QS60QcW41S611cW
00<HaQcW7>L0124800@U:BU:;F/]J`8aS0@]J`8aS003;F/YBRU:00<YBP8]J`8YBP03;F/U:BDY00<U
:@038@PU:BDY00XQ208Li`058@PLiacW7>LQ20057>LE6<H00aBU6<HHaP0C6<H25:D26<H011BU6<HH
aQS60QBU3AS60QBU1AS600@DYAS66<HHaP<DY@THaP<DY@`HaP@DY@LHaP0;5:DHaQS6Kg][FV]jJeY_
Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FPHHaP035:DHaQS600@HaP035:DHaQS6008HaP8DY@DH
aP035:DHaQS600<HaP035:DHaQS600HHaP035:DHaQS6008HaP035:DHaQS6008HaP8DY@lHaP8DY@/H
aP045:DHaQS66<H35:D@6<H011BU6<HHaQBU4QS600<DYAS66<H00QS60QBU1aS600<DYAS66<H00aS6
00<DYAS66<H04AS600DLiaS66<HHaQBU00DHaPLLi`DQ208U:@03:DX]Jbe[008aS003;F/YBRe[00D]
J`TYBPLU:@XQ20057>LQ22487>LQ20087>L46<H27>L86<H00aBU6<HHaP0J6<H25:DA6<H00aBU6<HD
Y@076<H25:D26<H35:D00aS65:DDY@0?6<H25:D96<H25:D66<H25:D7JeX10001JeX0000MJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP076<H25:D0
0aS65:DHaP036<H00aBU6<HHaP025:D56<H01AcW6<HHaQS65:D00QS600<DYAS66<H02QS600<DYAS6
6<H00QS60QBU1QS60QcW1AS60QBU3QS60QBU31S600<DYAS66<H021S600<DYAS66<H03AS600<DYAS6
6<H021S600<DYAS66<H01QS600<DYAcW7>L05AS60acW1AS60acW00<HaQcW7>L0124812DY00@YBRe[
;F/]J`8aS0@YBPH]J`DYBP039BTYBRDY008U:@8Q208U:@HQ20057>LQ21cW7>LQ20027>L00b487>LH
aP067>LH6<H27>L46<H011cW6<HHaQBU4AS600<DYAS66<H01aS600@DYAS65:DDY@8HaP8DY@<HaP03
5:DHaQS600dHaP035:DHaQS6008DY@036<HDYAS601DHaP0:5:DHaV]jJeY_Nf]JJgY[FVmk0001JgX0
01I[FP03001[FV]J00A[FQ4HaP037>LHaQS600/HaP035:DHaQS6008DY@/HaP<DY@dHaP035:DHaQS6
008HaP055:DHaQBU6<HDY@056<H00aBU6<HHaP036<H25:DA6<H00aBU6<HHaP0A6<H00aBU6<HHaP05
6<H00aBU6<HHaP056<H00aBU6<HHaP036<H011BU6<HHaQcW3AS600DLiaS66<HHaQcW00HHaP057>LH
aQS66<HLi`026<H00acW8@PU:@039BT012U:;F/]Jbe[0c6<0Re[00<YBRe[;F/012U:00@]JbU::DXY
BP8]J`8YBPHU:@<Q20037>LU:AcW008Li`058@PLiacW7>LQ20027>L00b487>LLi`077>LR6<H011BU
6<HHaQBU1AS60QBU0aS600<DYAS66<H01AS600<LiaS66<H03AS60QBU1AS600<DYAS65:D02aS600@D
YAS65:DHaP8DY@8HaP035:DHaQBU00PHaP035:DHaQS6008HaP035:E[FV]J00E[FP40005[FP0001e_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk014H
aP037>LHaQS600/HaP035:DHaQS6008DY@/HaP<DY@dHaP035:DHaQS6008HaP055:DHaQBU6<HDY@05
6<H00aBU6<HHaP036<H25:DA6<H00aBU6<HHaP0A6<H00aBU6<HHaP056<H00aBU6<HHaP056<H00aBU
6<HHaP036<H011BU6<HHaQcW3AS600DLiaS66<HHaQcW00HHaP057>LHaQS66<HLi`026<H00acW8@PU
:@039BT012U:;F/]Jbe[0c6<0Re[00<YBRe[;F/012U:00@]JbU::DXYBP8]J`8YBPHU:@<Q20037>LU
:AcW008Li`058@PLiacW7>LQ20027>L00b487>LLi`077>LR6<H011BU6<HHaQBU1AS60QBU0aS600<D
YAS66<H01AS600<LiaS66<H03AS60QBU1AS600<DYAS65:D02aS600@DYAS65:DHaP8DY@8HaP035:DH
aQBU00PHaP035:DHaQS6008HaP095:E_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J
2aS60QBU11S600<DYAS66<H02QS600<DYAS66<H01QS600<DYAS66<H06QS60QBU1AS60aBU1AS611BU
00<HaQBU6<H01AS600DDYAS66<HHaQBU00LHaP045:DHaQBU5:D66<H00acW6<HHaP076<H01ABU6<HH
aQS65:D07AS600<DYAS66<H02QS61QcW0R4812DY00@]Jc6<<H`aS0<]J`@YBP@]J`8YBP03;F/YBRU:
00<YBPLU:@<Q20087>LQ22488@PLib487>LQ20@Li`068@PLiacW7>LQ21cW0QS60acW2AS600<LiaS6
6<H01AS600<DYAS66<H011S600<DYAS66<H01aS60QBU11S600<LiaBU6<H01QS600@DYAS66<HDYA<H
aP035:DHaQS600<HaP035:DHaQS600<DY@THaP035:DHaQBU00HHaP035:DHaQS6008HaP035:DHaQS6
00LHaPM[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FP00JeY[NV]JKg][FV]j008DY@<HaP035:DHaQBU01@HaP035:DHaQS600<HaP8DY@@HaP035:DHaQS6
00HHaP035:DHaQS6018HaP035:DHaQS600<HaP035:DHaQS600dHaP045:DHaQBU5:D86<H25:D46<H0
0aBU6<HHaP0>6<H00aBU6<HHaP0:6<H00aBU6<HHaP026<H00aBU6<HHaP0:6<H00acW6<HHaP026<H0
0acW6<HLi`086<H00acW6<HHaP037>L28@P29BT2:DX012e[<H`e[CF]0be[00@YBRe[;F/]J`<YBP04
;F/YBRe[;F/2:DX2;F/2:DX79BT68@P00acW8@PLi`0>7>L96<H00acW5:DHaP0:6<H00aBU6<HDY@02
6<H00aBU6<HHaP066<H25:D00acW6<HHaP026<H00aBU6<HHaP066<H35:D011S65:DHaQS60QBU0QS6
00@DYAS66<HHaP<DY@036<HDYABU008HaP8DY@DHaP<DY@THaP035:DHaQS600HHaP035:DHaQS600PH
aP8DY@<HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J1aS600<DYAS66<H00aS6
0QBU0aS60QBU1QS60QBU11S600<DYAS66<H031S600<DYAS66<H01QS600<DYAS66<H011S600<DYAS6
6<H011S600<DYAS65:D011S600<DYAS66<H01AS600<DYAS66<H051S60QBU31S600@LiaS66<HDY@8H
aP8DYBDHaP@Li`046<HLiacW7>L26<H27>L28@P3:DX3;F/00c6<>LhaS002;F/01RU:;F/YBRU:;F/Y
BPD]J`LYBP<U:@038@PU:BDY00/Q20hLi`036<HLiaS600DHaP037>LHaQS600<HaP035:DLiaS600hH
aP035:DHaQS600/HaP035:DHaQS600XHaP8DY@<HaP035:DHaQS600DHaP8DY@@HaP035:DHaQS600dH
aP045:DHaQS66<H25:D36<H00aBU6<HDY@036<H011BU6<HHaQBU11S60QBU1f]J0@000F]J00005Vmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J0`0000A[FV]jJeY_N`<HaP8D
Y@036<HDYAS600DHaP035:DHaQS6008HaP8DY@DHaP065:DHaQS65:DHaQBU0aS60aBU1aS600DDYAS6
6<HHaQBU00hHaP055:DHaQS66<HDY@0?6<H25:D76<H011BU6<HHaQBU2aS600<DYAS66<H041S600<D
YAS66<H00QS600<DYAS66<H05QS600<DYAS66<H00QcW00<HaQcW7>L01AS600@LiaS66<HLi`8HaP<L
i`048@PU:BDY:DX3;F/0136<;F/]Jc6<2be[00DaS2e[;F/YBRe[00<YBP04;F/YBRU::DX29BT28@P0
0bDY8@PQ20078@P00acW8@PLi`0>7>L46<H00aBU6<HHaP066<H00acW6<HHaP0<6<H011BU6<HHaQS6
11BU4AS600<DYAS65:D011S600@DYAS66<HHaP8DY@<HaP035:DHaQS600hHaP@DY@PHaP035:DHaQS6
00LHaP035:DHaQS600DHaP06Kg][FV]jJeY_Nf]J0P000Fmk000FJeX00`00JeY[FP04JeX36<H25:D6
6<H27>L56<H25:D=6<H35:D36<H00aBU6<HHaP0;6<H00aBU6<HHaP026<H25:D46<H35:D66<H25:D6
6<H25:D86<H01QBU6<HHaQS65:DHaP8DY@LHaP035:DHaQS6010HaP035:DHaQS6008DYA4HaP035:DH
aQS600lHaP037>LHaQS600<HaP@Li`@Q20039BTYBRe[00<aS004;F/YBRe[:DX3;F/00bU:;F/]J`02
:DX5;F/2:DX00be[:DXYBP02:DX79BT=8@P:7>L26<H00acW6<HHaP0C6<H00aBU6<HHaP086<H00aBU
6<HHaP026<H00acW6<HHaP066<H011BU6<HDYAS60QBU0QS600<DYAS66<H03QS600<DYAS66<H00aS6
0QBU31S600@DYAS66<HDY@hHaP035:DHaQS600DHaPM[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j008HaP035:DHaQS6008HaP04
5:DHaQS66<H27>L;6<H00aBU6<HHaP096<H00aBU6<HHaP0;6<H25:D56<H00aBU6<HHaP086<H00aBU
6<HHaP046<H35:D>6<H00aBU6<HHaP076<H011BU6<HDYABU61S600<LiaS66<H01QS600<DYAS66<H0
4aS600@LiaS66<HLi`8HaPXLi`058@PU:BDY;F/aS002=Jd00c6<;F/YBP05;F/3:DX00be[:DX]J`02
<H`01Be[:DXYBRe[:DX00be[1BU:00<U:B488@P00bDY1R4811cW0R4811cW00@Q21cW6<HHaP8Lia0H
aP035:DHaQS601@HaP035:DHaQS600<HaP045:DHaQS65:D46<H25:D@6<H00aBU6<HHaP026<H00aBU
6<HHaP0:6<H00aBU6<HHaP076<H00aBU6<HHaP076<H011BU6<HHaQBU11S600TDYF]jJeY_Nf]JJgY[
FVmk00000F]j000FJeX00`00JeY[FP04JeX00aS65:DHaP0;6<H00aBU6<HHaP0=6<H00aBU6<HHaP0=
6<H25:D00aS65:DDY@025:D46<H00aBU6<HHaP046<H45:D?6<H25:D26<H00aBU6<HHaP0:6<H00aBU
6<HHaP066<H00aBU6<HHaP025:D26<H00aBU6<HHaP0?6<H00aBU6<HHaP0C6<H00acW6<HHaP076<H0
0acW6<HHaP067>L38@P00bDY:DX]J`03>Lh00cF];F/]J`06;F/00bU:;F/]J`02<H`6;F/00bU:;F/Y
BP05:DX00bDY:DXU:@029BT012489BTQ22DY124841cW00<HaQcW6<H021S600<DYAS66<H041S600<D
YAS65:D02QS60QBU00<HaQBU6<H00QS600<DYAS66<H01AS60QBU1AS600<LiaBU5:D021S60QBU0QS6
00<DYAS66<H00aS600@DYAS66<HDY@8HaP<DY@@HaP<DY@036<HDYABU008HaP035:DHaQS600<HaP8D
Y@<HaPM[FP40005[FP0001m_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FP00JeY_Nf]JJgY[FVmk6<HDY@0<6<H00aBU6<HHaP0=6<H00aBU6<HHaP0=6<H25:D00aS65:DDY@02
5:D46<H00aBU6<HHaP046<H45:D?6<H25:D26<H00aBU6<HHaP0:6<H00aBU6<HHaP066<H00aBU6<HH
aP025:D26<H00aBU6<HHaP0?6<H00aBU6<HHaP0C6<H00acW6<HHaP076<H00acW6<HHaP067>L38@P0
0bDY:DX]J`03>Lh00cF];F/]J`06;F/00bU:;F/]J`02<H`6;F/00bU:;F/YBP05:DX00bDY:DXU:@02
9BT012489BTQ22DY124841cW00<HaQcW6<H021S600<DYAS66<H041S600<DYAS65:D02QS60QBU00<H
aQBU6<H00QS600<DYAS66<H01AS60QBU1AS600<LiaBU5:D021S60QBU0QS600<DYAS66<H00aS600@D
YAS66<HDY@8HaP<DY@@HaP<DY@036<HDYABU008HaP035:DHaQS600<HaP8DY@<HaP08Kg][FV]jJeY_
Nf]JJgX0005_N`005V]J00<006]JJeX016]J1AS600<DYAS66<H021S600<LiaS66<H04AS600@DYAS6
6<HDY@8HaP8DY@DHaP035:DHaQS600DHaP035:DHaQS600@HaP055:DHaQS66<HDY@026<H25:D:6<H2
5:D96<H00aBU6<HDY@056<H00aBU6<HHaP066<H00aBU6<HHaP0;6<H00aBU6<HHaP066<H00acW6<HH
aP086<H00acW6<HHaP036<H25:D86<H01QcW6<HHaQS67>LHaP<Li`056<HLiacW7>LQ20027>L01248
9BT]Jc6<0cg_00<aS2e[<H`00be[0c6<0Re[1C6<12e[0S6<0be[00<aS2U::DX012U:00@U:BU:8@PU
:@<Q20047>LQ22487>L28@P:7>L:6<H00aBU6<HHaP0B6<H00aBU6<HHaP036<H25:D26<H00aBU6<HH
aP046<H011BU6<HHaQS60aBU2aS600<DYAS66<H01AS60QBU0QS60QBU0aS60aBU0aS600<DYAS66<H0
0aBU00@HaQBU6<HHaP@DY@hHaP035:DHaQS600M[FP40005[FP0001m[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j6<HDY@0@6<H25:D26<H00aBU6<HH
aP076<H00aBU6<HHaP046<H00aBU6<HHaP036<H00aBU6<HHaP0=6<H011BU6<HHaQS60QBU00<HaQBU
6<H01aS600@DYAS66<HHaP8DYAXHaP037>LHaQS6008Li`035:DHaQS600PHaP035:DHaQS600`HaP03
5:DHaQS6008DYA<HaP037>LHaQS600HLi`038@PLib48008Q20049BT]Jc6<>Lh2=Jd3;F/00c6<;F/]
J`03<H`2;F/00c6<=JdaS003<H`9;F/3:DX00be[:DXYBP049BT00b487>LLi`028@P@7>L00aS67>LH
aP066<H00acW6<HHaP0B6<H25:D00aS65:DHaP026<H00aBU6<HDY@026<H00aBU6<HHaP036<H00aBU
6<HHaP026<H01ABU6<HDYAS65:D00QS600<DYAS66<H01QS60aBU00<HaQBU6<H011S600<DYAS66<H0
0aS600DDYAS66<HHaQcW00<HaP045:DHaQS66<H25:D26<H00aBU6<HHaP046<H25:D76<H02ABUJgY[
FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP036<HDYABU00/HaP035:DHaQS6010HaP<D
Y@036<HDYAS6008HaP055:DHaQS66<HDY@086<H00aBU6<HHaP066<H00aBU6<HHaP056<H01aBU6<HH
aQS65:DHaQBU008HaP035:DHaQS600<HaP035:DHaQBU00HHaP035:DHaQS600XHaP045:DHaQBU6<H2
5:DJ6<H00acW6<HHaP0:6<H011BU6<HHaQcW1aS600<LiaS67>L00aS611cW0b4800DU:BU:;F/aS3W>
008e[@03;F/YBRe[00DaS004;F/aS36<<H`2=Jd4<H`7;F/7:DX49BT68@P00acW8@PLi`028@P87>L2
6<H00acW6<HLi`0@6<H25:D26<H00aBU7>LHaP036<H35:D76<H00aBU6<HDY@046<H01ABU6<HHaQS6
5:D00aS600<LiaBU6<H03AS600@DYAS66<HHaP8DY@@HaP035:DHaQS600@HaP035:DHaQS6008DY@03
6<HDYAS600DHaP8DY@HHaP035:DHaQS600LHaPM[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk01LHaP8DYA<HaP047>LHaQS66<H2
5:D>6<H00aBU6<HHaP0;6<H00aBU6<HHaP066<H25:D26<H25:D00aS65:DDY@086<H25:D66<H25:D@
6<H00aBU6<HHaP0B6<H25:D86<H47>L46<H47>L48@P00bU:;F/e[@02>Lh2=Jd01C6<;F/]Jbe[<H`0
12e[00<aS2e[<H`01S6<00@]Jc6<;F/]J`8aS006;F/YBRe[;F/YBRe[0bU:12DY1B480acW0R480acW
00<HaQcW7>L01AcW1AS600<LiaS66<H03AS600<DYAS66<H00QBU1QS600DDYAS66<HHaQBU008HaP03
5:DHaQS6008DY@HHaP<DY@PHaP8DY@@HaP<DY@<HaP8DY@PHaP8DY@8HaP037>LHaQS600@HaP045:DH
aQBU5:DA6<H02QBU6<I_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX00aS65:DLi`07
6<H00aBU6<HHaP096<H00aBU6<HHaP0=6<H00aBU6<HHaP036<H00aBU6<HDY@026<H011BU6<HHaQBU
11S600<LiaS66<H06aS600<DYAS66<H01aS60QBU1aS600<DYAS66<H01aS600<DYAS65:D02aS600<D
YAS65:D01AS600<DYAS66<H04QS60QcW11S61AcW1B480RDY00@YBS6<=JdicP@e[@03;F/aS36<008a
S08]J`<aS003=JdaS2e[00HaS0@]J`PYBP049BTYBRDY9BT48@P00acW8@PLi`028@P:7>L46<H00acW
6<HHaP0I6<H01ABU6<HHaQS65:D02QS600<DYAS66<H00QS60aBU2QS600<DYAS66<H00aS60QBU00<H
aQBU6<H00QS60QBU6AS600@DYAS65:DDY@/HaPM[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00hHaP8DY@8HaP<DY@8HaP035:DH
aQS6008HaP065:DHaQS65:DHaQBU3QS600<DYAS66<H01QS600<LiaS66<H00aS600<DYAS66<H03QS6
00<DYAS66<H00QS600@DYAS65:DDY@8HaP035:DHaQS600dHaP055:DHaQS65:DHaP035:D<6<H00aBU
6<HHaP026<H25:D46<H00aBU6<HHaP076<H00acW6<HHaP0:6<H00acW6<HHaP026<H57>L48@P29BT0
0bU:<H`e[@02=Jd5<H`00cF]<H`aS004<H`00cF]<H`aS005<H`013F]<H`]Jc6<0Re[1RU:0bDY00<Y
BR489BT01b480QcW0R481QcW0QS60acW0QS600<LiaS66<H02aS600<DYAS66<H00QS600<DYAS66<H0
21S600<DYAS66<H00aS600<DYAS66<H01AS600<DYAcW6<H011S600<DYAS66<H011S60aBU00<HaQBU
6<H01QS600<DYAS66<H00aBU00<HaQBU6<H06QS611BU1QS600Q[NV]JKg][FV]jJeY_N`000F]j000F
JeX30004JeX56<H25:D96<H00aBU6<HHaP056<H00aBU6<HHaP066<H00aBU6<HHaP036<H25:D76<H0
0aBU6<HHaP056<H00acW6<HHaP0H6<H00aBU6<HHaP0>6<H00acW6<HHaP036<H00aBU6<HHaP0>6<H0
0aBU6<HHaP026<H25:D86<H00aBU6<HHaP056<H00acW6<HLi`0>6<H011cW6<HLiacW0QS600@Lib48
8@PQ208U:@<YBP04<H`e[CF]=Jd5<H`00cF]<H`e[@03<H`01CF]<H`aS3F]<H`00cF]136<1be[0RU:
0RDY12U:0bDY0b4800<Lib488@P00R481QcW11S600<LiaS66<H00acW1AS600DLiaS66<HHaQcW014H
aP8DY@<HaP8DY@HHaP<DYA4HaP<DY@8HaP8DY@036<HDYABU008DY@046<HDYAS66<H35:DP6<H00aBU
JeY[FP04JeX20001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeX006]JKg][FV]jJeY_N`056<H25:D96<H00aBU6<HHaP056<H00aBU6<HHaP066<H00aBU6<HHaP03
6<H25:D76<H00aBU6<HHaP056<H00acW6<HHaP0H6<H00aBU6<HHaP0>6<H00acW6<HHaP036<H00aBU
6<HHaP0>6<H00aBU6<HHaP026<H25:D86<H00aBU6<HHaP056<H00acW6<HLi`0>6<H011cW6<HLiacW
0QS600@Lib488@PQ208U:@<YBP04<H`e[CF]=Jd5<H`00cF]<H`e[@03<H`01CF]<H`aS3F]<H`00cF]
136<1be[0RU:0RDY12U:0bDY0b4800<Lib488@P00R481QcW11S600<LiaS66<H00acW1AS600DLiaS6
6<HHaQcW014HaP8DY@<HaP8DY@HHaP<DYA4HaP<DY@8HaP8DY@036<HDYABU008DY@046<HDYAS66<H3
5:DP6<H02ABUKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP035:DHaQBU00hHaP03
5:DHaQS600@HaP035:DHaQS600@HaP035:DHaQBU008DY@8HaP035:DHaQS600DHaP045:DHaQBU6<H2
5:D00aS65:DLi`086<H011BU6<HDYABU4aS60QBU0aS600@DYAS66<HDY@8HaPDLi`@HaP035:DHaQS6
00THaP035:DHaQS600<HaP035:DHaQS601/HaP037>LHaQS600@HaPHLi`8HaP037>LQ2248008U:@<Y
BP05;F/aS3W>>Lhe[@02<H`00be[<H`aS003=Jd2<H`013F]<H`aS36<1SF]136<0be[0S6<00<]JbU:
;F/00bDY0RU:1BDY0b4800<Lib487>L00acW0aS60acW0QS600<LiaS67>L021S600<DYAS66<H021S6
00<DYAS66<H021S600HDYAS65:DDYAS65:D36<H35:D56<H011BU6<HHaQBU2QS61ABU0aS600@DYAS6
5:DDY@<HaP035:DHaQS6008DY@HHaP@DY@036<HDYAS600`HaP035:DHaQS600<DY@8HaP035:E[FV]J
00E[FP40005[FP0001i[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00
JeY[NV]JKg][FV]j5:D26<H25:D@6<H00acW6<HHaP046<H00aBU6<HHaP056<H011BU6<HHaQBU1QS6
00<DYAS66<H04QS600<DYAS65:D02QS600<DYAS66<H05AS60QBU11S600<DYAS66<H011S60QBU00<H
aQBU5:D00aS60QBU5aS600<LiaS66<H021S61AcW00@HaQcW7>LLi`8Q20<U:@8YBP8aS08icP07=Jda
S36<;F/aS3F]<H`023F]0SW>0cF]0c6<1Be[0c6<00<]JbU:9BT00bDY0RU:0bDY0b480QcW12480QcW
00<HaQcW7>L00QS60acW2AS600<LiaS66<H00aS600<DYAS66<H031S600@DYAS65:DDY@@HaP@DY@PH
aP8DY@<HaP035:DHaQS600@HaP8DY@036<HDYABU008HaP035:DHaQS600@DY@056<HDYAS65:DHaP03
5:D36<H01QcW6<HHaQS65:DHaP8DY@DHaP045:DHaQBU6<H25:D00aS65:DHaP086<H026]jJeY_Nf]J
JgY[FVmk0001JgX001I[FP03001[FV]J00A[FPHHaP037>LHaQS601THaP8Li`PHaP045:DHaQS65:D<
6<H25:D00aS65:DDY@0=6<H00aBU6<HHaP0@6<H25:D56<H25:D;6<H00aBU6<HDY@0@6<H27>LC6<H2
7>L36<H77>L00b489BTU:@03:DX2;F/01C6<>LhicSg_=Jd00S6<00H]Jc6<=Jde[C6<=Jd2<H`2=Jd6
>Lh2=Jd4<H`2;F/00c6<;F/]J`03:DX39BT00bU:9BTYBP049BT78@P011cW6<HHaQS61acW00<Q21cW
7>L06aS600@DYAS66<HHaP<DY@DHaP8DY@8HaP035:DHaQS600PHaP8DY@036<HDYABU00<HaP8DY@04
6<HDYABU5:D46<H00aBU6<HDY@026<H45:D56<H35:D011S65:DHaQS60QBU1AS600<DYAS66<H00QS6
00@DYAS65:DDY@DHaPM[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk008HaP055:DHaQS66<HLi`036<H00aBU6<HHaP026<H00aBU
6<HHaP025:D36<H00aBU6<HHaP036<H35:DE6<H00aBU6<HDY@025:D46<H00aBU6<HHaP0>6<H00acW
6<HHaP066<H00aBU6<HHaP056<H00aBU6<HHaP046<H011cW5:DDYAS60QBU2aS60QBU00<HaQBU6<H0
0QBU31S600<DYAcW6<H00aS600<LiaS66<H00aS60QcW11S611cW00DHaQcW7>LQ21cW00DQ20039BTY
BRU:00<]J`05=JdicT8@AS4icP03<H`2=Jd00c6<=JdaS005>Lh00cF]>Lhmk`03>Lh01CF]<H`aS2e[
<H`00Re[00DaS2e[:DXYBRDY00<YBP8U:@<YBP<U:@LQ20<Li`036<HLiacW00<Li`056<HLiacW6<HL
i`0Q6<H01ABU6<HHaQS65:D011S600<DYAS66<H011S600@DYAS66<HDY@DHaP8DY@LHaPDDY@8HaP04
5:DHaQS65:D86<H25:D56<H25:D26<H00aBU6<HHaP0;6<H02aBU6<HDYFmkJeY[NV]JKg][FV]j0000
0Fmk000FJeX00`00JeY[FP04JeX66<H01AcW6<HHaQBU6<H00QBU00<HaQBU5:D00QS60QcW11S600<D
YAS66<H00QS60QBU2aS60QBU1AS600<DYAS66<H00aS600<DYAS66<H00aBU00<HaQBU6<H03aS600<D
YAS66<H07AS600<DYAS66<H01QS60QBU00<HaQBU6<H01AS600<DYAS66<H00QS600<LiaS66<H02QS6
00<DYAS66<H00aS600@LiaS66<HLi`@HaP8Li`038@PLiacW00<Q208U:@03:DX]Jbe[008]J`05=Jdm
kdicE[E24002=Jd4>Lh4=Jd00cW>?NlicP02>Lh4?Nl00cW>=JdaS005<H`2;F/3:DX39BT012U:9BTU
:BU:0bDY00@Q21cW8@PQ20/Li`056<HLiaS66<HLi`046<H00acW6<HHaP066<H00aBU6<HHaP0B6<H0
0aBU6<HDY@066<H00aBU6<HHaP026<H35:D;6<H00aBU6<HHaP056<H35:D66<H35:D36<H011BU6<HD
YABU2AS600<DYAS65:D02AS61ABU1f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX00QS600<DYAS66<H011S600<DYAS66<H00QS6
00<DYAS65:D011S600<DYAS66<H011S600<DYAS66<H01AS600<DYAS66<H02AS60QBU0aS600<DYAS6
6<H00QS60QBU00@HaQBU5:DHaP@DY@@HaP035:DHaQS600DHaP035:DHaQS600/HaP8DY@`HaP047>LH
aQS66<H27>L66<H25:D26<H011BU6<HHaQBU21S60QcW1aS600HDYAS66<HHaQBU7>L46<H011cW6<HL
iacW0QS61AcW0b480bDY00<YBRe[;F/00c6<00@mkdicE[E6<@8e[@03>Lhe[CW>00DicPTmk`8e[@Ha
S08]J``YBPDQ20`Li`@HaP037>LHaQS600DHaP035:DHaQS600<HaP035:DHaQS600@HaP035:DHaQS6
00DHaP035:DHaQS6008DY@8HaP045:DHaQS65:D36<H011BU7>LHaQcW11S600<DYAS66<H00aS600DD
YAS66<HDYAS600<DY@@HaP045:DHaQBU5:D46<H00aBU6<HHaP076<H35:D46<H25:D00aS65:DHaP05
6<H01ABU6<HDYAS65:D01QS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04JeX26<H0
0aBU6<HHaP046<H00aBU6<HHaP026<H00aBU6<HDY@046<H00aBU6<HHaP046<H00aBU6<HHaP056<H0
0aBU6<HHaP096<H25:D36<H00aBU6<HHaP026<H25:D011S65:DDYAS611BU11S600<DYAS66<H01AS6
00<DYAS66<H02aS60QBU31S600@LiaS66<HHaP8Li`HHaP8DY@8HaP045:DHaQS65:D86<H27>L76<H0
1QBU6<HHaQS65:DLi`@HaP047>LHaQcW7>L26<H57>L38@P39BT00bU:;F/]J`03<H`013g_CW=F]DHa
0SF]00<icSF]>Lh01CW>2Cg_0SF]1S6<0Re[32U:1B4831cW11S600<LiaS66<H01AS600<DYAS66<H0
0aS600<DYAS66<H011S600<DYAS66<H01AS600<DYAS66<H00QBU0QS600@DYAS66<HDY@<HaP045:DL
iaS67>L46<H00aBU6<HHaP036<H01ABU6<HHaQBU6<H00aBU11S600@DYAS65:DDY@@HaP035:DHaQS6
00LHaP<DY@@HaP8DY@036<HDYAS600DHaP055:DHaQBU6<HDY@066<H7JeX10001JeX0000MKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`056<H00aBU
6<HHaP036<H25:D00aS65:DHaP0:6<H25:DK6<H00aBU6<HHaP036<H25:D46<H01ABU6<HHaQS65:D0
2AS60QBU2aS600<DYAS65:D01aS611BU3QS600<DYAS66<H00aS60QBU2aS600<LiaS66<H03AS61AcW
1b480RDY00@]JbU:;F/]J`<e[@8mk`04@Q0mkcF]=Jd5>Lh3?Nl00cW>?Nm24005?Nl2>Lh013F]>Lhi
cSW>0Sg_0S6<2Be[0RU:0bDY1B4811cW0QS61AcW00@HaQcW7>LLi`/HaP035:DHaQS600PHaP045:DH
aQBU5:D=6<H011BU6<HHaQBU21S60QBU1AS60aBU1QS60QBU0QS60aBU0aS611BU00DHaQBU6<HHaQBU
00@HaP8DY@@HaP045:DHaQBU5:D66<H25:D66<H02ABUKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03
001[FV]J00A[FP036<HDYAS6008HaP035:DHaQS600<HaP8Li`8DY@HHaP055:DHaQS66<HDY@066<H0
0aBU6<HHaP026<H25:D@6<H00aBU6<HHaP046<H00aBU6<HHaP026<H00aBU6<HDY@076<H011BU6<HH
aQBU6AS60QBU2AS600<DYAS66<H011S600<DYAS66<H00aS600<DYAS66<H05AS60QcW0QS60acW00@Q
22U:9BTU:@@Q20069BTYBRe[:DX]Jc6<0SF]0SW>00<mkcF]>Lh00SF]00@icSg_>LhicPDmk`05@Q16
<D8@@Q0mk`04>Lh00cF]>Lhmk`03?Nl013F]<H`aS3F]0S6<0SW>00@e[C6<<H`]J`@YBP@Q20`Li`04
6<HLiaS67>L46<H00acW6<HHaP0L6<H011BU6<HHaQBU11S600<DYAS66<H01aS600<DYAS66<H011S6
00<DYAS65:D011BU11S60QBU1QS60QBU00<HaQBU6<H00QS600<DYAS66<H041S600@DYAS65:DHaP<D
Y@<HaPM[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FP00JeY[NV]JKg][FV]j00<HaP8Li`PHaP035:DHaQS600`HaP065:DHaQS66<HDYAS60QcW00DHaQBU
6<HHaQBU00THaP8DY@<HaP035:DHaQS600THaP055:DHaQBU6<HDY@046<H00aBU6<HHaP086<H25:D2
6<H00aBU6<HHaP026<H00aBU6<HHaP086<H00aBU6<HHaP056<H00aBU6<HHaP026<H25:D00acW5:DD
Y@0;6<H27>L26<H00acW6<HHaP0<6<H011cW6<HLiacW0QS600<Lib489BT00b480bDY00@YBRe[:DXY
BP<e[@PicP03=JdicSg_008mk`A24004AS52448@?Nl2>Lh3=Jd00cW>?NlicP02>Lh2=Jd013g_@Q0i
cSW>13F]0Re[0bU:0RDY0b4811cW0QS600<LiaS66<H011cW00<HaQcW6<H031S600<DYAS66<H011S6
00<DYAS66<H031S600<DYAS66<H00aS60QBU31S600HDYAS66<HDYAS65:D36<H45:D011S65:DHaQBU
0QS600DDYAS66<HHaQBU00LHaP035:DHaQS600HHaP@DY@8HaP8DY@LHaP035:DHaQS600<HaP08JgY[
FVmkJeY[NV]JKg/0005[NP005V]J0`0016]J0aS60QBU31S600<DYAS66<H00aS600<DYAS66<H01QS6
00HDYAS65:DDYAS65:D66<H00aBU6<HHaP036<H35:D26<H00aBU6<HDY@0=6<H00acW6<HHaP036<H0
0aBU6<HHaP0G6<H00acW6<HHaP026<H00acW6<HHaP066<H00aBU6<HHaP0C6<H00acW6<HHaP0<6<H2
7>L26<H57>L28@P01RDY8@PYBRU:9BTYBP8]J`8aS08e[@@icP03=JdicSW>00<icP<mk`924006AS52
44Ha?NlicSg_13W>00<e[CW>>Lh0136<00<e[Cg_@Q0013W>0SF]0Re[0bU:00<U:B489BT00R481acW
00HHaQcW7>LLiaS67>L76<H00aBU6<HHaP046<H00aBU6<HHaP0:6<H25:D56<H01QBU6<HHaQBU6<HD
Y@8HaP@DY@037>LHaQS600DHaP045:DHaQS66<H25:D26<H00aBU6<HHaP036<H00aBU6<HHaP086<H0
0aBU6<HHaP076<H00aBU6<HHaP025:D00aS65:DDY@056<H25:D00aS65:DDY@046<H00aBU6<HHaP02
6<H00aBU6<I[FP05JeX20001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_N`0>6<H00aBU6<HHaP036<H00aBU6<HHaP0:6<H35:DD6<H00aBU
6<HHaP026<H00aBU6<HDY@036<H00aBU6<HDY@076<H00aBU6<HDY@036<H01QBU6<HHaQS65:DHaP8D
Y@`HaP035:DHaQS601PHaP035:DHaQS600HHaP037>LHaQS600DHaP035:DHaQS600<HaP037>LHaQcW
008HaPHLi`8Q20039BTYBRU:008YBP8]J`04<H`e[CW>>Lh2?Nl7>Lh00d8@AS56<@03AS400dYBAS52
4005?Nl4>Lh013F]<H`aS36<0SF]00@mkcW>?NlicP8e[@8aS08]J`<YBP8U:@@Q20XLi`8HaP037>LH
aQS600DHaP037>LHaQS600LHaP035:DHaQS600HHaP8DY@036<HLiaS6008HaP035:DHaQS600<HaP8D
Y@PHaP@DY@`HaP8DY@DHaP035:DHaQS6008HaP045:DHaQS66<H25:D86<H25:D36<H25:D<6<H011BU
6<HDYABU0aS600Q_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX00aBU6<HHaP026<H0
0aBU6<HHaP056<H011BU6<HHaQBU0aS600<DYAS66<H021S600<DYAS66<H03QS600@DYAS65:DDY@<H
aP035:DHaQS6018HaP<DY@@HaP035:DHaQS6008HaP035:DHaQS600THaP037>LHaQS600<DY@8HaP03
5:DHaQS600/HaP035:DHaQS600lHaP8Li`PHaP047>LHaQS67>L36<H57>L48@P012DY:DXYBRU:0be[
00@icSg_@Q12408icP03?NlicSg_00<mk`924003BU96<DHa00=:DP04AS5244Ha@Q06?Nl00cW>=Jda
S002<H`00cF]?NlicP03>Lh4<H`2;F/4:DX00bDY8@PQ20048@P67>L26<H011cW6<HHaQcW4aS600@L
iaBU6<HDY@LHaP035:DHaQS6008DY@036<HDYAS600HHaP035:DLiaS6008HaP8DY@PHaP8DY@036<HD
YAS600`HaP035:DHaQBU00@HaP035:DHaQS6008HaP8DY@@HaP045:DHaQS65:D36<H00aBU6<HHaP09
6<H00aBU6<HHaP07JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeX006]JJgY[FVmkJeY[NP056<H00aBU6<HHaP036<H25:D00aS65:DHaP066<H00aBU6<HH
aP036<H00aBU6<HHaP026<H00aBU6<HHaP0:6<H00aBU6<HHaP046<H00aBU6<HDY@096<H25:D36<H0
0aBU6<HHaP026<H00aBU6<HHaP026<H00aBU6<HHaP036<H00aBU6<HDY@066<H00aBU6<HHaP056<H0
11BU6<HHaQcW1AS60QBU4AS600<DYAS66<H04aS600<LiaS66<H00QS61AcW0R480RDY0bU:00@]Jc6<
<H`icPDmk`03@Q0mkcg_008mk`03@Q16<DYB009>L`03BU9>Ldic00I:DP96<@924004?NlicSF]=Jd2
>Lh013F]>LhmkcW>0SF]136<0Re[12U:00<U:B488@P00R481acW00@HaQcW7>LLi`@HaP037>LHaQS6
01<HaP075:DHaQS66<HDYAS65:D01aS600<DYAS66<H021S611BU00@HaQBU6<HHaP8Li`036<HDYAS6
00DHaP@DY@036<HDYAS6008HaP<DY@HHaP035:DHaQS600@DY@hHaP035:DHaQBU00DHaP0;5:DHaQS6
JgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FPDHaP035:DHaQS600<HaP8DY@036<HD
YAS600HHaP035:DHaQS600<HaP035:DHaQS6008HaP035:DHaQS600XHaP035:DHaQS600@HaP035:DH
aQBU00THaP8DY@<HaP035:DHaQS6008HaP035:DHaQS6008HaP035:DHaQS600<HaP035:DHaQBU00HH
aP035:DHaQS600DHaP045:DHaQS67>L56<H25:DA6<H00aBU6<HHaP0C6<H00acW6<HHaP026<H57>L2
8@P29BT3:DX012e[<H`aS3W>1Cg_00=243g_?Nl00Sg_00=244HaBU800Tic00=:DTicCW<01TYB0THa
0T8@00@mkcW>=Jde[@8icP04=JdicSg_>Lh2=Jd4<H`2;F/4:DX00bDY8@PQ20028@P77>L011S67>LL
iacW11S600<LiaS66<H04aS600LDYAS66<HHaQBU6<HDY@076<H00aBU6<HHaP086<H45:D011S65:DH
aQS60QcW00<HaQBU6<H01AS611BU00<HaQBU6<H00QS60aBU1QS600<DYAS66<H011BU3QS600<DYAS6
5:D01AS600<DYAS66<H01f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/01AS600<DYAS66<H03AS600<DYAS65:D05aS600<DYAS6
6<H00QS600@DYAS65:DHaP8DY@8HaP035:DHaQS600@HaP8DY@hHaP035:DHaQS600PHaP035:DHaQS6
008DYA`HaP035:DHaQS600DHaP057>LHaQS66<HDY@0;6<H37>L26<H00acW6<HLi`028@P27>L00b48
9BTU:@03:DX012e[<H`aS3F]2Cg_0THa0Tic00A:DU:DCW=>L`9BU003CW=:DTYB00E>L`04BU96<Cg_
?Nl5=Jd2>Lh3=Jd4<H`2;F/3:DX39BT28@P47>L00b487>LLi`047>L00aS67>LLi`0A6<H00acW6<HH
aP0=6<H45:D26<H00aBU6<HDY@056<H25:D46<H011BU6<HDYAS60QcW00HDYAS66<HDYAS65:D56<H0
0aBU6<HHaP046<H00aBU6<HHaP046<H00aBU6<HDY@046<H00aBU6<HDY@0D6<H02aBU6<HDYFmkJeY[
NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX:6<H00aBU6<HHaP026<H00aBU6<HHaP056<H0
0aBU6<HDY@076<H01ABU6<HHaQS65:D031S611BU0aS600<DYAS66<H00QS600<DYAS65:D05aS600<D
YAS66<H05aS600<DYAS66<H00QS600<DYAS66<H011S600<DYAS66<H06QS61acW0R480RDY0bU:00@]
Jc6<>LhicP8mk`8icP8mk`=2409>L`06E[EBU4icDYA>LdYB0THa00=:DTicCW<00e:D0UJe00I>LdYB
AS4mkcW><H`5=Jd3<H`6;F/00bU:9BTU:@029BT48@P77>L00aS67>LLi`026<H00acW6<HHaP046<H0
0acW6<HHaP0F6<H00aBU6<HHaP025:D36<H25:D00aS65:DDY@026<H011BU6<HHaQS60aBU2aS600HD
YAS66<HDYAS65:D26<H01ABU6<HHaQS65:D03QS600<DYAS66<H00QS60QBU0QS600<DYAS65:D011S6
00<DYAS65:D011BU0aS61f]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDYA4HaP8DY@XHaP045:DHaQS65:D36<H00aBU6<HHaP02
6<H25:D66<H25:D00aS65:DDY@055:D56<H45:D36<H00aBU6<HHaP086<H011BU6<HLiaBU5QS600<D
YAS66<H031S600@DYAS66<HHaP8DYA0HaP047>LHaQS67>L56<H47>L28@P00acW8@PQ20029BT00bU:
;F/]J`02<H`4>Lh2?Nl01CW>?Nlmkd8@AS400U:D00=JeUJeCW<00Tic14YB0Tic00=F]EkgG_L00V<H
00EJeU:DBU9243W>00@e[@03<H`e[C6<008aS08]J`05<H`]Jbe[;F/YBP049BT38@P57>L00aS67>LH
aP037>L26<H37>L96<H25:D96<H00aBU6<HHaP0;6<H011BU6<HDYABU1AS600@DYAS66<HHaP<DY@TH
aP035:DHaQBU014HaP8DY@/HaP035:DHaQS600DHaP035:DHaQS600HHaP0>5:DHaQS66<HDYAS6JgY[
FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J0QS600<LiaS66<H03AS60QBU2QS600@DYAS6
6<HHaP8DY@<HaP035:DHaQS600LHaP055:DHaQS66<HDY@066<H00aBU6<HHaP066<H011BU6<HHaQS6
0QBU1AS60aBU0QS600<DYAS65:D011S600<DYAS66<H011S600<DYAS66<H01AS600DDYAS65:DHaQcW
010HaP8DYA@HaP055:DHaQS67>LHaP067>L28@P39BT012U:;F/]Jbe[0SF]00<icSg_?Nl013g_00=2
44HaBU801U:D00=>LdYBCW<00Tic00EBU5JeF]IW>Fmk009g_@06Kg]JeTicBU9243g_13F]00@aS3F]
<H`aS0@]J`DYBP8U:@@Q20<Li`038@PLiacW00DLiaLHaP8DY@HHaP035:DHaQBU00@HaP@DY@THaP03
5:DHaQS6008HaP035:DHaQBU00<HaP035:DHaQBU00PHaP095:DHaQS66<HDYAS65:DHaQBU01DHaP03
5:DHaQS600LHaP035:DHaQS600<HaPM[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00DHaP045:DHaQS65:D<6<H00aBU6<HDY@02
6<H25:D36<H00aBU6<HHaP025:D26<H00aBU6<HHaP0<6<H01ABU6<HHaQS65:D00aS60QBU2QS60aBU
6AS600<LiaS66<H04QS600<DYAS66<H03QS600<LiaS66<H01QS600<LiaS67>L011S611cW12480RDY
00<YBRe[;F/00Re[00DaS3F]>LhicSg_00=24004?Nm244YBDY@4E[D015:DCW=>Ldic0e:D0UJe00=N
mf]JMkd00goo00ENme:DBU9243g_008icP05=JdaS36<<H`]J`03<H`2;F/00bU:;F/U:@039BT28@P9
7>L56<H27>L;6<H35:D76<H25:D36<H25:D26<H00aBU6<HHaP0;6<H01QBU6<HHaQBU6<HDY@XHaP<D
Y@DHaP045:DHaQS65:D36<H00aBU6<HHaP056<H00aBU6<HDY@036<H25:D26<H00aBU6<HHaP026<H0
11BU6<HHaQBU0QS60aBU1AS60aBU0QS600Q_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04
JeX00aBU6<HHaP0<6<H25:D46<H00aBU6<HHaP026<H35:D00aS65:DHaP0F6<H011BU6<HHaQBU1AS6
0QBU2QS600<DYAS66<H01aS600@DYAS66<HHaP8DY@8HaP047>LHaQBU7>LA6<H00aBU6<HHaP0G6<H0
0acW6<HHaP0:6<H87>L28@P29BT00be[:DX]J`02;F/01CF]>Lhmkcg_@Q000THa0T8@00=6<DYBDY@0
0e[F00EF]DicCW=>Le:D00=F]@05F]IF]E[FIcU_N`02Nmh01VmkG_MBU4Ha@Q16<@8mk`04=JdaS36<
;F/2<H`4=Jd00c6<:DXYBP039BT48@P67>L36<H27>L:6<H00aBU6<HHaP0G6<H00aBU6<HHaP036<H0
0aBU7>LHaP056<H00aBU6<HHaP025:D46<H00aBU6<HHaP026<H00aBU6<HHaP036<H00aBU6<HHaP02
6<H00aBU6<HHaP036<H00aBU6<HHaP025:D86<H01ABU6<HHaQS65:D01QS60QBU11S611BU0QS600<D
YAS66<H01f]J0@000F]J00005V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]J0`0000A[FVmkJeY[NPTHaP8DY@THaP045:DHaQS65:D66<H00aBU6<HHaP066<H00aBU6<HHaP02
5:D26<H00acW6<HHaP035:D26<H00aBU6<HHaP066<H01QBU6<HLiacW6<HDY@<HaP045:DHaQS65:D;
6<H00aBU6<HHaP0R6<H011BU6<HHaQS60QBU4aS600HLiaS66<HHaQcW6<H67>L00b489BTU:@02:DX0
1Be[<H`aS3F]<H`00Sg_00A244HaAS56<@92409:DP05DYAJeU[FF]IF]@02CW<2DY@2E[D2G_L2F]H0
2UkgIcUW>FLiG_MF]E:DCW=:DTHa0T8@00<icS6<<H`00S6<00H]JcF]=Jde[CW>;F/2:DX39BT28@P5
7>L00b487>LLi`027>L00aS67>LLi`026<H00acW6<HHaP0?6<H00acW6<HHaP066<H25:D01AcW6<HH
aQS65:D01AS600@DYAS65:DDYA8HaP035:DHaQS600DHaP035:DHaQS600hHaP045:DHaQS65:D36<H0
0aBU6<HDY@0?6<H25:D36<H25:D01V]jJeY_Nf]JJgY[FP80005[NP005V]J00<006]JJeX016]J2AS6
0QBU2AS600@DYAS66<HDY@HHaP035:DHaQS600HHaP035:DHaQS6008DY@8HaP037>LHaQS600<DY@8H
aP035:DHaQS600HHaP065:DHaQcW7>LHaQBU0aS600@DYAS66<HDY@/HaP035:DHaQS6028HaP045:DH
aQS66<H25:DC6<H01QcW6<HHaQS67>LHaPHLi`038@PU:BDY008YBP05;F/aS36<=JdaS002?Nl0148@
AS56<DHa0T8@0TYB00EBU5[FF]IJeUJe009>L`9BU09F]@9Nm`9JeP0:G_MW>FLiIcUNmeJeDYA>LdYB
AS42@Q000cW><H`aS002<H`01Re[=Jde[CF]>Lh]J`8YBP<U:@8Q20DLi`038@PLiacW008Li`036<HL
iacW008HaP037>LHaQS600lHaP037>LHaQS600HHaP8DY@057>LHaQS66<HDY@056<H011BU6<HDYABU
4QS600<DYAS66<H01AS600<DYAS66<H03QS600@DYAS66<HDY@<HaP035:DHaQBU00lHaP8DY@<HaP8D
Y@M[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00
JeY_Nf]JJgY[FVmk00PHaP035:DHaQS600hHaP045:DHaQS65:D46<H01ABU6<HHaQS65:D00QS600LD
YAS65:DDYAS65:DLi`076<H011BU6<HHaQS60QBU0QS600<DYAS65:D00QS600<DYAS65:D01AS600<D
YAS66<H01QS600<DYAS66<H08AS60QBU0aS600<DYAS66<H04QS600DLiaS66<HLiaS6008Li`036<HL
iacW00DLi`<U:@<YBP8]J`8e[@03>Lhmkd8@00=24005AS5244HaCW=BU003F]H01E:DCW=>Le:DF]H0
0Ukg00=W>F<HHaP00Ukg00ES66LiIcUJeU:D00=>L`=6<@03>LhaS3F]00HaS004;F/YBRU::DX29BT5
8@P97>L?6<H00acW6<HHaP046<H01ABU6<HHaQS65:D02AS60QBU11S60QBU0aS60aBU2QS600<DYAS6
6<H01AS600DDYAS66<HHaQBU00DHaP8DY@HHaP8DY@DHaP<DY@036<HDYAS6008DY@<HaP035:DHaQS6
00PHaP8DY@8HaP8DY@096<I_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0QBU0aS6
0QBU0QS600DDYAS66<HHaQBU00PHaP035:DHaQS600DHaP045:DHaQS66<H25:D:6<H011BU6<HHaQBU
0aS600<DYAS66<H00QS60QBU00<HaQBU6<H01AS600DDYAS66<HHaQBU008Li`035:DHaQBU008HaP04
5:DHaQS65:D36<H00aBU6<HHaP0<6<H00aBU6<HHaP0Z6<H011cW6<HLiaS61acW0b480RDY0bU:0be[
00<aS3F]>Lh00T8@0dHa00E244HaBU9>LeJe009Nm`05E[EBU4icDYAF]@03G_L01V<HG_MJeUkgF]IN
m`9S6003F]IBU4ic009>L`05BU96<DHa>LhaS003=Jd3<H`2;F/00bU:9BTU:@039BT38@P:7>LF6<H4
5:D36<H00aBU6<HHaP096<H00aBU6<HHaP066<H00aBU6<HHaP046<H011BU6<HHaQBU1AS600@DYAS6
6<HDY@@HaP8DY@DHaP8DY@DHaP035:DHaQS600LHaP035:DHaQS600/HaP035:DHaQBU008HaP<DY@03
6<HDYF]J00I[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FP00JeY[NV]JKg][FV]j008HaP035:DHaQS600THaP035:DHaQS600@HaP045:DHaQS65:D36<H0
0aBU6<HHaP036<H25:D<6<H00aBU6<HHaP0E6<H00aBU6<HHaP066<H25:D66<H25:DE6<H25:D<6<H0
0acW6<HHaP046<H00acW6<HHaP056<H00aBU6<HHaP076<H87>L28@P29BT3:DX3;F/00c6<>Lhmk`02
@Q05AS400dicDYAF]@02F]H01UJeCW=BU5[FG_MJeP=Nm`04F]IF]E:DDY@5F]H015:DBU9:DTHa0T8@
00<e[C6<=Jd00c6<12e[00<YBRDY9BT00RDY0b4800<Lib487>L01AcW0QS600@LiaS66<HLi`8HaP03
7>LHaQS600PHaP8Li`8HaP035:DHaQS600HHaP035:DHaQS600@HaP8DY@<HaP035:DHaQS600/HaPDD
Y@036<HDYAS600THaP<DY@036<HDYAS600<HaP8DY@8HaP035:DHaQS600DHaP035:DHaQS600PHaP03
5:DHaQS600`HaP0;5:DHaQBUJgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP<HaP8D
Y@HHaP8DYAPHaP037>LHaQS600DHaP055:DHaQS66<HDY@046<H00aBU7>LHaP056<H00aBU6<HHaP02
6<H00aBU6<HHaP066<H00aBU6<HHaP056<H00aBU6<HDY@026<H35:D86<H00aBU6<HHaP036<H011BU
6<HHaQcW81S600<LiaS66<H00aS60QcW00<HaQcW7>L00QcW00<HaR488@P012480RDY0RU:00H]Jc6<
<H`e[CW>@Q02AS43BU800dHaBU9>L`02DY@2E[D015:DCW=JeU[F0Ukg0U[F00ENmeJeE[EF]E:D00EF
]@04DYA>LdYBAS42@Q000cW>=JdaS002=Jd3<H`2;F/00bU:9BTU:@029BT38@P77>L00b486<HLi`06
6<H00aBU6<HHaP0;6<H00aBU6<HHaP066<H00aBU6<HHaP025:D36<H25:D66<H25:DD6<H00aBU6<HH
aP036<H25:D96<H00aBU6<HHaP025:D56<H35:D36<H00aBU6<HHaP026<H00aBU6<HDY@066<H00aBU
6<HHaP056<H00aBUJeY[FP05JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`026<H00aBU6<HHaP026<H011BU6<HHaQBU0aS60QBU
00<HaQBU6<H03QS60aBU00<HaQBU6<H00QS600<LiaBU5:D011S60QBU0QS600HLiaS66<HDYAS65:D2
9BT56<H25:D46<H00aBU6<HHaP056<H00aBU6<HHaP0=6<H27>LQ6<H00aBU6<HHaP096<H00acW6<HH
aP026<H011cW6<HHaQS61acW0b480bDY0RU:00T]Jc6<<H`e[CW>@Q16<DYBAS4014YB0U:D0eJe00EB
U5JeHaQW>F<H00=Nm`9JeP=F]@09DYAF]EJeE[EJeUJeDYA>LdYB008mk`8icP<e[@05<H`e[CF]<H`Y
BP029BT78@P87>L56<H00acW6<HHaP0U6<H00aBU6<HHaP066<H00aBU6<HHaP066<H011BU6<HHaQS6
0QBU00<HaQBU5:D00aS60QBU0QS600@DYAS66<HDY@<HaP035:DHaQS602<HaP08Kg][FV]jJeY_Nf]J
JgX0005_N`005V]J00<006]JJeX016]J0QBU1QS600<DYAS66<H00QBU0QS600<DYAS66<H02aS600<D
YAS66<H011S600HLiaS66<HDYAS65:D:6<H00aBU6<HHaP086<H00aBU6<HHaP036<H00aBU6<HHaP0=
6<H00aBU6<HHaP056<H00aBU6<HHaP056<H00acW6<HDY@0K6<H011cW6<HHaQcW1AS600<LiaBU6<H0
1aS600<LiaS66<H00acW00<Q21cW8@P00RDY00@Q22DY:DXYBP8]J`8aS003=Jdmkd8@00=6<@9:DP03
CW=BU5Je009JeP9F]@06DYAJeV<HIcUS66Li0V<H00ANmf<HG_MNm`AJeP9Nm`04F]IBU4icBU82@Q00
0cW><H`e[@03=Jd2<H`012e[:DXU:BDY1B481QcW00@HaQcW7>LLi`PHaP037>LHaQS600`HaP035:DH
aQS6008HaP057>LHaQS66<HDY@026<H00aBU6<HHaP056<H00aBU6<HHaP0:6<H00aBU6<HHaP026<H0
0aBU6<HHaP025:D56<H45:D00aS65:DHaP0L6<H00aBU6<HHaP0:6<H25:D36<H7JeX10001JeX0000M
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP0:
6<H00aBU6<HHaP0A6<H011BU6<HHaQBU1QS600<DYAS66<H02QS600<DYAS66<H03aS600<DYAS66<H0
31S60QBU7AS60QBU21S600<LiaS66<H031S600<DYAcW7>L00aS600<LiaS66<H01AcW1B480RDY0RU:
0Re[0cF]00<mkdHaCW<00TYB0Tic00=BU5JeE[D01E[F00=Nmf<HHaP00V<H0VLi0V<H0ekg15[F00EN
meJeDYA>LdYB0096<@03>Lhe[CW>00De[@03;F/U:BDY008U:@8Q208Li`038@PLiacW00HLi`DHaP03
7>LHaQS6008HaP065:DHaQS67>LHaQBU1AS600<DYAS66<H00QBU1QS600<DYAS66<H03AS600<DYAS6
6<H00QS600<DYAS65:D01AS60QBU2AS60aBU1AS600<DYAS66<H00QS600@DYAS66<HHaP8DY@THaP04
5:DHaQS66<H25:D46<H00aBU6<HHaP076<H25:D36<H026]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03
001[FV]J00A[FPXHaP035:DHaQS6014HaP045:DHaQS65:D66<H00aBU6<HHaP0:6<H00aBU6<HHaP0?
6<H00aBU6<HHaP0<6<H25:DM6<H25:D86<H00acW6<HHaP0<6<H00aBU7>LLi`036<H00acW6<HHaP05
7>L58@P29BT2:DX2;F/3=Jd00cg_AS5>L`02BU82CW<00e:DE[EF]@05F]H00ekgHaQS6002HaP2IcT2
HaP3G_L4F]H01EkgE[EBU4icBU800THa00<icSF]>Lh01CF]00<]JbDY9BT00RDY0R480QcW00<Q21cW
7>L01QcW1AS600<LiaS66<H00QS600HDYAS66<HLiaS65:D56<H00aBU6<HHaP025:D66<H00aBU6<HH
aP0=6<H00aBU6<HHaP026<H00aBU6<HDY@056<H25:D96<H35:D56<H00aBU6<HHaP026<H011BU6<HH
aQS60QBU2AS600@DYAS66<HHaP8DY@@HaP035:DHaQS600LHaP8DY@<HaPM[FP40005[FP0000=_N`00
00000`0000Q[NV]J000006]jJeY_Nf]J0P0000e_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`066<H2
5:D00aS65:DDY@076<H25:D56<H00aBU6<HHaP096<H011BU6<HDYAS611BU1AS60QBU1AS600<LiaS6
6<H051S600<LiaS66<H00QS600<LiaBU6<H04QS600<LiaS66<H021S600<DYAS66<H01aS600<LiaS6
6<H01AS600<LiaS66<H03AS61AcW1B480bDY0RU:00HaS3F]=JdicT8@AS42CW<3DY@3E[D4F]H016<H
IcUW>FLi0V]J0VLi0V<H15kg15[F0UJe00ABU4YB?Nlmk`@icP06?NlicSW>>LhaS2U:0RDY124800@L
ib487>LQ20HLi`LHaP037>LHaQS600THaP@DY@LHaP037>LHaQS600<DY@DHaP045:DHaQS65:D56<H2
5:D;6<H00aBU6<HHaP076<H25:D00aS65:DHaP026<H00aBU6<HHaP066<H25:D46<H00aBU6<HHaP05
6<H00aBU6<HHaP086<H00aBU6<HDY@066<H02QBU6<I_Nf]JJgY[FVmkJeY[NP000Fmk0003JeX01@00
JeY[FV]J00000V]J00@006]JJeX0009[FP03001[FV]J00=[FP03001[FV]J00A[FP035:DHaQS6008H
aP8DY@hHaP035:DHaQS600DHaP045:DHaQS65:D<6<H00aBU6<HHaP076<H00aBU6<HHaP0B6<H00aBU
6<HHaP046<H00aBU6<HHaP086<H00aBU6<HHaP0=6<H00aBU6<HHaP056<H00aBU6<HHaP0=6<H01AcW
6<HHaQS67>L02aS600<LiaS66<H00acW1B480bDY00<YBRe[;F/00cW>00E244HaCW=>Le:D00=F]@04
G_MF]EJeF]H2G_L016<HJeY[FW>L0Vmk0V]J0V<H00ANme[FG_MS60=F]@05DYAF]E:DCW=6<@02?Nl2
>Lh2?Nl2@Q02?Nl01CF]:DXU:B489BT012481acW1AS60acW0QS600<LiaS66<H01QS600@DYAS65:DD
Y@THaP045:DHaQS66<H35:D36<H011BU6<HDYABU0aS60QBU1QS60QBU1AS600<DYAS66<H00aS600DD
YAS66<HHaQBU008HaP035:DHaQS6008HaP035:DHaQS600<HaP035:DHaQS601hHaP035:DHaQS600<H
aPM[FP40005[FP0001I[NV]JKg/006]jJeY_Nf]JJgY[FP00JeY[NP00Kg][FP00JeY_Nf]JJgY[FP@0
0004Kg][FV]j5:D56<H011BU6<HDYABU11S600<DYAS66<H021S600<DYAS66<H00QS600@DYAS66<HH
aP8DY@<HaP035:DHaQS600@HaP035:DHaQBU01@HaP047>LDYAS65:D46<H00acW6<HHaP056<H00aBU
6<HHaP086<H00aBU6<HHaP036<H00aBU6<HHaP0I6<H00acW6<HLi`026<H37>L00aS67>LHaP056<H0
0acW6<HHaP046<H37>L38@P39BT2:DX00bDY<H`aS002>Lh01Cg_AS5:DTicDY@015Je00=JeUJeF]H0
0f<H00=W>F]JLi`00W>L00A_Nf]JJeYW>@9Nm`03E[ENmf<H00=JeP9BU003CW=:DT8@008mk`06>Li2
448@@Q16<D8@0Sg_00<e[Be[9BT00RDY124800<Lib487>L00acW0aS600<LiaS66<H07QS600<DYAS6
6<H02aS600<DYAS66<H00aBU2AS60QBU00<HaQBU5:D02AS600<DYAS66<H021S600@DYAS65:DDY@8H
aP045:DHaQS66<H35:D;6<H00aBU6<HHaP025:D46<H35:D26<H01F]jJeY_Nf]JJgX00`000F]j0003
JeX00`00JeY[FP02JeX20003JeX01000JeY[FP001F]J00<006]JJeX016]J00<HaQBU5:D011S600@D
YAS66<HDY@@HaP035:DHaQS600@HaP035:DHaQS600<HaP035:DHaQS600/HaP8DY@dHaP<Li`8HaP03
7>LHaQS600/HaP8DY@<HaP<DY@PHaP037>LHaQS600HHaP035:DHaQS6008HaP035:DHaQS6010HaP03
7>LHaQS600HHaP8Li`HHaP047>LHaQS67>L46<H47>L48@P59BT012U:;F/aS3F]0cg_0TYB0Tic1E[F
0Ukg00IS66]JJeY[FW>LMkd2Nmh00gNmJeY[FP02IcT015kgF]IJeV<H0ekg00EF]E:DCW=6<D8@008m
k`03=Jdmkd8@0096<@05@Q0icSW><H`YBP029BT38@P:7>L011S67>LHaQcW1QS600LLiaS66<HHaQcW
6<HLi`0>6<H00acW6<HHaP046<H00aBU6<HHaP026<H25:D26<H011BU6<HHaQS60QBU2aS600<DYAS6
6<H01aS60QBU0QS600<DYAS66<H01QS60QBU00<HaQBU5:D01AS600DDYAS66<HHaQBU00THaP045:DH
aQBU5:D26<H35:D36<H00aBU6<HHaP07JeX10001JeX0000MKg][FV]j001_Nf]JJgY[FP00JeY[NV]J
Kg/006]jJeX006]JJgY[FVmkJeX006]JKg][FV]jJeY_N`066<H00acW6<HHaP046<H00aBU6<HHaP05
6<H01ABU6<HHaQS65:D01AS600<DYAS66<H00aS600<DYAS66<H00QS60QcW6QS600<DYAS66<H00aS6
00<DYAS66<H03QS600<DYAS66<H03QS600<LiaS66<H011S600<DYAS66<H00QcW0aS600<LiaS66<H0
1aS600<LiaS66<H02AS60QcW00<HaQcW7>L01AcW1BDY0RU:00<aS3W>?Nl00Sg_00=244HaBU800U:D
00EJeUkgG_MS66Li009S6005IcU_NfmkLiag_@04Ool2Li`01VmkIcUW>EkgF]IS60=Nm`05F]IF]EJe
BU924002?Nl01SF]>Lhmkd8@?NlicP8aS004;F/YBRDY9BT38@P00acW8@PLi`037>L26<H37>L=6<H0
0aBU6<HHaP0:6<H25:D36<H27>L00aS65:DDY@0B6<H00aBU6<HHaP066<H00aBU6<HHaP076<H35:D8
6<H25:D66<H35:D;6<H00aBU6<HHaP096<H00aBU6<HHaP046<H026mkJeY[NV]JKg][FV]j0001Kg/0
009[FP8000A[FP<000=[FP8000I[FP03001[FV]J00A[FP8HaP035:DHaQS600<HaP035:DHaQS600HH
aP055:DHaQS66<HDY@046<H25:D86<H25:DJ6<H00aBU6<HHaP0=6<H00aBU6<HHaP0G6<H00aBU6<HH
aP0:6<H011BU6<HHaQBU0acW31S600<LiaS67>L01QS60QcW0QS60QcW00<Q21cW7>L00R4812DY00<Y
BRe[=Jd00THa0d8@00A:DTicDYAJeP=Nm`=W>@04HaQ[FW>LLi`8Ool01G>LIcUS66<HG_L00f<H00IJ
eUJeE[EBU4ic@Q02?Nl5>Lh01CF]<H`aS2U:9BT01R481acW41S600<DYAS66<H00aS60QBU2QS600<L
iaS66<H011S600<DYAS66<H041S600<DYAS66<H011S600LDYAS66<HHaQBU6<HDY@076<H00aBU6<HH
aP046<H00aBU6<HHaP036<H35:D26<H00aBU6<HHaP026<H00aBU6<HHaP036<H00aBU6<HHaP086<H2
5:D46<H00aBUJeY[FP05JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDY@8HaP035:DHaQS600`HaP045:DHaQS65:D@6<H0
0aBU6<HHaP066<H00aBU6<HHaP036<H00aBU6<HHaP026<H00aBU6<HHaP0>6<H011BU6<HDYAS60QBU
2aS600<DYAS66<H051S600<DYAS66<H01AS600<LiaS66<H00QS600DDYAS66<HHaQBU00@HaP037>LH
aQS600@HaP037>LHaQS6008Li`8Q20047>LQ21cW8@P29BT4:DX012e[=Je244YB0THa00E244YBDYAF
]E[F00=S6005JeYW>F]JJeY_N`02Mkd8Ool016mkIcUS66Li0Ukg0U[F00IF]E:DE[EBU4ic@Q02?Nl0
13W>=Jde[CF]0S6<0Re[00<YBRDY9BT01B480acW00DHaQcW6<HHaQcW00DHaP8Li`036<HLiaS6010H
aP055:DHaQS66<HDY@0<6<H00aBU6<HHaP046<H011BU6<HDYABU1QS60QBU0aS600<DYAS66<H011S6
00<DYAS66<H00QBU11S600DDYAS66<HHaQBU00/HaP065:DLiaS66<HDYAS60QBU11S600<DYAS65:D0
1aS60QBU11S600XDYAS6JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00<DYAS65:D0
0QS600<DYAS66<H031S600@DYAS66<HDYA0HaP035:DHaQS600HHaP035:DHaQS600<HaP035:DHaQS6
008HaP035:DHaQS600hHaP045:DHaQBU6<H25:D;6<H00aBU6<HHaP0D6<H00aBU6<HHaP056<H00acW
6<HHaP026<H01ABU6<HHaQS65:D011S600<LiaS66<H011S600<LiaS66<H00QcW0R4800@Lib487>LQ
208U:@@YBP04;F/e[D8@BU82AS401D8@BU9BU5JeF]H00f<H00E[FVLiJeY[FVmk009g_@Qoo`04Kg]W
>F<HIcT2G_L2F]H01UJeDYAF]E:DCW=2408mk`04>Lhe[CF]=Jd2<H`2;F/00bU:9BTU:@058@P37>L0
1AS67>LHaQS67>L01AS60QcW00<HaQcW6<H041S600DDYAS66<HHaQBU00`HaP035:DHaQS600@HaP04
5:DHaQBU5:D66<H25:D36<H00aBU6<HHaP046<H00aBU6<HHaP025:D46<H01ABU6<HHaQS65:D02aS6
00HDYAcW6<HHaQBU6<H25:D46<H00aBU6<HDY@076<H25:D46<H00aBU6<I[FP06JeX10001JeX0000P
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaS6
5:DHaP8DY@PHaP8DY@8HaP045:DHaQS65:D86<H00aBU6<HHaP046<H00aBU6<HDY@096<H01ABU6<HH
aQS65:D02QS600<DYAS66<H00aS600@DYAS66<HDY@LHaP055:DHaQS66<HDY@076<H01ABU6<HHaQS6
5:D03aS60QBU4aS60QcW1QS611cW0aS61AcW124800<U:BU::DX00RU:00@]JbU:=Jdmk`=6<@04@Q16
<DicE[D2G_L2HaP2JeX2Kg/3Nmh8Ool01WNmJeYS66LiG_MF]@9JeP06E[EBU5:DDYA:DT8@13W>0SF]
00<aS2e[:DX00RU:0b4800<U:B488@P021cW0QS600<LiaS66<H02aS600@DYAS66<HDY@THaP8DY@HH
aP8Lia<HaP8DY@DHaP035:DHaQBU008HaP055:DHaQS66<HDY@0>6<H00aBU6<HHaP026<H00aBU6<HH
aP056<H00aBU6<HHaP056<H00aBU6<HHaP0<6<H02aBU6<HHaVmkJeY[NV]JKg][FV]j00000Fmk000F
JeX00`00JeY[FP04JeX25:D56<H00aBU6<HHaP0<6<H25:D66<H00aBU6<HDY@096<H27>L:6<H00aBU
6<HHaP056<H00aBU6<HHaP086<H25:D46<H00aBU6<HHaP036<H011BU6<HHaQS60QBU00<HaQBU5:D0
21S600DLiaS66<HHaQBU018HaP035:DHaQS600dHaP067>LHaQcW7>LHaQcW0QS61AcW0R480RDY1Be[
0S6<00Xe[Cg_?Nlmkd8@AS5:DUJeG_MS609[FP05Kg]cW7>LLiakgP0:Ool01W_NKg][FV<HG_MBU0=F
]@9BU09>L`04AS4icSW>>Lh3=Jd0136<;F/]Jbe[0bDY12480QcW00@Q21cW7>LLi`@HaP037>LHaQS6
00dHaP035:DHaQS600/HaP035:DHaQS6008HaP<DY@DHaP8DY@DHaPLDY@PHaP045:DHaQS65:DA6<H0
11BU6<HHaQcW0QS600<DYAS66<H02AS600HDYAS66<HDYAS65:D56<H01ABU6<HHaQS65:D00aS600<D
YAS65:D01f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]J001[FV]jJeY_Nf]JJgX00QS600<DYAS66<H00aS600<DYAS66<H03AS600<DYAS66<H01AS600<D
YAS66<H00QS600<DYAS66<H011S600<DYAS65:D01AS600HDYAS66<HHaQBU6<H25:D01AS65:DHaQS6
5:D03QS600<DYAS66<H011S600<DYAS66<H01AS600<DYAS66<H031S600<DYAS65:D07AS600<LiaS6
7>L01acW0R4800@Lib488@PQ208YBP8]J`03:DX]Jbe[008aS0<icP04@Q16<DicF]H4IcT2Li`00fmk
Liag_@0:Ool01W>LKg]_NfLiHaQF]@ABU004CW=BU4YB@Q02?Nl3=Jd2<H`3:DX29BT68@P011cW6<HL
iacW11S600<LiaS66<H02AS600PDYAS66<HLiaS65:DHaQBU41S60aBU31S600PDYAS66<HDYAS65:DH
aQBU4aS600@DYAS66<HDY@<HaP055:DHaQS66<HDY@036<H25:D:6<H00aBU6<HHaP0>6<H35:D26<H0
26]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP8HaP035:DHaQS6008HaP8DY@076<HD
YAS66<HDYAS65:D01aS600<DYAS66<H00QS600@DYAS66<HDY@hHaP035:DHaQS600HHaP8DY@<HaP03
5:DHaQS600LHaP035:DHaQS6008DY@036<HDYAS6008DY@DHaP035:DHaQS600<HaP<DY@HHaP035:DH
aQS600DHaP045:DHaQS66<H25:DK6<H011cW6<HLiacW0QS611cW0R4800@Lib488@PU:@8YBP8]J`8Y
BP03;F/aS3F]00<icP06@Q1:DTicE[ES66Li0Vmk0WNm0Vmk00=g_GooOol027oo0Vmk00A[FVLiF]IB
U09>L`9:DP9>L`08BU9243W>>LhaS3F]>Lhe[@8]J`<U:@HQ20@Li`LHaP037>LHaQS602<HaP8DY@DH
aP055:DHaQS65:DHaP025:D011S65:DHaQBU11S600<DYAS65:D00QS60QBU0QS600<DYAS66<H021S6
00<DYAS66<H00aS600<DYAS66<H00aBU1aS600<DYAS65:D03QS600<DYAS66<H011S600<DYAS6JeX0
1V]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[
FVmkJeY[NV]JKg/00QBU11S600<DYAS66<H021S600<DYAS66<H021S60aBU0QS600<DYAS66<H00QS6
00<LiaS66<H01aS600<DYAS66<H05QS611BU0aS600<DYAS66<H02AS600@DYAS65:DDY@LHaP035:DH
aQS600<HaP035:DHaQS600`HaP035:DHaQS600PHaP037>LHaQS600<HaP067>LHaQS66<HLiaS611cW
1B4800<U:BU:;F/01Re[00XaS3F]>LhicTHaCW=BU5JeHaQ_N`9g_@05Nmi_Nf]JLiag_@08Ool2Li`0
1FmkJeYS65kgE[D014ic0dYB00I6<Cg_>LhicS6<=Jd2<H`00be[:DXU:@029BT38@P37>L00b487>LL
i`037>L36<H01AcW6<HHaQS67>L01AS600<DYAS66<H03AS60QBU0QS600<DYAS66<H01AS600<DYAS6
6<H00aS60QBU1QS60QBU0aS60QBU00<HaQBU5:D02AS600<DYAS66<H01AS600@DYAS66<HDY@<HaP04
5:DHaQS65:D46<H00aBU6<HHaP025:DA6<H00aBU6<HHaP086<H026mkJeY[NV]JKg][FV]j0001Kg/0
01I[FP<000A[FP8HaP035:DHaQS6018HaP035:DHaQS6008HaP035:DHaQS600hHaP035:DHaQS600@H
aP035:DHaQS6008HaP035:DHaQS600`HaP095:DHaQS66<HDYAS65:DHaQBU00@HaP035:DHaQS600@H
aP035:DHaQS6024HaP8DY@8HaP037>LHaQS600@HaP037>LHaQS600<HaPHLi`<Q20059BTQ22DY:DX]
J`03<H`3;F/2=Jd2@Q001TYBDYABU5JeIcU_N`9cW005MkecW7>LLiag_@02Nmh5Ool02G_NKg]_Nf]J
IcUS65[FDYA:DP03CW<01DYB@Q16<DHa?Nl00SW>00<e[CW>=Jd00Re[0bU:00DU:B488@PLib4800DL
i`<HaP8Li`056<HLiaS66<HLi`0=6<H00aBU6<HHaP046<H00aBU6<HHaP0F6<H00aBU6<HHaP026<H0
0aBU6<HHaP026<H00aBU6<HHaP046<H011BU6<HHaQBU11S600<DYAS66<H02aS600<DYAS66<H00aBU
00<HaQBU6<H05aS600<DYAS66<H00aS600<DYF]JJeX016]J0P000F]J00007F]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX00QS600<DYAS66<H04QS6
00<DYAS66<H00QS600<DYAS66<H03QS600<DYAS66<H011S600<DYAS66<H00QS600<DYAS66<H031S6
00TDYAS66<HHaQBU6<HDYAS65:D011S600<DYAS66<H011S600<DYAS66<H08AS60QBU0QS600<LiaS6
6<H011S600<LiaS66<H00aS61QcW0b4800DU:B489BTYBRe[00<aS0<]J`8e[@924006BU9BU5:DE[EW
>Fmk0W>L00Eg_G>LLiacW7Nm009kgPEoo`09Nmi_NfmkJeYW>F<HF]IBU4YB00=>L`05BU9244HaAS4m
k`02>Lh00cF]>Lhe[@02;F/3:DX01BDY8@PQ21cW8@P01AcW0aS60QcW00DHaQcW6<HHaQcW00dHaP03
5:DHaQS600@HaP035:DHaQS601HHaP035:DHaQS6008HaP035:DHaQS6008HaP035:DHaQS600@HaP04
5:DHaQS65:D46<H00aBU6<HHaP0;6<H00aBU6<HHaP035:D00aS65:DHaP0G6<H00aBU6<HHaP036<H0
2ABUJgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP036<HDYAS601<HaP8DY@036<HD
YAS600@HaP8DY@<HaP035:DHaQBU00<HaP8DYA4HaP037>LHaQS600THaP<DY@<HaP035:DHaQS600<H
aP035:DHaQS600<DY@HHaP8DY@036<HDYAS600hHaP037>LHaQS600@HaP035:DHaQS600@HaP035:DH
aQS6008HaP067>LHaQS67>LHaQcW11S60QcW00@HaQcW7>LLi`<Q208U:@03:DX]Jbe[00D]J`03<H`e
[CW>00924006BU9F]E:DE[EW>G>L0WNm00EcW6mkKg]g_Fmk009g_@06NmioogooOomkgW>L0fLi00=[
FVLiF]H00U:D0UJe00EBU4YBAS56<D8@00@icP04=JdaS36<:DX49BT38@P00acW8@PQ20027>L011S6
7>LLiacW1AS600<LiaS66<H03AS600<DYAS66<H00QcW4aS600DDYAS65:DDYAS6008DY@036<HDYAS6
008HaP065:DHaQBU5:DHaQBU2AS600@DYAS66<HDY@HHaP045:DHaQS65:D36<H01ABU6<HHaQS65:D0
0QS60QBU0aS60aBU1aS600<DYAS66<H041S61f]J0@000F]J00007fmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/HaQBU00PHaP045:DHaQS65:DC6<H0
0aBU7>LLi`026<H011BU6<HHaQS60QBU0aS600<DYAS66<H00QBU0QS600<DYAS66<H011S600<DYAS6
6<H031S600<DYAS66<H01QS600<DYAS66<H01AS600@DYAS66<HDY@@HaP035:DHaQS600@HaP8DY@PH
aP035:DHaQS600HHaP045:DHaQS65:D76<H01AcW6<HHaQS67>L00QS61AcW00@Q21cW7>LQ208U:@03
:DX]Jbe[00<]J`05:DX]Jc6<=JdicP02@Q001TicF]IJeUJeHaQ[FP9_N`03MkecW6Li009[FP9_N`03
Liag_G>L009cW007JeYW>FLiIcU_NfLiF]H00UJe0e[F00A>LdHaAS4mk`DicP04=JdaS2U::DX49BT3
8@P77>L011S67>LLiacW0aS600HDYAS67>LLiaS67>L96<H00aBU6<HHaP076<H25:D26<H00aBU6<HH
aP0;6<H011BU6<HHaQS61QBU00<HaQBU6<H00aS600@DYAS66<HHaP<DY@8HaP045:DHaQS66<H25:D2
6<H011BU6<HHaQS60QBU11S600HDYAS65:DDYAS65:DB6<H35:D:6<H02ABUKg][FV]jJeY_Nf]JJgX0
0001Kg/001I[FP03001[FV]J00A[FP<HaP8DY@dHaP8DY@HHaP8DY@THaP035:DHaQS6008DY@<HaP8D
Y@<HaP8DY@PHaP035:DHaQS600DHaP8DY@036<HDYAS600LHaP<DY@<HaP035:DHaQBU008DY@/HaP03
5:DHaQS601LHaP8Li`THaP037>LHaQcW00<HaP<Li`036<HLiacW00<Q20039BTYBRe[00@]J`8YBP04
;F/aS3F]>Lh2@Q001U:DG_MF]EJeHaQ[FP9_N`9cW09[FP03IcU_Nfmk00A[FP9W>@9S6006JeY_NfLi
E[EBU4ic0e[F00=>LdHa@Q000Sg_0cW>00<e[C6<;F/00RU:0RDY124800<Lib488@P011cW11S60QcW
1QS600<LiaS66<H04QS60QBU0QS600@DYAS66<HHaP8DYA4HaP055:DHaQBU6<HDY@0E6<H01ABU6<HH
aQS65:D00aS600@DYAS66<HHaP8DY@8HaP8DY@dHaP045:DHaQBU5:D;6<H7JeX10001JeX0000MJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP066<H0
0aBU6<HHaP066<H00aBU6<HHaP086<H27>L46<H01ABU6<HHaQS65:D01aS600<DYAS66<H05aS600<D
YAS66<H011S60QBU00<HaQcW6<H00aS600@DYAS66<HDY@DHaP035:DHaQS600LHaP8DY@THaP035:DH
aQS600@HaP035:DLiaBU008HaP037>LHaQS6008HaP035:DHaQS600XHaP037>LQ21cW008Li`<Q208Y
BPH]J`06:DX]Jbe[<H`e[Cg_0T8@00I>LekgF]IF]EkgIcT5Kg/01F]JIcUW>FLiJeX00f<H00YNmf<H
IcUW>F]JKg]S65:DBU9BU09Nm`04F]I>LdYBAS42?Nl2>Lh00cg_=JdaS002;F/012DY8@PQ22482acW
00DHaQcW7>LHaQcW00PHaP8Li`@HaP@DY@PHaP035:DHaQS600PHaP035:DHaQS600THaP035:DHaQS6
00HHaP035:DHaQS600LHaP045:DHaQS65:D:6<H25:D96<H00aBU6<HDY@066<H00acW6<HDY@0:6<H0
0aBU6<HHaP086<H026]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP036<HDYAS600DH
aP035:DHaQS600/HaP037>LHaQS600@HaP035:DHaQBU00@HaP035:DHaQS600LDY@036<HDYAS600@H
aP045:DHaQS67>L56<H00acW5:DHaP026<H00aBU6<HHaP096<H00aBU6<HHaP046<H00aBU6<HHaP05
6<H25:D56<H00aBU6<HHaP026<H00acW5:DHaP0:6<H00aBU6<HHaP0;6<H27>L;6<H47>L38@P01BU:
;F/]Jbe[:DX00be[00<YBRe[;F/00SF]00TicT8@@Q1BU5[FE[EJeUkgHaP00VLi00=[FVmkJeX01F<H
0e[F0Ukg0V<H00IW>F<HE[EBU4icE[D2F]H015JeCW=6<DHa13g_00<icSF]<H`00RU:00<U:B488@P0
0R4800<Lib488@P01AcW0aS60QcW00<HaQcW6<H021S600<LiaS66<H00QS600<DYAS65:D00aS60QBU
3AS60QBU11S600<DYAS66<H01QS600<DYAS66<H00QS60QBU0QS60QBU00DHaQBU6<HHaQBU008HaP03
5:DHaQS6008HaP<DY@036<HDYABU00LHaP035:DHaQS600<HaP035:DHaQS6010HaP055:DHaQS66<HD
Y@086<H00aBU6<I[FP06JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`036<H35:D46<H00aBU6<HHaP046<H00aBU6<HHaP036<H0
0aBU6<HHaP046<H011BU6<HDYABU0aS600<DYAS65:D05aS600<DYAS66<H00QS600DDYAS66<HHaQBU
008Li`036<HDYAS601PHaP045:DHaQBU5:D;6<H00aBU6<HHaP036<H011BU6<HHaQS60aBU21S600<L
iaS66<H00QS61QcW0R480RDY00<YBRe[;F/00RU:00<]JbU::DX00be[00<aS3F]>Lh00T8@00A>Le:D
E[EF]@9JeP9S6003IcU[FVLi00ANm`04E[EJeU[FF]H6G_L3DY@3E[D014icBU9:DTHa13g_00<icSF]
;F/00RU:00<U:B488@P0124800<Lib488@P00acW00<HaQcW7>L00QcW0QS600<LiaS66<H06QS600<D
YAS66<H02AS611BU2QS600@DYAS65:DDY@LHaP8DY@<HaP035:DHaQBU00DHaP8DY@<HaP035:DHaQS6
008HaP045:DHaQS65:D46<H01AcW6<HHaQS65:D02AS600<DYAS66<H02aS600TDYFmkJeY[NV]JKg][
FV]j00000Fmk000FJeX00`00JeY[FP04JeX26<H011BU6<HHaQBU0QS600<DYAS65:D02AS600<DYAS6
6<H03AS600<DYAS66<H00aS60QBU1QS60QBU11S600<DYAS66<H031S600<DYAS66<H00aS600<DYAS6
6<H03QS600<DYAS66<H011S60QBU1QS600<LiaS65:D06QS60QcW1AS611cW00DQ21cW8@PQ22DY008]
J`HYBP05;F/aS36<=JdicP02@Q001TYBCW=BU5[FE[EJeP9Nm`9S6003IcUS65[F00=JeP03E[EJeU[F
009Nm`=S6003F]I>Ldic009F]@9BU006CW=6<DYB@Q0mkd8@0Sg_0SF]00<]JbU::DX00bDY1B481AcW
00<HaQcW6<H02aS600<DYAS66<H01QS600@LiaS66<HHaP8DY@LHaP035:DHaQBU01<HaP8DY@<HaP03
5:DHaQS6008DY@8HaP035:DHaQBU008DY@8HaP8DY@THaP035:DHaQS600@HaP8DYADHaP055:DHaQS6
6<HDY@026<H00aBU6<HHaP046<H00aBUJeY[FP05JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP026<H011BU6<HHaQBU0QS600<D
YAS65:D02AS600<DYAS66<H03AS600<DYAS66<H00aS60QBU1QS60QBU11S600<DYAS66<H031S600<D
YAS66<H00aS600<DYAS66<H03QS600<DYAS66<H011S60QBU1QS600<LiaS65:D06QS60QcW1AS611cW
00DQ21cW8@PQ22DY008]J`HYBP05;F/aS36<=JdicP02@Q001TYBCW=BU5[FE[EJeP9Nm`9S6003IcUS
65[F00=JeP03E[EJeU[F009Nm`=S6003F]I>Ldic009F]@9BU006CW=6<DYB@Q0mkd8@0Sg_0SF]00<]
JbU::DX00bDY1B481AcW00<HaQcW6<H02aS600<DYAS66<H01QS600@LiaS66<HHaP8DY@LHaP035:DH
aQBU01<HaP8DY@<HaP035:DHaQS6008DY@8HaP035:DHaQBU008DY@8HaP8DY@THaP035:DHaQS600@H
aP8DYADHaP055:DHaQS66<HDY@026<H00aBU6<HHaP046<H02ABUJgY[FVmkJeY[NV]JKg/00001JgX0
01I[FP03001[FV]J00A[FPHHaP8DY@8HaP035:DHaQS600/HaP035:DHaQS600LHaP<DY@056<HDYABU
5:DHaP025:D36<H25:D00aS65:DDY@056<H011BU6<HHaQS60QBU0QS60QcW0aS60QBU21S600@DYAS6
6<HDY@8HaP035:DHaQBU00`HaP035:DHaQS600/HaP035:DHaQBU00PHaP035:DLiaS600DHaP047>LH
aQS66<H27>L76<H011cW8@PLib480bDY0bU:00<]JbU::DX00RU:0Re[00<aS3F]>Lh00T8@00I6<E:D
DYABU5JeF]H2G_L4HaP8G_L3F]H015kgE[E>LdYB15:D00A:DTHaAS52408mk`<icP03<H`]JbU:008U
:@@Q20HLi`056<HLiaS67>LHaP037>LH6<H011BU7>LHaQS60aBU00<LiaBU6<H00QS60aBU00<HaQBU
6<H01AS600DDYAS66<HHaQBU00PHaP8DY@046<HDYABU6<H45:D26<H00aBU6<HDY@026<H00aBU6<HH
aP025:D46<H011BU6<HDYABU1QS600<DYAS66<H02AS60QBU0QS600<DYAS66<H00aS600<DYAS65:D0
0aS600<DYAS66<H01f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00aS60QBU0aS60QBU2aS600<DYAS65:D03aS600<DYAS66<H0
0QBU4QS600<DYAS66<H02aS600<DYAS66<H011S600DDYAS66<HHaQBU00LHaP035:DHaQS6010HaP03
5:DHaQS600lHaP035:DHaQS600DHaP037>LHaQcW00<HaP8Li`<Q20@U:@03:DXU:BU:00<YBP8]J`04
:DXe[CF]>Lh2@Q001DHaDYABU5:DE[D01Ukg00=S65kgHaP00V<H0Ukg00=JeUJeG_L00Ukg00AJeUJe
CW=:DPI>L`03AS4mkcg_00<icP05=Jd]Jbe[:DXU:@058@P37>L28@P37>L96<H00acW6<HHaP0H6<H0
0aBU6<HHaP0O6<H25:D01AS65:DDYAS65:D02aS600@DYAS66<HHaP8DY@036<HDYABU008DY@036<HD
YAS600<HaP8DY@DHaP8DY@XHaP045:DHaQS65:D56<H026mkJeY[NV]JKg][FV]j0001Kg/001I[FP<0
00A[FPDHaP8DY@036<HDYAS600XHaP035:DHaQS600/HaP8DY@@HaP035:DHaQBU00<DYA8HaP<DY@03
6<HDYABU00XHaP035:DHaQS600<HaP035:DHaQS600<HaP035:DHaQS600HHaP035:DHaQS600LHaP03
5:DHaQS6008HaP035:DHaQS600HHaP037>LHaQS6014HaP@Li`8Q20HU:@DYBP<]J`09=JdicT8@@Q1:
DU:DF]IF]E[F00ANm`AJeP9Nm`06HaQNme[FF]INmf<H0Ukg00IJeUJeCW=>Le:DCW<2BU800dHaBU8m
k`02?Nl01CW>=Jde[C6<;F/00RU:0RDY124800<Lib487>L00acW00DHaQcW6<HHaQcW00@HaP035:DH
aQS600PHaP035:DHaQS600THaP8DY@@HaP<DY@8HaP035:DHaQS600dHaP035:DHaQS6008DY@046<HD
YAS66<H25:D00aS65:DHaP026<H25:D26<H011BU6<HHaQS60QBU1QS60QBU1AS60aBU41S600@DYAS6
6<HDY@DHaP8DY@@HaP035:DHaQS600I[FP80005[FP0001i[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:D26<H35:D26<H00aBU6<HDY@0;6<H00aBU
6<HDY@056<H011BU6<HDYAS60QBU3AS600<DYAS66<H01QS60QBU00<HaQBU6<H00QS60QcW1AS600<D
YAS66<H00QBU11S60aBU00DHaQBU6<HHaQBU00DHaP035:DHaQS600hHaP045:DHaQS65:DH6<H37>L0
11S67>LLiaS60QcW0R4800@U:B488@PQ208U:@03:DXU:BU:00<YBP8]J`03<H`e[CW>00924003BU9B
U5[F009JeP=Nm`9F]@EJeP03G_MJeU[F00=S609Nm`03E[EBU4YB009BU007CW=6<DHaAS5243W>?Nl0
0SW>0SF]0Re[00@YBRDY8@PQ208U:@058@PLib487>LQ20047>L00aS67>LLi`086<H00aBU6<HHaP03
6<H00aBU6<HHaP066<H00aBU6<HHaP036<H00aBU6<HHaP025:D36<H55:D00aS65:DHaP056<H00aBU
6<HHaP036<H25:D01QS65:DHaQBU6<HDY@HHaP045:DHaQS65:D36<H011BU6<HHaQS60aBU00<HaQBU
5:D00QS600<DYAS66<H01aS600@DYAS66<HHaP8DY@<HaP035:DHaQS600@HaP035:DHaQS600@HaP8D
Y@DHaP0:5:DHaV]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FPHHaP035:DHaQBU00DH
aP8DY@@HaP8DY@<HaP<DY@@HaP045:DHaQS66<H25:DB6<H00aBU6<HHaP066<H25:D@6<H011cW5:DH
aQBU1AS600<DYAS65:D021S600LLiaS66<HHaQBU6<HDY@096<H35:D00acW5:DHaP026<H00aBU6<HH
aP096<H47>L36<H37>L28@P79BT4:DX3;F/02SF]>Lhmkcg_AS5BU5[FG_MS65kg15[F0ekg0U[F0UJe
00AJeV<HIcUNm`9F]@9>L`04AS5BU5:DCW<2AS400d8@?Nle[@02>Lh01CF]<H`aS2e[:DX00RDY1R48
1QcW00@HaQcW6<HLi`PHaP035:DHaQS600@HaP035:DHaQS600@HaP8DY@DHaP8DY@<HaP035:DHaQBU
00<HaP8DY@@HaP8DY@8HaP035:DHaQS600HHaP035:DHaQS6008HaP8DY@8HaP8DY@046<HDYABU6<H3
5:D66<H25:D00aS65:DHaP056<H00aBU6<HHaP076<H25:DA6<H00aBU6<HHaP066<H7JeX10001JeX0
000NKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_
NacW0QBU11S600<DYAS66<H03aS600<DYAS66<H01QS60QBU00<HaQBU5:D01aS600<DYAS66<H01AS6
00@DYAS65:DHaP<DY@@HaP8DY@8HaP8DY@@HaP@DY@@HaP8DYA0HaP035:DHaQS600DHaP035:DHaQS6
01`HaPDLi`038@PLiacW00<Q208U:@038@PU:BDY008U:@DYBP04<H`]JcF]>Lh2?Nl00dHaCW=JeP02
IcT00ekgF]IJeP07F]H2E[D015kgIcUW>E[F0U:D0TYB00A>Le:DDYA:DP96<@05@Q0icSF]=JdicP03
<H`00be[:DXU:@029BT68@P47>L00aS67>LLi`037>L66<H00aBU6<HHaP0?6<H25:D76<H00aBU6<HH
aP036<H25:D56<H00aBU6<HHaP026<H25:D26<H25:D36<H00aBU6<HDY@066<H00aBU6<HHaP046<H0
11BU6<HDYABU11S600@DYAS65:DDY@<HaP8DYA@HaP035:DHaQS600@HaP045:DHaQS65:D46<H02QBU
6<I_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX00aS65:DHaP025:D011S65:DHaQBU
0QS60QBU2aS600<DYAS66<H011S60QBU00@HaQBU5:DDY@8HaP035:DHaQS600<HaP045:DHaQS65:D<
6<H65:D00aS65:DHaP056<H00aBU6<HHaP0A6<H00aBU6<HHaP096<H00aBU6<HHaP0>6<H00acW6<HH
aP096<H27>L26<H87>L38@P39BT012489BTU:BDY12U:0be[00PaS3F]?NlmkdYBCW=F]F<H0VLi0V<H
1Ekg0U[F00IF]E[FHaQW>EkgDY@2BU82CW<015:DE[E>LdHa0T8@00HmkcW>>Lhe[CW>=Jd2<H`00be[
:DXU:@029BT38@P00acW8@PLi`057>L00aS67>LLi`096<H00aBU6<HHaP096<H25:D00acW6<HHaP02
5:D26<H25:D36<H00aBU6<HDY@026<H00aBU6<HHaP026<H011BU6<HDYABU0QS600@DYAS66<HDY@TH
aP<DY@036<HDYAS600@HaP8DY@LHaP8DY@036<HDYAS6008HaP035:DHaQS600<HaP8DY@036<HDYAS6
01DHaP055:DHaQS65:DHaP025:D36<H7JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQS65:DHaP8DY@046<HDYAS65:D26<H25:D;
6<H00aBU6<HHaP046<H25:D011S65:DDYABU0QS600<DYAS66<H00aS600@DYAS66<HDY@`HaPHDY@03
6<HDYAS600DHaP035:DHaQS6014HaP035:DHaQS600THaP035:DHaQS600hHaP037>LHaQS600THaP8L
i`8HaPPLi`<Q20<U:@048@PU:BDY9BT4:DX3;F/0236<=Jdmkcg_BU9>LeJeHaP2IcT2HaP5G_L2F]H0
1UJeF]IS66LiG_MBU09:DP9>L`04DYAF]DicAS42@Q001Sg_>LhicSF]>Lhe[@8aS003;F/YBRDY008U
:@<Q20037>LQ21cW00DLi`036<HLiacW00THaP035:DHaQS600THaP8DY@037>LHaQS6008DY@8HaP8D
Y@<HaP035:DHaQBU008HaP035:DHaQS6008HaP045:DHaQBU5:D26<H011BU6<HHaQBU2AS60aBU00<H
aQBU6<H011S60QBU1aS60QBU00<HaQBU6<H00QS600<DYAS66<H00aS60QBU00<HaQBU6<H05AS600DD
YAS66<HDYAS6008DY@<HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J0aS60QBU
4AS600<DYAS66<H00aS60aBU00<HaQBU6<H00aBU11S611BU21S600DDYAS65:DDYAS6008DY@<HaP07
5:DHaQS66<HDYAS65:D02QS600DDYAS66<HHaQBU00LHaP8DY@<HaP8DY@<HaP065:DHaQS66<HDYAS6
0QBU11S600<LiaS66<H05aS600<LiaS67>L011cW0R4800<U:B489BT01bDY0bU:0be[00XaS3F]?Nlm
kdHaCW=JeV<HIcU[FP9W>@9Nm`9S6003F]IF]EJe009F]@04G_MS65[FDY@3BU801DicIcUg_E:D@Q00
0Sg_00=243g_>Lh00SF]0c6<00@]JbU::DXU:@@Q20@Li`@HaP037>LHaQS600LHaP037>LHaQS6008H
aP035:DHaQcW00XHaP<DY@DHaP8DY@<HaP035:DHaQS600<DY@@HaP035:DHaQS600/HaP035:DHaQS6
00LHaP8DY@056<HDYABU6<HDY@076<H25:D46<H25:D?6<H01ABU6<HHaQS65:D01aS600<DYAS65:D0
1AS60aBU1f]J0@000F]J00007VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]J001[FVmkJeY[NV]JKg/DY@8HaP8DY@<HaP035:DHaQS600<HaP035:DHaQS600<HaP@DY@lHaPDD
YA<HaP035:DHaQS600HHaP055:DHaQS66<HLi`026<H00aBU6<HHaP026<H011BU6<HHaQBU1QS600<D
YAS66<H00QBU11S600<DYAS66<H00QBU00@HaQBU6<HHaP8DY@hHaP8Li`065:DHaQS66<HLiaS60QcW
00<HaQcW6<H01QcW0R4822DY1RU:0Re[00PaS3F]@Q1244HaDYAJeV<H0V]J0fLi0V<H00=Nme[FE[D0
0eJe00AJeUJeCW=:DP=6<@04CW=cW7ooE[D3@Q02?Nl2=Jd4<H`2;F/39BT38@P27>L00aS67>LLi`03
7>L96<H00acW6<HLi`076<H00aBU6<HHaP046<H00aBU6<HHaP025:D56<H35:D36<H25:D26<H25:D4
6<H25:D46<H00aBU6<HHaP046<H25:D66<H01ABU6<HHaQS65:D00QS60QBU00@HaQBU5:DDY@8HaP03
5:DHaQBU008DY@036<HDYAS6008DY@<HaP@DY@@HaP035:DHaQS600hHaP8DY@@HaP0;5:DHaQBUKg][
FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FPhHaP8DY@056<HDYAS66<HDY@076<H00aBU
6<HHaP0>6<H00aBU6<HHaP086<H45:D00aS65:DHaP046<H01ABU6<HHaQS65:D01aS600@DYAS65:DD
Y@dHaP035:DHaQS6014HaP035:DHaQBU00dHaP037>LHaQcW00DHaP037>LHaQcW008Li`<Q20PU:@03
:DXU:BU:00@YBP8]J`09<H`e[Cg_?Nm6<DYBDYANmf<H00=[FP05Kg]g_F]JIcUJeP02E[D2F]H00eJe
DYA:DP05AS400e:DE[E:DP05?Nl00cW><H`aS003<H`2:DX29BT48@P47>L00aS67>LLi`0=6<H01ABU
6<HHaQS65:D00QS60QBU31S60aBU1QS600@DYAS65:DDY@@HaP8DY@HHaP035:DHaQS600LHaP<DY@03
6<HDYAS600@HaP045:DHaQBU6<H45:D36<H011BU6<HDYABU0QS600<DYAS66<H05aS600<DYAS66<H0
0QBU00<HaQBU6<H011S600<DYAS6JeX01V]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX00QS611BU1aS600DDYAS66<HHaQBU00HH
aP035:DHaQS600LHaP<DY@LHaP035:DHaQS600DHaP045:DHaQS65:D56<H00aBU6<HHaP056<H25:D=
6<H00aBU6<HHaP0>6<H00aBU6<HHaP025:D36<H00aBU6<HHaP066<H00aBU6<HHaP0?6<H97>L48@P0
1BDY8@PU:BDY8@P00RDY1bU:00@aS2e[=Jde[@8mk`06@Q1:DTicE[ENmf<H0V]J00EkgWooOomW>E[F
009F]@06F]IF]EJeCW=:DT8@0dHa00A244HaAS5240<mk`04>LhmkcW>=Jd2<H`2;F/2:DX29BT28@P5
7>L46<H00acW6<HHaP0F6<H25:D76<H25:D46<H25:D00aS65:DHaP036<H011BU6<HHaQS61ABU0QS6
00DDYAS66<HHaQBU00@HaP035:DHaQS600<HaP035:DHaQS6008HaP055:DHaQS66<HDY@036<H25:D0
0aS65:DHaP0=6<H00aBU6<HHaP096<H00aBU6<HHaP056<H25:D00aS65:DHaP056<H02ABUJgY[FVmk
JeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FPTHaP8DY@HHaP075:DHaQS66<HDYAS65:D01QS6
00<DYAS66<H03QS600<DYAS65:D00QBU0QS600@DYAS65:DDY@<HaP035:DHaQBU00<DY@056<HDYABU
6<HDY@036<H00aBU6<HHaP0>6<H00aBU6<HHaP076<H01ABU6<HHaQS65:D02aS600<DYAS66<H00QS6
00<DYAS66<H01aS600<LiaS66<H00acW0aS611cW00<Q22DY9BT01bDY00<YBRDY:DX00RU:12e[00@a
S3F]?Nlmk`924004BU9BU5:DF]H2HaP017_NOomoofLi0e[F00EF]E:DCW=:DTHa00M24003?NlicSW>
00@icP03=JdaS36<008]J`8YBP039BTYBRDY00<Q20DLi`@HaP047>LHaQS67>LJ6<H00aBU6<HHaP07
6<H00aBU6<HDY@026<H01ABU6<HDYAS65:D01AS600<DYAS66<H011S600<DYAS66<H01QS600<DYAS6
6<H00QBU0QS600<DYAS66<H02AS60aBU00<HaQBU6<H031S600<DYAS66<H00QS60QBU00<HaQBU6<H0
1AS60QBU21S600<DYF]JJeX01F]J0@000F]J00005VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]J0`0000E[FV]jJeY_NaBU00@HaP037>LHaQS600PHaP@DY@8HaP045:DHaQS6
5:D26<H01QBU6<HHaQS65:DHaP8DY@036<HDYABU00HHaP045:DHaQS65:D56<H011BU6<HHaQS60QBU
11S600<DYAS66<H00aS600<DYAS66<H00QcW00<DYAS66<H00QS600<DYAS66<H021S600<DYAS66<H0
0QS600<DYAS66<H00aS600<DYAS66<H011S600<DYAS66<H05aS60acW00<HaQcW6<H01AcW0R480bDY
0R4800@U:BU::DXYBP@]J`03:DX]Jbe[008aS007=Jdmkcg_?Nm244HaBU800U:D0Ukg00=_NgooLi`0
0Ukg0U[F00EBU4icBU96<D8@00Lmk`04>LhmkcF]=Jd2>Lh2=Jd2<H`2;F/012U:9BTU:BDY0R480QcW
00DQ21cW7>LLiaS6008Li`@HaP037>LHaQS6008HaP037>LHaQS600LHaP035:DHaQBU00@HaP035:DH
aQS600dHaP035:DHaQS600<DY@`HaP<DY@066<HDYAS65:DHaQBU0QS60QBU1AS60QBU0QS600<DYAS6
6<H00QS600@DYAS65:DDYALHaP<DY@8HaP045:DHaQBU5:D36<H00aBU6<HHaP076<H01VmkJeY[NV]J
Kg][FP80005_N`005V]J00<006]JJeX016]J0aS60QcW00DHaQcW6<HHaQBU00HHaP035:DHaQS600/H
aP035:DHaQS600HHaP035:DHaQBU00HHaP035:DHaQS6008HaP055:DHaQS65:DHaP035:D86<H011BU
6<HHaQBU0QcW0aS600<DYAS66<H01AS600<DYAS66<H00QS600<DYAS66<H00QS600<DYAS66<H011S6
00<LiaS66<H011S600<DYAS66<H01aS60QBU00<HaQBU6<H02AS600<LiaS66<H00aS61QcW0R480RU:
0bDY12U:22e[0S6<0cW>00E6<DicBU9:DTic009F]@05IcU_NfLiIcUNm`03E[D014YBAS52448@1cg_
13W>13F]00<aS2e[:DX00RU:0bDY0R481AcW00<HaQcW6<H03QS600DDYAS66<HHaQBU018HaP035:DH
aQS600DHaP@DY@056<HDYAS65:DHaP035:D:6<H00aBU6<HHaP026<H25:D26<H01aBU6<HHaQS65:DH
aQBU00HHaP8DY@036<HDYAS600lHaP8DY@THaP035:DHaQS600`HaP035:DHaV]J00I[FP40005[FP00
01e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j
00<HaP8Li`056<HLiaS66<HDY@066<H00aBU6<HHaP0;6<H00aBU6<HHaP066<H00aBU6<HDY@066<H0
0aBU6<HHaP026<H01ABU6<HHaQBU6<H00aBU21S600@DYAS66<HDY@8Li`<HaP035:DHaQS600DHaP03
5:DHaQS6008HaP035:DHaQS6008HaP035:DHaQS600@HaP037>LHaQS600@HaP035:DHaQS600LHaP8D
Y@036<HDYAS600THaP037>LHaQS600<HaPHLi`8Q208YBP<U:@@YBPP]J`8aS0<icP05AS5>LdYBBU9>
L`02E[D01FLiKg]W>FLiG_L00eJe00A:DTHa@Q1240Lmk`@icP@e[@03<H`]JbU:008YBP<U:@8Q20DL
i`036<HLiaS600hHaP055:DHaQS66<HDY@0B6<H00aBU6<HHaP056<H45:D01AS65:DHaQBU6<H00aBU
2QS600<DYAS66<H00QS60QBU0QS600LDYAS66<HHaQBU6<HDY@066<H25:D00aS65:DHaP0?6<H25:D9
6<H00aBU6<HHaP0<6<H02QBU6<I[NV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04JeX@6<H2
5:D36<H011BU6<HHaQBU3AS60QBU0QS60QBU00<HaQBU5:D00QS600DDYAS66<HHaQBU008HaP8DY@8H
aP035:DHaQS600HHaP035:DHaQS600PHaP045:DHaQS65:D46<H35:D46<H00aBU6<HHaP0>6<H25:D3
6<H00aBU6<HHaP0C6<H77>L28@P00bDY:DXU:@04:DX2;F/00bU:;F/]J`05;F/2<H`3=Jd01Cg_AS5:
DTHaBU800Tic15Je0e[F00AF]DicAS5240Hmk`924004?NlicSW>>Lh2=Jd2<H`2;F/3:DX00bDY8@PQ
20028@P27>L00b487>LLi`056<H01AcW6<HHaQS67>L00QS600<LiaS66<H00QS600<DYAS66<H031S6
00<DYAS66<H01AS60aBU1QS60QBU00<HaQBU5:D00QBU11S60QBU2aS60QBU2AS600<DYAS66<H011BU
0QS60aBU3QS600<LiaS66<H01AS600<DYAS66<H021S60aBU0QS600HDYAS66<HHaQBU6<H7JeX10001
JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]j
JeY_N`036<H25:D56<H00aBU6<HHaP046<H011BU6<HHaQBU2aS600@DYAS65:DDY@8HaP045:DHaQBU
5:D76<H011BU6<HHaQBU11S600<DYAS66<H00QS600<DYAS65:D011S600@DYAS65:DHaP8DY@036<HD
YAS601DHaP047>LHaQS65:D;6<H011BU6<HHaQBU2AS600<DYAS66<H00QS600@LiaS66<HHaP<Li`03
6<HLiacW00<Li`048@PLib489BT2;F/6:DX2;F/2<H`5;F/4<H`013W>?Nm2448@1TYB00=>Le:DDY@0
0U:D00=>LdHa@Q000T8@1Sg_0SW>0cF]0S6<1Be[00@YBRDY9BTU:@8Q20@Li`lHaP037>LHaQS6010H
aP035:DHaQS600@HaP<DY@HHaP<DY@8HaP035:DHaQBU008HaP035:DHaQS600<HaP8DY@@HaP8DY@<H
aP8DY@036<HDYAS600DHaP035:DHaQS600<HaP035:DHaQBU00<DY@036<HDYAS6008HaP035:DHaQS6
00@HaP@Li`@HaP<DYA4HaP8DY@08Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J1AS6
00@DYAS66<HHaP8DY@DHaP@DY@046<HDYABU6<H25:D;6<H01ABU6<HHaQBU6<H00aBU0aS600DDYAS6
6<HHaQBU00<HaP037>LHaQS6008HaP8DY@036<HDYAS600<HaP035:DHaQS600HHaP035:DHaQS600@H
aP8DYBTHaP8DY@056<HLiaS66<HLi`046<H37>L011S67>LLiacW0R4800<U:BU::DX01RU:00<]JbU:
;F/00Re[00@aS2e[;F/]J`@aS004=JdicSW>?Nl3AS42@Q02AS401dYBCW=>LdYBAS5244Ha00Lmk`8i
cP@e[@03<H`]Jc6<00@]J`03<H`]JbDY00DQ20DLi`hHaP045:DHaQBU5:D:6<H25:D46<H00aBU6<HH
aP076<H25:D36<H25:D46<H25:D26<H00aBU6<HHaP036<H011BU6<HHaQBU1AS600@DYAS65:DDY@PH
aP8DY@<HaP035:DHaQS600HHaP035:DHaQBU008DY@<HaP8Li`LHaP8DY@066<HDYABU5:DHaQBU2aS6
0QBU1f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
001[FV]jJeY_Nf]JJgX01QS600DDYAS66<HHaQBU00<HaP035:DHaQBU00/HaP065:DHaQBU6<HDYAS6
0QcW00HDYAS66<HHaQBU6<H25:D26<H011BU6<HHaQBU0aS600<DYAS66<H011S60QBU0QS600<DYAS6
6<H021S600<DYAS66<H02aS60QBU6QS600<DYAS66<H02aS611cW00<HaQcW7>L021cW0R480RDY00<Y
BRDY9BT01RU:0Re[00<YBRe[;F/00be[136<00@e[CW>>Lhmk`9240<mk`05@Q16<DYBBU96<@03@Q04
?Nl00cW>?NlicP02>Lh3<H`4;F/00bU:<H`]J`03;F/29BT28@P37>L00aS67>LHaP0U6<H55:D26<H0
11BU6<HHaQS60QBU00DHaQBU6<HHaQBU00LHaP035:DHaQS600<HaP035:DHaQS600<DY@DHaP8DY@PH
aP045:DHaQBU5:D26<H35:D76<H00aBU6<HHaP076<H00aBU6<HDY@036<H00aBU6<HHaP026<H25:D5
6<H35:D02aS65:DDYF]jJeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX;6<H25:D:6<H0
0aBU6<HHaP0;6<H01ABU6<HHaQS65:D03aS600DDYAS66<HHaQBU00<HaP8DY@DHaP<DY@036<HDYAS6
008HaP035:DHaQS600PHaP055:DHaQS66<HDY@056<H00aBU6<HHaP046<H00aBU6<HHaP0;6<H00acW
6<HHaP056<H01QcW6<HLib487>LHaPTLi`<Q20<U:@XYBP03;F/YBRe[00D]J`@aS08icP8e[@04>Lhe
[CF]?Nl3AS401Sg_@Q12448@?NlicP<mk`8icP03=JdaS36<00<aS08]J`03:DX]Jbe[008]J`<YBP03
9BTQ2248008Q20047>LHaQcW7>L;6<H00acW6<HHaP026<H00aBU6<HHaP0D6<H35:D26<H00aBU6<HH
aP0<6<H00aBU6<HHaP066<H011BU6<HHaQBU0QS600<DYAS66<H00QS60QBU2aS600<DYAS66<H00QBU
0QS600@DYAS66<HHaP8DY@@HaP035:DHaQS600@HaP045:DHaQS66<H25:D:6<H00aBU6<HHaP026<H0
0aBU6<I[FP06JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeX006]JKg][FV]jJeY_N`086<H27>L46<H011BU6<HHaQS60QBU0aS600<DYAS66<H00QBU31S6
00@DYAS65:DDY@8HaP035:DHaQBU00PHaP045:DHaQS65:D86<H00aBU6<HHaP0=6<H011BU6<HDYAS6
11BU00@HaQBU6<HDYA8HaP035:DHaQBU00HHaP045:DHaQS66<H27>L011BU7>LLiacW11S611cW00<Q
21cW7>L012480RDY2RU:2Re[0c6<1SF]00DicSg_?NlmkcW>00<mk`HicP8e[@8aS0<]J`03:DX]Jbe[
00@]J`05:DXU:BDY9BTQ20067>L00aS67>LHaP046<H00acW6<HHaP076<H00aBU7>LLi`025:D:6<H0
0aBU6<HHaP026<H00aBU6<HDY@046<H25:D26<H01QBU6<HHaQBU6<HDY@8HaP065:DHaQS65:DHaQBU
1QS600<DYAS66<H00QS60QBU4AS600@DYAS66<HHaP8DY@036<HDYABU00LHaP035:DHaQS600HHaP8D
Y@<HaP045:DHaQBU5:D36<H00aBU6<HHaP086<H026mkJeY[NV]JKg][FV]j0001Kg/001I[FP03001[
FV]J00A[FP<HaP8DY@@HaP037>LHaQS6008HaP035:DHaQS601LHaP035:DHaQBU008DY@DHaP8DY@@H
aP035:DHaQS600<HaP035:DHaQBU00HHaP035:DHaQS6008HaP055:DHaQBU6<HDY@026<H011BU6<HH
aQBU1QS600<DYAS66<H021S60QBU11S600<DYAS66<H00QS600<DYAS66<H01aS600<DYAS66<H01AS6
00<LiaS66<H00QS60QcW00<Q21cW8@P00QcW124812DY0bU:0Re[12U:00<]JbU::DX01be[1S6<0cF]
1SW>00<e[CW>>Lh013F]0S6<00@]Jc6<<H`aS08]J`04:DX]JbU::DX39BT01B489BTU:B487>L00R48
0QcW00<HaQcW6<H01AS600<DYAS66<H00aS600<DYAS65:D01aS600<DYAS66<H00QS600@DYAS65:DD
YA0HaP@DY@036<HDYAS6008HaP8DY@036<HDYAS600LHaP8DY@8HaP@DY@036<HDYAS600<HaP035:DH
aQS600LHaP035:DHaQS600@HaP035:DHaQBU00<DY@@HaP035:DHaQS600HHaP8DY@@HaP@DY@DHaP06
5:DHaQS66<HDYAS61f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX00aS60QBU11S600<LiaS66<H00QS600<DYAS66<H05aS600<D
YAS65:D00QBU1AS60QBU11S600<DYAS66<H00aS600<DYAS65:D01QS600<DYAS66<H00QS600DDYAS6
5:DHaQBU008HaP045:DHaQS65:D66<H00aBU6<HHaP086<H25:D46<H00aBU6<HHaP026<H00aBU6<HH
aP076<H00aBU6<HHaP056<H00acW6<HHaP026<H27>L00b487>LQ20027>L48@P49BT3:DX2;F/4:DX0
0be[:DXYBP07;F/6<H`3=Jd6>Lh00cF]>LhicP04=Jd2<H`012e[<H`aS36<0Re[00@YBRe[:DXYBP<U
:@058@PU:BDY8@PLi`028@P27>L00aS67>LHaP056<H00aBU6<HHaP036<H00aBU6<HDY@076<H00aBU
6<HHaP026<H011BU6<HDYABU41S611BU00<HaQBU6<H00QS60QBU00<HaQBU6<H01aS60QBU0QS611BU
00<HaQBU6<H00aS600<DYAS66<H01aS600<DYAS66<H011S600<DYAS65:D00aBU11S600<DYAS66<H0
1QS60QBU11S611BU1AS600hDYAS66<HHaQBU6<I[NV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[
FP04JeX96<H00acW6<HHaP0A6<H25:D26<H45:D26<H00aBU6<HHaP026<H00aBU6<HHaP056<H00aBU
6<HHaP056<H011BU6<HHaQS611BU11S600<DYAS66<H041S600<DYAS66<H01QS600<DYAS66<H01AS6
00@DYAS66<HDY@@HaP8DY@lHaP8Li`@HaP037>LHaQcW00<Li`@Q20@U:@PYBP8]J`03:DX]Jbe[00<]
J`03:DX]Jbe[008]J`HaS003=JdicSW>00<icP03=JdaS3F]00@e[@8aS0H]J`DYBP@U:@<Q20037>LQ
224800THaP037>LHaQS600@HaP057>LHaQBU6<HDY@0>6<H00aBU6<HHaP026<H00aBU6<HHaP036<H0
0aBU6<HDY@036<H25:D00aS65:DHaP056<H011cW5:DDYAS60QBU0aS600DDYAS66<HHaQBU008HaP05
5:DHaQS66<HDY@026<H011BU6<HHaQBU0QS60QBU0aS61QBU00DHaQBU6<HDYAS6008DY@046<HDYAS6
5:D96<H35:D00aS65:DDY@056<H011BU6<HDYABU11S600<DYAS6JeX01V]J0@000F]J00005VmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J0`0000A[FV]jJeY_N`THaP057>LH
aQS66<HLi`086<H00aBU6<HHaP096<H65:D66<H011BU6<HDYABU21S60QBU0QS600<DYAS66<H00aBU
3AS60QBU0QS600@DYAS66<HHaP8DY@8HaP8DY@HHaP035:DHaQS6018HaP037>LHaQS600<HaP8DY@@H
aP057>LHaQcW7>LHaP037>L38@P00acW8@PQ20029BT00bU:9BTU:@08:DX2;F/012U:;F/aS36<1be[
1c6<00<icSF]=Jd00SW>0S6<13F]00<aS2e[;F/01Be[00<YBRe[;F/00RU:12DY12480acW00<8@QS6
5:D01AS600<LiaS66<H01aS600<DYAS66<H011BU1QS600<DYAS66<H02QS60QBU00@HaQBU6<HDY@HH
aP035:DHaQBU00@HaP055:DHaQBU6<HDY@086<H25:D56<H35:D011S65:DHaQBU0aS600@DYAS66<HD
Y@PHaP035:DHaQBU00<DYA4HaP@DY@036<HDYABU00DHaP095:DHaQS6Kg][FV]jJeY_Nf]J0080005_
N`005V]J00<006]JJeX016]J0aS60QBU1QS60QBU00<HaQBU6<H03AS600@DYAS66<HHaP<DY@HHaP8D
Y@046<HDYAS65:D46<H00aBU6<HDY@0:6<H45:D011S65:DHaQS60QBU00<HaQBU6<H01aS600<DYAS6
6<H02AS60QBU00<HaQBU6<H05QS600<DYAS66<H02AS61acW1B481RDY2bU:00<aS2e[;F/01be[0S6<
00<]Jc6<=Jd01SF]0c6<0cF]00<aS2e[;F/00Re[1bU:0RDY0b480RDY00<Q21cW7>L011cW00<DYAS6
6<H02aS600<DYAS66<H00QS60QBU2QS600@DYAS66<HHaP8DYA8HaP8DY@PHaP@DY@8HaP035:DHaQBU
00<DY@8HaP045:DHaQBU5:D:6<H25:D011S65:DDYABU21S61ABU31S600<DYAS66<H01aS60QBU1AS6
1f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[
FV]jJeY_Nf]JJgX00aS60QcW00<DYAS66<H01AS600<DYAS65:D02AS600<DYAS66<H011S600<DYAS6
5:D02ABU11S600<DYAS66<H02AS60QBU00<HaQBU6<H00QS600<DYAS66<H021S600<DYAS66<H00QS6
00@DYAS66<HDY@HHaP035:DHaQS600HHaP035:DHaQS600hHaP<DY@036<HDYAS600/HaP8Li`036<HL
iacW008Li`DQ20039BTQ22DY00@U:@HYBPX]J`8YBP@]J`8aS08e[@DaS003;F/aS3F]00<e[@8aS0<]
J`<YBP8U:@8YBP039BTQ224800<Q20<Li`<HaP057>LHaQS66<HLi`036<H00aBU6<HHaP026<H25:DC
6<H00aBU6<HHaP025:D:6<H00aBU6<HHaP035:D00aS65:DHaP025:D26<H00aBU6<HHaP0D6<H011BU
6<HHaQBU1AS600<DYAS66<H021S600<DYAS66<H02QS600<DYAS66<H00aS60QBU0aS600DDYAS66<HH
aQBU00<HaP035:DHaQS600<HaP08JgY[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J0QS6
00<DYAS66<H00aS600<DYAS65:D01aS60QBU00<HaQBU6<H00QS600DDYAS66<HHaQBU00PHaP035:DH
aQBU00<HaP035:DHaQS600<HaP8DY@8HaP<DY@HHaP075:DHaQS66<HDYAS65:D01QS60QBU0QS600<D
YAS66<H041S600<DYAS65:D00QS60QBU1QS60QBU11S600@DYAS65:DDY@8HaP037>LDYAS600DHaP04
7>LHaQS66<H27>L00aS67>LLi`057>L48@P00bDY8@PQ20039BT00bU:;F/YBP06:DX2;F/2:DX9;F/3
<H`3;F/00c6<;F/aS002<H`00cF]<H`aS002<H`2;F/2:DX59BT00b489BTU:@038@P47>L36<H00acW
6<HLi`0=6<H25:D36<H25:D00aS65:DHaP0;6<H25:D46<H01ABU6<HHaQS65:D05QS60QBU00<HaQBU
5:D01AS600<DYAS66<H01AS600<DYAS66<H011S600<DYAS65:D021S600@DYAS66<HHaP<DY@PHaP03
5:DHaQS6008DY@046<HDYABU5:D76<H00aBU6<HHaP026<H00aBUJeY[FP05JeX10001JeX0000PKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaS65:DH
aP8DY@<HaP035:DHaQS600THaP035:DHaQS6008HaP035:DHaQS6008HaP035:DHaQS600LHaP035:DH
aQS600@HaP035:DHaQS600PHaP<DY@<HaP045:DHaQS65:D86<H011BU6<HHaQS60aBU00@HaQBU6<HH
aP8DY@LHaP<DY@dHaP8DY@THaP035:DHaQS600`HaP8Li`038@PLiacW00LQ20@U:@8YBP<]J`HYBP8]
J`8YBP<]J`03:DX]Jbe[008aS003;F/aS36<008aS08]J`XaS004;F/YBRU:9BT3:DX012DY:DXU:BDY
12480acW00@Q21cW6<HHaP8Lib8HaP045:DHaQS65:D26<H01aBU6<HHaQS65:DHaQBU008HaP<DY@`H
aP065:DHaQBU5:DHaQBU0aS600DDYAS66<HHaQBU00THaP035:DHaQBU00HDY@036<HDYABU008DY@8H
aP055:DHaQS66<HDY@036<H00aBU6<HHaP086<H25:D86<H011BU6<HHaQS60aBU00XHaQBUKg][FV]j
JeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J00<HaQBU6<H00QBU0aS600<DYAS66<H02AS600<D
YAS66<H00QS600<DYAS66<H00QS600<DYAS66<H01aS600<DYAS66<H011S600<DYAS66<H021S60aBU
0aS600@DYAS66<HDY@PHaP045:DHaQS66<H35:D011S65:DHaQS60QBU1aS60aBU3AS60QBU2AS600<D
YAS66<H031S60QcW00<Q21cW7>L01b4812DY0RU:0be[1RU:0Re[0RU:0be[00<YBRe[;F/00S6<00<]
Jc6<<H`00S6<0Re[2S6<00@]JbU::DXU:@<YBP049BTYBRDY9BT48@P37>L012487>LHaQS60QcW8QS6
00@DYAS66<HDY@8HaP075:DHaQS66<HDYAS65:D00QS60aBU31S600HDYAS65:DDYAS65:D36<H01ABU
6<HHaQS65:D02AS600<DYAS65:D01QBU00<HaQBU5:D00QBU0QS600DDYAS66<HHaQBU00<HaP035:DH
aQS600PHaP8DY@PHaP045:DHaQS66<H35:D00aS65:E[FP06JeX10001JeX0000MJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP026<H011BU6<HHaQS6
0QBU0QS600<DYAS66<H00aS600<DYAS66<H00aS600<DYAS65:D021S600<DYAS66<H00QBU31S60QBU
1QS60aBU4aS600@DYAS66<HDY@@HaP<DY@XHaP035:DHaQS601XHaP057>LHaQcW6<HLi`036<H27>L9
8@P49BT3:DX8;F/00bU:;F/]J`08;F/2<H`00be[<H`aS002<H`2;F/3<H`00be[<H`aS002;F/00c6<
;F/]J`02:DX00bDY:DXYBP049BT28@P27>L00b487>LLi`037>L26<H00acW6<HHaP056<H00acW6<HH
aP086<H00aBU6<HHaP036<H00aBU6<HHaP036<H00aBU6<HHaP036<H25:D011S65:DDYABU0aS600@D
YAS65:DHaP@DY@046<HDYAS67>L26<H25:D26<H25:D46<H00aBU6<HHaP035:D26<H00aBU6<HDY@0?
6<H00aBU6<HHaP046<H00aBU6<HDY@035:D26<H35:D36<H25:D36<H25:D36<H55:D46<H011BU6<HD
YABU0aS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04JeX00aS65:DHaP0B6<H35:D2
6<H25:D011S65:DHaQBU1aS600<DYAS66<H021S60QBU11S600<DYAS66<H00aS600<DYAS66<H01AS6
0QBU0aS600DDYAS65:DHaQBU00@HaP045:DHaQS65:DF6<H00aBU6<HHaP076<H27>L86<H011cW6<HH
aQcW0QS611cW1b480RDY00<Q22DY9BT00RDY0RU:42e[00<YBRe[;F/00Re[00<aS2e[;F/00Re[0RU:
1S6<0Re[00@YBRe[;F/YBPLU:@DQ20037>LQ21cW00HLi`046<HLiacW7>L@6<H25:D36<H25:DB6<H0
1ABU6<HDYAS65:D00aS60QBU0QS600<DYAS66<H02aS600@DYAS65:DHaP8DY@@HaP055:DHaQS66<HD
Y@046<H65:D26<H25:D01QS65:DDYABU6<HDY@8Li`035:DHaQS600DHaP045:DHaQBU5:D>6<H00aBU
6<HHaP026<H35:D7JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_N`036<H25:D26<H35:D36<H25:D66<H00aBU6<HHaP096<H35:D2
6<H00aBU6<HLi`046<H01ABU6<HHaQS65:D00QS600<DYAS65:D00QS60aBU00<HaQBU5:D01QS60QBU
2AS600<DYAS66<H00QS600<DYAS66<H00aBU2AS600<DYAS66<H01AS600<DYAS66<H011S600<DYAS6
6<H01aS600<DYAS66<H00QS600<LiaS66<H00QS611cW00<Q21cW7>L0124800<U:B489BT00RDY0RU:
0RDY0RU:0Re[0S6<1Be[00<aS2e[;F/00Re[00<aS2e[;F/00c6<1be[0c6<00<]Jc6<;F/01BU:00<U
:BU:9BT012DY12480acW00<HaR487>L00QcW0QS60QcW8QS61ABU00<HaQBU6<H00aBU00<HaQBU6<H0
0QS60QBU0QS600@DYAS66<HDY@<HaP035:DHaQS6008DY@8HaP8DY@HHaP<DY@DHaP035:DHaQS600DH
aP055:DHaQBU5:DHaP035:D36<H25:D;6<H00aBU6<HHaP076<H00aBU6<HDY@025:D26<H00aBU6<HD
Y@056<H026mkJeY[NV]JKg][FV]j0001Kg/00003JeX00000008000=[FP8000A[FP8000I[FP03001[
FV]J00A[FP036<HDYABU00XHaP8DY@036<HDYAS600DHaP8DYA4HaP8DY@DHaP<DY@@HaP075:DHaQS6
6<HDYAS65:D00QS60QBU21S600@DYAS65:DDY@8HaP<DY@@HaP075:DHaQS66<HDYAS65:D00QS60QBU
11S600<DYAS66<H00aS600<DYAS65:D00QBU3QS60QcW0QS60QcW00<HaQcW7>L00acW00<Q21cW8@P0
0R480bDY00<YBRDY9BT00RDY12U:0Re[0S6<22e[0RU:0Re[00<aS2e[<H`00c6<1Re[00@aS2e[<H`]
J`LYBPHU:@8Q208Li`8Q20DLi`<HaP8Li`038@PLiaS600lHaP035:DHaQS600/HaP8DY@LHaPDDY@/H
aP055:DHaQS66<HDY@076<H25:D26<H35:D46<H01ABU6<HHaQS65:D00aS60aBU0QS600@DYAS66<HD
Y@8HaP035:DHaQS601PHaP8DY@DHaP035:DHaQS6008DY@036<HDYAS600M[FP40005[FP0001e[NP00
Kg][FV]jJeY_N`00JgY[FP00JeY[NP00Kg][FP00JeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j00XHaP03
5:DHaQS600@HaP045:DHaQS65:D56<H35:D96<H00aBU6<HHaP066<H00aBU6<HHaP025:D011S65:DH
aQBU1QS60QBU0aS60QBU0QS600<DYAS66<H00aS600<DYAcW6<H01aS60QBU21S60QBU21S600<DYAS6
6<H00QBU1aS60aBU2aS61acW0b480bU:0RDY2RU:1Be[00<YBRe[:DX00be[0RU:0be[136<0Re[00<a
S2e[;F/01Be[22U:1RDY0b481QcW00<Q21cW6<H011cW00<HaQcW6<H00QS600<LiaS66<H03aS600@D
YAS66<HHaP8DYA0HaP055:DHaQBU6<HDY@0;6<H011BU6<HHaQBU11S611BU00<HaQBU5:D021S600DD
YAS66<HHaQBU00<HaP045:DHaQBU5:D36<H00aBU6<HDY@056<H00aBU6<HHaP066<H011BU6<HHaQBU
0aS600@DYAS66<HDY@<HaP8DY@HHaP<DY@08JgY[FVmkJeY[NV]JKg/0005[NP000V]J00<006]JJeX0
0V]J00@006]JJeX0009[FP04001[FV]J0005JeX40003JeX36<H25:D56<H55:D56<H00aBU6<HHaP08
6<H00aBU6<HHaP056<H25:D00aS65:DDY@036<H00aBU6<HHaP036<H25:D00aS65:DDY@025:D46<H0
1QBU6<HHaQBU6<HDY@HHaP035:DHaQBU00THaP035:DHaQS6008HaP035:DHaQBU00@HaP035:DHaQS6
00DHaP035:DHaQS600@HaP035:DHaQS600dHaP8Li`8Q20DLi`8Q20039BTYBRU:00HYBP<U:@DYBP8]
J`03:DX]Jbe[008YBP04;F/YBRU::DX9;F/2:DX3;F/012U:;F/]Jbe[1bU:1bDY00@Q21cW8@PQ20<L
i`<HaP037>LHaQcW008Lia@HaP@DY@LHaP055:DHaQS66<HDY@046<H00aBU6<HDY@035:D26<H25:D3
6<H00aBU6<HDY@0;6<H25:D36<H00aBU6<HHaP036<H00aBU6<HDY@086<H00aBU6<HHaP025:D26<H4
5:D46<H00aBU6<HHaP066<H00aBU6<HHaP0;6<H00aBU6<HHaP076<H25:D36<H5JeX30001JeX0000O
Kg][FV]j001_Nf]JJgX006mkJeX006]JKg/006]jJeX006]JJgY[FVmkJeX006]JKg][FV]jJeY_NaS6
5:D01AS600<DYAS66<H01QS600<DYAS66<H00QS600DDYAS66<HHaQBU00@HaP045:DHaQS65:D=6<H0
0aBU6<HHaP056<H00aBU6<HHaP026<H01ABU6<HDYAS65:D00QS600<DYAS65:D02aS60QBU0QS600<D
YAS66<H01AS60QBU1QS600<DYAS66<H01QS600<DYAS66<H01QS600<DYAS66<H00QS600@LiaS66<HH
aP8Li`8HaPPLi`8Q2004:DX]Jbe[;F/::DX00be[:DXYBP02:DX3;F/2:DX5;F/00bU:;F/]J`04;F/2
:DX6;F/00bU:;F/YBP03:DX01BDY:DXYBRDY8@P00bDY0b480QcW00<HaQcW7>L011cW00@HaQcW7>LL
i`PHaP037>LDYAS600lHaP037>LHaQS600@HaP045:DHaQBU6<H25:D36<H011BU6<HDYABU0QS600@D
YAS65:DHaP8DY@@HaP035:DHaQS600LHaP8DY@@HaP@DY@8HaP035:DHaQBU008DY@@HaP8DY@8HaP03
5:DHaQS600PHaP8DY@056<HDYABU6<HDY@026<H00aBU6<HHaP026<H00aBU6<HHaP086<H35:D36<H2
5:D66<H026mkJeY[NV]JKg][FV]j0001Kg/00005JeX006]JJeX00002JeX01000JeY[FP000V]J00@0
06]JJeX000E[FP03001[FV]J00A[FP036<HDYAS600@HaP035:DHaQS600HHaP035:DHaQS6008HaP05
5:DHaQS66<HDY@046<H011BU6<HHaQBU3AS600<DYAS66<H01AS600<DYAS66<H00QS600DDYAS65:DH
aQBU008HaP035:DHaQBU00/HaP8DY@8HaP035:DHaQS600DHaP8DY@HHaP035:DHaQS600HHaP035:DH
aQS600HHaP035:DHaQS6008HaP047>LHaQS66<H27>L26<H87>L28@P012U:;F/]Jbe[2RU:00<]JbU:
:DX00RU:0be[0RU:1Be[00<YBRe[;F/012e[0RU:1Re[00<YBRe[:DX00bU:00DU:BU::DXU:B4800<U
:@<Q208Li`036<HLiacW00@Li`046<HLiacW7>L86<H00acW5:DHaP0?6<H00acW6<HHaP046<H011BU
6<HDYAS60QBU0aS600@DYAS65:DDY@8HaP045:DHaQBU6<H25:D46<H00aBU6<HHaP076<H25:D46<H4
5:D26<H00aBU6<HDY@025:D46<H25:D26<H00aBU6<HHaP086<H25:D01AS65:DDYAS65:D00QS600<D
YAS66<H00QS600<DYAS66<H021S60aBU0aS60QBU1QS61f]J0@000F]J000026]jJeX00000JgY[FVmk
JeX2000016mkJeY[NV]J0P0000e[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP066<H25:D00aS65:DH
aP036<H00aBU6<HDY@056<H00aBU6<HHaP045:D00aS65:DHaP026<H25:D00aS65:DDY@036<H25:D2
6<H00aBU6<HHaP0;6<H25:D00aS65:DHaP025:D46<H25:D46<H35:D011S65:DDYABU1aS600@DYAS6
5:DHaP8DY@8HaP055:DHaQBU6<HDY@026<H011BU6<HHaQBU11S600<DYAS66<H01aS600<LiaS66<H0
1QS61QcW0R4800<U:BU:;F/00Re[2RU:00<]JbU::DX01Be[0RU:00<]JbU::DX00Re[00@YBRe[:DX]
J`8YBP@]J`03:DX]Jbe[008]J`8YBPHU:@8YBP@U:@<Q20037>LQ21cW00DLi`036<HLiaS600<HaP03
5:DLiaS600DHaP045:DHaQS65:D?6<H00aBU6<HHaP046<H01ABU6<HDYABU6<H00QBU21S60QBU11S6
00<DYAS66<H01aS60QBU11S60QBU0QS60aBU00<HaQBU5:D00aS600DDYAS66<HHaQBU00DHaP8DY@DH
aP045:DHaQS65:D36<H00aBU6<HHaP056<H35:D:6<H25:D56<H00aBU6<HHaP036<H026]jJeY_Nf]J
JgY[FVmk0001JgX001I[FP03001[FV]J00A[FPDHaP045:DHaQBU5:D26<H25:D36<H45:D36<H00aBU
6<HHaP046<H25:D66<H00aBU6<HHaP066<H00aBU6<HHaP045:D011S65:DHaQBU0aS60QBU00DHaQBU
5:DDYAS6008DY@036<HDYAS600DHaP035:DHaQS6008DY@THaP035:DHaQBU00DHaP035:DHaQS600DH
aP035:DHaQS600PHaP035:DHaQS600PHaP037>LHaQS600LLi`8Q2003:DX]Jbe[008]J`8YBP03;F/Y
BRU:00DYBP059BT]Jc6<<H`e[@02;F/7:DX00be[:DX]J`05:DX6;F/3:DX49BT2:DX49BT68@P87>L?
6<H00aBU6<HHaP036<H00aBU6<HDY@066<H00aBU6<HHaP025:D96<H00aBU6<HHaP035:D00aS65:DH
aP025:D00aS65:DDY@036<H00aBU6<HDY@056<H25:D86<H01ABU6<HHaQS65:D00QS600<DYAS66<H0
0aS600<DYAS66<H011BU0aS600<DYAS66<H00QS600@DYAS66<HDY@XHaP8DY@8HaP035:DHaQS6008H
aP8DY@8HaP<DY@036<HDYAS600HHaP035:E[FV]J00E[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00dHaP8DY@DHaP035:DHaQS6
008HaP<DY@8HaP035:DHaQS600@HaP035:DHaQBU00@HaP035:DHaQBU008DY@046<HDYABU6<H25:D3
6<H25:D46<H45:D011S65:DHaQBU1AS600@DYAS66<HDY@8HaP035:DHaQBU00DHaP<DY@036<HDYAS6
008HaP035:DHaQS6008HaP8DY@<HaP8DY@HHaP035:DHaQS600dHaPDLi`<Q20049BTYBRU::DX2;F/8
:DX012DY:DXU:Be[0S6<00<e[Be[:DX01BU:00<U:BU::DX01bU:0Re[00@YBRe[;F/]J`<YBP039BTY
BRU:008YBP8U:@03:DXU:BDY00<Q20LLi`056<HLiacW6<HLi`086<H45:D66<H45:D00aS65:DHaP02
6<H00aBU6<HHaP0>6<H25:D46<H35:D=6<H00aBU6<HHaP046<H25:D011S65:DDYABU0QS60aBU11S6
0QBU11S611BU0aS600DDYAS66<HHaQBU008HaP035:DHaQS600hHaP8DY@036<HDYAS6008HaP035:DH
aQS600<HaP035:DHaQS600DHaP08Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J1aS6
00<DYAS66<H01AS600<DYAS66<H011S600@DYAS66<HHaP8DY@036<HDYAS6008DY@`HaP8DY@036<HD
YAS600<DY@056<HDYABU6<HDY@026<H01ABU6<HDYAS65:D00QS600@DYAS66<HDY@HHaP045:DHaQS6
5:D56<H00aBU6<HHaP026<H011BU6<HHaQBU1AS600@DYAS66<HDY@HHaP035:DHaQS600<HaP045:DH
aQS65:D66<H00acW6<HHaP026<H37>L26<H37>L012487>LQ22DY0RU:00<]JbU:;F/022U:00<U:BU:
;F/00c6<00@]JbU:9BTU:@/YBPD]J`dYBPDU:@8Q20HLi`036<HLiacW00LHaP035:DHaQS6008HaP03
5:DHaQBU00DHaP035:DHaQS6008HaP035:DHaQS600THaP<DY@@HaP8DY@<HaP8DY@8HaP035:DHaQBU
00/HaP055:DHaQS66<HDY@036<H25:D011S65:DHaQBU0QS600<DYAS66<H01QS600@DYAS66<HDY@8H
aP055:DHaQS66<HDY@036<H00aBU6<HHaP025:D00aS65:DDY@046<H00aBU6<HDY@045:D26<H25:D4
6<H25:D;6<H00aBU6<I[FP06JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP076<H00aBU6<HHaP035:D26<H00aBU6<HHaP086<H2
5:D00aS65:DHaP0>6<H01ABU6<HDYAS65:D00aS600<DYAS65:D00QBU2QS60QBU0QS60aBU1QS60QBU
00<HaQBU6<H00QS60aBU00<HaQBU6<H031S600<DYAS66<H00aS60QBU2AS600<DYAS67>L01QS600<L
iaS67>L01AcW0R4800<U:BU::DX00RU:0Re[1BU:12DY00<YBRe[;F/00bU:00<U:BU::DX00RDY2RU:
0Re[00<YBRe[;F/00RU:0bDY0bU:0Re[00<YBRDY9BT00bDY0R4800<Lib487>L00acW2QS60QBU00<H
aQBU5:D02QS600<DYAcW6<H04aS60QBU0QS600DDYAS66<HHaQBU008HaP045:DHaQBU5:D36<H00aBU
6<HHaP076<H35:D01AS65:DDYAS65:D01aS60aBU00<HaQBU6<H00aS60QBU00<HaQBU6<H00QS600DD
YAS66<HDYAS6008DY@036<HDYABU008DY@LHaP035:DHaQS600@DY@8HaP045:DHaQS65:D26<H25:D2
6<H00aBU6<HHaP046<H026]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP035:DHaQBU
00HHaP045:DHaQBU5:D26<H25:D00acW6<HHaP096<H00aBU6<HHaP066<H011BU6<HHaQBU0aS600@D
YAS66<HHaP<DY@8HaP@DY@<HaP8DY@8HaP8DY@046<HDYABU5:D>6<H25:D46<H011BU6<HHaQBU0QS6
0QBU1aS600<DYAS66<H01AS600<DYAS65:D02AS600<LiaS66<H00QS600<LiaS67>L00QcW0QS600DL
ib488@PQ22DY008YBP04;F/YBRe[;F/2:DX29BT00bU:9BTU:@029BT3:DX29BT00b489BTU:@039BT3
:DX00bDY:DX]J`05;F/5:DX00bDY:DXQ20029BT2:DX2;F/00bU:9BTU:@058@P77>LP6<H25:D01AS6
5:DDYABU6<H00QBU00<HaQBU6<H00QBU0QS600<DYAS65:D00QBU00DHaQBU5:DHaQBU00<HaP8DY@8H
aP8DY@PHaP8DY@036<HDYAS600LHaP035:DHaQS6008DY@066<HDYAS66<HDYAS60QBU00DHaQBU6<HD
YAS600DDY@046<HDYAS65:D36<H25:D?6<H00aBU6<HHaP036<H35:D66<H00aBUJeY[FP05JeX10001
JeX0000OKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]j
JeY_NaS65:D041S60QBU0QS60QBU1aS600<DYAS65:D011BU21S600<DYAS66<H01AS60QBU00DHaQBU
6<HHaQBU00HHaP8DY@046<HDYABU6<H25:D46<H00aBU6<HHaP036<H25:D00aS65:DHaP046<H00aBU
6<HHaP026<H01ABU6<HHaQS65:D021S60QBU2aS600<DYAS66<H00aS60QcW00<HaQcW7>L011cW0R48
00<U:BU::DX00be[00<YBRe[:DX00RU:00<U:BU:9BT00RDY00@YBRDY9BTU:@8Q20<U:@XYBP<]J`DY
BP039BTQ224800@U:@<YBP<U:@<Q20TLi`036<HLiaS600lHaP055:DHaQS66<HDY@076<H011cW6<HH
aQS60aBU11S600<DYAS66<H01QS611BU0aS60aBU0aS600<LiaBU6<H00QS600<DYAS66<H01QS600DD
YAS66<HHaQBU00XHaPLDY@PHaP045:DHaQS66<H55:D00aS65:DHaP086<H00aBU6<HHaP036<H25:D0
11S65:DDYABU11S600<DYAS66<H00QBU00THaVmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[
FP04JeX00aS65:DHaP0?6<H25:D26<H25:D76<H00aBU6<HDY@045:D86<H00aBU6<HHaP056<H25:D0
1AS65:DHaQS65:D01QS60QBU00@HaQBU5:DHaP8DY@@HaP035:DHaQS600<HaP8DY@036<HDYAS600@H
aP035:DHaQS6008HaP055:DHaQS66<HDY@086<H25:D;6<H00aBU6<HHaP036<H27>L00aS67>LLi`04
7>L28@P00bDY:DXYBP03;F/00bU:;F/YBP02:DX00bDY:DXU:@029BT012U:9BTU:BDY0R480bDY2RU:
0be[1BU:00<U:B488@P012DY0bU:0bDY0b482AcW00<HaQcW6<H03aS600DDYAS66<HHaQBU00LHaP04
7>LHaQS66<H35:D46<H00aBU6<HHaP066<H45:D36<H35:D36<H00acW5:DHaP026<H00aBU6<HHaP06
6<H01ABU6<HHaQS65:D02QS61aBU21S600@DYAS66<HHaPDDY@036<HDYAS600PHaP035:DHaQS600<H
aP8DY@046<HDYABU5:D46<H00aBU6<HHaP025:D00aS6JeY[FP05JeX10001JeX0000NJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU0QS60aBU0aS6
00HDYAS67>LLiaS65:D86<H00aBU7>LDY@066<H00aBU6<HHaP026<H00aBU6<HDY@026<H35:D01AS6
5:DDYAS65:D01AS60aBU1AS600<DYAS66<H01AS600DDYAS66<HHaQBU00XHaP035:DHaQBU00PHaP8D
Y@DHaP8DY@036<HDYAS600LHaP035:DHaQS600LHaP035:DHaQS600@HaP<Li`8Q208Li`8Q20089BTY
BRU:;F/aS2e[:DXU:@<YBP059BTYBRDY9BTQ20089BT3:DX00be[:DXYBP09:DX3;F/012U:9BTU:B48
12DY0RU:00@U:B488@PQ20HLi`036<HLiacW008Li`hHaP035:DHaQS600<HaP8DY@<HaP035:DHaQS6
008HaP037>LHaQS600DHaP8DY@8HaP035:DHaQS6008HaP035:DHaQS600<HaP8DY@@HaP8DY@8HaP04
5:DHaQS65:D46<H25:D26<H35:D36<H55:D?6<H00aBU6<HHaP025:D26<H35:D00aS65:DHaP086<H2
5:D00aS65:DHaP0=6<H35:D011S65:DDYABU0aS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX30004
JeX;6<H25:D00aS65:DHaP056<H00aBU6<HHaP0:6<H25:D26<H00aBU6<HHaP086<H01ABU6<HHaQcW
5:D01aS60aBU1AS600<DYAS66<H02AS60aBU0aS600<DYAS66<H03AS600DDYAS66<HHaQBU008HaP8D
YADHaP037>LHaQcW00<Li`<Q20039BTYBRU:00<]J`<YBPPU:@8Q208U:@8Q20069BTYBRe[;F/e[Be[
22U:00<]JbDY:DX00RU:00<U:B488@P00R4812DY00@YBRDY9BTQ20LLi`036<HLiaS600hHaP8DY@HH
aP8DY@DHaP@Li`<HaP037>LHaQS600@HaP035:DHaQBU008HaP035:DHaQBU008HaP8DY@8HaP035:DH
aQS600LHaP035:DHaQBU008DY@8HaP8DY@LHaPHDY@HHaP035:DHaQS6008DY@DHaP035:DHaQS6008D
Y@8HaP045:DHaQBU6<H25:D86<H00aBU6<HHaP026<H25:D66<H25:D96<H25:D6JeX20001JeX0000N
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaS6
0QBU3AS600<DYAS66<H01QS600<DYAS66<H01AS600<DYAS66<H00QS600<DYAS66<H01AS60aBU31S6
0QBU00<HaQBU6<H00QS60QBU7AS60QBU0QS600@DYAS66<HDY@<HaP045:DHaQBU5:D26<H00aBU6<HH
aP0=6<H00acW6<HLi`057>L38@P39BT00be[:DXYBP04:DX49BT58@P49BT00bU:;F/]J`02;F/29BT7
:DX00bDY:DXU:@039BT48@P59BT38@P27>L012487>LQ21cW3AS600@LiaS66<HDY@LHaP035:DHaQS6
00PHaP037>LHaQS600DHaP8DY@<HaP035:DHaQS600LHaP035:DHaQS6008HaP<DY@@HaP8DY@8HaP03
5:DHaQS6008HaP<DY@8HaP045:DHaQBU5:D76<H25:D46<H00aBU6<HHaP046<H00aBU6<HHaP056<H2
5:D36<H01QBU6<HDYAS65:DHaP8DY@036<HDYAS6008DY@HHaP8DY@@HaP045:DHaQS65:D46<H02ABU
Kg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP8HaP<DY@HHaP8DY@036<HDYAS600@H
aP035:DHaQS6008HaP045:DHaQS66<H25:D00aS65:DHaP0;6<H00aBU6<HHaP056<H25:D66<H011BU
6<HHaQS60QBU0QS600<DYAS65:D01QS600<DYAS66<H00aS60QBU2AS600<DYAS66<H02QS600<DYAS6
5:D00aS600<LiaS66<H021S600<LiaBU6<H011S600<LiaS67>L011cW0b481BDY0RU:00DU:BU:9BTU
:BU:008U:@038@PU:B48008U:@038@PU:BDY008U:@`YBPLU:@048@PU:B488@P49BT012U:8@PQ2248
1QcW00<HaQcW6<H021S60QBU41S600<DYAS66<H03QS600<DYAS66<H00aS600<DYAS65:D01aS60QBU
1AS600<DYAS66<H00aS600<DYAS65:D021S60aBU1QS600PDYAS66<HDYAS65:DHaQBU1aS600<DYAS6
6<H00aS600<DYAS65:D011S60QBU0QS600<DYAS66<H02AS600@DYAS65:DDY@8HaP<DY@LHaP035:E[
FV]J00E[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FP00JeY[NV]JKg][FV]j00<HaP8DY@DHaP035:DHaQS600@HaP035:DHaQS6008DY@036<HDYAS600<H
aP8DY@036<HDYAS600<HaP035:DHaQBU008HaP045:DHaQS66<H35:D46<H011BU6<HDYABU11S600DD
YAS66<HHaQBU00<HaP035:DHaQS601XHaP8DY@XHaP035:DHaQS600LHaP045:DHaQS65:D96<H37>L0
1R487>LLiacW8@PLi`8Q208U:@038@PU:BDY008U:@048@PU:BDY9BT28@P29BT68@P29BT012U:;F/]
JbU:0RDY00<YBRDY:DX00RDY00<YBRDY9BT012DY00@Q22DY9BTQ20HU:@8Q20@Li`@HaP8LiaLHaP<D
Y@<HaP035:DHaQS600<HaP035:DHaQS600@HaP035:DHaQS6008HaP8DY@DHaP8DY@066<HDYAS65:DH
aQBU1AS60QBU0aS60aBU0aS611BU0QS600<DYAS65:D00QBU0QS600@DYAS66<HHaP8DY@036<HDYAS6
00lHaP055:DHaQBU5:DHaP025:D:6<H25:D46<H25:D86<H25:D36<H026]jJeY_Nf]JJgY[FVmk0001
JgX001I[FP03001[FV]J00A[FP036<HDYAS6008HaP035:DHaQBU00<HaP<DY@8HaP@DY@047>LHaQBU
6<H45:D26<H35:D00aS65:DDY@046<H011BU6<HHaQS60aBU1aS600<DYAS65:D00aS60QBU00<HaQBU
5:D011S60QBU11S600HDYAS65:DHaQBU6<H25:D46<H00aBU6<HHaP056<H45:D01AS65:DHaQBU6<H0
0QBU00DHaQBU6<HHaQBU00DHaP035:DHaQBU00@HaP035:DHaQS600PHaPHLi`8Q20067>LQ22488@PU
:B480RDY00<Q22DY9BT00R4800@U:B488@PQ20<U:@058@PU:BDY9BTYBP02;F/012U:9BTYBRU:2BDY
0b481BDY0R4800<U:B488@P01acW00<HaQcW7>L04aS600<DYAS66<H00QS60QBU11S600DDYAS66<HH
aQBU00/HaP8DY@8HaP035:DHaQS600PHaP035:DHaQS6008HaP055:DHaQS66<HDY@046<H011BU6<HH
aQS61ABU00<HaQBU6<H03AS600<DYAS66<H02QS600<DYAS66<H00QS60QBU00@HaQBU6<HDY@hHaP03
5:DHaQS6008HaP@DY@DHaP035:DHaV]J00I[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00dHaP035:DHaQBU00THaP045:DHaQS6
5:D26<H00aBU6<HDY@0>6<H25:D26<H00aBU7>LHaP076<H25:D76<H25:D46<H25:D@6<H00aBU6<HH
aP096<H00aBU6<HHaP066<H00aBU6<HHaP086<H35:D26<H011cW6<HLiacW0R4800@Lib487>LLi`HQ
208U:@<Q20049BTQ22489BT48@P59BT3:DX<9BT48@P29BT00b489BTQ20028@P012DY8@PQ22480QcW
0QS600<LiaS66<H00QS600<LiaS66<H03AS60QBU00<HaQBU6<H02QS600<DYAS66<H021S600<DYAS6
5:D00QS600<DYAS66<H01AS60QBU2AS600<DYAS66<H011S611BU0QS61ABU0aS60QBU0aS600<DYAS6
5:D00QBU1AS60QBU1AS600<LiaBU5:D011S600<DYAS66<H00QS600DDYAS65:DDYAS600<DY@LHaP03
5:DHaQS600`HaP0;5:DHaQS6Kg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FPdHaP03
5:DHaQBU00THaP045:DHaQS65:D26<H00aBU6<HDY@0>6<H25:D26<H00aBU7>LHaP076<H25:D76<H2
5:D46<H25:D@6<H00aBU6<HHaP096<H00aBU6<HHaP066<H00aBU6<HHaP086<H35:D26<H011cW6<HL
iacW0R4800@Lib487>LLi`HQ208U:@<Q20049BTQ22489BT48@P59BT3:DX<9BT48@P29BT00b489BTQ
20028@P012DY8@PQ22480QcW0QS600<LiaS66<H00QS600<LiaS66<H03AS60QBU00<HaQBU6<H02QS6
00<DYAS66<H021S600<DYAS65:D00QS600<DYAS66<H01AS60QBU2AS600<DYAS66<H011S611BU0QS6
1ABU0aS60QBU0aS600<DYAS65:D00QBU1AS60QBU1AS600<LiaBU5:D011S600<DYAS66<H00QS600DD
YAS65:DDYAS600<DY@LHaP035:DHaQS600`HaP035:DHaQS600M[FP40005[FP00021[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU1AS600<D
YAS66<H00QS60QBU00<HaQBU5:D01QS600<DYAS66<H00QS600<DYAS65:D01QS600@DYAS66<HHaP8D
Y@8HaP045:DHaQS65:D66<H00aBU6<HDY@025:D00aS65:DHaP036<H00aBU6<HDY@056<H011BU6<HD
YABU1AS60aBU1AS600<DYAS66<H01aS600<DYAS66<H00aS600<DYAS66<H06AS60QcW00<Q21cW7>L0
424800<U:B488@P00R482BDY00<YBRDY9BT022DY12480RDY0R480RDY0R4821cW1aS600<DYAS66<H0
0QS60aBU0QS600<DYAS66<H00aS60QBU00@HaQBU6<HHaP8DY@PHaP045:DHaQS65:D36<H25:D36<H2
5:D011S65:DHaQS60aBU21S600<DYAS66<H00QBU11S60QBU00<HaQBU6<H00aS600<DYAS65:D01AS6
00<DYAS66<H00aS600<DYAS66<H00aS60QBU1AS600<DYAS66<H01AS60aBU00@HaQBU5:DHaP8DYA8H
aP045:DHaQBU5:D36<H02aBU6<HDYF]jJeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX0
0aBU6<HHaP066<H25:D26<H01ABU6<HHaQS65:D00QS600@DYAS65:DHaPDDY@046<HDYABU5:D56<H6
5:D46<H00aBU6<HHaP096<H01ABU6<HDYAS65:D00QS600DDYAS65:DHaQBU00dHaP035:DHaQS600/H
aP035:DHaQS600@HaP035:DHaQS6008DY@<HaP035:DHaQBU00<HaP8DY@8HaP035:DHaQS600LHaP03
7>LHaQcW00@Li`048@PLib488@P27>L28@P00acW8@PQ20038@P00bDY8@PQ20068@P39BT3:DX012DY
8@PU:BDY12U:00@U:BU:;F/YBP@U:@03:DXU:BDY00<U:@8Q20@Li`@HaP037>LHaQS600PHaP035:DH
aQS600DHaP8DY@PHaP035:DHaQS600PHaP8DY@096<HDYABU6<HDYAS65:DHaQBU008HaP035:DHaQBU
01XHaP@DY@036<HLiaS600DHaP035:DHaQS600LHaP035:DHaQS6008DY@LHaP035:DHaQBU008DY@HH
aP055:DHaQS66<HDY@066<H00aBU6<HHaP066<H25:D56<H00aBU6<HHaP07JeX10001JeX0000MKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`026<H0
1ABU6<HHaQBU6<H00QBU1AS600@DYAS65:DDY@PHaP8DY@036<HDYABU00<DY@8HaP045:DHaQS65:D5
6<H00aBU6<HHaP0:6<H25:D26<H25:D011S65:DDYABU0QS600<DYAS65:D011S600<DYAS66<H00aS6
00<DYAS66<H01AS600<DYAS66<H01QS600<DYAS65:D01AS600@DYAS66<HDY@PHaP8DY@/HaP037>LH
aQcW00TLi`DQ20037>LQ224800PQ210U:@05:DX]JbU:;F/U:@02:DX39BT2:DX39BT28@P00acW8@PL
i`037>L00aS67>LLi`0;6<H25:D36<H25:D66<H01QBU6<HHaQS65:DLi`8DY@DHaP035:DHaQBU00DH
aP045:DHaQS65:D56<H00aBU6<HHaP056<H00aBU6<HHaP056<H00aBU6<HHaP056<H00aBU6<HHaP02
5:D00aS65:DHaP0E6<H00aBU6<HHaP026<H35:D01AS65:DHaQS65:D021S60aBU0QS60QBU1aS600DD
YAS66<HHaQBU008HaP8DY@DHaP08Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J0aS6
0QBU0aS60QBU2aS600@DYAS66<HDY@<HaP045:DHaQS66<H45:D26<H011BU6<HDYABU0aS60aBU0QS6
0QBU11S600<DYAS66<H011S600<DYAS66<H011S600<DYAS66<H00QS600HDYAS66<HHaQBU6<H25:D4
6<H25:DB6<H01ABU6<HHaQS65:D011S600<DYAS66<H041S61AcW00<Q21cW8@P00QcW0R4800@Lib48
8@PQ208Li`XQ20<U:@038@PU:BDY00PU:@04:DXaS3F];F/3:DX29BT3:DX012DY8@PQ224811cW11S6
00<LiaS66<H01aS60QBU21S60QBU00HHaQBU5:DDYAS65:D36<H25:D66<H01ABU6<HHaQS65:D00QS6
0QBU00<HaQBU6<H011S60QBU00@HaQBU6<HHaP8DY@046<HDYAS66<H45:D;6<H35:D00aS65:DHaP03
6<H35:D96<H25:D76<H55:D011S65:DHaQS60aBU1AS600<DYAS66<H021S60QBU1QS600<DYAS66<H0
1aS61f]J0@000F]J00005V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
0`0000A[FVmkJeY[NPDHaP8DY@046<HDYAS66<H25:D011S65:DHaQBU5aS600DDYAS65:DDYAS6008D
Y@8HaP055:DHaQS66<HDY@026<H45:D36<H00aBU6<HHaP046<H00aBU6<HDY@076<H00aBU6<HHaP03
6<H00aBU6<HHaP066<H00aBU6<HHaP0=6<H00aBU6<HHaP046<H25:D26<H011BU6<HHaQBU21S600<L
iaS67>L00QcW00<Q21cW7>L0224800<Lib488@P00RDY1R4812DY00<Q22DY9BT00RDY00<Q22DY8@P0
0RDY0bU:00<]JcF]<H`00bU:1RDY0R480QcW00@Q21S67>LLi`@HaP037>LHaQS600<HaP037>LHaQS6
008HaP035:DHaQS600DHaP8DY@056<HDYAS65:DHaP035:D26<H25:D26<H25:D011S65:DHaQBU11S6
0QBU0aS600<DYAS66<H011BU0aS60QBU0QS60QBU3QS600<DYAS66<H02AS600<DYAS66<H011S600HD
YAS66<HHaQBU6<H35:D00aS65:DDY@025:D46<H35:D00aS65:DDY@026<H25:D26<H35:D@6<H25:D4
6<H25:D01aS6JgY[FVmkJeY[NV]J0080005[NP005V]J00<006]JJeX016]J1AS600@DYAS66<HHaP8D
Y@8HaP035:DHaQS6008HaP8DY@<HaP035:DHaQBU008HaP035:DHaQBU008DY@DHaP045:DHaQS65:D7
6<H00aBU6<HHaP096<H00aBU6<HDY@035:D36<H00aBU6<HDY@025:D00aS65:DHaP036<H25:D01AS6
5:DHaQS65:D01aS600@DYAS65:DDYA0HaP035:DHaQBU008HaP8DY@HHaP8Li`HHaP<Li`038@PLiacW
00<Li`8Q208Li`038@PLib4800LQ20HU:@DQ20DU:@@YBP8]J`HU:@<Q20LLi`056<HLiacW6<HLi`0:
6<H00aBU6<HHaP026<H27>L66<H00aBU6<HHaP046<H00aBU6<HDY@025:D76<H01ABU6<HHaQS65:D0
11S60QBU0aS60QBU0aS60QBU41S60QBU0aS600<DYAS66<H00aS60aBU0aS600<DYAS66<H01AS600<D
YAS66<H01QS60QBU0aS600<DYAS65:D00QBU00@HaQBU5:DDY@DHaP037>LHaQBU00LHaP035:DHaQS6
00@HaP035:DHaQS600DHaPM[FP40005[FP0001m_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk6<HDY@076<H00aBU6<HDY@035:D66<H01ABU6<HDYABU
6<H00QBU00<HaQBU6<H00QBU00@HaQBU5:DHaP@DY@DHaP035:DHaQBU00HHaP035:DHaQS600`HaP03
5:DHaQS6008HaP045:DHaQS65:D86<H25:D:6<H011BU6<HHaQS611BU2QS600<DYAS66<H00QS600<D
YAS66<H03QS61acW00DQ21cW8@PLib48008Li`HQ20<U:@048@PU:BDY9BT48@P79BT2:DX00bDY:DXU
:@029BT00b489BTLi`058@P00acW6<HLi`036<H27>L;6<H00aBU6<HDY@035:D?6<H25:D36<H25:D3
6<H00aBU6<HHaP0<6<H35:D36<H25:D00aS65:DHaP025:D011S65:DHaQBU1aS600<DYAS65:D00QS6
00<DYAS66<H00QBU11S60aBU2AS60QBU00@HaQBU6<HHaP8DY@LHaP035:DHaQS6008HaP8DY@046<HD
YABU5:D;6<H01ABU6<HHaQS65:D01AS611BU00THaVmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00
JeY[FP04JeX00aS65:DHaP066<H00aBU6<HDY@035:D66<H01ABU6<HDYABU6<H00QBU00<HaQBU6<H0
0QBU00@HaQBU5:DHaP@DY@DHaP035:DHaQBU00HHaP035:DHaQS600`HaP035:DHaQS6008HaP045:DH
aQS65:D86<H25:D:6<H011BU6<HHaQS611BU2QS600<DYAS66<H00QS600<DYAS66<H03QS61acW00DQ
21cW8@PLib48008Li`HQ20<U:@048@PU:BDY9BT48@P79BT2:DX00bDY:DXU:@029BT00b489BTLi`05
8@P00acW6<HLi`036<H27>L;6<H00aBU6<HDY@035:D?6<H25:D36<H25:D36<H00aBU6<HHaP0<6<H3
5:D36<H25:D00aS65:DHaP025:D011S65:DHaQBU1aS600<DYAS65:D00QS600<DYAS66<H00QBU11S6
0aBU2AS60QBU00@HaQBU6<HHaP8DY@LHaP035:DHaQS6008HaP8DY@046<HDYABU5:D;6<H01ABU6<HH
aQS65:D01AS611BU00<HaV]JJeX01F]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDY@dHaP045:DHaQBU6<H25:D36<H00aBU6<HH
aP026<H45:D46<H25:D011S65:DHaQS60QBU1QS60QBU11S600<DYAS65:D00QBU0aS60aBU11S60QBU
2QS600@DYAS65:DDY@XHaP8DY@@HaP035:DHaQS600HHaP8DY@036<HDYAS600DHaP035:DHaQS600PH
aP037>LHaQS600<HaPPLi`8Q208Li`8Q20047>LQ22488@P<9BT38@P59BT2:DX79BT48@P27>L011S6
7>LLiacW1AS600<LiaS66<H02AS600<DYAS66<H02QS600<DYAS66<H00aS600@DYAS66<HHaP<DY@8H
aP035:DHaQS600<HaP065:DHaQS66<HDYAS60QBU0QS60QBU0aS60QBU0aS600<DYAS66<H01QBU11S6
00<DYAS66<H01ABU0QS600@DYAS65:DDY@@HaP055:DHaQS66<HDY@046<H25:D36<H00aBU6<HHaP02
6<H25:D46<H25:D01QS65:DDYABU6<HDY@HHaP035:DHaQS600<HaP8DY@THaP8DY@<HaP08JgY[FVmk
JeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00<DYAS66<H00QS600@DYAS66<HDY@DHaP8DY@8H
aP8DY@8HaP035:DHaQS600@HaP055:DHaQS66<HDY@026<H25:D36<H00aBU6<HHaP026<H011BU6<HH
aQBU0aS600<DYAS66<H011BU00<HaQBU6<H00QS60QBU11S600<DYAS66<H011S60QBU00<HaQBU5:D0
21S60QBU3aS600<DYAS66<H00aS60aBU1QS600@DYAS66<HDY@/HaPPLi`048@PLib488@P47>L58@P2
9BT28@P012DY8@PU:B4812DY00@Q22DY9BTU:@@YBP039BTYBRDY00<U:@048@PLib488@P67>L;6<H2
5:D86<H00aBU6<HHaP026<H25:D011S65:DHaQS60QBU0QS600DDYAS65:DDYAS600<DY@8HaP045:DH
aQS66<H25:D36<H25:D36<H00aBU6<HDY@045:D01AS65:DDYAS65:D01AS600<DYAS66<H00aS600DD
YAS66<HHaQBU00<HaP065:DHaQS65:DHaQBU2QS600@DYAS66<HDY@DHaP8DY@036<HDYAS6008HaP03
5:DHaQS600DHaP8DY@036<HDYABU008HaP035:DHaQBU00PHaP065:DHaQBU5:DHaQBU0QS60QBU0QS6
00<DYAS66<H01f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]J001[FVmkJeY[NV]JKg/021S600<DYAS66<H00QS60QBU00<HaQBU6<H02QS600@DYAS66<HD
Y@8HaP8DY@036<HDYABU00HHaP8DY@036<HDYAS6008DY@@HaP045:DHaQBU5:D26<H00aBU6<HHaP02
5:D46<H25:D<6<H011BU6<HHaQBU0aS600<DYAS66<H00QS60QBU1AS60QBU0aS61QBU0aS600<DYAS6
6<H03QS600<LiaS66<H02QcW00<Q21cW7>L00QcW1B4800<U:B488@P00RDY00<Q22DY8@P012DY00DQ
22DY9BTU:BU:008]J`<YBP@U:@038@PU:B48008Q20<Li`<HaP037>LHaQS600<HaP057>LDYAS66<HD
Y@056<H25:D86<H25:D36<H25:D86<H35:D46<H25:D46<H011BU6<HHaQS60aBU0aS60aBU2AS600@D
YAS66<HDY@<HaP8DY@@HaP8DY@DHaP065:DHaQS66<HDYAS61QBU1aS600DDYAS66<HDYAS6008DY@<H
aP035:DHaQS600<HaP<DY@8HaP8DY@<HaP035:DHaQS600DHaP035:DHaQS6008HaP035:DHaQBU008D
Y@8HaP0;5:DHaQBUKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FPDHaP035:DHaQS6
00<DY@<HaP045:DHaQS66<H25:D46<H011BU6<HDYABU0QS60QBU1AS600<DYAS66<H00QS60QBU00DH
aQBU5:DHaQBU00<HaP035:DHaQBU00<HaP<DY@LHaP8DY@<HaP8DY@8HaP035:DHaQS600DHaP035:DH
aQS600DHaP035:DHaQBU00DHaP035:DHaQS6008DY@036<HDYAS600<HaP<DY@DHaP8DY@056<HDYABU
6<HDY@056<H47>L26<H87>L28@P37>L48@P00acW8@PQ20038@P01RDY8@PU:BDY8@PU:@8YBP049BTY
BRDY9BT2:DX3;F/29BT2:DX28@P00bDY8@PQ20028@P47>L00aS67>LLi`076<H00aBU6<HHaP066<H0
0aBU6<HHaP036<H00aBU6<HHaP0;6<H45:D46<H00aBU6<HHaP046<H011BU6<HHaQBU0QS60QBU11S6
1QBU00<HaQBU5:D00QS60aBU1QS611BU1AS60QBU0QS60QBU0aS60QBU11S600@DYAS65:DDY@DHaP03
5:DHaQS6008HaP035:DHaQS600LHaP8DY@046<HDYAS66<H25:D00aS65:DDY@026<H00aBU6<HDY@02
5:D36<H01QBU6<HHaQS65:DHaP8DY@8HaP035:E[FV]J00E[FP40005[FP0001e[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j008HaP035:DHaQS6008D
Y@8HaP035:DHaQS6008HaP045:DHaQS65:D26<H25:D86<H011BU6<HHaQBU31S600<DYAS66<H01QS6
00<DYAS66<H00QS600<LiaS66<H011S600<DYAS66<H00QS600<DYAS66<H011S600TDYAS66<HHaQBU
6<HDYAS65:D00aS60QBU11S60QBU1AS611BU41S60QBU0QS60QcW1QS60QcW00<HaQcW7>L00QcW0R48
0QcW00@Q21cW7>LLi`<Q20039BTQ224800DQ208U:@038@PU:BU:008YBP8U:@@YBP<]J`@U:@<Q2003
7>LQ21cW00DLi`056<HLiaS66<HLi`026<H00acW5:DLi`086<H00aBU6<HHaP025:D96<H45:D26<H0
0aBU6<HDY@036<H55:D86<H25:D66<H45:D66<H25:D26<H011BU6<HHaQS60QBU2AS600<DYAS65:D0
11S62ABU0aS611BU0QS60QBU00<HaQBU5:D00QBU00@HaQBU6<HDY@<HaP035:DHaQBU008HaP8DY@03
6<HDYABU00DHaP055:DHaQS66<HDY@046<H00aBU6<HHaP046<H25:D026]jJeY_Nf]JJgY[FVmk0001
JgX001I[FP03001[FV]J00A[FP<HaP@DY@046<HDYAS65:D36<H00aBU6<HHaP036<H00aBU6<HDY@02
6<H00aBU6<HHaP0<6<H00aBU6<HHaP026<H01ABU6<HHaQS65:D01QS60QBU31S600<DYAS65:D02aS6
00<DYAS66<H011S600<DYAS65:D02AS60aBU1QS60QBU0aS600<LiaS66<H01AS600<DYAS66<H01AS6
0QcW1AS62QcW0b4800<Lib488@P00R480bDY00<Q22DY9BT00RDY00DYBRe[;F/]JbDY00LYBP<U:@DQ
20LLi`THaP035:DHaQS600<HaP055:DHaQBU6<HDY@0;6<H00aBU6<HDY@025:D46<H00aBU6<HHaP05
6<H011BU6<HDYABU11S600<DYAS65:D02QS60aBU00<HaQBU6<H00QBU00<HaQBU6<H00QBU1AS600@D
YAS65:DHaP<DY@056<HDYABU5:DHaP075:D36<H25:D00aS65:DHaP046<H00aBU6<HDY@046<H25:D0
0aS65:DDY@036<H35:D26<H25:D26<H00aBU6<HHaP0@6<H25:D26<H00aBUJeY[FP05JeX10001JeX0
000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_
N`036<H45:D011S65:DHaQBU0aS600<DYAS66<H00aS600<DYAS65:D00QS600<DYAS66<H031S600<D
YAS66<H00QS600DDYAS66<HHaQBU00HHaP8DY@`HaP035:DHaQBU00/HaP035:DHaQS600@HaP035:DH
aQBU00THaP<DY@HHaP8DY@<HaP037>LHaQS600DHaP035:DHaQS600DHaP8Li`DHaPXLi`<Q20037>LQ
2248008Q20<U:@038@PU:BDY008U:@05:DX]Jbe[;F/U:@07:DX39BT58@P77>L96<H00aBU6<HHaP03
6<H01ABU6<HDYAS65:D02aS600<DYAS65:D00QBU11S600<DYAS66<H01AS600@DYAS65:DDY@@HaP03
5:DHaQBU00XHaP<DY@036<HDYAS6008DY@036<HDYAS6008DY@DHaP045:DHaQBU6<H35:D01AS65:DD
YABU6<H01aBU0aS60QBU00<HaQBU6<H011S600<DYAS65:D011S60QBU00<HaQBU5:D00aS60aBU0QS6
0QBU0QS600<DYAS66<H041S60QBU0QS600TDYFmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[
FP04JeX00aS65:DHaP036<H25:D26<H35:D00aS65:DDY@026<H25:D:6<H00aBU6<HHaP066<H00aBU
6<HHaP026<H35:D56<H01QBU6<HHaQS65:DHaP8DY@056<HDYAS65:DHaP025:D46<H00aBU6<HHaP02
6<H00aBU6<HDY@096<H35:D00aS65:DDY@026<H00aBU6<HHaP0=6<H25:D00aS65:DHaP076<H011BU
6<HHaQBU0QS600<DYAS66<H01QS600<LiaS67>L01QcW00<Q21cW8@P00R4800<Lib488@P00R4812DY
00@YBRDY9BTU:@DYBP@]J`8YBP059BTQ22DY8@PU:@048@P27>L46<H00acW6<HHaP0:6<H00aBU6<HH
aP046<H00aBU6<HHaP036<H35:D26<H00aBU6<HHaP025:D00aS65:DDY@026<H00aBU6<HDY@046<H0
0aBU6<HHaP056<H25:D00aS65:DHaP0:6<H01ABU6<HHaQS65:D03AS600<DYAS66<H00QBU0QS600<D
YAS66<H011BU00@HaQBU5:DDY@PHaP<DY@<HaP@DY@HHaP<DY@8HaP<DY@<HaP035:DHaQS600<HaP04
5:DHaQBU6<H25:D56<H00aBU6<HHaP026<H00aBU6<HHaP07JeX10001JeX0000FJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX3000016]JKg][FV]j1AS600DDYAS65:DDYAS6
00<DY@LHaP045:DHaQS65:D26<H35:D;6<H00aBU6<HDY@025:D76<H01ABU6<HHaQS65:D00aS600@D
YAS65:DHaP@DY@<HaP@DY@036<HDYAS600HHaP035:DHaQBU008HaP8DYA@HaP035:DHaQS601THaPTL
i`038@PLib4800HQ208U:@DYBP039BTYBRDY00<YBP059BTYBRe[:DX]J`02:DX01BDY8@PQ21cW9BT0
0b481AcW71S600<DYAS65:D00aS600@DYAS65:DHaP/DY@036<HDYAS600THaP035:DHaQS600@HaP05
5:DHaQBU5:DHaP035:D00aS65:DHaP036<H25:D00aS65:DHaP026<H011BU6<HHaQBU1QS61QBU21S6
0aBU00<HaQBU5:D01QS60QBU0aS600<DYAS66<H00QS600<DYAS66<H021S611BU0aS600<DYAS66<H0
1AS600<DYAS66<H00QS60QBU00LHaV]jJeY_Nf]JJgY[FP020001JgX001I[FP03001[FV]J00A[FP03
5:DHaQBU008HaP035:DHaQBU00<HaP8DY@<HaP035:DHaQBU008DY@036<HDYAS600HHaP035:DHaQS6
00@HaP035:DHaQBU00@HaP045:DHaQS65:D86<H25:D36<H00aBU6<HDY@056<H011BU6<HDYABU0QS6
00<DYAS66<H00QS600DDYAS66<HHaQBU008HaP8DY@8HaP035:DHaQS600HHaP8DY@DHaP035:DHaQBU
00@HaP035:DHaQS600HHaP045:DHaQS65:D46<H00aBU6<HHaP076<H77>L00b487>LQ20027>L38@P2
9BT6:DX29BT00bU:9BTU:@039BT3:DX39BT48@P27>L28@P37>L86<H01AcW6<HHaQS65:D02QS60QBU
0QS600<DYAS65:D021BU0QS60aBU00<HaQBU5:D00QBU0QS60QBU00<HaQBU5:D011S60aBU0QS60aBU
0aS600LDYAS66<HDYAS65:DHaP025:D36<H00aBU6<HHaP045:D26<H00aBU6<HHaP056<H011BU6<HD
YAS60QBU0QS600<DYAS65:D00QS600<DYAS65:D00QS600<DYAS66<H021S60QBU1QS60QBU0QS600DD
YAS66<HHaQBU00@HaP<DY@`HaP035:DHaQS600<HaPM[FP40005[FP00021_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk6<HDYAS60QBU0QS600<DYAS6
6<H011S600@DYAS66<HDY@8HaP8DY@8HaP045:DHaQBU5:D36<H00aBU6<HHaP026<H35:D26<H011BU
6<HHaQBU21S60QBU0QS600@DYAS66<HDY@8HaP045:DHaQBU5:D86<H01ABU6<HDYAS65:D00aS600DD
YAS66<HHaQBU01LHaP035:DHaQS6008HaP035:DHaQBU00@HaP055:DHaQS66<HDY@026<H00aBU6<HH
aP076<H77>L68@P00bDY8@PQ20029BT4:DX2;F/00bU:9BTU:@029BT00b489BTU:@039BT28@P00bDY
8@PQ20028@P57>L00aS67>LHaP025:D46<H01AcW6<HLiacW6<H00QBU1AS600@DYAS66<HDY@PHaP03
5:DHaQS6008HaP045:DHaQS65:D66<H011BU6<HHaQS60QBU0aS60aBU21S60aBU0QS60QBU11S600<D
YAS66<H00QS600<DYAS66<H011S600<DYAS66<H00aBU0QS600@DYAS65:DDY@8HaP8DY@DHaP8DY@@H
aP035:DHaQS600LHaP8DY@THaP035:DHaQS6008DY@THaP035:DHaQBU008HaP035:DHaQS600DHaP03
5:DHaQS600<HaP08Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J0aS60QBU0QS60QBU
0QS60QBU0QS600<DYAS66<H03aS600<DYAS66<H00QS600@DYAS66<HDY@@HaP035:DHaQS600<HaP04
5:DHaQS66<H25:D36<H011BU6<HHaQBU1AS600<DYAS67>L00aS600@DYAS66<HDY@PHaP<DY@036<HD
YAS6008HaP055:DHaQS65:DHaP025:D36<H00aBU6<HHaP036<H25:D46<H00aBU6<HHaP076<H25:D2
6<H00aBU6<HHaP056<H67>L00b487>LQ20068@P29BT01RU:;F/]Jbe[<H`]J`8YBP039BTYBRDY00@U
:@LQ20037>LQ21cW00DLi`HHaP057>LHaQS66<HLi`0;6<H01ABU6<HDYABU6<H00aBU00<HaQBU6<H0
11BU1AS60QBU0QS60QBU3QS600<DYAS66<H011S600@DYAS66<HDY@<HaP035:DHaQBU00LHaP<DY@8H
aP8DY@046<HDYAS65:D36<H00aBU6<HHaP026<H25:D00aS65:DHaP055:D56<H25:D01AS65:DHaQS6
5:D011S60QBU0aS60QBU0QS600@DYAS65:DHaP8DY@8HaP8DY@HHaP035:DHaQS600HHaP035:DHaQS6
008HaP8DY@M[FP40005[FP0001e[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FP00JeY[NV]JKg][FV]j00PHaP8DY@036<HDYABU00DHaP8DY@046<HDYAS65:D26<H25:D46<H0
0aBU6<HDY@035:D46<H25:D86<H45:D96<H45:D011S65:DHaQS611BU2AS600<DYAS66<H011S600<D
YAS66<H00aBU0aS600<DYAS66<H01aS60QBU4aS600<DYAS66<H01AS600<Lib487>L00QcW0b4800<U
:B488@P00R480bDY00<]JcF]<H`00S6<00<]JbU::DX012U:12DY12480QcW0R481AcW0aS600<DYAcW
7>L02AS600<DYAS66<H021S600@DYAS66<HHaP8DY@036<HDYAS600<HaP<DY@HHaP8DY@8HaP8DY@8H
aP8DY@`HaP055:DHaQS66<HDY@056<H35:D26<H011BU6<HHaQBU1aS600<DYAS66<H00aBU00<HaQBU
5:D011BU1AS60QBU00DHaQBU6<HHaQBU00<HaP035:DHaQS6008HaP@DY@DHaP8DY@066<HDYAS65:DH
aQBU5QS60aBU00/HaQBU5:E[NV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00<HaQBU
6<H00QS600<DYAS66<H021S600<DYAS66<H00QS600<DYAS66<H01AS60QBU00<HaQBU6<H00QS611BU
00<HaQBU6<H01AS61QBU00<HaQBU6<H00aS600<LiaS66<H011S60QBU00@HaQBU5:DDY@<HaP035:DH
aQS600XHaP8DY@036<HDYAS600DHaP035:DHaQS600@HaP045:DHaQS65:D26<H25:D>6<H35:D76<H2
7>L00aS67>LLi`037>L88@P29BT00bU:;F/e[@02>Lh00c6<;F/YBP06:DX00bDY8@PQ20028@P:7>L=
6<H25:D:6<H00aBU6<HHaP046<H011BU6<HHaQBU0aS600DDYAS65:DHaQBU008HaP035:DHaQBU00LH
aP035:DHaQS600LHaP8DY@8HaP8DY@036<HDYAS6008HaP045:DHaQBU6<H25:D26<H25:D00aS65:DH
aP046<H01ABU6<HHaQS65:D00QS600@DYAS66<HDY@DHaP035:DHaQS600@HaP035:DHaQS600dHaP05
5:DHaQBU5:DHaP025:D46<H45:D00aS65:DDY@086<H00aBU6<HHaP036<H011BU6<HHaQBU0aS60QBU
1f]J0@000F]J00007fmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[
FVmkJeY[NV]JKg/HaQBU00<HaP035:DHaQS600PHaP035:DHaQS6008HaP035:DHaQS600DHaP8DY@03
6<HDYAS6008HaP@DY@036<HDYAS600DHaPHDY@036<HDYAS600<HaP037>LHaQS600@HaP8DY@046<HD
YABU5:D36<H00aBU6<HHaP0:6<H25:D00aS65:DHaP056<H00aBU6<HHaP046<H011BU6<HHaQBU0QS6
0QBU3QS60aBU1aS60QcW00<HaQcW7>L00acW22480RDY00<YBRe[=Jd00SW>00<aS2e[:DX01RU:00<U
:B488@P00R482QcW3AS60QBU2QS600<DYAS66<H011S600@DYAS66<HDY@<HaP055:DHaQBU6<HDY@02
6<H00aBU6<HDY@076<H00aBU6<HHaP076<H25:D26<H25:D00aS65:DHaP026<H011BU6<HDYAS60QBU
0QS60QBU00<HaQBU6<H011S600DDYAS66<HHaQBU008HaP045:DHaQS65:D56<H00aBU6<HHaP046<H0
0aBU6<HHaP0=6<H01ABU6<HDYABU6<H00QBU11S611BU00<HaQBU5:D021S600<DYAS66<H00aS600@D
YAS66<HDY@<HaP8DY@08Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J0QS600HDYAS6
6<HHaQBU6<H25:D36<H00aBU6<HHaP0<6<H00aBU6<HHaP036<H25:D26<H011BU6<HHaQBU11S600<D
YAS66<H01AS600<DYAS65:D00aS600<DYAS66<H00QBU21S60QBU00DHaQBU5:DHaQBU00@HaP<DY@LH
aP037>LHaQS6008DY@THaP037>LHaQS600<HaP035:DHaQS600LHaP037>LHaQS600hHaPTLi`038@PU
:BDY008U:@8YBP05<H`e[CF]=Jd]J`02:DX00bDY:DXU:@02:DX00bDY8@PQ200>7>L76<H00aBU6<HD
Y@076<H00aBU6<HHaP066<H00aBU6<HHaP086<H25:D<6<H35:D76<H25:D26<H00aBU6<HHaP036<H0
0aBU6<HHaP036<H00aBU6<HHaP076<H25:D01AS65:DDYABU6<H00QBU0QS60aBU0aS60QBU11S60QBU
1AS600<DYAS66<H00aS60aBU0aS60aBU0QS60QBU00<HaQBU5:D00aBU0aS60QBU11S60aBU00<HaQBU
5:D00aS611BU1QS600<DYAS66<H01f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX02AS600<DYAS65:D00QBU00<HaQBU6<H00aBU
3QS60QBU21S60QBU00<HaQBU6<H011S60aBU0QS60QBU0QS600<DYAS65:D00QS600<DYAS66<H01QS6
00<DYAS66<H04AS600HDYAS66<HDYAS65:DB6<H27>L56<H45:D00aS65:DHaP066<H00acW6<HHaP05
6<H00acW8@PQ20028@P39BT00bU:<H`aS002<H`00be[9BTU:@059BT58@P:7>L011S67>LHaQBU51S6
00DDYAS66<HDYAS600<DY@8HaP8DY@@HaP8DY@<HaP<DY@DHaP8DY@056<HDYAS65:DHaP035:D36<H2
5:D00aS65:DDY@026<H25:D00aS65:DDY@056<H00aBU6<HHaP026<H25:D46<H35:D56<H011BU6<HD
YAS60QBU1QS600<DYAS66<H00aS600<DYAS65:D00QS60QBU1QS600@DYAS65:DDY@@HaP045:DHaQS6
5:D46<H01ABU6<HDYABU6<H00QBU00DHaQBU6<HHaQBU00LHaP8DY@DHaP08JgY[FVmkJeY[NV]JKg/0
005[NP005V]J00<006]JJeX016]J1QS600<DYAS65:D01QS600<DYAS65:D03AS600<DYAS65:D00QS6
00HDYAS65:DDYAS65:D36<H00aBU6<HDY@0=6<H011cW6<HHaQBU11S600<DYAS65:D01aS600@DYAS6
5:DDY@<HaP035:DHaQS600`HaP035:DHaQBU00dHaP8DYALHaPDLi`036<HQ21cW00<Q20<U:@03:DX]
Jbe[00<YBP<U:@038@PU:BDY008U:@<Q20/Li`036<HLiacW01<HaP8DY@8HaP045:DHaQS65:D26<H0
1ABU6<HDYAS65:D00QS600<DYAS65:D00QS600LDYAS66<HHaQBU6<HDY@046<H00aBU6<HHaP056<H0
0aBU6<HDY@066<H00aBU6<HHaP066<H00aBU6<HHaP025:D86<H25:D00aS65:DHaP035:D01AS65:DH
aQS65:D01AS600<DYAS66<H00QS611BU1aS600<DYAS66<H011S60QBU0QS60QBU00@HaQBU5:DHaP8D
Y@<HaP035:DHaQBU00HHaP035:DHaQS600PHaP8DY@<HaPM[FP40005[FP0001i_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:D26<H35:D46<H45:D0
0aS65:DHaP035:D46<H01aBU6<HHaQS65:DHaQBU00DHaP035:DHaQBU008HaP055:DHaQS65:DHaP03
5:D86<H45:D36<H00aBU6<HHaP025:D36<H00aBU6<HHaP036<H00aBU6<HHaP025:DC6<H00aBU6<HH
aP056<H02ABU6<HHaQS65:DHaQBU6<HDY@056<H25:DE6<H47>L48@P29BT5:DX01BDY8@PU:B489BT0
0R4800<Lib488@P00R481AcW00<HaQcW6<H03QS600<LiaS66<H01QS600<DYAS66<H011BU1AS600<D
YAS66<H00aBU00@HaQBU6<HHaP8DY@DHaP035:DHaQBU008HaP@DY@046<HDYABU5:D26<H00aBU6<HD
Y@0<6<H00aBU6<HDY@046<H00aBU6<HHaP025:D36<H01ABU6<HDYAS65:D00aS600<DYAS66<H00QBU
0QS600DDYAS66<HDYAS6008DY@036<HDYABU00HHaP035:DHaQBU00@HaP045:DHaQBU5:D26<H25:D0
0aS65:DDY@025:D46<H00aBU6<HHaP035:D26<H00aBU6<HHaP046<H011BU6<HHaQS60aBU0aS600Q_
Nf]JJgY[FVmkJeY[NP000Fmk000FJeX30004JeX25:D36<H011BU6<HHaQS60QBU1aS60QBU00<HaQBU
6<H01AS600<DYAS65:D021S600<DYAS66<H00QBU2AS600<DYAS66<H031S600<DYAS65:D02QS60QBU
00<HaQBU6<H01QS60QBU0QS60acW0QS600<DYAS66<H01aS600<DYAS66<H01AS600@DYAS66<HDY@dH
aP8Li`THaPHLi`<Q20<YBPLU:@@Q20037>LQ21cW00PLia/HaP037>LDYAS600HHaP045:DHaQS66<H3
5:D56<H25:D26<H00aBU6<HDY@046<H85:D76<H00aBU6<HHaP046<H25:D26<H011BU6<HDYABU0QS6
00@DYAS66<HDY@<HaP8DY@HHaP8DY@<HaP8DY@8HaP035:DHaQS600DDY@036<HDYAS6008DY@036<HD
YAS6008DY@<HaP055:DHaQS66<HDY@056<H25:D00aS65:DDY@025:D36<H011BU6<HHaQBU0aS600<D
YAS66<H02QS60QBU1V]J0P000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDY@@HaP035:DHaQS600DHaP8DY@<HaP8DY@<HaP8DY@<HaP04
5:DHaQBU6<H25:D46<H00aBU6<HHaP036<H00aBU6<HDY@026<H00aBU6<HHaP026<H011BU6<HDYABU
0QS600DDYAS65:DHaQBU008HaP035:DHaQBU008HaP<DY@8HaP8DY@8HaP@DY@THaP035:DHaQS6008L
i`045:DHaQS66<H45:D01AS65:DHaQBU6<H00QBU0QS600<DYAS66<H01AS600@DYAS66<HHaP8DYA0H
aPPLi`8Q20HU:@PQ20<Li`038@PLiacW008Li`8HaP8Li`DHaP037>LHaQS601<HaP035:DHaQS600@D
Y@<HaP075:DHaQS66<HDYAS65:D00aS60QBU00<HaQBU6<H00QS600<DYAS66<H00QBU00<HaQBU5:D0
11BU00<HaQBU6<H00QBU0aS600@DYAS66<HHaP<DY@036<HDYAS600HHaP<DY@066<HDYABU5:DHaQBU
1AS600<DYAS65:D00QS60aBU0QS600<DYAS66<H01AS600<DYAS66<H00QBU00@HaQBU6<HHaP8DY@8H
aP<DY@LHaP035:DHaQS600<DY@<HaP8DY@@HaP035:DHaQBU00@HaP035:DHaQBU008DY@DHaP08JgY[
FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00<DYAS66<H00QBU0aS600<DYAS65:D01AS6
0QBU0aS600DDYAS66<HHaQBU00<HaP8DY@8HaP035:DHaQS6008HaP8DY@046<HDYABU5:D26<H35:D0
1AS65:DDYAS65:D02aS60aBU0QS600<DYAS66<H00aS600DDYAS66<HHaQBU008HaP@DY@036<HDYAS6
00DHaP035:DHaQS600<HaP035:DHaQS600dHaP035:DHaQS600DHaP035:DHaQS600/HaP037>LHaQS6
00DHaPHLi`8Q208U:@04:DXU:BDY9BT98@P97>L86<H00acW6<HHaP0=6<H00aBU6<HHaP066<H00aBU
6<HDY@025:D36<H35:D26<H011BU6<HHaQS60QBU11S61aBU2QS600<DYAS66<H00aS60QcW1aS600<D
YAS66<H00aS600<DYAS66<H00QS600DDYAS65:DHaQBU00PHaP085:DHaQS65:DHaQBU6<HDY@8HaP05
5:DHaQS65:DHaP045:D46<H00aBU6<HHaP0@6<H00aBU6<HHaP026<H45:D011S65:DHaQBU1AS600<D
YAS66<H00QBU00<HaQBU6<H01f]J0@000F]J00007VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DY@8HaP8DY@<HaP035:DHaQBU00DHaP8DY@<HaP05
5:DHaQS66<HDY@036<H25:D26<H00aBU6<HHaP026<H25:D011S65:DDYABU0QS60aBU00DHaQBU5:DH
aQBU00/HaP<DY@8HaP035:DHaQS600<HaP055:DHaQS66<HDY@026<H45:D00aS65:DHaP056<H00aBU
6<HHaP036<H00aBU6<HHaP0=6<H00aBU6<HHaP056<H00aBU6<HHaP0;6<H00acW6<HHaP056<H67>L2
8@P29BT012U:9BTU:BDY2B482AcW21S600<LiaS66<H03AS600<DYAS66<H01QS600<DYAS65:D00QBU
0aS60aBU0QS600@DYAS66<HHaP8DY@@HaPLDY@XHaP035:DHaQS600<HaP8Li`LHaP035:DHaQS600<H
aP035:DHaQS6008HaP055:DHaQBU6<HDY@086<H021BU6<HHaQBU6<HDYAS65:D26<H01ABU6<HHaQBU
6<H011BU11S600<DYAS66<H041S600<DYAS66<H00QS611BU00@HaQBU6<HDY@DHaP035:DHaQS6008D
Y@0;6<HDYAS6Kg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP035:DHaQS6008HaP06
5:DHaQS66<HDYAS60QBU0aS60QBU0aS600DDYAS66<HHaQBU008HaP045:DHaQS66<H75:D00aS65:DD
Y@025:D26<H011BU6<HHaQS60QBU0QS600<DYAS66<H00aS600@DYAS65:DDY@@HaP035:DHaQBU00DH
aP055:DHaQS65:DHaP025:D36<H00aBU6<HHaP056<H00aBU6<HHaP066<H00aBU6<HHaP026<H00aBU
6<HHaP086<H01ABU6<HHaQS65:D01aS600<DYAS66<H02aS600@LiaS67>LHaP@Li`8Q20@U:@LQ20@L
i`038@PLiacW00LLi`LHaP035:DHaQS600<HaP035:DHaQS600dHaP035:DHaQS6008HaP<DY@XHaP03
5:DHaQS600<HaP035:DHaQS600DHaP8DY@036<HDYAS6008HaP035:DHaQS600LHaP8Li`<HaP035:DH
aQS6008HaP8DY@<HaP8DY@036<HDYABU00HHaP8DY@8HaP045:DHaQBU5:D66<H00aBU6<HHaP096<H2
5:D36<H35:DJ6<H01ABU6<HHaQS65:D01AS60aBU0QS61f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX02QS600<DYAS66<H011S6
0aBU00<HaQBU6<H01AS600<DYAS65:D00QS60aBU0QS600<DYAS66<H02aS600@DYAS66<HDY@8HaP8D
Y@@HaP075:DHaQS66<HDYAS65:D021S600<DYAS66<H031S600<DYAS66<H00QS60QBU2QS60aBU0QS6
00<DYAS66<H01AS60QBU21S600DDYAS66<HHaQcW00HHaP8Li`036<HLiacW00<Li`8Q20<U:@HQ208L
i`078@PLiacW7>LQ21cW6<H00QcW0QS600@Lib488@PLiaHHaP8DY@DHaP035:DHaQS600<HaP8DY@DH
aP8DY@<HaP035:DHaQS600@HaP035:DHaQS600@HaP035:DHaQS6008DY@<HaP035:DHaQS6008DY@05
6<HDYAS66<HLi`026<H00aBU6<HDY@056<H00aBU6<HHaP036<H25:D00aS65:DHaP025:D01AS65:DD
YABU6<H01QBU00<HaQBU6<H01ABU2AS600@DYAS66<HHaP<DY@XHaP035:DHaQS600@HaP8DY@066<HD
YABU6<HDYAS60aBU0aS60QBU00<HaQBU6<H00QBU0aS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX0
0`00JeY[FP04JeX56<H01QBU6<HDYABU6<HLi`<DY@<HaP035:DHaQS600<DY@LHaP035:DHaQS600DH
aP<DY@036<HDYABU008DY@HHaP035:DHaQBU00HHaP045:DHaQBU5:D46<H00aBU6<HHaP036<H00aBU
6<HHaP036<H25:D36<H00aBU6<HDY@0;6<H35:D00aS65:DDY@0>6<H25:DH6<H57>L28@P00acW9BTU
:@028@P011cW8@PQ21cW0R4800<HaR487>L01QcW00DHaQcW6<HHaQcW008HaP037>LHaQS601DHaP03
5:DHaQS600hHaP@DY@LHaP055:DHaQS65:DHaP055:D56<H55:D<6<H35:D86<H35:D00aS65:DHaP06
6<H25:D:6<H25:D36<H00aBU6<HHaP036<H00aBU6<HHaP046<H00aBU6<HHaP066<H35:D46<H01ABU
6<HHaQS65:D011S600@DYAS65:DDY@<HaP<DY@036<HDYAS600@HaPM[FP40005[FP0001e_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk008HaP035:DH
aQS600@HaP035:DHaQS600<HaPDDY@<HaP035:DHaQS600PHaP@DY@036<HDYABU00DHaP<DY@DHaP8D
Y@8HaP8DY@8HaP035:DHaQS600HHaP8DY@<HaP<DY@@HaP8DY@036<HDYABU010HaP<DY@036<HDYAS6
00dHaP035:DHaQS600XHaP8DY@DHaP8Li`<HaPHLi`8Q20039BTQ224800<Li`8Q20DLi`046<HLiacW
7>L66<H00acW6<HLi`027>L86<H00aBU6<HHaP0>6<H00aBU6<HHaP056<H35:D56<H25:D26<H25:D0
0aS65:DHaP076<H25:D00aS65:DHaP056<H00aBU6<HHaP026<H00aBU6<HHaP036<H00aBU6<HDY@09
6<H25:D26<H00aBU6<HDY@026<H45:D011S65:DDYABU0aS600<DYAS65:D00QS611BU0aS600HDYAS6
6<HDYAS65:D56<H00aBU6<HHaP025:D=6<H00aBU6<HHaP046<H35:D46<H45:D00aS65:DDY@036<H0
26mkJeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FPLHaP035:DHaQS600<HaP8DY@LHaP8D
Y@XHaP@DY@046<HDYAS66<H35:D36<H011BU6<HHaQS60QBU1QS60aBU00DHaQBU6<HHaQBU00<HaP8D
Y@@HaP035:DHaQS600<HaP035:DHaQBU00<HaP<DY@<HaP055:DHaQS66<HDY@0F6<H00aBU6<HHaP08
6<H27>L;6<H27>L00aS67>LLi`038@P01AcW8@PQ22487>L00R481QcW0QS60acW00<HaQcW7>L00QS6
00<LiaS66<H011S600DLiaS66<HHaQBU00HHaP037>LHaQS600PHaP<DY@DHaP<DY@lHaP035:DHaQS6
00@HaP065:DHaQS65:DHaQBU31S600<DYAS66<H00QS600<DYAS66<H00QS600<DYAS66<H00aS60QBU
00<HaQBU5:D00QS600@DYAS66<HHaPHDY@8HaP8DY@PHaP8DY@HHaP035:DHaQBU00@HaP035:DHaQS6
00DHaP065:DHaQS66<HDYAS60QBU0QS600<DYAS66<H02aS600<DYAS66<H01AS600<DYAS6JeX01V]J
0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]j
JeY_Nf]JJgXDY@PHaP035:DHaQBU008DY@@HaP@DY@046<HDYAS65:D26<H00aBU6<HDY@026<H01QBU
6<HHaQBU6<HDY@<HaP8DY@@HaP065:DHaQBU5:DHaQBU0aS60aBU0aS600@DYAS65:DDY@<HaP035:DH
aQBU008HaP035:DHaQBU00DHaP035:DHaQS600<HaP8DYA8HaP035:DHaQS600HHaP045:DHaQBU5:D2
6<H25:D46<H00aBU6<HHaP056<H01ABU7>LHaQS67>L01AS61AcW1B4800<Lib488@P00QcW0R4800HH
aQcW6<HHaQcW6<H27>L01aS67>LHaQS67>LHaQcW01@HaP055:DHaQS66<HDY@056<H011BU6<HHaQBU
1AS600@DYAS65:DDY@PHaP8DY@XHaP@DY@<HaP035:DHaQBU008HaP045:DHaQBU5:D56<H35:D021S6
5:DDYAS65:DHaQBU6<H35:D01AS65:DDYAS65:D021S600<DYAS66<H00QBU0QS600@DYAS66<HDY@@H
aP065:DHaQS65:DHaQBU0QS600<DYAS66<H00aS600<DYAS65:D01QS600<DYAS66<H00QS600<DYAS6
6<H01ABU00@HaQBU6<HDY@DHaP035:DHaQBU00<HaP8DY@08JgY[FVmkJeY[NV]JKg/0005[NP0000=[
FP0000000P000f]J0P0016]J0P001V]J00<006]JJeX016]J00<HaQBU6<H00QBU1AS60aBU11S60aBU
00DHaQBU6<HDYAS600<DY@036<HDYABU008DY@@HaP035:DHaQS6008HaP8DY@DHaP<DY@036<HDYAS6
008HaPHDY@046<HDYAS66<H25:D00aS65:DHaP046<H00aBU6<HHaP0?6<H00aBU6<HHaP026<H00aBU
6<HHaP096<H00aBU6<HHaP036<H00aBU6<HHaP025:D=6<H00acW6<HHaP036<H011cW6<HHaQS60QcW
0QS61AcW0b4800<Lib487>L01AcW00<HaQcW6<H00aS600@LiaS66<HHaP8Lia0HaP045:DHaQS65:D<
6<H25:D56<H00aBU6<HHaP056<H25:D56<H011BU6<HHaQBU2QS60QBU1AS600<DYAS66<H01aS600<D
YAS66<H00QBU1AS60QBU0QS60QBU0aS600<DYAS66<H00aS600DDYAS66<HDYAS600DDY@DHaP055:DH
aQS65:DHaP035:D26<H25:D46<H25:D66<H011BU6<HHaQBU0aS600DDYAS66<HDYAS6008DY@046<HD
YABU5:D26<H25:D00aS65:DHaP056<H00aBU6<HHaP07JeX10001JeX0000PKg/006]jJeY_Nf]JJgX0
06mkJeX006]JKg/006]jJeX006]JJgY[FVmkJeX006]JKg][FV]jJeY_NaS65:DHaP8DY@DHaP<DY@@H
aP<DY@056<HDYAS65:DHaP035:D00aS65:DDY@025:D46<H00aBU6<HHaP026<H25:D56<H35:D00aS6
5:DHaP026<H65:D011S65:DHaQS60QBU00<HaQBU6<H011S600<DYAS66<H03aS600<DYAS66<H00QS6
00<DYAS66<H02AS600<DYAS66<H00aS600<DYAS66<H00QBU3AS600<LiaS66<H00aS600@LiaS66<HH
aP8Li`8HaPDLi`<Q20037>LQ21cW00DLi`036<HLiaS600<HaP047>LHaQS66<H27>L@6<H011BU6<HH
aQBU31S60QBU1AS600<DYAS66<H01AS60QBU1AS600@DYAS66<HDY@XHaP8DY@DHaP035:DHaQS600LH
aP035:DHaQS6008DY@DHaP8DY@8HaP8DY@<HaP035:DHaQS600<HaP055:DHaQS65:DHaP055:D56<H0
1ABU6<HHaQBU6<H00aBU0QS60QBU11S60QBU1QS600@DYAS66<HDY@<HaP055:DHaQS65:DHaP025:D0
11S65:DDYABU0QS60QBU00<HaQBU6<H01AS600/DYAS66<I_Nf]JJgY[FVmkJeY[NP00005_N`000V]J
00<006]JJeX01F]J00@006]JJeX0009[FP03001[FV]J00=[FP@000=[FP035:DHaQS6008DY@DHaP04
5:DHaQS65:D36<H35:D:6<H00aBU6<HHaP086<H01ABU6<HHaQS65:D00aS600<DYAS66<H00QS600@D
YAS66<HDY@8HaP8DY@<HaP8DY@8HaP8DY@8HaP8DY@<HaP035:DHaQS600PHaP8DY@THaP035:DHaQBU
00<HaP035:DHaQS600DHaP035:DHaQS600<HaP8DY@/HaP8Li``HaPHLi`038@PLib4800PLi`036<HL
iaS600<Li`036<HLiaS600HHaP035:DHaQS601DHaP035:DHaQS600@HaP035:DHaQS600HHaP055:DH
aQS66<HDY@076<H00aBU6<HHaP026<H00aBU6<HHaP025:D36<H00aBU6<HHaP026<H011BU6<HHaQBU
1AS600<DYAS66<H01AS600<DYAS66<H00QS61aBU00<HaQBU6<H03aS60aBU0aS60aBU1aS60QBU00@H
aQBU5:DHaP8DY@<HaP8DY@@HaP035:DHaQS600<HaP<DY@DHaP<DY@8HaP035:DHaQS600PHaPE[FP<0
005[FP0000Q[NV]JKg/006]jJeY_Nf]J0P0001=_Nf]JJgX006mkJeX006]JKg][FV]jJeX006]JJgY[
FVmkJeY[NP056<H01ABU6<HHaQS65:D02aS600<DYAS66<H00QS60aBU00@HaQBU6<HDY@<HaP<DY@DH
aP065:DHaQBU6<HDYAS60QBU1QS60aBU00<HaQBU5:D00aS600<DYAS66<H021S600<DYAS66<H00QS6
00HDYAS66<HDYAS65:D26<H00aBU6<HHaP026<H011BU6<HHaQS60aBU21S600@DYAS66<HDY@LHaP06
5:DHaQS65:DHaQBU1aS600<LiaS66<H021S600@LiaS67>LHaPlLi`DHaP8Lib0HaP035:DHaQS600LH
aP8DY@036<HDYAS6008DY@<HaP035:DHaQS600@HaP055:DHaQS66<HDY@076<H00aBU6<HHaP026<H0
1ABU6<HHaQBU6<H00QBU0aS60QBU1AS60QBU0aS600@DYAS66<HDY@HHaP8DY@8HaP8DY@<HaP035:DH
aQBU00<HaP035:DHaQS600HDY@036<HDYAS600<DY@046<HDYAS66<H35:D46<H01QBU6<HDYABU6<HD
Y@<HaP037>LDYABU008DY@8HaP045:DHaQS65:D36<H55:D00aS65:DHaP046<H02aBU6<HHaV]jJeY_
Nf]JJgY[FVmk00000F]j00001F]J001[FV]J00000f]J00<006]JJeX00V]J00@006]JJeX000E[FP03
001[FV]J00A[FPDHaP8DY@036<HDYABU00<DY@DHaP8DY@@HaP035:DHaQS6008HaP<DY@DHaP045:DH
aQS65:D96<H35:D;6<H45:D011S65:DHaQS60aBU1QS60QBU2AS60aBU1AS600@DYAS66<HHaP<DY@@H
aP047>LDYABU5:D26<H00aBU6<HHaP035:D96<H011BU6<HHaQS60QcW21S600DLiaS66<HLiaS600LL
i`046<HLiacW7>L<6<H00acW6<HHaP0:6<H00acW6<HHaP036<H35:D00aS65:DHaP076<H25:D46<H0
0aBU6<HHaP026<H45:D86<H25:D26<H00aBU6<HHaP056<H00aBU6<HHaP035:D36<H00aBU6<HHaP04
6<H00aBU6<HHaP056<H35:D66<H25:D66<H25:D76<H35:D011S65:DHaQBU0aS600<DYAS66<H00QS6
00@DYAS66<HDY@8HaP035:DHaQS6008DY@036<HDYAS600<HaP8DY@8HaP035:DHaQBU00<DY@8HaP03
5:DHaQBU00LHaP035:DHaQS600<HaP8DY@<HaP035:DHaQS600M[FP40005[FP0000Q_Nf]J000006mk
JeY[NV]J0`0000=[FVmkJeX00P0000e_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`036<H25:D00aS6
5:DDY@035:D;6<H011BU6<HHaQBU0QS600<DYAS65:D00aS60QBU0QS600<DYAS66<H01QS600<DYAS6
5:D00QBU00@HaQBU5:DDY@HHaP035:DHaQS6008HaP045:DHaQS65:D26<H25:D96<H25:D26<H01ABU
6<HHaQBU6<H00QBU1QS600<DYAS65:D01aS600<DYAS66<H00QS600<DYAS66<H011S600<DYAS66<H0
21S600<LiaS66<H00aS600<LiaS66<H01AcW00<HaQcW6<H00acW00@HaQcW7>LHaP<Li`036<HLiacW
00HHaP037>LHaQS600LHaP037>LHaQS6008HaP035:DHaQS600THaP035:DHaQS6008HaP<DY@HHaP03
5:DHaQBU00LHaPDDY@036<HDYABU00DHaP8DY@HHaP035:DHaQS600<HaP035:DHaQBU00PHaP035:DH
aQS600<HaP035:DHaQBU00@DY@046<HDYAS65:D86<H25:D<6<H65:D86<H00aBU6<HHaP025:D36<H0
0aBU6<HHaP025:D00aS65:DHaP026<H011BU6<HHaQBU2aS600DDYAS66<HHaQBU00@HaP<DY@8HaP08
Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J00<DYAS66<H01QS600<DYAS66<H00QS6
0QBU2QS60QBU1QS60QBU00HHaQBU6<HDYAS65:D46<H00aBU6<HHaP046<H25:D26<H35:D01AS65:DH
aQS65:D00aS600<DYAS66<H011S600HDYAS66<HHaQBU6<H35:D01AS65:DHaQS65:D00aS600<DYAS6
5:D00QBU0QS600DDYAS66<HHaQBU00<HaP035:DHaQS6008HaP8DY@8HaP045:DHaQS65:D26<H25:D7
6<H00aBU6<HHaP0E6<H67>L00aS67>LHaP036<H27>L<6<H00aBU6<HHaP025:D96<H00aBU6<HHaP04
6<H00aBU6<HHaP056<H00aBU6<HHaP036<H00aBU6<HHaP056<H45:D36<H011BU6<HDYAS60QBU0QS6
00@DYAS66<HDY@8HaP8DY@036<HLiaS6008HaP055:DHaQS66<HDY@036<H25:D00aS65:DHaP036<H3
5:D76<H25:D46<H25:D26<H25:D76<H00aBU6<HHaP026<H35:D26<H35:D26<H55:D01AS65:DHaQS6
5:D04QS600<DYAS67>L01AS600@DYAS66<HDY@HHaP035:E[FV]J00E[FP40005[FP0001i[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:D66<H011BU
6<HHaQBU2AS600DDYAS65:DHaQBU00<HaP035:DHaQBU008HaP075:DHaQS66<HDYAS65:D00QS60QBU
11S600<DYAS65:D00QBU0QS600<DYAS66<H00QBU0QS600<DYAS65:D01AS600<DYAS66<H00aBU00<H
aQBU6<H01AS600@DYAS65:DHaP@DY@8HaP<DY@@HaP035:DHaQS6008DY@PHaP8DY@<HaP035:DHaQS6
00@HaP047>LHaQcW7>L96<H00acW6<HHaP036<H27>L66<H47>L@6<H27>L66<H00aBU6<HHaP036<H0
0aBU6<HHaP026<H25:D00aS65:DHaP0>6<H25:D36<H00aBU6<HHaP056<H25:D36<H35:D011S65:DH
aQBU0aS600<DYAS65:D01QS600<DYAS66<H00QS60aBU31S60QBU0QS60QBU1QS600@DYAS66<HDY@LH
aP055:DHaQS66<HDY@036<H35:D00aS65:DHaP026<H55:D00aS65:DHaP025:D26<H011BU6<HHaQS6
0QBU0QS600<DYAS66<H01AS600<DYAS66<H02QS600DDYAS66<HHaQBU00HHaP08JgY[FVmkJeY[NV]J
Kg/0005[NP005V]J00<006]JJeX016]J0QS600<DYAS66<H02aS600<DYAS66<H00aS600<DYAS66<H0
0aS600<DYAS66<H00aS60QBU00@LiaS66<HDY@LHaP035:DHaQBU00@HaP035:DHaQBU00<DY@<HaP06
5:DHaQBU5:DHaQBU0QS600<DYAS66<H02QS600<DYAS65:D00QBU11S600@DYAS66<HDY@8HaP8DY@<H
aP8DY@8HaP035:DHaQS600<HaP035:DHaQS600THaP037>LHaQS600LHaP<Li`LHaP037>LHaQcW00DH
aP037>LHaQcW008HaP8Lia@HaP035:DHaQS600DHaP035:DHaQS600@HaP035:DHaQS600LHaP8DY@8H
aP035:DHaQBU00LHaP045:DHaQS66<H45:D00aS65:DDY@056<H00aBU6<HHaP035:D00aS65:DHaP04
6<H00aBU6<HHaP046<H011BU6<HHaQS60QBU00HHaQBU5:DDYAS65:D66<H00aBU6<HHaP026<H01ABU
6<HHaQS65:D011S600@DYAS66<HHaP8DY@056<HDYAS65:DHaP045:D00aS65:DHaP066<H00aBU6<HH
aP025:D36<H25:D36<H00aBU6<HHaP0D6<H00aBU6<HHaP066<H00aBU6<I[FP06JeX10001JeX0000M
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`02
6<H00aBU6<HHaP0;6<H00aBU6<HHaP036<H00aBU6<HHaP036<H00aBU6<HHaP036<H25:D011cW6<HH
aQBU1aS600<DYAS65:D011S600<DYAS65:D00aBU0aS600HDYAS65:DDYAS65:D26<H00aBU6<HHaP0:
6<H00aBU6<HDY@025:D46<H011BU6<HHaQBU0QS60QBU0aS60QBU0QS600<DYAS66<H00aS600<DYAS6
6<H02AS600<LiaS66<H01aS60acW1aS600<LiaS67>L01AS600<LiaS67>L00QS60QcW51S600<DYAS6
6<H01AS600<DYAS66<H011S600<DYAS66<H01aS60QBU0QS600<DYAS65:D01aS600@DYAS66<HHaP@D
Y@036<HDYABU00DHaP035:DHaQS600<DY@036<HDYAS600@HaP035:DHaQS600@HaP045:DHaQS66<H2
5:D01QS65:DDYABU6<HDY@HHaP035:DHaQS6008HaP055:DHaQS66<HDY@046<H011BU6<HHaQS60QBU
00DHaQBU6<HDYAS600@DY@036<HDYAS600HHaP035:DHaQS6008DY@<HaP8DY@<HaP035:DHaQS601@H
aP035:DHaQS600HHaP0:5:DHaVmkJeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FP/HaP8D
Y@dHaP<DY@@HaP045:DHaQS65:D36<H00aBU6<HHaP066<H35:D01AS65:DHaQS65:D01QS600LDYAS6
6<HHaQBU6<HDY@056<H01ABU6<HHaQBU6<H00aBU0QS600@DYAS65:DHaP@DY@046<HDYABU6<H35:D0
11S65:DHaQBU11S60aBU;aS600<LiaS66<H021S60QcW41S60QBU1QS600<DYAcW6<H011BU0QS60QBU
1AS60aBU0QS600<DYAS66<H00QS60QBU00HHaQBU6<HDYAS65:D36<H25:D00aS65:DDY@046<H011BU
6<HHaQS60QBU1AS600<DYAcW7>L00QS600<DYAS66<H021S60QBU00<HaQBU6<H02AS600@DYAS65:DD
Y@DHaP<DY@8HaP8DY@HHaP8DY@<HaP8DY@046<HDYAS66<H55:D26<H35:D66<H25:D56<H25:D00aS6
5:DHaP036<H25:D;6<H7JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP036<H25:D96<H25:D26<H25:D:6<H00aBU6<HHaP086<H3
5:D011S65:DHaQBU1aS600<DYAS66<H00QS600DDYAS66<HHaQBU008HaP<DY@036<HDYAS6008HaP8D
Y@HHaP035:DHaQS6008DY@@HaP035:DHaQBU00DHaP<DY@HHaP035:DHaQS600<HaP035:DHaQS6008H
aP<DY@PHaP035:DHaQS600HHaP037>LHaQS600`HaP047>LHaQS66<H27>L00aS67>LHaP036<H27>L;
6<H011BU6<HHaQS60aBU0aS60QBU21S60QBU21S600@DYAS66<HHaP<DY@036<HDYAS600PHaP<DY@<H
aP035:DHaQS6008DY@8HaP045:DHaQBU5:D36<H25:D66<H00aBU6<HHaP056<H00aBU6<HHaP0;6<H3
5:D76<H25:D26<H45:D26<H35:D;6<H25:D36<H25:D00aS65:DDY@035:D36<H00aBU6<HHaP025:D0
1AS65:DHaQS65:D01AS600<DYAS66<H00QS60aBU1AS600/DYAS66<I[NV]JKg][FV]jJeY_N`00005[
NP005V]J00<006]JJeX016]J0QBU1AS600<DYAS66<H00aS600@DYAS65:DDY@hHaP035:DHaQS600LH
aP035:DHaQS600<HaP8DY@036<HDYABU00@DY@8HaP8DY@8HaP035:DHaQS600<HaP8DY@036<HDYABU
00DHaP035:DHaQS600<HaP<DY@036<HDYABU008DY@036<HDYAS600@HaP035:DHaQBU00THaP<DY@PH
aP035:DHaQBU00HHaP035:DHaQS600hHaP037>LHaQS6024HaP045:DHaQS65:D66<H00aBU6<HHaP08
6<H00aBU6<HHaP026<H00aBU6<HDY@055:D86<H00aBU6<HDY@066<H25:D26<H00aBU6<HHaP025:D0
0aS65:DDY@046<H011BU6<HDYABU11S600<DYAS66<H00QS600<DYAS65:D01QS600<DYAS66<H00QBU
0aS600<DYAS66<H01QBU00<HaQBU5:D00QBU0QS600DDYAS66<HHaQBU00<HaP055:DHaQBU5:DHaP02
5:D36<H25:D00aS65:DDY@065:D00aS65:DHaP046<H00aBU6<HHaP0>6<H011BU6<HHaQBU11S600<D
YF]JJeX01F]J0@000F]J00005VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]J0`0000A[FV]jJeY_N`8DY@036<HDYABU00DHaP035:DHaQS600<HaP035:DHaQS6008HaP035:DH
aQBU008HaP<DY@036<HDYAS600@DY@XHaP045:DHaQS65:D26<H01QBU6<HDYAS65:DHaP8DY@<HaP03
5:DHaQS600HHaPHDY@DHaP035:DHaQS600THaP8DY@8HaP055:DHaQS66<HDY@0:6<H011BU6<HHaQS6
0QBU0aS600DDYAS66<HDYAS6008DYB0HaP057>LHaQS66<HDY@036<H25:D26<H00aBU6<HHaP086<H0
11BU6<HDYABU1QS60QBU00<HaQBU5:D03AS600HDYAS66<HDYAS65:D<6<H25:D011S65:DHaQBU0QS6
00<DYAS65:D00QBU1aS600DDYAS66<HHaQBU00@HaP8DY@<HaP035:DHaQS600PHaP8DY@<HaP8DY@@H
aP8DY@8HaP035:DHaQS600<HaP065:DHaQS66<HDYAS60aBU00HHaQBU5:DDYAS65:D26<H55:D011S6
5:DHaQBU1AS600<DYAS66<H00aS60aBU0aS60aBU0QS600<DYAS66<H00QBU00@HaQBU6<HDY@DHaP06
Kg][FV]jJeY_Nf]J0P000Fmk000FJeX00`00JeY[FP04JeX36<H25:D66<H25:D011S65:DHaQBU1aS6
00HDYAS65:DDYAS65:D36<H011BU6<HHaQBU2AS600<DYAS66<H011S60QBU0aS60QBU11S600HDYAS6
6<HHaQBU6<H25:D36<H00aBU6<HHaP036<H25:D86<H25:D26<H00aBU6<HDY@025:D00aS65:DHaP03
6<H25:D011S65:DHaQS60QBU1QS60aBU0QS600<DYAS65:D02aS600<LiaS66<H00aS600<LiaS66<H0
7AS611BU0aS60aBU1aS600<DYAS66<H011S60aBU0QS60QBU00@HaQcW6<HHaP8DY@8HaP035:DHaQBU
008HaP<DY@@HaP035:DHaQS6008HaP035:DHaQS600<HaP<DY@XHaPHDY@036<HDYAS600@HaP035:DH
aQS600PHaP@DY@046<HDYAS66<H35:D00aS65:DHaP026<H45:D26<H35:D36<H00aBU6<HDY@045:D0
11S65:DDYAS60QBU00<HaQBU6<H00QBU00<HaQBU5:D01AS60aBU00<HaQBU6<H011BU00<HaQBU6<H0
0aS60aBU2AS600<DYF]JJeX01F]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX00aBU31S60QBU0aS600<DYAS66<H00aS60QBU00HH
aQBU6<HHaQBU6<H35:D86<H01ABU6<HHaQBU6<H00QBU00<HaQBU6<H021S600DDYAS65:DDYAS6008D
Y@036<HDYAS600<DY@<HaP8DY@@HaP035:DHaQS600<HaP035:DHaQBU008DY@<HaPDDY@`HaP035:DH
aQS600DHaP035:DHaQS601/HaP035:DHaQS6008HaP8DY@036<HDYAS600HHaP035:DHaQS6008HaP@D
Y@036<HDYAS600@HaP8DY@036<HDYAS600DHaP035:DHaQS600dHaP035:DHaQS6010HaP055:DHaQS6
6<HDY@036<H00aBU6<HDY@076<H00aBU6<HDY@076<H00aBU6<HHaP036<H00aBU6<HHaP066<H011BU
6<HDYABU0aS600<DYAS66<H00QBU0QS60aBU0aS60QBU0aS60QBU0QS600<DYAS65:D011BU00@HaQBU
6<HHaPDDY@<HaPDDY@XHaP055:DHaQS65:DHaP035:D66<H00aBU6<HDY@025:D26<H02ABUJgY[FVmk
JeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP<DY@`HaP8DY@<HaP035:DHaQS600<HaP8DY@06
6<HDYAS66<HDYAS60aBU21S600DDYAS66<HDYAS6008DY@036<HDYAS600PHaP055:DHaQBU5:DHaP02
5:D00aS65:DHaP035:D36<H25:D46<H00aBU6<HHaP036<H00aBU6<HDY@025:D36<H55:D<6<H00aBU
6<HHaP056<H00aBU6<HHaP0K6<H00aBU6<HHaP026<H25:D00aS65:DHaP066<H00aBU6<HHaP026<H4
5:D00aS65:DHaP046<H25:D00aS65:DHaP056<H00aBU6<HHaP0=6<H00aBU6<HHaP0@6<H01ABU6<HH
aQS65:D00aS600<DYAS65:D01aS600<DYAS65:D01aS600<DYAS66<H00aS600<DYAS66<H01QS600@D
YAS65:DDY@<HaP035:DHaQS6008DY@8HaP<DY@<HaP8DY@<HaP8DY@8HaP035:DHaQBU00@DY@046<HD
YAS66<H55:D36<H55:D:6<H01ABU6<HHaQBU6<H00aBU1QS600<DYAS65:D00QBU0QS600<DYF]JJeX0
1F]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[
FVmkJeY[NV]JKg/00QS600<DYAS66<H00QS60QBU0QS60QBU0QS600<DYAS66<H00QBU00<HaQBU5:D0
1AS600@DYAS66<HHaP8DY@8HaP035:DHaQS6008HaP035:DHaQS600<DY@036<HDYABU008DY@8HaP04
5:DHaQBU5:D26<H55:D26<H25:D26<H01QBU6<HDYABU6<HDY@<HaP<DY@056<HDYAS65:DHaP055:D6
6<H25:D36<H55:D00aS65:DHaP036<H00aBU6<HHaP066<H25:D<6<H00aBU6<HHaP056<H00acW6<HH
aP027>L?6<H00aBU6<HHaP046<H00aBU6<HHaP096<H35:D46<H25:D66<H00aBU6<HHaP0M6<H00aBU
6<HHaP026<H25:D86<H011BU6<HHaQBU0QS60QBU11S600DDYAS66<HHaQBU00THaP<DY@<HaP8DY@HH
aP055:DHaQS65:DHaP035:D011S65:DDYABU0QS600<DYAS65:D00aBU0QS600@DYAS65:DHaP8DY@8H
aP065:DHaQBU5:DHaQBU0QS600<DYAS66<H021S61QBU00<HaQBU6<H00aBU0QS600<DYAS65:D00aS6
00/DYAS65:E_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00<HaQBU6<H00QS600@D
YAS66<HDY@@HaP8DY@DHaP035:DHaQBU00PHaP8DYA0HaP055:DHaQS66<HDY@026<H00aBU6<HHaP03
6<H00aBU6<HHaP036<H011BU6<HDYAS611BU0QS600<DYAS66<H01AS60aBU1AS60QBU11S600<DYAS6
6<H00aS600@DYAS65:DDY@@HaP035:DHaQS600XHaP045:DHaQS65:DG6<H00acW6<HHaP0@6<H00aBU
6<HHaP056<H01ABU6<HHaQBU6<H00aBU00<HaQBU5:D05QS60QBU00<HaQBU5:D01AS600<DYAS65:D0
1AS600DDYAS66<HHaQBU008HaP065:DHaQS66<HDYAS60QBU0QS60aBU1QS600DDYAS66<HHaQBU00@H
aP035:DHaQS600THaP8DY@036<HDYAS6008HaPDDY@<HaP8DY@<HaP8DY@<HaP035:DHaQS6008DY@03
6<HDYAS600<DY@8HaP065:DHaQS65:DHaQBU0QS600DDYAS66<HHaQcW00HHaP055:DHaQS66<HDY@02
6<H00aBU6<HHaP056<H00aBU6<HHaP046<H00aBUJeY[FP05JeX10001JeX0000MJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP055:D011S65:DDYAS6
1ABU0QS600@DYAS66<HHaP<DY@036<HDYAS600<HaP045:DHaQS65:D36<H01ABU6<HHaQS65:D00aS6
00@DYAS66<HHaP8DY@@HaP035:DHaQBU00<HaP8DY@036<HDYAS600PHaP<DY@@HaP@DY@<HaP035:DH
aQBU008DY@036<HDYAS600DDY@046<HDYAS65:D46<H25:D>6<H25:D00aS65:DHaP026<H00aBU6<HH
aP086<H00acW6<HHaP0C6<H00acW5:DHaP025:D011S65:DHaQS60QBU0QS600@DYAS65:DDY@DHaP04
5:DHaQS66<H25:D00aS65:DDY@0?6<H00aBU6<HHaP086<H25:D00aS65:DHaP056<H45:D26<H25:D:
6<H011BU6<HDYAS60QBU0QS600<DYAS66<H00QBU00HHaQBU6<HHaQBU6<H25:D56<H01ABU6<HHaQS6
5:D01aS600DDYAS66<HHaQBU008HaP035:DHaQBU00@HaP035:DHaQS600DDY@<HaP035:DHaQS6008D
Y@8HaP8DY@DHaP035:DHaQS600@HaP8DY@036<HDYAS6008HaP8DY@8HaP035:DHaQS600<HaP045:DH
aQS66<H35:D26<H026]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP8HaP035:DHaQS6
00DHaP035:DHaQS6008HaP035:DHaQS600dHaP8DY@HHaP035:DHaQS6008DY@<HaP035:DHaQS6008H
aP045:DHaQS65:D76<H00aBU6<HDY@025:D26<H00aBU6<HDY@025:D86<H25:D011S65:DHaQS60QBU
0QS600<DYAS65:D00aBU1AS60aBU0QS600<DYAS66<H01QS600@DYAS66<HHaP<DY@8HaP035:DHaQS6
00DHaP055:DHaQS66<HDY@066<H27>L56<H00aBU6<HDY@066<H00aBU6<HHaP066<H25:D00aS65:DH
aP036<H00aBU6<HHaP026<H00aBU6<HHaP035:D36<H35:D46<H25:D011S65:DHaQS60QBU1QS60QBU
0aS600@DYAS66<HHaP8DY@@HaP8DY@@HaP055:DHaQS66<HDY@046<H00aBU6<HHaP026<H011BU6<HH
aQS60QBU0QS60QBU1AS60aBU1AS600@DYAS65:DHaP8DY@8HaP035:DHaQBU00<HaP035:DHaQS600LD
Y@DHaP<DY@036<HDYAS600<HaP055:DHaQS65:DHaP025:D011S65:DDYAS60QBU0QS600<DYAS66<H0
2AS600<DYAS66<H02AS60QBU0QS61ABU00<HaV]JJeX01F]J0@000F]J00007FmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/02AS600@DYAS66<HHaP8D
Y@8HaP035:DHaQS6008HaP035:DHaQS600DHaP045:DHaQS65:D26<H00aBU6<HDY@026<H011BU6<HH
aQBU0QS60QBU0aS60QBU11S60QBU11S600<DYAS66<H00aS60QBU1AS600<DYAS66<H00aS600@DYAS6
6<HDY@8HaP8Li`8DY@LHaP8DY@HHaP035:DHaQS600<HaP045:DHaQS66<H25:D46<H00aBU7>LHaP08
6<H00aBU6<HHaP096<H00acW6<HDY@046<H00aBU6<HHaP056<H00aBU6<HHaP066<H011BU6<HHaQBU
0QS600@DYAS66<HHaP<DY@DHaP035:DHaQS600@HaP035:DHaQS6008HaP8DY@036<HDYAS600HHaP03
5:DHaQS6008HaP045:DHaQS66<H35:D011S65:DHaQBU1QS600<DYAS65:D01AS600<DYAS65:D00QBU
0QS600<DYAS65:D011S600@DYAS66<HDY@8HaP045:DHaQBU5:D26<H55:D00aS65:DHaP066<H01QBU
6<HDYABU6<HDY@8HaP045:DHaQS66<H25:D00aS65:DHaP036<H25:D36<H25:D00aS65:DHaP025:D0
11S65:DDYABU1aS60aBU0QS600DDYAS66<HHaQBU00HHaP035:DHaQS600DHaP045:DHaQBU5:D36<H0
2aBU6<HHaVmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX00aBU6<HDY@056<H00aBU
6<HDY@035:D<6<H25:D01AS65:DHaQS65:D00aS600<DYAS65:D00QS61ABU00@HaQBU6<HDY@8HaP03
5:DHaQS6008HaP035:DHaQS6008DY@LHaP<DY@036<HDYABU00<DY@036<HDYAS600@HaP045:DHaQBU
5:D36<H00aBU6<HHaP026<H25:D56<H25:D00aS65:DDY@035:D56<H25:D96<H25:D66<H25:D00aS6
5:DHaP0<6<H00aBU6<HHaP056<H00aBU6<HHaP036<H65:D26<H35:D66<H00aBU6<HHaP036<H35:D0
0aS65:DDY@046<H35:D26<H00aBU6<HHaP056<H00aBU6<HHaP0<6<H00aBU6<HHaP025:D26<H00aBU
6<HHaP096<H011BU6<HDYABU0QS60QBU0aS600DDYAS66<HHaQBU00DHaP8DY@046<HDYABU6<H25:D;
6<H00aBU6<HHaP025:D26<H45:D36<H35:D00aS65:DDY@025:D00aS65:DDY@036<H35:D36<H00aBU
6<HHaP026<H25:D66<H25:D26<H35:D96<H75:D26<H7JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP056<H00aBU6<HHaP026<H0
0aBU6<HHaP036<H00aBU6<HHaP036<H00aBU6<HHaP035:D36<H00aBU6<H@Q0036<H00acW5:DHaP02
6<H45:D66<H25:D26<H00aBU6<HHaP036<H25:D86<H00aBU6<HHaP055:D46<H00aBU6<HDY@025:D2
6<H011BU6<HHaQS61ABU0aS60aBU00<HaQBU6<H01AS600@DYAS66<HDY@LHaP035:DHaQS600PHaP8L
i`@HaP035:DHaQS601dHaP<DY@<HaP055:DHaQS66<HDY@026<H00aBU6<HDY@026<H00aBU6<HHaP03
6<H25:D00aS65:DHaP026<H011BU6<HHaQcW11S611BU41S600@DYAS65:DHaP8DY@DHaP065:DHaQS6
5:DHaQBU0QS61ABU00<HaQBU6<H021S60QBU0QS60QBU00<HaQBU6<H00QS60QBU00<HaQBU6<H00aS6
0QBU1QS600@DYAS65:DHaPDDY@036<HDYAS600DHaP8DY@036<HDYABU00HHaP8DY@046<HDYAS65:D4
6<H25:D00aS65:DDY@026<H25:D96<H25:D36<H02aBU6<HHaV]jJeY_Nf]JJgY[FVmk00000F]j000F
JeX30004JeX56<H00aBU6<HHaP026<H00aBU6<HHaP036<H00aBU6<HHaP036<H00aBU6<HHaP035:D3
6<H00aBU6<H@Q0036<H00acW5:DHaP026<H45:D66<H25:D26<H00aBU6<HHaP036<H25:D86<H00aBU
6<HHaP055:D46<H00aBU6<HDY@025:D26<H011BU6<HHaQS61ABU0aS60aBU00<HaQBU6<H01AS600@D
YAS66<HDY@LHaP035:DHaQS600PHaP8Li`@HaP035:DHaQS601dHaP<DY@<HaP055:DHaQS66<HDY@02
6<H00aBU6<HDY@026<H00aBU6<HHaP036<H25:D00aS65:DHaP026<H011BU6<HHaQcW11S611BU41S6
00@DYAS65:DHaP8DY@DHaP065:DHaQS65:DHaQBU0QS61ABU00<HaQBU6<H021S60QBU0QS60QBU00<H
aQBU6<H00QS60QBU00<HaQBU6<H00aS60QBU1QS600@DYAS65:DHaPDDY@036<HDYAS600DHaP8DY@03
6<HDYABU00HHaP8DY@046<HDYAS65:D46<H25:D00aS65:DDY@026<H25:D96<H25:D36<H00aBU6<HH
aP06JeX20001JeX0000OKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX0
06]JKg][FV]jJeY_NaS65:D02AS60QBU41S600@DYAS66<HDY@DHaP035:DHaQBU00@HaP045:DHaQS6
5:D66<H00aBU6<HDY@076<H25:D011S65:DDYABU2QS60QBU1QS60QBU0QS60aBU1aS611BU1aS600<D
YAS66<H00aS60aBU0QS60QBU0aS60QBU0QS600<DYAS66<H031S600<DYAS66<H00QS600<DYAS66<H0
2AS600DDYAS66<HHaQBU008HaP055:DHaQS66<HDY@0A6<H00acW5:DHaP026<H00aBU6<HHaP0:6<H0
0aBU6<HHaP0L6<H00aBU6<HHaP035:D00aS65:DHaP0:6<H011BU6<HHaQBU1AS60QBU11S600@DYAS6
6<HDY@8HaP8DY@@HaP045:DHaQS66<H25:D00aS65:DDY@075:D00aS65:DDY@025:D46<H35:D26<H2
5:D00aS65:DDY@036<H00aBU6<HDY@036<H011BU6<HDYABU0QS600DDYAS66<HHaQBU00PHaP<DY@8H
aP095:E_Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J1AS600<DYAS65:D00QBU00<H
aQBU5:D00aS600<DYAS66<H01aS60QBU0aS600<DYAS66<H021S60QBU11S600DDYAS66<HHaQBU00@H
aP<DY@<HaP8DY@8HaP045:DHaQS66<H25:D36<H25:D56<H00aBU6<HDY@076<H25:D76<H01ABU6<HD
YABU6<H00QBU00<HaQBU6<H01QS600<DYAS66<H02AS600<DYAS66<H01AS60QBU1aS600<DYAS66<H0
3AS600<DYAS66<H01aS600<DYAS66<H00QS60aBU3AS600@DYAS65:DHaP@DY@XHaP045:DHaQS66<H2
5:D86<H25:D46<H00aBU6<HHaP025:D26<H00aBU6<HDY@026<H00c6<5:DDY@025:D26<H00aBU6<HD
Y@025:D36<H35:D>6<H35:D00aS65:DHaP056<H35:D46<H011BU6<HHaQBU1QS600<DYAS66<H00QBU
0QS600@DYAS65:DHaP8DY@8HaP045:DHaQS66<H45:D00aS65:DDY@086<H55:D96<H25:D56<H7JeX1
0001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[
FVmkJeY[NP025:D01QS65:DDYABU6<HDY@8HaP<DY@XHaP<DY@LHaP037>LHaQS600DHaP@DY@<HaP03
5:DHaQS6008HaP045:DHaQS65:D36<H011BU6<HHaQBU11S600<DYAS66<H011BU00DHaQBU5:DHaQBU
00HHaP035:DHaQBU01HHaP055:DHaQS66<HDY@076<H00aBU6<HHaP066<H25:D26<H25:D36<H00aBU
6<HHaP056<H011BU6<HHaQS60QBU3aS60aBU00<HaQBU6<H011S600@DYAS66<HHaP8DY@8HaP8DY@@H
aP035:DHaQS600<HaP8DY@8HaP055:DHaQS66<HDY@046<H00aBU6<HHaP036<H00aBU6<HHaP0C6<H0
11BU6<HDYB480aS600<DYAS65:D01aS600@DYAS66<HHaP<DY@036<HDYAS6008DY@@HaP035:DHaQS6
008HaP035:DHaQS600DHaP035:DHaQS6008HaP<DY@036<HDYABU00LDY@8HaP@DY@036<HDYABU00<D
Y@8HaP8DY@036<HDYAS600THaP045:DHaQBU5:D46<H00aBU6<HHaP066<H011BU6<HHaQS60QBU0QS6
00TDYF]jJeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX36<H55:D26<H00aBU6<HHaP07
6<H011BU6<HHaQBU1AS60QBU00<HaQBU6<H00QS600@DYAS66<HDY@@HaP035:DHaQS6008DY@@HaP05
5:DHaQS66<HDY@036<H25:D26<H35:D011S65:DDYABU11S600<DYAS66<H00QBU11S600<DYAS65:D0
1QS600<DYAS66<H00aS600<DYAS65:D031S600@DYAS66<HHaP<DY@8HaP045:DHaQS65:D76<H00aBU
6<HHaP046<H00aBU6<HHaP036<H00aBU6<HDY@025:D01AS65:DDYAS65:D01AS600<DYAS65:D011S6
0aBU00<HaQBU6<H02QS600<DYAS66<H00aS600<DYAS66<H00aS60QBU0QS600<DYAS66<H01AS600@D
YAS66<HHaP8DY@<HaP035:DHaQS600@HaP8DY@DHaP8DY@8HaP8DY@036<HDYAS600<DY@8HaP035:DH
aQS6008HaP035:DHaQS600@HaP035:DHaQS600DHaP<DY@036<HDYABU008DY@DHaP8DY@036<HDYAS6
00LHaP<DY@036<HDYABU00`DY@8HaP035:DHaQBU00@HaP045:DHaQS65:D26<H00aBU6<HHaP056<H0
1ABU6<HHaQBU6<H00QBU00<HaQBU6<H00QBU1QS60QBU0aS600<DYAS65:D01f]J0@000F]J00007Fmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QS6
11BU0aS600DDYAS66<HHaQBU00LHaP035:DHaQS600HHaP8DY@036<HDYAS600DHaP055:DHaQS66<HD
Y@036<H00aBU6<HHaP036<H00aBU6<HDY@036<H00aBU6<HHaP056<H00aBU6<HDY@025:D36<H00aBU
6<HDY@045:D00aS65:DDY@036<H25:D36<H00aBU6<HHaP045:D26<H011BU6<HHaQS60QBU1aS600@D
YAS66<HHaP8DY@HHaP8DY@<HaP035:DHaQS6008DY@/HaP035:DHaQS600THaP045:DHaQBU5:D66<H0
0aBU6<HHaP035:D00aS65:DDY@096<H01ABU6<HHaQS65:D021S60QBU0QS60QBU0QS600@DYAS65:DH
aP8DY@DHaP8DYA4HaP035:DHaQS6008DY@8HaP035:DHaQS6008HaP045:DHaQS66<H25:D26<H25:D2
6<H25:D01AS65:DHaQS65:D00QS60QBU00<HaQBU5:D01AS600@DYAS66<HDY@8HaP8DY@DHaP035:DH
aQBU008DY@8HaP8DY@036<HDYABU00DDY@8HaP8DY@046<HDYAS65:D76<H00aBU6<HHaP045:D00aS6
5:DHaP025:D00aS65:DDY@026<H35:D26<H011BU6<HHaQBU0QS600/DYAS65:E_Nf]JJgY[FVmkJeY[
NP00005_N`005V]J00<006]JJeX016]J0aS60QBU00<HaQBU6<H03QS600@DYAS66<HHaP8DY@8HaP04
5:DHaQS65:D26<H25:D>6<H45:D00aS65:DDY@025:D66<H011BU6<HDYABU3QS600<DYAS65:D011S6
00<DYAS66<H01AS600@DYAS66<HDYA8HaP035:DHaQBU008DY@<HaP045:DHaQBU7>L86<H00aBU6<HH
aP0A6<H011BU6<HDYABU1AS600@DYAS65:DHaP8DY@8HaP035:DHaQBU008DY@8HaP035:DHaQS600<D
Y@8HaP035:DHaQS600DHaP8DY@lHaP035:DHaQS600<HaP035:DHaQBU00THaP<DY@056<HDYAS66<HD
Y@026<H00aBU6<HHaP026<H00aBU6<HHaP025:D00aS65:DDY@025:D26<H45:D26<H00aBU6<HHaP02
6<H01ABU6<HHaQS65:D011S60QBU0aS60QBU1AS600<DYAS66<H00QBU0QS611BU1AS61ABU00@HaQBU
6<HDY@`HaP<DY@046<HDYABU5:D66<H35:D36<H00aBU6<HHaP026<H00aBU6<I[FP06JeX10001JeX0
000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[
NP056<H00aBU6<HDY@096<H00aBU6<HHaP026<H00aBU6<HDY@036<H011BU6<HHaQBU1AS611BU11S6
00DDYAS66<HHaQBU00<HaP8DY@036<HDYAS600<HaP8DY@hHaP035:DHaQBU00@HaP035:DHaQS600`H
aP035:DHaQS600@HaP035:DHaQS6008HaP@DY@<HaP8DY@<HaP055:DHaQS66<HDY@026<H25:D011S6
5:DDYABU4aS600<DYAS65:D01AS600@DYAS65:DHaP8DY@8HaP<DY@046<HDYABU6<H25:D86<H011BU
6<HDYABU4AS600DDYAS65:DDYAS6008DY@036<HDYAS600LHaP035:DHaQBU008DY@8HaP045:DHaQBU
5:D26<H00aBU6<HHaP056<H25:D011S65:DHaQBU0aS60aBU21S60aBU00<HaQBU5:D02AS611BU0QS6
11BU1QS60aBU00DHaQBU6<HHaQBU00HHaP8DY@036<HDYABU00THaP8DY@036<HDYABU00<DY@HHaP03
5:DHaQS6008HaP<DY@PHaP0;5:DHaQS6JgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[
FPDHaP035:DHaQBU00THaP035:DHaQS6008HaP035:DHaQBU00<HaP045:DHaQS65:D56<H45:D46<H0
1ABU6<HHaQS65:D00aS60QBU00<HaQBU6<H00aS60QBU3QS600<DYAS65:D011S600<DYAS66<H031S6
00<DYAS66<H011S600<DYAS66<H00QS611BU0aS60QBU0aS600DDYAS66<HHaQBU008HaP8DY@046<HD
YABU5:DC6<H00aBU6<HDY@056<H011BU6<HDYAS60QBU0QS60aBU00@HaQBU5:DHaP8DY@PHaP045:DH
aQBU5:DA6<H01ABU6<HDYABU6<H00QBU00<HaQBU6<H01aS600<DYAS65:D00QBU0QS600@DYAS65:DD
Y@8HaP035:DHaQS600DHaP8DY@046<HDYAS65:D36<H35:D86<H35:D00aS65:DDY@096<H45:D26<H4
5:D66<H35:D01AS65:DHaQS65:D01QS60QBU00<HaQBU5:D02AS60QBU00<HaQBU5:D00aBU1QS600<D
YAS66<H00QS60aBU21S600<DYAS66<H01f]J0@000F]J00007VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DY@8HaP8DY@046<HDYAS65:D46<H01ABU
6<HHaQS65:D011S600@DYAS66<HDY@@HaP045:DHaQBU5:D<6<H45:D26<H011BU6<HHaQBU0QS611BU
0QS600<DYAS66<H00aS600<DYAS66<H00aBU00@HaQBU6<HDY@/HaP045:DHaQS65:D26<H25:D46<H0
11BU6<HHaQBU3aS600@DYAS66<HDY@PHaP0648@HaQBU5:DHaQBU0aS60QcW00<HaQBU5:D01AS600@D
YAS66<HDY@PHaP@DY@DHaP@DY@046<HDYAS66<H25:D36<H00aBU6<HHaP035:D00aS65:DHaP056<H0
0aBU6<HDY@086<H011BU6<HDYABU4aS600<DYAS66<H011S600<DYAS66<H011BU00<HaQBU6<H011BU
0QS600<DYAS66<H00QS600@DYAS66<HDY@@HaP8DY@036<HDYAS600<HaP035:DHaQBU008HaP8DY@04
6<HDYAS66<H25:D36<H00aBU6<HDY@056<H25:D26<H25:D01QS65:DDYABU6<HDY@8HaP065:DHaQBU
5:DHaQBU11S60QBU0QS60QBU0aS600<DYAS66<H011S600<DYAS66<H00QS600<DYAS65:D00QS600<D
YAS66<H00QBU00THaVmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX00aBU6<HHaP02
6<H65:D<6<H00aBU6<HHaP046<H25:D01AS65:DHaQS65:D02AS60QBU0QS60QBU00HHaQBU5:DHaQBU
6<H45:D76<H00aBU6<HDY@036<H011BU6<HDYAS60QBU0QS600<DYAS66<H00QBU0aS600@DYAS66<HH
aP8DY@8HaP8DY@036<HDYAS600<DY@036<HDYAS6010HaP055:DHaQS66<HDY@0<6<H27>L26<H00acW
6<HHaP036<H00aBU6<HDY@096<H00aBU6<HHaP036<H011BU6<HDYABU2aS611BU1QS600<DYAS66<H0
0aS600@DYAS66<HDY@`HaP035:DHaQS6008DY@8HaP035:DHaQBU00PHaP035:DHaQS6008HaP035:DH
aQS600PHaP045:DHaQS66<H35:D66<H00aBU6<HDY@026<H25:D00aS65:DHaP046<H00aBU6<HDY@06
6<H011BU6<HDYAS60QBU00@HaQBU6<HDY@<HaP035:DHaQS600<DY@<HaP035:DHaQS600PDY@8HaP<D
Y@056<HDYAS66<HDY@046<H00aBU6<HHaP035:D01AS65:DHaQS65:D00QS611BU11S600<DYAS66<H0
1f]J0@000F]J00007f]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[
FV]jJeY_Nf]JJgXHaQBU00<HaP055:DHaQBU6<HDY@036<H01ABU6<HHaQS65:D011S611BU11S600<D
YAS66<H031S60QBU0QS600@DYAS66<HDY@8HaP035:DHaQS6008DY@8HaP035:DHaQS600DHaP035:DH
aQS600@HaPHDY@PHaP035:DHaQS600dHaP055:DHaQS66<HDY@0<6<H25:D<6<H01ABU6<HHaQBU6<H0
0acW1aS600<DYAS66<H00QBU1QS600<DYAS66<H03AS600@LiaS66<HHaP<DYA<HaP<DY@LHaP055:DH
aQS65:DHaP045:D26<H021BU6<HDYABU6<HDYAS65:D56<H011BU6<HHaQS60aBU00<HaQBU5:D00QBU
00@HaQBU5:DDY@<HaPHDY@036<HDYABU008DY@8HaP8DY@036<HDYAS600LHaP<DY@8HaP8DY@<HaP05
5:DHaQS65:DHaP025:D26<H00aBU6<HDY@086<H35:D01QS65:DHaQBU6<HDY@<HaP035:DHaQS600`H
aP8DY@<HaP8DY@@HaP055:DHaQS66<HDY@026<H026]jJeY_Nf]JJgY[FVmk0001JgX001I[FP<000A[
FP8HaP035:DHaQS600@HaP055:DHaQS66<HDY@036<H35:D26<H25:D00aS65:DHaP0=6<H011BU6<HH
aQS60QBU0QS600DDYAS66<HHaQBU00LHaP045:DHaQS65:D36<H01ABU6<HDYAS65:D00aS61QBU0aS6
00DDYAS66<HDYAS6008DY@036<HDYAS6008HaP045:DHaQS65:D46<H25:D46<H01ABU6<HHaQS65:D0
0QS600<DYAS66<H00aS60QBU1AS60QBU11S60QBU0QS600<DYAS66<H01QS600<DYAS66<H04AS60QBU
0aS61QBU0QS60QBU51S600@DYAS65:DDY@8HaP035:DHaQS600HHaP8DY@8HaP8DY@LHaP045:DHaQS6
5:D46<H45:D66<H85:D01QS65:DHaQS65:DHaP<DY@066<HDYABU5:DHaQBU1AS60QBU00<HaQBU5:D0
0aS60QBU00@HaQBU6<HHaP<DY@046<HDYABU5:D26<H25:D00aS65:DDY@046<H011BU6<HHaQBU0aS6
00<DYAcW6<H00QS60aBU11S600<DYAS66<H011S600DDYAS66<HHaQBU00<HaP035:DHaQBU00<DY@@H
aP<DY@8HaPI[FP80005[FP00021_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU0QS600<DYAS65:D01AS600<DYAS65:D01aS600<DYAS65:D0
0QBU00@HaQBU5:DDYA4HaP055:DHaQS66<HDY@066<H00aBU6<HDY@045:D00aS65:DDY@026<H45:D3
6<H011BU6<HHaQS60QBU00<HaQBU6<H02AS60QBU2QS600<DYAS65:D05aS600@DYAS66<HDY@HHaP05
5:DHaQS66<HDY@046<H00aBU6<HDY@026<H55:D00aS65:DDY@035:D26<H00aBU6<HHaP056<H011BU
6<HDYABU0aS600<DYAS65:D00aBU00DHaQBU5:DHaQBU008HaP8DY@<HaP035:DHaQS6008HaP8DY@03
6<HDYAS600hHaP8DY@DHaP055:DHaQS65:DHaP025:D46<H011BU6<HDYAS60QBU0aS600<DYAS66<H0
11BU0QS61ABU00@HaQBU5:DDY@8HaP8DY@<HaP8DY@DHaP035:DHaQS600DDY@036<HDYAS6008DY@03
6<HDYAS600<HaP8DY@046<HDYABU5:D26<H35:D76<H00aBU6<HHaP056<H011BU6<HDYAS60aBU00<H
aQBU5:D00QBU00DHaQBU5:DHaQBU008HaP8DY@8HaP0;5:DHaQBUKg][FV]jJeY_Nf]JJgX00001Kg/0
01I[FP03001[FV]J00A[FP035:DHaQS600DHaP8DY@@HaP035:DHaQS600<HaP035:DHaQS600DHaP05
5:DHaQS66<HDY@096<H35:D46<H45:D76<H00aBU6<HHaP066<H45:D36<H55:D26<H00aBU6<HDY@05
5:D56<H01ABU6<HHaQS65:D00QS600<DYAS66<H00QS600<DYAS65:D02aS600@DYAS65:DDY@DHaP03
5:DHaQS600DHaP035:DHaQBU00/HaP047>LHaQS65:D56<H011BU6<HHaQS611BU11S611BU1QS600<D
YAS66<H00QS600<DYAS66<H00QBU0QS600<DYAS65:D01ABU00<HaQBU6<H00aS60QBU11S60QBU00<H
aQBU6<H00QBU2QS600<DYAS65:D00aBU0QS600@DYAS65:DHaP<DY@056<HDYABU6<HDY@046<H00aBU
6<HDY@025:D36<H25:D01AS65:DDYAS65:D02aS611BU00<HaQBU6<H00QS60QBU0QS60QBU11S60QBU
2AS611BU0QS600<DYAS65:D00aS61ABU00<HaQBU5:D021S600<DYAS65:D011BU0aS600<DYAS66<H0
1ABU1f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
001[FV]jJeY_Nf]JJgX00QBU0aS60QBU1aS600<DYAS66<H01QS60QBU00<HaQBU5:D00aS60QBU11S6
0QBU0QS600@DYAS66<HDY@8HaP037>LHaQS6008HaP055:DHaQBU6<HDY@036<H00aBU6<HDY@026<H2
5:D36<H25:D00aS65:DDY@036<H55:D00aS65:DHaP056<H00aBU6<HHaP025:D=6<H25:D46<H00aBU
6<HHaP0=6<H011BU6<HDYAS61QBU1aS600<DYAS66<H00QBU2AS600DDYAS65:DDYAS600<DY@@HaP04
5:DHaQS66<H25:D46<H25:D011S65:DHaQS60QBU00<HaQBU6<H01AS60QBU00<HaQBU5:D021S600PD
YAS66<HHaQBU6<HDYAS60QBU2AS600<DYAS66<H00QS61QBU00DHaQBU6<HHaQBU00<HaP035:DHaQS6
00@HaP035:DHaQS600DHaP8DY@@HaP8DY@056<HDYAS66<HDY@036<H01QBU6<HDYABU6<HDY@8HaP04
5:DHaQBU5:D36<H55:D36<H25:D36<H011BU6<HDYAS60aBU0aS600<DYAS66<H00QS600<DYAS66<H0
1ABU0QS61ABU0aS600<DYAS66<H00aBU1AS60QBU0aS600Q[NV]JKg][FV]jJeY_N`000F]j000FJeX0
0`00JeY[FP04JeX25:D36<H25:D76<H00aBU6<HHaP066<H25:D00aS65:DDY@036<H25:D46<H25:D2
6<H011BU6<HHaQBU0QS600<LiaS66<H00QS600DDYAS65:DHaQBU00<HaP035:DHaQBU008HaP8DY@<H
aP8DY@036<HDYABU00<HaPDDY@036<HDYAS600DHaP035:DHaQS6008DY@dHaP8DY@@HaP035:DHaQS6
00dHaP045:DHaQBU6<H65:D76<H00aBU6<HHaP025:D96<H01ABU6<HDYABU6<H00aBU11S600@DYAS6
6<HHaP8DY@@HaP8DY@046<HDYAS66<H25:D00aS65:DHaP056<H25:D00aS65:DDY@086<H021BU6<HH
aQS65:DHaQBU6<H25:D96<H00aBU6<HHaP026<H65:D01AS65:DHaQS65:D00aS600<DYAS66<H011S6
00<DYAS66<H01AS60QBU11S60QBU00DHaQBU6<HHaQBU00<HaP065:DHaQBU5:DHaQBU0QS600@DYAS6
5:DDY@<HaPDDY@<HaP8DY@<HaP045:DHaQBU6<H35:D36<H00aBU6<HHaP026<H00aBU6<HHaP055:D2
6<H55:D36<H00aBU6<HHaP035:D56<H25:D36<H7JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`026<H45:D36<H01QBU6<HDYABU
6<HDY@DHaP8DY@036<HDYAS600HHaP035:DHaQS600<HaP8DY@8HaP035:DHaQS600THaP<DY@066<HD
YABU5:DHaQBU0QS60aBU00@HaQBU6<HHaP8DY@<HaP035:DHaQS6008DY@8HaP035:DHaQBU00<HaP@D
Y@@HaP8DY@<HaP035:DHaQS600DHaP045:DHaQS66<H25:D66<H011BU6<HHaQBU3AS600@DYAS65:DH
aP<DY@/HaP045:DHaQS66<H25:D66<H00aBU6<HHaP036<H25:D00acW6<HDY@046<H011BU6<HDYAS6
0QBU4QS600<DYAS66<H00QS600@DYAS66<HDY@8HaPHDY@8HaP045:DHaQBU5:D86<H00aBU6<HDY@06
6<H01ABU6<HHaQS65:D00QS600<DYAS65:D01QS60aBU00<HaQBU5:D00QS600@DYAS66<HDY@DHaP03
5:DHaQBU008DY@<HaP<DY@<HaP045:DHaQS65:D46<H35:D96<H25:D011S65:DHaQS60QBU0QS60aBU
00<HaQBU5:D00aBU21S60QBU21S600<DYAS65:D00aBU00THaVmkJeY[NV]JKg][FV]j00000Fmk000F
JeX00`00JeY[FP04JeX25:D011S65:DDYAS60QBU1AS600<DYAS65:D01AS60aBU00<HaQBU6<H03aS6
0QBU3QS60QBU1AS60aBU0QS600<DYAS66<H00aS60QBU0QS611BU00<HaQBU5:D00QBU0QS600<DYAS6
6<H00QS611BU1aS60QBU1AS60aBU1QS60QBU0QS600TDYAS66<HHaQBU6<HDYAS65:D00QS600DDYAS6
6<HHaQBU00<HaP037>LHaQS6008DY@DHaP035:DHaQBU008HaP<DY@DHaP<DY@036<HDYAS600DHaP8D
Y@8HaP035:DHaQS600DHaP035:DHaQBU008HaP065:DHaQS65:DHaQBU1AS600@DYAS65:DHaP8DY@@H
aP035:DHaQS6008DY@8HaP8DY@036<HDYAS600@HaP035:DHaQBU00HHaP055:DHaQBU5:DHaP025:D2
6<H00aBU6<HHaP026<H:5:D00aS65:DDY@026<H45:D26<H00aBU6<HHaP025:D00aS65:DDY@025:D0
11S65:DDYAS60aBU0QS61ABU00<HaQBU6<H00QBU11S600<DYAS66<H00aS600@DYAS65:DDY@@HaP04
5:DHaQBU5:D26<H25:D011S65:DHaQS60aBU0aS600<DYAS65:D00aS60aBU00@HaQBU5:DDY@8HaP03
5:E[FV]J00E[FP40005[FP0001m[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FP00JeY[NV]JKg][FV]j6<HDY@066<H25:D36<H011BU6<HHaQBU1AS600<DYAS65:D00aS600@D
YAS66<HDY@8HaP8DY@<HaP035:DHaQBU00<HaP035:DHaQS600PHaP035:DHaQS600<HaP8DY@DHaP04
5:DHaQBU6<H35:D00aS65:DHaP066<H45:D46<H011BU6<HHaQBU0QS600<DYAS66<H00QBU0QS60QBU
1aS600<DYAS66<H00QS60QBU1QS61ABU00<HaQBU6<H00QBU1aS600<DYAS66<H01QS600DDYAS66<HH
aQBU00@HaP035:DHaQS600@HaP<DY@8HaP8DY@@HaP035:DHaQS600@DY@036<HDYAS600@HaP035:DH
aQBU00HHaP045:DHaQBU5:D36<H00aBU6<HHaP025:D26<H00aBU6<HDY@026<H00aBU6<HHaP066<H2
5:D36<H25:D00aS65:DHaP036<H55:D36<H25:D01AS65:DDYAS65:D00QS600<DYAS65:D00QBU0aS6
11BU00HHaQBU6<HHaQBU6<H35:D36<H25:D36<H01QBU6<HHaQBU6<HDY@8HaP<DY@<HaP8DY@8HaP05
5:DHaQS66<HDY@046<H00aBU6<HDY@046<H011BU6<HHaQBU0QS60QBU00<HaQBU5:D00QBU0QS600DD
YAS65:DHaQBU00<HaP035:DHaQBU008HaP0>5:DHaQS65:DHaQBUJgY[FVmkJeY[NV]JKg/0005[NP00
5V]J00<006]JJeX016]J2AS611BU00<HaQBU6<H011S600<DYAS66<H00QS61ABU0aS600<LiaS66<H0
0QBU00<HaQBU6<H00aS61ABU2AS611BU0aS611BU0aS600<DYAS66<H00aS600<DYAS66<H00QBU00HH
aQBU6<HDYAS65:D26<H011BU6<HHaQcW0QS60QBU0QS600<DYAS66<H01QS600DDYAS65:DDYAS6008D
Y@LHaP035:DHaQS600HDY@8HaP<DY@/HaP035:DHaQS600`HaP035:DHaQBU00<HaP8DY@DHaP055:DH
aQS66<HDY@056<H00aBU6<HHaP025:D96<H35:D00aS65:DHaP046<H00aBU6<HDY@056<H00aBU6<HH
aP026<H01ABU6<HHaQS65:D00QS600<DYAS66<H00QS600<DYAS66<H01AS600DDYAS66<HHaQBU00<H
aP@DY@8HaP075:DHaQS66<HDYAS65:D00QS600HDYAS66<HHaQBU6<H65:D76<H00aBU6<HHaP026<H0
11BU48@@Q1BU0QS611BU1QS60QBU0QS60QBU11S60QBU00<HaQBU6<H02AS60QBU00<HaQBU5:D00QS6
0QBU0QS600<DYAS65:D02aS600<DYAS65:D01f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/01AS600<DYAS65:D02AS60aBU0QS6
0QBU00<HaQBU5:D00aBU00<HaQBU6<H03QS600<DYAS66<H00QS60aBU0QS60aBU00<HaQBU6<H00QBU
0aS600DDYAS66<HHaQBU00@HaP8DY@@HaP@DY@@HaP045:DHaQS65:D=6<H00aBU6<HDY@036<H00aBU
6<HHaP036<H25:D00aS65:DHaP035:D36<H01ABU6<HDYAS65:D02aS600@DYAS66<HDY@LHaPDDY@8H
aPDDY@@HaP045:DHaQS66<H25:D00aS65:DDY@035:D46<H25:D66<H35:D011S65:DDYABU0aS600@D
YAS66<HDY@8HaP8DY@DHaP8DY@@HaP@DY@<HaP035:DHaQS600@HaP047>LHaQS66<H35:D46<H45:D0
0aS65:DDY@045:D36<H011BU6<HDYABU0aS60aBU0QS600<DYAS65:D00QBU11S60QBU0aS60aBU0QS6
00HDYAS66<HHaQBU6<H35:D36<H011BU6<HHaQS60QBU0QS60QBU41S60QBU0aS600<DYAS66<H01aS6
00@DYAS66<HHaP8DY@<HaP08Kg][FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J00<HaQBU
6<H00QS600HDYAS66<HHaQBU6<H45:D26<H00aBU6<HHaP026<H011BU6<HDYAS60QBU0aS600<DYAS6
5:D01AS600<DYAS66<H00QBU00<HaQBU6<H00aS60aBU1aS60QBU00<HaQBU6<H011BU0QS600HDYAS6
5:DDYAS65:D26<H45:D00aS65:DDY@025:D36<H25:D011S65:DHaQBU0aS60QBU00<HaQBU6<H00aS6
00<DYAS66<H011BU0QS60QBU00DHaQBU5:DDYAS6008DY@036<HDYAS6008HaP<DY@066<HDYABU5:DH
aQBU0aS61QBU1AS600@DYAS65:DDY@<HaP065:DHaQBU5:DHaQBU0QS600<DYAS66<H00aBU0QS600<D
YAS66<H011S60aBU00<HaQBU6<H01aS600<DYAS66<H02aS600<DYAS66<H00QS60aBU1QS611BU4QS6
0QBU00@HaQBU5:DHaP@DY@<HaP8DY@@HaP045:DHaQBU5:D26<H011BU6<HHaQBU1AS600<DYAS66<H0
0aS61aBU00@HaQBU6<HDY@LHaP035:DHaQS600HHaP035:DHaQS6008HaPHDY@036<HDYABU00`HaP<D
Y@LHaP<DY@@HaP035:DHaV]J00I[FP40005[FP0001I[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FP<00007JeY_Nf]JJgXDYAS65:D00QS611BU11S600<DYAS65:D00QS60QBU
00@HaQBU6<HDY@8HaP@DY@046<HDYAS66<H25:D011S65:DHaQBU1AS600<DYAS66<H00QS60QBU0aS6
00<DYAS66<H011S600<DYAS66<H00QBU00<HaQBU6<H01QS60aBU0QS60aBU00<HaQBU5:D00QBU0QS6
00HDYAS66<HDYAS65:D26<H35:D96<H011BU6<HHaQBU0aS600<DYAS66<H00aS60aBU11S600DDYAS6
6<HHaQBU00/HaP045:DHaQS66<H25:D00aS65:DHaP026<H45:D011S65:DHaQS61QBU2AS600@DYAS6
6<HHaP8DY@8HaP035:DHaQS6014HaP055:DHaQS66<HDY@066<H00aBU6<HHaP036<H00aBU6<HHaP02
5:D01AS65:DHaQS65:D011S600DDYAS65:DDYAS6008DY@056<HDYABU5:DHaP035:D00aS65:DDY@02
6<H25:D56<H011BU6<HHaQBU0aS611BU00<HaQBU5:D00QBU1AS600@DYAS66<HDY@DHaPHDY@HHaP8D
Y@046<HDYABU6<H25:D011S65:DDYABU11S60aBU0QS600@DYAS66<HHaP@DY@8HaP8DY@8HaP035:DH
aQS600`HaP06JgY[FVmkJeY[NV]J0P000F]j000FJeX00`00JeY[FP04JeX00aBU6<HDY@026<H45:D4
6<H00aBU6<HDY@026<H25:D011S65:DHaQBU0QS611BU00@HaQBU6<HHaP8DY@046<HDYAS65:D56<H0
0aBU6<HHaP026<H25:D36<H00aBU6<HHaP046<H00aBU6<HHaP025:D00aS65:DHaP066<H35:D26<H3
5:D00aS65:DDY@025:D26<H01QBU6<HHaQBU6<HDY@8HaP<DY@THaP045:DHaQS65:D36<H00aBU6<HH
aP036<H35:D46<H01ABU6<HHaQS65:D02aS600@DYAS66<HHaP8DY@036<HDYAS6008HaP@DY@046<HD
YAS66<H65:D96<H011BU6<HHaQS60QBU0QS600<DYAS66<H04AS600DDYAS66<HHaQBU00HHaP035:DH
aQS600<HaP035:DHaQS6008DY@056<HDYAS66<HDY@046<H01ABU6<HDYABU6<H00QBU00DHaQBU5:DD
YAS600<DY@036<HDYABU008HaP8DY@DHaP045:DHaQS65:D36<H45:D00aS65:DDY@025:D56<H011BU
6<HHaQBU1AS61QBU1QS60QBU00@HaQBU5:DHaP8DY@046<HDYABU5:D46<H35:D26<H011BU6<HHaQS6
11BU0QS60QBU0QS600<DYAS66<H031S61f]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/01QS600@DYAS66<HDY@<HaP035:DHaQS6
008DY@8HaP<DY@8HaP045:DHaQS65:D36<H00aBU6<HHaP026<H25:D00aS65:DHaP026<H00aBU6<HH
aP035:D46<H00aBU6<HHaP036<H25:D00aS65:DHaP086<H25:D00aS65:DDY@036<H25:D56<H011BU
6<HHaQBU2AS60QBU00<HaQBU5:D021S60QBU0QS60QBU11S600@DYAS65:DHaP<DY@<HaPHDY@036<HD
YAS6008DY@046<HDYAS65:D46<H25:D36<H35:D56<H35:D00aS65:DHaP035:D26<H00aBU6<HHaP02
6<H25:D26<H00aBU6<HDY@036<H25:D66<H00aBU6<HHaP025:D011S65:DDYABU0aS60QBU1aS60aBU
00<HaQBU6<H031S600<DYAS66<H00aS60aBU00@HaQBU6<HDY@@HaP<DY@046<HDYABU5:D46<H45:D0
1AS65:DHaQBU6<H011BU0QS600<DYAS66<H00aS60QBU11S600<DYAS66<H00QS60aBU1aS600HDYAS6
6<HDYAS65:D46<H00aBU6<HHaP0<6<H011BU6<HDYAS60QBU00<HaQBU6<H00QS60aBU2QS600TDYFmk
JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX86<H00aBU6<HDY@025:D01AS65:DDYAS6
5:D021S60aBU0aS600LDYAS66<HHaQBU6<HDY@036<H25:D00aS65:DHaP036<H25:D00aS65:DHaP02
5:D00aS65:DDY@026<H00aBU6<HHaP076<H00aBU6<HHaP066<H25:D00aS65:DDY@035:D36<H00aBU
6<HHaP056<H00aBU6<HHaP0=6<H00aBU6<HHaP056<H011BU6<HDYAS60QBU0QS600<DYAS65:D02aS6
0aBU11S600<DYAS66<H00QS600HDYAS66<HDYAS65:D26<H01ABU6<HHaQS65:D00QS61QBU1QS60QBU
00<HaQBU5:D00QBU00HHaQBU6<HDYAcW6<H45:D26<H00aBU6<HHaP066<H01QBU6<HHaQBU6<HDY@@H
aP055:DHaQS65:DHaP035:D26<H011BU6<HDYAS60QBU0QS600@DYAS66<HHaP8DY@DHaP045:DHaQS6
6<H25:D56<H00aBU6<HHaP086<H25:D26<H25:D36<H35:D46<H45:D36<H45:D26<H00aBU6<HHaP03
5:D36<H00aBU6<HDY@046<H011BU6<HHaQS60aBU00<HaQBU6<H011S600DDYAS66<HDYAS6008DY@04
6<HDYAS65:D66<H011BU6<HHaQS60QBU00<HaQBU6<H01f]J0@000F]J000086]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDYAS65:D56<H00aBU6<HD
Y@035:D66<H35:D00aS65:DDY@046<H011BU6<HHaQBU1QS611BU0aS600<DYAS66<H01AS600<DYAS6
6<H00QBU0QS600<DYAS66<H00QBU00<HaQcW6<H00QS600<DYAS66<H01AS600<DYAS66<H00QS600<D
YAS65:D00aBU0QS600<DYAS66<H011BU0QS600@DYAS65:DDYA8HaP035:DHaQS600@HaP035:DHaQS6
00DHaP8DY@8HaP8DY@@HaP045:DHaQS66<H25:D011S65:DHaQS60aBU00<HaQBU5:D00QBU0QS61ABU
00<HaQBU5:D01aS600<DYAS66<H01QS60QBU0aS60QBU0QS61QBU00@HaQBU5:DHaP@DY@LHaP8DY@03
6<HDYAS600<DY@DHaP035:DHaQS600<DY@<HaP<DY@046<HDYAS65:D46<H35:D56<H011BU6<HDYABU
11S60aBU00<HaQBU5:D00aBU0QS600@DYAS66<HDY@PHaP045:DHaQS66<H45:D36<H35:D011S65:DD
YAS60QBU11S611BU11S600<DYAS66<H021S60aBU0QS600<DYAS65:D00QBU2QS600TDYF]jJeY_Nf]J
JgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX55:D26<H25:D00aS65:DHaP046<H00aBU6<HDY@03
5:D011S65:DHaQBU0aS60aBU11S60QBU11S60QBU1AS60aBU0aS600<DYAS65:D03aS60QBU0QS60QBU
00@HaQBU5:DDY@HHaP8DY@HHaP<DY@046<HDYABU5:D26<H00aBU6<HHaP056<H00aBU6<HHaP066<H2
5:D36<H25:D26<H00aBU6<HDY@036<H25:D36<H25:D26<H011BU6<HDYABU0QS600@DYAS65:DHaP8D
Y@8HaP8DY@8HaPDDY@8HaPDDY@<HaP035:DHaQS6008HaP<DY@036<HDYAS600`HaPDDY@066<HDYABU
5:DHaQBU0QS611BU00@HaQBU5:DDYA/HaP075:DHaQS66<HDYAS65:D011S600<DYAS66<H01ABU1AS6
00HDYAS66<HHaQBU6<H35:D56<H00acW6<HHaP026<H011BU6<HHaQS60aBU0QS60aBU00<HaQBU5:D0
1AS600@DYAS66<HDY@DHaP<DY@056<HDYAS66<HDY@036<H35:D011S65:DDYABU0QS60QBU0aS600<D
YAS65:D00QS600@DYAS66<HHaP8DY@M[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk008DY@036<HDYABU00<HaP035:DHaQS6008H
aP@DY@<HaP035:DHaQS6008DY@THaP8DY@THaP8DY@036<HDYAS6008HaPPDY@8HaP8DY@THaP<DY@03
6<HDYABU008HaP8DY@<HaP035:DHaQBU00<DY@056<HDYAS65:DHaP035:D26<H25:D26<H00aBU6<HH
aP056<H00aBU6<HHaP066<H00aBU6<HHaP026<H25:D26<H25:D46<H00aBU6<HDY@026<H25:D011S6
5:DDYAS60QBU0QS600<DYAS66<H00QBU00DHaQBU5:DDYAS600<DY@@HaP@DY@046<HDYABU6<H25:D6
6<H011BU6<HHaQBU0aS600<DYAS66<H01aS61ABU00@HaQBU6<HDY@HHaPDDY@<HaP<DY@8HaP035:DH
aQBU008HaP<DY@<HaP035:DHaQS600<DY@8HaP035:DHaQS600DHaP045:DHaQS66<H25:D26<H85:D0
0aS65:DDY@035:D00aS65:DDY@056<H27>L26<H45:D26<H35:D46<H00aBU6<HHaP026<H25:D00aS6
5:DDY@025:D00aS65:DHaP026<H35:D01AS65:DDYAS65:D01QS60QBU1AS600<DYAS66<H00QBU2aS6
00Q_Nf]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX00aS65:DDY@025:D011S65:DHaQBU
1AS600@DYAS66<HDY@@HaP035:DHaQS600PHaP035:DHaQS600<DY@8HaP035:DHaQS600@HaP035:DH
aQBU00<DYA0HaP035:DHaQS6008DY@HHaP8DY@046<HDYAS65:D26<H25:D011S65:DHaQBU0QS60QBU
00DHaQBU6<HHaQBU00<HaP035:DHaQS6008HaP8DY@036<HDYAS600XHaPDDY@046<HDYABU6<H25:D0
0aS65:DHaP026<H25:D00aS65:DHaP025:D00aS65:DDY@055:D00aS65:DDY@025:D46<H25:D36<H7
5:D00aS65:DHaP046<H00aBU6<HHaP036<H00aBU6<HHaP086<H00aBU6<HHaP036<H00aBU6<HDY@04
6<H25:D00aS65:DHaP046<H45:D36<H25:D26<H011BU6<HHaQS60QBU00<HaQBU5:D00QBU00@HaQBU
5:DDY@HHaP035:DHaQBU008HaP055:DHaQS66<HDY@056<H35:D36<H00aBU6<HDY@025:D00aS67>LL
i`026<H37>L01ABU6<HHaQBU6<H01aBU0QS600@DYAS65:DDY@@HaP@DY@066<HDYAS66<HDYAS60aBU
00<HaQBU6<H00QS60QBU00<HaQBU5:D01aBU0aS60QBU00@HaQBU6<HDY@@HaP035:DHaQS600M[FP40
005[FP0001i[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]J
Kg][FV]j6<H45:D011S65:DHaQBU1AS600@DYAS66<HDY@@HaP035:DHaQS600PHaP035:DHaQS600<D
Y@8HaP035:DHaQS600@HaP035:DHaQBU00<DYA0HaP035:DHaQS6008DY@HHaP8DY@046<HDYAS65:D2
6<H25:D011S65:DHaQBU0QS60QBU00DHaQBU6<HHaQBU00<HaP035:DHaQS6008HaP8DY@036<HDYAS6
00XHaPDDY@046<HDYABU6<H25:D00aS65:DHaP026<H25:D00aS65:DHaP025:D00aS65:DDY@055:D0
0aS65:DDY@025:D46<H25:D36<H75:D00aS65:DHaP046<H00aBU6<HHaP036<H00aBU6<HHaP086<H0
0aBU6<HHaP036<H00aBU6<HDY@046<H25:D00aS65:DHaP046<H45:D36<H25:D26<H011BU6<HHaQS6
0QBU00<HaQBU5:D00QBU00@HaQBU5:DDY@HHaP035:DHaQBU008HaP055:DHaQS66<HDY@056<H35:D3
6<H00aBU6<HDY@025:D00aS67>LLi`026<H37>L01ABU6<HHaQBU6<H01aBU0QS600@DYAS65:DDY@@H
aP@DY@066<HDYAS66<HDYAS60aBU00<HaQBU6<H00QS60QBU00<HaQBU5:D01aBU0aS60QBU00@HaQBU
6<HDY@@HaP0;5:DHaQS6JgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP035:DHaQS6
00dHaP045:DHaQS66<H25:D26<H00aBU6<HHaP0=6<H011BU6<HDYABU0QS600<LiaS66<H011BU00@H
aQBU5:DDY@<HaP045:DHaQS65:D36<H35:D26<H55:D36<H55:D;6<H55:D00aS65:DHaP026<H25:D0
0aS65:DHaP046<H00aBU6<HHaP026<H25:D011S65:DHaQS60QBU11S600<DYAS66<H00aS60aBU00<H
aQBU6<H00aS60QBU00<HaQBU6<H011S600<DYAS66<H00QBU00@HaQBU5:DHaPDDY@<HaP055:DHaQBU
5:DHaP025:D26<H35:D36<H25:D26<H00aBU6<HHaP076<H011BU6<HHaQBU0QS60aBU00<HaQBU6<H0
0QS60QBU0QS600@DYAS66<HDY@PHaP035:DHaQBU008DY@8HaP055:DHaQBU5:DHaP035:D00aS65:DD
Y@026<H00aBU6<HHaP025:D26<H55:D66<H00aBU6<HHaP025:D01QS65:DDYAS65:DHaP<DY@@HaP04
7>LHaQS65:D56<H25:D26<H55:D00aS65:DDY@036<H00aBU6<HDY@026<H00aBU6<HHaP055:D011S6
5:DDYAS611BU00<HaQBU6<H00QBU1aS600@DYAS66<HDY@XHaPM[FP40005[FP0000Q_Nf]J000006mk
JeY[NV]J0P0000A[NV]JKg][FP80000=Kg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QS600<DYAS6
6<H031S61ABU1QS600<LiaS66<H01AS60QBU0QS60QBU2QS600<DYAS66<H00aBU0QS600<DYAS65:D0
0QS60QBU1AS600<DYAS66<H00aBU0QS600PDYAS66<HHaQBU6<HDYAS611BU0QS600<DYAS66<H01aBU
3AS600<DYAS65:D00aS600<DYAS66<H00QBU11S600@DYAS65:DDY@8HaP@DY@HHaP8DY@036<HDYABU
008HaPHDY@<HaP<DY@DHaP035:DHaQBU008DY@036<HDYABU00@HaP045:DHaQBU5:D46<H00aBU6<HH
aP026<H01ABU6<HHaQS65:D011S60QBU0QS60QBU0QS60QBU11S600@DYAS66<HDY@<HaP8DY@8HaP04
5:DHaQBU6<H45:D00aS65:DHaP035:D26<H35:D26<H01aBU6<HHaQBU6<HDYAS600HDY@PHaP8DY@LH
aP055:DHaQBU5:DHaP025:D36<H25:D00aS65:DHaP046<H00aBU6<HHaP026<H011BU6<HHaQBU0QS6
0aBU0QS600<DYAS66<H01ABU0QS611BU0QS60QBU0QS600TDYAS66<HDYAS65:DHaQBU6<H00QBU1QS6
11BU00THaVmkJeY[NV]JKg][FV]j00000Fmk00001F]J001[FV]J00000V]J00@006]JJeX0009[FP04
001[FV]J0005JeX00`00JeY[FP04JeX26<H011BU6<HHaQBU11S600@DYAS66<HDY@<HaP045:DHaQS6
6<H25:D26<H55:D26<H00aBU6<HHaP036<H011BU6<HHaQBU0aS600<DYAS65:D00QS60QBU00<HaQBU
6<H01AS600DDYAS66<HDYAS600<DY@036<HDYAS6008HaP8DY@DHaP035:DHaQS600@DY@DHaP055:DH
aQBU5:DHaP025:D46<H00aBU6<HHaP026<H35:D01AS65:DDYAS65:D011S60aBU21S600<DYAS65:D0
11BU00<HaQBU6<H011BU00@HaQBU6<HDY@DHaP8DY@036<HDYAS600DDY@8HaP035:DHaQS6008HaP8D
Y@8HaP8DY@036<HDYABU008HaP8DY@8HaP<DY@<HaP035:DHaQBU008HaP055:DHaQS65:DHaP035:D3
6<H011BU6<HDYAS60QBU0QS600<DYAS65:D00QBU1AS600<DYAS66<H01AS60QBU11S600DDYAS66<HD
YAS6008DY@DHaP045:DHaQS65:D46<H25:D:6<H25:D26<H45:D36<H011BU6<HHaQBU0QS600<DYAS6
6<H01AS611BU21S600<DYAS66<H00aS611BU1aS60aBU1aS600@DYAS65:DDY@8HaP045:DHaQBU5:D2
6<H25:D26<H45:D00aS65:DHaP07JeX10001JeX0000FJgY[FVmk001[NV]JKg/006]jJeX006]JJgX0
06mkJeX006]JKg][FV]jJeX400001FmkJeY[NQBU6<H011BU00<HaQBU6<H031S600@DYAS65:DDY@<H
aP8DY@@HaP8DY@036<HDYAS600DHaP045:DHaQS66<H25:D46<H25:D00aS65:DHaP026<H25:D011S6
5:DHaQS60QBU11S60aBU00<HaQBU6<H00QBU0QS60QBU00<HaQBU6<H00aBU2AS600<DYAS66<H011S6
0aBU00DHaQBU5:DDYAS6008DY@036<HDYABU00<HaP035:DHaQS600DHaPDDY@8HaP8DY@8HaP035:DH
aQS6008HaP<DY@DHaP8DY@8HaPDDY@036<HDYABU00@HaP045:DHaQBU5:D26<H35:D46<H35:D36<H4
5:D26<H011BU6<HHaQS60QBU0QS600<DYAS65:D00QBU0QS60aBU11S600<DYAS66<H00aBU00DHaQBU
6<HDYAS600HDY@046<HDYABU5:D36<H25:D36<H25:D46<H25:D26<H00aBU6<HDY@026<H35:D00aS6
5:DDY@026<H00acW6<HHaP066<H01ABU6<HDYABU6<H00QBU0aS600<DYAS66<H00QS600<DYAS65:D0
11S60QBU21S60QBU00<HaQBU5:D00QBU00DHaQBU5:DHaQBU008HaP035:DHaQS6008HaP@DY@<HaP04
5:DHaQS66<H45:D011S65:DDYABU21S60QBU00E[NV]JKg][FV]j00<0005[NP0016]J00@006]JJeX0
009[FP04001[FV]J0002JeX00`00JeY[FP03JeX00`00JeY[FP04JeX26<H011BU6<HHaQS60QBU00<H
aQBU5:D00aS600<DYAS66<H011BU0QS600<DYAS66<H00QS600DDYAS65:DDYAS600<DY@8HaP035:DH
aQcW00<HaP045:DHaQS65:D76<H35:D66<H35:D01QS65:DHaQS65:DHaP8DY@046<HDYAS66<H25:D3
6<H25:D66<H00aBU6<HHaP046<H00aBU6<HHaP0:6<H25:D01QS65:DHaQBU6<HDY@<HaP055:DHaQS6
6<HDY@046<H25:D00aS65:DDY@045:D00aS65:DHaP066<H00aBU6<HHaP026<H25:D26<H45:D011S6
5:DDYABU0aS60QBU00<HaQBU5:D00QBU0QS60QBU00<HaQBU5:D00QBU00@HaQBU6<HDY@<HaP8DY@<H
aP8DY@056<HDYABU5:DHaP035:D00aS65:DDY@025:D011S65:DDYABU0aS600<DYAS66<H011S611BU
0aS60QBU00<HaQBU5:D00QS600@DYAS66<HHaP8DY@<HaP035:DHaQS6008HaP055:DHaQS65:DHaP03
5:D00aS65:DHaP046<H00acW6<HHaP036<H25:D26<H011BU6<HHaQBU2QS600<DYAS66<H011BU1QS6
0aBU00@HaQBU6<HDY@<HaP<DY@036<HDYAS6008HaP8DY@<HaP035:DHaQS6008DY@036<HDYAS6008H
aP035:DHaQS600<DY@LHaP065:DHaQS65:DHaQBU1f]J0@000F]J00007Fmk001[NV]J001[FV]j001_
Nf]J001[FVmk001[NV]J001[FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00aS61ABU0QS60aBU0QS600<D
YAcW6<H01AS60aBU0aS600<DYAS65:D00QS60aBU0aS60QBU0aS60QBU0QS600LDYAS66<HDYAS65:DH
aP025:D46<H00aBU6<HHaP026<H00aBU6<HHaP075:D46<H65:D56<H00aBU6<HHaP055:D86<H011BU
6<HHaQS60QBU00<HaQBU5:D00QBU00<HaQBU5:D00aS600<DYAS65:D00QS611BU21S600<DYAS66<H0
1QS60aBU0QS600<DYAS66<H00QBU0aS600<DYAS66<H00QBU00<HaQBU6<H00QS60aBU00<HaQBU6<H0
0QBU0QS600HDYAS65:DDYAS65:D56<H00aBU6<HDY@026<H00aBU6<HDY@025:D66<H00aBU6<HDY@03
5:D011S65:DDYABU11S60QBU00THaQBU5:DDYAS65:DHaQBU6<H00aBU00@HaQBU6<HDY@@HaP035:DH
aQBU008DY@<HaP035:DHaQS600DHaP045:DHaQS65:D56<H25:D00acW5:DHaP0>6<H00aBU6<HHaP07
6<H25:D011S65:DHaQBU1AS60aBU0aS60QBU00<HaQBU6<H00QS600@DYAS65:DDY@LHaP035:DHaQBU
00@DY@036<HDYABU008DY@036<HDYABU008HaP035:DHaQS6008HaP095:E_Nf]JJgY[FVmkJeY[NP00
005_N`000V]J0P0016]J0P0016]J0P001V]J00<006]JJeX016]J1ABU00DHaQBU5:DHaQBU00<HaP03
5:DHaQS600HHaP045:DHaQBU5:D36<H011BU6<HHaQBU11S600<DYAS66<H011S600DDYAS66<HDYAS6
00DDY@THaP8DY@@HaPHDY@046<HDYAS65:D26<H35:D26<H00aBU6<HHaP036<H01ABU6<HDYABU6<H0
0QBU0QS600<DYAS66<H00QS611BU00@HaQBU5:DDY@<HaP035:DHaQS600DHaP<DY@8HaP065:DHaQS6
6<HDYAS60aBU1QS600@DYAS65:DHaPDDY@<HaP@DY@036<HDYAS6008HaP065:DHaQS65:DHaQBU1QS6
00@DYAS66<HHaP@DY@PHaP8DY@056<HDYABU5:DHaP025:D00aS65:DHaP065:D26<H35:D011S65:DD
YABU1AS62ABU1QS60QBU1AS600<DYAS66<H011S600<DYAS65:D01AS600<DYAS66<H011S600@DYAS6
6<HDY@@HaP055:DHaQS66<HDY@036<H00aBU6<HHaP025:D:6<H00aBU6<HDY@026<H011BU6<HHaQBU
1aS600HDYAS66<HHaQBU6<H25:D00aS65:DHaP056<H00aBU6<HDY@026<H45:D26<H00aBU6<HHaP02
6<H011BU6<HDYABU0aS600<DYAS65:D01f]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX01ABU00DHaQBU5:DHaQBU00<HaP035:DH
aQS600HHaP045:DHaQBU5:D36<H011BU6<HHaQBU11S600<DYAS66<H011S600DDYAS66<HDYAS600DD
Y@THaP8DY@@HaPHDY@046<HDYAS65:D26<H35:D26<H00aBU6<HHaP036<H01ABU6<HDYABU6<H00QBU
0QS600<DYAS66<H00QS611BU00@HaQBU5:DDY@<HaP035:DHaQS600DHaP<DY@8HaP065:DHaQS66<HD
YAS60aBU1QS600@DYAS65:DHaPDDY@<HaP@DY@036<HDYAS6008HaP065:DHaQS65:DHaQBU1QS600@D
YAS66<HHaP@DY@PHaP8DY@056<HDYABU5:DHaP025:D00aS65:DHaP065:D26<H35:D011S65:DDYABU
1AS62ABU1QS60QBU1AS600<DYAS66<H011S600<DYAS65:D01AS600<DYAS66<H011S600@DYAS66<HD
Y@@HaP055:DHaQS66<HDY@036<H00aBU6<HHaP025:D:6<H00aBU6<HDY@026<H011BU6<HHaQBU1aS6
00HDYAS66<HHaQBU6<H25:D00aS65:DHaP056<H00aBU6<HDY@026<H45:D26<H00aBU6<HHaP026<H0
11BU6<HDYABU0aS600/DYAS65:E[NV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00<D
YAS66<H01QS60QBU0QS600<DYAS65:D00QS60QBU0aS60aBU1aS600<DYAS66<H00aS60QBU1QS60QBU
00<HaQBU5:D00aS61QBU0QS600<DYAS65:D00QBU00@HaQBU5:DHaPHDY@8HaP055:DHaQS65:DHaP02
5:D56<H01ABU6<HDYAS65:D01AS600<DYAS66<H00aS60aBU00<HaQBU5:D00QBU00<HaQBU6<H00aS6
0QBU00<HaQBU5:D00aBU11S600<DYAS66<H00QS600@DYAS66<HLi`DHaP035:DHaQS6008DY@8HaP<D
Y@046<HDYABU5:D46<H011BU6<HHaQBU0QS60QBU1AS61ABU0QS600@DYAS66<HDY@8HaP055:DHaQS6
6<HDY@036<H00aBU6<HHaP036<H25:D011S65:DDYABU1QS60QBU00<HaQBU5:D00aS600<DYAS65:D0
0QBU00<HaQBU6<H03ABU11S600<DYAS66<H021S600<DYAS66<H01QS600<DYAS66<H00QS60QBU0QS6
0QBU11S600@DYAS66<HDY@@HaP035:DHaQBU008HaPPDY@<HaP8DY@8HaP035:DHaQS600<HaP035:DH
aQS600<DY@056<HDYABU6<HDY@056<H00aBU6<HHaP026<H25:D26<H35:D26<H25:D00aS65:DHaP02
6<H00aBU6<I[FP06JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_NaS65:DHaP8DY@<HaP8DY@066<HDYABU5:DHaQcW0QBU0aS60QBU
0aS60QBU00<HaQBU6<H01AS60aBU0QS61ABU00@HaQBU5:DHaP<DY@056<HDYABU5:DHaP045:D56<H2
5:D26<H55:D26<H25:D46<H011BU6<HDYAS60aBU00<HaQBU5:D031S60QBU0QS600<DYAS65:D00QBU
0QS600<DYAS66<H00QBU0aS600<DYAS65:D00QBU0QS60QBU3AS600@DYAS66<HHaP<DY@046<HDYAS6
6<H35:D36<H35:D36<H011BU6<HHaQBU0aS600@DYAS66<HHaP<DY@THaP035:DHaQS600LHaP8DY@06
6<HDYABU6<HDYAS61ABU00@HaQBU6<HHaP<DY@HHaP8DY@036<HDYAS600<HaP<DY@<HaP085:DHaQBU
5:DHaQBU6<HDYADHaP8DY@@HaP035:DHaQS600<DY@<HaP035:DHaQBU008DY@8HaP8DY@036<HDYAS6
008DY@056<HDYAS65:DHaP045:D26<H35:D36<H25:D56<H011BU6<HDYAS61ABU0QS60QBU0QS600@D
YAS66<HHaP@DY@056<HDYAS65:DHaP035:D26<H02ABUKg][FV]jJeY_Nf]JJgX00001Kg/001I[FP03
001[FV]J00A[FP035:DHaQBU00@HaP045:DHaQS66<H25:D26<H00aBU6<HHaP025:D26<H00aBU6<HH
aP036<H35:D56<H011BU6<HHaQBU0QS60aBU1AS60aBU0aS600DDYAS66<HHaQBU008HaP8DY@HHaP04
5:DHaQS66<H45:D26<H35:D01AS65:DDYABU6<H00QBU1QS60aBU4AS60aBU0aS600DDYAS65:DHaQBU
008HaP065:DHaQBU6<HDYAS60QBU1aS600@DYAS66<HHaP8DY@8HaP035:DHaQS600XHaP<DY@8HaP04
5:DHaQS66<H25:D00aS65:DHaP046<H00aBU6<HDY@025:D01aS65:DDYABU6<HDYAS600LDY@HHaP8D
Y@036<HDYABU00@DY@PHaPXDY@036<HDYABU00<HaP035:DHaQS6018HaP035:DHaQS600LHaP8DY@05
6<HDYABU5:DHaP045:D36<H01ABU6<HHaQS65:D00aS60aBU00@HaQBU5:DHaPDDY@8HaP@DY@056<HD
YABU5:DHaP075:D00aS65:DDY@036<H45:D36<H25:D26<H35:D36<H00aBU6<HDY@065:D00aS65:E[
FP06JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX0
06]JJgY[FVmkJeY[NP026<H011BU6<HHaQBU0QS60aBU0aS600<DYAS66<H00aS600<DYAS65:D02aS6
0QBU1AS60aBU0aS621BU0aS60aBU0QS600DDYAS65:DDYAS6008DY@036<HDYABU008HaP8DY@HHaP8D
Y@036<HDYAS600<DY@046<HDYAS65:D;6<H00aBU6<HHaP026<H00aBU6<HHaP046<H25:D26<H25:D0
0aS65:DHaP056<H00aBU6<HHaP035:D46<H011cW5:DDYAS60QBU1aS600@DYAS66<HHaP8DY@8HaP04
5:DHaQS65:D36<H95:D66<H25:D36<H25:D00aS65:DDY@026<H00aBU6<HHaP035:D56<H00aBU6<HH
aP035:D46<H00aBU6<HHaP025:D011S65:DHaQS61QBU11S60QBU4QS600<DYAS66<H011S600<DYAS6
6<H01QS60aBU0aS600<DYAS66<H011S600<DYAS65:D01QBU00<HaQBU5:D01QBU00<HaQBU5:D00aS6
00DDYAS65:DHaQBU00<HaP8DY@036<HDYAS600<DY@@HaP035:DHaQS600DHaP8DY@<HaP8DY@LHaPDD
Y@0:6<HDYF]jJeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP<HaP8DY@056<HDYAS65:DH
aP065:D00aS65:DDY@025:D00aS65:DHaP046<H00aBU6<HDY@056<H00aBU6<HHaP065:D00aS65:DD
Y@025:D36<H00aBU6<HHaP096<H25:D00aS65:DDY@025:D46<H25:D00acW5:DHaP026<H00aBU6<HH
aP026<H35:D00aS65:DHaP036<H25:D00aS65:DHaP096<H01QBU6<HHaQS65:DHaP8DY@036<HDYABU
008HaP8DY@046<HDYABU5:D56<H35:D36<H25:D<6<H00aBU6<HHaP076<H45:D66<H25:D36<H00aBU
6<HHaP036<H25:D01aS65:DHaQS65:DHaQBU008HaP035:DHaQS6008HaP<DY@<HaP8DY@036<HDYAS6
00@HaP045:DHaQS66<H25:D26<H011BU6<HDYABU1aS60aBU0aS600<DYAS65:D00aS600<DYAS65:D0
1AS600@DYAS66<HDY@<HaP035:DHaQS6008DY@@HaP8DY@PHaP075:DHaQS66<HDYAS65:D00QS61aBU
00<HaQBU5:D00QBU00@HaQBU5:DDY@8HaP<DY@036<HDYABU00<HaP055:DHaQBU5:DHaP025:D36<H0
0aBU6<HHaP026<H01ABU6<HHaQS65:D00aS61ABU1AS61QBU00<HaQBUJeX01V]J0@000F]J00007Fmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QBU
00@HaQBU5:DDY@8HaP<DY@<HaP8DY@036<HDYABU008DY@036<HDYAS6008DY@@HaP@DY@@HaP8DY@8H
aP8DY@<HaP8DY@056<HDYABU6<HDY@046<H25:D00aS65:DHaP046<H00aBU6<HDY@076<H00aBU6<HD
Y@065:D36<H25:D<6<H35:D96<H00aBU6<HDY@025:D26<H35:D26<H00aBU6<HHaP036<H00aBU6<HH
aP026<H25:D86<H00aBU6<HDY@025:D01QS65:DDYABU6<HDY@<HaP<DY@046<HDYABU5:D36<H25:D8
6<H35:D00aS65:DHaP025:D46<H35:D36<H45:D00aS65:DHaP036<H011BU6<HHaQS60QBU2aS61ABU
0aS600<DYAS66<H011S60QBU1QS600<DYAS66<H00QBU00DHaQBU6<HHaQBU00<HaP035:DHaQS6008H
aP<DY@8HaP<DY@@HaP045:DHaQBU6<H25:D00aS65:DHaP026<H01ABU6<HHaQS65:D00QS600<DYAS6
6<H011BU00<HaQBU5:D02ABU11S611BU00HHaQBU5:DHaQBU6<H25:D26<H00aBU6<HHaP066<H01ABU
6<HHaQS65:D00QS61ABU00/HaQBU5:E_Nf]JJgY[FVmkJeY[NP00005_N`005V]J0`0016]J00<DYAS6
5:D00QBU00@HaQBU6<HDY@8HaP8DY@<HaP035:DHaQBU008DY@036<HDYABU008HaP8DY@036<HDYABU
00DHaP035:DHaQBU00HHaP8DY@LHaP035:DHaQBU00DHaP035:DHaQBU00@HaP045:DHaQS66<H25:D0
11S65:DHaQS60QBU0QS60QBU0QS600DDYAS66<HHaQBU00<HaP055:DHaQS66<HDY@026<H00aBU6<HH
aP026<H011BU6<HDYABU1AS600@DYAS65:DDY@8HaP065:DHaQS66<HDYAS60QBU0QS60aBU0aS611BU
21S600<DYAS66<H00QS611BU0QS60aBU00<HaQBU6<H011BU11S60QBU00@HaQBU6<HHaP<DY@<HaP8D
Y@046<HDYABU6<H75:D00aS65:DDY@026<H45:D26<H35:D46<H00aBU6<HDY@025:D00aS65:DHaP03
6<H55:D26<H011BU6<HHaQBU0QS600<DYAS66<H011BU00@HaQBU6<HDY@<HaP8DY@8HaP<DY@<HaP06
5:DHaQS66<HDYAS60QBU00<HaQBU6<H00QS60QBU1QS600<DYAS66<H021S600DDYAS66<HDYAS600HD
Y@036<HDYABU00THaPHDY@036<HDYAS600<DY@8HaP035:DHaQS6008HaP<DY@036<HDYABU008HaP03
5:DHaQBU008HaP@DY@8HaPI[FP80005[FP0001m[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaP035:D011S65:DHaQBU0QS60QBU0aS600<DYAS6
5:D00QBU00<HaQBU5:D00QS60QBU00<HaQBU5:D01AS600<DYAS65:D01QS60QBU1aS600<DYAS65:D0
1AS600<DYAS65:D011S600@DYAS66<HHaP8DY@046<HDYAS66<H25:D26<H25:D26<H01ABU6<HHaQS6
5:D00aS600DDYAS66<HHaQBU008HaP035:DHaQS6008HaP045:DHaQBU5:D56<H011BU6<HDYABU0QS6
00HDYAS66<HHaQBU6<H25:D26<H35:D36<H45:D86<H00aBU6<HHaP026<H45:D26<H35:D00aS65:DH
aP045:D46<H25:D011S65:DHaQS60aBU0aS60QBU00@HaQBU5:DHaPLDY@036<HDYABU008HaP@DY@8H
aP<DY@@HaP035:DHaQBU008DY@036<HDYAS600<HaPDDY@8HaP045:DHaQS65:D26<H00aBU6<HHaP04
5:D011S65:DHaQBU0aS60QBU0QS60aBU0aS600HDYAS66<HHaQBU6<H25:D00aS65:DHaP026<H25:D6
6<H00aBU6<HHaP086<H01ABU6<HHaQBU6<H01QBU00<HaQBU5:D02AS61QBU00<HaQBU6<H00aBU0QS6
00<DYAS66<H00QS60aBU00<HaQBU5:D00QS600<DYAS65:D00QS611BU0QS600Q[NV]JKg][FV]jJeY_
N`000F]j000FJeX00`00JeY[FP04JeX26<H45:D36<H25:D26<H25:D56<H25:D00aS65:DDY@036<H0
1QBU6<HHaQS65:DHaP8DY@<HaP8DY@<HaP045:DHaQS66<H25:D26<H00aBU6<HHaP036<H25:D46<H5
5:D00aS65:DHaP036<H011BU6<HHaQBU0QS60QBU0QS60QBU0QS60QBU2QS600<DYAS66<H02QS600HD
YAS66<HHaQBU6<H65:D00aS65:DHaP026<H00aBU6<HHaP025:D26<H00aBU6<HDY@045:D36<H25:D6
6<H00aBU6<HDY@035:D00aS65:DDY@026<H25:D00aS65:DDY@025:D46<H00aBU6<HHaP055:D86<H0
11BU6<HHaQS60QBU00@HaQBU6<HDY@DHaP045:DHaQBU5:D46<H35:D00aS65:DDY@046<H00aBU6<HH
aP066<H011BU6<HHaQBU0QS60QBU1AS600<DYAS65:D011BU1QS600@DYAS66<HDY@8HaPDDY@8HaP03
5:DHaQS600DHaP035:DHaQS6008HaP035:DHaQBU00@HaPLDY@8HaPPDY@036<HDYAS600HHaP045:DH
aQBU5:D86<H00aBU6<HHaP025:D26<H00aBU6<HDY@025:D021S65:DDYAS65:DHaQBU6<H55:D00aS6
JeY[FP05JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeX006]JKg][FV]jJeY_NaS65:DHaP8DY@@HaP065:DHaQBU5:DHaQBU1AS60aBU21S600<DYAS66<H0
0aBU0QS600<DYAS66<H00QBU00<HaQBU6<H01aS60QBU11S60QBU0QS600HDYAS66<HHaQBU6<H25:D6
6<H35:D36<H01ABU6<HDYAS65:D00aS600DDYAS66<HDYAS600<DY@<HaP045:DHaQS65:D36<H011BU
6<HHaQS60QBU00@HaQBU6<HHaP<DY@8HaP035:DHaQS6008DY@8HaP035:DHaQS600<DY@lHaP8DY@03
6<HDYABU00<DY@8HaP<DY@@HaP035:DHaQS600LHaP@DY@8HaP035:DHaQS600/HaP8DY@8HaPPDY@08
6<HDYABU5:DHaQBU6<HDY@<HaP035:DHaQS6008HaP045:DHaQS66<H35:D96<H021BU6<HHaQS65:DH
aQBU6<H35:D00aS65:DHaP046<H25:D011S65:DHaQBU0QS600@DYAS66<HDY@8HaP8DY@036<HDYABU
00HHaP<DY@8HaP8DY@@HaP035:DHaQS6008HaP045:DHaQS65:D36<H25:D00aS65:DDY@056<H25:D0
0aS65:DDY@096<H35:D011S65:DDYAS61ABU0QS600<DYAS66<H00QS61QBU00Q_Nf]JJgY[FVmkJeY[
NP000Fmk000FJeX00`00JeY[FP04JeX00aBU6<HDY@036<H35:D26<H25:D26<H00aBU6<HHaP026<H0
11BU6<HHaQBU0QS61ABU1AS60aBU00<HaQBU6<H00QBU11S600<DYAS66<H00aS600<DYAS66<H011BU
0QS600DDYAS65:DDYAS600<DY@8HaP8DY@036<HDYAS600DHaP035:DHaQS600PHaP035:DHaQBU00<H
aP045:DHaQS66<H25:D00aS65:DHaP066<H75:D00aS65:DHaP076<H011BU6<HHaQBU0QS60QBU0aS6
0QBU0QS60aBU1aS600<DYAS66<H00QS600<DYAS66<H00QBU1QS600<DYAS65:D00QBU0aS60aBU0QS6
0aBU11S60aBU00@HaQBU5:DDY@@HaP045:DHaQBU5:D26<H00aBU6<HHaP056<H25:D46<H00aBU6<HH
aP025:D:6<H65:D26<H25:D36<H35:D011S65:DHaQS61aBU00LHaQBU5:DDYAS65:DHaP025:D56<H2
5:D46<H00aBU6<HDY@026<H35:D36<H011BU6<HHaQS60QBU00<HaQBU5:D00aBU00<HaQBU5:D00aBU
11S600<DYAS66<H01ABU1QS60QBU0aS61QBU0QS60QBU0aS600<DYAS65:D00QS600<DYAS66<H01f]J
0@000F]J0000of]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_N`00SF]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J0?m[FW=[FP40005[FP000?m_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX008e[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP3oJeYcJeX10001JeX0003oJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j001_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]JKg/006]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgX006mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
N`00005[NP005V]J00<006]JJeX016]J00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX02F]J
00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J
00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J
00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX02F]J
00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J
00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX026]J
00<006]JJeX02F]J00<006]JJeX026]J00<006]JJeX02F]J00<006]JJeX01f]J0@000F]J00005Vmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]Jo`00M`000Fmk003oJeZ>JeX0
003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]J005[NP00of]JSV]J0000ofmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NP00SF]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP01Kg/00001\
\>"],
  ImageRangeCache->{{{0, 396}, {396, 0}} -> {-26.0307, -22.1602, 0.872225, \
0.872225}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  DensityGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[3]="]
}, Closed]],

Cell["\<\
Read in and display a 100x100 subset of this FITS image. Small subsets of \
images are read in very quickly... this example takes under a second on my \
slowest machine.\
\>", "Text"],

Cell[BoxData[
    \(\(im = 
        FitsRead["\<C:\\Windows\\Desktop\\N2715_lr.fit[115:215,105:205]\>"];\)\
\)], "Input",
  CellLabel->"In[4]:="],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListDensityPlot[im, Mesh \[Rule] False, 
      PlotRange \[Rule] {1000, 3000}]\)], "Input",
  CellLabel->"In[5]:="],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% DensityGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0192308 0.00952018 0.0192308 0.00952018 [
[.01923 -0.0125 -3 -9 ]
[.01923 -0.0125 3 0 ]
[.20963 -0.0125 -6 -9 ]
[.20963 -0.0125 6 0 ]
[.40004 -0.0125 -6 -9 ]
[.40004 -0.0125 6 0 ]
[.59044 -0.0125 -6 -9 ]
[.59044 -0.0125 6 0 ]
[.78085 -0.0125 -6 -9 ]
[.78085 -0.0125 6 0 ]
[.97125 -0.0125 -9 -9 ]
[.97125 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01923 -6 -4.5 ]
[-0.0125 .01923 0 4.5 ]
[-0.0125 .20963 -12 -4.5 ]
[-0.0125 .20963 0 4.5 ]
[-0.0125 .40004 -12 -4.5 ]
[-0.0125 .40004 0 4.5 ]
[-0.0125 .59044 -12 -4.5 ]
[-0.0125 .59044 0 4.5 ]
[-0.0125 .78085 -12 -4.5 ]
[-0.0125 .78085 0 4.5 ]
[-0.0125 .97125 -18 -4.5 ]
[-0.0125 .97125 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 1 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01923 0 m
.01923 .00625 L
s
[(0)] .01923 -0.0125 0 1 Mshowa
.20963 0 m
.20963 .00625 L
s
[(20)] .20963 -0.0125 0 1 Mshowa
.40004 0 m
.40004 .00625 L
s
[(40)] .40004 -0.0125 0 1 Mshowa
.59044 0 m
.59044 .00625 L
s
[(60)] .59044 -0.0125 0 1 Mshowa
.78085 0 m
.78085 .00625 L
s
[(80)] .78085 -0.0125 0 1 Mshowa
.97125 0 m
.97125 .00625 L
s
[(100)] .97125 -0.0125 0 1 Mshowa
.125 Mabswid
.06683 0 m
.06683 .00375 L
s
.11443 0 m
.11443 .00375 L
s
.16203 0 m
.16203 .00375 L
s
.25724 0 m
.25724 .00375 L
s
.30484 0 m
.30484 .00375 L
s
.35244 0 m
.35244 .00375 L
s
.44764 0 m
.44764 .00375 L
s
.49524 0 m
.49524 .00375 L
s
.54284 0 m
.54284 .00375 L
s
.63804 0 m
.63804 .00375 L
s
.68564 0 m
.68564 .00375 L
s
.73324 0 m
.73324 .00375 L
s
.82845 0 m
.82845 .00375 L
s
.87605 0 m
.87605 .00375 L
s
.92365 0 m
.92365 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01923 m
.00625 .01923 L
s
[(0)] -0.0125 .01923 1 0 Mshowa
0 .20963 m
.00625 .20963 L
s
[(20)] -0.0125 .20963 1 0 Mshowa
0 .40004 m
.00625 .40004 L
s
[(40)] -0.0125 .40004 1 0 Mshowa
0 .59044 m
.00625 .59044 L
s
[(60)] -0.0125 .59044 1 0 Mshowa
0 .78085 m
.00625 .78085 L
s
[(80)] -0.0125 .78085 1 0 Mshowa
0 .97125 m
.00625 .97125 L
s
[(100)] -0.0125 .97125 1 0 Mshowa
.125 Mabswid
0 .06683 m
.00375 .06683 L
s
0 .11443 m
.00375 .11443 L
s
0 .16203 m
.00375 .16203 L
s
0 .25724 m
.00375 .25724 L
s
0 .30484 m
.00375 .30484 L
s
0 .35244 m
.00375 .35244 L
s
0 .44764 m
.00375 .44764 L
s
0 .49524 m
.00375 .49524 L
s
0 .54284 m
.00375 .54284 L
s
0 .63804 m
.00375 .63804 L
s
0 .68564 m
.00375 .68564 L
s
0 .73324 m
.00375 .73324 L
s
0 .82845 m
.00375 .82845 L
s
0 .87605 m
.00375 .87605 L
s
0 .92365 m
.00375 .92365 L
s
.25 Mabswid
0 0 m
0 1 L
s
.01923 .99375 m
.01923 1 L
s
.20963 .99375 m
.20963 1 L
s
.40004 .99375 m
.40004 1 L
s
.59044 .99375 m
.59044 1 L
s
.78085 .99375 m
.78085 1 L
s
.97125 .99375 m
.97125 1 L
s
.125 Mabswid
.06683 .99625 m
.06683 1 L
s
.11443 .99625 m
.11443 1 L
s
.16203 .99625 m
.16203 1 L
s
.25724 .99625 m
.25724 1 L
s
.30484 .99625 m
.30484 1 L
s
.35244 .99625 m
.35244 1 L
s
.44764 .99625 m
.44764 1 L
s
.49524 .99625 m
.49524 1 L
s
.54284 .99625 m
.54284 1 L
s
.63804 .99625 m
.63804 1 L
s
.68564 .99625 m
.68564 1 L
s
.73324 .99625 m
.73324 1 L
s
.82845 .99625 m
.82845 1 L
s
.87605 .99625 m
.87605 1 L
s
.92365 .99625 m
.92365 1 L
s
.25 Mabswid
0 1 m
1 1 L
s
.99375 .01923 m
1 .01923 L
s
.99375 .20963 m
1 .20963 L
s
.99375 .40004 m
1 .40004 L
s
.99375 .59044 m
1 .59044 L
s
.99375 .78085 m
1 .78085 L
s
.99375 .97125 m
1 .97125 L
s
.125 Mabswid
.99625 .06683 m
1 .06683 L
s
.99625 .11443 m
1 .11443 L
s
.99625 .16203 m
1 .16203 L
s
.99625 .25724 m
1 .25724 L
s
.99625 .30484 m
1 .30484 L
s
.99625 .35244 m
1 .35244 L
s
.99625 .44764 m
1 .44764 L
s
.99625 .49524 m
1 .49524 L
s
.99625 .54284 m
1 .54284 L
s
.99625 .63804 m
1 .63804 L
s
.99625 .68564 m
1 .68564 L
s
.99625 .73324 m
1 .73324 L
s
.99625 .82845 m
1 .82845 L
s
.99625 .87605 m
1 .87605 L
s
.99625 .92365 m
1 .92365 L
s
.25 Mabswid
1 0 m
1 1 L
s
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
.01923 .01923 translate
.96154 .96154 scale
101 string
101 101 8 [101 0 0 101 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
30332F353036313130372E3834333136333033373334363433363134343035393A353535
31373838353E3C404244484A494F56585C6363555755545C58595958565357564E514C4F
4B47484A45464447413C413B3F413F413B373C393B3B3A363733333032
2D3132323330342B303032332F3232383535323035353437343334383632333836353736
373B35373837363E41494B4E545A5F5B60635F58575C5A5756575A5454535852574F4A4C
4E4D4746443F483C3E423D3A3E403F413A3E3D3D3B3B39383536353336
2F313332323433343431313135303332353637363433323233332F353637353733313734
33333A3F3C383A4042484D4D586163615E5C565757575C595C57555A565451544C4F4E48
4A4C4746443D41423E443E413D3F3F423C3B3C403B353A383837353637
34323533323133302E3335332F3433323033363631363634393634343B31383637353636
37353936383E3C42474E51575C616E685C585558585C57575E575B5B57545954534E4C48
4B484946424246423E413E3C3F3F3D3D3F3B3F3B383838373737353332
32323232373533363633343131343332363234373634353236343333343237343A383934
383A3E37393E414650515B5D5D61706359535C57565855595F595B5752545251524D484A
4749424743414440444040403B3F3D3D3C3E3B3F3D383B3E3638313534
2D3131343333323435353431323036363233303133332F363139373838373335363B3537
393435383E42484D515A5C615E5B615A59585B5C5A5D5D5D655B5957585352585253524D
48444846474643424442413E423F3D3C3B39383A383B3A3A3839373537
33373134323534353432323432362D343634343336333737353135363636383637363738
3A383A41404648525A60625D515957595A575D5C56555D5A5D5C5756585755544E4E4F4B
4D484646434642414145404140403D3D3D3B3B3D3E3834373931323236
313432353433312F3532313232313332343431343232363235343A33363D353738393D39
383D3E3D404B4C59606B665C555B5B5F5B5657575B565C605F56555A565C585456515351
4C4748494843474142413C3E3E3C413A3C3F3C403F373A3E3737323437
3731332F3335353133343436343432323033313431343A34353734363436313837383839
3A3D3C42414A555C7277685D5C5B5D5F5C5A565A5C61645E5C585B5E575B57575756524E
514C494649464A4640423D3C3D3D3C3B3D3C3C3838393B383738353733
3533323931323235323235353933343236352F35333433353336393635343932393A373C
3D3C453E474A58627E7C665F605F5A5E62605E5A616467605F5C5F5C60605A5D60575054
56524D51444A47423E40413E40433F3C393F3A38393C37363636353333
34303233313234342F33342E35343330323231363334363637323633343A3633393B383C
3F423B46444E5A63736D5E5E59605E5C60615D5D656C6560605B5B5E5F5A5F5C59575456
58564F484848423C44403C3C3D3F3E3E3A3A3D383B3B383B3438373535
33353035333330373A313232333731323632352F3131393432353237353E353A35373B3A
3F3F40454B575C67706A5F575B606065605C616263686B6361615C5F5D5B5B5857555757
52524D4C4B4E45424142413F433D403E3C3C3B3B3B3A37363C34383335
3537343737333333343632333635333633332F35343032343434393A38393737353C3B3A
3F424346515F6970726A625D5E5E615C5A5F615E6767666664625E605C5D605953595856
5853524B4C494946424340383D3E403D3C39373A383D38393834333231
342F303133312D3432313537313530343431323331353333353A3234343738393D3F4141
45474649536069706A6C685B606065635B61666569625B60616467625D5C5F5750535754
54534F504E4B4343473C413D42433E3F3A3B3B3D3838373433363E3436
312E2E3232312F303231303436323638343333313133333630373B3632333738393C3D45
4647474B51616F6C6C656464616867656163646D66646363646568615C645C5356555455
504D4D53464F4744434440403B3D443E3B3B39383837393B3834373934
3132323330312F3130353133333A323835333237333633373536343138373C38373B4044
44484C5056636F6E6E65626261696768666668676568676A6C636460615C585B5C595957
524F505151524C4A4743423F4243413F3C39383C38373637393536393A
333530333234303332303236343534343632353739343136323638383A393B39373E4045
484951525A6573736A645E62626B6A6865686665676C6B6F686B646161605F5E62625D51
594C4F51534F4C4A494642443D413C3B393537353A3B37373934393633
33343231373135313435323533343430363A333535363534353538393A3A39353E3C4346
4D4B4D56636575766E635D606967686C6A6A6D6E6B70736F6E6767675E5F595B66666059
564D4E4F52534D4E4544423F3A4041413B38353A3A3639393A35363635
3236313434393236343235333333313633333332323431393837343B3C38383D3D424A49
5356515B6370737C6A625E676E696669606E6F70747473726869606360615E645D5D5457
524F4F574C514D4E484042404043403A3636343C3A3A39393A413A3832
323435373436352F3A3632333934343736303A363637333B3B3C363C38403C404044444D
51545B5E6C76848A7266676D6E676A64717174756F747974716961665F655D635956554D
52524D4D5251524A484440434040413D3B383139393D39373A3A343836
3033353438303137353031353635332F363333313A3534383736373B3C423F3C4242494E
525C5A5C6C7E9EAE876B7170717068696C717A777673787D7B716E65606062625B5B5550
504F4F534F4E504B49463E41423E3B3E3C383C3C3D3D35383735393531
31353130333A3233333136342F3533332F3936343738333B383739393C3F3F424048494F
54585B63657C9CAC8E6F716D7675747377787A7E7A787A7B7C6C6966626366625F5B5756
545456545755525146444343413E3B383939393C39383B373435353936
3433353433333434333936353233313530353235373634383A3C3D4044454346464E4F58
54585C6C6F7B7F807B6D7674727377787E777B817C7A7B7B77776C7070767A66655D5D5D
5B5D5D5A5D55504F4F45414345403E3A3B36373E38383A3639383A3337
3536303534343337363333323635333531333732393B3433393947504F4A4544474C5558
5759666C7276786E756C707972727C7B7D7C838A83807A7677756D74787E7A6F64676C67
67706867665C545050474244423F3C3938393E39383A38353836383137
3231343336383433383033323632343234333734343934383D373B464C454246484E525B
5757696D727676717072746B717978798083868983807D736C6A6F75797171686D7B8377
736D6D695E5957544E484242413E3E3D36373B37373A3A393639343232
3335323336373438343237373635353634363434383835373B3B393D42424846514E545A
5A60676E737773746E777174777F7C7F878886897F77797471716E75726363646B7E8377
74726D6C5F5856554F474943413B3D3B3D393C37383833393736353736
34313133333633383237313630372F343430343B313A34363B3B3A3B3D42474C50545359
5C646E7479787675767275738389888B8A9089857C7B7B79767473726E65646A6E7D757E
766C67625B5857534D49464044423D3E3A383839383A37363932373535
3436303533323731383832333133323135383638373433383A393B404347444A51565C5A
5F707A8075767C777B787C8286948F8992908C8688867B7C7B7978706A66666A7A737273
736563635C5B5751514B44414340403E3E3B3C3838373A36343A353733
2E3535353131303632343132353331343433353833363737393A393C40474A4C56555B60
62717F7F7D79817C7A7C8689929F99929A9693929190918881847B746C68706F767B706C
6D6462625D5E5451504B464642403E3C3C3A3939363839323537353A33
323132323A30372F3631363334353633333439383637383738413F3E434A4B54565E6363
69797E7B78787F7F798C9A9C97A39B9DA1999597999D9898928D7D7A6A6C686B75716F6A
696163635E5854524E4B4943403B3E3D433F3B3C3B39373D3934343735
3136303735323236343134323232363635353731363A3A38393C3F4147484D50555E6071
777C7970787C8081839CADA69EA09F978F97989EA0A4A7AE9D938E7C71676E696D6C6767
655F5E5E5D59554E484A474341433E3B3C3B3839363837363A34303136
3531333134353433353434303338373138363731363A393A3E423F4441494D535D626777
7774777A77797F828EA2B4A89E9D9D9291979A9EADB9BCC6B1A29383756F6C6864686566
615E62595C58514F4C4B4641423F3F3C3D363935393935373A3A383435
34343332333131343634313532343637372F333937393A3A3A3C42464A4B4E535C5D696E
747978787D7E818D97A1A6A3A1A49F97989CA1A9B7CBDEEBDCB19C9082786C6869666863
635B5F59565450504D4A444241403E39403B3A39393938373736373433
3135363032333238373433353534343B3539373435393B3F394440424C4E575D595E646D
73747980827C8495A2ABACA9A19F9D9DA3A5A9AFBFD7ECFFFFBDA0928778756A6162665E
61665E5E54594F4C4A47413D3D3F3B3C3A383D36373637383837333236
32363032353034323A343436323636323536373A383939393B3F4345494C58545B5E6B71
7C7B818880808995A6B0B4AC9F9E9BA6AAAEB4AFB6CFDEF1D8B8A18D878C7E6B62645C60
646D6F6D6152524E49454442403D3B3B3C3D3636373934323734363135
3437343233343135363236353437343338363638343A3C393F3A454C4952505861686778
7983888883849096A7B6B2AC9E99A3A7ADBDBCB5B6BDC9CFBEACA199938C807164626263
5D6968725D55514C4A46423D3C3C3D403C3A3B39363A34353835353234
313732323231343134333330353833363933383639393B3C3C3E4B4B4F5155575E686C72
7D8386878A878D9CA6B6B1A49A99A7B0BBC8C0C0BDBFC6C9B5A19E9F9C8F8E7665686663
6365665D5752504847464240403C3E3D3F39393A383230343334343235
343333323434363333343734363439343A373B3A38383D3D41424B4E5254525C5C646A75
83868B8B868B919FADBCACA09BA3ACBEBFC2BFB2B8B4B9C1B6A69F9A9894887066686968
67615D59554D4E49484645403E3C3F3C3B3D3839393536373632313334
3732393334313433332F3536373737323537393A3B3B3B3B3C404A4A5053575A5D607278
86878F8A8B8A99A5ADB6AC9FA5B5BCB5BCBBB6ACA7A6B2B7B2B2A297968F846D66686665
675D5D59524F4D494643423D423D3D3F3E3F3737383338363339333232
3534343137323534343438343234353939373D3D38364141444345484D55555961697182
8B8D93938F9099A1A3AFA19EB1B7BBBAB4BAADA9ABA3AAADAEADA699938C807068626868
65635F5A524F4A4A4645413F3A3B3C393E40373836363635322F333534
32363333363230323536383336363A3536373B383B3E3E3D4345494A4C52575A65697381
8D938F919293A2A4ABA9A1ACC0C8C5BDBEB7B5AEAFADA7AEAFB2ACA29D937F7170686869
646960524D4B4747454340413E3C3B3E39383936353336383333323331
3933373933343637392F333235363337373A373A3D3C403A4349464D50545B5F636E7885
8F8E8E949BA7AAB2B0ACA6B7C4C8C0CAC2BCC0B8BBB5B0B1BFBBB4A39F9586716569696B
6E6559514D4A474444463E3D3D3D39373D393C37343537373034383632
343333373134323633302F393334373835343A3A3B3E414041454A4C5451585E696F7E88
9893909AA1ADACB5B0B1B6BBC7C7C4C5CFC0C3BEBABAB7B0B6BBAFA49F928B726A726D6E
6B685C4E4B4C49433F3D41393C3A3F3E3D3A3735313639323435372D35
3532363738353634343734323234343636373C393C3E434341444B4C4F5456666D73808D
989AA0A4A9ADAEB0B0B3C5C9CCCDD0D2CFC4C4BBBBBCB8B3B1B1AEAAA2927E7173757378
727367564B484544403D403C403A3E3A3D3A3534343235343832353135
32343738343238343533343137333737353837393B3D404447454C4A4E535A5D7277808C
989FA5AAA9B9ACADB4B9C2D1D6E0DCDBD1C7C5BBB2BEC0AEACA4ADA59B8E7872737D7C80
80796C554F47484041403E3A3C3C3F3D32363734353839363438323333
31393336383A363934313233353837333737373E3B3D4343454C4951504E6065747E8A93
9CA2A8A9AFB2AEB5C2C3CFD3E1E3E5DCD6CBBDB8AFBBC5B6B2A5A59D91827A7180878688
837D69584E4B484340423B413E393C3737353B35363735323434333636
3534363A3836343432353837393337373638393E414040454D4C4C4C5359616C787D9096
9E9FB3B4B4B7BABEC6D0D5E2EAF5F2EBD7C8C9BDB0B2C5BFBEAAA29E8D81796D78838D8A
827362544D4D47443F3C3B3C3C3938383A333830393537333333383433
3436343238353336343332343236383B373C383B3F3A3E48494F4E515463737A7D848E97
A5A7B1B8C0CAC1C5CADDE4EEFAFFFEFAE7DBCEC8BAB4C1BEBEB0ACA997857A6E7179877F
736759504A4947423D403B393A3A363A383A3235363736363230333037
35343137363538353833353433343B39373639403C3F45464F4E4F555C698F8F8383929F
A4B5B8BEC8CACAC7D6E0FAFFFFFFFFFFF8E1CCC6C0B8C0C0B7ADADA798847B7073727370
6A60564D47464341413B3E3B3D3B393533333634353436323434313333
3432372F3536353936373335343A3537393B413C413F464B4F5556535C6B85928B8691A5
AEB5C0C4D4CDD1D3DCECFFFFFFFFFFFFFBDAC9C7CDBFBAB1A8A6ABA699877D726F6D6E63
645F574949454343403A3C39353A343833353330373835393234303532
3036383834323535373839383434363B393B3D404243464E5454565A5568788A89868B9E
AFB8BEC6D2D7D9DEF2F1FFFFFFFFFFFFFFEED7C7C9BDAFB2ABA7A3A192857670766F6E66
5C565045464248433F3C3C3E3938383534383233353636333736333434
3236343637343537363A373A373B37313C3B3B3E4144494F5B5C585861656D7A7F818F93
ADBFC4D1DDE7E3E2F1FFFFFFFFFFFFFFFFF0D9D1C4BDA7ABAEA4A49D998876716F6D6960
5A584949494042423C3F403A393A373734383133363732363734333234
3033323736353535363936393B383A3A3C3E413C404643535459575B5B67657774848D9F
B3CACACCE2E5DEE4E8FFFFFFFFFFFFFFFFE3DBD9CCC1A8A1A1A19FA1938679696D6C6662
55544D4B43414444403E363A3D34333738313633353634323332342F34
3335313635373533323836383435383B3C3B403E42424F50565B57525B6269707782929E
ADC5CADDECEADBDFE8FDFFFFFFFFFFFFF9DDDDD7CCB5A39894939B99938171636B706D5D
594F4B47464643423E3939383535373336373933373432333230343337
313133353935353437393735393738383C3E414244454B505B595E5C5C5E676F768A99A5
ACC0D8E8F1DFD2E0EBFAFFFFFFFFFFFFE0D8D7C7BDA89C9B989093918D7F70676F65645D
544D4C4445423F3F403B3C383B3936363934363539343434312D323335
353932353133363A3634333639393A393E3F414448464E565A66625F5F5F6B6F8490A2A7
ACCCDEE6EBE3E0E5EDF7FFFFFFFFFFF3DBD3CFC1B5A0939E9C938789887E706F716B5F58
55504943473E423B3E3A38363737393739373439353234353135343132
343635333A33383538323536383A36383F3C4045494D52585F5F5A5F5D6068768096A8A5
AECAE2E8E7DFDAE8DDE8F7FFFFFBF2E3CECAD6CCB6A6A2A9A7938F88827671716E606054
4F4D484441413D403B3B373C3A3A343733373834353634323433333033
353536343732393435363935383A3A3B3B423F444D4F545D5C5B59545C636C70849DB3B0
ADC0D7DEEAE1CDD5D2DDE7E8E4E7E1D7C9C8DACDB2ABACB7B09E8F887D73717774686156
544E4B484640403B3D383B3B38373E3B3736362E353932383735313537
353334373732343835393337393C39373A3D45454C54585B5D5E56575B61697385A5BFAF
ABC4D3DFE2E3D4D4CFD8D7D7D2D0CFCAC6D7D8C8ACA39EB2B29C8F867C7A74756A655A55
504B464643403E413D3A38393633373938363335353238343735343635
362E3432313235363335363538403A3C3F42455055585D5C595B575859656A79869CBCB1
ADB8CED8DEDBDCD1C9CED0C7C3C1BBC1CAD4D9C1A494A2B9B19895897B7C76786D645B58
4D47443F3F3E3E3B3D393B3A3B37393038353437333531313838353433
3232383935343737343436363739383F3C424752585B5A575A5B565B5969697687A0B0AA
B3BEC2CDD6DBD3C6C3C5C5B5B7B6BFBFC1CBC1AFA099ACB5A9988F8C7E7B7E716B605652
4C4742463F41403C3B3938373737383239343130353637353338343432
323234333437383534363138383A3F3E42414850595957565951595C5A666C77839AA4A9
ADB5B7C4CDD0C8BFBBBAAFB2B3B6BBBEBDBEBAA7A0A0ADAA9E91958B7F7E7D736D5C5750
4944424244423E403B383D3439383A33363B3536303735363533373734
3534333135363A3837343535393C3D3F463F464B585F55565254545B6164697187939AA7
B0ABB1B8C5C5CDC6B3B4B5AFB0B2B9BFC2C6B49D9FA8AFA3998B91877C817F6E685B5751
484844404143423D3D3D3B3739353336333434333630342E3437313434
31353438373438393535363537373F423F44484A515754585456565A5C656872848FA0A5
A5ACB3BAC1C7C5C7BDBABEB9BDB8B8B2B5B8AE9E96A2A7A2968F8D857C797570615C574C
4B4442463D3C3C3D3C3639373836393A36343036303233313330343330
352F3233333433363A34383734383C4243474C4B4B534D5154545A5957686874848BA4A5
A2A8B8BCBEBBBDC3BEC3C1BEB8B7AFB9BDB7A89A959E9D9B999B8D7E7976756A5F59504B
454547413D3E39403D393836373433343531343A343535353031333132
32343534323434363732353438393E394143494E484F4E5651545558585E6B708193A3B1
ACB2BBBBB8B3B3B3B7B9C3BBB7B5B8C0BCB4AA9B98A1999489917C7C79776C635D55504B
484644423E403A3C39373C3737393535372F3332323332303130332B31
333733333635363A3E3A3138373E3A40454845474A4B504B5550565D5C606B738393A1B7
B3B7BCB9AEABB0B3B0B1BAB3B5C1C6C0BAAEA697A1A49D8D8A8274797370695D58534B47
424A433D413E443A383D363B38363536343731352D333432302F343331
32323435353137383A383A363539393F404349494A4C4D4E5451535A5E5F6B737C8FA0B5
BFC2BBB5B3B1B9BCB9B1ADABB2C1CEBFAF9E9B8FA1A19D8B86796E73706E655D574E4B45
474341413C3C3C3D3D3739343A363235313234322E31343431342F3332
3733333330373637393C3B3B413C3E41434648474A4D4E4E52555656605E68717A8F9EB3
C8C9BFB6B6B3B6B4B1B4ABA8BACCCBB1A5929199A4A5938881756E6B7266645A574F4E48
4542404142393A3A37383939383933333332342E333237353533323333
33353436333936333B3C3A3A3C3F3D424346494A474D4F4F56545458595D656C7D9199AF
C4CFCBC5BBBBBBB9B8B3AAB2C3CCBDA5979B98A4A8988E857E76756A716D635E544E4C4B
42443F443E3B3F3C3936393835343533323637342F3230343632323030
33353634333635313937383B3D3F43434845484C4B4E4F4E4F5156595B5D636C7D8F9DB3
C3CDD3CBBCB8C0C0B6ABACACBDC3B0A192979ECEE8A7857F807873696967665C56504A47
45433F3C3B3A3637373B3630373433343734383033342D353832353432
323636383438333B3739383F3E3D4143484B4A494B4C4A50525454565B5E616E808BA1B0
C4D0D2CEC9C5C1BFB7AFAEABB6AE9F908C8F9AE5FFAB84837D7C6C686463605A5A4E4949
42403F3F373939383B35353231323631343837393435323337332E3231
31383238333734343836393D4045434B4B4B484A4A4C524D57515153595D646F7C8A97A4
B9C7D3D5DCE9D7C8B7AFB0B2AFA1928A8C8C8DA6AF907F7A7A7A716667626055534D4943
47413D393E3A373A3433363437323432343736312F35312F31332E2F31
333635323639383B393D383C4242444648454A474D4C515253505456605E6B6B7B84949D
ADBAC4D5F2FFFCCDB5ADB1AEA99B91838A89848D8D847D7A7678716A63625C55514F4944
423C3C393B3736373A3531363536323434363732353633313134323230
3436383636383A3337373B3E3E414A484C4E4A4D4E4B504F5157595D5A5D626D7D818794
A1A7B5C6F5FFFFCEB0B0AEA79C95898680828785837E74777775716F63625A53514D5048
42423B3D3C383936373639363338343733353336303130343236323332
343237363838373A363B3A3B3B46474B4F4F444D5153505B595B565E5960636D797E868D
94A3A7BDDBF8E5BFBCB2A598938C817E7D7879797877786E71726C6F60605B544B494A43
433C423D3B3C363837313335383434353A33323430323633302F332F34
333436383536373738383D3D3E404650514E4E5252545659595A5A5A5D5B646770768B99
92959DA8C9DBCCC8BAAFA9928884817E7A79787F7F7573716E686A68635D55544E484842
423D3E3B38363E353536373434363330343534332F33332F3333323437
33323335353434393C3C393D3C41474C504C5351565858545B5D5C5E5C5D6464686B7F8D
918E929DACADAFADB0B4AF9C8E807B7F797B7C82817972736F6864625D5853534F474240
423E413E383335343638363333393038323531332F3336353235313333
33393534343838373B3A3B3C413F474F585852515154555B5C615D5D5D5F5D6566677078
808490939492959EA2A4A798888783827C7B7E7C7B76716C6A64625F5B5C5A564D4E4842
453E3E3A363536343632333434323433303B3331333231323233313231
3836333835353738383F373E3D414449535252555253545A555C58635E5F5E6063666F74
717B8A8D8684898F959C958887887E7B7A7E787B767269696D6458605D585E625B4E4747
42433B393839383632363532323335313131332F302E32313031313430
3C3C3F3A36383B393A3F39383E47454E4C514E575756535654595759595E5C5E62666669
70737E837E7B7B848C928B858681797978747877767067645E5B5E5C54605E5E5A4C4D45
41383B343B363237353436323332303435363430363030343332333433
373D433B343B38393E3A3B3B404243494E4E50555551545754565B575D5E5E595D616366
6770756E716D6F7B8F8B7E81808378717A7873736C67676262595B56595F5B5757534B43
40433E373A3A34373430343235373338363330332E3133323234333234
39383A3735363C3B3C3C423E414447434A4F555055565556565458585959585B5D5B5861
63656E6869696F727879727D7C7D73747471706C6863635F5D57595A595B58514D4C4A47
3C3A383A393138343136373835343036363232302E3D2F2F3532343233
353538353737383E403D403D4041444449494F5252505959535757585757595D5A585A5E
626563676469686A71737774746A71726B6E6865605F60625E5A565957514D4E474A4A40
3F433C39363A36353637342E373532322F302E3131333231312E333635
35343A3934373439363D3C3F444143454C4C4B525350545052525A5857595A5B58565E5F
5C5E61646261656C707375716A64696F6A6865615C5D5C5A5956565156534F4C4D464442
3D40323436353337313139353336313038332E322D3233323231333030
34323639363A37383C3E43403943464849504E535051535253545859575962645E5A5C5D
5B5D626160626567736E727066636A6868645E5E5D5F5958575B5852504E494B46434140
3E3815372C373137333C36343333333532342D30342F2F2F3137343633
32333637343A39383B3C3C40404444494E4B4E4D51505351525453535456635E5E5A595D
595F5F655F62686A6C6B6C696066646A69635D5C5D595253525352534E4846434948423D
3D3A3B392F36363533343336313230332D323434352E30323031363031
373435363338373E3A3A3F4143424649454B494B4C515154525659585B5B5B5B5C595857
575C5E5C60626869666467635E62696B6E66635B5D5855534D4F50514B474342443C3F3B
36333C3237303930352E343232302E3236353432313230313230323532
343A3537363A37383D3D3E3F3F4141424648474D4F4E5658565055535654585E575C5C59
5B5C595C5D6164655E5D635E636165686562625B5C51574E4F4E49474A4845433F3E383A
373438363933353633363530333332302F2F3332362F352F3431333332
3536373535343B3B433D3D4443454545444A4A4A515159595D54565354545B5A5759585A
565E5D605C636260635D6363676363646461605857554F50514F504B494141403F3E3F41
39343E3E36363434373334343132313033313033303633343132333631
3439343837333C3B4141414341464443494D4B515354585E5F5E595858565C59595D585B
5A5B5C615E666062605D6563635F64605D605D5B51544D514E4C4C4B45463E41393A3C38
393539363032343332383734343537323332362E323432332E31353032
3633393637393F3E414040434444494C44494A4B5357585B5E5D5E5B5E595B5D5D5C5B5B
575C5C5F635E5F5C5B576164636060625D575A5A504E4D4D4E4D4B454641433D433F3D39
383B393038393931353636353331373032333234312D3034312F2F3031
343136353C3C3E41393F46434549464A4A4B504F4F56565A5F605B58595A5D63595F5B66
5B5962605E5E5B595D5F6065625A615D545456564E514C4B4B4D474341403F3F3740383D
3C373A3C31373736373431303432313430343535313632333233323231
36383837383D3B3E403E4245484A4E514A4B4A535255545D58615F5B5C595A5C58545759
59605D6464605D5C5D5C5F625D60585250535555534F49484A4A43463F3F40393C3B393C
3132383C473935353535333431343634333133362F3033313530313030
37363C3F3C3B393B40404751504D4F5251505052535556545B5D5D5B575D54595850575A
5E5A6161615B5B60595C585F5E5B555050545550504A4F4C4C49474641383C3C3D39443A
353A383D32383334393437363235373731303232313332322F3135322E
383942463D393C3E41464C525157555054514C4F50545553555957585B57545B5352595C
5F5F5E5E5A5D55545D5C575F5E5B5251545055514D4F4A4A4B48433C42413D3836363739
3238393534313130343237313333363433333437332F2D2F3532333331
34393B3B3B3F3B3D4244525859555455535555545354595455565E585854575B58595D57
5B5D5D595950565C585E5D5E56595250524D51504E464A4B4743403D3D373D393B383B35
383836323335343232382F343333333732353434333431333238313232
3737393A3B3C3C40424D525E5B555353545055535756585657585D5D5C55565856595854
58575B5658595D5D545A5F5C57564F4E4F4E4C50514D4A4A4744443D413B3A3A3C393739
3732332D3A36303034352F36322F303131343431343232323532302F30
3434383C3B3B3B3E4445555E5C5A5756585356525457534D5862685E585654575354535A
505853555455585B5B5D5D5656534E4E4F52504C4B4F4B46464240413B393A3C3A393833
3734363537323331333030332F333032312F312D33363236332F34342F
3637383B393E4041454950575C5B555756565554554B504F5C60695D57515451534F5654
5557545456575A59575C585256524E5152524F4F50494947473F3C3E3B3C3A373A383638
343535353133332F2F2F34353230342E2E2D2F332F34312F3034323132
383A3935353A3F433F454B555A565A555352535252514C555C61615C544E4D5252565352
51505452585A5A5A5C56565252515155505451514F4A494A44413F3D3A3A3E34393C3135
323435352F3232352F352F313131302E353630332D3233323234353132
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[5]:=",
  ImageSize->{397, 397},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006=0001SA000`40O003h00O0?m_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgX008e[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX00Fmk003oJeZ>JeX0003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J005[NP00of]JSV]J0000ofmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00SF]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FP01Kg/00?m[FXi[FP0001a[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP800011Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_N`0400000f]JJgY[FP020000@6]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
0`0004=[FV]j000006]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FP03000016]JKg][FV]j0P0003m[FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX00P0000A[NV]JKg][FP80
000kKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NP05000026mkJeX00000Kg][FV]jJeX2000016]jJeY_Nf]J0F]j000KJeX01000JeY[FP00@6]J00<0
06]JJeX00f]J00@006]JJeX0041[FP04001[FV]J0002JeX00`00JeY[FP0lJeX01000JeY[FP000V]J
00@006]JJeX003e[FP04001[FV]J0002JeX01000JeY[FP00?6]J00D006]JJeY[FP00009[FP04001[
FV]J0002JeX00`00JeY[FP02JeX0002VKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]j001_Nf]J001[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_N`00JgY[FP00JeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX50000
hF]J001[FV]j001_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]J001[FVmk001[NV]J001[FV]j001_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_N`00JgY[FP00JeY[NP00Kg][FP00JeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j001_
Nf]JJgX006mkJeX006]JKg/006]jJeX006]JJgY[FP01Kg/001][FP04001[FV]J0012JeX01@00JeY[
FV]J00000V]J00<006]JJeX0>f]J00@006]JJeX0009[FP04001[FV]J000nJeX30003JeX01000JeY[
FP00?V]J0P000f]J00@006]JJeX003a[FP05001[FV]JJeX00002JeX01000JeY[FP000V]J00<006]J
JeX00V]J0000of]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_N`00JgY[FP00JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeX006]JJgX006mkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]j001_Nf]J001[FVmk
001[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg/006]jJeY_Nf]J001[FVmk001[NV]JKg][FV]jJeY_N`00SF]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00Kg][FP00JeY_N`00JgY[FP00JeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg/0
06]jJeX006]JJgX006mkJeX006]JKg][FP01JgX001a[FP80049[FP8000A[FP80041[FP8000=[FP80
041[FP<000=[FP8003m[FP8000A[FP8003a[FP8000A[FP8000A[FP8000E[FP000?m_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgX008e[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX00Fmk003oJeZ>JeX0003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J005[NP00of]JSV]J0000ofmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00SF]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FP01Kg/00?m[FXi[FP000?m[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/008e[FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX00F]j000FJe[o001g0001JeX0003o
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`00
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_N`00JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j001_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j001_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]j001_Nf]JJgY[FVmkJeY[NV]JKg][FV]j002=JeY_Nf]JJgY[FP00JeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgX006mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgX006mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/006]jJeY_
Nf]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00<006]JJeX0A6]J00<006]JJeX0A6]J
00<006]JJeX0AF]J00<006]JJeX0A6]J00<006]JJeX0A6]J00<006]JJeX01f]J0@000F]J0000of]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_N`00SF]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J0?m[FW=[FP40005[FP0000i_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]J0P000?m_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX007e[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NP00005_N`003F]J00@006]JJeX000E[FP03001[FV]J0?m[FW=[FP40005[FP0001I[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NP00Kg][FP00JeY_Nf]JJgY[FP@0003oKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]j001`JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX30001
JgX000e[FP04001[FV]J0005JeX00`00JeY[FP04JeX45:D00aS65<DDa@045:D26<H00aC56<HDY@02
5:D46<H:5:D46<H45:D36<H01aC56<HDaAS65<DHaQC500@DY@<HaP<Da@056<HDYABU5:DHaP035<D5
6<H35<D00aS65<DHaP046<H00aC56<HDa@025<D56<H45:D01aC56<HDaAS65<DHaQC500<DY@8HaP03
5<DHaQS6008HaP036>HHaQSV00@HaP055<DHaQS66<HDa@036<H35:D=6<H00aC57>LLi`037>L00aSV
7>LHiP047>L01Qd77>LM1ad78@PM1`@Q20038BPQ224X00@Q20<U:@PYBP03;F/YJRe[00<]R`06<H`]
Rbf;;H/YBRE91bU:00TUBBU:9DTYBRE9:DXYJRe[:FX04bU:0RE922U:22DY00<Q:2DY9BT00bDY0b4X
224800<Q:2488BP00R4800<M1b488@P00b4800<M1b487@L00b4811cW00<HiQcW6>H01AcW0aSV3QcW
0QSV00<LiaS66<H00QS600@HiQcW6>HLi`<HaP057>LHiQSV6>HLi`026>H011S66>HHaQSV1aS60QC5
00DHaQC55<DDaAS600DDY@035<DDYF]J00E[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_N`00JgY[FP00JeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00DDY@036<HDYABU008DY@054:@HaQC5
6<HDa@035:D02QC56<HDaAS65:DHaQBU5<DDYAS611BU0QS600HDaAS64:@DYA2T5:D46<H02AC56<HD
aAS65:DHaQBU6<HDY@025<D01AS65<DHaQBU6<H01ABU00<HaQBU6<H00QS601HDaAS65:DHaQBU6<HD
aAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaP8Da@0:5:DHaQBU6<HDaAS65<DHaQC56<H35:D0
5AS65<DHaQC57>LHaQSV6<HLiaS67>LDaAS65<DHaQC56<HDaAS65<DHaP025<D01ABU6<HHaQS65<D0
2AS600TDaAS66>HLiaSV7>LHiQcW6>H011cW00HQ21cW8@PLib487@L68@P01RDY8@PU:B488BPQ20<U
:@039DTYBRE900DYBP0=:FX]JbUZ<H`]Jc6<;F/aS2e[<H`UBBU:9DT00bU:00`UBBU:9DTYBRE9:DXU
BBU:9DT]JbUZ;F/5:DX01Be[:DXYJRU:;F/012U:00LUBBU:9DTYBRDY:DXU:@03:DX01bE9:DXUBBU:
9DTU:B4X008U:@08:DXU:BE98BPU:B4X9BTQ:0<U:@058@PU:B488@PM1`068@P03RDY8@PQ21d78@PM
1b487@LQ21d78@PM1b487@L28@P00ad77>LQ20027>L02ASV7>LLiad77>LQ21S67>LHaP037>L011SV
8@PLiad71AcW00DQ21cW7@LHaQcW008HaP065<DHaQS67>LHiQcW0QSV00`HaQcW6<HLiaS67>LHaQcW
6<HLiaS67>L26<H00aC56<HDa@036<H01QBU6<HDYAS65:DHaPDDY@0:6<HDYAS6JeY[NV]JKg][FV]j
0001Kg/000i[FP8000I[FP03001[FV]J00A[FP@DY@8Da@036<HDYABU008DY@045<DHaQS66<H35:D4
6<H:5:D46<H45:D36<H011C56<HDaAS60aC511BU0aS600@DaAS65<DDa@<DY@8Da@036<HDaAS600<H
aP8Da@046<HDaAS65<D56<H011C56<HDaAS60QC511S611BU00LDaAS65<DHaQC56<HDa@035:D00aC5
6<HHaP046<H00aSV6<HHiP026<H01AC56<HHaQS65<D00aS600<DaAS65:D00QBU2aS600<DaAS66<H0
1AcW00<HiQcW6>H011cW00HM1acW7@LM1b487@L48@P00b4X8@PQ:0048@P39BT8:DX01Be[:FX]Jbf;
<H`00bf;00<aS2f;:DX00RU:00DUBBU::DXYBRE900<YBP079DTYBRE9:DXYJRe[:FX04RU:0RE92BU:
22DY00<Q:2DY9BT00bDY00<Q:2DY8BP0224800@Q:2488BPM1`LQ20037@LQ21d700<Q20@Li`036>HL
iaSV00@Li`8HiP037>LHiQcW00hLi`8HiP@HaP046>HLiaSV7>L36<H26>H02AcW6>HHiQSV7>LHaQSV
6<HHiP086<H35<D00aS65<DDa@055:D00aC55:E[FP05JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQ2T5:D@Y08DY@0=6<HDYAS6
5:DHaQBU6<HDYAS65:DHaQBU6<H011BU00LDaAS65<DDYA245:D@Q0085:D01AS65:DHaQBU6<H00QBU
00T@Y1BU5:DHaQBU6<HDYAS65:D01AS600XDaAS65<DHaQC56<HDYAS65:DHaP@DY@095<DHaQC56<HD
aAS65<DHaQC500<HaP0=5<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<D00aS600/DaAS65<DHaQBU6<HD
YAS65:DHaQBU00<HaP075<DHaQC56<HDaAS65<D00aS600DDaAS65<DHaQC500@HaP087>LHaQcW5<DH
aQC56<HDa@LHaP0M5<DHaQC56<HDaAS66>HLiaSV8@PLiad77>LU:B488BPQ22DY8@PU:B4X9BTQ:2U:
9DTYBRE9;F/YBP03;F/01BUZ;F/YBRe[:DX012e[00<aS2e[<H`012e[12U:00XUBBU::DX]JbUZ;F/Y
JRe[:DX]J`8YBP059DTYBRE9:DXUB@03:DX012E9;F/YBRe[0RU:00TUBBU:9DTYBRE9:DXU:BU:9BT0
1BU:00HU:BU:9BTYBRE9:DX29BT04B4X9BTQ22DY8@PU:B4X9BTQ:2DY8BPU:B4X9BTQ:2DY7@L00b48
00XM1b487@LQ21d78@PM1acW6>HLi`@Q200E6>HLiaSV7>LHiQcW6>HQ21cW8@PHiQcW6>HLiaS67>LH
aQcW6>HLiaSV00DLi`0N6>HLiacW7@LLib486<HLiaS67>LHiQcW6>HLiaSV7>LHiQcW6>HLiaS67>LH
aQcW6<HLiaS66>HHaQcW11S601TDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQC56<I[FVmk
JeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP035:D@Y1BU00DDY@065<DDYAC55:DDaABU0QC5
00<HaQC55:D00aBU00<HaQC56<H00a2T21BU00DDaABU5<DDYAS6008Da@DDY@065<DDYAC55:DDaABU
0aS600DDaAS66<HHaQC5008HaP035<DDYAC500DDY@036<HDaAS6008HaP055<DHaQC56<HDa@056<H0
0aC56<HDa@025<D011S65<DHaQC521S600DDaABU5<DDaAS6008DaADHaP067>LHiQSV6>HHaQC541S6
1acW0b4800@U:B4X8BPQ:0<U:@039DTYBRE9008YBP03:FXYBRe[00<]J`8YJP03;F/YJRe[008]J`<]
R`04<H`]Jbe[;F/7:DX022UZ;F/YJRe[:DXYJRU::FX;:DX02bUZ:DXYJRE9:DXUBBU:9DTYBRE9:DX0
0RE912U:00@U:BE99BTUB@<YBP@U:@068BPQ224X8BPU:B4X1BDY00<Q:2DY9BT01b4800@M1b487@LQ
20<Li`@Q20037>LHiQcW00DLi`047@LLiacW6>H27>L011SV6<HHiQS63QcW00<HiQS66>H011cW00DH
iQcW7>LLiaSV00<Li`<HiP037>LHiQSV00XHaP055<DHaQS66<HDa@036<H35<D46<H6JeX10001JeX0
000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_
Na2T5:D@Y08DY@0=5<DDYAS65:DHaQBU6<HDYAS65:DHaQBU6<H011BU00LDaAS65<DDYA245:D@Q008
5:D01AS65:DHaQBU6<H011BU00L@Y1S65:DHaQBU6<HDY@056<H02QC56<HDaAS65<DHaQBU6<HDYAS6
11BU00/DaAS65<DHaQC56<HDaAS65<DHaQC500<HaP0;5<DHaQC56<HDYAS65:DHaQC56<HDa@036<H0
2aC56<HDaAS65:DHaQBU6<HDYAS65:D00aS600TDaAS65<DHaQC56<HDaAS65<D00aS600DDaAS65<DH
aQC5008HaP067>LHaQcW5<DHaQC50aS600DDaAS66<HHaQC500<HaP0M5<DHaQC56<HHiQcW6>HM1acW
8@PLib4X8@PU:B489BTQ22DY8BPU:B4X:DXUBBU:9DT]JbU:;F/YJP03;F/00bU:;F/YBP04;F/00c6<
;F/aS002;F/00bUZ;F/YBP05:DX022E9;F/YJRe[:FX]JbU:;F/4:DX01BE9:DXUBBU:9DT00RU:00<]
JbU:;F/00RU:00TUBBU:9DTYBRE9:DXU:BU:9BT01BU:00<U:BU:9BT00bU:018Q:2DY9BTU:B489BTQ
22DY8BPU:B4X9BTQ:2DY8BPU:B4X9BT28@P02Ad78@PM1b487@LQ21d78@PM1`037>L48@P05ASV7>LH
iQcW6>HLiaSV8@PLib486>HLiaSV7>LHaQcW6<HLiaSV7>LHiP087>L06b487>LM1aS67>LHaQcW6>HL
iaSV7>LHiQcW6>HLiaSV7>LHaQcW6<HLiaS67>LHaQcW6<HHiP046<H06AC56<HDaAS65<DHaQC56<HD
aAS65<DHaQBU6<HDYAS65<DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0aBU00<@
Y1BU5:D00QBU00HDaABU5<DDYAC55:D45<D45:D01AS65<DHaQ2T5:D00Q2T1aBU00@DaABU5<DDY@8D
a@036<HDYABU00<DY@065<DDYAC55:DDaABU1AS600@DaAS66<HHaP8Da@035:DDaABU00@DY@<HaP07
5<DHaQS66<HDaAS65<D01AS600@DaAS65<DHaP<Da@036<HDaAS600LHaP035<DDYAC500<Da@THaP03
5<DHaQS600XHaP8HiP037>LHiQS6014HaPLLi`<Q208Q:0039BTQ:2DY008U:@039DTYBRE9008YBP03
:FXYBRe[00@]J`<YJP<]J`04;H/aS2f;;H/3;F/7:DX022UZ;F/YJRe[:DXYJRU::FX;:DX02BUZ:DXY
JRE9:DXUBBU:9DTYBP039DT5:DX012DY9DTU:BE90bU:12DY00HQ:2488BPQ:2DY8BP79BT00b4X8@PQ
20058@P011d78@PM1b480acW124800<LiaSV7>L01AcW00<M1acW7>L00QcW0QSV00<HaQSV6<H03QcW
00<HiQS66>H01QcW00@HiQcW7>LLi`<HiP057>LHiQSV6>HLi`086<H00aC56<HHaP056<H011C56<HD
aAS60QC50aS61V]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]J001[FV]jJeY_Nf]JJgX@Y0@DY@0Z6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DH
aQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU5<DDYAS65:DDaABU6<HDYAS65<DHaQC511BU00hHaQBU
6<HDYAS65:DHaQC56<HDaAS65<DHaQC50aS601PDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DH
aQBU6<HDYAS65:DHaQBU6<H45:D02A2T6<HDaAS65<DHaQC56<HDa@036<H03AC56<HDaAS65<DHaQBU
6<HDYAS65:DDaABU00<HaP0;5<DHaQC56<HDaAS65:DHaQBU6<HDY@026<H011cW6<HLiaSV0acW00<H
iQcW6<H011S600<LiaS67>L00acW00@Q21cW8@PLi`<Q20078BPU:B4X9BTQ:2DY8BP012U:01H]Rbe[
<H`]Jc6<;F/aS2e[;H/]Jc6<:FX]JbUZ;F/YJRe[:FXYBRE9:DXUB@<YBP059DTYBRU::DXUB@02:DX0
0be[:FX]J`02:FX01RU:;F/YJRe[:FX]J`8YBP0>9DTYBRE9:DXUBBe[:DX]JbE9:DXUBBU:9DTYBP8U
B@099BTYBRDY:DXUBBU:9DTU:B4X00<U:@058BPU:B4X9BTQ:0048@P022DY8@PU:B4X9BTQ:2DY7@L3
8@P02Ad78@PM1b487@LQ21d77>LHiP027>L01Ad77>LQ21cW8@P00QcW00THiQcW6>HQ21d78@PHiQcW
6>H00QcW00PQ21cW7@LHiQcW6>HLiaSV0acW014HiQcW7>LQ21cW8@PHiQcW6>HLiaS67>LHaQcW6>HL
iaSV00<Li`0:6<HLiaS67>LDaAS65<DLiaS67>L:6<H02AC56<HDaAS65<DHaQC56<HDa@036<H01f]J
Kg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J1aBU00hDaAS65<DDaABU5<DDYAC55:DDaABU
6<HDaAS60aC500PHaQC56<HDaAS65<DHaQC52aBU0QS600<DaAS65:D00QBU0QC500@HaQC55<DDY@8D
a@lHaP045<DHaQC56<H25<D01aBU5<DDYAC55:DDaABU008Da@056<HDaAC55<DHaP045:D00aC56<HH
aP096<H00aC56<HHaP026<H25<D00aS65<DDY@025:D46<H011C56<HDaAS60aC500LHaQC55<DDaASV
6<HHiP047>L00aSV7>LHiP046<H011SV6<HHiQS611cW00<M1acW7@L00b4800<U:B4X9BT00RDY00<Q
:2DY:DX00bU:0be[0Rf;00<aS2f;;F/01Re[00@YJRe[:FX]J`hYBP04:FX]JbUZ;F/3:DX012e[:FX]
JbUZ0bU:00<UBBU::DX00RU:00<YJRU::DX012U:00<UBBU:9BT00bDY00LUBBU:9DTYBR4X9BTQ:007
9BT58@P01R4X8@PU:B4X9BTQ:0LQ20047@LQ21d78@P27>L00aSV7>LLi`027>L00ad77>LM1`047>L0
0ad78@PM1`097>L00aSV7>LLi`067>L00ad77>LM1`027>L00aSV7>LHiP026>H011cW6>HLiaSV1AcW
0aSV00LHaQC56<HHaQSV6<HHiP0:6<H00aC56<HHaP086<H6JeX10001JeX0000MKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`025:D0;A2T5:DDYAC5
5:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQBU5<DDYAS6
5:DDaABU6<HDYAC55:DHaQC56<HDa@045:D031S65:DHaQBU6<HDYAS65<DHaQC56<HDa@<HaP0J5<DH
aQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaP8DY@094:@DYABU
6<HDaAS65<DHaQC500<HaP055<DHaQC56<HDa@036<H02ABU6<HDYAS65:DHaQBU6<HDa@036<H02AC5
6<HDaAS65:DHaQBU6<HDY@026<H00acW6<HLi`027>L011SV7>LHiQcW1AS600<LiaS67>L00acW00@Q
21cW8@PLi`<Q20078BPU:B4X9BTQ:2DY8BP012U:01@aS2e[;H/]Jc6<;F/aS2e[<H`]Jbf;:FX]JbUZ
;F/YJRe[:FXYBRE90bU:00DUBBU::DXYBRE900<YBP079DT]JbUZ;F/YJRe[:DX00RUZ00@]JbUZ;F/U
B@<YBP0I9DTYBRE9;F/YBRe[9DTYBRE9:DXUBBU:9DTYBRDY9DTU:BU:9DTYBRE99BTQ:2DY8BP00bDY
00<Q:2DY8BP0124800LU:B489BTQ:2DY8BPU:@028@P02ad78@PM1b487@LQ21d78@PM1acW6>H00QcW
00DQ21cW7@LLib48008Li`096>HLiaSV8@PM1b486>HLiaSV008Li`067@LLib486>HLiaSV0acW00<H
iQcW7>L00QcW00hQ21cW8@PHiQcW6>HLiaS67>LHaQcW6>HLiaSV0acW00XHaQcW6<HLiaC56<HDaAcW
6<HLi`PHaP0E5<DHaQS66<HDaAS65<DHaQC56<HDaAS65<DHaV]JJgY[FVmkJeY[NP00005_N`005V]J
00<006]JJeX016]J1aBU0aC500dHaQBU5<DDYAC55:DDaABU6<HDaAS65<DHaP035<D01QS65<DHaQC5
6<HDa@/DY@045<DHaQS66<H35:D00aS65<DDa@025<D01ABU5<DHaQS65<D03AS600<DaAS65<D00QC5
00THaQBU5<DDYAC55:DDaABU6<H00aC500<HaQC55<D011BU0QS600<DaAS66<H01AS600<DaAS66<H0
1AS60aC50aBU11S600HDaAS65<DHaQC56<H35<D01AS65<DHiQS66>H011cW00<HiQcW6>H011S600@H
iQS66>HHaP@Li`037@LLiad700<Q20<U:@048BPU:BDY9BT4:DX3;F/0136<;H/]Rbf;1be[00@YJRe[
:FX]J`hYBP04:FX]JbUZ;F/3:DX012e[:FX]JbUZ1BU:00@UBBU::DXYJPHYBP039DTYBRDY00<U:@07
9DTYBRE9:DXQ:2DY8BP01bDY1B4800HQ:2489BTQ:2DY8BP78@P01Ad78@PM1b486>H01QcW00<M1acW
7@L011cW00<M1b487@L04QcW00<M1acW7@L00QcW00@HiQcW6>HLi`<HiP037>LHiQcW00<Li`8HiP03
7>LHiQS600<HaP036>HHaQSV00`HaP035<DHaQS600HHaPI[FP40005[FP0003E[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDYAS65:DH
aQC56<HDaAS65:DHaQBU6<HDYAS65:DDaABU6<HDYAS600HDY@0<4:@DYABU6<HDYAS65<DHaQC56<HD
YAS60QBU00`@Y1BU5:DHaQC56<HDYAS65:DHaQBU6<H55:D061S65:DHaQC56<HDaAS65<DHaQC56<HD
YAS65:DHaQC56<HDaAS65<DHaQC56<HDa@8HaP0<6>HHaQS65<DHaQC56<HDaAS65<DHaQC50QS600HL
iaS67>LDYAS65:D56<H04AC56<HHaQS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D00aS600<DaAS6
5<D00QS600HLiaS66<HDaAS65<D36<H01aSV7>LHiQcW6>HLiaSV008Li`038@PLib48008Q20037@LU
:B4X008U:@04:DXU:BE99DT2:DX02be[:FX]JbUZ;H/]Jc6<<J`e[C6/=Jd0136<00<YJRe[:FX01BU:
00<UBBU::DX01RU:00H]JbUZ;F/YJRU:9DT3:DX01RE9:DXYJRe[:FX]J`8YBP099DT]JbU:;F/YBRe[
:DX]JbE900<YBP069DTYBRE9:FXYBRe[0RU:00dUBBU:9DTYBRDY:DXQ:2DY8BPU:B4X9BTQ:0048@P0
0bDY8@PU:@058@P04bDY8@PQ:1d78@PM1b487>LQ21cW8@PLib487@LQ21d78@PLib48008Li`0?6>HL
iaSV8@PLiad76>HLiaSV7>LHiQcW6>HLiaSV00<Li`096>HLiacW7>LHiQcW6>HLiaSV00<Li`056>HL
iaS67>LHaP037>L011SV7>LHaQcW31S600DDaAS66<HHaQC500<HaP0C5<DHaQC56<HDaAS65:DHaQBU
6<HDYAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX00aS65<DHaP025<D021BU5<DH
aQS65<DHaQC56<H25<D00aBU5<DDY@045:D25<D00aS65:DDY@055:D01QS65<DDaAC56<HDa@<HaP8D
a@@DY@035<DHaQC5008Da@056<HDaAC55:DDa@045:D35<D86<H35:D86<H00aC56<HDa@076<H021C5
6<HDaAS65<DHaQC57>L36>H35:DJ6<H00aC56<HHaP0:6<H57>L01QSV7>LHiQd77>LM1`@Q20LU:@<Y
BP03:FX]JbUZ00@]J`@e[@@aS003;F/YJRe[00@YBP039DTYBRU:00LYBP04:FX]JbUZ;F/7:DX4;F/3
:DX3:FX012e[:FXYJRUZ1BU:00<UBBU::DX00bU:00LUBBU:9DTYBRE9:DXUB@059BT00b4X9BTQ2003
8@P38BP;8@P01acW7@LLiad77>LM1acW00@Q20037@LLiad700`Li`036>HLiaSV00TLi`056>HLiacW
7>LHiP037>L36>H57>L36>HE6<H011C56<HHaQS60aC500@HaQBU5<DDY@I[FP40005[FP0003E_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5<DHaQC5
6<HDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU5<DDYAS600HDY@0<4:@DYABU6<HDYAS6
5<DHaQC56<HDYAS611BU00X@Y1S65<DHaQBU6<HDYAS65:DHaPDDY@0H6<HDYAS65<DHaQC56<HDaAS6
5<DHaQBU5<DDYAS65<DHaQC56<HDaAS65<DHaQC50QS600`LiaS66<HDaAS65<DHaQC56<HDaAS65<D2
6<H01QcW6<HLiaBU5<DDY@DHaP035<DHaQC500<HaP0A5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DH
aQC56<HDa@026<H01QSV6<HHaQC56<HDa@<HaP076>HLiaSV7>LHiQcW6>H00QcW00@Q21cW8@PM1`8Q
20069BTQ:2DY9BTUBBDY0bU:00`UBBe[:FX]JbUZ<H`]Jbf;<J`e[C6/=Jd4<H`00bUZ;F/YJP05:DX0
0bE9:DXYBP06:DX012e[:FX]JbUZ0bU:00TUBBU::DXYBRUZ;F/YJRe[9DT00RU:00L]JbU:;F/YBRe[
:DX]J`02:DX022E9:DXUBBU:9DT]JbU::FX2:DX03BE9:DXUBBU:9BTYBR4X9BTQ:2DY8BPU:B4X00@Q
20039BTQ22DY00DQ200C8BPQ22DY7@LQ21d78@PLib487>LQ21cW8@PM1b487@LQ21cW8@P00QcW00dH
iQcW6>HM1acW8@PHiQcW6>HLiaSV7>LHiP037>L03ASV7>LLiacW6>HLiaSV7>LHiQcW6>HLiaSV00<L
i`056<HLiaS67>LHiP037>L00aS67>LHaP0=6<H01AC56<HHaQS65<D00aS6014DaAS65<DHaQBU6<HD
YAS65:DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00lDaABU5<DDYAC55:DDaABU
5<DDYAC55:DDaABU5<D011S600@DaAS66<HHaP8Da@LHaP8Da@066<HDaAS65<DHaQC51aBU00@DaAS6
5<DHaP@Da@035:DDaAS600<HaP075:DDaABU6<HDaAS65<D01aS600DDaAS65<DHaQC5008HaP045<DD
YAC55:D36<H01QC56<HDaAS65<DHaP<Da@056<HDaAS65<DHaP025<D00aBU5<DHaP036<H01QC56<HD
aASV6<HHiPPHaP035<DHaQC500@HaP046>HHaQSV6<H37>L76<H87>L38@P79BT2:FX00be[:FX]J`02
;F/00bUZ;F/]J`04;F/4=Jd0136<;H/]Rbf;0bU:00LUBBU:9DTUBBUZ;F/YJP0;:DX00bE9:DXYBP04
:DX4;F/3:DX012UZ;F/YJRUZ0bU:00/UBBDY9DTU:BU:9DTYBRDY9DTU:BE900DU:@049DTU:BDY8BP2
9BT48@P00b4X8@PQ:0078@P011d77>LM1acW0b480ad700PQ21cW7>LLib487@LQ21d70acW00@M1b48
7@LQ20/Li`8HiPDLi`056>HLiacW7>LHiP037>L00aSV7>LLi`037>L011SV7>LHiQSV1QcW00@HiQS6
6<HHaP<HiP@Li`LHaP@DY@8HaP055<DHaQC56<HDa@06JeX10001JeX0001RJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDYAS65:DHaQBU6<HD
YAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HD
aAS65:DHaQBU5<DDYAS65:DHaQC56<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDYAS65<DHaQC56<HD
a@<HaP0U5<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU
6<HDaAS65<DHaQBU6<HDYAS65<D00aS600HDaAS66<HLiaS67>L36<H00acW6<HHiP026<H00aC56<HD
a@046<H021cW6<HLiaSV7>LHiQS65<D26<H01QSV6<HLiaSV7>LHiP8Li`067@LLib487@LQ21d712DY
014YBRDY9DTYBRe[:DX]JbUZ;F/YJRe[:FX]JbUZ;H/]Jc6<00@e[@06;F/aS2e[<H`YBRUZ0RU:00LU
:BU:9BT]JbUZ;F/UB@03:DX00bE9:DXUB@05:DX01BE9:DXUBBUZ:DX00be[00PYJRe[:DXYJRU:;F/Y
BRe[12U:01HU:BU:9BTYBRE9:DXUBBU:9BTYBRDY9DTU:BU:9BTYBRDY:DXQ:2DY8BPU:@<Q20039BTQ
22DY00DQ20089BTQ224X7>LQ21cW8@PM1`<Q200:7>LQ21cW7@LLib487@LQ21d78@P37>L012487@LQ
21d72acW00DHaQcW6<HLiaSV00<Li`0A6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaS67>LHaP03
7>L011SV7>LHiQcW0aS600XLiaS67>LHaQcW6>HLiaC56<HDa@DHaP0A5:DDaABU6<HDaAS65<DHaQC5
6<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP0?5<DDYAC55:DDaABU5<DDYAC55:DD
aABU5<DDYAC500HHaP<Da@THaP<Da@046<HDaAS65<D75:D00aC56<HDa@025<D01AS65<DDaABU5<D0
11S600LDYAC55:DHaQC56<HDa@076<H00aC56<HDa@036<H25<D00aBU5<DDY@036<H011C56<HDaAS6
0aC500LHaQC55<DDaAS65<DHaP025<D00aBU5<DHaP036<H01QC56<HDaASV6<HHiPPHaP035<DHaQC5
00@HaP046>HHaQSV6<H37>L76<H87>L38@P79BT01Re[:FXYJRUZ;F/YJP<]J`03:FX]Jbe[008]J`@e
[@8]R`03<H`]RbU:008YBP<UB@04:DXYJRe[:FX=:DX00bE9:DXYBP02:DX4;F/3:DX3:FX03be[:DXY
BRU:9DTU:BE99BTYBRE9:DXU:BE99BTUB@059BT00bE99BTU:@029BT00b4X8@PQ20028@P00b4X8@PQ
:0078@P011d77>LM1acW0b4800@M1b487@LM1`<Li`048@PM1b487@L37>L011d78@PM1b4831cW0QSV
1QcW00LHiQcW7>LLiaSV7>LHiP057>L36>H57>L00aSV7>LLi`036<H011SV7>LHiQSV0acW1aS611BU
00LDaAS66<HHaQC56<HDa@06JeX10001JeX0000FKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeX300004f]JJgY[FVmk5:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU00<H
aP0a5<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDYAC55:DHaQBU5<DD
YAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HDaAS65<D00aS602DDaAS65<DHaQC56<HD
aAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HD
Y@036<H00aC56<HDa@026<H00acW6<HLi`036<H00aSV6<HLi`026<H00aC56<HDa@046<H01QcW6<HL
iaSV7>LHiP<HaP035<DLiaS6008HiP067>LHiQcW7>LQ21cW0Qd700<Q21d79BT00bDY014UBBDY:DXY
BRe[:DX]JbUZ;F/YJRe[:FX]JbUZ<H`]Jbf;00@e[@06;F/aS2e[<H`YBRe[0RU:00HU:BU:9BT]JbUZ
;F/2:DX01BE9:DXUBBU:9DT01BU:00DUBBU:9DT]JbU:008YJP<]J`06:DX]JbU:;F/YBRe[0RU:01PU
BBU:9BTYBRDY:DXUBBU:9DTYBRDY:DXU:BU:9BTUBBDY:DXU:BU:8BPU:B4X9BT38@P00bDY8@PU:@02
8@P02Ad78@PQ224X8@PU:AcW8@PLi`038@P01ad78@PLib487>LQ21cW008M1`038@PM1b4800<Li`04
8@PM1b487@L;7>L00aS67>LHaP037>L06QSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6<HL
iaS67>LHiQcW6>HLiaSV7>L36<H02QcW6<HLiaS67>LHiQcW5<DHaQC511S6010DaABU6<HDYAS65<DH
aQC56<HDaAS6JeY[NV]JKg][FP80005_N`005V]J00<006]JJeX016]J0aBU00<@Y1BU5:D01ABU00@D
aAS65<DHaP<Da@036<HDaAC5008Da@055:DDaAC56<HDa@036<H02AC56<HHaQS65<DHaQC56<HDa@04
5:D00aC55:DDa@035:D76<H00aBU5<DDY@025<D00aS65<DDY@075:D25<D01AS65<DDaAC56<H00aBU
11S60aBU4QS60aC500<HaQC56<H01AS600<LiaSV6>H00QS600<DaAS66<H01aS600<DaAS65<D01aS6
0acW00@M1acW7@LLi`@Q20039BTQ:2DY00@U:@07:DXYJRU:;F/YJRe[:FX01be[0bUZ12e[00<YJRU:
:FX022U:00HYJRe[:FX]JbUZ;F/2:FX00bU::FXYJP03;F/01BUZ;F/]Jbe[:FX00Re[00DaS2f;<H`a
S2e[008YJP/YBP8UB@05:DXU:BE99BTUB@04:DX012DY9DTU:BU:12E900HU:BE99BTU:B4X9BT48@P0
0ad78@PM1`0?8@P47@L021cW7@LM1b487@LQ21cW7@L97>L00ad77>LM1`047>L021SV7>LLiacW6>HL
iaSV7>L26>H96<H00aSV6<HHaP036<H26>H01acW6<HHiQS66>HHaQSV00hHaP035<DHaQS6008HaPI[
FP40005[FP00021[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[
NV]JKg][FV]j4:@DYA2T0QBU02THaQBU5<DDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU
6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS600@DY@0>5<DHaQC56<HDaAS6
5<DHaQBU6<HDYAS65:DHaPDDY@096<HDYAC55:DHaQBU6<HDYAS6008DY@0<4:@DYAC56<HDaAS65:DD
aABU7>LHaQSV11S600<DaAS66<H01AS600dDaAS66<HHaQBU6<HDYAS65<DHaQC56<HDa@026<H01QcW
6<HHaQC56<HDa@<HaP0;5<DLiaS66>HDaAS65<DHaQC56<HDa@046<H01QcW6>HLiacW8@PLi`DQ200=
8BPU:B4X:DXU:BE99BT]JbU:;F/YJRe[:FX00Re[00PaS2e[;F/YJRe[:FX]JbU:0Re[00T]Rbe[<H`Y
BRe[:DX]JbU::FX01BU:01l]JbU:;F/YJRe[:FX]JbU:;F/YBRe[:FX]JbUZ;F/YJRe[:FX]JbUZ;F/]
Rc6<;H/aS2U:;F/YBRe[:DXYJP08:DX01bDY:DXU:BU:9BTYBRDY00DYBP089BTYBRDY:DXU:BU:9BTY
BP8U:@038BPU:B4X00<Q20037@LQ21d700DQ20057@LQ21d78@PM1`038@P05ad78@PLib487>LQ21cW
8@PLib487@LQ21d78@PLib487>LQ21cW7@LHiQcW6>H00QcW00@Q21cW7>LHiP<Li`0<6>HLiaSV7>LH
iQcW6<HLiaS67>LHaQSV1AS600<LiaS67>L01AS600TLiaS67>LHaQcW6<HLiaS67>L01AS600<HiQS6
6<H00QS600DDaAS65<DHaQC500<HaP07JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX0
0aBU4:@DY@085:D01QC56<HDaAS65<DHaP<Da@046<HDaAC55:D25<D011S65<DHaQC50aS600LDaAS6
6<HHaQC56<HDa@045:D00aC55:DDa@035:D76<H00aBU5<DDY@045<D85:D01AS65<DDaAC56<H00QC5
0aBU11S60aBU4QS600@DaAS65<DDa@8HaP035<DHaQS6008HaP8HiP037>LDaAS600XHaP055<DHaQC5
6<HDa@056<H37>L011d77>LM1acW12481bDY00LYBRUZ:DX]JbUZ;F/YJP07;F/00bUZ;F/YJP04;F/0
0bUZ:DXYJP08:DX3:FX00be[:FX]J`02:FX01BU::FX]Jbe[:FX00be[00DYJRe[;F/]JbUZ00<aS004
;H/YJRUZ;F/<:DX29DT012DY9DTU:BE912U:00<U:BE99BT00RE900HYBRE99DTU:BE98BP39BT48@P0
0ad78@PM1`0>8@P27@L012487@LM1acW0Qd700DQ21d78@PLiad700TLi`037@LLiad700HLi`046>HL
iaSV7>L36>H00acW6<HHaP076<H00aSV6<HHaP036<H02AcW6>HHiQS66>HHaQSV6<HHiP0<6<H00aC5
6<HHaP046<H6JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeX006]JKg][FV]jJeY_NaBU6<HDY@<HaP195<DHaQBU5<DDYAS65<DHaQC56<HDYAS65<DHaQC5
6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65<DHaQ2T
5:D@Y1BU5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDYAS65:D00aS6
00HDaAS66<HHaQC56<H25<D04ABU6<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D01AS600<D
aAS65<D00aS600DDaAS65<DHaQC500HHaP037>LHaQcW00DHaP057>LHaR487>LM1`047>L00ad78@PM
1`058@P01bDY:DXU:Be[:DX]JbU:00@]J`0><H`]Jc6<:FX]JbUZ:DXU:BE99BT]JbU::FXUB@@YBP05
;F/YBRUZ:DX]J`02:DX08BE9:DXYBRe[:FX]JbUZ;F/YJRe[9DTYBRE9:DXUBBU:9DT]JbUZ;F/YJRe[
:DX]JbUZ;F/YJRe[:FX]JbUZ:DXUB@03:DX00bE9:DXYBP05:DX04BE9:DXUBBU:9DTYBRE9:DXUBBDY
8BPU:B4X9BTQ:2DY8BP00bDY00HQ22DY8@PU:B4X9BT48@P071d78@PM1b487@LQ21d78@PLib487>LQ
21d78@PLib487>LQ21cW8@PLiad77>LQ21cW8@PM1b481AcW00<M1acW8@P021cW01PHiQcW6>HLiaSV
7>LHiQcW6>HLiaS67>LHaQcW6<HLiaS67>LHiQcW6>HLiaSV7>L46<H00aC56<HDa@046<H061cW6<HH
iQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQC56<I[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX0
16]J0QC500<HaQC56<H00QS611BU00TDaAS65<DHaQBU5<DDYAS65<D00aS600PDaAS66<HHaQC56<HD
aAS60QC500hDYAC55:DDaABU5<DDYAS65<DHaQBU5<DDYAC50aS600HDYA2T5:DDYAS65<D56<H01QC5
6<HDaAS65<DHaP<Da@036<HDaAS6008HaP<Da@THaP035<DHaQBU00<DY@8HaP035<DHaQS6020HaP03
6>HHaQS600<HaP036>HHaQSV00LLi`PQ20079DTU:BE9:DXYJRU::FX00be[00<]Rbe[;H/012e[12DY
2bU:00<YJRU::FX012U:0Re[00DYJRe[:FX]JbUZ00<YBP059DTYBRU::DXYJP03;F/01BU::FXYBRe[
:FX00be[00<YJRe[:DX03BU:00PUBBU::DXYBRE9:DXUBBU:2RDY00@Q:2DY8BPQ:08U:@038BPQ2248
00TQ20<M1`@Q20037@LLiad700PLi`8Q20037@LLiacW00dLi`056>HLiacW7>LHiP037>L011SV7>LH
iQcW0aSV00<LiaSV6>H00QcW00<HiQcW7>L00QcW0aS600<DaAS65<D021S611BU00LDaABU5<DDYAC5
5:DDa@036<H6JeX10001JeX0000RJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDYAS65<D26<H0BQC55:DHaQBU6<HDaAS65<DHaQBU6<HDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDaAS6
4:@DYA2T5:DDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS6
5<D36<H05aC56<HDaAS65<DHaQBU5<DDYAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D01AS600TD
aAS66<HHaQC56<HDaAS65<D00aS600<DaAS66<H00QS600<LiaS67>L01AS600DLiaS67@LLib4800@L
i`037@LQ21d700DQ20079BTYBRDY;F/YBRe[:DX012e[00daS2e[<H`YJRe[:FXUBBDY:DXU:BUZ:DX]
J`02:DX022E9:DXYBRUZ:DX]JbU:;F/4:DX07BE9;F/YJRe[:FX]JbUZ;F/UBBU:9DTYBRE9:DXUBBe[
:FX]JbUZ;F/YBRe[:FX]JbUZ;F/YJRe[:FX00bU:00<UBBU:9DT01BU:014UBBU::DXYBRE9:DXUBBU:
9DTYBRE99BTQ:2DY8BPU:B4X00<U:@088BPU:B489BTQ22DY8BPU:@@Q200L7@LQ21d78@PM1b487@LQ
21cW8@PLib487@LQ21cW8@PLib487>LM1acW8@PLiad77>LQ21d78@P57>L00b487>LM1`087>L061SV
7>LHiQcW6>HLiaSV7>LHiQcW6<HLiaS67>LHaQcW6<HLiaSV7>LHiQcW6>HLi`@HaP055<DHaQC56<HD
a@026<H061SV6<HLiaBU5<DDYAS65:DHaQBU6<HDYAS65:DHaQC56<I[FVmkJeY[NV]JKg/0005[NP00
5V]J00<006]JJeX016]J00@HaQC55<DDa@<HaP@DY@075<DHaQC56<HDYAC55:D00aS600DDaAS65<DH
aQC500<HaP0B5<DHaQC55<DDYAC55:DDaABU5<DDYAS65<DHaQBU5<DDYAC50aS60aBU00<@Y1S65<D0
1AS600PDaAS65<DHaQC56<HDaAS60aC50aS600@DaAS65<DDa@XHaP@DY@035<DHaQS6028HaP036>HH
aQS600<HaP036>HHaQSV00LLi`PQ20079DTU:BE9:DXYJRU::FX00be[00<]Rbe[;H/00Re[00<YJRe[
9BT00bDY2bU:00<YJRU::FX012U:00LYJRe[;F/]JbUZ;F/YJP05:DX02BE9:DX]Jbe[:FX]JbU::FXY
BP03;F/012UZ;F/YJRe[42U:00HUBBU:9DTYBRE9:DX:9BT38BP012DY8BPU:BDY2b4800@M1b487@LM
1`<Q20037@LLiad700PLi`037@LQ2248014Li`056>HLiacW7>LHiP037>L36>H01QcW6>HHiQSV7>LH
iPHLi`<HaP035<DHaQC500PHaP@DY@075<DDYAC55:DDaABU5<D00aS61V]J0@000F]J0000=FmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAC55:DH
aQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<H00QBU04H@Y1BU5:DHaQC56<HD
YAS65:DHaQBU5<DDYAS65:DHaQBU6<HDYAS65:DHaQBU5<DDYAS65:DHaQBU6<HDYAS65<DHaQC56<HD
aAS65:DDaABU6<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HD
aAcW6<HLi`8HaP0=5:DHaQC56<HDaAS66>HLiaSV6<HDaAS65<D021S600DLiaS67>LHiQcW008HiP03
6<HLiaS600<HaP0;6>HLiaSV7>LHiQcW6>HLiaSV7>LHiP037>L02b489BTQ22DY8BPU:B4X9BTYBRe[
:DX012e[01He[C6<=Jd]Rc6<;H/aS2UZ;F/YJRU:9DTYBRE9;F/YBRe[:DX]JbU:;F/YJP<]J`09:DX]
JbU::DXUBBU:9DTYBRE900<YBP049DTYBRU:;F/2:DX01bE9:DXUBBe[:FX]JbUZ00<]J`0E:FX]Jbe[
;F/UBBU:9DTYBRE9:DXUBBe[:DX]JbE9:DXUBBU::FX]JbUZ00DYBP0@9DTYBRE9:DXUBBU:9BTYBRDY
9DTU:BU:9BTUBBDY:DX29BT01B4X9BTQ22487@L01R4800<Q:2488@P0124800<Lib487@L00b4802DL
ib487>LQ21cW8@PLiad77>LQ21SV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiR487>LM1aS67>LH
aQcW6>HLiaSV7>LHiP037>L00aSV7>LHiP057>L03ASV7>LHaQS65<DLiaS67>LHaQcW6>HLiaC500<H
aP0E5<DHaQS66<HDYAS65:DHaQC56<HDaAS65<DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]J
JeX016]J11BU00HHaQC56<HDYAC55:D25<D36<H011C56<HDaAS60aC51aBU0QS60QC500<DYAC55:D0
11BU00LDaABU5<DDYAC55:DDa@035:D00aS65<DDa@025<D02ABU5<DDaAS65<DHaQC56<HDa@045:D0
0aS65<DHaP025<D01QBU5<DDYAC55:DDa@<HaP0:5<DDYAC55:DHaQC56<HDaAS65<D26<H01QSV6<HH
iQC56<HDa@@HaP<Li`035<DHaQS600/HaP8Li`036>HLiaS600HHaP<Li`056>HLiacW7>LHiP067>L0
22DY8BPQ:24X9BTQ:2DY8BP3:DX4;F/00c6/=Jda[004<H`012e[:FX]JbE90bU:0bUZ00@]JbUZ:FXY
JPD]J`8YJP/YBP8YJP03;F/YBRU:008YBP03:FX]JbUZ00P]J`DYBP049DTYBRU::FX5:DX00be[:FX]
J`04:DX00bE9:DXUB@04:DX49BT00bU:9DTUB@049BT00b4X9BTQ:00?8@P37@L48@P47>L00ad77>LM
1`047>L00aSV7>LHiP077>L00aSV7>LHiP047>L01QSV6<HHiQS67>LHiPDLi`036>HLiaSV00PLi`@H
aP036>HHaQSV00<Li`LHaP035:DDaABU008Da@036<HDaAS600<HaPI[FP40005[FP00021[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU0QC5
01<HaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DDa@045:D09a2T6<HDaAS65:DHaQBU
6<HDYAS65:DDaABU6<HDYAS65:DHaQBU5<DDYAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaQBU
6<HDY@025<D06aS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAcW
6<HLi`026<H03aBU6<HDaAS65<DHaQSV7>LHiQS65<DHaQC56<HDa@066<H02ASV6<HLiaSV7>LHiQcW
6<HHiP046<H02aSV7>LHiQcW6>HLiaSV7>LHiQcW6>H00acW00/Q22DY8@PU:B4X9BTQ:2DY:DXYJRU:
00@]J`0D=JdaS3F];H/aS2f;<H`YJRe[:FXYBRE9:DXUBBe[:DX]JbU:;F/YBP<]J`04:FX]JbU:;F/2
:DX00bE9:DXUB@03:DX00bE9:DXYBP02:DX02Re[:DXYBRE9:DXUBBe[:FX]JbUZ1Be[01<YJRe[9DTY
BRE9:DXUBBU:9DT]JbU:;F/UBBU:9DTYBRUZ;F/YJP05:DX042E9:DXUBBU:9DTYBRDY9DTU:BU:9BTY
BRDY:DXU:BE90RDY00<Q:2DY7@L0224800<U:B488@P0124802TLib488@PQ21d78@PLiad77>LQ21cW
8@PLib487>LM1aSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQd77>LQ21S67>LHaQcW6>HLiaSV
00<Li`056>HLiaSV7>LHiP037>L01ASV7>LLiacW5<D00QS600LLiaS67>LHaQcW6>HLi`026<H03AC5
6<HHaQS65<DHaQBU6<HDYAS65<DHaQC500<HaP07JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[
FP04JeX46<H35:D011C56<HDaAC511BU00<DaAS65<D00aS600@DaAS66<HHaP@DY@<Da@086<HDaAS6
5<DHaQC56<HDa@<HaP065<DHaQC56<HDaAS60QC500HDYAC55:DDaABU5<D45:D35<D45:D011S65<DH
aQC50aBU00<DaAS65<D00QS600HHiQS66<HDaAS65<D76<H00aC56<HDa@056<H00aC56<HDa@036<H4
5:DC6<H011SV6<HLiaSV0acW00HHiQcW7>LM1acW7@L37>L0124X8@PQ:2480bU:00PUBBe[:FX]JcF]
=Lde[CG=0cW>136<00X]JbUZ;F/YJRe[:FX]JbUZ;F/YJP<]J`03:FX]Jbe[00<]J`06:FX]JbU::FXY
BRUZ0bU:00HYJRU::FXYBRe[:FX5;F/012f;<H`]Rc6<12e[00<YJRe[:FX00bU:0bUZ12e[12U:00<]
JbUZ:FX03bU:00<UBBDY9DT022DY00<Q:2DY8BP03R4800<Q:2488BP012481AcW00@M1acW7>LHiP<L
i`056>HLiaSV7>LHiP037>L011SV7>LHiQcW0aSV00<LiaSV7>L00QcW00PHiQcW6>HLiaSV7>LHiQcW
2QS600@HiQcW6>HHiPXHaP035<DHaQS600DHaP<Da@I[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk008HaP085<DHaQBU5<DDYAS6
5:DHaP@DY@0:4:@HaQBU6<HDaAS65<DHaQC56<H25<D07QBU6<HDYAS65:DHaQC56<HDaAS65<DHaQC5
6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaPDDY@0N6<HDYAS65:DDaABU6<HDaAS65<DH
aQBU5<DDYAS65<DHaQC57>LHaQcW5<DHaQC56<HDaAS65<DHaQC50aS6014DaAS65<DHaQC56<HDaAS6
5<DHaQC56<HDaAS65:DDaABU00<HaP035<DHaQS600PHaP0W6>HHaQcW6<HLiaS67>LHiQcW6>HLiaSV
7>LHiR487>LQ21cW8@PLiad78@PU:B489BTUBBU:9DTYBRUZ;F/YJSW>=JdicSF]>LhecCW>00@aS00?
:FX]JbUZ;F/YJRe[:FX]JbU:;F/YJRe[:FX]JbUZ00<]J`06:FX]JbUZ;F/YBRe[0RU:01PUBBU::DX]
JbU:;F/YJRe[:FXaS2e[;H/]Jc6<;H/aS2f;;F/YJRe[:FX]JbUZ;F/3:DX01be[:DX]JbU:;F/YJRe[
008YBP049DTYBRU:;F/4:DX01BE9:DXYBRU:9DT00bU:00HUBBU:9DTYBRDY:DX29BT0524X9BTQ:2E9
9BTYBR4X9BTQ:2DY8@PQ:2489BTM1b487@LU:B488BP28@P02Ad78@PQ22DY8@PU:Ad78@PM1`047>L0
92487>LQ21SV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaS67>LHaQcW6>HLiaSV7>LH
iQcW6>HLiaSV7>LHiPPHaP056>HHaQcW6<HLi`046<H00aC56<HHaP056<H01AC56<HDaAS65<D00aS6
00TDYAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX46<H35:D35<D00aS65:DDY@02
5:D35<D00aS65<DHaP026<H00aC56<HDY@035:D02aC56<HDaAS65<DHaQC56<HDaAS65<D00aS600HD
aAS65<DHaQC56<H25<D01QBU5<DDYAC55:DDa@@DY@035<DHaQC500@DY@046<HDaAS65<D35:D00aC5
6<HDa@026<H021SV6<HHaQC56<HDaAS65<D56<H00aC56<HDa@056<H00aC56<HDa@036<H45:DC6<H0
0aSV6<HLi`027>L00aSV7>LHiP027>L00ad77>LM1`037>L01R4X8@PQ:248:DXUB@8YBP07;F/YJRe[
=JdecCF]=Ld00cW>136<0be[00@YJRe[:FX]J`<YJP03;F/YJRe[00H]J`06:FX]JbU::FXYBRUZ0bU:
00HYJRU::FXYBRe[:FX5;F/012f;<H`]Rc6<12e[00<YJRe[:FX00bU:00@YJRe[:FXYJP<]J`@YBP8Y
JP03;F/YBRU:00dYBP039DTU:BE900PU:@038BPU:B4X00hQ20038BPQ224X00@Q20DLi`037@LLiacW
008Li`056>HLiaSV7>LHiP037>L021SV7>LLiacW6>HLiaSV7>L36>H02acW6>HLiaSV7>LHiQcW6>HL
iaSV7>L02QS60aSV00<LiaS66<H02QS600<DaAS66<H00aS600<DaAS65<D01V]J0@000F]J00007V]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDa@<H
aP065:DHaQBU6<HDYAS60QBU02`@Y1BU5:DHaQBU6<HDaAS65<DHaQC56<HDaAS65:DDaABU6<HDYAS6
5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS61ABU00HHaQBU6<HD
YAS65:D25<D051S65<DHaQBU6<HDYAS65<DHaQC57>LHaQcW5<DHaQC56<HDaAS65<D36<H03QC56<HD
aAS65<DHaQC56<HDaAS65<DHaQC56<H25<D00aBU6<HDY@056<H00aC56<HHaP066<H051cW6<HHiQS6
7>LHaQcW6>HLiaSV7>LHiQcW6>HQ21cW8@PLiad77>L28@P03RDY8@PU:BE9:DXUBBU::FX]JbUZ>Lhe
[CW>=Jd3>Lh4<H`03BUZ;F/YJRe[:FX]JbUZ;F/YBRe[:FX]JbUZ00<]J`08:FX]JbUZ;F/YJRe[:DX]
J`8YBP0H9DTYBRU:;F/YBRe[:FX]JbUZ;H/]Jc6<;F/aS2f;<H`]Rbe[:FX]JbUZ;F/YJRe[0bU:00P]
JbU:;F/YBRe[:FX]JbE912U:00@]JbU::DXUB@<YBP0<9DTYBRU::DXUBBU:9DTYBRE9:DXU:BU:0RDY
01@Q:2DY8BPYBRDY9DTQ:2DY8BPU:B489BTQ224X7@LQ21d78BPQ22DY0R4800TM1b488@PU:B489BTM
1b487@L011cW02@Q21cW8@PHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHaQcW6<HL
iaSV7>LHiQcW6>HLiaSV7>LHiQcW6>H86<H01AcW6<HLiaS67>L01QS600<DaAS66<H00aS600<DaAS6
5<D00aS600/DaAS65:DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00<HaQC56<H0
0QS60QC500@DYAC55:DDa@@HaP<DY@065<DDYAC55:DDaABU0QC50aS600TDYAC55:DDaABU5<DDYAS6
5<D00aS600<DaAS66<H00aS600HDaAS65<DHaQC56<H25<D00aBU5<DHaP036<H00aC56<HHaP045:D0
0aS65<DHaP026<H35<D01QS65<DHaQC56<HDa@<HaP<Da@dHaP055<DHaQS66<HDa@056<H00aBU5<DD
Y@046<H00aSV6<HHiP046<H01ASV7>LHiQcW6>H00acW00HHiQcW6>HQ21d78@P47>L38@P0124X8@PQ
:24812U:00<]Rbe[;H/013g_00<ikSg_>Nh0136<2Re[00@YJRU::FXYBP@]J`03;H/]Jbf;00L]J`04
:FXYBRUZ:DX3;F/00bf;<H`]R`05<H`8;F/00bUZ;F/]J`03;F/00bUZ;F/YJP07;F/012U::FXYBRUZ
0Re[00<YJRe[;F/00Re[12U:0bDY00<UBBU:9DT012U:00<UBBDY9DT00RDY00<Q:2DY9BT00bDY00<M
1b487@L00R4800<Q:2488@P00b4800<M1acW7@L04QcW00@M1b487@LM1`@Li`036>HLiaSV00<HaP@L
i`046<HHiQS66>H76<H00acW6>HLi`0>6<H011C56<HHaQS60aC500@HaQC55<DDa@I[FP40005[FP00
02]_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk
5<DHaQC56<HDYAS65:DHaQBU6<HDYAcW6<HHiP026<H07QBU5<DDYAS65:DHaQBU6<HDYAS65<DHaQC5
6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAcW6<HHiP8HaP0@5:DHaQC56<HDaAS65:DHaQBU6<HD
aAS65<DHaQC56<H25:D07Q2T5:DDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDYAS65<DHaQC56<HDYAS6
5:DHaQC56<HDaASV6<HLi`8HaP0<5<DHaQC56<HDaAS65<DHaQC57>LHaQSV0QS600PDYAS66<HLiaS6
6>HHaQcW0QS601DDaAS66<HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW7@LQ21d77>LHiQcW6>H0124800<U
:B489BT012U:00h]Jc6<;F/mkcW^?NlikSg_>Nhmkbf;<H`]Rc6<0Re[00<YJRe[;F/01Be[00LYBRe[
:DX]JbUZ;F/YJP02;F/00c6<;F/]J`04;F/01BUZ;F/YBRe[:DX00Re[00H]Rbe[<H`]Rc6<;H/3<H`0
1Bf;;F/]Jbe[:FX00be[00DYJRe[:FX]JbUZ00<]J`03:FX]Jbe[00L]J`07:DX]JbU:;F/YJRe[:FX0
0be[00@UBBU::DXYBP<U:@0>:DXUBBU:9DTYBRE9:DXU:BU:9BTYBR4X9BTQ:08U:@0:9DTU:B487@LQ
21d79BTQ22DY7@L38@P01acW8@PLiacW6>HLiaSV00@Li`067@LLib486>HLiaSV11cW00<Q21cW8@P0
11cW00THiQcW6>HLiaS67>LHaQcW6>H00acW00<HaQcW6<H011S600HHiQS67>LHiQcW6>H36<H07AC5
6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS6JeY[NV]JKg][FV]j00000Fmk
000FJeX30004JeX36<H35<D01AS65:DDaABU5<D011S60aBU00LDaABU5<DDYAC55:DDa@026<H02AC5
6<HDYAC55:DDaABU5<DDY@036<H00aC56<HHaP056<H35<D00aS65<DHaP025<D00aBU5<DHaP056<H0
0aC55:DDY@025:D36<H35<D01AS65<DHaQC56<H00aC500<HaQC56<H00QS60QC53QS600<DaAS65<D0
1AS600<DYAC55:D011S600<HiQS66>H011S600<HiQcW6>H00acW00DHiQcW6>HLiaSV00<Q20@Li`<Q
20048BPQ224X8@P4:DX00bf;;F/]R`04?Nl00cW^?NlikP04<H`:;F/012UZ:DXYJRU:12e[00<]Rbe[
;H/01be[00@YJRU::FXYBP<]J`03;H/aS2f;00DaS0P]J`03:FX]Jbe[00<]J`03:FX]JbUZ00L]J`03
:DXYJRU:008YJPH]J`@YBP<U:@039DTYBRE900@YBP039DTU:BE900@U:@078BPU:BDY9BTM1b487@L0
0R4800<Q:2488@P00b4800<M1acW7@L04QcW0ad700<Q21cW7>L00QcW00<HiQcW6>H00aS611cW00@H
aQSV6<HHiPLHaP037>LHiQcW010HaP045<DHaQC56<H35<D00aS65<E[FP04JeX20001JeX0000QJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQC56<HD
aAS600<DY@0F6<HDYAS65:DHaQBU6<HDYAC55:DHaQBU6<HDYAS65<DHaQC56<HDaAS65<D45:D05AS6
5:DHaQC56<HDaAS64:@DYA2T6<HDaAS65<DHaQC56<HDYAS65:DHaP035:D02QS65:DHaQBU6<HDYAS6
5:DHaQBU0QC5010HaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDa@<HaP0?5<DHaQBU6<HDYAS6
5<DHaQC56<HDYAS65<DHaQC5008HaP087>LHaQS65<DHaQC56<HDY@8HaP056>HHaQcW6<HLi`056<H0
11cW6>HLiaSV11cW01@Q21cW8@PHaQcW6<HQ21d78@PM1b487@LQ224X9BTQ:2DY:DX]JbU:0Re[00Da
S2e[>Lhe[CW>008e[@08<J`e[BUZ;F/YJRe[:FX]J`8YJP03:DX]JbU:00<]J`07:FX]JbUZ;F/YJRe[
:FX012e[00<aS2e[;H/00Re[014YJRe[:FX]JbUZ;F/]Rc6<;H/e[C6/=Jda[36<;H/aS2f;00L]J`0;
:DX]JbU:;F/YBRe[:DX]JbUZ;F/YJP03;F/01BU:;F/YBRe[:FX00be[00HYJRe[:FXYJRU:;F/2:DX0
2BE9:DXUBBU:9DTYBRE9:DXUB@03:DX01BE9:DXUBBU:8BP00bDY2R4800/Lib487>LQ21SV7>LHiR48
7@LQ21d700<Li`0=6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>H00acW014HiQcW6>HLiaSV7>LHaQcW
6<HLiaS67>LHaQcW6>HLiaSV00@HaP057>LHaQcW6<HLi`056<H01QcW6<HLiaC56<HDa@LHaP0?5<DH
aQC56<HDaAS65<DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00@HaQC56<HDa@@D
Y@055<DDYAC55<DHaP025<D35:D02aC55:DDaABU6<HDaAS65<DHaQC56<H00aBU00PHaQC55<DDaAS6
5<DHaQC50aBU0QS600@DaAS65<DHaP<Da@036<HDaABU00<DY@055<DDYAC55:DDa@055:D56<H35<D0
0aS65<DHaP0:6<H011BU5<DDYAC511S60aC500LHaQC56<HDaASV6<HHiP046<H00aC56<HDa@046<H2
6>H00acW6<HHaP026<H00aSV7>LHiP047>L00ad77>LM1`027>L26>H48@P00ad78@PM1`049BT012UZ
:DXYJRU:0Rf;00HaS3G==LdecCW><J`2=Jd7;F/4:DX8;F/00bUZ;F/YJP07;F/01BUZ;F/]Jbe[:FX0
0Re[00LaS2f;<H`a[3F]<J`e[@02<H`00bf;<H`]J`06;F/2:FX012e[:FXYJRUZ22e[00<YJRU::FX0
12e[00@YJRe[:FX]J`PYBP039DTYBRU:00XYBP@U:@XQ20037@LLiad7008Li`066>HLiad78@PM1b48
11cW00THiQcW6>HLiaSV7>LHiQcW6>H031cW00HHiQS66>HHaQSV6<H26>H37>L36<H01AcW6>HHiQSV
7>L00QSV11S600LHiQcW6>HHaQC56<HDa@076<H01AC56<HHaQS65<D00QS61V]J0@000F]J00008Fmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DaAS6
5<DHaP035:D061S65:DHaQBU6<HDYAS65:DHaQBU5<DDYAS65:DHaQC56<HDaAS65<DHaQC55:D@Y08D
Y@0E6<HDYAS65<DHaQC56<H@Y1BU4:@HaQC56<HDaAS65<DHaQBU6<HDYAS600<DY@0N6<HDYAS65:DH
aQBU6<HDYAC55:DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC50aS600dDYAS6
5:DHaQC56<HDaAS65:DHaQC56<HDa@026<H021cW6<HHaQC56<HDaAS65:D26<H01AcW6<HHiQS67>L0
1AS600HLiaSV7>LHiQcW6>H27>L052487>LQ21S67>LHaR487@LQ21d78@PM1b488BPU:B4X9BTYBRe[
:DX2;F/01C6<;F/icSF]>Lh00SF]00da[3F]:FX]JbUZ;F/YJRe[:FX]JbU::FXYBP03;F/01bUZ;F/Y
JRe[:FX]JbUZ00@]J`03;H/]Jc6<008]J`0A:FX]JbUZ;F/YJRe[;H/aS2f;=Jda[3F]<J`aS2f;<H`]
R`07;F/02BU:;F/YBRe[:DX]JbU:;F/YJP03;F/01BUZ;F/YBRe[:DX00be[00TYJRe[:FX]JbUZ;F/Y
BRUZ9DT00bU:00LUBBU:9DTYBRE9:DXUB@03:DX012E9:DXUBBU:0RDY00<Q:2DY8@P02B4800/Lib48
7>LQ21SV7>LHiR487@LQ21d700<Li`0;6>HLiaSV7>LHiQcW6>HLiaSV7>LHiP037>L04aSV7>LHiQcW
6>HLiaSV7>LHaQcW6<HLiaS67>LHaQcW6>HLiaSV00@HaP057>LHaQcW6<HLi`056<H01QcW6<HLiaC5
6<HDa@DHaP0A5<DHaQS66<HDaAS65<DHaQC56<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J
00A[FP046<HDaAS65<D45:D00aC55:DDa@035<D03aS65:DDYABU5<DDYAC55:DHaQC56<HDaAS65<DH
aP035:D25<D01QS65<DHaQC56<HDa@<DY@085<DHaQS66<HDaAS65<DHaP<Da@@DY@055<DDYAC55:DD
a@055:D46<H25<D011S65<DHaQC52aS600@DYAC55:DDa@@HaP0:5<DHaQC56<HDaAS65<DHiQS66>H4
6<H35<D46<H00acW6>HHiP046<H00aSV7>LHiP047>L011d77>LM1acW0QSV00<Lib488@P00R4800<M
1b487@L012DY00DYJRU::FXYBS6<008]R`04=LdicSG==Ld2=Jd00c6/;F/]J`05;F/4:DX8;F/00bUZ
;F/YJP09;F/01BUZ;F/]Jbe[:FX00c6<00Da[3F]<J`e[Bf;00<aS0P]J`<YJP03;F/YJRUZ00L]J`03
:FXYBRUZ00@]J`04:FX]JbUZ;F/8:DX00bE9:DXYBP0::DX49BT:8@P00ad77>LM1`027>L01QSV7>LM
1b487@LQ20@Li`076>HLiaSV7>LHiQcW6>H03QcW00LHiQS66>HHaQSV6<HHiP027>L00aSV7>LHaP02
6<H26>H01AcW6>HHiQSV7>L011S60aSV00@HaQC56<HDa@THaP055<DHaQS66<HDa@06JeX10001JeX0
000TJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[
NQBU6<HDYAS65<DHaQC511BU00/HaQC56<HDaAS65:DHaQBU6<HDYAS600<DY@<HaP0Y5<DLiaS67>LD
YAS65:DDaABU6<HDYAS65:DHaQBU6<HDYAS65:DHaQC56<HDYAC55:DHaQBU6<HDYAS65<DHaQC56<HD
YAS65<DHaQC56<H@Y0045:D061S65:DDaABU6<HDYASV6<HLiaC56<HDaAS65:DHaQBU6<HDaAS65<DH
aQBU6<HDY@<HaP0A5<DHaQC56<HHiQcW6>HHaQC56<HDaAcW6<HLiaC56<HDa@036<H02AC57>LHaQcW
6<HLiaS67>LHaP037>L00aSV7>LLi`047>L01b487@LQ21d79BTQ22DY008YBP069DTYBRUZ;F/YJRe[
0S6<00<]RcF]=Jd00cF]00<aS3F]:FX00be[00LUBBU::DX]JbU:;F/YBP07;F/012f;<H`]Rc6<12e[
00dYJRe[:FXaS2e[;H/]Jc6<;F/aS2e[<H`]J`04<H`00cF]<H`e[@02<H`022e[<H`]Jbf;;F/aS2e[
;H/2;F/00bUZ;F/YJP03;F/02RUZ;F/YJRe[:DX]JbU:;F/YBRe[1RU:00DUBBU:9DTYBRE900<YBP0]
9DTYBRU::DXUBBU:9BTYBRDY:DXU:BU:8BPU:B4X9BTQ:2DY8BPU:B489BTQ22DY8BPU:Ad78@PM1b48
7>LQ21cW8@PLiad77>LQ21cW8@PLib487>LM1aSV00@Li`068@PLiacW6>HLiaSV0acW01PHiQcW6>HL
iaSV7>LHiQcW6>HLiaSV7>LHaQcW6<HLiaS67>LHaQcW6<HLiaS67>L26<H03AC56<HHaQS65<DHaQSV
7>LHiQcW5<DHaQC500DHaP0=5:DHaQBU6<HDaAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[
FP04JeX01aS65<DDaAC56<HDaAS600@DY@8HaP045<DHaQC56<H35<D00aS65<DDY@025:D56<H00aSV
6<HDY@035:D01QC55:DDaABU5<DDY@8Da@036<HDaAC500<HaP@DY@035<DDYAC500@HaP055:DDaABU
6<HDa@026<H;5:D46<H01AC56<HDaAC55:D00QC50aS600@DYAC55:DDa@@HaP035<DHaQS600<Li`8H
aP045<DHaQS66>H46<H011C56<HHaQS60aSV00DLiaS66>HHaQSV00XLi`8Q20057@LQ224X9BTQ:004
:DX012e[:FX]JbUZ0c6<13F]00<aS36/<H`012e[0bU:00@YJRe[:FXYJPL]J`<aS007;H/]Jbe[;F/Y
JRe[:FX012e[00D]Rbe[;H/]Jc6<008]R`@aS007<J`e[C6/<J`]Rc6<;H/01be[00<YJRe[:FX01Be[
00@YJRe[;F/]J`<YJP03;F/YJRUZ00XYBP039DTYBRU:00DYBP069BTUBBDY9DTU:BE90RDY00PQ:2DY
9BTU:B4X9BTQ:2DY0R4X0bDY0b480Qd700<Liad77>L011cW00<M1acW7>L01acW00<Q21d77@L00QcW
00<HiQcW7>L01acW00HHiQcW6>HLiaSV7>L36>H01AcW6>HHiQSV7>L011SV00<HaQSV6<H01QS600HL
iaSV7>LHiQS65<D56<H35<D011S65<DHaQS61V]J0@000F]J000096mkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:DHaQC56<HDa@@DY@0;6<HD
aAS65<DHaQBU6<HDYAS65:DHaP035:D05QS65<DHaQS67>LHaQcW5:DDaABU6<HDYAS65:DHaQBU6<HD
YAS65:DHaQBU0aS6018DYAS65:DDaABU6<HDYAS65<DHaQC56<HDYAS65<DHaQC56<H25:D0<A2T5:DD
YAC55:DHaQBU5<DDYAcW6<HHiQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HD
aAS66>HLiaSV6<HDaAS65<DLiaS67>LDaAS65<DHaQC5008HaP0:7>LHaQcW6<HLiaS67>LHaQcW6>H3
7>L00aSV7>LLi`027>L022487@LQ21d79BTQ22DY9DT3:DX01BUZ;F/YJRe[;H/00S6<1CF]00HaS3F]
;F/]JbUZ;F/2:DX01BE9;F/YBRe[:DX01be[00@]Rc6<;H/aS0@]J`0=:FX]JbUZ;H/]Jc6<;F/aS2e[
<H`]Jc6<;F/0136<00<e[C6<=Jd00S6<00P]Jc6<;F/aS2e[;H/]Jc6<0Re[010YJRe[:FX]JbUZ;F/Y
JRe[:FX]JbU:;F/YBRe[:DX]J`PYBP059DTYBRE9:DXUB@03:DX0:RE9:DXYBRU:9BTYBRDY:DXU:BU:
8BPU:B4X9BTQ:2DY8BPU:B489BTQ22DY8BPU:Ad78@PM1b487>LQ21cW7@LLib487>LQ21cW8@PLiad7
7>LQ208Li`046>HLiacW8@P27>L00aSV7>LHiP037>L061SV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaS6
7>LHaQcW6<HLiaS67>LHaQcW6<HLi`@HaP0;5<DHaQC56<HHiQcW6>HLiaC56<HDa@056<H03ABU6<HD
YAS65<DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0aS600LDaAS66<HHaQC56<HD
a@096<H35<D01AS65<DDaAC56<H00aC500<HaQC56<H00aS600@DaABU5<DDY@8Da@DHaP045<DHaQS6
6<H35<D36<H00aC56<HDa@025<D00aS65<DDY@035:D36<H011C56<HDaAS61ABU00`DaABU6<HDaAS6
5<DHaQC56<HDaAS65<D66<H00aSV6<HHaP0>6<H021C56<HHaQcW6>HLiaSV7>L26>H011S66>HHaQSV
0acW00@M1acW7@LLi`8M1`DQ20<U:@@]J`@aS0@e[@09=Lde[CG=<H`a[36<;H/]Jbf;010]J`06:FX]
JbUZ:FXYBRUZ2be[3c6<00H]Rc6<;H/]Rbe[;H/8;F/00bUZ;F/YJP07;F/4:DX39DT>:DX29DT01RU:
9DTUBBDY9DTU:@8Q:0059BTQ:2DY8BPU:@0:8@P00acW7@LLi`027@L00b487@LLi`037>L46<H;7>L0
1QSV7>LLiacW6>HLi`THaP036>HHaQS600<HaP<Li``HaP045<DHaQC56<H25<D011BU5<DDYAC50aBU
1V]J0@000F]J00008V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[
FV]jJeY_Nf]JJgXDaAS65<DHaQC50aS600DDaAS65<DHaQC500<HaP0a5<DHaQBU6<HDYAS65:DHaQBU
6<HDYAS65:DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQC5
6<HDaAS65:DHaQBU6<HDYAS64:@00aBU00LDaAS65<DHaQC56<HDa@045:D05AS65:DHaQC56<HDaAS6
5<DHaQC56<HDaAS65<DLiaS66>HHaQcW6<HLi`056<H011cW6<HHaQC50aS6014DaAS65<DHaQC56<HH
iQcW6>HLiaS67>LHaQcW6<HLiaS600@Li`0=8@PLib487>LQ21cW8@PM1b487@LUBBDY:DX00Re[00DY
JRe[<H`a[36<00@e[@03>Lhe[CW>008e[@0?<H`e[Be[<H`]Jc6<:FX]JbUZ;F/YJRe[:FX]JbUZ008]
J`0:;H/]Jc6<:FX]JbUZ;F/YBRe[:DX3;F/02BUZ<H`]Jbf;:FX]JbUZ;F/]R`03<H`042f;<H`aS36<
;H/aS2f;<H`]Rc6<;H/aS2f;<H`]Jc6<0Re[00<YJRe[:FX00be[00LYJRe[:FX]JbUZ;F/YJP04;F/0
0bUZ:DX]J`02:DX01RDY:DXYBRUZ:DX]J`@YBP039DTYBRE900<YBP0>9BTYBRDY:DXU:BU:9BTYBR48
9BTQ22DY8BPU:@8Q:00@8@PU:B488BPQ22DY7@LQ21d78@PLib487>LQ21cW8@P47>L46<H01aSV7>LH
iQcW6>HLiaSV00<Li`0:6>HLiaSV7>LHiQcW6>HLiaS66>H46<H011C57>LHaQcW11S600<HiQcW6>H0
11S600<HiQS67>L011S601DDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAC5JeY_Nf]JJgY[FVmk0000
0F]j000FJeX00`00JeY[FP04JeX00aS65<DHaP046<H00aC56<HDa@086<H25<D01AS65<DDaAC56<H0
0aC500@HaQC56<HDa@@HaP055<DDYAC55:DHaP025<D66<H35<D00aS65<DHaP026<H35<D011S65<DD
aAC511BU00<HaQC56<H00QS600<DaAS65:D011BU00`DaABU6<HDaAS65<DHaQC56<HDaAS65<D66<H0
0aSV6<HHaP0@6<H011C57>LHiQcW0aSV00DLiaS66>HHaQSV00<Li`057@LLiad77>LQ20027@L48@P3
9BT4;F/4<H`4=Jd02CG==JdecC6<<J`aS2f;;F/]R`02;F/00bUZ;F/]J`0;;F/01RUZ;F/YJRUZ:DXY
JP/]J`laS006;H/aS2f;;H/]Jbf;22e[00DYJRe[:FX]JbUZ00D]J`@YBP039DTYBRE900lYBP@UB@04
9BTUBBDY9BT38BP00bDY8BPU:@0:8@P00acW7@LLi`047@L47>L46<H00acW6>HLi`0:7>L011SV7>LH
iQcW2AS600<HiQS66<H00aS600<LiaSV7>L031S600<DaAS65<D00QC500DHaQBU5<DDYAC500<DY@I[
FP40005[FP00021_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_
Nf]JJgY[FVmk5<DHaQC50aS600DDaAS65<DHaQC500<HaP0b5<DHaQS66<HDYAS65:DHaQBU6<HDYAS6
5:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS6
5<DHaQBU6<HDYAS65:DHaP8DY@094:@DYAC56<HDaAS65<DHaQC500@DY@0E6<HDYAS65<DHaQC56<HD
aAS65<DHaQC56<HDaASV6<HLiaS67>LHaQcW00DHaP036>HHaQS6008HaP0E5<DHaQS66<HDaAS65<DH
aQSV7>LHiQcW6<HLiaS67>LHaQcW6<HLiaSV008Li`0>8@PLib487>LQ21cW8@PM1b487@LYBRDY9DTY
JP<]J`04<H`e[C6<<J`3=Jd00cW>=JdicP02=Jd03c6<=Jd]Jc6<;F/aS2UZ;F/YJRe[:FX]JbUZ;F/Y
JP02;F/0336<;F/]RbUZ;F/YJRe[:DX]JbU:;F/YJP8]J`07;H/]Jc6<:FX]JbUZ;F/00S6<018]Rc6<
<H`aS2f;<H`]Rc6<;H/aS2f;<H`]Rc6<;H/aS2e[<H`2;F/00bUZ;F/YJP03;F/01bUZ;F/YJRe[:FX]
JbUZ00D]J`05:DXYJRU::DXU:@02:DX00be[:DXYJP04:DX00bE9:DXUB@03:DX082DY:DXU:BU:9BTY
BRDY:DXQ22DY8@PU:B4X9BTQ:2DY8@PQ:2489BTQ224X7@LQ21d78@PLib487>LQ21cW8@P47>L46<H0
1aSV7>LHiQcW6>HLiaSV00<Li`066>HLiaSV7>LHiQcW0QSV00DHaQcW6<HHaQC5008HaP037>LHaQcW
00@HaP036>HLiaSV00@HaP037>LHaQSV00@HaP0E5<DHaQC56<HDYAS65:DHaQBU6<HDYAC55:DHaV]J
JgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00@HaQC56<HDa@/DY@<Da@DDY@084:@DYAC5
6<HDaAS65:DDa@DDY@<HaP045<DHaQS66<H45:D00aC56<HHaP045:D01aS65<DHaQC56<HDaAS600<D
Y@045<DDYAC55:D25<D00aS65<DDY@025:D011C56<HHaQS60aC500HHaQC55<DDaAS65<D26<H01QSV
6<HHiQBU5<DDY@8Da@066<HDaAS65<DHaQC52QS600<LiaSV7>L031cW0R4800<M1b488@P02B480bE9
12e[136<0cF]00<aS36/<H`00S6/00<e[C6/<H`00c6<0RUZ22e[00<aS2f;<H`00S6<0bf;00<]JbUZ
:FX012e[1S6<00P]Rc6<<H`aS2e[;H/]Jbf;0bUZ22e[00<aS2f;<H`0136<00<]Jbf;;F/00be[00<Y
JRe[:FX01Be[12U:0bDY00@YBRE99DTUB@<YBP069DTYBRE9:DXUBBU:0bE900<YBRE99BT02RDY0b4X
00DQ21d77@LM1b48008M1`@Q20036>HLiaSV00LLi`036>HM1acW00@M1`038@PLiacW00DLi`056>HH
aQSV6>HLi`036>H00acW6>HHiP027>L00aSV6<HHaP096<H011C56<HDaAS60aC50aS611cW00<DaAS6
5<D011S61V]J0@000F]J00008V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]J001[FV]jJeY_Nf]JJgXDaAS65<DHaQ2T1QBU01PDaABU6<HDYAS65:DHaQBU5<DDYAS64:@DYA2T
6<HDaAS65<DHaQBU6<HDYAS65:D25<D011S65<DHaQC50QS600LDaABU6<HDYAS65<DHaP045:D09QC5
6<HDaAS65<DHaQC56<HDYAC55:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDYAS65:DHaQBU
6<HDaAS65<DLiaS67>L26<H02aBU6<HDYAS65<DHaQC56<HDaAS65<D01QS600HHiQS67>LHiQcW6>H3
7>L03ASV8@PLiad77>LQ21cW7@LLib487@LQ21d700<Q200:7@LQ21d78BPQ22DY8@PYBRDY:DX4;F/0
0c6<<J`aS005=Jd01S6<=JdaS3F]<J`e[@@aS003:DX]JbU:00L]J`0E;H/aS2f;<H`]Jc6<;F/aS2U:
;F/YBS6<;F/]Rbe[<H`]Rc6<;H/aS2f;008aS005=JdaS36<;F/aS002;F/00bU:;F/YBP04;F/01Rf;
;F/aS2f;<H`]R`<aS00=;H/aS2e[<H`YJRe[:FX]JbUZ;F/YJRe[:FX00Re[00@YBRE9:DXYBP@U:@03
:DXU:BU:008YBP0=9DTYBRE9:DXUBBU:9DTYBRDY:DXU:BU:8BP01bDY00dQ:2DY8BPU:B489BTLib48
7>LQ21cW8@PLi`038@P00ad77>LHiP027>L01R487>LM1aSV7>LHiP8Li`058@PLib487>LQ20027>L0
51SV7>LLiacW6>HLiaS67>LHaQcW6<HLiaS67>LHaQcW6<HLiaSV7>L86<H07aC56<HHaQS65<DHaQBU
6<HDYAS65<DHaQC57>LHiQcW6>HHaQC56<HDaAS65<DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<0
06]JJeX016]J00@HaQC56<HDa@/DY@035<DHaQC500LDY@065<DHaQC56<HDYAC51ABU00<HaQC56<H0
11S611BU0QS600<DaABU5:D00QBU00LHaQC56<HDaAS65<DHaP035:D01AC55:DDaABU6<H00aC50aBU
0QS600@DaAS65<DHaP<Da@036<HDaAS600<HaP066>HHaQSV5:DDaABU0QC500HHaQC56<HDaAS65<D:
6<H37>L00aSV7>LLi`097>L00ad78@PQ200;8@P00bE9:DXUB@04;F/4<H`3=Jd00c6<<J`aS002<J`0
0cF]<J`aS003<H`00be[:FXYJP07;F/3<H`3;H/0136<;H/YJRUZ1Be[136<00<]Rc6<<H`00c6<00H]
Jbf;;F/]RbUZ;F/2:FX7;F/00c6<;H/aS004<H`01Be[;H/]Jbe[:FX00be[00<YJRe[;F/00be[12U:
0bDY0RE900<YBRE9:DX00RU:00PUBBU:9DTYBRE9:DXUBBU:0bE92bDY00<Q:2DY8BP00Qd700@Q21d7
7@LM1`DQ20036>HLiaSV00DLi`056>HLiacW7@LLi`027@L00b487@LM1`077>L00aSV6<HHiP036>H0
1QcW6>HHiQSV7>LHiP8Li`/HaP035<DHaQC5008Da@036<HDaAS6008HaP@Li`035<DHaQC500@HaPI[
FP40005[FP00025_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_
Nf]JJgY[FVmk5<DHaQC56<H00QBU00<@Y1BU5:D00QBU01`HaQBU5<DDYAS65:DHaQBU6<HDYAC54:@D
YA2T6<HDaAS65<DHaQBU6<HDYAC55:DHaQC56<HDa@<HaP085<DHaQBU5<DDYAS65<DHaP@DY@065<DH
aQC56<HDaAS60QC501hDYAS65:DHaQBU6<HDYAS65:DHaQBU5<DDYAS65<DHaQC56<HDYAS65:DHaQBU
6<HDaAS65<DLiaS67>L26<H02ABU6<HDYAS65<DHaQC56<HDa@036<H00aC56<HHaP026<H021cW6<HH
iQSV7>LHiQcW6>H27>L02Qd77>LQ21cW7@LLib487>LQ21d70b4800`M1b487@LQ21d79BTQ224X8@PY
BRDY:DX4;F/0136<=JdaS36/13F]00HaS3F]<H`e[C6/=Jd4<H`00bU:;F/YBP07;F/05Bf;<H`]Rc6<
;F/aS2e[<H`YBRe[:DX]Rbe[<H`]Jc6<;H/aS2f;<H`]R`02<H`01C6/<H`aS2e[<H`00Re[00<YBRe[
:DX012e[00PaS2e[;H/]Rc6<;H/aS2f;0c6<00T]Jc6<:FX]JbUZ;F/YJRe[:FX00be[00DYJRU::DXY
BRE900@U:@04:DXU:BU:9DT3:DX02RE9:DXUBBU:9DTYBRDY:DXU:BU:0RDY00<Q:2DY9BT00bDY00lQ
:2DY8BPU:B489BTLib487>LQ21cW8@PLib487@L00R4800XLiaSV7>LLiad77>LQ21SV7>LHiP8Li`05
8@PLib487>LQ20027>L00aSV7>LHiP037>L03QS67>LHaQcW6<HLiaS67>LHaQcW6<HLiaSV7>L:6<H0
7AC56<HDaAS65:DHaQBU6<HDaAS65<DLiaSV7>LHiQS65<DHaQC56<HDaAS6JeY[NV]JKg][FV]j0000
0Fmk000FJeX00`00JeY[FP04JeX<5:D01AC55:DDaABU5<D03ABU00<DaABU5:D01aBU00<HaQC56<H0
11S600<DYAC55:D01aS600HDaAS65<DHaQC56<H35<D00aS65<DDY@065:D35<D011S65<DDaAC511S6
0aBU11S600@HiQcW6>HHiP<HaP045<DDYAC55:D25<D<6<H00acW6>HLi`027>L01ASV7>LQ22487@L0
2b480b4X12DY12e[1CF]00Ha[3F]<J`e[C6/=Jd2<H`022f;<H`]Rc6<;H/aS2f;<H`4;F/8<H`00bf;
<H`aS003;F/00bf;<H`]R`02;H/00c6<;H/e[@02=Jd0236/<H`aS36<;H/aS2f;<H`3;H/02S6<;H/]
Rbf;<H`]Rc6<;H/aS2f;1C6<12e[00XYJRe[:FXaS2f;<H`YJRe[:FX]J`<UB@DYBP059DTYBRU::DXU
B@03:DX00bE9:DXU:@059BT01B4X9BTU:BDY8BP00RE900<YBR488@P00R480bDY1B4800@M1b487@LQ
20<M1`038@PM1acW00LLi`8HiP037>LHiQcW008Li`037@LQ21d700DLi`<HiP037>LHiQSV018HaP<H
iPDHaP035<DHaQS600LHaP035<DHaQC500I[FP40005[FP0003=[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU5<D@Y1BU4:@DYA2T5:D@Y1S6
5:DHaQBU6<HDYAS65:DDaABU6<H00QBU00<@Y1BU5:D00QBU00LHaQBU6<HDYAS65:DDa@045:D03AC5
6<HDaAS65<DHaQC56<HDYAS65<DHaQC500DHaP0L5<DHaQC56<HDYAS65:DHaQBU6<HDYAS65:DDaABU
6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<H35:D36<H011C57>LHaQcW0QS600/DaAS65:DHaQBU6<HD
YAS65:DHaQC500HHaP0B6>HHaQcW6>HLiaSV7>LHiQcW6>HQ21d78@PM1b487@LQ21d70b4800hM1b48
8@PU:B489BTQ22E99BTYBRe[;H/]Jc6<0SF]018a[3F]<J`e[C6/=Jda[3F]<J`aS2f;<H`]Rc6<;H/a
S2f;<H`2;H/00be[<H`]J`05<H`0:Bf;<H`]Rc6<;H/aS2e[;H/]Jc6<;F/aS2e[<H`]Rc6<<J`e[C6/
=Jd]Rc6<;H/aS2f;<H`]Rc6<;F/aS2e[<H`]Jc6<;H/aS2f;<H`]Rc6<;H/01C6<01l]Jbf;:FX]JbUZ
;F/]Rc6<;H/]JbUZ;F/YJRU:9BTYBRDY:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRE9:DXUB@039BT01b4X
9BTQ:2DY8BPU:B4X008U:@09:DXU:B487@LQ21d79BTQ:2DY008Q200<7@LQ21d78@PM1b487>LQ21cW
8@PM1b4821cW01PHaQcW6<HLiaSV7>LHiR487@LQ21SV7>LHiQcW6<HLiaS67>LHaQcW6<HLiaS66>H8
6<H02QC56<HHaQSV6<HLiaS67>LHaQcW11S600<DaAS65<D00aS600@DaAcW6<HHiP8HaP095<DHaV]J
Kg][FV]jJeY_N`00005[NP005V]J0`0016]J31BU00DDaABU5<DDYAC500dDY@035<DDYABU00LDY@03
6<HDaAS600@HaP035:DDaABU00LHaP045<DHaQC56<H35<D011S65<DDaAC51aBU00<DaAS65<D00QC5
00<HaQC56<H00aS60aBU11S60aSV00TLiaS66<HHaQC55:DDaABU6<H00QC52aS600@LiaSV7>LHiP<L
i`037@LQ224800/Q20048BPU:B4X8BP39BT4;F/5=Jd01c6/=Jda[3F]<J`e[Bf;00<aS006;H/aS2f;
<H`]Rc6<12e[2S6<00@]Rbe[;F/]J`<]R`06<H`]Rc6<;H/e[C6/0SF]0c6<00H]Rc6<;H/aS2f;<H`3
;H/01S6<;H/aS2f;<H`]R`LaS0@]J`0<:FX]JbUZ<H`]Rc6<:FX]JbUZ;F/UBBU:0RE91RU:00DUBBU:
9DTYBRE900<YBP@U:@058BPU:BDY9BTQ:0029BT00bU:9DTUB@048@P39BT58@P00ad78@PM1`027@L0
12487@LQ21d72AcW0QSV0QcW00DHiQcW7@LQ21d700@Li`8HiP057>LHiQSV6>HLi`0B6<H00aSV7>LH
iP056<H00aC56<HHaP076<H00aC56<HDa@05JeX20001JeX0000[Kg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU5<DDYAS65:DHaQBU6<HDYAS65:DH
aQBU6<H01ABU00@HaQBU5<D@Y08DY@035<DDYAS600@DY@0E5<DHaQC56<HDYAC55:DHaQBU6<HDYAS6
5:DHaQBU7>LHaQcW5:DHaQBU00DHaP095<DHaQC56<HDYAS65:DHaQBU00<HaP0=5<DHaQBU6<HDYAS6
5<DHaQC56<HDYAS65<D00aS600XDaAS65<DHaQC56<HDaAS65<DHaP8Da@035:DHaQS600DHaP055<DL
iaSV7>LHiP036<H00aC56<HHaP026<H00acW6<HLi`037>L01R487@LQ21d78@PM1`DQ20038BPU:B4X
00@U:@08:DXUBBU:;F/aS2e[<H`a[0<e[@0I<J`e[C6/=Jda[3F]<J`aS2f;<H`]Jc6<;F/aS2e[<H`]
Jc6<;F/]Rbe[=JdaS36/;H/01c6<00L]Rc6<;H/aS2f;<H`]R`05<H`01Bf;<H`]Rc6<;H/01c6<00l]
RcF]<H`e[C6/=Jda[3F];F/aS2e[<H`]Rc6<;H/012e[00H]Rbe[<H`YJRe[:FX4:DX03Be[:DX]JbU:
;F/YJRe[:DX]JbU::FXYBRe[008YBP069DTYBRU::DXU:BU:0RDY00<Q:2DY9BT012DY00lUBBDY:DXU
:BE99BTYBRDY:DXQ:2DY8BPU:B489BT00R4800XM1b488@PQ21cW8@PLib487>LQ208Li`0=6>HLiacW
8@PLib487>LQ21cW8@PLiad76>H00acW00HHiQcW6>HHiQS67>L56<H00acW6>HLi`066<H01AC56<HD
aAS65<D011S601PLiaS66>HDaAS65<DHaQC56<HDaASV6<HLiaS67>LHaQcWJeY[NV]JKg][FV]j0001
Kg/001I[FP03001[FV]J00A[FP@DY@065<DDYAC55:DDaABU11C500<HaQBU5:D03aBU0aS600<DaABU
5:D00QBU0QC500@HaQC55<DDa@8HaP066>HHaQC55:DDaABU1AS600@DaAS65<DHaP8Da@035:DDaABU
00@HaP8Da@DHaP<Da@DHaP035<DHaQS600<HaP045<DHaQC56<H35:D76<H011SV7>LHiQcW1aS600<L
iaSV6>H011cW00LM1b487@LQ21d78@PM1`048@P00bDY8BPU:@049BT3:DX0136<;H/]Rbf;2cF]0S6<
0Rf;00H]Jbf;;F/]Rbe[;H/4;F/U<H`01c6/<H`e[C6/=Jda[36<00<]R`03<H`]Rc6<00P]J`03:FX]
JbU:00<YBP<YJP04;F/YJRe[:FX;:DX00bDY9DTU:@0@9BT01BE99BTU:B4X9BT00R4X00<Q224X8@P0
0b4800HM1b487@LM1acW7@L57>L00ad77>LM1`037@L00b487>LLi`067>L00aSV7>LHaP076<H00aSV
7>LHiP0C6<H00aC56<HHaP086<H00aSV6<I[FP05JeX10001JeX0000[JgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDYAC55:DHaQBU6<HDYAS6
5:DHaQBU6<H01ABU00<DaABU6<H00QBU00@@Y1S65:DDa@@DY@0E5<DHaQC56<HDYAS65:DDaABU6<HD
YAS65:DHaQBU7>LHaQcW5:DHaQBU00DHaP0;5<DHaQC56<HDYAS65:DHaQBU6<HDa@036<H02ABU6<HD
YAS65<DHaQC56<HDY@036<H041C56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAC511S600LDaAS6
6<HLiaSV7>LHiP056<H01AC56<HHaQcW6<H011cW00HQ21d78@PM1b487@L58@P00b4X9BTQ:0049BT0
1bU:9DTYBRe[<H`]Jc6<008e[@0J<J`e[C6/=Jda[3F]<J`e[C6/<H`]Rc6<;F/aS2e[<H`]Jc6<;F/]
Rbe[<H`]Jc6/<H`e[@8aS003;H/aS36<00<aS007;H/aS2f;<H`]Rc6<;H/00c6<00L]Rc6<<H`aS2f;
<H`]R`05<H`04Bf;<H`aS3F]<H`e[C6/=Jda[3F];F/aS2e[<H`]Rc6<;H/012e[00HaS2e[;H/YJRe[
:FX4:DX03Be[:DX]JbU:;F/YJRe[:DXYJRU:;F/YBRUZ00@YBP059DTYBRDY:DXQ:0089BT03bU:9BTU
BBDY:DXU:BU:9BTYBR4X9BTQ:2DY8@PU:@048@P02Ad78@PLib487>LQ21cW8@PHiP047>L02B487>LQ
21cW8@PLiad77>LQ20027>L021SV7>LHiQcW6>HLiaS66>H56<H00acW6>HLi`046<H02AC56<HHaQS6
5<DHaQC56<HDa@026<H061SV6<HLiaC56<HDaAS65<DHaQC57>LHaQSV6<HLiaS67>M[FVmkJeY[NV]J
Kg/0005[NP005V]J00<006]JJeX016]J11BU00HDaABU5<DDYAC55:D25<D00aS65<DDa@0A5:D011S6
5<DHaQS611BU00DHaQC55<DDaAS6008Da@076<HHiQS65<DDYAC55:D00aS600@DaAS66<HHaP@Da@03
5:DDaABU00DHaP8Da@@HaP035<DHaQC500/HaP045<DHaQC56<H35:D76<H011SV7>LHiQcW1aS60QSV
1AcW00LM1b487@LQ21d78@PM1`048@P00bDY8BPU:@049BT3:DX2;H/00c6<;H/e[@0:=Jd02Rf;<H`a
S2f;;F/]Rbe[;H/]Jbf;12e[6S6<00<]Rc6<<H`0236<00Ha[36<=Jda[3F]<J`2;H/01C6<;H/aS2f;
<H`022e[00<YJRe[:DX00bU:00<YJRe[:FX00RUZ00<]JbUZ:DX02RU:00<U:BE99BT042DY00DUBBDY
9BTQ:2DY008Q:0038@PQ:24800<Q20@M1`037>LM1acW00@Li`057@LLiad77@LQ20027@L87>L00aSV
7>LHaP076<H00aSV7>LHiP0N6<H00aSV6<I[FP05JeX10001JeX0000TKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC511BU00/H
aQBU6<HDYAS65:DHaQC56<HDaAS600<DY@076<HDYAS65:DHaQBU6<H01ABU01hHaQBU6<HDaAS65<DH
aQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDaAS65<D36<H011C56<HHaQSV
0QS600DDaAS65<DHaQBU008Da@0:6<HDaAS65:DHaQBU6<HDaAS65<D86<H03QcW6<HLiaS66>HHaQcW
6<HLiaS67>LHaQSV5<D36<H00aSV7>LHiP047>L012487@LQ21d7124801LQ:2489BTU:BE99BTYBRDY
:DXU:Be[:DX]Jbf;<H`]Rc6<=JdicSF]>Lhe[CW>008e[@09<H`e[C6<<H`]Rc6<:FX]JbUZ008]J`0=
<H`]Jc6<;F/aS2e[=JdaS3F]<H`e[C6<=Jd0136<00<]Rc6<;H/01C6<00D]Rc6<;H/aS2f;00<aS009
;H/e[C6/=Jda[3F]<H`e[C6<00<e[@DaS005=JdaS36<;H/aS002;H/01Re[<H`]Jbf;;F/aS0H]J`0I
:FX]JbUZ;F/YJS6<;F/aS2e[<H`]Jc6<:FX]JbUZ9DTU:BU:9BTYJRU:;F/Q:2DY8BP00bDY00PQ:2DY
9BTUBBDY:DXU:BU:12DY00XQ:2DY8BPU:B489BTQ224X8@PU:@8Q200@7@LQ21cW8@PLib487@LQ21d7
8@PHiQcW6>HQ21cW7@L27>L021SV7>LHaQcW6<HLiaS67>L26<H01AC56<HDaAS65<D00aS600PDaAS6
6<HLiaS67>LHaQcW11S600<DaAS66<H00QS600TLiaS66>HDaAS65<DHiQS67>L00QS600dDaAS65<DH
aQBU6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP8Da@036<HDaAS6008HaP@DY@<D
a@086<HDYAC55:DHaQC56<HDa@<DY@065<DHaQC55<DDYAC51ABU00@DaABU5<DDY@<HaP055<DHaQC5
6<HDa@036<H01AC56<HDaAS65<D01AS600<DYAC55:D00aS600<DaAS66<H01AS600@DaAS65<DHaP<D
Y@@HaP035<DDYAC500/HaP036>HHaQSV00@HaP<HiP037>LHaQS600DHaPLLi`8Q20037@LQ224800HQ
20@U:@089DTU:BE9:DXYJRU:<H`]R`8aS005>LhecCG==LdicP02=Ld01c6<<J`aS36/;H/aS2f;00@]
J`07;H/]Jbf;;F/]Rbe[;H/0136/00<aS36/<H`01C6<00<]Rc6<<H`02C6<00P]Rc6<<H`aS36/=Jda
[3F]0c6/13F]136<00Le[C6/<J`]Rc6<;H/aS00C;F/01Bf;;F/]Rbe[;H/00Re[00<YJRe[9BT00bDY
0bU:00@U:B4X9BTQ:0LU:@<UB@03:DXU:BDY008U:@068BPU:B4X8BPQ224X1b4800PM1acW7@LLib48
7@LQ21d71acW00@HiQcW6>HLi`<HiPDHaP035<DHaQS600DHaP055<DHiQS66>HHaP026>H00acW6<HH
aP0:6<H00aC56<HHaP076<H00aC56<HDa@06JeX10001JeX0000TJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDYAS65<DHaQC511BU00/HaQBU
6<HDYAS65:DHaQC56<HDaAS600<DY@076<HDYAS65:DHaQBU6<H01ABU020HaQBU6<HDaAS65<DHaQC5
6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDaAS65<DHaQC511S600DLiaS66<HD
aAS6008Da@0=5:DHaQC56<HDaAS65:DHaQBU6<HDaAS65<D021S600dLiaS67>LHaQcW6<HLiaS67>LH
aQSV6<HLi`026<H01AC56<HHiQcW6>H011cW00@Q21d78@PM1`@Q200G9BTQ224X9BTYBRDY:DXU:BU:
9BT]JbU:;F/]Rc6<;H/aS3F]>Lhe[CW>=JdicP02=Jd02C6<=JdaS36<;H/aS2UZ;F/YJP02;F/03C6<
;F/aS2e[<H`]JcF]<H`e[C6<=JdaS3F]00@aS003;H/aS2f;00DaS007;H/aS2f;<H`]Rc6<;H/00S6<
00/e[C6/=Jda[3F]<H`e[C6<=Jda[3F]00DaS00==JdaS36<;H/aS2f;<H`]Jbf;;F/aS2e[;H/012e[
01dYJRe[;F/]JbUZ;F/YJS6<;F/aS2e[<H`]Jc6<:FX]JbUZ:DXU:BE99BT]JbU::FXQ:2DY8BPU:B4X
00@U:@05:DXU:BU:9BTYBP029BT0324X9BTQ:2DY8BPU:B489BTQ22DY8@PQ:08Q200@7@LQ21cW8@PL
ib487@LQ21d78@PHiQcW6>HM1acW8@P27>L021SV7>LHaQcW6<HLiaS66>H26<H00aC56<HDa@036<H0
0aC56<HDa@026<H01AcW6<HLiaS67>L00QS600DDaAS66<HHaQC5008HaP096>HHaQcW5<DHaQC57>LH
aQSV008HaP0=5<DHaQC56<HDYAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX02aS6
5<DDaAC56<HDaAS65:DDaABU5<D011BU0aS611BU0aS611BU00DDaAS65<DHaQC5008HaP095<DDYAC5
5:DHaQC56<HDaAS600<Da@066<HDaAS65<DHaQC50aBU1AS600<HiQS66<H00aC500<HaQC56<H00QS6
00<DaAS66<H01AS600DDaAS65<DHaQC500<HaP035<DHaQS600PHaP076>HHaQSV6<HHiQS66>H01QS6
00<DaAcW7>L00acW00<HiQcW7@L00Qd71B4800@Q:2DY9BTU:@<Q:0039BTQ:2DY00@YBP03;H/aS2f;
00<aS005;H/icSW>>Lhec@03>Lh4=Jd3;H/00be[:FX]J`04;F/;<H`00cF]<J`e[@02<J`01C6<<J`a
S36/<H`00S6/1cF]0S6/13F]00@ecCW>=Ldec@Le[@XaS0L]J`@YBP04:FX]JbUZ:FX7<H`3;F/7:DX0
0bDY8BPU:@099BT01RE99BTUBBE9:DXUB@<U:@058BPU:BDY9BTM1`038@P01Qd78@PM1ad77>LM1`DL
i`036>HHaQSV00/Li`8HiP037>LHaQS6008HaP035<DHaQS6008HaP056>HHaQSV6<HHiP0;6<H00aSV
6<HHiP0<6<H00aC56<HHaP06JeX10001JeX0000/Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC56<HDYAS65:DHaQBU5<DDY@<H
aP035:DHaQBU008Da@0O6<HDaAC55:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65:DH
aQBU6<HDaAS65<DHaQC56<H00aBU02@HaQC56<HDaAcW6<HLiaBU6<HDYAS65<DHaQC56<HDaAS65<DH
aQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D46<H03ASV6<HLiaS67>LHaQcW6<HLiaS6
7>LHaQSV008HaP0g5<DHaQSV7>LHiQcW6>HLiaSV8@PLib487>LQ21d78@PM1bDY8BPU:B489BTQ22DY
8BPU:B4X:DXUBBU:9DTaS2e[<H`]Rc6<;H/aS3G=>LhecCW>=LdicSG==Jda[3F]<J`aS2e[<H`YJRe[
:FX012e[00<e[C6<<J`0236<01Da[3F]<J`e[C6<=JdaS3F]<H`e[C6<=Jda[3F]<J`e[C6/=JdaS3F]
<H`013F]00<icSF]>Lh013F]00Da[3F]<J`e[Bf;00<aS009;H/aS36<<H`]Rc6<:FX]JbUZ00<]J`0@
:FX]JbU::FXYBRe[:DX]JbU:<H`]Rc6<;H/aS2f;<H`3;F/00bUZ:DX]J`02:DX02BE9:DXQ:2DY8BPU
:B4X9BTQ:0039BT06R4X:DXU:BU:9BTYBRDY:DXQ:2DY8BPU:B4X9BTQ:2487@LQ21d78@PM1b487>LQ
21cW8@P27>L01ASV7>LHaQcW6<H011cW00TQ21cW7@LLib487>LM1aS67>L01QS602dDaAS65<DLiaS6
7>LHaQcW6<HLiaC56<HDaAcW6<HHiQS67>LHaQSV6<HLiaS67>LDaAS65<DHaQC56<HDaAS65<DHaQC5
6<HDaAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX25<D02AS65<DHaQC56<HDYAC5
5:DDa@045:D36<H45:D00aS65<DHaP045:D00aC56<HDa@036<H25<D00aBU5<DDY@036<H35<D021S6
5<DHaQC56<HDaAS65<D35:D56<H01QSV6<HDaAC56<HDa@<HaP035<DHaQS600DHaP095<DHaQS66<HD
aAS65<DHaQC500<HaP035<DHaQS600HHaP076>HHaQSV6<HHiQS66>H011S600<DaAS66<H01AcW00@H
iQcW7@LQ208M1`@Q208U:@<Q:0039BTQ:2DY008U:@@YBP<]R`06<H`]Rc6<<H`icSG=1CW>13F]00<]
Rc6<;H/00be[00@YJRe[;F/]J`/aS003=Jda[3F]008a[006<H`a[36<<J`aS36/0SF]00<a[3F]=Jd0
0SF]0S6/1CF]0cG=00<icSF]=Jd01CF]2S6<1be[12U:0bUZ00<]Jc6<<H`01C6<0be[1bU:0bDY00<Q
:2DY9BT01RDY00<UBBDY9DT00bE900<U:B4X9BT012DY0R4800@M1b487@LQ208M1`037>LM1acW00@L
i`036>HHaQSV00`Li`8HiPHHaP085<DHaQS66>HHaQSV6<HHiP/HaP036>HHaQSV008HaP035<DHaQS6
00THaP035<E[FV]J00A[FP40005[FP0005=[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU6<HDaAS65<DHaQBU6<HDYAC55:DHaQBU6<HDaAS6
5:DDaABU6<HDaAS65<DHaQBU5<DDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS6
5<DHaQC56<HDaAS600<DY@0T6<HDaAS65<DLiaS67>LDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC511S600dLiaS66>HHaQcW6<HLiaS67>LHaQSV6<HL
i`026<H0=aC56<HHiQcW6>HLiaSV7>LHiR487>LQ21cW8@PM1b487@LU:B4X9BTQ22DY8@PU:B4X9BTQ
:2U:9DTYBRE9<H`]Jc6<;H/aS2f;<H`ecCW>=LdicSG=>LhecCF]<J`e[C6/<H`]Jc6<:FX]JbUZ00@]
J`03<J`aS3F]008aS003;H/aS36<00<aS00E<J`e[C6/=JdaS3F]<H`e[C6<=JdaS3F]<J`e[C6/=Jda
[3F]<H`e[C6<00@e[@03>Lhe[CW>008e[@06<J`e[C6/=Jda[3F]0S6<00D]Rc6<<H`aS2f;00<aS005
:FX]JbUZ;F/YJP02;F/03bUZ:DX]JbU:;F/YBRe[:DXaS2f;<H`]Rc6<;H/aS004;F/03bU::FXYBRU:
9DTYBR4X9BTQ:2DY8BPU:B4X9BTQ:0029BT06RU:9BTYBRDY:DXU:BU:8BPU:B4X9BTQ:2DY8BPQ21d7
8@PM1b487@LQ21cW8@PLib486>H37>L00aS67>LHaP047>L02Ad77>LQ21cW7@LLib486<HLi`066<H0
;AC56<HDaAcW6<HLiaS67>LHaQcW5<DHaQC56>HHaQcW6<HHiQS67>LHaQcW6<HLiaC56<HDaAS65<DH
aQC56<HDaAS65<DHaQC56<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP045<DDYAC5
5:D36<H45:D01aC56<HDaAS65<DHaQC500@HaP035<DDYAC500@HaP095<DHaQC55<DDYAC55:DHaQC5
00<HaP<Da@066<HDaAC55<DHaQC511BU0aS60aC500DHaQC55<DDaAS600@Da@055:DDaABU5<DDY@02
5<D00aS65<DHaP035:D;6<H011C56<HDaAS60aSV00@LiaSV7>LHiPLHaP8Li`0<6>HLiacW7>LHiQd7
7>LM1b488BPQ224X12480bE912U:0bDY00LYJRe[:FXYJRf;<H`]R`04=Jd2=Ld02cW>=LdmkcW^?Nla
S36/<H`a[2e[;H/01Be[0c6<13F]2c6<0be[2cF]00TicSG=>LhecCW>=LdicSG=>Lh00SG=00<e[CG=
=Jd01c6<12e[0bf;2be[00@]Rc6<;H/aS08]J`07:FX]Jbe[;F/YJRU:9DT01BU:00<U:BE99BT022DY
0bU:00<Q:2DY8BP01bDY00@Q:2DY9BTU:@@Q20<Li`037@LLiad700PLi`047@LQ21d77@L37>L00aSV
6<HHiP086<H01QC56<HDaAS66>HLi`8HiP066<HHiQS66>HHaQSV1aS600@HiQS66>HHaP<Li`046<HH
iQS66>H46<H00aBU5<DDY@06JeX10001JeX0000^Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC56<HDYAC55:DHaQC56<HDaAS6
5<D26<H08QcW6<HHiQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65:DH
aQBU6<HDYAS65:DHaQBU0QC501lHaQC56<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU
6<HDaAS65<DHaQBU5<DHaQcW6<HHiP046<H03QC56<HHaQS65<DHaQC57>LHaQcW6>HLiaSV7>L76<H0
1QcW6>HLiaSV7>LHiP8Li`0K8@PLibDY8@PU:B488BPQ22DY8@PYBRDY:DXUBBU:9DTYBRDY9DTU:Be[
:DX]JbU:<H`]Jc6<00De[@09>Lhe[CW>>NhmkcW^=JdaS3F]008aS006;F/aS2UZ;F/YJRe[0S6<00/]
RcF]<J`e[C6/=JdaS36/;H/aS2f;008aS003=JdaS36/00<]J`06=Jda[3F]<J`e[C6/1CF]00hecCW>
=LdicSG=>LhecCW>=JdicSF]>Lhe[CW>1C6<00<e[C6<;F/00be[00<aS2e[<H`01Be[01H]Rbe[;F/Y
JRe[:FXaS2f;<H`]Rbe[:FX]JbUZ;F/YJRe[9DTYBRE9:DXUB@<YBP059BTYBRDY9BTQ:0039BT04b4X
9BTYBRU:9DTU:B4X9BTQ:2U:9BTUBB4X9BTQ:2DY8BPU:B4X00@Q20@Li`038@PLib4800LLi`038@PL
ib4800@Li`0S6<HLiaS67>LDaAS65<DHaQC56<HDaAS65<DHaQC57>LHiQcW6<HLiaS67>LHaQcW6<HL
iaS66>HHaQcW6<HHiQS67>LHaP027>L01B487>LLiaS67>L01QS600TDYAS6JeY[NV]JKg][FV]j0000
0Fmk000FJeX00`00JeY[FP04JeX011C55:DDaABU0aS611BU00LDaAS65<DHaQC56<HDa@046<H00aC5
5:DDa@046<H01aC56<HDaAC55:DDaABU00<HaP<Da@056<HDaAC55<DHaP035<D45:D36<H00aC56<HD
a@025<D01AS65<DDaAC56<H00QC500DDYAC55:DDaABU008Da@036<HDaAS600<DY@/HaP0;5<DHaQC5
6<HHiQcW6>HLiaSV7>LHiP076<H01ASV7>LLiacW6>H00QcW00LM1acW7@LQ224X8@PQ:0048@P00bE9
:DXUB@04:DX39BT3:FX012e[;H/]Rbf;13F]00dicSG==LdecCg_>Nhmkc6<<J`aS36/;F/]R`05;F/3
<H`4=Jd;<H`3;F/;=Jd01cW>=LdicSG=>LhecCW>00<ec@04>Lhe[CG==Jd7<H`4;F/00bf;<H`]R`0;
;F/01Bf;<H`]Rc6<:FX00be[00DYJRe[;F/YBRE900DYBP039BTUBBDY00PU:@<YBP038BPU:B4X00DU
:@038BPU:BDY00<U:@@Q20<Li`037@LLiad700PLi`<M1`078@PLiacW7>LHiQS66>H021S600HDaAS6
5<DHaQSV7>L26>H01QS66>HHaQSV6<HHiPLHaP046>HHaQSV6<H37>L011S66>HHaQSV11S600<DYAC5
5:D01V]J0@000F]J0000:V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
001[FV]jJeY_Nf]JJgXDYAS65:DHaQC56<HDaAS65<DHaQC56<HDa@<HaP095<DHaQC56<HDaAS65<DH
aQC500<DY@0B4:@LiaS67>LDaAS65<DHaQBU6<HDYAS65:DHaQBU6>HHaQcW0QS600DDaAS65<DHaQC5
00<HaP055<DHaQC56<HDa@035:D03AS67>LHaQcW5<DHaQC56<HDaAS65<DHaQC500<HaP0D5:DHaQS6
7>LHaQcW6<HLiaS67>LHiQcW6>HHaQC56<HDaAcW6>HLi`@HaP@Li`036>HLiaSV00LLi`0=7@LQ21d7
8@PM1b487@LU:B4X9BTQ:2U:9BT00RE901@YBRE9:DXYBRe[:DX]JbUZ;F/YJSF]<J`e[CG=>LhecCW>
?Pm243h?0T8@00Y6<D8@>Lhe[CW>=JdaS2f;<H`]R`<aS009<J`e[C6/=Jda[3F]<J`aS2f;008aS00F
=JdaS3F];H/aS2f;<H`e[CG==JdicSF]=Lde[CW>=LdicSG=>LhecCW><J`2=Jd01cW>=LdicSG=>Nhi
cSg_008icP08=LdicSF]=Lde[CW><H`a[08aS007;F/]Rbe[<H`]Rc6<:FX00be[03@]Rc6<;H/]JbUZ
;F/YJS6<;F/aS2e[:FXYBRe[9DTYBRE9:DXUBBU:9DTU:B4X9BTQ:2U:9BTYBRDY:DXU:BU:8BPU:B4X
9BTQ:2DY8BPYBRDY:DXU:BE99BTYBRDY:DXU:BU:8@PU:@HQ20037@LQ21cW00@Li`038@PLib4800LL
i`097@LLib486>HLiaSV7>LHaQcW00HHaP095:DDaABU7>LHaQSV6<HLiaS6008HiP057>LHiQSV6<HL
i`026<H021C56<HHaQcW6<HLiaS67>L26<H00aC56<HDa@056<H02AC56<I[FVmkJeY[NV]JKg/00001
JgX000M[FP@000=[FP8000I[FP03001[FV]J00A[FP065<DDYAC55:DHaQC50aS600<DaAS66<H00aS6
00<DaAS65<D01aS611BU00<HaQSV6<H011S600DDaABU5<DDYAS6008Da@@HaP075<DHaQC56<HDaAS6
5<D01aS60aBU00<HiQS66>H031S600<DaAS65<D00QSV00TLiaSV6>HHiQcW6>HLiaSV7>L011S600<H
iQcW6>H011S60acW00<HiQcW6>H021cW00PQ21d78@PM1b487@LQ224X1RDY00DYBRE9:DXUBBe[008Y
JP@]J`03<J`e[C6/00@icP0;@Q0n3d8@?Pm2<48@@S0e[CG==Jdec@07<H`00cF]<J`e[@04=Jd3<H`0
0c6/<H`a[002<H`00bf;<H`]R`07=Jd01CG=>LhecCW>=Ld00SW>0cF]00<ecCW>=Ld01CW>00<ecCW>
=Ld013F]0c6<12e[0c6<12e[00<aS2f;<H`00Re[00@YJRe[;H/aS08]R`XYBP038BPU:BDY008U:@05
9DTU:BE99BTUB@029BT01B4X9BTU:BDY8BP00RDY00<UBBDY9DT00bDY00LUBBDY9DTU:B4X8@PQ:004
8@P00ad78@PM1`047>L012487@LM1ad73AcW0aSV00<LiaS66<H00QS611BU1aS600<LiaSV7>L01aS6
00XHiQS66>HHaQSV6<HHiQC56<HDa@PHaPI[FP40005[FP0002Q_Nf]JJgY[FVmkJeY[NP00Kg][FV]j
JeY_N`00JgY[FP00JeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU6<HDaAS65<DHaQC56<HDa@<H
aP0=5<DHaQC56<HDaAS65<DHaQC56<HDaABU4:@00QBU014LiaS67>LDaAS65<DHaQBU6<HDYAS65:DH
aQBU7>LHaQSV008HaP075<DHaQC56<HDaAS65<D00aS600<DaAS65<D00aBU00/HaQcW6<HLiaC56<HD
aAS65<DHaQC500<HaP035<DHaQBU008HaP0A7>LHaQcW6<HLiaS67>LHiQcW6>HHaQC56<HDaAcW6>HL
i`046<H47>L00aSV7>LHiP077>L04Ad78@PM1b487@LQ21d79BTQ:2DY8BPUBBDY:DXUBBU:9DT00RU:
010]JbU:;F/YJRe[:FXe[C6/=JdecCW>=LdicSh?@Q0n3`924008AS5243W>=JdicSF]<H`]R`<aS009
;H/aS36/=Jda[3F]<J`e[C6/00@aS00E=JdaS3F];H/aS2f;<H`e[CW>=JdecCF]>Lhe[CW>=LdicSG=
>LhecCW>008e[@08<J`icSG=>LhecCg_>LhikP8icP0@=LdicSF]>Lhe[CG=<H`e[C6<;H/]Jc6<;F/a
S2f;<H`2;F/032UZ;F/]Rc6<;H/]JbUZ;F/YJS6<;F/aS08]J`0X:DXYJRE9:DXUBBU:9DTYBRE99BTQ
:2DY8BPYBRDY:DXU:BU:9BTYBR4X9BTQ:2DY8BPU:B4X:DXU:BU:9BTYBRDY9DTU:BU:9BTYBR489BT6
8@P00ad78@PLi`047>L00b487>LQ20077>L02B487>LM1aSV7>LHiQcW6<HLi`056<H04AC55:DHaQBU
6>HHaQcW6<HHiQS67>LHiQcW6>HLiaS66>H01AS600DLiaS67>LHaQcW008HaP035<DHaQC500DHaP09
5<DHaV]JJgY[FVmkJeY[NP00005_N`0026]J00<006]JJeX00V]J00@006]JJeX000E[FP@000=[FP08
5<DDYAC55:DHaQC56<HDa@LHaP035<DHaQC500DHaP035<DHaQBU00<DY@036<HHiQS600@HaP045<DD
YAC55:D25<D56<H01aC56<HDaAS65<DHaQC500LHaP<DY@036>HHaQSV00`HaP<Da@057>LHiQSV6>HL
i`036>H00acW6>HLi`046<H00aSV7>LHiP046<H37>L00aSV7>LHiP087>L01b487@LQ21d78@PM1b48
008U:@038BPU:BDY008U:@04:DXUBBU:9DT2:FX5;F/00c6/=Jda[004>Lh02d8@?Pm243h?@S12448`
=JdecCF]=Ld01c6<1cF]0c6<00<a[36<<J`00S6<00<]Rc6<;H/01cF]00<ecCW>=Ld00cW>00LecCF]
=Jde[CG=>Lhec@05>Lh00cG=>Lhec@04=Jd3<H`4;F/3<H`4;F/3<H`012UZ;F/]Jbe[0bf;00<aS2U:
:DX01RU:00DUBBU:9BTU:B4X008U:@059DTU:BE99BTUB@049BT00b4X9BTU:@029BT00bE99BTUB@03
9BT01bE99BTUBBDY8BPQ224X00@Q20037@LQ21d700@Li`8M1`038@PM1acW00XLi`046>HLiacW7>L2
6>H46<H45:D76<H37>L76<H02QSV6<HHiQS66>HHaQSV5<DHaQC521S616]J0`000F]J00007F]jJeY_
Nf]JJgY[FVmkJeY[NP00Kg][FV]j001_Nf]J001[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX01ABU00XH
aQBU6<HDaAS65<DHaQC56<HDa@<HaPDDY@035<DDYAS6008HaP045<DHaQC56<H55:D04AS65:DDaAC5
6<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<H00QBU018@Y1S65<DHaQC56<HDYAS65:DHaQBU6<HDYAS6
5:DDaAS67>L26<H01AC56<HDaAS65<D01aS600DDaAS65<DHaQC500<HaP075<DLiaS67>LHiQcW6>H0
0QcW00@Q21cW7>LHiP<Li`0]6>HLiacW8@PLib487>LQ21cW9BTQ224X8@PU:B4X9BTQ:2U:9BTYBRUZ
;F/YJRe[:DX]JbU:;F/YJRe[:FXe[C6/=JdikSg_>NhmkdYbCW=:LTicD[AF]E:d00=24003?Ple[C6<
008e[@03>Lhe[CG=00<e[@03=Lde[CW>00@e[@DaS00><J`aS3F]<J`e[C6/>Lhe[CG==JdmkcW>?Nle
c@<icP07=LdicSG=>Lhe[CW>=Jd013W>00TmkcW^?NlicSg_>LhmkcF]=Ld00SF]00Ha[3F]<J`aS2f;
<H`7;F/01c6<;F/aS2e[<H`]Jc6<008]J`09:DX]JbU:;F/YBRe[9DTYBRE900LU:@058BPU:BDY9BTQ
:0049BT012U:9BTU:B4X0bDY00<Q:2DY9BT00bDY00HQ22DY8@PU:B488BP28@P01Ad78@PHiQcW6>H0
0QcW00DQ21cW7@LLib48008Li`0=6>HLiaSV7>LHaQcW6>HLiaSV7>LHiQcW6>H011S6018LiaSV7>LH
iQcW6>HLiaSV7>LHiQcW6>HLiaSV6<HDaAS65<D56<H021C56<HDaAS65<DLiaS66>H26<H03AC56<HD
aAS65:DDaF]JKg][FV]jJeY_N`00005[NP001f]J00@006]JJeX0009[FP04001[FV]J0005JeX00`00
JeY[FP04JeX45:D00aS65<DDa@026<H01AC56<HDaAS65<D011S61aBU11S600<DaAS66<H021BU00<H
aQC56<H01QS60aC500<HaQC55:D00QBU11S60aC500@HaQC55<DDa@@DY@036>HHaQSV008HaP055<DH
aQC56<HDa@0?6<H36>H01acW6>HLiaSV7@LLiad700@Li`0:6>HLiaSV7@LLiad77>LM1acW7@L48@P5
9BT01BE99BT]JbUZ;F/00RUZ00TYBRUZ:FX]JbUZ;F/a[3F]<J`013g_14ic0eJe148@0c6/3SF]236<
00<e[C6/=Jd013F]00<icSW^>Lh01cW>00@ecCW>=Ldec@@icP8mk`<ikP03?NlikSF]00He[@03;H/a
S36<00P]J`09;H/]Jbf;;F/]Rbe[;H/YJRe[00<YJP03;F/YJRU:008YBPlU:@03:DXUBBE900/U:@8Q
:0039BTQ224800DQ20PLi`037@LLiad700@Li`036>HLiaSV00DLi`036>HLiaS600<HaP096>HLiaSV
7>LHiQcW6>HLiaSV00<Li`056>HLiaS66<HDa@0?6<H011C56<HHaQS60aBU1V]J0@000F]J000026mk
JeY[NV]JKg][FV]jJeX2000016]jJeY_Nf]J0P0000e_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`05
5:D02QS65:DHaQC56<HDaAS65<DHaQC50aS61ABU00@HaQBU6<HDa@<HaP035<DHaQBU00@DY@0B5<DD
YAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQ2T0QBU00hHaQC56<HDaAS65:DHaQBU6<HDYAS6
5:DDaABU0QS600PLiaS66<HDaAS65<DHaQC51AS600TDaAS66<HHaQC56<HDaAS65<D00QS600HLiaS6
7>LHiQcW6>H27>L00b487>LLi`027>L00aSV7>LHiP027>L0;2487>LQ21cW8@PLib4X8@PU:B489BTQ
:2DY8BPYBRDY:DXYJRe[:FX]JbU:;F/YBRe[:FX]JbUZ=Jda[3F]>NhmkcW^?Nm:LTicBW9>Le:dE[EB
]48@?Pl2@Q001cF]<H`e[CF]=Lde[CW>00<e[@03>Lhe[CG=00@e[@DaS00==JdaS36/<J`e[C6/=Lde
[CW>=JdmkcW>?Nl00SW>00TecCW>=LdicSG=>Lhe[CW>=Jd013W>00TmkcW^?NlicSg_>LhmkcF]>Lh0
0SF]00Ha[3F]<J`aS2f;<H`7;F/01c6<;F/aS2e[<H`]Jc6<008]J`09:DX]JbU:;F/YBRe[9DTYBRE9
00TU:@058BPU:BDY9BTQ:0029BT00bU:9BTU:@029BT00b4X9BTQ:0059BT01R489BTQ224X8@PU:@8Q
20057@LQ21SV7>LHiP027>L01Ad77>LQ21cW8@P00QcW00dHiQcW6>HLiaS67>LHiQcW6>HLiaSV7>LH
iP046<H04QcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HHaQC56<HDa@<HaP0:5<DHaQS66<HDaAS6
5<DHiQS67>L26<H03AC56<HDaAC55:DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J
11BU0aS621BU0aC500HHiQS66>HHaQC55:D25<D01QS65<DDaAC56<HDa@@DY@DHaP035<DHaQBU00<D
Y@045<DHaQS66<H35<D00aS65<DDa@045:D86<H00aC56<HDa@076<H35<D011cW6>HHiQSV3QS600@L
iaSV7>LHiPPLi`037@LLiacW00<Li`LQ20@U:@039DTYBRE900@YBP8YJP06;F/]Rbf;;H/aS2f;0S6<
00/mkcW^?NlikTicBW9>LdYbE[EB]5Je00A6<@Xe[@@icP08=LdicSW>>LhecCW>=LdicP<ec@PicP03
>NhicSg_00<mk`03>NhicSW^00DicP03>NhicSW^00<ikP07?NlikSg_>Nhe[C6/=Jd01c6<00@]Rbe[
;H/]J`8]R`DaS004;F/]Rbe[;H/4;F/3:FX7:DX01bE9:DXUBBU:9DTYBRE900DYBP039DTYBRU:00<Y
BP069DTYBRU:9DTU:BE912DY124800<M1b487@L00Qd700HQ21d77@LM1b487@L77>L36>H?6<H00aSV
7>LHiP076<H36>H00acW6<HHaP026<H01AC56<HDaAS65<D00aS600<DaAS66<H01QS61V]J0@000F]J
000086]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]J
JgXDYAS65:D25<D01AS65<DDaABU6<H01ABU01@HaQBU6<HHaQcW6<HLiaBU6<HDYAS65:DHaQBU6<HD
YAS65:DHaQBU0QC500PHaQC56<HDaAS65<DDYA2T0QBU00THaQC56<HDYAS65:DHaQBU6<H011BU00d@
Y1cW6<HHiQC56<HDaAS65<DHaQC56<HDa@076<H01QBU6<HHaQcW6<HLi`HHaP075<DLiaS66>HHaQcW
6<H00QSV00@LiaSV7>LHiP<Li`066>HLiacW8@PLib4811cW0b4800@U:B488BPQ20<U:@038BPYBRE9
00HYBP0H;F/YBS6<;F/aS2e[<H`]Rc6<>NhmkcW^?Nm:LTicBW9>Le:dE[EB]4Ha@S16<D8`13F]01@i
cSF]=Lda[3F]<J`icSG=>LhecCW>=LdicSG=>LhecCW>=JdicSF]0cW>00<ecCW>>Lh00SW>00TmkcW>
?NlikSg_>NhmkcW>?Nl01CW>014mkcW>?NlicSg_>LhmkcW^?NlikSg_<J`e[C6/=JdaS36/008aS00@
;H/aS2e[<H`]Jc6<;F/aS2e[<H`]Rc6<;H/aS2e[<H`2;F/01RUZ;F/YBRe[:DX]J`8YBP0E9DTYBRE9
:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRE900<YBP0J9DTYBRDY:DXU:BU:9BTUBBDY
8@PM1b487@LQ21d78@PLib487>LQ21cW8@PLib487>LQ208Li`066>HLiaSV7>LHaQcW1AS6014HiQS6
7>LHaQSV6<HLiaS66>HHaQcW6>HLiaS67>LHaQSV00<HaP037>LHaQcW008HaP0@5<DHaQC56<HDaAS6
5<DHaQC56<HDaAS65<DLiaS66>H26<H02AC56<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J
00A[FP@DY@036<HDaAS600PDY@095<DHaQC56>HHaQSV6<HDaABU00@Da@046<HDaAC55<D45:D56<H0
0aC56<HDY@035:D26<H35<D01AS65<DDaAC56<H011BU21S600<DaAS65<D01aS600<DaAS65<D00QSV
00<LiaSV6<H03AS600@LiaSV7>LHiPPLi`037@LLiacW00<Li`LQ20@U:@039DTYBRE900@YBP05;F/Y
JRUZ;H/aS002;H/2<H`032f;?NlikSg_>Ni>LdYbCW=:LUJeD[AF]@A6<@Pe[@03<J`e[CW>00DicP04
=LdicSG=>Lh3=Ld00cW>=LdicP07>Lh00cW^>Lhmk`03?Nl00cW^>LhikP05>Lh03CW^>LhikSW^?Nli
kSg_>NhmkcW^=Jda[3F]00LaS005;H/]Jbf;;F/aS002;H/4<H`012e[;H/]Jbf;0be[0RUZ00<]JbUZ
:DX01RU:00LUBBU:9DTYBRE9:DXUB@05:DX00bE9:DXYBP05:DX29DT00bDY9DTU:@039BT48@P011d7
8@PM1b480ad700@Q21d77@LM1`LLi`036>HLiaSV00lHaP036>HLiaSV00LHaP046>HLiaSV6>H46<H0
0aC56<HDa@036<H00aC56<HHaP086<H6JeX10001JeX0000WKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU5<DDYAS65<DHaQC56<HDYAC500DDY@0L
6<HDYAS66<HLiaS67>LDYAS65:DHaQBU6<HDYAS65:DHaQBU5<DDYAS65<DHaQC56<HDaAS65<D35:D0
2Q2T6<HDaAS65:DHaQBU6<HDYAS60QBU00d@Y1BU5:DHiQS67>LDaAS65<DHaQC56<HDa@036<H00aC5
6<HHaP036<H01QBU6<HHaQcW6<HLi`@HaP0=5<DHaQS66>HHaQcW6<HHiQS67>LHiQcW6>H00acW00PH
iQcW7>LLiaSV8@PLib4811cW0b4800HQ:2489BTQ22DY8BP29BT00bU:9DTYBP05:DX06Re[:DXaS2e[
<H`]Jc6<;H/aS3W^?NlikSg_BW9>LdYbCW=B]5JeD[A6<D8`AS52<3F]<J`2=Jd05SG==JdicS6/=Jda
[3W>=LdicSG=>LhecCW>=LdicSG=>Lhe[CW>=JdicSG=1SW>00TmkcW>?NlikSg_>NhmkcW>?Nl01CW>
00dmkcW>?NlicSg_>LhmkcW^?NlikSg_<J`e[@02<J`0536<=JdaS36<;H/aS2e[<H`]Jc6<;F/aS2e[
<H`]Rc6<;H/aS2e[<H`4;F/01BU:;F/YBRe[9DT00bU:014UBBU:9DTYBRE9:DXUBBU:9DTYBRE9:DXU
BBU:9DTYBRE900<YBP0L9DTYBRE9:DXU:BU:9BTYBRDY:DXU:B487@LQ21d78@PM1b487>LQ21cW8@PL
ib487>LQ21cW7@L27>L01QSV7>LHiQcW6<HLi`DHaP0A7>LHaQSV6<HLiaS66>HHaQcW6<HLiaSV7>LH
aQSV6<HLi`036<H00acW6<HLi`046<H03QC56<HDaAS65<DHaQC56<HDaAS65<DHiQS67>L26<H02AC5
6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP036<HDaAS600<Da@036<HDaAS6008H
aP045<DHaQC56<H35<D03QS65<DDaAC56<HDaAS65<DHaQC56<HDaAS65<D;6<H00aBU5<DDY@045<D4
5:D00aS65<DHaP045:D26<H25<D00aBU5<DDY@036<H00aC56<HHaP056<H00aC56<HDa@046<H01QSV
6<HHiQSV7>LHiP<Li`036>HLiacW008Li`048@PM1b487@L38@P37@L88@P79BT4:DX00bE9:DXUB@04
:DX01Re[:FX]Jc6/=Jda[0@e[@04?NlikSW^>Nh7?Nl013W^?NlikSW^0SF]00<a[3W>>Lh00cW>00De
cCW>=JdecCF]00@ec@PicP<mk`@icP03>NhmkcW^00Dmk`05>NhmkcW>>NhicP04>Nh013g_>NhikSW^
1cW>00<a[3F]<J`023F]13W>00<ikSW>>Nh0136<00D]Rc6<<H`]JbUZ00<]J`09:FX]Jbe[;F/YJRe[
:FX]JbUZ00<]J`04:FX]Jbe[;F/2:FX00bU::FXYJP03;F/00bE9:DXYBP0;9BT28@P00ad77>LLi`02
7>L27@L00b487@LQ20028@P77>L01ASV6<HHiQS67>L00QSV21S60acW21S600<HiQS66>H02aS600DH
iQS66>HDaAS6008Da@<HaPI[FP40005[FP0003a[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HD
YAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65:D26<H061cW6<HHiQC56<HDaAS65<DHaQC56<HDYAS6
5:DHaQBU6<HDYAC55:DHaQC56<HDa@@DY@0<6<HDaAS65:DHaQBU6<HDaAS65<DHaQC50aS600DDaAS6
5<DHaQC500HHaP0:7>LHaQcW6>HLiaSV7>LHiQcW6>H37>L05Ad78@PM1b487@LQ21d78@PLib487>LQ
21d78@PM1b487@LQ224X9BTQ:0039BT00b4X:DXYBP03:DX00bE9:DXYBP03:DX03BUZ;F/YJSF]<J`e
[C6/=Jda[3F]>LhmkcW>00<mk`03>Nhmkcg_00<mk`0C>LhmkcW>=Jda[3F]=LdicSG=>LhecCW>=Ldi
cSF]>Lhe[CW>=Jd00cW>00<ecCW>>Lh00cW>00<ikSg_>Nh00cW>00<ecCg_>Lh00Sg_015243g_?Pli
kSg_>NhmkcW>?NlicSg_>LhmkcW>?NlicSg_008icP09=LdicSW>>LhecCF]<J`e[C6/00Le[@0^=Ldi
cSG=?NlicSg_>LhaS2f;<H`]Rc6<;H/aS2UZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbU:;F/YJRe[:FX]
JbUZ;F/YJRe[:DX]JbU:;F/YJRe[:FXYBRE9:DX69BT04b4X9BTU:BDY8BPQ21d78@PLib487>LM1acW
8@PLib487@LQ21d700DLi`086>HLiaS67>LHaQcW6<HLi`8HaP035<DHaQC500<HaP045<DLiaSV7>L7
6<H00acW6<HLi`026<H01QC56<HHaQSV6<HLi`<HaP037>LHaQcW008HaP035:DHaQBU00<HaP07JeY_
Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX01AS65<DHaQC56<H00QC50QS600PDaAS65<DH
aQC56<HDaAS60aC500THaQC56<HDaAS65<DHaQC56<H00aC52AS600DDaAS65:DDaABU008Da@036<HD
aABU00<DY@<HaP@DY@095<DHaQS65<DDYAC55:DHaQC500THaP035<DHaQC500@HaP086>HHaQSV6>HL
iaSV7>LHiPHLi`098@PM1b487@LQ21d78@PM1b48008M1`LQ20LU:@@YBP039DTYBRE900@YBP06;F/Y
JRe[<J`e[C6/13F]0SW^00<mkcW^?Nl01Sg_0cW^00@mkc6/=Jde[@DicP05=LdicSF]=Lde[@02=Ld0
0cW>=LdicP07>Lh3?Nl4>Lh3>Nh5?Nl01CW^?NlicSW^>Lh00SW^00HmkcW^>NhikSg_>Nh7>Lh00c6/
=Jda[008=Jd4>Lh00cW^>LhikP06<H`01Bf;;F/]Jbe[:FX00be[00@YJRe[;F/]J`<YJP03;F/YJRe[
008]J`06:FX]Jbe[:FXYBRUZ0Re[00DYJRe[:DXYBRE900/U:@037@LQ224800@Li`078@PM1ad77@LQ
21d78@P01acW00@HiQS66>HHaP8HiP037>LHaQS600HHaP<Li`PHaP036>HHaQSV00/HaP036>HHaQSV
00<Da@@HaPI[FP40005[FP0002A_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FP00JeY_Nf]JJgY[FVmk5<DHaQC56<HDaAS65<D45:D041S65<DHaQC56<HDaAS65<DHaQC56<HD
YAS65:DHaQC50aS601XDaAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU
6<HDaAS60QC500DDYAS65:DHaQBU00<HaP055<DHaQBU6<HDY@026<H01ASV6<HLiaS67>L00QS600DD
aAS65:DHaQBU008HaP087>LHaQSV6<HLiaS68@PM1`8Q20DU:@048BPU:B489BT28@P01ad78@PM1b48
7@LQ21d700<Q20078BPU:B4X:DXUBBU:9DT01BU:01<UBBU::DXYJRU:;F/]Rc6<;H/e[C6/=Jda[3W>
=JdicSG=>Lhec@05>Lh02C6/=Jda[3W>=LdicSG==Jda[005=Jd06cW>>NhicSW>=JdicSF]>Lhe[CW>
>NhmkcW^?NlicSg_>LhmkcW^?NlikSg_>NhmkcW^@Q0mk`02@Q001DHa@Q16<Cg_@Q001Sg_00LicSg_
>LhicSG=>Lhec@03>Lh02CG=>LhecCF]<J`e[C6/>Lhec@05>Lh01cW^?NlikSg_>LhmkcW>00<e[@05
<J`aS2f;<H`]R`03<H`01C6/=Jda[36<;H/00c6<00<]Rc6<=Jd00cF]1S6<00`]Rc6<;H/aS2f;;F/Y
JRe[9DTYBRE9:DX79BT03B487@LQ21cW8@PLib487@LQ21d78@PLib4800@Li`086>HLiaSV7>LHaQcW
6<HHiP<HaP076>HHaQcW6<HLiaSV7>L00QSV00<HaQcW6<H011S600<LiaS66<H011S600<DaAS66<H0
0aS600<DaAS65<D011S600hDaABU6<HDYAS65<DHaV]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[
FP04JeX36<H011C56<HHaQS611BU00`DaAS66<HHaQC56<HDaAS65<DHaQC56<H25<D76<H25<D01AS6
5<DDaAC56<H00QC500<DYAC56<H00aS600@DaAS66<HHaP<Da@036<HDaAS600@DY@035<DHaQC500@H
aP045<DDYAC55:D36<H36>H01AcW5<DHaQC56<H00aC51aS612481bDY00@Q:2488BPM1`<Q20037@LQ
21d700@Q20039BTQ:2DY008YBP039DTYBRU:00XYBP<aS003<J`e[C6/008e[@03=Lde[CW>00LicP<e
[@8icP05=LdicS6/=Jda[004=Jd3>Lh033F]=Lde[CG==JdecCF]?NlikSg_>Nhmk`<ikP0>?NlikSg_
>NhmkcW^?Nln3d8@?Pm2<48@@S12408n3`03@Q0mkcg_008mk`04>LhikSW>>Nh8>Lh02CG=>Lhe[CF]
<J`e[CG=>Lhec@04>Lh3?Nl013W>>NhicSW^13F]00<]Rc6<;H/0136<00<e[C6/=Jd01c6<13F]2c6<
00LYJRe[:FXYBRE9:DXUB@079BT38@P01ad77>LM1acW8@PM1b48008M1`037>LM1acW00<Li`036>HL
iaSV00/HaP@Li`LHaP036>HHaQSV00@HaP035<DHaQS600/HaP@DY@<HaPI[FP40005[FP0002A[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC5
6<HDaAS65<D45:D03QS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:D36<H09AC56<HDaAS65:DHaQBU
6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU5<DDYAS65:DHaQC5
00<HaP035:DHaQBU008HaP057>LHaQcW6<HLi`026<H01AC56<HDYAS65:D00QS600HHiQS67>LHaQSV
6<H38@P01Ad79BTU:BDY8BP00bDY00TQ22DY8@PQ21d78@PM1b487@L00b4800TM1b488BPU:B4X:DXU
BBU:9DT022U:010]JbU::FX]Rc6<;H/e[C6/=Jda[3W>=JdicSG=>Lhec@DicP09<J`e[C6/>LhecCW>
=Lde[C6/00De[@0K>LhmkcW>>Lhe[CW>=JdicSF]>LhikSg_>NhmkcW>?NlicSg_>NhmkcW^?NlikSg_
>Ni243g_00924005AS5244Ha?Nm24006?Nl01CW>?NlicSW>=Ld00cW>00/ecCW>=LdicSG==Jda[3F]
<J`icSG=00DicP09>NhmkcW^?NlicSg_>Lhe[C6/008e[@03<H`]Rc6<008aS005;H/aS36/=Jda[003
<H`012f;<H`aS36<13F]136<00h]Rc6<<H`aS2f;<H`]Rbe[:FX]JbE9:DXUBBU:1bDY0b4800XLib48
7>LQ21d78@PM1b487>LQ208Li`0:6>HLiacW7>LHiQcW6<HHiQS67>L36<H02acW6<HHiQS67>LHiQcW
6>HLiaS66>H01AS600<LiaS66<H011S600<DaAS66<H00aS600<DaAS65<D01AS600<DYAC55:D00aS6
00M[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP036<HDaAS600@HaP@DY@8HaP085<DH
aQC56<HDaAS65<DHaP<Da@THaP<Da@036<HDaAC5008Da@035:DDaAS600DHaP<Da@036<HDaAS6008H
aP@DY@<Da@@HaP045<DDYAC55:D36<H01QSV7>LHiQSV5<DHaP<Da@036<HDaAS600HHaP@Q20LU:@03
8BPQ224X008Q20057@LQ21d78@PM1`048@P012DY8BPU:BE93RU:0c6<00<a[3F]<J`00SF]00<ecCF]
>Lh01cW>0cF]00LecCW>>LhicS6/=Jda[004=Jd3>Lh02SF]=Lde[CG==JdecCF]?NlikSg_0cW^00<m
kcW^?Nl00Sg_00@ikSg_>Nhmk`<n3`03@S12448`0092408n3`@mk`04>LhikSW>>Nh:>Lh01c6/=Jde
[CF]=LdicSG=00@icP<mk`04>LhikSW>>Nh4=Jd00bf;<H`]R`04<H`00cF]<J`e[@07<H`4=Jd;<H`0
1bUZ;F/YJRU:9DTYBRE900LU:@<Q20077@LLiad77>LQ21d78@P00Qd700<Liad77>L00acW00<HiQcW
6>H02aS611cW1aS600<HiQS66>H01QS600<DaAS66<H02AS611BU0aS61V]J0@000F]J0000;VmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:DH
aQBU5<DDYAS65<DHaQC56<HDYAS65:DHaQC51AS600LDaAS65<DHaQBU6<HDY@036<H55:D06QS65:DH
aQBU6<HDYAS65<DHaQC56<HDYAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65:D36<H031C56<HDaAS6
5<DHaQC56<HDaAcW6<HHiP8HaP035<DHaQS600<HaP056>HLiaSV6<HDa@026<H051cW6<HLiaS68@PM
1b488BPU:B4X9BTM1b487@LQ21cW8@PLib487@L58@P02R4X9BTQ:2U:9BTYBRDY;F/YBRe[0RU:01XU
BBU::DXYBRE9:DXaS36/<H`e[C6/=Jda[3W>=JdicSG=>LhecCW>=LdicSG=>Lhe[CW>1CF]014icSF]
>LhecCW>=LdicS6<=JdaS3W>=JdecCF]?NlicSW^00DicP03?NlicSW^00<mk`06@Q0mkd8@?Nm243h?
0T8@00E2<48@AS4mkd8@00Hmk`0?>NhmkcW^>Lhe[CW><J`e[C6/=JdaS3F]<H`e[C6/008e[@0>>Lhe
cCW>>LhikSW>?Nle[CG==JdicSF]=Lde[@@aS006=Jda[3F]>LhmkcW>0Sg_00A243g_>Lhec@<icP0E
=JdicS6/=Jda[3F]<J`e[C6/=JdaS36/<H`]JbUZ;F/YJRe[:DXYJRE900<YBP079DTYBRE99BTQ:2DY
8BP00b4800XLib487>LQ21cW8@PLib487>LM1`8Li`0?6>HLiaSV7>LHiQcW6>HLiaSV7>LDaAS65<DH
aQC5008HaP037>LHaQcW008HaP055<DHaQS66<HDa@026<H031cW6<HLiaS67>LHaQcW6<HHiQC56<HD
a@8HaP0D7>LHaQS65<DHaQC56<HDYAS65:DHaQBU6<I[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]J
JeX016]J00<DaABU5<D011BU00@DaAS65<DHaP<Da@THaP045<DHaQC56<H25<D36<H45:D25<D01AS6
5<DDaABU5<D011S600HDYAC55:DHaQC56<H25<D02ABU5<DDaAS65<DHaQC56<HDa@056<H01AC56<HD
aAS65<D01AS600<DaAS65<D011S600<LiaSV7>L011S600@HiQcW6>HHiP<Q20049BTQ:2DY8BP38@P0
11cW7@LLiad71b4812DY00DUBBDY9DTYJRe[008YJPLYBP<aS003<J`e[CF]008e[@03=Lde[CW>00Li
cPPe[@06=Lde[CW>=LdicSG=0S6/1CF]2cW>0cg_00@n3d8@?Pln3`M2408n3`03@Q0mkcg_008mk`04
>Nhmkcg_?Nl3=Ld00cF]<J`e[@02<J`00c6<<J`e[@03=Jd2>Lh00cG=>LhicP02>Lh7=Jd4<H`0136/
=Jde[Cg_0cW^00=243h??Pl013W>00<ecCW>=Ld00cF]00Da[3F]=Jde[C6/00<aS0@]J`PYBP039DTY
BRDY00<U:@<Q20077@LLiad77>LM1acW7@L031cW00<HiQcW7>L01aS600@HiQcW6>HHiPLHaP076>HH
aQSV6<HHiQS66>H02QS600/DaAS65<DHaQBU5<DDYAC55:DDaABU00I[FP40005[FP0002i[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU6<HD
YAS65:DHaQC56<HDaAS65:DHaQBU6<HDa@DHaP075<DHaQC56<HDYAS65:D00aS61ABU01`HaQBU6<HD
YAS65:DHaQC56<HDaAS65:DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU6<HDa@<HaP0:5<DHaQC5
6<HDaAS65<DHiQS67>L26<H00aC56<HHaP036<H00aSV7>LHiP036<H05AC57>LHaQcW6<HQ21d78@PQ
:2DY8BPU:Ad78@PM1b487>LQ21cW8@PM1`058@P02R4X9BTQ:2U:9BTYBRDY;F/YBRe[12U:01PUBBU:
:DXYBS6<=JdaS3F]<J`e[C6/>Lhe[CW>=LdicSG=>LhecCW>=LdicSF]=Ld5=Jd04CW>=JdicSG=>Lhe
cCW><H`e[C6<=Lde[CW>=JdikSW>?Nl01CW>00<ikSW>?Nl00cg_00I243g_@Q0mkd8@?Pl2@Q001DHa
@Q12<3g_@Q001Sg_00dikSg_>NhicSF]>Lha[3F]<J`e[C6<=JdaS003=Jd00c6/>Lhec@02>Lh02Sg_
>LhikSF]>Lhe[CG==JdicSF]136<00He[C6/=JdicSg_>Lh2?Nl00d8@?NlicP02>Lh04SG=>Lhe[CW>
<J`e[C6/=Jda[3F]<J`e[C6<=JdaS2e[:FX]J`8YJP0=:DX]JbU::DXUBBU:9DTYBRE99BTQ:2DY8BP0
0b4800XLib487>LQ21cW8@PLib487>LQ208Li`0=6>HLiaSV7>LHiQcW6>HLiaSV7>LDaAS65<D00aS6
00@DaAcW6<HLi`@HaP035<DHaQS6008HaP0<7>LHaQcW6<HLiaS66>HHaQcW5<DHaQC50QS601@HiQS6
6<HDaAS65<DHaQBU6<HDYAS65:DHaV]JKg][FV]jJeY_N`000F]j000FJeX30004JeX00aC55:DDa@04
5:D01QC56<HDaAS65<DHaP8Da@PHaP035<DHaQC5008Da@@HaP@DY@036<HDaAC5008Da@035:DDaAS6
00<HaP065:DDaABU6<HDaAS60QC500HDYAC55<DHaQC56<H35<D56<H01AC56<HDaAS65<D01AS600<D
aAS65<D011S60acW11S60aSV00lLib488@PQ22DY8BPU:B4X8@PM1b487>LM1acW7@L01b4812DY00<U
BBDY9DT00bUZ00<]JbU::DX01BU:0c6<0SF]00Da[3F]=JdecCF]00PicPPe[@07=Lde[CW>=LdicSG=
=Jd00S6/13F]2cW>0cg_0ch?2D8@0Sh?1Sg_00PikSg_=LdicSG==Jda[3F]0S6/00<aS36/=Jd00cF]
00<ecCW>>Lh013W>1cF]136<0SF]00Da[3W^>NhmkcW^008n3`03@Q0icSW>008icP<ec@03=Jda[3F]
008e[@03<J`e[CF]00<aS0@]J`PYBP039DTYBRDY00<U:@<Q20077@LLiad77>LM1acW7@L03QcW00<H
iQS66<H01AS60aSV00<LiaS66<H01AS600LHiQS66>HHaQSV6<HHiP0:6<H02aC56<HDaAS65:DDaABU
5<DDYAC55:D01F]J0P000F]J0000<6mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQC56<HDa@<H
aP035<DHaQC500DHaP055<DHaQBU6<HDY@036<H08AC56<HHaQS65<DHaQC56<HDaAS65<DHaQC56<HD
aAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDa@096<H00aC56<HDa@046<H031cW6<HL
iaS67>LHaQSV6<HLiaS67>LHiP8Li`068@PLib487>LQ21cW1B4801<M1b489BTYBRDY9DTQ:2DY8BPY
BRE9:DXUBBe[:DX]JbU:;F/YBP05;F/2<H`02Bf;=Jda[3F]<J`icSF]>Lhec@03>Lh05CF]>Lhe[CW>
=LdicSG==Jda[3F]<J`icSG=>Lhe[CG==JdicSG=>Lhec@03>Lh02CG=?Nlmkcg_>NhmkcW^?NlikP03
?Nl00ch?@Q124005@Q001Sh?@Q12448`@Q16<@8mk`08>NhicSW>>LhecCg_>LhikP8icP08=LdicSF]
>Lhe[CG==JdicP8e[@0K<J`e[C6/>LhecCW>=JdicSF]>Lh]Jc6<;F/aS2e[<H`]Jc6<;H/aS2f;=Jda
S3F]>NhmkcW^008mk`04@Q0mkcW>=Ld3>Lh03CG=>Lhe[CW>=JdicS6/=Jda[3F]<J`e[C6/00<]J`0=
:FXYBRU::DXUBBU:9DTYBRE9:DXUBBDY8BP00RDY00dQ21d78@PQ224X8@PU:AcW8@PLib487>LM1`02
7>L031S67>LHaQcW6>HLiaS67>LHaQcW6>HLi`8HiP096<HLiaS67>LHiQcW6>HHaQC500THaP075:DH
aQBU7>LHaQSV5<D00aS600TDaAS65<DHaQC56<HDaAS65<D00aS600TDaAS6JeY[NV]JKg][FV]j0000
0Fmk000FJeX00`00JeY[FP04JeX25<D00aS65<DHaP026<H00aBU5<DDY@045<D96<H00aC56<HHaP03
6<H01aC56<HDaAC55:DDaABU00/HaP055<DHaQS66<HDa@056<H00aC56<HDa@056<H01aC56<HDaAS6
5<DHaQC500LHaP035<DHaQS600@HaP057>LHiQSV6>HLi`026>H46<H27>L021SV7@LLiad77>LM1acW
7@L78@P79BT00bE9:DXUB@02:DX01RUZ:DXYJRU::FXYBP@]J`<aS0@e[@<ec@DicP<ec@03>LhecCW>
00@e[@<icP@e[@03>LhecCW>00@icP@mk`03>NhmkcW^00@mk`i240<mk`PicP03=LdicSG=00/e[@8i
cP8ec@03=JdecCF]008]R`09<H`]Rbf;;H/aS2f;<H`]Rc6<00<a[0@mk`03@Q0n3ch?00@icP06=Ldi
cSG==Lde[CG=13F]00@a[3F]<J`e[@@]J`PYBP039DTYBRDY00<U:@LQ208M1`038@PM1acW008Li`<H
iP037>LHiQcW008Li`<HiP037>LHiQcW00@HaP046>HLiaSV7>L:6<H35<D=6<H00aC56<HHaP066<H6
JeX10001JeX0000^JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]J
JgY[FVmkJeY[NQBU6<HDYAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65<D36<H01AC56<HDaAS65<D0
1AS600LDaAS65:DHaQBU6<HDa@036<H07aC56<HHaQS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DH
aQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500THaP055<DHaQS66<HDa@026<H031cW6<HLiaS67>LH
aQcW6<HHiQS67>LHiP8Li`068@PLib487>LQ21cW1B4801<M1b489BTUBBDY:DXQ:2DY8BPYBRE9:DXU
BBe[:DX]JbU:;F/YBP05;F/02Bf;<H`aS3F]<J`e[C6/>Lhe[@03>Lh03CG=>Lhe[CW>=JdicSG=>Lhe
cCF]<J`e[C6/00<icP03=JdicSF]008ec@04>LhecCW>=Ld2>Lh01Sg_>NhmkcW^?NlikP<mk`05>Nhm
kd8@@Q0n3`05@Q001ch?@Q1244Ha@Q12<3W^008mk`07>LhecCW>>LhikSW>?Nl00SW>00PecCW>=Jde
cCF]>Lhe[CG=0SF]01/a[3F]<J`icSG=>Lhe[CW>=JdicRe[<H`]Jc6<;F/aS2e[<H`]Rc6<;H/e[C6<
=JdikSg_>Nh00Sg_00=243g_>Lh00SW>014ecCW>=LdicSF]>Lhe[CW><J`e[C6/=Jda[3F]<J`]JbUZ
008]J`<YBP079DTYBRE9:DXUBBU:9DT00bDY00<Q:2488@P00R4800TU:B488BPLib487>LQ21cW8@P0
0QcW01DHaQcW6<HLiaSV7>LHaQcW6<HLiaSV7>LHiQcW6<HHiQS67>LHiQcW6>H02aS600HDYAS65:DH
iQS67>L26<H02AC56<HDaAS65<DHaQC56<HDa@036<H02aC56<HDaAS6JeY_Nf]JJgY[FVmk00000F]j
000FJeX00`00JeY[FP04JeX011S65<DHaQC51aBU00<DaAS65<D00QC500<HaQC56<H00aS60QC51AS6
00<DYAC55:D011S611BU0aS611BU0aS611BU00LHaQC56<HDaAS65<DHaP035:D01AS65<DHaQC57>L0
0aSV0aBU00LHaQSV6<HHiQC56<HDa@046<H26>H021cW6>HHiQSV7>LHaQSV6<H46>H01acW6>HLiacW
7@LLiad700@Q20038BPU:B4X00@U:@03:DXUBBU:00<UB@DYBP07:FX]JbUZ<H`]Rc6<;H/00cF]00<e
cCW>=Ld04cW>00De[CG==JdicSG=00<icP8ec@04?Pm243h??Pl7@Q02@S001DHa@S12<48@@S0014Ha
148@00Pn3d8@@Q0mkcW^?NlikSg_0cW^00<mkcW^>Nh023W>00<ecCW>=Ld00SG=00<icSF]=Ld01CF]
00HaS2f;<H`]Rc6<;H/2<H`00c6/<H`a[003=Jd3?Nl013W^>LhicSW>13g_0cW>00@e[C6/=Jda[0@a
S005;H/]Jbf;:FX]J`02:FX7:DX29DT00bU:8BPU:@029BT78@P47>L01b487@LQ21cW7@LLiad7008L
i`036>HLiacW008Li`036>HHaQSV00lHaP036>HHaQSV00/HaP035<DDYAC500@HaP055<DHaQS66<HD
a@026<H6JeX10001JeX0000fKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeX006]JKg][FV]jJeY_NaC56<HDaAS65:DDaABU6<HDYAC55:DHaQBU6<HDYAS65:DHaQC56<HDaAS6
5:DHaQBU1AS600dDYAS66<HHaQC56<HDYAC55:DHaQC56<HDa@045:D36<H02a2T5:DDYABU5<DHaQC5
6<HDaAS65<D00aBU00<DaAS65<D00QS600TLiaS67>LDYAC55:DLiaS67>L00QS600DDaAS65<DHaQC5
008HaP0@7>LHaQcW6<HLiaS67>LHaQcW6<HLiaS67>LHiQcW6>H27>L00b487>LQ20028@P00ad79BTQ
:0059BT072E9:DXUBBU:9BTYBRDY:FXYBRe[:DX]JbUZ;F/]Rc6<;H/aS36/=Jda[3W>=LdicSG=?Nli
cSW^13W>01`ecCW>=LdicSG=>LhecCW>=LdicSG=>Lhe[CW>=LdicSG=>Lhe[CW>=Je243g_@Q0mkdHa
@Q12<0E24006AS5244Ha@Q16<D8@14Ha04M2<48@AS4n3d8@?Pm243W^?NlikSg_>LhmkcW>?NlicSg_
>LhmkcW>>NhecCW>=LdicSG=>LhecCW>=JdicSF]>Lhe[CW><J`e[C6/=Jd]Rc6<;H/aS2f;<H`]RcF]
<H`e[C6<=Jda[3F]>NhmkcW^?NlecCW>=LdmkcW^?NlikSW>=LdicS6/=Jda[3F]008aS004;H/aS2e[
<H`2;F/00bU:;F/YBP03:DX01RE9:DXYBRU:9BTYBP8U:@0:8BPU:B4X8BPQ22DY7@LQ21d78@P47>L0
1Qd78@PM1b487>LQ208Li`086>HLiaSV7>LHiQcW6<HLi`THaP036>HHaQcW00DHaP037>LHaQS6008H
aP035<DHaQC5008HaP087>LHaQSV5:DHaQBU6<HDa@<HaP0=5<DHaQC56<HDaAS6JeY[NV]JKg][FV]j
00000Fmk000FJeX00`00JeY[FP04JeX011S65<DHaQC51aBU0aC500@HaQC55<DDa@DHaP8Da@@HaP03
5:DDaABU00@HaP@DY@<HaP@DY@<HaP@DY@076<HDaAS65<DHaQC56<H00aBU00@HaQC56<HDa@8HiP03
7>LHiQBU008DY@076<HHiQS66>HDaAS65<D011S600DLiaSV6>HHiQcW008HiP036<HHiQS6008HiP03
7>LHiQcW008Li`046>HM1acW7@L48@P00b4X9BTQ:0049BT01BU:9DTYBRE9:DX00RE912U:00LYJRe[
:FXaS2f;<H`]R`03=Jd00cG=>Lhec@0=>Lh00cG=>LhicP03>Lh00cF]=Lde[@03>Lh3=Ld013W>?Pln
3ch?248@00=6<D8`@S000T8`00=2448`AS400dHa1T8@00@n3cg_>Nhmk`<ikP05?NlikSW^>Nhmk`08
>Lh013G=>LhecCW>0SG=00<e[CG==Jd013F]0c6<00<]Rc6<;H/00S6<00<a[36<<J`00cF]00LmkcW^
?NlmkcW>=LdicP04?Nl3>Lh013F]<J`e[C6/136<00<]Rbe[;H/00bUZ00<]JbU::DX01RU:0RE90RDY
00<Q:2DY8@P01R4811cW00PQ21d78@PLiad77>LM1aSV1QcW00<HiQS66>H03aS600<HiQS66>H02aS6
00<DaABU5<D01QS600DDaAS66<HHaQC500I[FP40005[FP0003I[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDYAS65:DDaABU6<HDYAS6
5:DHaQBU6<HDYAS65<DHaQC56<HDYAS65:D56<H00aBU6<HDa@036<H00aBU6<HDY@025<D00aS65<DD
Y@035:D00aS65<DHaP025:D02A2T5:DDaAS65<DHaQC56<HDa@035:D00aC56<HDa@026<H02AcW6<HL
iaBU6<HDYAcW6<HLi`026<H01AC56<HDaAS65<D00QS6010LiaS67>LHaQcW6<HLiaS67>LHaQcW6<HL
iaSV7>LHiP8Li`048@PLib487@L28@P00bDY8BPU:@049BT072E9:DXUBBU:9BTYBRDY;F/YBRUZ:DX]
JbUZ;F/]Rc6<;H/aS36/=Jda[3W>=LdicSG=>NhicSg_13W>01`ecCW>=LdicSG=>LhecCW>=LdicSG=
>Lhe[CW>=LdicSG=>Lhe[CW>=Je243g_@Q0mkd8`@Q16<@E24006AS5244Ha@Q16<D8@1DHa04M2448`
?Pm243h?@Q0ikSg_>NhmkcW>?NlicSg_>LhmkcW>>NhicSg_=LdicSG=>LhecCW>=LdicSF]>Lhe[CW>
=JdicS6/=Jda[3F];H/aS2f;<H`]Rc6<;H/e[C6<=JdaS3F]<J`e[CW^?NlikSg_=LdicSG=?NlikSg_
>NhicSG=>Lha[3F]<J`e[Bf;00<aS006;F/aS2e[;F/YBRe[1RU:00@UBBU:9BTYBP8U:@0:8BPU:B4X
9BTQ224X7@LQ21d78@P47>L01Qd78@PM1b487>LQ208Li`086>HLiaSV7>LHiQcW6<HLi`THaP037>LH
aQSV00DHaP047>LHaQS65<D36<H02AC56<HHaQSV6<HLiaBU6<HDY@036<H03aC56<HDaAS65<DHaQC5
6<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP046<HDaAS65<D36<H45:D26<H011C5
6<HDaAS60QC500<DYAC55:D00aS611BU1aS600DDaABU5<DDYAS6008Da@@DY@065<DHaQC55<DDYAC5
11BU00<DaAS66<H03aS600LDaAS65<DHaQC56<HDa@046<H00aSV6<HHiP046<H36>H47>L012487@LM
1ad70b4800<M1b487@L00R4800<Q:2489BT00bDY0bU:00PYJRe[:FX]JbU::FXYBRUZ0be[13F]00<i
kSW>>Nh013g_0SW>00<ecCW>>Lh00SW>00@mkcW^?NlikP<icP<ikP0;?NlicSW>>LhmkcW^?NlikSh?
?Nln3`04@Q000dIABU96D@04AS44@Q000dIAAS56D@04AS400d8`AS52<00;@Q000cW^?NlikP02>Nh0
1Sg_>NhikSW^?NlikPLicP<e[@03<J`aS36/00TaS006<J`aS36/>LhikSW>0SG=00@icSG==Lde[@@e
c@06>LhecCG==LdaS2f;0c6<0bf;00XaS2f;;H/YJRe[:FX]JbUZ;F/YJP@YBPLU:@<Q:0048@PM1b48
7@L47>L00b487@LM1`0>7>L36>H01AcW6>HLiaSV7>L02aS600<HiQS66>H011S600LDaAS65<DHiQS6
6>HDa@076<H35<D6JeX10001JeX0000TKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_NaC56<HDaAS65<DHaQC511BU00`HaQC56<HDaAS65:DHaQBU6<HD
YAS65<D26<H011C55:DHaQBU1aS601TDYAS65:DHaQBU6<HDYAS65:DDaABU6<HDYAS65:DHaQBU6<HD
YAS65:DHaQC56<HDa@036<H00aC56<HDa@076<H01aC56<HDaAS65<DHaQBU00HHaP0:7>LHaQSV6<HL
iaS67>LHaQcW6<H47>L00b487>LQ20028@P01ad78@PM1b487@LU:B48008U:@0A:DXU:BE99DTYBRE9
;F/YJRe[:FX]JbU:;F/YBRe[:FX]J`04=Jd00cW>?NlicP04?Nl033W>=LdicSG=>LhecCW>>NhmkcW^
?Nlec@8icP03?NlicSg_00@icP03>NhmkcW^008mk`0:@Q0mkd8@?Pm243h?BU96DDYB@S03AS401T8@
AS52448`AS5:DPI6<@05@S16<Ch?@Q0n3`05@Q0043h?@Q0n3cg_>LhmkcW>?NlikSg_>LhmkcW>?Nli
cSg_1CW>0cF]02XaS3F]<H`e[Bf;<H`]Rc6<;H/aS2f;=JdaS3F]<H`mkcW>?Nle[CW>=JdicSF]>Lhe
[CW>=JdicSF]>Lhe[CW>;H/aS2f;<H`]Jc6<;F/aS2e[<H`2;F/01bUZ;F/YJRe[:DX]JbE900<YBP06
9BTYBRDY9DTU:BU:0RDY00`Q22DY7@LQ21d78@PLiad77>LQ21cW8@P87>L03aSV7>LHiQcW6>HLiaSV
7>LHaQcW6<HLiaSV7>LHiP096<H011C57>LHaQcW0QS600DDaAS65<DHaQC5008HaP087>LHaQS65<DH
aQC56<HDa@<HaP095:DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00@HaQC56<HD
a@<HaP@DY@045<DHaQS66<H45<D00aBU5<DDY@036<H45:D76<H011C55:DDaABU0QC500<HaQBU5:D0
0QBU11C500<DYAC55:D00aBU0QS600<DaAS66<H03AS600@DaAS65<DHaP<Da@@HaP036>HHaQSV00@H
aP046>HLiaSV6>H37>L27@L00b487@LQ20028@P00ad78@PM1`028@P00b4X8@PU:@039BT3:DX022UZ
;F/YJRe[:DXYJRU::FX3;F/4=Jd00cW^>LhikP04?Nl00cG=>LhicP04>Lh013g_>NhmkcW^0cW>00@i
kSg_>NhikP<icP07?NlikSg_>Nhn3cg_?Pl0148@00=6DDYBAU4014Ha148@00=6DDHaAU4014Ha00=2
<4Ha@S002d8@0cW^00DmkcW^?NlikSg_00<ikPLicP<e[@03<J`aS36/00TaS005<J`aS36/>LhikP02
>Lh4=Ld013F]=LdecCW>0cG=00<icSG=<H`00S6<0bf;00TaS2f;;H/]Rc6<:FX]JbUZ;F/00bUZ12U:
1bDY00LQ:2DY8BPQ21d78@PM1`047>L27@L00b487>LLi`0<7>L00aSV7>LHiP026>H00acW6>HLi`0;
6<H00aSV6<HHiP046<H01QC56<HDaASV6<HHiP8HaP035<DHaQS600<HaP035<DHaQC500I[FP40005[
FP0002][NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][
FV]j4:@DYA2T5:DDaAS65<DHaQC56<HDaAS65<DHaP025<D01QBU6<HDYAC55:DHaP<DY@0Z6<HDaAS6
5<DHaQBU6<HDaAS65<DHaQBU6<HDYAC55:DHaQBU6<HDaAS65<DHaQBU6<HDYAC55:DHaQC56<HDaAS6
5<DHaQC56<HDYAS65<DHaQC51AS600TDYAS65:DHaQC56<HDaAS65<D00aS600lDaAS66<HHiQS67>LH
aQcW6<HLiaS66>HHaQcW6>H01AcW01DM1b488@PU:B489BTQ22DY8BPU:BE9:DXUBBU:9DTYBRE9;F/Y
BRe[:DX012e[00PaS2e[<H`e[CW>=JdecCW^0cg_00TikSg_?NlmkcW^?NlicSW^>Lh00Sg_010n3cg_
@Q0ikSg_>NhmkcW>?NlicSg_>Nhmkch?@Q0n3`92400@AS5244YBAS5:DTHaCW=:LTicBU9:LTYBCW=6
<DYBAS42BU803dicBU9:DTIABU96DDYBAS5:DTHaBU96<DYBAS5:DP05AS4014IAAS5:DTHa0d8@00Xm
kch??Nm243h?@Q0n3cg_>Lhmk`8icP06=LdicS6/=Jda[3F]0c6<1CF]00Da[3F]=JdicSG=008icP03
?NlicSg_00@e[@0@<J`e[C6/=Jda[3F];H/aS2f;<H`]Jc6<;F/aS2e[<H`2;F/032UZ;F/YJRe[:FX]
JbE9:DXUBBU:9BTUB@HU:@0<8@PU:Ad78@PM1b487@LQ21d78@PLib481QcW01LHiQcW6>HLiaSV7>LH
iQcW6>HLiaS67>LHaQcW6<HHiQS67>LHaQSV5<DHaQC500@HaP036>HHaQcW008HaP055:DHaQC56<HD
a@036<H021C56<HDaAS65<DLiaS67>L26<H02ABU6<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[
FV]J00A[FP@DY@036<HDaAS6008HaP065<DHaQS66<HDaAS62QBU11S600LDYAC55:DHaQC56<HDa@04
5:D00aC55:DDa@026<H01AC56<HDaAS65<D011BU00LHaQC56<HDaAS65<DHaP035<D00aS65<DHaP06
6<H25<D?6<H00aSV6<HHiP046<H00aSV7>LHiP047>L48@P01R4X8@PQ:24X9BTQ:0LYBP<YJPH]J`03
;H/]JcF]00<e[@Tmk`03>NhicSW>008icPDmk`05>NhmkcW>>NhicP02>Nh00cg_>Ni24006@Q0014Ha
AU56<DIA0dic14YB00M6DDHaAU56<DYbBU9:LP04BU84AU400dHaAU56<@0;AS43@Q04?Nl00d8@?Pm2
4003>Nh023g_=LdicSG==Jda[3F]<J`3<H`8=Jd3>Lh2>Nh00cg_>Nhe[@02=Jd01C6/=Jda[3F]<J`0
0SF]00<aS2f;<H`00Rf;00H]Jbf;;F/]Rbe[;H/2;F/00bUZ;F/]J`02;F/012U:9DTYBRE91bDY0b4X
224800<M1acW7@L021cW00XHiQcW6>HLiaSV7>LHaQSV6<HHiQ8HaP035:DDaABU00<HaP055<DHaQS6
6<HDa@046<H00aSV6<HHiP035<D6JeX10001JeX0000cKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_Na2T5:D@Y1BU5<DHaQC56<HDaAS65<DHaQC56<HD
aAS65:DDaABU6<HDYAC500<DY@0L6<HDaAS65<DHaQBU6<HDaAS65<DHaQBU5<DDYAS65:DHaQBU6<HD
aAS65<DHaQBU6<HDYAS65:D25<D031S65<DHaQC56<HDaAS65:DHaQC56<HDa@DHaP075:DHaQBU6<HD
aAS65<D00aS600<DaAS66<H00QS600`LiaS67>LHaQcW6<HHiQS67>LHaQcW6>H57>L28@P04ad79BTQ
22DY8@PU:B4X9BTUBBU:9DTYBRE9:DXUBBe[:DX]JbU:00@]J`07<H`]Jc6<=JdecCF]>Lh00Sg_01di
kSg_?NlmkcW^?NlikSg_>LhmkcW>>Nhmkd8@?Nln3cW^?NlikSg_>LhmkcW>?NlikSg_?Pm243h?0092
4006@S1244YBAS5:DTHa0dic00M:DTicBU9:LTHaBU96<@02BU803dicBU9:DTIABU96DDYBAS5:DTHa
BU96<DYBAS56D@05AS4014YBAS56DDHa0d8@00<mkd8@?Nl00Sh?00E243h??NlicSg_008icP06=Ldi
cS6/=Jda[3F]0c6<1cF]00<a[3W>=Ld00SW>00<mkcW>?Nl013F]010a[3F]<J`e[C6/=Jd]Rc6<;H/a
S2e[<H`]Jc6<;F/aS08]J`0<:FX]JbUZ;F/YJRe[9DTYBRE9:DXU:BU:1RDY00`Q22DY7@LQ21d78@PM
1b487@LQ21cW8@P67>L05aSV7>LHiQcW6>HLiaSV7>LHiQcW6<HLiaS66>HHaQcW6<HHiQS67>LDaAS6
5<D011S600<LiaS66>H00QS600LDYAS65<DHaQC56<HDa@036<H01QC56<HDaAcW6<HLi`8HaP095:DH
aV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J11BU0aS600DDaAS66<HHaQC500<HaPXD
Y@@HaP075:DDaABU6<HDaAS65<D011BU00@DaABU5<DDa@<HaP<Da@@DY@0:6<HDaAS65<DHaQC56<HD
aAS65<D36<H00aC56<HHaP026<H25<D@6<H00aSV6<HHiP046<H00aSV7>LHiP047>L48@P01R4X8@PQ
:24X9BTQ:0DYBP049DTYBRUZ;F/2:FX5;F/00bf;;F/e[@03=Jd7?Nl00cW^?Nlmk`04>Lh5?Nl01CW^
?NlicSW^>Lh00SW^00<mkcW^@Q001T8@00A6<DIAAS56D@=>L`A:DP07AU56<DIAAS5:LTYBBW8014YB
00I6DDYBAU56DDHaAU4<AS43@Q04?Nl01D8@?Pm243W^?Nl00SW^00LecCW>=Lde[C6/=Jda[003<H`8
=Jd3>Lh013g_>NhikSW^0cF]00<a[3F]<J`00cF]00@a[36<;H/aS08]R`07;F/]Rbe[;H/]Jbf;:FX0
1Re[00@YBRE9:DXUB@LU:@038BPU:B4X00PQ20037@LLiad700PLi`0:6>HLiaSV7>LHiQcW6<HHiQS6
6>HB6<H01ABU5<DDYAS65<D01aS600TDaAS66<HHiQS66>HDaAS65<D01V]J0@000F]J0000;f]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDYAS65:DH
aQBU6<HDYAS65:DHaQBU6<HDYAS65:DLiaS67>L011BU0QS600/DaABU5:DDYA2T6<HDaAS65:DHaQBU
008Da@0J6<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDa@8H
aP037>LHaQSV00@HaP055<DHaQC56<HDa@076<H00aC56<HHaP026<H00b487>LM1`047>L00aSV7>LH
iP027>L08R487>LQ22489BTQ22DY8@PU:B48:DXUBBU:9DTYBRE9:DXYJRe[:FXaS2e[<H`]Jc6<;F/a
S2e[=JdaS36/>LhmkcW>0SW^00DmkcW^?NlicSg_00PicP8mk`0E>Nhmkcg_?NlikSg_>LhikSW>AS52
<4HaBU9>LdYBCW=:LTicBW9:DTIA009:DP0:DYA>Le:DBU9>LdYBCW=:LTicBW82CW<01DjCCW=>LdYB
BW800TYB00=6DDYBAU4014YB00I>LdYBCW=:LTicBW87BU805DHaBU96<DYB@S16<D8`?NlikSg_>Nhm
kcW>?NlaS3F]<H`e[C6/=Jda[004<H`02cF]<H`e[C6<>LhecCW>=JdecCF]>Lh00SF]018a[3F]<H`e
[C6<=JdaS36/;F/aS2e[;H/]Jc6<;F/aS2e[<H`2;F/00bUZ;F/YJP03:DX01RE9:DXUBBU:9BTYBP8U
:@0>8BPU:B4X9BTQ22DY8@PU:B488BPLib487>LQ20@Li`096<HLiaS67>LHiQcW6>HLiaSV008Li`03
8@PLiacW008Li`0>6>HLiaS67>LHaQcW6>HLiaS67>LHaQcW6<HLi`8HaP0A5<DHaQS67>LHiQcW6<HH
iQS67>LDaAS65<DHaQC56<HDa@036<H02aC56<HDaAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00
JeY[FP04JeX00aC55:DDa@055:D02QC55:DDaABU5<DDYAC56<HHiQS611BU0aS611BU0aS611BU1AS6
0aC500<HaQC56<H00QS600<DaAS66<H00QS60QC500PHaQC55<DDaAS65<DHaQC56AS62acW00HQ21d7
7@LM1b4X8@P28BP01RDY8BPQ:2E9:DXUB@@YBP<]J`<]R`04<H`]Rbf;;H/4<H`4>Lh3?Nl013W^?Nli
kSW^1cW>1cg_13W>00Y2<4Ha@S1:LTYBBW9:DTicBW9>L`A:DP03CY=BU4jC009:LP03CW=:LTic009>
L`04BW9>LdicCW<4BU800dIABU9:DP07BU82CW<00dYbCW=:DP06BU800dIAAS56D@04AS403SW^?Nlm
kcg_>LhikSW><J`aS36/<H`e[C6/=Jd4<H`0136/=Jda[36/0SW>00<ecCF]=Jd01CF]00@aS36/<H`a
[0<aS0@]J`05<H`]Rbf;;H/aS002;H/4;F/4:DX01RE9:DXUBBE99BTUB@@U:@<Q:0DQ20<M1`<Li`04
6>HLiaSV6>H67>L011SV7@LM1b4811cW0aSV00DLiaSV7>LHiQcW00<HiPLHaP8Li`036>HHaQS600<H
aP055<DHaQC56<HDa@056<H00aC56<HHaP06JeX10001JeX0000FKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX300005V]JJgY[FVmk5:DHaQBU6<HDYAC55:DHaQBU6<HDYAS6
5:DHaQBU7>LHaQcW11BU00DDaAS66<HDYA2T008DY@0R6<HDaAS65:DDaABU6<HDaAS65<DHaQBU6<HD
YAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65<D26<H00aSV6<HLi`046<H00aC5
6<HDa@036<H01AC56<HHaQS65<D01QS600<M1acW8@P00QcW00DHiQcW7>LLiaSV008Li`0Y8@PLib48
8@PU:B489BTQ22DY8@PYBRE9:DXUBBU:9DTYBRUZ;F/YJS6<;F/aS2e[<H`]Jc6<;F/a[36<=JdicSW^
>LhmkcW^?NlikSg_>Lhmk`08>Lh01CW^?Nlmkcg_>Nh00Sg_00hikSW>?NlicTHa@S16<DYBCW=:DTic
BW9>LdYb0dYB00]6DE:DCW=BU4YBCW=:DTicBW9>LdYb009>L`05DYA>LdYbBU9>L`02BU801DIABU9:
DTYBAU400TYB00I:LTYBCW=:LTicBW87BU805DHaBU96<DYB@S16<D8`?NlikSg_>NhmkcW>?NlaS3F]
<H`e[C6/=Jda[004<H`033F]<H`e[C6<>LhecCW>=JdicSF]=Lda[0<e[@0@<H`e[C6<<J`aS3F];F/]
Rbe[<H`]Jc6<;F/aS2e[<H`2;F/00bUZ;F/YJP03:DX01RE9:DXUBBU:9BTYBP8U:@0>8BPU:B4X9BTQ
22DY8@PQ:2489BTLib487>LQ20@Li`096<HLiaS67>LHiQcW6>HLiaSV008Li`048@PLiacW6>H37>L0
31S67>LHaQcW6>HLiaS67>LHaQcW6<HHiP@HaP0A5<DLiaSV7>LHaQcW6<HHiQC56<HDaAS65<DHaQC5
6<HDa@036<H01aC56<I[FV]jJeY_Nf]J0080005_N`005V]J00<006]JJeX016]J11BU0aS611BU11S6
00XDaAS66<HDaABU5<DDYAC55:DDa@@HaP035<DHaQC500@DY@036<HDaAS6008Da@095:DDaABU5<DD
YAC55:DDaABU00THaP055<DHaQS66<HDa@046<H45:D46<H01ASV6<HHiQS66>H021S600<HiQcW6>H0
21cW1b4800<Q:2DY9BT012DY00@YBRE9:DXYBPL]J`@e[@<icP04?NlikSg_>Nh3>Lh4=Jd3>Lh013g_
>NhmkcW^1cg_00Ln3d8@?Pln3dYbCW=:LP03E[D015:dDYABU5:D2dic14YB0dHa1dYB1dic00=>Te:D
CY<01E:D0eJe00E>LdYbCW=>LdYB0096D@A6<@03>NhmkcW^00@e[@<aS0@e[@@aS007<J`e[CF]=Jda
[3F]<J`02C6<00<]Rbe[;F/02Be[00@YJRe[;F/]J`@YBP039DTYBRU:00<U:@DQ20038BPQ224800<Q
208M1`038@PM1acW008Li`<M1`038@PLiacW008Li`<HiP057>LHiQcW6>HLi`026>HE6<H00aSV6<HH
iP026<H00aC56<HDY@075:D36<H6JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDY@8Da@036<HDaABU00<DY@036<HDaAS6
008HaP0C5<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:D00QC501`HaQC56<HDYAS6
5:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDa@<HaP035:DHaQBU008D
a@096<HDaAcW6<HLiaS67>LHaQcW00DHaP066>HHaQcW6>HLiaSV0acW00<HiR487>L00Qd71R4800@U
:B4X9BTQ:0<U:@079DTYBRE9:DXYJRe[:FX012e[00@icSF]=Lde[@<icP07>NhmkcW^?NlicSg_>Lh0
13F]0cW>00<ikSg_>Nh01Sg_01@n3cg_@Q0mkd8@?Nm>LdYbCW=B]5JeD[AF]DjCDYA>TdicBW9>LdYb
1Dic00I:LTicBU9:DTIABU82AS401D8`BU9:DTYBAU400dYB00=:LTicBW8014ic00IBU4jCDYA>Te:D
CY<2DY@02EJeD[AF]DYbCW=:LTicAS5:DP02AS402d8`AS52<3g_>NhmkcF]>Lhe[CG=;H/00S6<00<e
[C6/=Jd00S6/00XaS3F]<H`e[C6/=Jda[3F]<J`e[@8aS009;H/aS36<<H`]Rc6<;H/aS2UZ00<]J`0:
:FX]JbUZ;F/YJRe[:FX]JbUZ;F/2:FX02BU:;F/UBBU:9DTYBR4X9BTQ:0048@P012DY8@PU:Ad70b48
00XLib487>LQ21cW8@PLib487>LQ208Li`0<6>HLiaS67>LHaQcW6>HLiaSV7>LHaQcW1AS600HLiaS6
6>HDaAS65<D56<H01AC56<HHaQS65<D00QS600HLiaS67>LDaAS65<D45:D03QS65:DDaABU6<HDaAS6
JeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FP@DY@<HaP@DY@HHaP8Da@065:DDaABU5<DD
YAC511S600<DaAS65<D011BU00<HaQC56<H00QC500TDYAC55:DDaABU5<DDYAC55:D01aS600DDaAS6
6<HHaQC500HHaP@DY@@HaP056>HHaQSV6<HHiP086<H00aSV7>LHiP087>L78@P29BT00b4X9BTU:@02
9BT3:DX00bE9;F/]J`05;F/4=Jd3>Lh013g_>NhmkcW^0cW>13F]0cW>00@mkcW^?NlikPLmk`<n3`06
@Q1:LTicBW9F]E:d0UJe0e:D2dic14YB0dHa1dYB1dic00=>Te:DCY<01E:D0eJe0dic00A:LTIAAU5:
DPA6<@03>NhmkcW^00@e[@<aS0@e[@@aS08e[@05<J`e[C6/=Jda[007<H`00bf;<H`aS00=;F/00bUZ
;F/YBP05:DX012E99BTU:BDY1B4800<Q:2488@P012480ad70acW00@M1b487@LM1`<Li`8HiP057>LH
iQcW6>HLi`036>H00acW6<HHaP0C6<H00aSV6<HHiP026<H00aC56<HDY@075:D36<H6JeX10001JeX0
000TKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_
NaBU5<DDYAS65<DHaQC511BU0aS6034DaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HD
YAC55:DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HD
a@036<H041C56<HDYAC55:DHaQC56<HDaAcW6<HLiaS67>LHaQcW1AS600PLiaS67>LHiQcW6>HLiaSV
0QcW00<M1acW8@P00R4800<M1b488@P00R4800@U:B4X9BTQ:0<U:@079DTYBRE9:DXYJRe[:FX012e[
00hecCF]>Lhe[CW>=LdicSW^?NlikSg_>LhikSW>13F]0cW>00<ikSg_>Nh01Sg_01A243g_@Q0mkd8@
?Nm>LdYbCW=B]5JeD[AF]DjCDYA>TdicBW9>LdYb0dic00E:LTicCW=>LdIA00=:DP05@S16<DHaBU96
D@05BU800dYbCW=:LP04CW<015:DCY=BU4jC0e:D00Y>TeJeD[AF]DYbCW=:LTicAS5:DP96<@0:@S16
<D8`?NlikSg_=JdecCF]>Lh2<H`04Bf;=Jda[3F]<J`e[C6<<J`aS3F]<J`e[C6/=Jda[3F];H/00c6<
00H]Rc6<<H`aS2f;<H`2;F/05bUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbUZ;F/YBRUZ9DTYBRE9:DXQ
:2DY8BP0124800<U:B489BT00R4800`M1b487>LQ21cW8@PLiad77>LQ21cW8@P27>L031SV7>LHaQcW
6<HLiaSV7>LHiQcW6<HLi`DHaP066>HHaQcW5<DHaQC51aS600<DaAS65<D00QS600HLiaS67>LDaAS6
5<D45:D03QC55:DHaQBU6<HDaAS6JeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FP<HaP04
5<DDYABU5:D35<D00aS65:DDY@025:D01AC56<HDaAS65<D00aS600@DaAS65<DHaP8Da@8HaP095<DH
aQC56<HDaAS65<DHaQC500<DY@<Da@PHaP<DY@/HaP@DY@@HaP036>HHaQSV00<HaP046>HHaQSV6<H4
7>L00ad77>LM1`047>L00ad78@PM1`047>L38@P0124X9BTU:BDY0bE900L]JbUZ;F/]Jbf;;F/]R`04
<H`8>Lh00cG=>Lhec@04>Lh00cW^>LhikP07>Lh4?Nl00ch??Nln3`04AS401dicCY=>Le[FE]EJeUKE
00=BU0A>L`05BW9>LdicCW=:LP02CW<00dIAAS56D@04AS45BU800dYbBU9>L`03CW<3E[D4F]H015[f
G_MJmUkg0f<H15Je00E>TdicCY=6DDYB0096D@03?Pm243h?00<icP08=Lde[CF]=Jda[3F]<J`e[@@a
S003;H/aS2f;00DaS003;H/aS36<00<aS0L]J`04;H/]Jbf;;F/3:DX012UZ;F/YJRe[0bU:1bDY00LQ
:2DY8BPU:B4X9BTQ:0048@P47>L00ad77>LM1`087>L01QSV7>LHiQcW6>HLi`XHaP035<DHaQS600DH
aP035<DHaQS600@HaP076>HHaQSV6<HHiQS66>H011S600<DaAS65<D00aS600<DaF]JJeX016]J0@00
0F]J0000AF]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_
Nf]JJgXDaAS65<DHaQBU6<HDYAS65:DHaQBU5<DDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65:DH
aQBU6<HDaAS65<DHaQC56<HDaAS65<DHaP035:D011S65:DHaQBU1AS600DDaAS65:DDaABU00DHaP03
5<DHaQC500<HaP035:DHaQBU008Da@0C6<HDaAcW6<HLiaS66>HHaQcW6<HLiaS67>LHiQcW6>HQ21cW
8@P011cW01DHiR487@LQ21cW8@PLiad78@PU:B489BTQ:2DY8BPYBRDY:DXYJRe[:FX00Re[00<aS2e[
<H`00S6<00D]RcW>>LhicSG=00<icP05=LdicSG=>Lhec@02>Lh00cg_>Lhmk`02>Lh01CG=>LhicSW^
>Lh00cg_010ikSg_?Nm243g_AS52<4Ha@S1BU4icDYAFeE[FE]EJeP=BU00BCW=:LTicBW9>LdYbCW=:
LTicBW9>LdHaBU96<DYBAS56DDHa0dYB02U6DDicBU9>LdYbCW=:LTicD[AF]E:dF_IJeUkgF]INme[f
G_MJmV<HGaMS65JeE]EF]E[FCW=BU4icBU96<DYBAS5243g_@Q0ecCW>=LdicS6/00<e[@03<J`e[C6/
00DaS003;H/aS36<00<aS00=;H/aS2f;<H`]Rc6<;H/aS2e[;H/YJRe[:FX00Re[00XaS2e[<H`YBRe[
:DX]JbUZ;F/YJP<YBP049BTUBBDY:DX29BT04R4X9BTQ:2DY8BPU:B489BTM1b487@LQ21cW8@PLiad7
7>LQ208Li`0D6>HLiacW7>LHiQcW6>HLiaSV7>LHiQcW6>HHaQC56<HDaAcW6<HHiP8HaP085<DHaQS6
7>LHaQSV6<HLi`8HaP055<DHaQC56<HDa@026<H01AcW6<HLiaS67>L01QS600dDaAS65<DHaQC56<I[
FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP046<HDaAS66<H35:D011C56<HDaAC511BU
00<DaAS65<D00aS600@DaAS65<DHaP<Da@036<HDaAS6008HaP075<DHaQC56<HDaAS65<D00aBU00@D
aAS65<DDa@LHaP<DY@/HaP@DY@@HaP036>HHaQSV00<HaP046>HHaQSV6<H47>L00ad77>LM1`047>L0
0ad78@PM1`047>L38@P29BT01B4X9BTUBBU:9DT00be[00@YJRf;;F/]R`@aS0PicP03=LdicSG=00@i
cP03>NhicSW^00LicP@mk`03?Plmkch?00A6<@07CW=>TdicF]IFeE[FE]D00e:D1Tic00Q:LTicCW=>
LdYbAU56<DIA14Ha1DYB00=:LTYBCW<00dic00=F]E:dE[D015[F00AJmUkgF_INm`=S60AF]@03CY=>
LdjC00=6D@06BU8n3ch??PlicSG=0SW>0cF]00@a[3F]<J`e[@@aS003;H/aS2f;00/aS0L]J`04;H/]
Jbf;;F/3:DX012UZ;F/YJRe[0bU:1bDY00@Q:2DY8BPU:@<Q:0@Q20@Li`037@LLiad700PLi`036>HL
iaSV00<Li`PHaP035<DHaQS600THaP035<DHaQS6008HaP076>HHaQSV6<HHiQS66>H011S600DDaAS6
5<DHaQC5008HaPI[FP40005[FP0005A_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FP00JeY_Nf]JJgY[FVmk5<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DH
aQBU5<DDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65:DDaABU6<HDaAS65<DHaQBU6<HDYAS65<DH
aQC56<HDaAS65<D36<H01AC56<HHaQBU4:@00QBU00LHaQBU6<HHaQSV6<HLi`026<H06AC57>LHaQSV
6<HLiaS67>LHaQcW6<HLiaS67>LHaQcW6>HLiaSV8@PLib487>LQ21d7008Q200N9BTQ22DY8@PU:B48
9BTQ:2DY8BPYBRDY:DXYJRe[:FX]JbUZ;F/YJS6/<H`e[C6<=Jda[3F]<J`icSG=0SW>00<ikSW>?Nl0
1cW>00HmkcW^?NlikSg_>Nh2?Nl0248@?Nln3d8`AS52<4YBAU42BU804TjCCW=BU4jCDYA>Te:DCW=B
U4icDYA>LdjCCW=BU4jCDYA:LP=>L`03AU5:DTYB00A:DP06CW=:LTicBW9>Tdic0U:D00=B]5:DE[D0
0U[F01MFeFLiHaQW>F<HKg][NVmkJgYg_G>LMke[NVmkJgY_NeJeF]IF]DicBW9>LdYb00=6<@04?Nm2
43g_@Q03>Lh013F]<J`e[C6/136<00He[C6<<J`]Rc6<;H/5<H`02Be[<H`]Jc6<;F/aS2U:;F/YBP03
;F/012UZ;F/YBRUZ0RU:00HUBBU:9DTYBRE9:DX89BT0424X9BTQ:2DY8@PU:Ad78@PM1b487>LQ21cW
8@PLiad71QcW00HHiQcW6<HLiaS66>H47>L041S67>LHaQcW6<HLiaS66>HHaQcW6<HHiQS67>LHaQSV
21S600TDaAS66<HHaQC56<HDaAS65<D00aS600dDaAS65<DHaQBU6<I[FV]jJeY_Nf]JJgX00001Kg/0
01I[FP03001[FV]J00A[FP076<HDaAS65<DHaQC56<H00aC500@HaQBU5<DDY@@Da@LDY@035<DHaQC5
00DHaP035<DHaQC500@DY@<HaP035:DDaABU008Da@036<HDaAS600XHaP<DY@8Da@036<HDaAS600/H
aP086>HHaQSV6<HHiQS66>HHaP8HiP077>LHiQcW7>LM1acW7@L00b4800@Q:2488BPQ208Q:0DU:@<U
B@07;F/YJRe[:FX]JbUZ;F/0136<13F]00<ecCW>=Ld02cW>00<ikSg_?Nl023g_00=6<D8`AS4014YB
0dic15:D0TjC00=BU4icCW<00Tic00=>Te:DCY<014ic1dYB00=:LTicBW8014ic15:D0e[F0f<h00=W
>FmkKg/00Vmk0g>l00A_Nf]jKg][NP=F]@04BW9>LdYbCW<3AS4013h??Nln3cg_0cW>00@a[3F]<J`e
[@haS003;H/aS2f;008]R`06<H`]RbUZ:FX]JbUZ12e[1bU:00<UBBU:9DT02BDY00@Q:2DY8@PQ:08Q
20067@LQ21d77@LLiad72acW11S60acW00PHiQcW6>HHiQS66>HHaQSV71S600LDaAS65<DHaQC56<HD
a@06JeX10001JeX00014JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX0
06]JJgY[FVmkJeY[NQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65:DHaQBU5<DDYAS65:DD
aABU6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDY@8Da@0<6<HDaAS65:DHaQBU6<HDaAS65<DHaQC5
0aS600DDaAS66<HHaQC500@DY@086<HDYAS66<HLiaS66>HDa@8HaP0H6>HHaQcW6<HLiaS67>LHaQcW
6<HLiaS67>LHaQcW6>HLiaSV8@PLib487>LQ21d70R4801hU:B489BTQ22DY8@PU:B4X9BTQ:2U:9BTY
BRUZ;F/YJRe[:FX]JbUZ=JdaS36/<H`e[C6/=Jda[3W>=Ld2>Lh00cg_>LhikP07>Lh01Sg_>NhmkcW^
?NlikP8mk`06?Plmkd8@@S16<D8`0dYB00=6DE:DCW<00TjC00eBU4jCDYA>Le:DCW=>TdicDYA>Le:D
CY=BU002CW<01DYbCW=:DTYBAU4014YB00a>LdYbCW=:LU:DCW=>Te:DE[EBU5:dE]D2F]H05VLiHaQW
>F<HKg][NVmkJgYg_G>LMke[NVmkJgY_NeJeE]EF]DicBW9>LdYb0dHa00@mkd8@?Nm240<icP04=Jda
[3F]<J`4<H`01S6/<H`e[Bf;<H`]R`DaS00;;F/aS2e[<H`]Jc6<:DX]JbU:;F/YJP03;F/02RU:;F/Y
BRU:9DTYBRE9:DXUBBU:22DY010Q:2DY8BPU:B489BTM1b487@LQ21cW8@PLib487>LQ20HLi`056>HL
iaS66>HHaP057>L041S67>LHaQcW6<HLiaS67>LHaQSV6<HLiaS66>HHaQcW1QS600TDaAS66<HHaQC5
6<HDaAS65<D00aS600lDaAS65<DHaQC56<HDYAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[
FP04JeX02AS65<DHaQC56<HDaAS65<DHaP025<D00aBU5<DDY@025<D00aS65<DDY@065:D00aC56<HD
a@056<H00aC56<HDa@045:D01QS65<DHaQBU5<DDY@8Da@036<HDaAS600XHaP<DY@046<HDaAC55<D<
6<H021SV6<HHiQS66>HHaQSV6<H26>H01acW6>HLiacW7@LLiad700<Q20058BPQ224X8@PU:@028BP4
9BT01bE9:DXUBBe[:FX]JbUZ00<]J`@aS0@e[@03=LdicSG=00/icP8mk`03>Nhmkcg_00Hmk`=6<@A:
DP=>L`EBU09>T`A>L`03CY=BU4jC00A>L`M:DP03BW9>LdYb00A>L`ABU0=JeP04HcQW>F<hHcP4Kg/0
1g>lMkec_6mkJgY_Nf]j00=F]@04BW9>LdYbCW<3AS4013h??Nln3cg_0cW>00@a[3F]<J`e[@haS0<]
R`05<H`]Rbf;;H/]J`03:FX4;F/7:DX00bE9:DXUB@079BT01R4X9BTU:BDY8@PQ:08Q20067@LQ21d7
7@LLiad72acW11S60acW0aSV00DLiaS66>HHaQSV01`HaP045<DHaQC56<H35<D6JeX10001JeX0000X
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU
5<DDYAS65<DHaQC56<HDaAS65<D45:D02QS65:DHaQBU6<HDYAS65:DHaQBU1AS601LDaAS65<DHaQC5
6<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC5008HaP097>LHaQS65<DHaQC56>HHaQcW
008HaP0A5<DHaQC56<HDaAS65<DHaQC57>LHaQSV6<HLiaS67>LHiP037>L01aS67>LHaR487@LQ21d7
00@Li`0:8@PLib488BPU:B4X9BTQ:2DY8BP3:DX01bE9;F/YJRe[:DX]JbU:008YJP0@;F/YJS6<;H/a
S2f;=Jda[3F]<J`icSF]>LhecCW>=Ld2>Lh00cW^>Lhmk`03?Nl06d8@?Nm243W^?NlikSg_?Pm243h?
@Q16DDYBAU5BU4icDYA>LeJeDYAF]E:dE[EB]5JeDYAB]002DY@00dicCY=>L`03CW<01dYbCW=:LTic
BW9>LdYb009>L`08DYA>Le:DCY=BU4jCD[ABU0=F]@05D[AF]EkgG_MJmP03JeX01FMIMkec_7NmLk`0
1goo00=JmUkgF_H014ic00=:DTHaBU800T8@00<n3d8@>Lh00cW>00HecCW>=Lde[C6<=Jd2<H`08be[
;H/]Jc6<;F/aS2f;<H`]Rbe[:FX]JbUZ;H/]Jc6<;H/aS2f;<H`YJRe[:FX]JbUZ;F/YJRU:9DTYBRE9
;F/YBRUZ8BP00bDY00HQ:2DY8BPU:B489BT28@P03Ad78@PLiad77>LQ21SV7>LHiQcW6>HLiaSV00<L
i`0:6>HLiaS67>LHiQcW6>HLiaS67>L56<H01QcW6>HLiaSV6<HDa@<HaP055<DHaQC56<HDa@036<H0
0aC56<HHaP056<H05AC56<HHaQS65:DHaQBU6<HDYAS65:DHaQC56<I[FV]jJeY_Nf]JJgX00001Kg/0
01I[FP03001[FV]J00A[FP@DY@LHaPDDY@035<DDYAS600@Da@035:DDaAS600LHaP035<DHaQC5008D
a@036<HDaAS6008HaP0<5<DHaQS66<HDaAS65<DHaQC56<HDaAcW0QSV0QS600<DaAS66<H01aS600<D
aAS65<D01aS60aSV11cW0aS600@M1b487@LQ20DLi`067@LLibDY8BPU:B4X0bDY12U:00<YJRe[;F/0
12U:0be[00@]Rc6<;H/aS08e[@09<J`e[CG=>LhecCW>=LdicSG=00<icPDmk`0H?Plmkcg_>NhmkcW^
@Q0n3d8@?Pm:DTIABU9>LdjCCW=>Te:dE[EB]5JeD[AF]E:d0e:D24ic00E:LTicCW=>LdYb009>L`03
DYA>TdjC009BU003CY=BU5:D009BU0AF]@=Nm`A[FP04Lkag_G>lMkd7Ool00ekgF_INm`04CW<00dHa
AU56<@04@Q03>Lh023G=>LhicSW><H`a[36<<J`3;F/012f;;F/]Rbe[0c6<1be[136<1be[00<UBBU:
9DT012U:1BDY00HQ:2DY8@PQ:2488BP38@P77>L00aSV7>LLi`057>L36>H00acW6>HLi`026>H00aS6
6>HHaP036<H27>L00aSV7>LHaP0G6<H021C56<HDaAC55:DDaABU5<D36<H6JeX10001JeX0000PJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HD
Y@8Da@066<HDaAS65<DHaQC511BU00XHaQBU6<HDYAS65:DHaQBU6<HDY@LHaP0E5<DHaQC56<HDYAS6
5:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC5008HaP0:7>LHaQS65<DHaQC57>LHaQSV5<D36<H0
3AC56<HDaAS65<DHaQC56>HHaQcW6<HLiaS600<Li`096>HLiaS66>HHaR487@LQ21d700@Li`0<8@PL
ib488BPU:B4X9BTQ:2DY8BPYBRE90RU:01P]JbUZ;F/YBRUZ:DX]JbUZ;F/YJS6<;H/aS2f;=Jda[3F]
<J`icSF]>LhecCW>=Ld2>Lh00cg_>Lhmk`03?Nl07d8@?Nm243W^?NlikSg_?Pm243h?@Q16DDYBAU5B
U4icDYA>LeJeDYAF]E:dE[EB]5JeDYAF]E:DCY=>Le:D009>L`09BW9>LdYbCW=:LTicBW9>LdYb009>
L`08DYA>Le:DCY=BU4jCE[EBU09B]0=F]@05F_INmekgJeYWF@02JeX017NmLkag_G>l1goo00=JmUkg
F_H014ic00A:DTHaBU8n3`=240@icP0G=LdicSG==JdaS3F]<H`]Rbe[<H`]Jc6<;F/aS2f;<H`]Rbe[
:FX]JbUZ<H`]J`02;H/04C6<;H/aS2UZ;F/YJRe[:FX]JbUZ:DXUBBU:9DTYJRU:;F/00RDY00PQ:2DY
8BPU:B4X9BTQ22DY124800dLib487>LM1aSV7>LHiQcW6>HLiaSV7>LHiP037>L021S67>LHiQcW6>HL
iaS67>L56<H021cW6>HLiaSV6<HDaAS65<D36<H01AC56<HDaAS65<D02aS601<DaAS65:DHaQBU6<HD
YAS65:DHaQC56<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP045<DDYAC55:D36<H5
5:D00aC55:DDa@026<H00aC55:DDY@025:D02AS65<DHaQBU5<DDYAC56<HHiP026<H01aC56<HDaAS6
5<DHaQC500<HaP045:DDaABU5<D76<H011C55:DDaAC52QS600<HiQS66>H03aS60QSV1AcW0ad700<Q
21d78@P01B4800<U:B4X9BT012U:00<UBBU:9DT00RUZ00<]JbUZ;F/00Re[0c6/1CF]00@ikSg_>Nhm
k`<ikP<mk`A240Lmk`A240=:DPABU0=F]@07F]IFeE[FE]EF]E:dE[D01dic0TYb00=>LdYbDY@00e:D
00=B]5:DD[@015Je00=FeE[FE]D015Je0e[F16Li16mk0gNm16]J0e[F14ic00=2<4HaAS40148@00=6
<D8`AS4013g_00da[3F]<J`a[2e[;H/]Jc6<;H/aS2f;;F/YJP05;F/01Bf;<H`]RcF]<J`01cF]00<a
[3F];F/00be[00HU:BE99BTUBBDY9DT49BT48@P01Qd78@PQ22487@LQ208M1`037>LM1acW00DLi`04
6>HLiaSV7>L36>H01QcW6>HLiaSV7>LHiP<Li`hHaP075<DHaQC56<HDYAC55:D01AS600<DaAS66<H0
0aS611BU00<DaAS66<H01V]J0@000F]J000096mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:DHaQC56<HDa@@DY@076<HDYAS65:DHaQC56<H0
11BU02TDaAS65<DHaQBU6<HDYAcW6<HLiaC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DH
aQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC500<HaP065<DHaQS67>LHaQcW0aS6010LiaS66>HHaQcW
6<HHiQS67>LHaQSV6<HLiaS67>LHiP8Li`068@PLib487>LQ21d70R4800HQ:2489BTQ:2DY8BP5:DX0
5bE9:DXYBRe[:DX]JbUZ;F/YJSF]<H`e[C6<=Lde[CW>=JdmkcW^?NlicSg_>Lh00Sg_00<n3cg_@Q00
0d8@1cg_00]244Ha@Q12<4IABU96DE:DCY=BU4jC00=F]@07E]EJeUKEF]IB]5JeD[@00dic00]:LTic
BW9>LdYBCW=:DTicCY=BU4jC009BU009E[EBU5JeD[AF]E:dF]IFeE[F009F]@05D[AF]EKEF]IFe@03
IcT026<hKg][NVmkJgYkgWNmMmd4JeX3F]H01djCCW=BU4icAS52<4Ha0092400<?Pm2448`AS52<3g_
>NhmkcW^=JdaS3F]0S6<00T]Jc6<;H/aS2f;<H`YJRe[:FX012e[00HaS2f;<H`a[3F]<J`3=Jd04S6/
=Jda[3F]<J`aS2e[;H/]JbU:9BTYBRDY:DXU:BU:8BPU:@8Q:00>8@PU:B488@PM1b487@LQ21d78@PL
ib487>LQ20@Li`0G6>HLiaSV7>LHaQcW6<HLiaS67>LHiQcW6>HLiaSV7>LHiQS65<DHaQC56<HDa@03
6<H011C56<HHaQSV0QS600LDaAS65<DHaQBU6<HDa@036<H01QC56<HDaAS65<DHaP8Da@0=5:DHaQBU
6<HDaAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX011C55:DDaABU0aS61ABU00@D
aABU5<DDa@8HaP@DY@096<HDaAS65:DDaABU5<DHaQSV008HaP075<DHaQC56<HDaAS65<D00aS600@D
YAC55:DDa@LHaP035<DDYAC5008HaP035<DHaQS600HHaP036>HHaQSV00lHaP037>LHiQSV00@Li`04
7@LQ21d77@L28@P00ad78@PQ20028@P00bDY8BPU:@04:DX012E9:DXUBBe[0bUZ0be[00@a[3F]<J`a
[0@e[@03>NhmkcW^008ikP03?NlikSg_008mk`A240Lmk`A24003BU96DDYB00ABU0=F]@07F]IFeE[F
E]EF]E:dE[D024ic0dYb15:D00=B]5:DD[@015Je00=FeE[FE]D015Je0e[F16Li16mk0gNm16]J0e[F
14ic0THa00=2<48@@Q000T8@00=6<D8`AS4013g_0c6/00Xe[Be[;H/]Jc6<;H/aS2f;;F/YJPD]J`03
;H/aS2f;00<e[@03<J`e[CF]00De[@@]J`069BTUBBDY9DTU:BE912DY1R4800@M1b487@LQ208M1`03
7>LM1acW00<Li`046>HLiacW7>L36>H01QcW6>HHiQSV7>LHiP<Li`036>HLiaS600dHaP075<DHaQC5
6<HDYAC55:D01AS600<DaAS66<H00aS611BU0QS600<DaF]JJeX016]J0@000F]J00005V]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J0`0000][FVmkJeY[NQBU6<HDYAS65<DH
aQC500@DY@076<HDYAS65:DHaQC56<H011BU02/DaAS65<DHaQBU6<HDYAcW6<HLiaC56<HDaAS65<DH
aQC56<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDa@046<H0
0acW6<HLi`036<H03QSV6<HLiaS66>HHaQcW6<HHiQS67>LHaQcW6<H37>L01aSV8@PLib487>LQ21d7
008Q20069BTQ224X8BPU:B4X1BU:01LUBBU::DX]JbU:;F/YJRe[:FXe[C6<=JdaS3W>=JdecCF]?Nli
kSg_>LhmkcW>008mk`03@Q0mkd8@00=240Lmk`0;@Q12<48@AS56DDYBAU5BU4jCDYA>T`03E[D02EKE
F]IFeE[FD[AF]E:dCW=:LP03CW<02DYbCW=:DTicBU9>LdjCDYA>T`02DY@02UJeDYAF]E:dE[EB]5[F
E]EJeU:d0eJe00EFeE[FE]EW>F<h009W>@07Kg][NVmkJgYggGNmNmh016]J0e[F00QBU4icCY=>LdHa
@S16<Ch?0d8@00Y2<4Ha@S0mkcW^?NlikSF]<H`e[@8aS009;F/aS2f;<H`]Rc6<:FX]JbUZ00@]J`08
<H`]Rc6<<J`e[C6/=Jda[0<e[@0D<J`e[C6/;H/]Jc6<;F/YBRDY:DXU:BU:9BTYBR4X9BTQ:2DY8@PQ
:08Q200:7@LQ21d78@PM1b487>LQ21cW8@P47>L06ASV7>LHiQcW6<HLiaS67>LHaQcW6>HLiaSV7>LH
iQcW6>HHaQC56<HDaAS65<DHaQC500@HaP087>LHaQS65<DHaQC56<HDY@<HaP0E5<DHaQC56<HDaAS6
5<DHaQC56<HDYAC55:DHaQC56<I[FVmkJeY[NV]J0080005[NP005V]J00<006]JJeX016]J00@HaQC5
6<HDa@<HaP075<DHaQC56<HDYAC55:D00QC500HHaQC56<HDaAS65<D35:D00aC56<HHaP046<H011C5
5:DDaABU1AS600DDaAS65<DHaQC500DHaP035<DHaQC5008DaA4HaP055<DHaQS66>HHaP026>H00acW
6>HHaP036<H37>L01AS66>HHaQSV7@L00b4800<Q:2DY8BP0124800<UBBDY9DT012DY0bU:12e[1c6<
1cW>00A243h??Pln3`M24007?Pm243h?@Q0n3d8@?Pl014Ha0dYB15:D0e[F00=FeEJeE]D00UJe00=B
]5JeCW<00dic0dYB0TjC00=BU4jCDY@00e:D0eJe00UJmUkgG_MNme[fG_MJmU[FE]D01E[F0Ukg00=J
mUkgHaP00f<H0fLi15kg00=F]E:dE[D014ic14YB0dIA00=6<D8`AS4013g_13F]00T]Rc6<;H/]Rbe[
;H/]Jbf;;F/012f;00@aS2UZ;F/]J`PaS003=Lde[CG=008]J`05:FX]JbU::DXUB@059BT01R4X9BTQ
224X8@PQ:0<Q20037@LLiad7008Li`096>HLiacW7>LHiQcW6>HLiaSV00<Li`036>HLiacW00<Li`05
6>HLiaS66>HHaP046>H00acW6<HHaP056<H01QSV6<HHiQC56<HDa@<HaP035<DHaQS600DHaP075<DH
aQS66<HDYAC55:D00QC500<HaQC5JeX01F]J0@000F]J000086mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DaAS65<D36<H0:AC56<HDaAS65<DHaQBU
6<HDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC5
6<HDaAS65<D00aS600LDaAS65<DHaQBU6<HDY@056<H01AC56<HDaAS65<D01AS600dDaAS65<DLiaS6
7>LHaQcW6>HLiaS67>LHaP026>H37>L03AS67>LHaR487@LQ21d79BTQ:2DY8@PQ:248008U:@03:DXU
:BDY00<U:@<YBP04;F/aS2e[;H/4<H`00bf;<H`aS004>Lh01SW^>Lhmkcg_@Q0mk`U24006?Nm243h?
@Q0n3d8@14Ha00E6DDYBAU5BU4jC009BU00FF]IFeE[FE[EJeUJeF]IB]5JeD[A>LdYbCW=:LTicBU9:
LTicDYA>Le:DCY<3DY@06E:dE[EB]5kgF_INme[fG_MJmUkgE]EJeUKEF]IFeE[FE]ENme[fG_MJmVLi
HaQS>6<H00=W>@0EF_INme[fG_MB]5JeD[A>TdicDYA>LdYbBU9>LdHaBU96<DYB@S16<D8`00@mk`0A
>Lhe[CG==JdaS2f;<H`]Jc6<;F/aS2e[<H`]Jc6<;F/aS002;F/01RUZ;F/aS36/<H`e[@@aS00D=Jdi
cSF];F/YJRe[:FXYBRE9:DXU:BU:9BTUBB4X9BTQ:2DY8@PU:@8Q200C7@LQ21cW8@PLib486>HLiaSV
7>LHiQcW6>HLiaSV7>LHiQcW6>H01AcW014HiQcW6>HLiaS67>LHaQcW6<HLiaS66>HHaQcW5<DHaQC5
008HaP087>LHaQS65<DHaQC56<HDa@DHaP0E5<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaV]JJgY[
FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00@HaQC56<HDa@<HaP075<DHaQC56<HDYAC55:D0
11C500@HaQC56<HDa@<DY@8HaP035<DHaQS6008HaP045<DDYAC55:D76<H00aC56<HDa@056<H011C5
6<HDaAS60QC541S600DDaAS66<HHiQS6008HiP037>LHiQS600<HaP<Li`046<HHiQS66>H28@P01Ad7
8@PQ:2DY8BP0124800<UBBDY9DT012DY0bU:12e[1c6<1cW>0Sh?00=243h?@Q001T8@0ch?00A243h?
@Q0n3`A6<@=:DPABU0=JeP03E]EF]EKE009F]@03D[AF]Dic00=>L`=:DP04DYA>TdjCCY<4DY@00eJe
D[AF]@02G_L01E[fG_MJmUkgF_H00e[F00EFeE[FF]IJeU[f00=Nm`AS60=W>@ANm`03E[EB]5Je00A>
L`=:DP96D@05BU96DDHa@S16<@04?Nl4=Jd02Bf;<H`]Rbf;;F/]Rbe[;H/]J`02;H/00c6<;H/]R`02
;F/00bUZ<H`aS006<H`013G==JdecBUZ0be[00<UBBU::DX01BDY00HQ:2DY8@PQ:2488BP38@P00ad7
7>LM1`047>L02ASV7>LHiQcW6>HLiaSV7>LHiP077>L01ASV7>LHaQSV6<H00QSV00<LiaSV6>H01aS6
00PHiQS66>HDaAS65<DHaQC51aS600TDaAS66<HHaQC56<HDYAC55:D00QC500<HaQC5JeX01F]J0@00
0F]J000086]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_
Nf]JJgXDYAS65:D25<D36<H081BU6<HDYAS65:DHaQBU6<HDYAS65:DDaABU6<HDaAS65<DHaQBU5<DD
YAS65<DHaQBU6<HDYAS65:DHaQBU6<H35:D011S65<DHaQC50aS600XDYAS65:DHaQC56<HDaAcW6<HH
iP8HaP035:DHaQS600<HaP035<DHaQC5008HaP0L7>LHaQSV6<HLiaS67>LHaQcW6>HLiaSV7>LHiQcW
6>HLiaSV7>LHiRDY8@PU:B489BTQ22DY8BP49BT03BU:9BTYBRE9:DXUBBU:9DTYBRUZ;F/YJRe[00<a
S00:=Jda[3F]<J`icSF]>Lhe[Cg_>Nh2?Nl0248@?Nm243h?@Q0n3d8@?Pl2@Q000dHa@Q16<@02@Q00
7Sh?@Q12<4Ha@S16<DYbCW=:LU:DCY=BU4jCF]IFeE[FE[EJeUJeE]E>Te:DCY=>LdYBCW=:DTYbBU9>
L`9BU003CY=BU5Je00=F]@03F]INme[F00AS60MNm`05F_INme[fG_MJmP02G_L03F<HGaMS65lGHcQS
66LiE]EJeUKEF]I>LdjC009>L`03BW9>LdYb00=>L`05BW9>LdYbCW=2<003AS402D8`AS52<3W>=Ldi
cSG=<H`]R`05<H`05bf;<H`]Rc6<;F/aS2e[<H`]Jc6<;H/aS2f;<H`]Rc6<;H/aS2UZ;F/YJRU:9DT0
0bU:00<U:BU:9BT00bDY1R4800PM1b487@LQ21cW8@PLib4821cW00THiQcW6>HLiaSV7>LHiQcW6>H0
0acW00`HiQcW6<HHiQS67>LHaQSV6<HLiaS67>L46<H00aBU6<HDY@045:D071S65<DHaQBU6<HDYAS6
5<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04
JeX45:D36<H02QBU5<DDYAC55:DDaABU5<DDYAC51ABU00<HaQC56<H011BU00@DaAS65<DHaP<Da@04
6<HDaAC55<D35:D26<H00aC56<HHaP026<H25<D00aS65<DHaP066<H35<D?6<H02ASV7>LHiQcW6>HL
iaSV7>LHiP057>L38BP012DY8BPQ:24X1bDY0RU:00<UBBU::DX00RU:12e[0c6<00<a[3F]<J`00SF]
00@ecCF]=LdikP8mk`04@Q0n3ch??Pl8@Q000d8`@Q124003@Q03AS40148`CW=:LTic15:D0e[F15Je
00MBU4jCDYA:DTYbBU9:LP03BU84DY@4E[D2F_H00ekgHaQS6002HaP8G_L00e[fG_MNm`03G_L7HaP3
F]H00eKECW=>L`09CW<00dYbCW=:LP07AS44>Lh2<H`00bf;<H`aS005<H`3;H/01S6<;H/]Rbf;<H`]
R`HaS003;F/YJRe[00@YBP039BTUBBDY00@U:@XQ20037@LLiad700PLi`036>HLiaSV00LLi`046>HL
iacW7>L96<H00aSV6<HHaP036<H00aC55:DDa@045:D00aC56<HDa@025<D031S65<DHaQC56<HDaAS6
5<DHaQBU5<DDY@8Da@8HaPI[FP40005[FP00021_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DDaABU0aS600lDaAS65:DHaQBU6<HDYAS65:DHaQBU
6<HDYAS65:D00QC5014HaQC55<DDYAS65:DHaQC56<HDYAS65:DHaQBU6<HDYAS600<DY@046<HDaAS6
5<D36<H02QBU6<HDYAS65<DHaQC56>HHaQcW0QS600<DYAS66<H00aS600<DaAS65<D00QS601XHiQS6
7>LHaQSV6<HLiaS67>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV9BTQ22DY8@PU:B480bDY00TQ:2DY9BTU
BBDY:DXUBBU:9DT00bU:00@YJRe[:FX]J`<aS00:=Jda[3F]<J`icSF]>Lhe[Cg_>Nh2?Nl01T8@?Nm2
43h?@Q0n3`=24005?Pm6<D8@AS4n3`03@Q007D8`AS52<4HaBW9>LdYbDYA>Te:DCY=JeUKEF]IF]EKE
E[EJeTjCDYA>TdicBU9>LdYBCW=:DTYbCY<00e:D15Je00=JeUkgF]H016<H1ekg00=JmUkgF_H00ekg
00iJmV<HGaMS65lGIcUS66<hE]EJeUKEF]I>Le:D0Tic00Y:LTicBW9>LdYbCW=:LTicBW9>L`96<@0;
@S16<D8`AS52<3W>=LdicSG=<H`]R`05<H`05Bf;<H`]Rc6<;F/aS2e[<H`]Jc6<;H/aS2f;<H`]Rc6<
;H/aS2UZ;F/YJP03:DX012E9:DXU:BU:12DY124800XM1b488@PQ21d78@PLib487>LQ20PLi`0;6>HL
iaSV7>LHiQcW6>HLiaSV7>LHiP027>L02aSV6<HLiaS66>HHaQcW6<HLiaS67>L011S600<DYAS65:D0
11BU01`HaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQC56<I[FV]jJeY_Nf]JJgX0
005_N`005V]J00<006]JJeX016]J11BU0aS600XDYAC55:DDaABU5<DDYAC55:DDa@DDY@036<HDaAS6
00@DY@035<DHaQC5008Da@066<HDaAC55<DHaQC50aBU00<DaAS66<H01AS60aC51aS600<DaAS65<D0
3aS60aSV00HLiaSV7>LHiQcW6>H57>L00b4X9BTQ:0028BP00bDY8BPU:@069BT00bE9:DXYBP04:DX4
;F/3<H`00c6/=Jda[002=Jd00cG==Jdec@02?Nl01CW^?Pln3d8@?Pl0248@00=2<48@@Q000d8@00M6
<D8`AS56<DicBW9>L`04DY@3F]H4E[D01e:DCY=BU4YBBW9:DTYb00=:DPABU0AF]@03G_MJmU[f00AS
60iNm`MS6004F]IFeE[FF]H;CW<00dYbCW=:LP07AS44>Lh00bf;<H`aS007<H`00bf;<H`]R`02;H/0
0c6<;H/aS002<H`00bf;<H`aS002<H`3;F/4:DX00bDY9DTU:@049BT:8@P00ad77>LM1`087>L00aSV
7>LHiP057>L00aSV7>LLi`037>L96<H00aSV6<HHaP036<H00aC55:DDa@045:D011C56<HDaAS60aC5
00/HaQC56<HDaAS65<DHaQBU5<DDYAC5008HaP035<E[FV]J00A[FP40005[FP00049[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDYAS6
5:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQC56<HDaAS6
5<D36<H02AC56<HDaAS65<DHaQC56<HDa@026<H02ASV6<HLiaC56<HDaAcW6<HLi`056<H01acW6<HL
iaS67>LHaQcW00LHaP067>LHiQcW6>HLiaSV0QcW00`M1acW8@PLib487>LU:B489BTQ22DY8BP29BT0
8RU:9BTYBRE9:DXUBBU:9BTYBRDY;F/YJRe[:FX]JbUZ;F/]Rc6<;H/e[C6<=JdaS3W>=LdicSG=@Q0m
kd8@?Pm243h?0T8@00E6<D8@AS5244Ha00924005?Pm2448@AS524002AS400dIAAS5:DP02CW<034Yb
E[EB]5JeD[ANme[fG_MB]5JeD[AF]@A>L`0=BU9>LdYBDYA>Le:DD[AF]E:dE[EJmUkgF_H00ekg00AJ
mV<HG_MS60ANm`03E[EJeUJe00EJeP0AE]EJeUKEF_IJeUkgF]IO5ekgHaQFeE[FE]EJeTjCDYA>T`03
CW<014YbCW=:DTic14YB00=6DDYBAU40148@13F]00<aS2f;<H`01C6<00<a[36<<H`01S6<00X]Jc6<
;F/]RbUZ;F/YJRe[:DX]J`8YBP0<9DTYBRE99BTQ:2DY8BPU:B4X9BTQ224X1R4800HM1b487@LQ21d7
8@P47>L05ASV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHaQcW6<HLiaSV7>LHiP046<H0:QcW6<HH
iQS67>LHaQSV5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DDaABU6<HDYAS65:DH
aQC56<I[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00DHaQC56<HDaAS600<Da@066<HD
aAC55:DDaABU0QC500HHaQC56<HDaAS65<D36<H35<D021S65<DDaAC56<HDaAS65<D56<H00aC56<HH
aP036<H00aC56<HDa@056<H01QC56<HHaQSV6<HHiP<HaP037>LHiQSV008HiP036<HHiQS600LHaP8L
i`056>HLiacW7>LHiP047>L00ad77>LM1`038BP49BT052E99BTUBBDY:DXUBBU:9BTUBBDY9DTYJRe[
:FX]JbUZ;F/YJS6<;H/2<H`0136/<H`a[3G=0cW>0ch?148@00E6<D8`@S12<4Ha0092<0=24092<003
AS52<4Ha00=6<@=>L`0;D[AF]EJeE[EJmUkgF_IF]E:dE[EB]003CW<02dYbCW=:LTYbCY=BU4jCE[EB
]5JeD[@025kg00=O5ekgGaL00ekg00AFeEJeE]EF]@=JeP03E]EJeUKE00EJeP=Nm`AJeP=BU0A>L`03
BU9:LTYB00E:DP03AU5:DT8@00=240@e[A4aS0D]J`03:FX]Jbe[00DYBP059DTYBRDY9BTQ:0049BT;
8@P00ad78@PQ20077>L00aSV7>LHiP057>L00aSV7>LHiP026>H00acW6>HLi`027>L<6<H00aC56<HH
aP0:6<H00aC55:DDa@045:D35<D011S65<DHaQC51V]J0@000F]J0000@6mkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DaAS65<DHaQBU6<HDYAS65:DH
aQBU6<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS65<DHaQC50aS600/DaAS6
5<DHaQC56<HDaAS65<DHaQC5008HaP097>LHaQSV5<DHaQC57>LHaQcW008HaP0:5<DHaQS67>LHaQcW
6<HLiaS67>L76<H01QcW6>HLiaSV7>LHiP8Li`0<8@PLib487>LQ21cW9BTQ22DY8@PU:B4X0RDY028Y
BRDY:DXUBBU:9DTYBRDY:DXU:Be[:FX]JbUZ;F/YJRe[;H/aS2f;=JdaS3F]<H`icSG=>LhecD8@?Nm2
43h?@Q0n3`924005AS5244Ha@Q16<@02@Q001Ch?@Q1244Ha@Q000THa00A:DTHaAU5:LP9>L`0;E[EB
]5JeD[ANme[fG_MB]5JeD[AF]@04CW<03dYBCW=:DU:DCW=BU5:dE[EB]5JeF_INme[fG_MJmP02G_L0
0f<HG_MS6002G_L01E[fG_MF]E[FE[D01E[F01=FeE[FE]ENme[FF_IJeV<HG_MO5eKEF]IFeE[FCY=B
U4jCCW=:LP03CW<00dYBCW=:DP03BU800dIABU96D@04@Q04=Jd00c6<;H/aS005<H`00cF]<H`aS004
<H`032f;<H`]Jbf;;F/aS2UZ;F/YJRe[:DXYJP8YBP0<9DTYBRE99BTQ:2DY8BPU:B4X9BTQ22DY1R48
00HM1b487@LQ21d78@P47>L01aSV7>LHiQcW6>HLiaSV00<Li`0;6>HLiaSV7>LHaQcW6<HLiaSV7>LH
iP046<H03QSV6<HLiaS66>HHaQcW5<DHaQC56<HDaAS65<D36<H06AC56<HDaAS65:DHaQBU5<DDYAS6
5:DHaQBU6<HDYAS65<DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00<HaQC56<H0
0aC500PHaQC55<DDaAS65:DDaABU0QC500HHaQC56<HDaAS65<D36<H00aC56<HDa@025<D01QS65<DH
aQC56<HDa@DHaP035<DHaQS600<HaP035<DHaQC500DHaP065<DHaQS66>HHaQSV0aS60QSV00DLiaSV
6>HHaQSV00PHaP056>HLiacW7>LHiP067>L01Ad77>LM1b4X9BT00R4X0bDY01@UBBDY9DTU:BU:9DTY
BRDY9DTU:BE9:FX]JbUZ;F/YJRe[:FXaS2f;0S6<00<a[36<<J`00SW>00DecCW>?Pm243h?00A24092
<005AS52<48`@S16<@03@Q0014Ha@S12<48`14Ha0dic0UJe00UB]5JeF_INme[fE[EB]5JeD[@00dic
0dYb00Q>LdjCCY=>TeJeD[AF]E:d25kg00=O5ekgGaL00ekg00AFeEJeE]EF]@=JeP03E]EJeUKE00EJ
eP=Nm`AJeP=BU0A>L`03BU9:LTYB00E:DP03AU5:DT8@00=240@e[A4aS0L]J`05:FXYBRU::DXUB@03
:DX00b4X9BTU:@049BT=8@P00ad77>LLi`057>L00aSV7>LHiP057>L011SV7>LHiQcW0QSV0QcW00<H
iQcW6<H061S600<DaABU5<D011BU00<DaAS65<D00QC500<HaQC5JeX01F]J0@000F]J00007V]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDa@<HaP06
5:DHaQBU6>HHaQcW0QS601DDYAS65:DHaQC56<HDYAC55:DHaQC56<HDaAS65:DHaQBU6<HDYAS64:@0
0aBU00TDaAS65<DHaQC56<HDaAS65<D00aS600dDaAS66<HHaQC56<HDYAS65:DHaQC56<HDa@046<H0
61cW6<HHiQS67>LHaQcW6<HLiaS67>LHaQcW6<HLiaS67>LHaQcW6<HLiaSV7>LHiP@Li`069BTQ22DY
8@PU:B481RDY00@YBRDY:DXUB@8YBP06;F/YBRe[:FX]JbUZ12e[00@icSF]>Lhe[@@icP04@Q0n3d8@
?Pl3@Q008D8`AS56<DHa@Q16<D8@AS5244Ha@Q16<D8@AS5:DTYbBU9>LdjCDYA>TeJeD[AF]E:dF]IF
eE[FD[AF]E:dE[E:LP02CW<05E:DCY=BU4jCF]IFeE[FF_INme[fG_MFeE[FE]EJeU[fG_MJmUkgF]IN
m`02F]H01eKEF]IB]5JeD[AF]DjC00=BU00ECY=BU4jCF]IF]E[FE[EJeUKEF]IF]E[FE[EJeUJeF]IF
]E:DCW=BU4ic00=:DP05AU5:DTIABU92<002AS402T8@?Pm243h?=Jda[3F]<J`aS2f;1C6<00L]Rc6<
;H/aS2f;<H`]R`03<H`02RUZ;F/YJRe[:FX]JbUZ;F/YBRe[0RU:00`U:BU:9BTU:B4X9BTQ:2DY8BPU
:B488BP28@P03Ad78@PM1b487>LQ21cW8@PLib486>HLiaSV008Li`088@PLiacW6>HLiaSV7>LHiP<L
i`056>HLiaSV7>LHiP037>L01ASV6<HHaQS65<D01aS600<DYAS65:D01AS600DDaAS65:DHaQBU008H
aP0D6>HHaQS65:DHaQBU6<HDYAS65:DHaQBU6<I[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX0
16]J11S600<DaABU5<D011S600<DaAS65<D00QC500<HaQC55:D00aBU00<HaQC56<H00aC500@HaQC5
5<DDa@@DY@036<HDaAS600lHaP045<DDYAC55<D:6<H01ASV6<HHiQS67>L00aSV00DLiaSV6>HHiQcW
00<HiP047>LHiQcW6>H57>L01b488BPQ224X8@PQ:24800@U:@03:DXUBBE900DYBP03:FXYBRe[008]
J`08:FX]Jbe[;F/e[CG==Jdec@@icPM240=6<@03@Q12<48@0092<006AS52<48`@Q12<48@14YB0e:D
00AB]5JeE[EF]@=JeP04E[EB]5JeD[@3CW<2DY@02DjCDYAJeU[FE]ENme[fG_MJmP03F]H015KEG_MJ
mUkg0e[f00QNme[FF]IJeUJeD[AF]E:d1e:D00AF]EKEE[EFe@=JeP0;E]EF]EKEE[EFeEJeE]E>LdjC
CW=>T`07BU83AS4013h?@Q0n3d8@0SF]00<a[3F]<H`02C6<00<]Rc6<<H`0136<00<]JbUZ;F/00Re[
00<YJRe[;F/00bU:00@U:BE99BTUB@HU:@038BPQ224800DQ20@M1`057>LM1acW7>LHiP027>L00ad7
7>LM1`027>L01ASV7>LLiacW6>H02acW31S60QC51aS60QC500<HaQC56<H00QS60aC500PHaQBU5<DD
YAC55:DDaABU1V]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]J001[FVmkJeY[NV]JKg/00QS600PDaAS65:DHaQBU7>LHaQSV0QS600TDYAS65:DHaQC56<HD
YAS65:D00QC500THaQC56<HDYAS65:DHaQBU6<H00QBU00T@Y1BU5<DHaQC56<HDaAS65<D00aS600DD
aAS66<HHaQC500<HaP095:DHaQBU6<HDaAS65<DHaQC5008HaP0H6>HHaQcW6<HLiaS67>LHaQcW6<HL
iaS67>LHaQcW6<HLiaS67>LHaQcW6>HLiaSV11cW00HU:B489BTQ22DY8@P69BT00bU:9BTYBP02:DX0
1bE9;F/YBRe[:FX]JbUZ00@]J`04>Lhe[CW>=Jd4>Lh00d8@?Pm24002@Q008Sh?@Q16<DHa@S16<D8@
AS5244Ha@Q16<D8@AS5244HaBU9>LdYBBW9>Te:DCY=F]E:dE[EB]5[FE]EJeU:dE[EB]5Je0Tic01I:
LU:DCY=BU4jCF]IFeE[FF_INme[fG_MFeE[FE]EJeU[fG_MJmUkgF]INm`9JeP06E]EJeU:dE[EB]5Je
0U:D00U>Te:DCY=BU4jCF]IF]E[FE[D00e[F019F]E[FE[EJeUJeF]IF]E:DCW=BU4icBU96DDYBAU5:
DTIABU82AS402d8`@Q0n3d8@?Ple[C6/=Jda[36<;H/01C6<01@]Rc6<;H/aS2f;<H`]Rc6<;H/aS2UZ
;F/YJRe[:FX]JbUZ;F/YBRUZ0RU:00<U:BU:9BT00bDY00HQ:2DY8BPU:B489BT28@P03Ad78@PM1b48
7>LQ21cW8@PLib486>HLiaSV008Li`0:8@PLiacW6>HLiaSV7>LHiQcW6>H37>L01ASV7>LHiQcW6>H0
0QcW00<HaQC56<H00QS600<DaAS66<H00aS600<DYAS65:D01AS600DDaAS65:DHaQBU008HaP0D7>LH
aQS65:DHaQBU6<HDYAS65:DHaQBU6<I[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J0aS6
00PDaAS65<DHaQC56<HDaAS611BU0aS6018DaABU5<DDYAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS6
5<D56<H01AC56<HDYAC55:D00QC500@HaQC56<HDa@`HaP<Li`056>HLiacW7>LHiP076<H77>L01B48
7@LQ21d78@P00Qd70R4800<M1b488@P00R480bDY00DQ:2U::DXYBRE900HYBP@]J`<aS0@e[@07?Nln
3cg_?Pm2<4Ha@S000dHa00A2<4IAAU5:DP=6D@03BU96<DHa00E6<@A:DP=>L`04CY=BU4jCCY<3E[D7
CW<4E[D3F]H00ekgF_IJmP02F_H01E[FF_IJeU[FE]D00U[F00=JmU[FF_H00UJe00=B]5:DDY@00U:D
0U:d00AF]DjCCY=>T`9BU004D[ABU5:dD[@7E[D00e:dE[EBU003DY@3BU82AU401DYBAU56<D8`AS40
13g_13F]0c6<00@]Rbe[;H/]J`TaS0<]R`03<H`]Rbe[00<]J`07:FXYBRUZ9BTUBBDY9DT00bDY00LQ
:2488BPQ224X8@PQ:0068@P00ad77>LLi`067>L00aSV6<HHiP036>H011cW6>HLiaSV11S61acW4QS6
00HDaAS66<HDaABU5<D45:D011C56<HDaAC50QS600DDaAS65<DHaQC500I[FP40005[FP0000U[NV]J
Kg][FV]jJeY_Nf]JJgX00`0001][NV]J000006]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC5
6<HDaAS65<DHaQC56<H00QC501LDYAS65:DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS6
5<DHaQC500DHaP0=5<DHaQC56<HDYAS65:DHaQC56<HDaAS65<D00QS600DHiQS67>LHaQSV00@HaP08
6>HLiaSV7>LHiQcW6>HLi`@HaP0E5<DHaQC57@LLib487>LM1acW8@PM1b487@LQ21cW8@PLib487@LQ
21d700<Q200=8BPU:B4X9BTUBBU:9DTYBRE9:DXUBBUZ:DX00Re[01@]Rbe[<H`aS36/<H`icSF]=Lde
[D8@?Nm243g_AS5244Ha@S16<D8`0THa00E:DTHaBU96<DYB0096<@03@S16<DHa00=6<@03BU9:LTYB
009>L`06CY=>Le:DCW=BU4ic0eJe00e>Le:DCW=>TdYbCW=:LUKEE[EJeUJeF]IFe@02F]H05ekgF]IN
me[FG_MJeUkgE]EJeUKEG_MJeUkgF]IF]E:dE[EBU5JeDYAB]5:DE[D00U:D01m>Le:DCW=F]E:DE[EB
U5JeD[AF]E:dE[EB]5JeD[AF]E:dDYA>Te:DCY=>LdYBBW96<DYBAS5:DT8`AS52<004?Nl4=Jd3<H`0
0be[<H`]J`09<H`01bf;<H`]Jc6<;F/aS2UZ00<]J`06:DX]JbU::DXU:BU:0RDY00PQ:2DY8@PU:B48
9BTQ22DY0R4800/M1b487@LQ21d78@PLib487>LM1aSV00<Li`0A6<HLiaS67>LHaQcW6<HLiaSV7>LH
aQSV6<HLiaSV7>LHiP047>L36<H05QC56<HHaQS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU
6<HDYAS60QBU01<@Y1S65:DHaQBU6<HDaAS65<DHaQC56<I[FVmkJeY[NV]JKg/00001JgX000Y[FP04
001[FV]J0002JeX00`00JeY[FP03JeX00`00JeY[FP04JeX00aS65<DHaP026<H01QC56<HDaAS65<DH
aP@DY@<HaP045<DDYAC55:D36<H02aC56<HDaAS65<DHaQC56<HDaAS65<D01AS600DDaAS65:DDaABU
008Da@036<HDaAS6008HaP035<DHaQS600PHaP037>LHiQcW008Li`036>HLiacW00LHaPLLi`038@PM
1b4800<M1`038@PM1b4800DQ20069BTQ:2DY9BTYBRE90bU:00<UBBU::DX00RU:12e[0c6<13F]00@m
kch??Nln3`=2<005AS52<4HaAS5:DP03AU400dYBAU56D@07AS44BU83CW<3CY<015:DE[EF]EJe1dic
15Je0e[F0U[f00ENme[fF_IJeU[f009JeP07E]EJeU[FF_IJeU[fD[@00UJe15:D00EF]E:dD[A>Te:D
009>T`04DYAB]5:DD[@2E[D00e:dE[EF]@05E[D4DY@4BU83AU400dHa@S16<@04?Nl4=Jd3<H`012f;
;F/]Rbe[1c6<00@]Rc6<<H`aS0<]R`@]J`07:FXYBRUZ9BTUBBDY9DT00bDY00LQ:2488BPQ224X8@PQ
:0048@P00ad78@PQ20087>L01ASV6<HHiQSV7>L00aSV00<LiaSV6<H00aS61acW51S60QC500<DYAC5
5:D00aBU0aC500<HaQC56<H00QS600<DaAS65<D01V]J0@000F]J00001fmkJeY[NV]JKg][FV]j00D0
000:Kg/006]jJeX006]JJgY[FVmkJeX400004V]jJeY_NaC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU
5<DDY@<HaP0A5:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDa@056<H03AC56<HDaAS65:DH
aQBU6<HDaAS65<DHaQC5008HaP057>LHaQSV6<HLi`026<H02QC56<HHiQcW6>HLiaSV7>LHiQcW11S6
00TDaAS65<DQ21cW7@LLib487>L00Qd700XQ21d78@PLib487>LQ21d78@PM1`<Q200S8BPU:B4X9BTU
BBU:9DTYBRE9:DXUBBe[:DXYJRe[<H`]Jbf;<H`e[C6<=Lde[CW>=Je243g_@Q0mkdHa@Q16<D8`AS52
<002AS401DYBAS5:DTHaBU8024Ha01U:DTicBU9:LTicDYA>Le:DCW=BU4icE[EB]5JeCW=>TdicDYA:
LTicBW9JeUJeE]EF]@04F]H05ekgF]INme[FG_MJeUkgE]EJeUKEG_MJeUkgF]IF]E:dE[EBU5:dDYAF
]E:DE[D00U:D01m>Le:DCW=F]E:DE[EBU5JeD[AF]E:dE[EB]5JeD[AF]E:dDYA>Te:DCY=:LTYBCW=6
<DYBAS5:DT8`AS52<004?Nl4=Jd3<H`00be[<H`]J`09<H`01Rf;<H`]Jc6<;F/aS08]J`04:FX]JbU:
;F/2:DX00bDY:DXU:@039BT01R489BTQ22DY8@PU:@8Q200:7@LQ21d78@PM1b487>LM1acW8@P27>L0
3aSV7>LHaQcW6<HLiaS67>LHaQcW6>HLiaS67>LHaP026>H00acW6>HLi`037>L00aS65<DHaP046<H0
4aC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQ2T008DY@0?6<HDYAS65:DHaQC56<HD
aAS65<DHaV]JJgY[FVmk00<0005_N`001f]J00@006]JJeX0009[FP04001[FV]J0005JeX00`00JeY[
FP04JeX011C55:DDaABU0aS60aC500@HaQC55<DDa@@HaP035<DDYAC500DDY@035<DDYAC5008HaP03
5<DHaQS600HHaP8Da@PHaP036>HHaQSV008HaP035<DHaQS600HHaP8HiP037>LHiQS600<HaP8HiPTL
i`046>HLiacW8@P27@L28@P00ad78@PQ20048@P02B4X8@PU:B4X9BTQ:2E99BTUB@05:DX012UZ:DXa
S2f;1C6<00@ecCW>=Ldec@@mk`96<@05@S16DDIABU96D@08AS400dIAAS5:DP03AU402DjCCW=>Tdic
DYA>Te:DD[AF]@02D[@3DY@4CW<00eJeD[AF]@04G_L4HaP01ElGHaQS65kgF_H01Ekg1U[F00=FeEJe
E[D01EJe00AB]5JeE[EF]@ABU0MF]@09E]EF]EKED[AF]E:dE[E>LdjC00A>L`04BW96DDIABU84?Nl7
=Jd;<H`00bf;<H`]R`04<H`4;F/012E99BTUBB4X0bDY0b4X2B4800DM1b487@LQ21d700/Li`046>HL
iaSV7>L36>H47>LA6<H35<D96<H35<D01QS65<DDaABU5<DDY@8Da@036<HDaAC500<DY@I[FP40005[
FP0003=[NV]JKg][FV]jJeY_Nf]J001[FP00JeY[NP00Kg][FP00JeY_Nf]JJgY[FP00JeY[NV]JKg][
FV]j5:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65:DHaQBU6<H00aBU00XHaQBU6<HD
YAS65<DHaQC56<HDa@DHaP095:DHaQBU6<HDaAS65<DHaQC5008HaP0:7>LHaQcW5<DHaQC56<HDaAS6
5<D46<H00acW6<HLi`056<H031cW6<HLiaSV7>LHiQcW6>HLiaSV7>LHiP8Li`0@8@PLib487@LQ21d7
9BTQ224X8@PU:B489BTQ:2DY8BP29BT012U:9BTYBRE90RU:00H]JbU:;F/]Rc6<;H/2<H`0336/<H`i
cSF]>Lhe[D8@?Nln3cg_AS52<096<@04BU96<DYB@S02AS4084IAAS5:DTHaBU96<DYBAS5:DTHaBU9>
Le:DCW=BU4jCDYA>TeJeDYAF]E:DE[EBU5:dCW=BU4icCY=B]5JeD[@4G_L3HaP02elGHaQO5f<HF_IN
me[fG_MJmUkgF_H00e[F00UFeE[FE]EJeU:dE[EB]5JeD[@00eJe00EB]5JeD[ABU4jC009BU003E[EB
]5Je009F]@04D[AF]EJeF]H2E[D03e:dE[EB]5:DCW=BU4YbCW=:LTicAS5:DTHa?NlikP02?Nl00cG=
=JdicP04=Jd7<H`00cF]<H`a[002<H`01Rf;<H`aS3F]<H`a[0@]J`0:9BTYBRDY9BTQ:2DY8BPU:B48
9BT28@P031d78@PQ22487@LQ21d78@PM1b487>LQ20DLi`0C8@PLiad76>HLiaSV7>LHiQcW6>HLiaS6
7>LHiQcW6>HLiaS66>H01AS600<LiaS66>H00QS600dDaAS65<DHaQC56<HDYAS65:DHaQC56<HDa@03
6<H06ABU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DDaF]JKg][FV]jJeY_N`00005[NP00
2F]J0P000f]J0P001V]J00<006]JJeX016]J00@DaABU5<DDY@<HaP035<DHaQC5008Da@036<HDaAC5
00<HaP035<DDYAC500DDY@045<DDYAC55<D;6<H25<D76<H00aSV6<HHiP0;6<H011cW6>HHiQSV11S6
00<LiaSV6>H02QcW00DHiQd77@LQ21d700PQ20098BPQ22DY8BPU:B4X9DTU:BE900DYBP03:FXYBS6<
008aS004;H/aS36<<H`3=Ld00cW>?Nlmk`02?Nl0148`AS56<DYB0dIA24Ha00e6DDHaAU56DDYBAU5>
TdicCY=>Le:DCY=BU003D[@015JeDYABU5:D14ic00=F]E:dE[D015kg1V<H00EO5ekgG_MNme[f00=N
m`AJeP03E]EJeU[F00UF]@03D[AF]E:D00=BU0MF]@09E]EF]EKED[AF]E:dE[E>LdjC009>L`04BW9>
LdicBU82AU44?Nl7=Jd;<H`00bf;<H`]R`04<H`4;F/00bE99BTUB@029BT01B4X9BTQ:2DY8BP01b48
00@M1b488@PQ20<M1`/Li`076>HLiaSV7>LHiQcW6>H011cW3aS600@DaAS66<HHaP8Da@LHaP8Da@08
6<HDaAC55<DHaQBU5<DDY@@Da@046<HDYABU5:D6JeX10001JeX0000TKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaS66>HHaQcW5:DHaQBU0aS600@D
aAcW6<HHiP8HaP0;5:DHaQC56<HDaAS65<DHaQC56<HDa@026<H00aSV6<HLi`025:D05A2T5:DDYAS6
5:DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDa@036<H01QC56<HHaQcW6<HLi`8HaP0=5<DH
aQS67>LHaQcW6>HLiaSV7>LHiQcW6>H00acW00<HaQcW6<H00QcW00DQ21cW8BPQ22DY008Q20057@LQ
21d79BTQ:0059BT01bE9:DXUBBe[:DX]JbU:00@]J`06<H`]Jc6<<J`e[C6/13W>01I243h?@Q0n3dHa
@S16<D8`AS52<4Ha@S16<D8`BU96DDYBAU5>LdYBCW=>T`ABU005E[EBU5JeE[EJeP06E[D01U:dE[E>
Te:DCY=BU09JeP05E]ES65lGHaQO5`03HaP4G_L01V<HIcUS66LiG_MS609Nm`03F_INme[f00=Nm`EJ
eP06G_MJeU[FE]EJeUKE15Je00AJeUJeE]EJmP=Nm`0:F]INme[FF]IFeE[FE]EBU4icDY@2CW<03dYb
CW=6DDYBAU5243h?@Q0n3cW>=JdecCF]<H`]R`02<H`05cF]<H`a[36<=JdaS3F]<H`e[C6<=Jda[3F]
;H/aS2f;<H`YBRUZ:DX]JbDY9DT00RDY00LQ:2DY8BPU:B489BTM1`038@P07Ad78@PM1b487@LQ21d7
8@PM1b486>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHaQSV5<D00aS601DHiQcW6>HHiQS6
7>LHaQcW6>HLiaSV6<HDaAS65<DHaQC56<HDaAS65<D00aS600@DaAS66<HHaP@DY@035<DHaQC500DH
aP0=5<DHaQC56<HDYAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX46<H25<D96<H3
5<D011S65<DHaQC52QS611BU0QC500@HaQC55<DDY@8Da@LHaP<Da@PHaP036>HHaQS600<HaP046>HH
aQSV6<H37>L00aSV7>LHiP047>L01ASV6<HHiQS68@P00Qd722480RDY00<Q:2DY9BT00RDY00<YBRE9
:DX00bUZ12e[00@aS2f;;H/]R`<e[@@icP924003?Pm244Ha00U6<@04AU5:DTIABU83BW84DY@01e:d
DYAB]5:DE]EF]EKE00AF]@03D[AF]E:d00ABU0=JeP03GaMS65lG00AS60ANm`0:HcQS66<hHaQO5ekg
GaMJmUkgF_H4G_L4F]H2F_H00ekgE]EJeP02F]H7E[D5G_L2F_H01eKEF]IFeE[FCY=BU4jC00A>L`=:
DPA240@e[@03;H/aS36<00LaS004<J`e[C6/<J`3=Jd3<H`00bf;:DXYBP02:DX39BT01R4X9BTU:BDY
8@PQ:0HQ20057@LQ21d78@PM1`048@P77>L01ASV7>LLiacW6>H00acW1aS600<LiaSV7>L011S600@H
iQcW6>HLi`@HaP035<DHaQC500XHaP@DY@036<HDaAS600PHaP035:DDaABU00I[FP40005[FP0002I[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j6<HL
iaS66>HDYAS65:DHaQC50QS600<HiQS67>L00QS600TDYAS65<DHaQC56<HDaAS65<D00aS600DDaAcW
6<HHiQ2T00@DY@0@6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:D36<H02AC56<HHaQS65<DL
iaS67>LDa@046<H02QcW6<HLiaSV7>LHiQcW6>HLiaSV0acW00<HaQcW6<H00QcW00DQ21cW9BTQ224X
008Q20057@LQ21d79BTQ:0059BT02BE9:DXUBBe[:DX]JbU:;F/YJP02;F/01S6<;F/aS36/=Jda[0@i
cP04@Q0n3d8@?Pl3AS403T8`AS52<4Ha@S16<D8`BU96DDYBAU5>LdYBCW<2DY@01DjCDYABU5JeDY@0
0UJe00=JeUJeE[D015Je00MB]5JeCY=BU4jCDYAFe@02F]H016<HGaMS65lG0f<H15kg00IS66LiHaQW
>EkgHaP2G_L00e[fG_MJmP03G_L5F]H01UkgF]IJeUKEF]IFe@AF]@03E]EF]E[F009Nm`04F_INme[F
G_L2F]H01eKEF]IFeE:DCW=BU4Yb00=>L`0=AU5:DTIA@Q0n3d8@?PlecCF]>Lhe[C6<;H/00S6<01La
[36<=JdaS36/<H`e[C6<=JdaS3F]<J`e[Bf;<H`]Rc6<:DX]JbU::FXU:BU:008U:@068BPU:B4X9BTQ
22DY0R4801XM1b487@LQ21d78@PM1b487@LQ21d78@PHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW
0QSV014HaQcW6<HHaQC56<HHiQcW6>HLiaS66>HHaQcW6>HLiaSV00<HaP095<DHaQC56<HDaAS65<DH
aQC500<HaP035<DHaQBU00<DY@035<DHaQC500DHaP0=5<DHaQC56<HDYAS6JeY_Nf]JJgY[FVmk0000
0F]j000FJeX00`00JeY[FP04JeX56<H25<D86<H01aC56<HDaAS65<DHaQC500XHaP@DY@036<HDaAC5
008Da@055:DDaAS66<HDa@046<H25<D00aS65<DHaP076<H00aSV6<HHaP036<H02QSV6<HHiQS67>LH
iQcW6>HLiaSV11cW00@HiQS66>HHaP8M1`TQ20038BPU:BDY00@U:@05:DXUBBU::FX]J`02:FX3;F/2
;H/00c6<;H/e[@02=Jd4>Lh013h?@Q12448@2THa00M6DDYBAU5:DTYbCW=:LP04DY@01e:dDYAB]5:D
E]EF]EKE00AF]@03D[AF]E:d00ABU0=JeP03GaMS65lG00AS60ANm`0:HcQS66<hHaQO5ekgGaMJmUkg
F_H4G_L4F]H00ekgF_IJmP02F]H00eKEF]IF]@06E[D4G_L2F_H01EkgE]EJeUKEF]H00djC14ic00=:
DTIABU80148@13F]0S6<00<]Rc6<<H`01C6<0c6/13F]00@aS2f;<H`aS0@YBPDU:@048BPU:B488BP6
8@P01Ad78@PM1b487@L012481AcW00DHiQcW7>LLiaSV00<Li`036>HLiaS600HHaP<Li`@HaP046>HL
iaSV7>L46<H01AC56<HDaAS65<D021S611BU00<HaQC56<H021S600<DYAC55:D01V]J0@000F]J0000
:6mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/D
aAS65<DHaQBU6<HDYAS65:DHaQBU0aS600<DaAS65:D00QC500dHaQC56<HDYAS65:DHaQC56<HDaAS6
5:DHaP065:D0112T5:DHaQSV0QS600LDYAS65:DHaQC56<HDa@076<H01AC56<HDaAS65<D00QS6014L
iaS67>LHaQcW6<HLiaS67>LHaQcW6>HLiaSV8@PLiad700DLi`0D8@PLib487@LQ21d79BTQ22DY8@PU
:B4X9BTUBBU:9DTYBRDY9DTU:@@YBP06;F/YJRe[<H`e[C6<0S6/0SF]00@mkcW^?NlikPA240=:DP04
AS5:DTHaBU83AS404TicBU9>LdYBDYA>LdjCD[AF]E:dE[EB]5JeD[AF]EKEF]IFe@AF]@06E]EF]E[F
E]EJeUKE0U[F00=Nme[FHaP00V<H00eO5f<HHaQS65lGHaQO5f<HGaMS65lGHaQS>002IcT4G_L03F<H
G_MS65[fG_MJmUkgF]INme[FG_MJeUkg00AJeP07E]EF]EJeE[EFeE[FE]D00U[F00ANme[FE[EB]09F
]@03DYA>Te:D009>L`04BW9>LdHaBU82AS401T8@AS5243W>=JdicP8e[@0@<H`e[CF]>Lhe[CW><J`e
[C6/=Jda[3F]<J`e[C6<=Jd4<H`052UZ;F/YJRe[8BPU:B4X9BTQ22DY8@PU:B4X9BTQ22DY8@PQ:1cW
8@P27>L01ASV7>LLiacW6>H00QcW00hM1acW8@PHaQSV6<HLiaSV7>LHiQcW6<HLiaS60acW00XHiQcW
6>HLiaSV7>LHiQcW6<HLi`8HaP0@5<DHaQS66<HDaAS65:DDaABU6<HDaAS65<DLiaS66>H26<H02aBU
6<HDaAS65<DHaQC56<HDaAS65<D00QS600hDYA2T5:D@Y1S65<DHaV]JJgY[FVmkJeY[NP000Fmk000F
JeX00`00JeY[FP04JeX00aS65<DHaP035<D011S65<DDaAC51AS60aBU00<HaQC56<H00QC500<DYAC5
6<H00aS600<DaAS65<D021BU0aS60aC500@HaQC56<HDa@THaP0=5<DHaQC56<HDaASV6<HHiQS66>HH
aQSV6<H00QSV3acW0R4800DM1b488@PQ:248008Q:0069BTQ:2U:9DTYBRE90bDY12U:0be[136<0cF]
13g_148@0dYB0TIA00=:DTIAAS400THa00A:DTYbBU9:LP=>L`03E[EB]5Je009F]@06D[AF]E:dF]IF
eE[F1eJe15[F0U[f00=Nmf<HHaP01V<H00EO5f<HGaMS65lG009S60=W>@ANm`03GaMS65lG00ANm`07
F_IJeU[fF]IJmU[FF_H015[F0eJe15[F00=Nme[fF_H015Je00=>Te:DCY<014ic00=6DDHaAU400d8`
00]6<CF]=Lde[CG=<H`a[36<=Lde[CG=008e[@03<J`e[CF]00<e[@03<J`e[C6/00@aS004;F/YJRe[
:FX39BT38BP012DY8BPU:B4X1B480Qd71QcW00<HiQcW7>L00QcW11S600LLiaSV7>LHaQSV6<HHiP08
7>L01QSV7>LHaQSV6<HHiPHHaP035<DDYABU008DY@LHaP095:DDaABU6<HDaAS65<DHaQC500DHaP8D
Y@054:@DYAS66<HDa@06JeX10001JeX0000ZJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQC50QS6014DaABU
6<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDYAS600@DY@064:@DYABU5:DHaQcW0QS600DDYAS65:DH
aQC500<HaP035<DHaQS600<HaP055<DHaQC56<HDa@026<H04AcW6<HLiaS67>LHaQcW6<HLiaS67>LH
iQcW6>HM1acW8@P01AcW01@M1acW8@PM1b487@LU:B489BTQ22DY8BPU:BE9:DXUBBU:9BTYBRDY12U:
00H]JbUZ;F/aS36/<H`3=Jd01C6/?NlikSg_>Nh0148@0dYB00A6<DYBAS5:DP=6<@0BCW=:DTicBU9>
TdicDYAB]5JeD[AF]E:dE[EB]5JeE]EJeUKE15Je00IJeUJeE]EFeE[FE]D2F]H015kgF]IS65lG0f<H
00YO5f<HGaMS65lGHaQO5f<HGaMS609W>@03HcQNmekg009Nm`0=HaQNmf<HF_INme[fG_MJeUkgF]IN
me[FG_L00U[F00=FeE[FF]H00eJe00=FeE[FE]D00U[F00=Nme[FE[D00UJe00EB]5:DCY=BU4Yb00=>
L`0:AS5:DTHaAS5244Ha@Q0icSF]>Lh2=Jd0436<=Jde[CW>=JdicS6/=Jda[3F]<J`e[C6/=JdaS3F]
136<01@YJRe[:FX]Jb4X9BTQ:2DY8@PU:B489BTQ:2DY8@PQ:2489BTLib4811cW00<HiQcW6>H00QcW
00HQ21cW7@LHaQcW6<H26>H04acW6>HLiaS67>LHaQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHaQcW00@H
aP075<DHaQC56<HDYAS65:D00QC500DHaQC56>HHaQcW008HaP095:DHaQC56<HDaAS65<DHaQC500<H
aP0?5<DDYA2T5:D@Y1S65<DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00LHaQC5
6<HHaQC55:DDa@0>6<H02aC56<HHaQS65<DHaQC56<HDaAS65<D01AS600TDaAS65<DDaABU5<DDYAC5
5:D00QC500HHaQC56<HDaAS65<D:6<H011cW6>HLiaSV11S600<LiaSV7>L011cW00<M1b487@L00Qd7
00<Q21d77>L00QcW00@M1b487@LQ20<Q:0049BTQ:2DY8BP49BT00bU:9DTYBP04:DX3<H`3=Jd0136/
=LdecCW>13g_00A2<4HaAS56<@=:DP04BW9:DTYbBU83CW<00djCDYA>T`04DY@3E[D00e:dE[EF]@09
E[D3E]D00f<HGaMS6004HaP00f<hIcUS>003IcT00f<hIcUW>@02IcT00fMIIcUWF@04IcT025lGHaQO
5f<HGaMS65lGG_L3F_H01ekgF_IJmU[fG_MJmUkg00AJeP03E]EJeUKE00]F]@0:DYAB]5:DCY=>LdjC
CW=6DDHaAU44?Nl4=Jd00cG=>Lhec@03>Lh3=Ld5>Lh013F]=Lde[CW>0cG=136<0bU:00@Q:2DY8BPQ
:0HQ20047@LQ21d78@P47>L00aSV7>LLi`057>L00aSV7>LHiP037>L011S66>HHaQSV0acW00@HiQS6
6>HHaP<Li`056<HHiQS66>HDa@036<H01QC56<HDaAS65<DHaP8Da@095:DDaAS66<HDaAS65<DHaQC5
00@HaP045<DDYAC55<D36<H45:D00aC56<HHaP06JeX10001JeX0000XKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaC56<HDaAS65:DHaQBU6<HDaAS6
5<D36<H04AC56<HHaQS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D00aS6044DaAS65<DHaQC56<HD
YAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HHiQcW6>HLiaS66>HH
aQcW6>HLiaSV7>LHiQcW6>HQ21cW8@PLib487>LQ21cW8@PLib487@LQ21d79BTQ22DY8@PU:B4X00<U
:@0?8BPU:BE9:DXUBBU:9DTYBRE9<H`]Rc6<<J`e[C6/008e[@03>Lhe[Cg_00<mk`04AS52<4Ha@S04
BU800dicBU9>L`03CW<01U:DCY=BU4jCE[EBU09B]006E[EB]5JeD[AF]E:d25Je00IJeUJeF]IO5f<H
GaL2HaP026<hHaQW>F<hIcUS>6LiHcP6IcT016]JIcU[FV<h0VLi019S65lGHaQO5f<HGaMS65[FG_MJ
eUkgF]INme[FG_MJmUkgF_H5F]H025JeF]IF]E[FE[EFeEJeF]H2E[D07e:dE[EB]5JeDYAF]DicDYA>
Le:DAS5:DTHa?NlikSg_>NhicSF]=Lde[CW>=JdicSG=>LhecCW>=JdicSF]00DicP06=JdicSF]>Lhe
[CW>0S6<00P]Rc6<9DTYBRE99BTQ22DY124800LM1b487@LQ21d78@PM1`057>L00aSV7>LLi`037>L0
0aSV7>LHiP057>L041S67>LHaQcW6>HLiaS67>LHaQcW6>HLiaSV7>LHaQcW0QS601<DaAS65<DHaQC5
6<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDa@056<H01QBU6<HDYAS65<DHaP8Da@0=5:DHaQBU6<HD
aAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX36<H25<D00aBU5<DHaP0;6<H00aC5
6<HHaP036<H01aC56<HDaAS65<DHaQC500DHaP095<DHaQC55<DDYAC55:DDaABU008Da@066<HDaAS6
5<DHaQC52QS600@LiaSV7>LHiP@HaP037>LHiQcW00@Li`<M1`048@PM1ad77@L37>L01Qd78@PM1b48
8BPU:@<Q:0039BTQ:2DY00<U:@03:DXUBBU:00@YBP<aS005=Jda[3F]=JdicP02=Ld4?Nl2AS400d8`
AS5:DP02BU8014YbBU9:LTYB0dic00=>Te:DCY<015:D00=F]E:dE[D035Je00=FeE[FE]D00f<H00UO
5f<HHaQS66<hIcUS>6LiHcP01VLi00=WFFLiIeT016Li00MO5f<HGaMS65lGHaQO5`02F_H02EkgF_IJ
mU[fG_MJmUkgF_INm`04F]H3E]D;E[D02U:DD[ABU4jCCW=>TdicAU56<DIA13g_13F]0cG=00<icSG=
>Lh00SW>0SG=13W>00<e[CG==Jd00SG=00<icSG=<H`00c6<0bU:0b4X00<U:B488@P00R4800HM1b48
8@PQ21d78@P67>L00aSV7>LLi`037>L00aSV7>LHiP037>L011S66>HHaQSV0acW00/HiQS66>HHaQcW
6>HLiaS66>HHaQSV008HaP085<DHaQC56<HDaAS65<DHaP8Da@035:DDaAC500<HaP035<DHaQC500@H
aP035<DDYAC5008HaP035<DHaQBU00<DY@8HaP035<E[FV]J00A[FP40005[FP0002Y[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDYAS6
5:DHaQC56<HDaAS65<D56<H03aC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDa@036<H06AC56<HD
aAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500<HaP076>HLiaSV7>LH
aQcW6<H00QSV01`LiaSV7>LHiQcW6>HQ21cW8@PLib487>LQ21cW7@LLib487@LQ21d79BTQ22DY8@PU
:B4X9BTQ:0<U:@0=9DTYBRE9:DXUBBU:9DTaS2f;<H`a[3F]<J`00SF]00<icSF]?Nl00cg_00A6<D8`
AS52<0A:DP03CW=:DTic00=>L`0>DYA>Te:DCY=B]5:DE[EB]5JeD[AF]E:dE[EB]0QF]@06F]IF]E[F
GaMS65lG0V<H00QW>F<HIcUS>6LiHcQW>F<h1VLi00=[FVLiJeX00VLi01=S>6<HGaMS65lGHaQO5f<H
F]INme[FG_MJeUkgF]INme[fG_MJmP05F]H025JeF]IF]EKEE[EJeUJeE]D2E[D07e:dE[EB]5JeDYAF
]DicDYA>Le:DAS5:DTHa?NlikSg_>NhecCF]>Lhe[CW>=JdicSG=>LhecCW>=JdicSF]00DicP07=Jdi
cSF]>Lhe[CW>;H/00c6<00HUBBU:9DTU:B489BT48@P01ad78@PM1b487@LQ21d700DLi`036>HLiacW
00<Li`036>HLiaSV00DLi`0@6<HLiaS67>LHiQcW6<HLiaS67>LHiQcW6>HLiaS67>L26<H04aC56<HD
aAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC500DHaP0E5:DHaQBU6<HDaAS65<DHaQBU5<DD
YAS65<DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00HDaABU5<DDYAS65<D96<H0
1aC56<HDaAC55:DDaABU00<HaP055<DHaQC56<HDa@036<H35<D011S65<DHaQC50aBU11S600<DaAS6
5<D01aS600<DaAS66<H031S600<LiaSV6>H00QcW00<HiQcW7>L00QcW00@Q21d78@PM1`<Q200;7@LQ
22488@PQ:2DY8BPU:B488BPQ20049BT01bU:9DTUBBU::FXYBRUZ008]J`05:FXecCF]=Lde[@03>Lh4
?Nl0148`AS52<4Ha0dYB14ic00MBU4jCDYABU5:dDYAB]003DY@4F]H015JeD[AF]E:d0eJe00=FeE[F
E]D015[F00AO5ekgGaMNm`=W>@A[FPA_N`03JgY_Nf]j009[NP03Kg][NVLi009W>@AS6004GaMS66<H
G_L3F_H00eKEE[EFe@08G_L4E[D01e:dE[EB]5:DCY=BU5:d00=F]@04CY=BU5:DCW<3BW83AS44>Lh0
0cF]=Lde[@02=Ld00cW>=Ldmk`02?Nl00cW^?NlikP08?Nl4>Lh01cF]<J`e[C6/:DXUBBU:00@U:@LQ
20hLi`0:6>HHaQSV6<HLiaSV7>LHiQcW6>H77>L011SV5<DDYAC521S600DDaAS65<DHaQC500dHaP03
5<DHaQC500<HaP035:DDaABU00@Da@046<HDaAC55<D6JeX10001JeX0000TKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC50aS6
00<DaAS66<H00aS600DDaAS65:DHaQBU00DHaP0A5<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU
5<DDY@036<H01AC56<HDYAS65<D00aS600LDaAS66<HHaQC56<HDa@056<H03AC56<HHaQSV6<HLiaS6
7>LHaQcW6>HLiaSV00<Li`057@LQ21d78@PM1`038@P03Ad78@PM1bDY8BPU:B4X9BTQ22DY8BPU:B4X
008U:@0>:DXU:Be[:DX]JbU:;F/YJRe[=JdicSF]>Lhec@8icP@mk`04AS52<4Ha@S03BU82CW<07DYb
CW=>Te:DCY=F]E:DE[EBU5JeDYAB]5[FG_MJeU[fD[AF]E:dE[EB]5JeD[AJeUKEF]IFeE[fF]H00Ukg
00YS65kgHaQW>FMIIcU[FVMIJeYWF@=_N`0=JgY_Nf]jKg][FVmkJeY_NfLiJeYW>F<HGaL00f<H00eO
5f<HF]INme[FG_MF]E[FE[ENme[fG_MJmP04G_L065JeD[AF]E:dE[EB]5JeCY=BU4jCE[EB]5JeD[AB
U4jCDYA:DTicBU9>Ld8`AS52<0DicP0==JdicSF]>Lhe[CW>>NhmkcW^?NlikSg_>Nh01cg_00/icSW^
>Lhmkc6/=Jda[3F]9DTYBRE900<U:@038BPQ224800DQ20<Li`037@LLib4800@Li`0?6>HLiaSV7>LH
aQcW6<HLiaSV7>LHiQcW6>HLiaSV00<Li`0=6>HLiaSV7>LDYAS65:DHaQC56<HDaAS65<D00aS600DD
aAS65<DHaQC500@HaP037>LHaQSV008HaP055<DHaQC56<HDa@056<H04ABU6<HDYAS65:DHaQBU6<HD
YAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX01QC55:DDaABU6<HDa@THaP075<DH
aQC55<DDYAC55:D00aS600<DaAS65<D00aS60aC500HHaQC56<HDaAS65<D35:D46<H35<D96<H00aC5
6<HHaP0:6<H26>H00acW6>HLi`057>L012487@LQ21d71B4800TM1b488BPU:B4X9BTQ224X8@P012DY
0RE90RU:00@YJRU::FXYJP8]J`04=Lde[CG==Jd3>Lh4?Nl0148`AS52<4Ha0dYB14ic15:D00=B]5:D
D[@00e:D15[F00YF]E:dE[EB]5JeD[AF]EKEF]IFe@AJeP04GaMNmelGG_L3IcT4JeX4Kg/016]jKg][
NVmk0f]j0fLi1V<H00QO5e[fF_INme[fE]EF]EKE25kg15Je00IB]5JeD[ABU4jCDY@2E[D01E:dE[EB
U5:DCY<00TYb00=>LdYbAS400THa13W>00<e[CG==Jd013G=00<mkcW^?Nl00Sg_00<ikSg_?Nl01Sg_
13W>00@e[C6/=Jda[0<YBP@U:@LQ20hLi`0:6>HHaQSV6<HLiaSV7>LHiQcW6>H57>L01QSV7>LLiaC5
5:DDa@PHaP035<DHaQC500<HaP035<DHaQS600THaP035<DHaQC500<HaP035:DDaABU008Da@066<HD
aAC55<DHaQC51V]J0@000F]J00005V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]J0`0000Y[FVmkJeY[NQBU6<HDYAC56<HLi`8HaP075:DHaQBU6<HDaAS65<D011S600TLiaS6
7>LDaAS65<DLiaS66>H00QS6014DaAS65:DHaQBU5<DDYAS65:DHaQBU6<HDYAS65<DHaQC500DHaP05
5<DHaQBU6<HDY@036<H01AC56<HHaQS65<D00aS6020HiQcW6>HLiaS67>LHaQcW6>HLiaSV8@PLib48
7>LQ21cW8@PM1b487@LU:B4X9BTQ:2DY8@PQ:248:DXU:BE912DY00<Q:2DY8BP012e[01@aS2f;<H`e
cCW>=LdicSW^?NlikTHa@Q16<D8@BU96<DYBAS5>LdYb0Tic00=BU4icDY@00e:D00IB]5:DE[EBU5Je
D[@2E[D04U[FE[EJeU:dE[EB]5JeE]EJeUKEHaQNmf<HG_MS65kgHaQS>0AW>@0KJeYW>FnKKg]cW6mk
Lia_Ng>LKi]cW6nKLia[NVmkJgY_NfMIJeYWFFLiHaQW>F<HG_MJmUkg00AJeP0RD[AF]EJeG_MJeUkg
F]IS65lGHaQO5e[FE]EJeUJeF]IF]E[FCY=BU4jCDYA>Te:DCY=>LdYbCW=6<DYBAS56DCg_@Q02?Nl0
1cW>?NlicSG==JdicSF]00<mk`924005?Pm243h?@Q0n3`05@Q002Sg_@Q0ikSg_>Nhmkc6<=JdaS36/
0bU:00LU:B4X9BTQ:2DY8@PU:@048@P00acW8@PLi`047>L01R487>LQ21S67>LHaP8Li`058@PLiad7
6>HLi`026>H01QS67>LHaQcW6>HLi`8HaP0<5<DHaQS66<HDaAS65<DHaQC57>LHaQcW0QS600LDaAS6
5<DHaQC56<HDa@036<H07AC56<HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS6
5<DHaV]JKg][FV]jJeX00P000F]j000FJeX00`00JeY[FP04JeX45:D36<H011C56<HDaAC51aS600<H
iQS66>H021S600<DaAS65<D011BU00DDaABU5<DDYAS6008Da@8HaP035<DHaQS600HHaP8Da@`HaPDL
i`8HiP8Li`046>HLiad78@P37@L00b487@LQ20028@P0124X9BTQ:2DY12482RDY12e[0S6<00D]RcW>
=LdicSG=00<mk`04@Q12<48@@S03AU4024YBBW9>LdYbCY=>LdjCCW<7DY@3E[D00eKEE[EFe@05E[D3
F]H00ekgGaMNm`04GaL4IcT00f]JIeUWF@04Kg/00fnKLia_V`03Li`01FnKKg][NVmkJgX00f]J0f<h
00AW>E[fG_MNm`AJeP=F]@04F_INme[fF_H2HaP00elGHaQJeP02F]H015KEE[EFeEJe0e:D00E>Te:D
DYABU4Yb009>L`A6<@07?Plmkch?>LhikSW>>Nh013F]0cg_2d8@0ch?00@mkcW^?Nlmk`@aS0<YBP@U
:@038BPU:B4X00@Q208M1`038@PLiacW00<Li`037@LLiacW00<HiPLLi`@HaP036>HLiaSV00LHaP04
5<DHaQS66<H36>H00acW5<DHaP0:6<H031C56<HDYAC55:DHaQC56<HDaAS65<DHaP<Da@PHaPI[FP40
005[FP00021_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]J
JgY[FVmk5:DDaABU0QS600XHiQS66<HDYAS65:DHaQC56<HDa@@HaP097>LHaQcW5<DHaQC56>HHaQcW
008HaP0A5<DHaQBU5<DDYAS65:DHaQBU6<HDYAS65:DHaQC56<HDa@036<H02AC56<HHaQS65:DHaQBU
6<HDa@036<H07QC56<HHaQS65<DHaQSV7>LHiQcW6<HLiaS67>LHiQcW6>HQ21cW8@PLib487>LQ21d7
8@PM1bDY8BPU:@8Q:0068@PU:B489DTU:BU:12DY00<Q:2DY8BP012e[01@aS2f;<H`ecCW>=LdicSW^
?NlikTHa@Q16<D8@BU96<DYBAS5>LdYb0Tic00=BU4icDY@00e:D00AF]E:DD[ABU0AF]@0AF]IF]E[F
D[AF]E:dE[EFeE[FE]ES65kgHaQNmf<HG_MS6002IcT07V<hIcUW>F]JIcUcW6mkKi]_Ng>LKg]cW6nK
Lia_Vg>LJgY_Nf]jKg]WFF]JIeUW>F<HIcUS65kgF_INm`AJeP9F]@0PD[ANme[FG_MJeV<HGaMS65lG
F]IFeE[FE[EJeUJeF]I>Te:DCY=BU4jCDYA>TdicBW9>LdHaAU56<DYB?Nm2408mk`07>LhmkcW>>Lhe
[CG==Jd00cg_00Ln3d8@@Q1243h?@Q0n3`05@Q002Sg_@Q0ikSg_>Nhmkc6<<J`aS3F]0bU:00LU:B4X
9BTQ:2DY8@PU:@048@P00acW8@PLi`047>L01R487>LQ21S67>LHaP8Li`0>7@LLib486>HLiaSV7>LH
aQSV6<HLiaSV7>LDa@<HaP0:5<DHaQS66<HDaAS65<DLiaS67>L26<H01AC56<HDaAS65<D00aS6024D
aAS65<DHaQC56<HDYAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDaAS6JeY[NV]JKg][
FV]j00000Fmk000FJeX00`00JeY[FP04JeX45:D36<H35<D86<H00aSV6<HHiP086<H00aC56<HDa@04
5:D011C55:DDaABU0QC500<HaQC56<H02QS60QC52aS611cW0QSV00<LiaSV7>L00QcW0ad700/Q21d7
7@LM1b487@LQ224X9BTQ:2DY00@Q20XU:@@]J`07;H/aS36<>LhecCW>=Ld00cg_00I2448`@Q12<4IA
BU82AU401dYbCW=:LTjCCW=>Tdic00MBU0=F]@03E]EF]EKE00EF]@06F]IFeE[FG_MO5ekg0UlG00=S
65lGIcT00fLi0VMI00=[FVmkKg/00Vmk0fnK00QcW6nKLiacW6mkJgY_Nf]j0f]J00AS>6LiHcQS>09N
m`03F_IJeU[F009JeP=F]@=JmP03G_MO5f<H009S60=JeP06E]EF]EKEE[EBU4jC0e:D00E>Te:DCW=>
LdYb00A6<@07?Plmkch?>LhikSW>>Nh013F]0cg_2d8@00<n3d8@?Pl00cg_00<ikS6<<H`00S6<0bU:
12DY0b4X1B480Qd71AcW00HM1acW6>HHiQcW6>H77>L46<H00aSV7>LHiP096<H011C56<HHiQcW0QSV
0QS600<DaAS66<H01AS6010DaAS66<HHaQBU5<DDYAS65<DHaQC56<HDaAS65<DHaP8Da@LHaPI[FP40
005[FP0002][NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]J
Kg][FV]j5<DHaQC56<HDaAS65<DHaQC56<HDaAcW6<HLi`046<H04AC56<HDaAS65<DHaQC56<HDaAS6
5<DHaQBU6<HDaAS65<D01AS600HDaAS66<HHiQS67>L26<H01ABU6<HHaQS65<D00aS600DDaAS65<DH
aQC500@HaP067>LHaQSV6>HLiaSV0QcW00<Q21cW7>L01QcW014M1b487@LQ224X9BTQ:2DY8BPU:B4X
9BTQ:2DY8BPU:B4X008U:@0=:DXU:BU::DX]JbU:;H/]Jc6<;F/e[C6/=Jd013W>00<ikSg_>Nh014Ha
00Q:DTIABU96DDicBW9>LdYb0dic00UF]E[FE[EJeUKEF]IFeE[FE]D00e[F00eFeE[FF]INme[FG_MJ
mUkgF_IS65lGHaQO5`03IcT05VMIJeYWFF]JKg]cW6mkMkecW7NmLiakgWOMNmig_G_NMkekgW>LMkec
W7Nm0V]J00=WFF<HHaP00V<H00MW>F<HHcQJmUkgF_INm`03E[D04E[FE[EJeUJeHaQO5f<HGaMNme[f
G_MJmUkgF_INme:DE[D00U:D01E>Le:DCW=>LdYbCW=2<4Ha@S16<Cg_@Q0mkcW^>LhmkcW>=Jda[3F]
<J`00cW>00Xmkd8@?Nm2448`AS52<4Ha@Q16<@92400D?Nm243F]>Lhe[CW>;F/aS2e[<H`UBBU:9DTU
:B4X9BTQ:2DY8BPU:@8Q20057@LQ21d78@PM1`037>L03QSV7>LHiQcW6<HLiaS67>LHiQcW6>HLiaSV
7>L26>H00aS67>LHaP086<H01QcW6<HHaQBU6<HDY@@HaP<DY@096<HLiaS66>HDaAS65<DHaQC500<H
aP095:DHaQBU6<HDYAS65:DHaQBU00DHaP0=5<DHaQC56<HDYAS6JeY_Nf]JJgY[FVmk00000F]j000F
JeX00`00JeY[FP04JeX00aS65<DHaP026<H00aC56<HHaP046<H00aSV6<HHiP086<H02aC56<HDaAS6
5<DHaQBU5<DDYAS65<D03AS600<DaAS65<D04QS63QcW0b48010M1bDY9BTU:B4X9BTQ:2DY8BPU:B4X
9BTQ:2DY8BPYBP<UB@<YBP@]J`03<J`e[C6/00@icP<mk`A6<@A:DPM>L`0:F]IFeEKEE]EJeUKEF]IF
eE[FE]D4F]H015[fF]IJmU[F0ekg16<H0fLi00][FVMIJeY[FVnKKg]_Vg>LLkacW7>l009kgP9gg@03
MkeggGNm009c_003Mkec_6]J009[FPMS6004G_MJmUkgG_L4E[D00eKEE[EFe@02HaP00elGHaQNm`06
G_L01e:dDYAB]4icCY=>LdjC00=>L`04AS52<4HaAS43?Nl4>Lh2=Jd00c6/=JdicP02>Lh2?Pl00d8@
?Pm6<@02AS40248@@S12448`?Nln3cg_>Lh3=Ld01bf;;F/]Rbe[:DXUBBU:008U:@058BPU:BDY9BTQ
:0058@P00ad78@PLi`037>L011SV7>LHiQcW0aSV00LLiaSV7>LHiQcW6>HLi`0;6<H00aSV6<HHiP03
5<D56<H35:D56<H00aC56<HHaP036<H00aC56<HDa@025<D011S65<DDaAC51AS600@DaAS65<DHaP<D
a@I[FP40005[FP0002]_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00
JeY_Nf]JJgY[FVmk5<DHaQC56<HDaAS65<DHaQC56<HDaAcW6<HLi`046<H04AC56<HDaAS65<DHaQC5
6<HDaAS65<DHaQBU6<HDaAS65<D01aS600@DaAcW6<HHiP8HaP035:DHaQC500<HaP075<DHaQS66<HD
aAS65<D011S600HHiQS67>LHiQcW6>H27>L00ad77>LLi`067>L04Ad78@PM1b488BPU:B4X9BTQ:2DY
8BPU:B4X9BTQ:2DY8BP00RDY00dYBRDY:DXYBRUZ:DXaS2e[;H/]JcF]<J`e[@04>Lh00cW^?NlikP04
AS401TYBAU5:DTIACW=:LP=>L`0=BW9>LeJeF]IF]E[FE]EJeUKEF]IFeE[FE]D015[F00YNme[FG_MJ
mUkgF_IS65lGHaQO5`=W>@0GIeU[FVMIJeY_Ng>LKg]g_G>LMkecW7_NMmekgWNmNmig_G_NLiag_G>L
MkeWF@02JeX4HaP01f<hHaQW>E[fG_MJmUkg00=F]@08F]IF]E[FE[ES65lGHaQO5`=Nm`06F_INme[f
G_MBU5Je0U:D00a>Le:DCW=>LdYbCW=2<4Ha@S16<Cg_?Pl2?Nl01cW>>NhicSF]<J`e[C6/00<icP0:
?Nm243g_@Q12<4Ha@S16<D8@AS42@Q005Cg_@Q0e[CW>=JdicRe[<H`]Jc6<9DTYBRE99BTQ:2DY8BPU
:B4X9BTM1`038@P01Ad78@PM1acW6>H00acW010HiQcW6<HLiaS67>LHiQcW6>HLiaSV7>LHiQcW6<HH
iPTHaP067>LHaQS65:DHaQBU11S60aBU00LHaQSV6<HLiaC56<HDa@036<H02aC56<HDYAS65:DHaQBU
6<HDYAS65:D01AS600dDaAS65<DHaQBU6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[
FP046<HDaAS65<D36<H021C56<HDaAS65:DDaABU5<D46<H011C56<HHaQS60QC511S600<DaAS65<D0
0QC500dHaQC55<DDYAC55:DHaQC56<HDYAC55:DDa@076<H00acW6>HHiP046<H00aSV7>LHiP046<H2
6>H00acW6>HLi`027>L011S66>HHaQSV0acW1b482bDY00@YBRE9:DXUB@8]R`04<H`ecCG==Ld2>Lh0
13W^>LhmkcW^0Sg_00=243h?@Q0014Ha14YB0U:D00=>Te:DDY@00U:D0eJe15[F0ekg00AS>6<HHcQS
60ANm`07HaQO5f<HHaQS>6<HHcP00Vmk00E[NW>LKi]cW6nK00=g_@04NmikoW_NNoh7Ool017_nNmik
oW_N0g>L00A[NVmkJgY[NP=W>@AS6007F_IJeU[fE]EJeUKEF]H02ekg0eJe00AB]5JeD[AF]@=BU0A:
DP=24004>LhikSW>>Nh7=Jd4>Lh3@Q04?Nl00cG=>Lhec@04<H`4:DX39BT012488BPQ224X1b4800<M
1acW7@L00QcW00<HiQcW7>L00QcW0QSV00<LiaSV6<H00aS600LHiQS66>HHaQSV6<HHiP036<H00aSV
6<HHiP056<H01aSV6<HHiQBU5<DDYAC5008HaP035<DHaQS600`HaP035<DDYAC500@DY@045<DHaQC5
5<D45:D36<H6JeX10001JeX0000/JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX006]JJgY[FVmkJeY[NQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDY@<HaP0O5<DHaQC5
6<HDYAS65:DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65<D01QS6
00<LiaS66<H00QS600@DaAcW6>HLi`DHaP047>LHaQcW6>H37>L01QS67>LHaQcW6>HLi`DQ20038BPQ
22DY008U:@078BPU:B4X9BTQ:2U:9BT00RE901PYBRE9:DX]Jc6<;F/icSF]>Lhe[Cg_>LhmkcW^?Nli
kSg_?Pm243h?AS52<4Ha@S03BU801DIADYA>Te:DCY<00e:D00=F]E[FE[D015[F0ekg01eS66LiHaQW
>EkgGaMNmf<HGaMS65lGIcUS66LiHaQ_Nf]jKg]_Vg>LKi]cW7>lMkec_7ooNmioog_N00=oo`08Noio
og_nOomkgWooNmioo`9cW008Ki]_Nf]JKg][FVLiHcQW>@AS600:F]INme[FF]IFeE[FE]EO5ekgHaP2
G_L01U[fG_MJmUkgF_INm`AF]@07D[AF]E:dE[EBU5:dAU400dYB00dn3d8@?PlmkcW>?NlicSF]<J`e
[C6/=Lde[@02>Lh00cW^>Lhmk`02@Q002Ch??Nlmkcg_>NhicSF]>Lh]R`03<H`012U:;F/YBRUZ12DY
00HQ22DY8@PQ:2489BT28@P011d78@PLib480QcW00<HiQcW6>H00acW00hHaQcW6<HLiaS67>LHaQSV
6<HLiaS67>LHaQcW0QS600HDaAS66<HLiaS67>L36<H00acW6<HLi`026<H02aBU6<HDYAS65<DHaQC5
6<HDaAS65<D00aS600XDaAS65<DHaQC56<HDYAS65:DHaP<DY@036<HDYAS600DDY@<HaP07JeY_Nf]J
JgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX011S65<DHaQC50aS600PDaAS65<DHaQBU5<DDYAC5
1QS60aC51AS600@DaAS65<DHaP@Da@0:5:DDaABU6<HDaAS65:DDaABU5<D76<H26>H00acW6<HHaP02
6<H00aSV7>LHiP046<H011cW6>HHiQSV0acW00@HaQSV6<HHiP<Li`LQ20/U:@05:DXUBBU:9DTaS002
;H/01cG=>LhecCG=>LhikSW>00<mk`04>Ni243h?@Q04AS44BU800djCDYABU004DY@3E[D4F]H3G_L0
16<hHaQS>6<H15kg16<H00AS>6<HHcQ[NP9_N`04Lia_Vg>LKi/3Mkd017_NNoikgW_n1goo00AkoW_N
NoikgP=cW0=[NP04Kg]W>FLiIcT4HaP01e[fF]IJmUKEF]IFeE[F00]Nm`=F]@04D[AF]E:dE[D3DY@4
BU801d8@?Pm243W>>NhicSW^00Le[@@icP=240@mk`<ec@@aS0@YBP<U:@048@PQ:2488BP78@P011d7
7>LM1aSV1acW0aSV11S600LHiQS66>HHaQSV6<HHiP036<H00aSV6<HHiP056<H01QSV6<HHiQBU5<DD
Y@8DaA0HaP035<DDYAC500@DY@<Da@036<HDYABU008DY@<HaPI[FP40005[FP0002a_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5<DHaQC56<HDaAS6
5<DHaQC56<HDaAS65:DHaQBU0aS601lDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HD
YAS65:DHaQC56<HDaAS65:DHaQBU6<HDa@066<H011cW6<HHaQC50QS600<LiaSV7>L01AS600<LiaS6
7>L00QcW00PHiQcW6<HLiaS67>LHiQcW1B4800@U:B489BTQ:0<U:@0Q8BPU:B4X9DTU:BU:9DTYBRE9
:DX]Jc6<;F/icSF]>Lhe[Cg_>LhmkcW^?NlikSg_?Pm243h?AS52<4Ha@S1:DTIA009:DP03DYA>Te:D
009BU005CY=BU5JeE]EF]@04F]H3G_L01f<HIcUS66LiG_MS65kg009O5`0DHaQO5fLiHaQW>F<HKg][
NVmkKi]cW6nKLiac_7NmLkaoog_NOomkgP=oo`09Noioog_nOomkgWooNmioofnK009cW007Kg][FVmk
JeYW>F<hIcT016<H00YJeUkgF]IJeUKEF]IFeF<HG_MO5`9Nm`06F_INme[fG_MJmUkg15Je00IB]5Je
D[AB]5:DE[D2BU805DIABU8n3d8@?PlmkcW>?NlicSF]<J`e[C6/>Lhe[CG=>LhmkcW>>Nhn3`02@Q00
1cg_>Nhmkcg_>Lhe[CW>008aS006;H/aS2U::FXYBRe[12DY00LQ22DY8@PU:B488BPM1`038@P01acW
8@PLiacW6>HLiaSV00<Li`0>6<HLiaS67>LHaQSV6<HLiaS67>LHaQcW6<HLi`8HaP065<DHaQS67>LH
aQcW0aS600<LiaS67>L00QS600TDYAS65:DHaQC56<HDaAS65<D00aS600`DaAS65<DHaQC56<HDaAS6
5:DHaQBU6<H35:D00aS65:DHaP055:D02QS65<DHaV]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[
FP04JeX36<H011C56<HDaAS611BU21S600<DaAS66<H01AS600<DaAS66<H00QS60QC500<HaQC56<H0
0QS600@DaAS65<DHaP<Da@056<HDaAS65<DLi`036>H>6<H57>L00aSV7>LLi`037>L00ad78@PQ2004
8@P;9BT2:DX01BE9;F/YJRe[:FX00c6<1dHa00E243h??Pln3d8@008n3`04AS56DDHaAU44CW<00djC
DYA>T`03F]H015KEF]IJeU[F15kg0f<H00MS>6<HHcQS66<hHaQS>004HaP4JeX3Kg/017_nNmikoW_N
4Woo0g_N16mk00=S>6LiHcP016<H0ekg15[F1ekg15[F00=F]E:dE[D00UJe00=B]5JeDY@00U:D14YB
00=243h?@Q000cW^00<mkcF]=Jd00SF]13G=00@e[CG==JdicP8ec@@e[@03<H`a[36<00@]J`@YBP<U
:@PQ20<M1`LLi`036>HLiaSV00@Li`8HiP037>LHiQcW008Li`<HiP037>LHaQS6008HaP045<DHaQS6
6<H35<D00aS65<DDa@046<H00aC56<HDa@036<H00aC56<HDa@056<H02QBU5<DDYAS65<DHaQC56<HD
aAS611BU00<DaAS65<D00QC500<HaQC5JeX01F]J0@000F]J00008f]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDaAS65<DHaQC56<H00QC500DDYAS6
5:DHaQC500<HaP055<DHaQC56<HDa@056<H00aC56<HDa@036<H04ABU6<HDYAS65<DHaQC56<HDaAS6
5<DHaQBU6<HDYAS65<D00QS600DLiaS67>LHaQSV00@HaP035<DHaQC5008HaP037>LHaQSV00@Li`05
6>HLiaSV7>LHiP027>L01b487@LQ21d78@PM1b48008U:@058BPU:B4X9BTQ:0039BT03B4X:DXUBBU:
:FX]JbUZ;F/aS3F]<Ha6<D8`00=6<@0=@S16<Cg_@Q0mkd8@?Nm243g_BU96<DYBAS400dic00M:LU:D
CY=BU5KEF]IFe@04F]H015kgF_INme[f16<H00MW>F<HIcUS66LiHaQW>@02HaP01ElGJeYWFF]JIeT0
0fmk00=kgWooNmh04goo0g_N00]cW6mkKi]_NfLiHcQW>ElGHaQO5f<H00=Nm`AJePMNm`9JeP09E]EJ
eU:dE[EB]5JeD[AF]E:d00=BU0A:DP07?Pm243h??NlicSg_>Lh013F]00TicSF]>Lhe[CW>=JdicSF]
>Lh01SF]00<aS3F];F/00be[00TUBBU:9DTYBR4X9BTQ:2487@L00b4800`M1b487>LQ21cW8@PLiad7
7>LQ21cW7@L27>L0CAS67>LHiQcW6>HLiaS67>LHaQcW6>HLiaSV7>LHaQcW6<HHiQS67>LDaAS65<DH
aQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65<DH
aQC56<HDaAS65<DHaQBU5<DDYAS65:DHaQBU6<HDYAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00
JeY[FP04JeX00aS65<DHaP026<H00aC56<HDY@035:D:6<H01AC56<HHaQS65<D01aS60aC500<HaQC5
6<H00QS600@DaAS65<DHaP<Da@066<HDaASV6>HLiaSV3QS61AcW00<HiQcW7>L00acW0R4800<M1b48
8@P00R482bDY00LUBBU::DX]JbUZ;F/YJP03<H`7AS42?Pl02D8@?Pln3ch?@Q16<DIAAS56D@04CW<0
1DjCDYA>Te[FE]D01E[F15kg0f<H00MS>6<HHcQS66<hHaQS>004HaP4JeX3Kg/017_nNmikoW_N4Woo
0g_N16mk00=S>6LiHcP016<H0ekg15[F1ekg15[F0eJe00AB]5JeE[EF]@=BU0A:DP05@Q0n3d8@>Nhm
k`02>Nh4=Jd01cG=>LhecCG==JdecCF]008ec@03>Lhe[CF]008e[@03<H`a[36<00@]J`@YBP039BTQ
:2DY00LQ208M1`038@PM1acW00HLi`<HiPDLi`<HiP057>LHiQcW6>HLi`026>H66<H35<D011S65<DD
aAC51AS600DDaAS65<DHaQC500<HaP035<DHaQS600<HaP0:5:DDaABU6<HDaAS65<DHaQC56<H45:D3
5<D011S65<DDaAC51V]J0@000F]J0000:6mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DaAS65<DHaQC56<HDaAS65:DDaABU0aS600LDaAS65<DHaQC5
6<HDa@056<H00aC56<HDa@036<H04ABU6<HDYAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<D00QS6
00DLiaS67>LHaQcW008HaP055<DHaQS66<HDa@026<H00aSV6<HLi`047>L00aSV7>LHiP037>L02ASV
8@PM1b487@LQ21d78@PQ:0039BT01B4X9BTQ:2DY8BP00RDY00XYBRE9:DXYJRe[:FX]Jc6<<J`aS0=6
<@0A@S16<DHaAS4mkd8@?Nm243g_@Q0mkdYBAS5:DTHaCW=:LP02CW<01U:DCY=BU5KEF]IFe@AJeP04
G_MJmUkgF_H4HaP026LiHaQW>F<HIcUS66LiGaL2HaP016]JIeU[FVMI0fmk00=kgWooNmh04goo0g_N
00]_VfmkLia_NfLiHcQW>ElGHaQO5f<H00=Nm`AJePMNm`0>E]EJeU[FF]IB]5JeD[AF]E:dE[EB]5:D
CY=BU0A:DP07?Pm243h??NlicSg_>Lh013F]00TicSF]>Lhe[CW>=JdicSF]>Lh01SF]00<aS3F];F/0
0be[00LUBBU:9DTYBR4X9BTQ:0038@P03Qd78@PM1b487>LQ21cW8@PLib487>LM1acW8@P27>L0>QS6
7>LHiQcW6>HLiaS67>LHaQcW6>HLiaSV7>LHaQcW6<HLiaS66>HDaAS65<DHaQBU6<HDYAS65:DHaQBU
6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65<DHaQC56<HDaAS60QC5014D
YAS65:DHaQBU6<HDYAS65:DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00<HaQC5
6<H00QC500<DYAC56<H00aS611BU00<DaAS66<H01AS600<DaAS66<H02QS60QC500<HaQC56<H00QS6
00<DaAS65<D00QS600@HiQS66<HDa@THaP036>HLiaSV00DLi`036>HLiaSV00LLi`<Q20049BTQ:24X
8BP39BT2:DX00bE9:DXYBP03:DX01bE9:DXUBBe[:FX]JbUZ008a[003=Jdn3d8@00924004AS56DDHa
AS43@S03@Q04AS42DY@00djCDYAF]@02E[D3F]H03EKEG_MNmekgGaMS65lGHaQWFF]JIeUW>F<h00IW
>@0:JeYWFFMIJgY_Nf]jKg]c_7NmLk`GOol2Noh016]JJgY[FV]j1f<H0fLi15kg00AJeU[fF]IJmP=F
]@MBU004D[AF]E:dD[@3DY@4BU83@Q0023W^?Nlmkcg_=JdecCF]=Ld4=Jd00c6/=Jde[@03=Jd00bf;
<H`]R`02;H/00c6<;H/]J`03;F/4:DX78@P011d78@PQ22480ad700<Q21d77@L011cW00LHaQSV6<HL
iaSV7>LHiP086<H01QSV6<HHiQC56<HDa@DHaP8Da@8HaP045<DHaQC56<H25<D35:D76<H00aC56<HH
aP046<H01aC55:DDaABU6<HDaAS600@DY@8HaP055<DHaQBU5<DDY@06JeX10001JeX0000VJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQC56<HDaAS6
5:DHaQBU6<HDa@8HaP0A5:D@Y1BU5:DHaQC56<HDaAS65<DHaQC56<HDaAcW6<HHiP026<H00aC56<HD
a@036<H04ABU6<HDYAS65<DHaQC56<HDaAS65<DLiaS67>LDaAS65<D00aS600LDaAcW6<HHiQS67>LH
aP027>L01R487>LM1aSV7>LHiP8Li`048@PLiacW6>H27>L01R487@LQ22489BTQ20<U:@0`8BPYBRE9
:DXUBBU:9DTYBRE9:DXU:BU::FX]JbUZ;F/aS3F]<Ha243h?@Q0n3dYBAS5:DT8@AS5244Ha?Pm243h?
BU96<DIAAS5BU4jCDYA>TeJeD[AF]EKEF]IFeE[F0ekg00YS65lGHaQO5f]JIeU[FV<hIcUS>09W>@0=
IeUW>F]JIcU[FVLiKg][NVmkJgYg_G>lMkd05Woo00]kgWooNmi_Nf]JKg][FVLiHaQS>5lG00=S6005
HcQW>F<hG_MJmP03G_L01U[FG_MJeUKEE[EJePABU006CY=BU4jCE[EBU5Je0U:D00M>Te:DBU9>LdYB
BW8n3`02@Q001cg_>NhmkcW^>Lhe[CW>008e[@0A<J`e[C6/=Jda[3F]<J`e[C6/<H`]Jc6<;F/aS2f;
<H`YJP03;F/02RE9:DXYBRU:8@PU:B488BPQ22DY0R4800PM1b487>LQ21cW8@PLib481QcW010HaQcW
6>HLiaSV7>LHaQcW6<HHiQC56<HDaAcW6<HLi`8HaP035<DHaQS600<HaP0:5:DHaQBU6<HDaAS65<DH
aQBU6<H45:D00aC56<HHaP056<H03AC56<HDaASV6<HLiaBU6<HDYAS65<DHaQC500@DY@0>6<HDaAS6
5<DHaQBU6<I[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00<HaQC56<H00QC500<DYAC5
6<H00aS611BU0QS600<DaAS66<H04AS60aC500HHaQC56<HDaAS65<D26<H00aSV6<HHaP026<H00aC5
6<HHaP056<H36>H57>L00aSV7>LHiP077>L38@P28BP00bDY8BPU:@029BT00bE9:DXYBP05:DX39DT0
1Be[:FX]JbUZ=Jd00S6/0T8@00Dn3d8@AS56DDHa0092<003AS52<48@009240A6<@04CY=BU5:DDY@3
E[D015[FE]EJeU[F0ekg00MO5f<HGaMS66MIJeYWF@03IcT00f<hIcUW>@02IcT2IeT026]JJgY_Nf]j
Kg]c_7NmLk`FOol2Noh01GooJeY[NV]JJgX01f<H00=W>F<hIcT015kg00AJeU[fF]IJmP=F]@MBU0=B
]004E[EBU5:DDY@4BU83@Q02?Nl01SW^?Nle[CG==Jdec@He[@04<J`e[CF]=Jd3;H/0136<;H/aS2f;
12e[12U:2B480ad700DQ21d77@LM1b4800@Li`076<HHiQS67>LHiQcW6>H01AS600TDaAS66<HHiQS6
6>HDaAS65<D011S60QC500<HaQC56<H00QS60aC500@HaQBU5:DDY@THaP035<DHaQS6008HaP075<DD
YAC55:DHaQC56<H011BU00LDaAS66<HHaQBU5<DDY@06JeX10001JeX0000FKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX3000016]JJgY[FVmk11BU00<DaAS65<D02AS600lD
aAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<D01QS600<HiQS67>L011S600TDaAS65<DHaQC56<HD
aAS65<D00QS600`LiaS66>HHaQcW6<HLiaS67>LHiQcW6>H67>L07b487>LQ21cW8@PLib487@LQ224X
9BTQ:2DY9DTYBRE9:DXUBBU:9DTYBRE9:DXUBBe[:DX]JbE9:DXUBBU:00<aS0@icP0EAS5244Ha@Q16
<D8@@S0n3d8@?Pm6<D8@AS5244icBW9>LdYbE[EB]5Je00AJeP0;F_INme[fHaQO5f<HGaM[FVLiJeYW
F@04JeX03FmkJeY[NV]jKg][NW_NMkekgWNmMmeg_G_N01Uoo`07Mkec_7NmLka[FVMIJeX00V<H00UO
5f<HHaQS>6<HG_MJmUkgF_H00eJe00UB]5[FE[EJeU:DE[EBU5JeCY<00e:D015>Le:DCW=>TdicDYA6
<DYBAS5:DSh?@Q0n3cW>=LdicSG=00@e[@04>LhecCW><J`3=Jd03S6/=Jda[36<;H/aS2f;;F/YJRe[
9DTYBRE9:DX49BT02Ad78@PM1b487@LQ21d78@PLi`058@P01AcW8@PLiacW6>H00acW00`HiQcW6>HL
iaSV7>LHiQcW6>HLiaS66>H86<H01aC56<HDaAS65<DHaQC500DHaP0E5:DHaQBU6<HDYAS65<DHaQC5
6<HDaAS65<DHaQC56<HDaAS65:DHaQC500<HaP0C5<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaV]J
JgY[FVmkJeX00P000Fmk000FJeX00`00JeY[FP04JeX45:D;6<H02AC56<HDaAC55:DDaABU6<HDa@03
6<H00aC56<HHaP0=6<H01aC56<HDaAS65<DHaQC500@HaP037>LHiQSV00@HaP036>HLiaSV00<Li`03
6>HLiacW008Li`037@LLiad700<M1`@Q20@U:@06:DXUBBU:9DTYBRE91RU:00<YJRU::DX00RU:00@U
BC6<<H`aS0@icP03@Q12<48@00M24004@S16<D8`@S04CW<3E[D4F]H3G_L4HaP00fLiIeUW>@08JeX3
Kg/017NmMmeg_GOM0gNm6Goo17Nm0f]J1f<H00AJmUkgG_MNm`EF]@03E]EF]EJe00=B]0=BU004CY=B
U4jCCY<3CW<014IAAS56DDHa0d8@13W>13F]0cW>1cF]136<00<YJRe[:FX012U:12DY1b4800<Liad7
7>L01B480Qd711cW00LHiQcW6>HLiaSV7>LHiP047>L;6<H01AC56<HHaQS65<D01AS600<DYAC55:D0
0QC500<HaQC56<H00QS600<DaAS66<H01AS60aC51aS600<DaAS65<D00QC500HHaQC56<HDaAS65<D6
JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]J
JgY[FVmkJeY[NP045:D00aC56<HDa@096<H03AC56<HDYAS65:DHaQC56<HDaAS65<DHaQC500PHaP03
7>LHaQSV00@HaP095<DHaQC56<HDaAS65<DHaQC5008HaP0<7>LHaQcW6<HHiQS67>LHaQcW6>HLiaSV
1QcW01lQ21cW8@PLib487>LQ21d78@PQ:2DY8BPU:BE9:DXUBBU:9DTYBRE9:DXUBBU:9DT]JbU:;F/U
BBU:9DTYBP03<H`4>Lh04THa@Q16<D8@@S1244Ha?Pm243h?AS5244Ha@Q1>LdYbCW=:LP=F]@AJeP09
F_INme[fHaQO5f<HGaM[FVLi00=[FP0@IeU[FV]JJgY[FVmkJgY_Nf]jNmig_G_NMkekgWNmMmdIOol0
17NmLkag_G>l0f]J00=O5f<HHaP00V<H00QW>F<HG_MJmUkgF_IF]E:d0UJe00MJeUJeF]IBU5JeDYAF
]@02DY@04djCDYA>Le:DCW=BU4icCY=6<DYBAS5:DSh?@Q0n3cW>=LdicSG=00@e[@03>LhecCW>008e
[@0@<J`e[C6/=Jda[36<;H/aS2f;;F/YJRe[9DTYBRE9:DX49BT02Ad78@PM1b487@LQ21d78@PLi`05
8@P00acW8@PLi`037>L03QSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaS67>L86<H01aC56<HDaAS65<DH
aQC500DHaP0C5:DHaQBU6<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:D00aS601LDaAS65<DH
aQC56<HDYAS65:DHaQC56<HDaAS65<DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J
11BU2aS600LDaAS65<DDaABU5<DDY@036<H00aC56<HHaP0?6<H01aC56<HDaAS65<DHaQC500@HaP8H
iP037>LHaQS6008HaP<HiP037>LHiQcW00DLi`057@LLiad77@LQ20027@L38@P49BT01RU:9DTYBRE9
:DXUB@HYBP04:FXYBRU:9DT2:DX3<H`4>Lh00d8@@S124007@Q03@S000dHaCW=>L`02CW<3E[D4F]H3
G_L4HaP00fLiIeUW>@08JeX3Kg/017NmMmeg_GOM0gNm6Goo17Nm0f]J1f<H0Ukg00=JmUkgE[D015Je
00IFeEJeD[AB]5JeD[@3DY@3CY<02e:DCW=>LdicAU56<DIAAS5243h?@Q0013W>13F]0cW>1cF]136<
00<YJRe[:FX012U:12DY00<Q21d78@P0124800<Liad77>L012480Qd700<Q21cW7>L00QcW00LHiQcW
6>HLiaSV7>LHiP047>L=6<H00aC56<HDa@056<H00aBU5<DDY@045<D00aS65<DHaP086<H00aC56<HD
a@076<H35<D021S65<DHaQC56<HDaAS65<D6JeX10001JeX0000/Kg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC56<HDaAS65<DH
aQC56<HDa@<HaP095<DHaQC56<HDaAS65<DHaQC500<HaP065<DHaQS67>LHaQcW0QS600@DaAS66<HL
i`@HaP055<DLiaS67>LDa@036<H04ABU6<HDYAcW6>HLiaSV7>LHaQcW6<HLiaS67>LHiQcW6>H00QcW
00PM1acW8@PM1b487@LU:B480R4X0bDY00LYBRe[:DX]JbUZ;F/YJP07:DX03be[;H/]Jc6<;H/aS2f;
=Jda[3F]<J`mkcW>?NlikP04?Nl0148@?Nm6<D8`0THa00M:DTHaBU96<EJeD[AF]@02G_L03E[fG_MO
5f<HGaM[FVLiIeUW>FmkJgY_NfnK00=cW00;Kg]cW6mkLia_Ng>LKg]ggGNmNmig_@0LOol4Mkd03V]j
JeY_NfLiIeUW>F]JGaMS65lGG_MJmUkgF_H3DY@05TjCE[EBU5JeD[AF]E:dE[E>Te:DCY=BU4jCDYA>
TdicBW9>LdYBBW9:DTic0d8@00LicSG=>LhecCW>=Jdec@04=Jd01c6/=Jda[3F]<H`e[C6<00D]J`03
:DX]JbU:00<YBP0;8@PQ:2489BTQ224X8@PU:B488BPQ20047>L01B487>LQ21cW8@P00QcW00DHiQcW
6>HLiaSV00DLi`0:6<HLiaS67>LHaQSV6<HLiaS67>L46<H01aC56<HHaQS65<DHaQC500@HaP0:5<DD
YAS65:DHaQBU6<HDaAS65<D36<H02AC56<HDYAS65:DHaQC56<HDa@036<H05AC56<HDaAS65:DHaQBU
6<HDYAS65:DHaQC56<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP045<DDYAC55:D3
6<H00aC56<HDa@096<H01AC56<HDaAS65<D021S600<HiQS66>H01AS600<HiQS66>H011S60aSV11S6
0aBU00HHiQcW6>HLiaSV7>L36>H00acW6>HLi`067>L00ad78@PM1`048@P49BT01be[:FXYJRUZ;F/Y
JRe[00LYBP@]J`<aS007<J`e[CF]=JdicSW^>Lh01cg_14Ha00A6DDYBAU56D@9F]@03D[ANmekg009N
m`03HaQO5f<H00AW>@03JgY_Nfmk00EcW0A_V`03Kg]_VgNm00=g_Aaoo`Ag_@=[FPAW>@04HaQO5f<H
F_H3G_L4DY@3D[@4E[D01e:DCY=BU4jCDYA>Te:D009>L`03BW9:DTYB009:DP=240@icPPe[@03<J`e
[CF]00<aS0@]J`03:DXYJRU:00@YBP/Q20<Li`0:7@LLiad77>LM1acW7@LHiQcW6>H87>L00aSV6<HH
iP056<H00aSV6<HHiP086<H00aC56<HHaP036<H45:D00aC56<HDa@076<H011BU5<DDYAC521S600@D
aAS65<DHaP8Da@035:DDaABU008Da@036<HDaF]J00E[FP40005[FP0003I[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU6<HDaAS65<DHaQC5
6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D36<H00aC56<HDa@026<H011cW6<HLiaC511S6
00@LiaS66<HDa@8HaP037>LHaQcW008HaP0C5<DHaQBU5<DDYAcW6>HLiaSV7>LHaQcW6<HLiaS67>LH
iQcW6>H00QcW00PQ21cW8@PM1b487@LQ:2480bDY00TQ:2DY:DX]JbU:;F/YJRe[:FX01bU:00<]Jc6<
;F/00Rf;00PaS2f;=Jda[3F]<J`mkcW>0cg_00DikSg_?Nln3cg_00=6<@09@S1:DTHaBU96<EJeD[AF
]E[f00=Nm`0:GaMS65lGIeUW>F]JIcU_Nf]jKg/2Li`03FnKLia_Ng>LKg]cW6mkLia_Ng_NMkeggGNm
01aoo`Ag_@0@Kg][FV]jIcU[FVLiIeUO5f<HGaMNme[fG_MJmU:DCY<2DY@05EJeDYAF]E:dE[EB]5Je
CY=BU4jCDYA>Te:DCY=>LdYbCW=:DTicBU9:LP03@Q000cW>=LdicP02=Ld03CF]>Lhe[CF]<J`e[C6/
=Jda[3F]<H`a[36<00D]J`03:DX]JbU:00<YBP0;8@PU:B488BPQ22DY8@PQ:2489BTQ20047>L01B48
7>LQ21cW8@P00QcW00<HiQcW6>H01acW00XHaQcW6<HLiaS67>LHaQcW6<HLi`8HaP095<DHaQS66<HD
aAS65<DHaQC500DHaP0;5:DDaABU6<HDYAS65<DHaQC56<HDa@036<H01ABU6<HDYAS65<D00aS601LD
aAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65<DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]J
JeX016]J11BU00LHaQC55<DDYAC55:DDa@0:6<H01AC56<HHaQS65<D04AS600@HiQcW6>HHiP<HaP06
6>HHaQSV6<HHiQS60QSV00<LiaSV7>L021cW00<HiQcW6>H00acW124800<M1b487@L00RE900@YBRE9
:DXUB@TYBP<YJP04;F/YJRUZ:FX5<H`00bf;<H`icP03>Lh01cG=>LhecD8@?Pm2448`00=6<@A>L`=F
e@06HcQS66<hHaQS>6<H0V<h00IW>F<hIcU_NfnKKg/3Li`00fnKKg]_N`02Kg/00g>LKi]cW004Li`L
Ool3Ki/017>LJgY[NV]j16]J00=W>F<hIcT015kg15:D4Dic00A:DTIAAU56D@=240@icP@aS08e[@05
<J`e[C6/=Jda[003<H`01Be[;H/]Jbf;9DT00bU:00DUBBU:9DTU:B4X00<U:@8Q:0047@LQ21d77@L3
7>L01b487@LQ21d78@PM1b4800LLi`@HaP0>6>HHaQSV6<HLiaSV7>LDaAS65<DHaQC56<HDa@LHaP@D
Y@@HaP<Da@036<HDaAS600@HaP075<DHaQC56<HDYAC55:D00QC500@HaQC55<DDY@8Da@036<HDaAS6
00@DY@035<DHaQC500I[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00DDY@066<HDYAS65:DHaQBU0aS6014DaAS65<DHaQC56<HD
aAS65<DHaQC56<HDaAS65<DHaQC5008HaP066>HHaQcW5<DHaQC50QS600DHiQS67>LHaQcW00DHaP0<
7>LHaQcW6<HLiaS67>LHiQcW6>HLiaSV0QcW00HM1acW8@PHiQcW6>H47>L04B487@LQ21d78@PLib48
9BTYBRDY:DXUBBU:9DTYJRU:;F/00RU:00XUBBU::DX]JbU:;F/YBRe[:DX]J`8aS005;H/aS2f;<H`]
R`03>Lh03CG=>LhecCW>?Pm243h?AS52<4Ha@S1>LdYb009>L`0LF]IF]E[FHaQW>F<HIcUS66LiHaQW
>F<hIcUS>7>LKg]cW6nKLia_Vg>LJgY_Nf]jKg]_Vg>LKi/4Li`LOol00g>LKg]cW002Kg/03F]JKg][
FV]jJeY_Nf<hIcUS>6<HG_MO5ekg00EBU008CW=>TdicDYA>LdjCCW=BU09>L`0`BW9>LdicCY=>Le:D
AS5:DTHaBU8n3d8@?PlmkcW>>NhicSF]<H`a[36<=Jda[3F]<J`e[C6/=Jd]Rc6<;H/aS2e[<H`]JbU:
9DTYBRE9:DXUBBU:8BPU:B4X9BTQ22DY0R4800DLib487>LQ21cW008M1`068@PM1b487@LQ21d71AcW
00DHiQcW5<DHaQC5008HaP0>7>LHaQcW6>HLiaSV6<HDaAS65<DHaQBU6<HDa@HHaP0V5<DDYAS65:DH
aQC56<HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DH
aQC55:D@Y08DY@0:6<HDaAS6JeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FP@DY@8Da@05
6<HDYAC55:DDa@086<H01AC56<HHaQS65<D00aS600<DaAS66<H03AS60aSV00XLiaS66<HHaQSV6<HH
iQS66>HHaP8HiP037>LHiQcW00PLi`036>HLiaSV00<Li`@Q20<M1`06:DXUBBE99DTYBRE92BU:00<Y
JRe[:FX00RUZ00<]JbUZ<H`00S6<00@]Rc6<<H`aS0@icP06=LdicSG=@Q0n3d8@0THa00=2<4HaCW<0
0dic00UFeE[FE]ES>6<HHcQS66<hHaP00V<h00QW>F<hIcU_NfnKKg]cW6nK0W>L16mk00=cW6nKLi`0
17>L77oo00M_Vg>LKi]_Vf]jKg][NP04JeX00fLiHcQW>@04G_L4DY@ACW<2AU400dYBAU524002@Q04
>Lh4<H`01c6/=Jde[CF]<J`e[C6/00<aS004;F/]Rbe[;H/2:DX01BE9:DXUBBU:9DT00bDY0b4X00DU
:Ad77@LM1b4800<Li`078@PM1b487@LQ21d78@P01acW11S600@HiQS66>HHaP<Li`045<DHaQC56<H3
5<D76<H45:D46<H00aC56<HDa@036<H031C56<HHaQS65<DHaQC56<HDYAC55:DHaP@Da@065:DDaAC5
6<HDaAS611BU00<DaAS65<D01V]J0@000F]J00007F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgX01ABU00PHaQBU6<HDYAS65:DHaQC50aS600lDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D00QS600HLiaS66>HDaAS65<D26<H01AcW6<HLiaS67>L0
1AS600`LiaS67>LHaQcW6<HLiaSV7>LHiQcW6>H27>L01R487>LM1aSV7>LHiP@Li`0A8@PM1b487@LQ
21cW8@PU:BU:9BTYBRE9:DXUBBe[:DXYJP04:DX02BE9;F/YBRe[:DX]JbU:;F/]R`03<H`04Bf;<H`]
RcW>=LdicSG=>LhecCW>?Pm243h?AS52<4Ha@S000dic01e:LU[FE[EJeV<HIcUS66LiHaQW>F<HIcUS
>6LiHcQcW6mkLia_Vg>LKi]cW6]jKg][NVmkKi]cW6nK00AcW1aoo`03Lia_Ng>L009_N`0=JeY_Nf]J
Kg][FV]jHcQW>F<hGaMNmf<HG_L015:D00U>TdicDYA>LdjCCW=BU4icCY<014ic02i:LU:DCW=>TdHa
BU96<DYB?Pm243h?>NhicSg_>Lha[36<=JdaS3F]<J`e[C6/=Jda[3F];H/aS2f;<H`]Jc6<;F/YBRE9
:DXUBBU:9DTYBR4X9BTQ:2DY8@PU:@8Q200=7>LQ21cW7@LLib487@LQ21d78@PM1b487@L01AcW00DH
iQcW5<DHaQC5008HaP0<7>LHaQcW6>HLiaSV6<HDaAS65<DHaQBU0aS600<DaAS66<H00aS602<DYAC5
5:DHaQC56<HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAS6
5<DHaQC500<DY@0;4:@HaQC56<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP076<HD
aAC55<DHaQC56<H011BU1QS600<DaAS66<H01AS60aC500@HaQBU5<DDY@/HaP035<DHaQC500<HaP03
5<DHaQS6008HaP8HiP057>LHiQcW6>HLi`036>H87>L01ad77>LM1acW7@LLiad700LU:@<YBP<YJP03
;F/YBRU:008YBP049BTUBBDY;F/3:FX00bf;;F/]R`04<H`3=Jd4>Lh01ch??Nln3dHaAU56<DIA00A>
L`05D[AF]EJeHaQO5`02HaP00f<hHaQS>002Kg/016]jKg]c_7Nm0W>l00=cW7>lLi`00V]j00=_Nf]j
Kg/00Vmk17>L00=koWooOol05Woo0g_N00E[NVmkKg]_Nf]j009_N`A[FP03IcUS>6Li009JeP04E]EJ
eTjCDY@2CY<4BU800dIABU96D@02AU401DYBBW9:LTYbCW<014YB0dIA0cg_13F]00L]Rc6<;H/]Rc6/
=Jda[007=Jd012UZ;F/]Jbe[0bU:1BDY0b4X2R480Qd700DQ21d77@LLiad700@Li``HaP055<DHaQS6
6<HDa@046<H25<D00aS65<DHaP0:6<H35<D56<H01AC56<HDYAC55:D00QC500DHaQC55<DDYAC500@D
Y@045<DHaQC56<H35<D46<H6JeX10001JeX0000`Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC56<HDYAC55:DHaQC56<HDaAS6
5<DHaQC50aS600TDaAS65<DHaQBU6<HDYAS65:D01AS600<DaAS65<D01AS600DDaAS65<DHaQC500HH
aP087>LHaQcW6>HLiaSV7>LHaPDLi`036>HLiaSV008Li`068@PLib487>LQ21cW1bDY00HUBBU:9DT]
JbU:;F/2:DX01BE9:DXYBRU:9BT00RU:00X]JbU:;F/]Jc6<;F/e[C6<<J`aS0<e[@0E=LdicSW>>Lhm
kd8@?Nm:DTHaBU96<DicBW9>LdYbE[EB]5JeGaMS65lG009S600BIcUS66mkJgY_Nf]jMkec_7NmLiag
_G>LMke[FVmkJeY_Nf]j0Vmk17>L00=oog_nOol05Woo00YkgWooNmi_Nf]jKg][NVmkJgY_N`9[FP0=
IeU[FV<hIcUS>5[FE]EJeUKEDYA>Le:DCW<00dYB01m6DDYBAU5:DTHaBU96<DicBU9>LdYBBW9:DTic
AS5:DTHaBU8mkch??NlicSF]=Lde[C6<;F/aS2e[=JdaS005=Jd01S6/=Jda[2e[:FX]J`8YJP0::DX]
JbDY9BTQ:2DY8@PU:B489BT28@P03Qd78@PM1b487@LQ21d78@PLib487>LQ21cW8@P27>L02QSV7>LH
iQSV6<HLiaS66>HHaQcW11S600LDaAS65<DHaQC56<HDa@036<H02ABU6<HDYAS65<DHaQC56<HDa@02
6<H00aSV6<HLi`026<H051BU6<HHaQS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<H35:D0
4QS65<DHaQC56<HDYAS65:DHaQC56<I[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J0QC5
00<HaQC56<H00QS611BU11S600<DaAS66<H01AS600@DaAS65<DHaP8Da@035:DDaABU00/HaP055<DH
aQC56<HDa@066<H02QcW6>HHiQSV7>LHiQcW6>HLiaSV21cW00LM1acW7@LLiad77>LM1`079BT3:DX0
12UZ;F/YJRUZ12U:00<U:BE99BT00RUZ00D]JbUZ;H/]Jbf;00@aS0<e[@@icP07?Plmkch?AS56DDHa
AU4014ic0UJe00UB]6<HHaQS65lGHcQS66<hJgX00fmk00Qc_7NmLkac_7>LLkacW6mk0f]j0fmk17>L
0Woo00=koWooOol057oo0g_N0Vmk00E[NVmkKg]_Nf]j00A[FP04IcUS>6LiE]D3F]H3CY<00e:DBU9:
DP02BU8014IABU96DDYB0TIA00A:LTicBW9:LP=:DP96D@03BU96DCg_008mk`@e[@<]R`04<H`a[36/
<J`5=Jd01S6/=Jd]Jbe[:FX]J`<YBP@U:@8Q:0039BTQ:24800XQ20@M1`037>LM1acW00<Li`hHaP03
5<DHaQS600DHaP<Da@/HaP035<DHaQC500DHaP055<DHaQBU5<DDY@055<D00aBU5<DDY@035:D01QC5
6<HDaAS65<DHaP8Da@<HaPI[FP40005[FP0003Y[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU5<DDYAS65:DHaQBU6<HDYAS65<DHaQC56>HH
aQcW5<DHaQC56<HDaAS65<DHaQC56<HDa@@HaP047>LHaQSV5<D36<H01QC56<HDaASV6<HLi`8HaP03
5<DHaQS600HHaP067>LHiQcW6>HLiaSV0QcW010Q21cW7@LLib487>LQ21d78@PM1b487@LQ21d79BTQ
20DU:@0G:DX]JbU:;F/YBRUZ:DX]JbUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbUZ<H`]R`02<H`05CF]
<J`e[CG=>LhecCW>@Q16<D8@CW=:DTYbBU9BU4jCDYA>TeJeD[AF]@04G_L3JeX4Li`00gOMMkekgP02
Kg/01V]jKg]W>F]JIcU[FP=_N`06MkecW7NmLiaoog_N5goo0fmk1f]J00MO5f<HHaQS65[fG_MJmP04
DY@01dicBW9>LdYbCW=:DTic00A:DP=6<@0<BU96<DYBAS5:DTHaAU52<4Ha@S16<CW^0Sg_13F]02La
S2f;<H`aS3F]<J`e[Bf;<H`]Rc6<;H/aS2f;;F/YJRe[:FXYBRE9:DXQ:2DY8BPU:B4X9BTQ:2DY7@LQ
21d78@PM1b487@LQ21cW8@P00QcW00DHiQcW7>LLiaSV00DLi`066<HLiaSV7>LHiQcW1AS600DLiaS6
6>HHaQcW008HaP055<DHaQC56<HDa@026<H031SV6<HHaQC56<HDaAS65<DHaQC56<HDa@8HaP0V7>LH
aQSV5<DHaQC56<HDaAS65<DHaQC56<HDYAC55:DHaQ2T5:D@Y1S65:DHaQBU6<HDYAS65:DHaQC56<I[
FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J1aBU00@DaAS65<DDa@8HaP035<DHaQS600DH
aP075<DHaQS66<HDaAS65<D03AS600<DaAS66<H03AS600<HiQcW6>H021cW00HM1acW7@LLib487@L3
8@P01Ad78@PQ:2DY8BP012DY0RUZ00<]JbU::DX00RU:12e[00LYJRe[:FX]JbUZ;F/YJP03;F/4<H`3
=Jd4>Lh00d8`@Q12<004BU801djCDYABU5:DD[AF]E:d00ANm`=[FPAcW0=g_@A_N`04IeUW>FMIIcT3
Kg/3Lk`017NmNmikoW_N5Woo0fmk1f]J16<H00=Nme[fG_L015:D00M:LTicBW9>LdYbCW=:LP04BU83
AS43AU400dYBAS56<@02AS400d8`AS56<@03?Nl4=Jd4<H`3=Jd3<H`01Bf;<H`]Rc6<:FX00be[00DU
BBU:9DTU:B4X00<U:@058BPU:B4X8@PM1`038@P011d78@PLiad731cW0aSV00@LiaSV7>LHiP@HaP03
7>LHiQSV00hHaP035<DHaQC500DHaP035<DHaQS600DHaP095<DHaQC56<HDaAS65<DHaQC500PDY@03
5<DDYAC500<Da@046<HDaAS66<H6JeX10001JeX0000lKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU5<DDYAS65:DDaABU6<HDYAS65:DHaQC56<HD
aAcW6<HHiQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC50QS600<HiQS67>L00QS600PDaAS65<DHaQC5
7>LHaQSV2aS600HLiaSV7>LHiQcW6>H27>L041d77>LQ21cW8@PLib487@LQ21d78@PM1b487@LU:B48
1BDY01DYBRe[:DXYJRU:;F/YBRe[:FX]JbUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX00c6<01H]RcF]=Jde
[CG=>LhecCW>@Q16<D8@BW9:DTicBU9BU4jCDYA>TeJeD[AF]@ANm`=[FPAcW004Nmig_GOMJgX3Kg/0
16LiJeYW>F]J0fmk00Ig_G>LMkecW7ooNmhGOol3Kg/5JeX02FMIJeYS66<HGaMS65[fG_MJmP04DY@0
1dicBW9>LdYbCW=:DTic00A:DP=6<@0;BU96<DYBAS56DDHaBU92<4Ha@S16<@02?Nl00cW^=Jde[@02
=Jd3<H`09Bf;=Jde[CF];H/aS2f;<H`]Rc6<;H/]JbUZ;F/YJRU:9DTYBR4X9BTQ:2DY8BPU:B4X9BTM
1b487@LQ21d78@PM1b487>LQ21SV00<Li`036>HLiacW00DLi`066<HLiaSV7>LHiQcW1AS600DLiaS6
7>LHaQSV008HaP055<DHaQC56<HDa@026<H031cW6<HHaQC56<HDaAS65<DHaQC56<HDa@8HaP0V6>HH
aQcW5<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DDaA2T5:D@Y1S65:DHaQBU6<HDYAS65:DHaQC56<I[
FV]jJeY_Nf]JJgX0005_N`005V]J0`0016]J1aBU0aC500<HaQC56<H01QS600TDaAS66<HHaQC56<HD
aAS65<D07AS600<HiQcW6>H021cW00PM1acW7@LLib487@LQ21d70b480b4X12DY00<]JbUZ:FX012U:
12e[00LYJRe[:FX]JbUZ;F/YJP03;F/4<H`3=Jd4>Lh00d8`@Q12<004BU82DY@01DjCDYAB]5JeD[@0
15kg0f]J17>L0gNm16mk00AWFFLiIeUW>@=_N`07Lkag_G>lLkakgW_nNmh05Woo0fmk1f]J16<H0ekg
15:D00A:LTicBW9>L`=:LPA:DP=6<@04AU5:DTIAAU46AS40148`?Nlmkcg_13F]136<0cF]00<aS2f;
<H`00S6<00T]Rc6<;F/]JbUZ;F/UBBU:9DT00bDY00TQ:2DY8BPU:B4X8@PM1b487@L00b4800<Liad7
7>L02acW00LHiQcW6>HLiaSV7>LHiP046<H26>H00acW6<HHaP0<6<H00aC56<HDa@076<H00aC56<HH
aP036<H02AC56<HDaAS65<DHaQC56<HDa@055:D01A2T5:DDYAC55:D00QC500<HaQC55<D00QS600<D
aF]JJeX00f]J0P000F]J000086]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]J001[FV]jJeY_Nf]JJgXDaAS65<D26<H02AcW6<HHaQBU6<HDYAS65<DHaP025<D031BU6<HDYAS6
5:DHaQC56<HDaAcW6<HLi`8HaP0O5<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHiQS67>LHaQSV6<HL
iaS67>LHaQcW6<HHiQS67>LHiQcW6>H011cW00HQ21cW7@LM1b487@L58@P02Ad78@PM1bDY8BPU:BE9
:DXUB@02:DX02Be[:DXaS2f;<H`]Rc6<;F/aS004;F/01BUZ;F/]Jbe[:FX00Re[00/e[C6<=JdaS3F]
<J`e[Ch?@Q0n3d8@00=6<@04DYA>Le:DCW<3DY@05TjCE[EB]5JeHcQW>F<hIcU[NVmkJgYcW6nKLia_
VgNmLiag_FmkLia_Ng>L16mk00M_Vg>LKi]g_G>lMkec_003NmhBOol01WNmNmig_G_NJeY_N`9[FP03
IcU[FVLi00=W>@07G_MS65kgGaMFeE[FE]D014ic00i:DTHaBU96<DicBW9>LdYbCW=:LTicAS5:DTHa
0d8@00@n3dHa@Q12<0A24003>NhmkcW^00De[@0:<J`e[CF]=Lde[CW><H`e[C6<=Jd2;F/00bUZ:DXY
BP03:DX00bE9:DXU:@039BT02B489BTQ224X7>LQ21cW8@PM1`028@P04AcW6>HLiacW8@PLib486<HL
iaS67>LHiQcW6>HLiaS67>L011S600DDaAS65<DHaQC500<HaP065<DHaQS66>HHaQcW0QS600@DaAS6
6<HHiP@HaP0_5<DHaQC56<HDaAcW6<HHiQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAC55:DH
aQC56<HDaAS65<DHaQC56<HDYAC55:DHaQBU6<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J
00A[FP036<HDaAS600@HaP045:DDaABU5<D26<H00aC56<HDY@025:D25<D00aS65<DHaP036<H00aSV
6<HHiP046<H011C56<HDaAS60aC52QS600<HiQS66>H011S62acW00@Q21d78@PM1`LQ20<U:@@YBP03
:FXYBRUZ00@aS004;F/]Rbe[;H/:;F/0136/=Jda[36/0cF]00A243h?@Q0n3`=6<@04CW=>TdicCY<4
DY@01e:dE[EB]6LiHcQW>F<h00=_N`AcW003Lkag_G>l009_V`03Lia_Vfmk00=_N`=cW004Lkag_GNm
Mkd3NmhBOol2Mmd01G_NMme[NV]jKg/00fMI00A[FVLiIcUW>@ANm`03F]IFeE[F00A>L`04AU5:DTIA
AU44CW<00dYbCW=:LP02AU400dYB@Q124009@Q03?Nl;=Jd2<J`00cF]<J`]J`02;F/4:DX00bE9:DXY
BP049BT58@P27@L48@P37>L011d77>LM1acW0QSV1AcW00<HaQSV6<H07AS600<DaAS65<D01AS600DD
aAS66<HDaABU008Da@036<HDaAS6008HaP035<DDYABU008DY@<HaP045<DHaQC56<H55:D00aC55:E[
FP05JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX0
06]JKg][FV]jJeY_NaC56<HDa@8HaP0G6>HHaQS65:DHaQBU6<HDaAS65<DHaQBU5<DDYAS65:DHaQC5
6<HDaAcW6<HLi`026<H08AC56<HDaAS65<DHaQBU6<HDYAS65<DHaQC57>LHaQSV6<HLiaS66>HHaQcW
6<HLiaS67>LHaQcW6>HLiaSV7>LHiP027>L01Qd77>LQ21d78@PM1`DQ20097@LQ21d79BTQ:2DY9DTY
BRE9008YBP09;F/YBS6<;H/aS2f;<H`]Jc6<008]J`05:FX]Jbe[;F/YJP03;F/01BUZ=JdaS3F]<H`0
0cF]00@n3d8@?Pm240=6<@06DYA>Le:DCW=BU4jC0U:D01EF]E:dE[ES>6LiHcQW>F]jKg][NW>LKi]c
W6nKMkecW7NmKg]cW6mkLi`016mk00Y_Vg>LKi]g_G>lMkec_7_NMmekgQ9oo`06MkekgWNmNmi[FVmk
0V]J00eW>F]JIcUW>F<hIcUNmelGG_MS65KEF]IFe@04CW<044YBAS5:DTHaCW=:LTicBW9>LdYbCW=6
<DYBAS5243h?0T8@00=2<48@AS40148@00<ikSg_>Nh01cF]00Ta[3W>=JdecC6<=JdaS3F]:FX00Re[
1BU:00<UBBU:9BT00bDY00LQ224X8@PU:AcW8@PLi`038@P00ad77>LHiP027>L03B487>LQ21S67>LH
aQcW6>HLiaSV7>LHaQcW00@HaP035<DHaQC500<HaP095<DHaQS66<HDaAcW6<HHiQC500@HaP047>LH
aQS65<D36<H05aC56<HDaASV6<HLiaC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:D00QC5
00DHaQC56<HDaAS6008Da@0=5:DHaQBU6<HDYAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[
FP04JeX011S65<DHaQC50aS600@DaAS66<HHaP<Da@8HaP036>HHaQS600<Da@DHaP035<DHaQS6008H
aP045<DDYAC55:D36<H00aC56<HHaP066<H00aSV6<HHaP066<H47>L00aSV7>LHiP047>L38@P01Ad7
8@PQ22488BP012DY00<UBBDY:DX00bU:1be[00@YBRUZ:DXYJPH]J`04:FX]Jbe[;F/4<H`3>Lh4?Nl3
BU86DY@00djCDYAF]@02E[D01f<hHaQS>6<HKi]_NfnK00McW0A_N`04JgY[FV]jJeX3Li`2Kg/00f]j
Kg]cW002Li`4Nmh7Ool3Noh01GOMMkeggGNmLi`00fnK0fLi00AS66<hHaQS>0=[FP04IcUS>6LiHcP3
F]H4DY@014icCY=>LdjC1e:D00=:DTIAAU4014Ha0d8@00@n3cg_?Plmk`<icP/e[@L]J`049DTYBRE9
:DX69BT00b4X8@PQ20038@P00ad78@PLi`077>L00aSV7>LLi`037>L00aSV7>LHiP026>H00acW6>HH
aP036<H00acW6>HLi`026>H02AS66>HHaQSV6<HHiQC56<HDa@056<H25<D86<H00aC56<HDa@036<H0
41C56<HHaQS65<DHaQC56<HDYAC55:DHaQC56<HDaAS60aC500<HaQC55<D011BU00<DaAS65<D01V]J
0@000F]J0000<V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]j
JeY_Nf]JJgXDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DLiaS67>LDYAS65:D56<H00aC56<HD
a@036<H02aBU6<HDYAS65<DHaQC56<HDaAS65<D011S600HLiaS67>LDaAS65<D46<H37>L00aSV7>LH
iP057>L00ad78@PM1`028@P022DY8@PU:B4X9BTQ:2U:9BT5:DX2;F/02BUZ;F/]Jbe[:FX]JbU:;F/Y
BP03;F/00bUZ;F/YJP04;F/4<H`00cW>=LdicP04?Nl00dIABU96D@05DY@03DjCDYA>TeJeD[AF]F<H
IcUS66LiKg]cW6mk00EcW00:Ki]cW6mkKg][NVmkJeY_Nf]JKg/3Li`016mkJgY_Nf]j0g>L00AggG_N
NmikgPQoo`0VNmioogNmNmig_G_NKg]cW6mkLiaS>6LiHcQW>F<HIcUS66]JIeU[FV<hIcUS>6LiE]EJ
eUKEDYA>Te:DCY=BU4icDYA>Le:dDYAF]@9BU004CY=BU4HaBU84AS402T8`@Q12448@?Nm243g_@Q0e
cCW>0SG=00He[CW>=JdecCF]>Lh2=Jd00c6/=Jd]J`06;F/012U:9DTYBRE90bDY00@Q:2DY8BPU:@@Q
200=7@LQ21d77@LLib487>LM1acW8@PHiQcW6>H01AcW00LHaQcW6<HLiaS67>LDa@036<H02QSV7>LH
iQcW6<HLiaS67>LHaQcW0QS600TDaAS66<HHaQC56<HDYAS65:D00aS600<DaAS66<H00aS601XDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQC56<HDaAS65:DHaQBU6<HDYAS61ABU00XHaQBU6<I[
FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J00@HaQC56<HDa@DHaP045<DHaQC56<H25<D0
0aS66>HHaP025<D00aS65<DHaP026<H00aC56<HHaP046<H01QC55:DDaABU6<HDa@XHaP036>HHaQS6
00HHaP@Li`036>HLiaSV00@Li`038@PM1b4800@Q208U:@058BPU:BDY9DTU:@04:DX7;F/012U::FXY
BRUZ12e[00<YJRe[;F/00be[136<0cW>13g_0dYB15:D00A>Te:DDYABU0=F]@07HcQS66<hHaQ_Vfmk
Ki/01g>L16mk00A[NV]JJgY[FP=cW004JgY_NfmkKg/3Li`4Nmh7Ool01g_nOomkoWOMMkeggGNm009_
V`03Lia_VfLi009W>@04HaQS>6<HHcP3JeX016LiHcQW>F<h0e[F15:D00A>LdjCCW=>T`MBU096D@03
BU96<DHa0096<@=24004?Plmkch??Nl3>Lh;=Jd7;F/012E9:DXUBBU:12DY00<Q:2DY9BT01B4800<M
1b487>L02AcW00@HiQcW7>LLi`<HiP047>LHiQSV6>H46<H00acW6>HLi`026>H02AS66>HHaQSV6<HH
iQC56<HDa@046<H25<D96<H01AC56<HDaAS65<D01AS600XDaAS65<DHaQBU5<DDYAS65<DHaP<Da@05
6<HDaAC55<DHaP045:D35<D6JeX10001JeX0000bKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAcW
6<HLiaBU6<HDY@DHaP035<DHaQC500<HaP0;5:DHaQBU6<HDaAS65<DHaQC56<HDa@046<H01QcW6<HL
iaC56<HDa@@HaP037>LHiQcW008Li`036>HLiacW00<Li`037@LQ21d7008Q20088BPQ22DY8BPU:B4X
:DXU:@DYBP05:FX]Jbe[;F/YJP03;F/02BU:;F/YBRe[:FX]JbUZ;F/YJP04;F/4<H`00cW>=LdicP04
?Nl00dIABU96D@05DY@03DjCDYA>TeJeD[AF]F<HIcUS66LiKg]cW6mk00McW008JgY_NfmkKg][FVmk
JeY_N`=cW004Kg][NVmkJgX3Li`2Nmh00gOMNmioo`07Ool09g_NOomg_G_NMkekgVmkLia_Ng>LHcQW
>F<hIcUS66LiHaQ[FVMIJeYS>6LiHcQW>EKEF]IFeE:DCY=BU4jCDYA>Le:DCW=F]E:DD[A>T`03DY@0
1DHaBU96<DHa@S000THa0d8@00hmkd8@?Nm243G=>LhecCW>=JdecCF]>Lhe[CG=0SF]00<a[3F];F/0
1Re[00/YBRE9:DXUBBDY8BPU:B4X9BTQ:2DY00@Q200=7@LQ21d78@PLiad77>LQ21cW7@LHiQcW6>H0
1AcW00HHaQcW6<HLiaS67>L26<H031C56<HHiQcW6>HLiaS67>LHaQcW6<HLi`8HaP035<DHaQC500<H
aP055:DHaQBU6<HDa@076<H06QC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65<DHaQC56<HD
YAS65:DHaQBU6<H55:D02QS65:DHaV]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX36<H0
0aC56<HHaP056<H00aC56<HDa@046<H011C55:DDaABU0aS600<DaAS65<D00aS600<DaAS66<H03QS6
00LHiQS66>HHaQSV6<HLi`036>H01AcW6>HHiQcW7@L01AcW00@Q21d78@PM1`LU:@059DTYBRE9;F/Y
JP03;F/012UZ;F/YJRe[0RUZ12U:00LUBBU:9DT]JbUZ;F/YJP02;H/0136<<J`e[C6/13F]00U243h?
@Q0n3dicBW9>LeKEF]H00UKE1UJe00=B]5kgG_L00Ukg0f]J16mk00=cW7>lLi`016mk00]W>F<hIcUW
>F]JIeU[FVLiIeUW>FMI009_N`03JgYcW7>L00EcW003Ki]cW6nK00AcW0A_N`A[FPMS6003JeY[NV]J
00=W>@04HcQFeEJeE]D3D[@01EJeD[AF]E:dE[D00e[F15Je0dic14Ha0d8@0cg_00@ikSG==LdicP@e
[@@icP03=LdicSG=00@aS0<]J`@YBP039DTYBRE900@U:@8Q:0039BTQ:24800HQ20PLi`8HiP8Li`03
6>HLiaS6008HaP8HiP067>LHiQSV6>HLiaSV1aS611cW0aSV2aS60aBU0aS600PDaAS66<HHaQBU5<DD
YAC521S600<DaAS65:D00aBU0QS600<DaAS66<H00QS61V]J0@000F]J0000<V]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDaAS65<DHaQC56<HDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDYAS65:D26<H031SV6<HHaQC56<HDaAS65<DHaQC56<HDa@8HaP06
7>LHaQSV5<DHaQC511S6018LiaS67>LHaQcW6<HLiaS67>LHaQcW6<HLiaS68@PLib486>H37>L02Ad7
8@PM1b488BPU:B4X9BTQ:0029BT04BU:9DTYBRUZ;F/YJRe[:FX]JbUZ;F/YBRe[:DX]JbU::FX00RU:
00DUBBU::FX]JbUZ008]J`06<H`]JcF]<J`e[C6/0cF]00Xn3d8@?Pm244YbCW=:LU[FE[EJePIF]@03
D[AF]Ekg00=Nm`0MIeU[FV]JKg][NVmkJgYg_G>LMke_NfnKKg]cW6<hIcUS>6LiIeU[FVMIJeYW>F]J
IcU_Nf]jKg]_V`07Li`02VnKLia_Vg>LKi]cW6mkKi]_Ng>L0V]J00AWFF]JHaQS>0ES6009Kg][FVmk
HcQW>F<hIcUF]E[F009F]@0:DYAF]E:DE[EB]5JeD[AJeUKEF]H4E[D00dYbCW=:LP03AS404D8`@Q12
448@>NhmkcW^?Nle[CW>=JdecCF]>Lhe[CW>=Ld00cW>00<ecCW><H`00c6<018]Jbf;;F/YBRE9:DXU
BBU:9DTYBR4X9BTQ:2DY8@PU:B489BT48@P00ad78@PM1`077>L01aS67>LHaQcW6>HLiaSV00@HaP07
7>LHaQcW6<HLiaS67>L011S600XDaAS66<HLiaSV7>LHaQcW6<HLi`8HaP0?5<DHaQC56<HDaAS65<DH
aQC55:D@Y1BU5<DHaQC5008HaP067>LHaQS65:DHaQBU1AS600dDaAS65<DHaQC56<HDYAC55:DHaQC5
6<HDa@036<H01f]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00<HaQC56<H00QS600<D
aAS66<H00aS600<DaAS65<D011S600@DaABU5<DDY@<HaP055<DHaQC56<HDa@0;6<H00aC56<HHaP04
6<H01QSV6<HHiQS66>HHaP8HiP047>LHiQSV6>H27>L00ad77>LLi`037>L01R487@LQ21d79BTQ:0DU
:@039DTYBRE900<]J`04:FX]JbUZ;F/3:FX00be[:DXYBP02:DX022E9:DXUBBe[:FX]JbUZ<H`2;H/0
0c6/=Jda[004=Jd0148@?Pm243h?0dic0eKE00=JeUJeE[D00UJe00=B]5JeE[D015kg0f]J16mk00=c
W7>lLi`016mk0fLi00US>6]JJeY[FVLiIeUW>FMIJgX00Vmk1g>L00=_Vg>LKi/017>L16mk16]J1f<H
00E[FV]jJeYW>F<h009W>@05E]EF]EKED[AF]@03D[@00eJeD[AF]@03F]H4E[D3CW<4AS43@Q001Cg_
>Nhmkcg_>Lh00SG=13F]13W>00<ecCW>=Ld0136<0be[12U:00<UBBU:9DT01BDY0b4X1b481acW0QSV
00<LiaSV7>L00QcW0aS600DLiaSV6>HHiQcW00<HiPLHaP<Li`8HiP037>LHiQS600XHaP<DY@036<HD
aAS600@HaP045:DDaABU5<D:6<H45:D00aC56<HHaP046<H6JeX10001JeX0000XKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaC56<HDaAS65:DHaQBU
6<HDaAS65<D36<H02aC56<HHaQS65:DHaQBU6<HDaAS65<D01AS600TDaAS66<HHiQS67>LDYAS65:D0
0aS600dDaAcW6<HHiQS67>LHiQcW6<HLiaS66>HDa@026<H05QcW6<HLiaS67>LHiQcW7@LQ21d78@PM
1b487@LQ224X9BTQ:2U:9DTYBRE912U:010]JbU:;F/YJRe[:FX]JbUZ;F/YJRU:9DTYBRE9:DXUB@8Y
BP0F;F/YBRe[;F/]Rbe[=JdaS36/<H`icSF]>Lhn3d8@?Pm244jCDYA>TekgF_H2G_L00eJeD[AF]@02
E[D02E:DE[EO5f<HGaMS66LiJeYW>@03Kg/04F]jLia_Ng>LKg]cW6mkLiaWFF]JIeU[FVMIJeYWFFLi
HcP00VLi1F]J00QWFF]JJeY[FVMIJeYW>F]J16Li00eS>6LiIcUW>F<HIcUS66LiGaMS65lGJeYWF@05
JeX4HaP01U:dE[EB]5:DCW=BU09>L`0?BW9>LdYbF]IF]E[FE[EJeUJeF]I:LTicBW96<D8`0096<@09
@Q0n3d8@>NhmkcW^?NlicSg_008icP0@=LdicSG=>LhecCW>=Lde[C6<=Jd]Rc6<;H/aS2U:;F/2:DX0
0bE9:DXUB@039BT03R489BTQ22DY7@LQ21d78@PM1b487@LQ21cW8@P47>L00aSV7>LHiP027>L02B48
7>LLiaSV7>LHiQcW6<HLi`056<H02QSV6<HLiaC56<HDaAS65:DHaQBU11S600<LiaS66>H011S6014D
aAS65<DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU00DHaP035<DHaQC500<HaP0E5<DHaQC56<HD
aAS65<DHaQC56<HDaAS65<DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0aS60aC5
00@HaQC56<HDa@PHaP065<DDYAC55:DHaQC51aS600<DaAS66<H00QS60QC500<HaQC56<H01QS600<H
iQcW6>H021S600<HiQS66>H00QcW00DHiR488@PQ21d700<Q20077@LU:B4X9BTUBBU:9DT012U:00H]
JbUZ:FXYJRe[:FX5;F/7:DX2:FX00be[:FX]J`02;F/4<H`3=Ld01d8@?Pm2448@DYA>Te:D00ANm`AF
]@07D[AF]E:dHaQO5f<HGaL00VMI00=[FVmkKg/00fmk00=_VfmkLi`00fnK00M[FVMIJeYWFF]JIeU[
FP04IcT:JeX016LiIeUW>FMI1fLi00=S>6<HHcP016<H1f]J16<H00=F]E:dE[D00djC00=BU4icCW<0
0Tic00YF]EKEE[EFeEJeE]EF]DicBW9>L`A6<@=24003?NlikSg_008ikP07>LhikSG=>LhecCW>=Ld0
0cW>00DaS36/<H`aS2f;008aS003:FXYBRUZ008YBP039DTYBRDY008U:@8Q:0039BTQ:24800HQ20<M
1`038@PLiacW00XLi`046>HLiaS66>H<6<H35<D?6<H011C56<HDaAC50QS600@DaAS66<HHaP8Da@03
5:DDaAS600<HaP035<DHaQC500DHaP055<DHaQC56<HDa@076<H00aC5JeY[FP04JeX10001JeX0000Z
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQC5
6<HDaAS65:DHaQBU6<HDaAS65<DHaQC50aS600TDaAS65:DHaQBU6<HDaAS65<D01AS600/DaAS66<HL
iaS66>HDYAS65:DHaQC5008HaP0;6>HHaQcW6<HLiaSV7>LHaQSV6<HLi`026<H05aC57>LHaQcW6<HL
iaSV7>LM1b487@LQ21d78@PM1b488BPU:B4X:DXUBBU:9DT012U:00h]JbU:;F/YJRe[:FX]JbUZ;F/Y
JRU:9DTYBRE912U:01@]JbU:;F/]Jc6<;F/a[36<=JdaS3W>=JdicSh?@Q0n3d8@CY=BU4jC0ekg015J
mUJeE[EF]E:dE[EBU5JeGaMS65lGHaQW>F]JIcU_Nf]j009_N`0>Lia_Ng>LKg]cW6mkLiaWFF]JIeU[
FVMIJeYWF@=W>@05HcQ[FV]JJeYWF@03JeX01VMIJeY[FV]JIcU[FPIW>@09HcQW>F<HIcUS66LiGaMS
65lG00=[FP04IeU[FV]JJeX4HaP01U:dE[EB]5:DCW=BU09>L`0=BW9>LdYbF]IF]E[FE[EJeUJeF]I:
LTicBW800dHa00Y2<48@?Pm243W^?NlikSg_>Lhmk`8icP0@=LdicSG=>LhecCW>=Lde[C6<=Jd]Rc6<
;H/aS2U:;F/2:DX00bE9:DXUB@039BT03R489BTQ22DY7@LQ21d78@PM1b487@LQ21cW8@P47>L00aSV
7>LHiP027>L02Ad77>LLiaSV7>LHiQcW6<HLi`056<H031cW6<HHiQC56<HDaAS65:DHaQBU6<HDa@8H
aP036>HHaQcW00@HaP0A5<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DHaQBU6<HDY@056<H06aC56<HH
aQS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<I[FVmkJeY[NV]JKg/00001JgX001I[FP03
001[FV]J00A[FP036<HDaAS6008HaP<Da@036<HDaAS600LHaP065<DDYAC55:DHaQC51AS600<DaAS6
6<H01AS60aC51aS600<HiQcW6>H021S600@HiQS66>HHiP8Li`038@PM1b48008Q20097@LQ22489BTQ
:2DY9DTYBRE900@YBP8YJP03;F/YJRe[00H]J`LYBP04;F/YJRUZ:FX3;F/4<H`00cG=>Lhec@03@Q00
13h?DYABU5:D15kg15Je0e:d00ES65lGHaQO5f]J009WF@E_N`0=Ki]_NfnKKi]cW6nKJeYWFF]JIeU[
FVMIJeX016Li2V]J00AW>FMIIcUWF@MW>@03HcQS66<h00AS60M[FPAS6005E[EB]5JeCY=BU002CY<4
CW<02UJeE]EF]EKEE[EFeEJeCW=:LTic14Ha0d8@00<mkcW^?Nl00SW^00DicSW^=LdicSG=00<icP04
=LdicS6<<J`4<H`01Bf;:FXYBRUZ9DT00bU:12DY0b4X1b4800@M1b487@LM1`XLi`066>HLiacW7>LH
aQSV31S600@DaAS65<DDa@hHaP<Da@036<HDaAS6008HaP065<DHaQS65<DDYAC511S600<DaAS65<D0
1aS600<DaAS65<D01AS600<DaAS66<H01V]J0@000F]J0000B6mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DaAS65<DHaQ2T5:D@Y1S65<DHaQC56<HD
YAS65:DHaQBU5<DDYAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HD
a@@HaP<Li`096<HLiaS67>LHiQcW6>HLiaSV008Li`078@PLib487@LQ21d78@P012DY00<UBBU:9DT0
12U:00d]JbUZ;F/YJRe[:FX]JbU:;F/YBRe[:DX]J`02:DX00bE9:DXYBP04:DX06bUZ:DX]Jbf;<H`]
RcF]<H`e[C6<?NlicSW^?Pm243h?@Q1:LTicBW9Nme[fG_MJmUKEE[EJeP02E[D00e:dE[EJeP03F]H0
0f<hIcUS>004JeX056mkJgY_Nf]JKg][FVmkJgY_Nf]jKg]W>F]JIcUS>6<HIcUS66LiHcP5IcT2HaP0
2elGHaQNmf<HG_MS65kgGaMJeUkgF]H00Ukg02=S65kgGaMS66LiHaQ[FVMIJeYWFF]jJeY_NekgGaMN
mf<HCY=BU4jCBU96DDYBAU5BU4icDYA>LekgF]IJmUJeF]IF]EKE00A:DP0AAU5:DTIA@S1244Ha>Lhm
kcW>?NlikSg_>NhicSG=>Lhec@04>Lh02CF]<J`e[Bf;<H`]Rc6<:DX]J`05:DX00bDY8BPU:@028@P0
1Qd78@PM1b487@LQ208Li`0B6>HLiacW7>LHiQcW6>HLiaSV7>LHiQcW6<HLiaS67>LHiQcW0QSV00`H
aQcW6<HLiaS67>LHaQcW6<HLiaS67>L46<H011C57>LHaQSV11BU11S601PDaAS65<DHaQC56<HDaAS6
5<DHaQBU6<HDYAS65<DHaQC56<HDYAC55:DHaQBU5<D86<H04AC56<HDaAS65<DHaQC56<HDYAS6JeY[
NV]JKg][FV]j00000Fmk0007JeX30004JeX20006JeX00`00JeY[FP04JeX46<H35:D021C56<HDaAS6
5:DDaABU5<D35:D01QC55:DDaABU6<HDa@DHaP035<DHaQC500<HaP035<DHaQS600THaP<Li`066>HH
aQSV6<HLiaSV1QcW00<M1acW8@P00R4800<M1bDY9BT00RDY1bU:00LYJRe[;F/]JbUZ;F/YJP03:DX0
12UZ;F/YJRUZ2bU:136<00<a[36<<J`00cW>148@00M>LdYbCW=JmUkgF_INm`06E[D00e:dF]IJeP02
F]H3IcT4JeX4Kg/3JgX016mkJgY_Nf]j0fLi16<H1fLi0f<H0elG00=S65kgG_L00Ukg0e[f15kg00IS
>6<HHcQWFF]JIeT4JeX4G_L02e:DCY=BU4IABU96DDYBCW=>TdicCY<00e[F15Je1DYB00=6DDYB@Q00
0T8@0SW^00DmkcW^?NlikSg_00PicP08<J`e[CF]<H`]Rc6<;H/]J`8YJP@YBP8U:@038BPQ224800<Q
20037@LQ21d700lLi`8HiP8Li`036>HLiaS6008HaP8HiP067>LHiQSV6<HHiQS60QSV00<LiaS66<H0
1AS611BU1AS600DDaAS65<DHaQC500@HaP8Da@036<HDaAS6008HaPLDY@THaP095<DHaQC56<HDaAS6
5<DHaQC500I[FP40005[FP0004Q[NV]JKg][FV]jJeY_N`00JgY[FP00JeY[NP00Kg][FP00JeY_Nf]J
JgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<H@Y1BU4:@HaQC56<HDaAS65:DHaQBU5<DDYAS65:DHaQBU
6<HDaAS65<DHaQC56<HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<D46<H37>L01aS67>LHaQcW
6>HLiaSV00<Li`086>HQ21cW8@PM1b487@LQ20@U:@039DTYBRE900@YBP0=;F/YJRe[:FX]JbUZ;F/Y
BRUZ:DX]JbU:;F/012U:00<UBBU::DX00RU:01/]JbU::FX]Rc6<;H/e[C6<=JdaS3W^>Lhmkch?@Q0n
3d8@BW9>LdYbG_MJmUkgF_IJeUJeE]D00UJe00=B]5JeF]H00e[F00=S>6LiHcP016]J00e_Nf]jKg][
FVmkJeY_Nf]jKg][NVmkIcUWF@02IcT01F<HHcQS66LiHcP01FLi00eO5f<HHaQS65kgHaQNmelGG_MS
65[FG_MJeP02G_L08elGG_MS66<HIcUS66]JIeU[FVMIKg][FV]jG_MS65kgGaM>Te:DCY=:DTIABU96
DE:DCW=BU4icF_IJeUkgE[EFeEJeF]H014YB0156DDYBAU56<D8@@S0icSg_>LhmkcW^?NlikSW>=Ldi
cSG=00@icP09=Jda[3F];H/aS2f;<H`YBRe[00DYBP049BTQ:2DY7@L38@P01Ad78@PM1b486>H00acW
01hHiQcW7>LLiaSV7>LHiQcW6>HLiaS67>LHaQcW6>HLiaSV7>LHaQSV6<HLiaS67>LHaQcW6<HLiaS6
7>L26<H01QC56<HHaQSV6<HLi`@DY@@HaP075<DHaQC56<HDaAS65<D00aS600HDYAS65:DHaQC56<H2
5<D01ABU6<HDYAC55:D02AS6014DaAS65<DHaQC56<HDaAS65:DHaV]JKg][FV]jJeY_N`00005[NP00
1f]J00@006]JJeX0009[FP04001[FV]J0005JeX40003JeX46<H35:D021C56<HDaAS65:DDaABU5<D3
5:D011C55:DDaABU1aS60aC500<HaQC56<H01QS600<DaAS66<H00aS60acW00HHiQS66>HHaQcW6>H6
7>L011d77>LQ21d70R4812DY00<YBRE9:DX012U:0Re[00DYJRe[:FX]JbUZ00<YBP<YJP03;F/YBRU:
00TYBP07<H`]Rc6<<H`a[36<<J`00cW>148@00M>LdYbCW=JmUkgF_INm`04E[D00e:dE[EF]@04F]H3
IcT4JeX3Kg/2JgX01VmkJgY_Nf]jKg][NP=W>@AS60MW>@=S6004GaMS65lGGaL3G_L2F_H00ekgF_IN
m`03G_L01V<hHaQS>6MIJeYWF@A[FPANm`0;DYA>Te:DAU5:DTIABU9>LdjCCW=>T`03F]H4E[D3BU80
14IABU9:DTYB0d8@00LmkcW^>NhikSg_>Nhmk`08>Lh2=Jd01C6/<H`]Rc6<;H/00RUZ00<]JbU::DX0
0RU:00<Q:2DY9BT01B4800<M1b487@L03QcW0QSV00<LiaSV7>L00QcW0aS600<LiaSV6>H00QSV00@H
aQSV6<HLi`8HiPLHaP@DY@<HaP075<DHaQS66<HDaAS65<D01AS60aC500<HaQC56<H01aBU1aS600PD
aAS66<HHaQC56<HDaAS60aC516]J0`000F]J00001fmkJeY[NV]JKg][FV]j00<0000JJgY[FVmk001[
NV]J001[FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:DHaQBU6<HDY@@HaP037>LHaQSV008HaP07
5<DHaQC56<HDaAS65<D00aS6014DaAS66<HHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500<HaP04
5<DLiaS66>H46<H02ASV7>LLiacW6>HLiaSV8@PLi`038@P01Ad78@PU:BU:9BT012U:00P]JbU:;F/Y
BRe[:DX]JbE912U:00D]JbU:;F/YBRe[008YBP0E9DTYBRE9;F/YBRe[:DXYJRU:;F/aS36/<H`e[C6<
<J`aS3W>=LdicSh?00=240=>L`EF]@09DYAF]E:DF]IF]E[FF_INme[f009Nm`08HaQNmfLiHcQW>F<h
JeYWF@9[FP0HKg][FVmkIcU[FVLiJeYO5f<HGaMS65kgHaQNmf<HGaMS65lGHaQO5f<HE]EJeUKE0e[F
00EFeE[FE]EJeU[f00=Nm`03F_INmekg009Nm`09HaQNmfLiHaQW>F<HGaMNmf<H009F]@03D[AF]Dic
00=>L`0>BU9:LTYBE[EB]5JeD[AJeUKEF]IBU5JeDYAB]0=:DP03AS52<4Ha0096<@0@@S16<CW^?Nli
kSg_>LhmkcW>?NlikSg_>NhicSF]=Ld2=Jd00c6<=Jd]J`03;F/01RE9:DXUBBU:9BTYBP8U:@0=8BPU
:B488@PM1b487>LQ21cW8@PM1b487@L00acW00@HiR487>LM1`@Li`0:6>HLiaSV7>LHaQcW6<HHiQS6
7>L66<H01AC56<HHaQS65<D00aS600LDaAS66<HHaQBU6<HDY@026<H01AcW6<HHaQC56<H00QC500<D
YAS65:D00aBU00LDaAS65<DHaQC56<HDa@036<H01aC56<HDaAS65:DHaQBU00DHaP0A5<DHaQC56<HD
aAS65<DHaQBU6<I[FV]jJeY_Nf]JJgX00001Kg/000Q[FP03001[FV]J009[FP04001[FV]J0005JeX0
0`00JeY[FP04JeX01aC55:DDaABU5<DDYAC500PHaP075<DHaQS66<HDaAS65<D01aS600LDaAS65<DH
aQC56<HDa@0B6<H37>L00aSV7>LHiP027>L00ad77>LQ20038@P012E99BTUBBDY0bU:0bUZ00<]JbU:
:FX01BU:00DYJRU::FXYJRe[008YJP@YBP03:FX]JbUZ00@YBPLaS0<icPA240=>L`AF]@07DYAB]5:D
D[AFeE[FE]D015kg00AO5ekgGaMS>0=W>@=[FP05Kg][NV]jJgY[FP03IeT3HaP015lGHaQO5elG0V<H
00EO5f<HHaQS65lG00YJeP]Nm`04HcQW>F<hHcP3G_L4E[D3CW<4BU801E:dE[EB]5JeE]D00U[F15:D
0dYB14Ha00=2<4Ha@S0013g_0SW^1Cg_13F]0c6/12e[0bU:00LU:BE99BTUBB4X9BTQ:0048@P011d7
7>LM1acW0b4831cW00@HiQcW6>HLi`8HiQDHaP045<DDYAC55:D36<H011C56<HDaAS61aBU0aS600<D
aAS66<H01AS600@DaAS66<HHaP<Da@<HaP0;5<DHaQC56<HDaAS65<DHaQBU5<DDY@06JeX10001JeX0
0008JgY[FVmkJeY[NV]JKg][FP<00003JeY[NV]J0080000DJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]J
JgXDYAS65:DHaQBU6<HDY@@HaP036>HHaQcW008HaP055<DHaQC56<HDa@036<H05AC56<HHaQS65<DH
aQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDa@026<H00aSV6<HLi`046<H27>L02ASV7>LHiQcW
6>HQ21cW8@PM1`038@P00bDY:DXU:@04:DX01be[:DX]JbU:;F/YBRe[008YBP089DTYBRU:;F/YBRe[
:DX]J`8YBP0D9DTYBRE9;F/YBRe[:DX]JbU::FXaS3F]<H`a[36<=JdaS3W>=LdicP924003?Pm244ic
009>L`EF]@09DYAF]E:DF]IF]E[FF_INme[f009Nm`08HaQNmfLiHcQW>F<hJeYWF@9[FP0JKg][FVmk
IcU[FVLiJeYO5f<HGaMS65kgHaQNmf<HGaMS65lGHaQO5f<HE]EJeUKEF]IFe@=JeP05E]EJeUkgG_MJ
mP03G_L025[fG_MNmelGG_MW>F<HIcT2HaP00ekgGaMB]003E[D3CW<03dYbBU9>LdYBE[EB]5JeD[AJ
eUKEF]IBU5:dDYAF]@03BU83AS404T8`AS52<4Ha>NhmkcW^?NlicSg_>LhmkcW^?NlikSG==JdicP8e
[@03<H`e[Be[00<]J`069DTYBRE9:DXU:BU:0RDY00<Q:2DY7@L00b4800TLib487>LQ21d78@PM1acW
6>H00QcW00<M1acW8@P011cW00XHiQcW6>HLiaS67>LHaQcW6<HHiP@HaP055<DHaQS66<HDa@036<H0
0aC56<HHaP036<H00aBU6<HDY@026<H02ASV6<HHaQC56<HDaAS65:DDa@045:D02AC56<HDaAS65<DH
aQC56<HDa@036<H01AC56<HDYAS65:D01AS6014DaAS65<DHaQC56<HDaAS65:DHaV]JKg][FV]jJeY_
N`00005[NP005V]J00<006]JJeX016]J00HDaABU5<DDYAC55:D25<D01AS65<DHaQC56<H00aC500<H
aQC56<H01QS600LDaAS66<HHaQC56<HDa@046<H45:D86<H00aSV6<HLi`077>L00ad77>LM1`037>L4
8@P49BTA:DX49BT4:DX01bUZ;F/YJRUZ:DXYJRU:00<aS004<J`e[C6/=Jd3>Lh2?Pl0248@?Pm:LTYB
BW9>Te:DCY<5DY@01E:dE[EF]E[FE]D01E[F00AO5f<HGaMS609W>@03HcQW>FLi009W>@AS60=Nm`=J
mP04G_MJeU[fF]H4E[D01EKEE[EFeEKEF]H00UKE0e[F0U[f00=Nme[fG_L02Ukg00=JeU[fF]H015:D
1dic0UJe00EB]5JeDYAB]5:D00A>L`=6<@04AU5:DTYBBU83@S09?Nl013W^?NlecCW>0SG=0SF]00Da
[2e[:FX]JbUZ00<YBP@U:@@Q20097@LQ21d77@LLiad77>LM1acW008M1`058@PM1b487>LM1`087>L3
6>H01QcW6<HHaQS67>LHiP8Li`046<HDaAS65<D86<H01QSV6<HDaAC56<HDa@<HaP<HiP037>LDaAS6
00DHaP@DY@DHaP035<DHaQS600DHaP<Da@THaP035<DHaQC500I[FP40005[FP00031_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU6<HDYAS6
5:DHaQC56<HDaAS65:DHaQBU6<HDaAS65<D76<H03AC56<HDaAS65<DHaQC56<HDaAS65:DDaABU00PH
aP047>LHaQcW6>H37>L02ASV7>LHiR487>LQ21cW7@LLi`058@P49BT01RU::FXYBRe[:DXYJP@YBP03
9DTYBRE9008YBP0H;F/YBRE99BTYBRDY:FXYBRe[:DX]JbUZ;F/YBRe[:DX]Jbf;<H`]RcF]<J`e[C6/
0cW>01Dmkd8@?Nm244YBCW=:DU:DCY=BU4jCD[ABU5JeDYAF]E:dE[EFeE[FE]D00e[F00MFeF<HGaMS
65lGIcUS>005IcT4HaP02U[fG_MNmekgF]INme[FG_MJeUkg0UJe00]B]5JeE[EJeUJeF]IF]E[FE[EJ
eUKE009JeP0AG_MJeUkgF_INme[fG_MJmUkgF_INme[fG_MJmUkgF]INm`02DY@00djCDYA>L`06CW<0
5EJeD[AF]E:dE[EBU5JeBW9>LdYbCW=6<DYBAS5:DTIABU96DDHa@Q16<@02?Nl033W^?NlikSg_>Nhm
kcW^?NlikSW>=JdicP8e[@07<J`e[BUZ;F/YJRe[9DT00RU:12DY01PQ:2489BTM1b487@LQ21cW8@PL
ib487>LQ21cW8@PM1b487@LQ21cW8@PHiQcW6>H57>L00aS67>LHaP036<H01aSV7>LHiQcW5<DHaQC5
008HaP036>HHaQS600<HaP0=7>LHaQcW5:DHaQBU6<HDaAS65<DLiaS67>L00QS600HDaAS65<DHaQC5
6<H45:D26<H04AC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<D00aS600dDaAS66<HHaQC5
6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP065<DDYAC55:DDaABU0QC500<HaQC5
6<H00aC500@HaQC56<HDa@THaP055<DHaQC56<HDa@046<H45:D86<H00aSV6<HLi`077>L00ad77>LM
1`037>L48@P49BTA:DX49BT4:DX01bUZ;F/YJRUZ:DXYJRU:00<aS004<J`e[C6/=Jd3>Lh02T8@?Pln
3ch?BW9:DTYbCY=BU4jC1E:D0UJe00EB]5[FF]IJeUKE00=JeP05GaMS65lGHaQS>006IcT4HaP3G_L0
1e[fG_MJmU[fF]IJmU[F00AF]@03E]EF]EKE00=Fe@AJeP04G_MJmU[fF_H5G_L00e[fG_MNm`03G_L0
0e[FF_IJeP04DY@7CW<01e:dE[EF]EJeDYAB]5:D00A>L`=6<@9:DP05AU5:DT8`AS52<007?Nl013W^
?Nlmkcg_0cG=00PicS6/=Jde[Be[:FX]JbUZ0bU:12DY124800TM1b487@LM1acW7@LLiad77>L00Qd7
00DQ21d78@PLiad700PLi`046>HLiaSV6>H36<H37>L01ASV6<HDaAS65<D021S600<HiQS66<H00aC5
0aS600@HiQcW6>HHiP8HaP035<DHaQS6008HaP@DY@<HaP035<DHaQS600DHaP045<DHaQS66<H25<D8
6<H00aC56<HDa@06JeX10001JeX0000^JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65<D36<H0
0aC56<HHaP036<H03QC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU5<D76<H00acW6<HLi`027>L031SV
7>LHiQcW6>HQ21cW8@PLib487>LM1`@Q20@U:@06:DX]JbU::FXYBRe[0RU:00DUBBU:9DTYBRE9008Y
BP0b:FXYBRU:9BTUBBDY;F/YBRUZ:DX]JbUZ;F/YBRe[:DX]Jbf;<H`]RcF]<J`e[C6/>LhecCW>?Nm2
43g_@Q1:DTicBU9BU4jCDYA>TeJeDYAB]5:DE[EB]5JeE]EJeUKEF]IFe@9JeP06HaQO5f<HGaMW>F<h
1FLi16<H0Ukg00UJmUkgF]INme[FG_MJeUkgD[@015Je00QJeUJeF]IF]E[FE[EJeUKE0U[F019Nme[F
G_MJmUkgF_INme[fG_MJmUkgF_INme[fG_MJeUkgCY<3DY@7CW<05UJeD[AF]E:dE[EBU5JeBW9>LdYb
CW=6<DIAAS5:DTIABU96DDHa@Q16<CW^0cg_00XikSg_>NhmkcW^?NlikSW>=JdicP8e[@06<J`e[BUZ
;F/YJRe[0RU:00<UBBDY9BT00bDY01LQ224X7@LQ21d78@PLib487>LQ21cW8@PLib487@LQ21d78@PL
ib486>HLiaSV00DLi`036<HLiaS600<HaP076>HLiaSV7>LDaAS65<D00QS600<LiaS66<H00aS600dL
iaS67>LDYAS65:DHaQC56<HDaAcW6<HLi`026<H01QC56<HDaAS65<DHaP@DY@0A5<DHaQS66<HDaAS6
5<DHaQC56<HDaAS65<DHaQBU6<HDY@036<H03aC56<HHaQS65<DHaQC56<I[FVmkJeY[NV]JKg/00001
JgX001I[FP03001[FV]J00A[FP036<HDaAS6008HaP045<DHaQC56<H25<D45:D26<H00aC56<HHaP02
6<H00aSV6<HHiP086<H01AC56<HDaAS65<D00aS600<DaAS66<H00aS600<HiQcW6>H00acW00@HiQcW
7>LLi`@Q20<Li`@Q20049BTQ:24X8BP3:DX4;F/2:DX00bE9:DXYBP02:DX02bE99BTUBBE9:DXUBBU:
9DTYBRE9:DX00bUZ12e[00<aS2f;<H`0136<0cF]148@0TIA0TYB00=:LTYBBW8015:D15Je0e:d0eJe
15[F0V<H00EO5f<HHaQS65lG00=W>@04HcQS66<HHaP3E]D01U[FE]EJeUKEF]IFe@9JePQF]@03E]EF
]E[F00=JePANm`03GaMNmelG00AS6005E]EJeUKECW=:LP05CW<4DY@4E[D00djCDYA>T`04BU82@S05
AS43@Q0013g_>NhmkcW^1cg_13F]0c6<0RUZ00<]JbUZ:DX00RU:12DY224800<M1b487@L01acW00HM
1b487@LM1acW7@L47>L01ASV7>LLiacW6>H00acW0aSV2AS600DDaAS65<DHaQC500@HaP8Da@096<HD
aAS65<DHaQC56<HDaAS600<Da@@HaP<DY@045<DHaQC56<H35:D011S65<DHaQC50aS611BU00LDaAS6
5<DHaQC56<HDa@06JeX10001JeX0000bKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_NaC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU5<DDYAS65<DHaQC5
6<HDa@8HaP037>LHaQS600@HaP0I5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DLiaS66>HHaQcW6>HL
iaSV7>LHiQcW6>H00QcW00XQ21d78@PM1acW6>HLiad78@PM1`8Q20039BTQ22DY00<YBP<]J`0E:FXY
BRE9:DXUBBU:9DTYBRDY:DXU:BU:9DTYBRE9:DXUBBU:9DT]JbU:008]J`0>;H/]Jc6<;H/aS2f;=Jda
S36/<H`icSF]=Ldn3`=24009AS5:DTHaCW=:DTicBU9BU4jC009BU0=F]@03DYAF]E:D009F]@03E]EF
]E[F00=JeP0IHaQO5f<HGaMS65lGHaQS>6LiHcQW>ElGHaQO5e[FE[EJeUJeF]IFeE[FE]EJeUKEF]H0
0UJe00=B]5JeE[D00UJe00IJeUJeF]IJeUkgF]H2F_H4G_L036<HG_MS65lGHaQO5e[FE]EJeTYbCW=:
LP=>L`03BW9BU5:D009BU0=F]@0ED[ABU4icDYA:DTicBU9:LT8@AS5244IAAS5:DTHa@Q0n3d8@>Nhm
kcW^008mk`03?Plmkcg_008mk`05>Nhe[C6/=Jda[003<H`01BU:;F/YBRe[9DT00RU:00@UBBDY:DXU
:@LQ20047@LQ21d78@P37>L06B487>LM1acW8@PLib487>LQ21cW8@PHiQcW6>HLiaSV7>LHiQcW6>HL
iaS67>LHaQcW008HaP045<DHaQS66>H26<H071C56<HDaAS65:DHaQC56<HDaAS65:DHaQBU6<HDaAS6
5<DHaQC56<HDaAS65:DHaQC56<HDaAS60aBU00`HaQC56<HDaABU4:@DYAC56<HDaAS65<D26<H04QC5
5:DHaQBU6<HDaAS65<DHaQC56<I[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J0aS600@D
aAS65<DHaP<Da@036<HDYABU008DY@035<DHaQS600@HaP036>HHaQSV00PHaP035<DHaQC500<HaP03
5<DHaQS600DHaP056>HLiaSV7>LHiP057>L48@P37>L48@P28BP00bDY8BPYBP02:DX4;F/00bE9:DXY
BP04:DX03RE99BTUBBE9:DXUBBU:9DTYBRE9:DXYJRe[:FX4;F/00c6<;H/aS004<H`3=Jd4@Q001dYB
AU56DDYBBW9:DTYb00ABU0=F]@9B]003E[EB]5Je009F]@AJeP05GaMS66<HHaQO5`02HaP016LiHcQW
>FLi0f<H00=FeE[FE]D00UKE00=JeUKEF]H00U[F00=FeEJeE[D01UJe00=FeEJeF]H00e[F15kg00=O
5ekgGaL016<H00=FeE[FE]D00dic00A:LTicCW=>L`ABU0AF]@=>T`A:DP03AS52<48`00A6<@=24004
?NlikSg_>Nh7?Nl4=Jd3<H`012e[:FXYJRUZ0bU:12DY224800<M1b487@L01acW11d700DLiad77>LL
iaSV00<Li`046>HLiacW7>L36>H00acW6>HHaP066<H011C56<HHaQS60aC51AS60aC500XHaQC56<HD
aAS65<DHaQC56<HDa@@HaP<DY@045<DHaQC56<H35:D011S65<DHaQC50aS611BU00LDaAS65<DHaQC5
6<HDa@06JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeX006]JJgY[FVmkJeY[NQBU6<HDY@8Da@066<HDaAS65<DHaQC51AS600DDaAS65<DHaQC500@HaP03
7>LHaQSV008HaP0?5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500<HaP055<DHaQcW7>LHiP02
7>L00b487>LLi`027>L00aSV8@PM1`068@P022DY8@PU:BDY:DXU:BU:9DT3:DX00bE9:DXYBP03:DX0
5BE9:DXUBBU:9DTYBRE9:DXUBBU:9DT]JbU:;F/YJRe[:FX]Jbf;<H`]R`04<H`01cW>=JdicSh?@Q0n
3d8@0096<@03@S1>Ldic009>L`0:DYA>Te:DCY=BU4jCDYAB]5JeD[@2E[D025[FE[ENme[FG_MJeUlG
G_L3HaP01ElGHaQO5f<HGaL00f<H01=O5ekgF_INme[fG_MJeUkgF_INme[fG_MJeUkgF]INme[fG_MJ
mP07F]H01eJeF]IF]E[FE]EJeUKE00AJeP0@E[EB]5JeBW9>LdYbCW=6DDYBAU5BU4icDYA>Le:DCY<3
DY@01TicDYA6DDYBAU5:DP96<@0>@S16<D8`AS52<48@?Pm243W^?NlikSg_>Lhmk`8icP03=LdicSG=
00@e[@07;H/]Jc6<:FX]JbUZ;F/00RU:02LUBBDY8BPU:B4X9BTQ22DY7@LQ21d78@PLib487>LQ21d7
8@PM1acW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQS65<D00QS600@LiaS66>HDa@LH
aP0A5<DHaQC56>HHaQcW6<HLiaS67>LDaAS65<DHaQC56<HDa@045:D00aS65<DHaP055:D03AS65:DH
aQBU6<HDYAC55:DHaQBU6<HDYAS600<DY@076<HDaAS65<DHaQBU6<H011BU00M[FVmkJeY[NV]JKg/0
0001JgX001I[FP03001[FV]J00A[FP@DY@066<HDaAS65<DHaQC52AS600<DaAS65<D01aS600DDaAS6
6<HHaQC500LHaP035<DHaQS600HHaP@Li`037@LLiad700@Li`037@LQ21d700@Q20048BPQ224X8@P3
9BT4:DX00bE9:DXUB@05:DX00bE9:DXYBP08:DX022UZ:DX]JbUZ;F/YJS6<;H/5<H`01cF]=Lde[D8@
?Pm243h?00=6<@A>L`05CY=BU5:DDYA>T`02DY@01EJeD[AF]E:dF]H00UKE00AJeU[fF]IJmP=Nm`MS
6005GaMS66<HHaQJmP03G_L00e[FF_IJeP04G_L3F]H2G_L00e[fG_MJeP06F]H015KEE[EFeEJe1e[F
0eJe14ic0dYB00A>LdjCCW=>T`ABU003CW=>Tdic00A:DP=6<@05@S16<DHaAS4n3`02@Q0013g_>Nhm
kcW^1CW>00<ecCW>=Jd00cF]12e[00<YJRe[:FX00bU:00@Q:2DY8BPU:@<Q:0038@PM1b48008M1`03
7>LM1acW00<Q208Li`096>HLiaSV7>LHiQcW6>HLiaSV00<Li`046>HLiaSV7>LE6<H00aSV6<HHiP04
6<H011C56<HDaAS611BU0aS611BU00DDaABU5<DDaAS6008Da@<DY@8Da@036<HDaABU008DY@045<DH
aQC56<H35<D011S65:DDYABU1V]J0@000F]J0000:6mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAC55:DHaQC56<HDaAS65<DHaQC51aS600<DaAS6
5<D011S600<HiQS67>L00QS600dDaAS65<DHaQC56<HDaAS65<DHaQC56<HDa@036<H01AC56<HHaQS6
6>H011cW00@Q21cW7>LHiP8Li`038@PM1b4800DQ20069BTQ22DY9BTUBBDY0bU:00<UBBU:9DT01BU:
01DUBBU:9DTYBRE9:DXUBBU:9DTYBRE9;F/YBRe[:FX]JbUZ;F/]Rc6<;H/0136<00PicSF]>Lhn3d8@
?Pm2448`0THa14ic00YBU4jCDYA>Te:DCY=BU5:dE[EB]09F]@08F]IF]EkgF]INme[FHaQNm`9O5`=S
6005GaMS65lGHaQO5`02HaP04UkgF_INme[fG_MJeUkgF_INme[fG_MJeU[fF]INme[fG_MJmPMJeP07
E[EJeUJeF]IFeE[FE]D015[F00iF]E:dE[E:LTicBW9>LdIABU96DE:DCW=BU4ic0e:D00U>Te:DCW=B
U4IABU96DDYB@S000dHa00a2<4Ha@S1243h?@Q0ikSg_>NhmkcW>>Nh2>Lh00cG=>Lhec@04=Jd0236<
;F/]RbUZ;F/YJRe[9DT2:DX09RDY8BPU:B4X9BTQ22DY7@LQ21d78@PLib487>LQ21d78@PM1acW6>HL
iaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQS65<D26<H00aSV6<HLi`026<H00aC56<HHaP03
6<H04AC56<HDaAcW6<HHiQS67>LHaQcW5<DHaQC56<HDaAS65<D011BU00<HaQC56<H01ABU00dHaQBU
6<HDYAS65:DHaQBU5<DDYAS65:DHaP035:D01aS65<DHaQC56<HDYAS600@DY@07JeY[NV]JKg][FV]j
00000Fmk000FJeX00`00JeY[FP04JeX45:D36<H00aC56<HDa@096<H00aC56<HDa@096<H01AC56<HH
aQS65<D03QS611cW00<M1acW7@L011cW00<M1b487@L0124800@Q:2488BPQ20<U:@@YBP039DTYBRE9
00DYBP039DTYBRU:00PYBP06:FXYBRe[:FX]JbUZ1c6<00Le[CG==Je243h?@Q0n3`03AS44CW<2DY@0
2DjCDYABU5:DCY=F]E:dE[EB]002E]D2F]H00e[fF]IJmP03G_L5HaP00elGHaQS6003HaP2G_L01E[f
G_MJeU[fF]H015kg0e[F00AJmUkgG_MNm`MJeP06E]EF]EKEE[EJeUKE1E[F0eJe14ic0dYB00A>LdjC
CW=>T`ABU003CW=>Tdic00A:DPE6<@09@S16<D8@@Q0n3cg_>NhmkcW^00<icP04=LdicSW>>Lh4=Jd4
;F/00bUZ;F/YJP03:DX02R4X9BTQ:2DY8BPU:B4X8@PM1b480Qd700<Liad77>L00b48018HiQcW7>LL
iaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LE6<H00aSV6<HHiP046<H011C56<HDaAS611BU0aS6
11BU00<DaABU5<D00aC500DHaQBU5:DDYAS600<Da@<DY@065<DHaQC56<HDaAS60QC50aBU1V]J0@00
0F]J00008V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_
Nf]JJgXDaAS65<DHaQ2T0QBU01`HaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDYAS65:DHaQC5
6<HDaAcW6<HLiaC56<HDa@DHaP075<DHaQS66<HDaAS65<D00aS600<HiQcW6>H00QcW00HQ21cW8@PL
ib487>L38@P03R4X9BTQ:2DY8@PU:B489BTQ22DY8@PYBRDY:DX29BT05B4X9BTU:BE99BTYBRE9:DXU
BBU:9DTYBRE9;F/YBRe[:DX]JbU::FXUB@03:DX7<H`01SW>=LdicSh?@Q0n3`92400<AS5245:DCY=B
U4jCDYA>Te:DCY=BU4ic1E:D0e[F00iNme[fG_MJmUkgF_INme[FG_MJeUkgF_INme[f0Ukg011S65kg
G_MJmUkgF_IS65kgHaQNmf<HG_MO5e[fG_MJmPEJeP0=E]EJeUJeE[EB]5JeF]IJmU[FG_MJmUkgF_H0
0e[F03YFeE:DDYABU4YBCW=:DTicAU5:DTIACW=:LTicBW9>LdYbCW=:LTicBU9>LdYBBW9:DTicBU9>
LdYBAS52<4Ha@S0mkcW^?NlicSg_>LhikSG=>LhecCW>=LdicSG==JdaS3F]<H`]JbUZ;F/YBRUZ:DX]
J`@U:@0;8@PU:B488@PM1b487@LQ21d78@PM1`038@P03AcW8@PLiacW6>HLiaSV7>LHiQcW6<HHiQS6
00DLi`066>HLiaSV7>LHaQSV11S600DDaAS65<DHaQC500<HaP0V5<DHaQC56<HDYAS65<DHaQC56<HD
aAS65<DHaQBU5<DDYAS65<DHaQC57>LHaQcW5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaPDDY@0D
5<DDYAS65:DHaQBU6<HDYAC55:DHaQBU6<I[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX016]J
00@HaQC56<HHaP@DY@055<DDYAC55<DHaP035<D01AS65<DHaQC56<H00aC51QS600HHiQS66<HDaAS6
5<D76<H00aC56<HDa@046<H01QcW6>HLiaSV7@LLi`@M1`@Q20059BTQ:2DY8BPU:@038BP01BDY8BPQ
:2E9:DX00RE90RDY00<Q:2DY9BT00RDY00HYBRE9:DXUBBU:9DT2:DX00bUZ:DXYJP07:DX7<H`01cG=
>LhecD8@?Pm243h?0092<006AS5>Te:DCY=BU4jC0e:D00=>LdjCCW<015:D0e[F00=JmUkgF_H015kg
0U[f00=Nme[fG_L00Ukg00AJmUlGGaMS60ANm`=O5`MNm`MJePAF]@AJeP=Nm`AJeP=BU003BW9:DTYb
009:DP03AU5:DTic00E>L`05BW9>LdYbCW=:LP04BU82BW800dic@S16<@02AS43?Nl9>Lh01SG=>Lha
S36/<H`a[0<]J`@YBP<U:@<Q:0039BTM1b48008Q20037@LQ224800@Q20DLi`036>HLiacW008Li`@H
aP<Li`046>HLiacW7>L>6<H02aC56<HDaAS65<DHaQC56<HDaAS65<D00aS611BU00<DaAS65<D00QS6
00PHiQS66<HDaAS65<DHaQC50aS600DDaAS66<HHaQC500LDY@<Da@036<HDYABU00<DY@035<DDYF]J
00E[FP40005[FP00025_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00
JeY_Nf]JJgY[FVmk5<DHaQC56<H00QBU01d@Y1S65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU
6<HDYAS65<DHaQC57>LHaQcW5<DHaQC500LHaP055<DHaQC56<HDa@036<H00aSV7>LHiP027>L05b48
7>LQ21cW8@PLib487@LQ224X9BTQ:2DY8@PU:B489BTQ22DY8@PYBRDY:DX00RDY00DQ:2DY9BTYBRDY
008UB@0=:DXUBBU:9DTYBRE9;F/YBRe[:DXYJRU:;F/00RU:00<UBBU:<H`01S6<00HicSG=>Lhn3d8@
?Pl2@Q0034Ha@Q1BU4jCDYA>Te:DCY=BU4jCDYA>L`EBU0=JeP0>G_MJmUkgF_INme[fG_MJeUkgF]IN
me[fG_MJmP9Nm`0@HaQNmekgF_INme[fHaQNmf<HG_MO5ekgHaQJmUkgF_H7F]H01e:dE[EF]EJeF]IN
me[F009JmP04G_MJmU[FE]D2F]H3DY@0;dYBCW=:DTicAU5:DTIACW=:LTicBW9>LdYbCW=:LTicBU9>
LdYBCW=:DTYbBU9>LdYBAS52<4Ha@S0mkcW^?NlicSW^>LhmkcG=>LhecCW>=LdicSG==JdaS3F]<H`0
0be[00@YBRe[:DXYJP@U:@098@PU:B488@PM1b487@LQ21d700<Q20047@LQ21cW7@L27>L02QSV7>LH
iQcW6>HLiaS67>LHaQSV11cW00HHiQcW6>HHiQS67>L46<H00aC56<HDa@036<H03QC56<HDaAS65<DH
aQBU6<HDaAS65<DHaQC56<H25<D061BU6<HDYAS65<DHaQC57>LHaQcW5<DHaQC56<HDaAS65<DHaQC5
6<HDaAS65<DHaPDDY@0D6<HDYAS65:DHaQBU5<DDYAS65:DHaQBU6<I[FV]jJeY_Nf]JJgX0005_N`00
5V]J0`0016]J00PDaABU5<DDYAS65<DHaQC50aS600dDaAS65<DHaQBU5<DDYAS65<DHaQC56<HDa@08
6<H011C55:DDaABU0aS600<DaAS65<D021S611cW0aS611cW00<M1b487@L0124812DY0b481bDY12U:
0bDY00<UBBU:9DT00bU:00<UBBU::DX01RU:0be[0c6/13F]13g_00E:DTIAAU5>Te:D009>T`AF]@07
D[AF]E:dE]EF]EKEE[D00U[f00ENme[fF_IJmUkg00AJeP=Fe@05F]IFeEKEE]EJeP02E]D7F]H2GaL0
1F<HGaMJmU[fG_L015[F00=FeE[FF]H015[F1Ekg00UJmUkgE]EJeUKEF]IBU5:dDY@00TYb00=>LdYb
BU800TYB14ic14YB00=6DDYBAU40148@0dHa00LikSg_>NhmkcW^?NlikP07>Lh0136/=Jda[3F]0bf;
00HaS2UZ;F/]JbU:9DT2:DX39BT0124X9BTQ:24X224800HM1b487@LM1acW7@L77>L01aSV6<HHiQS6
7>LHiQcW00LHaP047>LHiQcW6>H;6<H01AC56<HHaQS65<D01AS611BU00/DaAS65<DDaABU5<DDYAC5
5:DDaABU008Da@056<HDYAC55:DDa@0:5:D01QC56<HDaAC54:@DY@8@Y0<DY@E[FP80005[FP0003Y[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DH
aQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HDa@<HaP0;
5:DHaQBU6<HDaAS65<DHaQC56<HDa@046<H041SV6<HLiaSV7>LHiQcW6<HHiQS68@PLiad77>LQ21cW
0R4800LU:B488BPQ:2DY8BPU:@038@P00bDY8BPU:@029BT04B4X9BTUBBU:9DTYBRDY9DTU:BU:9DTY
BRE9:DXUBBU:9DT01bU:00LYJRe[:FXe[C6<=JdaS003=Jd03Sg_?Plmkd8@AS5:DTHaDYA>Le:DCW=J
eUJeE]D2E[D01E:dE[EF]E[FE[D00U[F00ENme[FG_MJeUkg00AJeP0;E[EJeUJeF]IF]E[FE[EJeUJe
F]IF]@03F]H02UKEG_MJeU[fG_MS65kgHaQJeUkg0U[F00EFeE[FF]IJeUKE00EJePANm`0>F_INme[f
F]IFeE[FE]EF]E:DE[E:DTicBU9>L`=:DP07CY=>Le:DCW=:LTYBCW<00TYB00E6DDYB@Q12<48@0096
<@0>AU56<Cg_>NhmkcW^?NlikSg_>LhmkcW>>Nhec@8icP07=Jda[3F]<J`aS2e[<H`00Re[00HYJRe[
9DTYBRE9:DX49BT00b489BTQ20038@P02Qd78@PM1b487@LQ21d78@PLib480QcW00<HiQcW6>H00acW
00XHaQcW6<HLiaSV7>LHiQcW6<HHiP@HaP056>HLiaSV7>LDa@036<H01QC56<HHaQSV6<HLi`8HaP05
5<DHaQC56<HDa@036<H05QC55:DDYABU4:@HaQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS6
1ABU00<DaABU6<H00aBU00<HaQBU6<H00QBU00d@Q1BU48@HaQBU5<E[FVmkJeY[NV]JKg/00001JgX0
01I[FP03001[FV]J00A[FP065<DDYAC55:DHaQC50aS600lDaAS65<DHaQC56<HDYAC55:DHaQC56<HD
aAS65<D021S600HDaABU5<DDYAS65<D36<H00aC56<HHaP066<H47>L36<H47>L37@L48@P49BT38@P7
9BT4:DX39BT01BE9:DXUBBU:9DT02RU:0be[00@a[3F]<J`a[0<e[@@mk`96D@05BU9>TdjCCY=BU004
E[D025:dE[EB]5KEE[EFeEJeG_L3F_H00ekgF_IJmP03F]H2E]D01E[FE]EFeEKEF]H00eKE25[F00ES
65lGGaMO5ekg009JmPIJeP03E]EJeU[F009JePENm`0:F_INmeKEF]IFeE[FDYAB]5:DCW<3BW83BU84
CW<4BU800dIABU96D@04@Q03AS401cW^?NlikSg_>NhmkcW^00LicP06<J`e[C6/=Jd]Rc6<0Rf;0Re[
00DYJRU::DXYBRE900<U:@<Q:0039BTQ224800HQ20067@LQ21d77@LLiad71acW00LHiQS66>HHaQcW
6>HLi`076<H011cW6>HLiaSV3AS600DDaAS66<HHaQC500<HaP@DY@<Da@096<HDYAC55:DDaABU5<DD
YAS6008Da@045:DDaABU5<D:5:D35<D011S64:@@Y12T11BU1V]J0@000F]J0000B6mkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:DHaQC56<HD
aAS65<DHaQC56<HDaAS65<DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HD
aAS65<DHaQC56<HDa@@HaP0@7>LHaQSV6>HLiaSV7>LHaQcW6<HM1acW8@PLib487>L28@P01b4X8@PU
:B4X9BTQ:2DY00<Q20<U:@0C8BPU:B4X9BTUBBU:9DTYBRDY:DXU:BU:9DTYBRE9:DXUBBU:9DT01bU:
00LYJRe[:FXe[C6<=JdaS003=Jd03Sg_@Q0mkch?AS5:DTHaDYA>Le:DCW=FeEJeF]H2E[D01E:dE[EF
]E[FE[D00U[F00ENme[FG_MJeUkg00AJeP0=E[EJeUJeF]IF]E[FE[EJeUJeF]IF]E[FE]D00U[F00UJ
mU[FG_MNmf<HG_MS65[FG_L015[F00=FeE[FE]D01E[F15kg00iJmUkgF_IJeUKEF]IFeEJeDYAF]DYB
CW=:DTic0dYB00=BU4icCY<00Tic01Q:DTYbBU9:DTIABU9244Ha@Q12<4HaBU96<Cg_>NhmkcW^?Nli
kSg_>LhikSW>?Nl2>Lh023G==Jda[3F]<J`aS2e[<H`2;F/01RUZ;F/UBBU:9DTYBP@U:@038@PU:B48
00<Q200:7@LQ21d78@PM1b487@LQ21cW8@P27>L00aSV7>LHiP037>L01QS67>LHaQcW6>HLi`8HiP0:
6<HLiaS66<HDaAS66>HLiaSV7>L26<H021C56<HHaQS65<DLiaS66>H26<H01aC56<HDaAS65<DHaQC5
008HaP0E5:D@Y1BU5:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS600DDY@036<HDYAC5
00<DY@036<HDYAS6008DY@0=48@DYA245<DDYAS6JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[
FP04JeX25<D00aS65<DHaP026<H35<D011S65<DDaAC51AS600<DaAS66<H011S600<HiQS66>H00acW
00@HiQS66>HHaP@DY@LHaP<Li`046>HHaQSV6<H37>L28@P00ad78@PQ20058@P00ad78@PQ20028@P0
0b4X8@PQ:0038BP59BT38BP00bU:9DTYBP049BT4:DX2;F/01BUZ;F/YJRe[:FX00be[0c6/00De[CG=
=LdecD8@00<n3`03BU96DDIA00A>L`AJeP03E]EJeUKE00AJeP03G_MJmUkg00AJePAF]@=B]0AF]@03
F]IFeEKE00MF]@04F_IJeU[fF]H2E]D00e[FE]EJeP02F]H3G_L4HaP4G_L00e[fF]IJmP04E[D3DY@4
BU83CW<01DjCDYA>Te:DBW800dic00e2<4HaAS52<48@@S1243h??Nln3cG=>Lhec@04>Lh2=Ld00cW>
=Lde[@02=Jd4<H`012UZ;F/]Jbe[12U:0bE900<U:B4X8BP0124800LLiad77>LQ:2488BPQ20027@L0
0b487@LLi`097>L01b487@LQ21d76>HHaQSV00@HaP8Li`036>HHaQS6008HaP8HiP037>LHiQS600HH
aP035<DHaQS600DHaP035<DHaQS600<HaP@DY@8HaP055<DDYABU5:D@Y0025<D01aS65<DHaQC56<HD
YAC500PDY@065<DHaQC56<HDaAS60QC50aBU1V]J0@000F]J00008V]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDYAS65:DHaQC50aS6020DYAS65:DH
aQBU6<HDYAS65<DHaQC56<HDaAS65<DHaQC57>LHaQcW6<HLiaSV7>LHaQcW6<HLiaBU6<HDYAC511S6
00TDaAS66<HLiaSV7>LHaQcW6<H011cW00@Q21d78@PM1`<Q20057@LQ21d78@PM1`048@P01RDY8@PU
:B489BTQ20<U:@078@PU:B489BTUBBU:9DT012DY00XYBRE9:DXUBBe[:FX]JbUZ;F/YJP@]J`0B=Jda
S3F]<H`icSF]>Lhmkd8@?Nm244HaBU96<E:DCW=>Tdic0e[F00EFeE[FE[EJeUKE00=JeP0=F_INme[f
F_IJeUkgF]IF]E:dE[EBU5JeDY@01UJe00=JeUJeE[D00eJe01EJeUJeE]EJeUkgF]INmeJeF]IF]E[F
E]EJeUKEGaMNmf<HGaMS65lGHaP015kg00]JeUkgF]IF]E:dE[EB]5:DCY=BU4IA00=:DP0PCW=BU4ic
DYA>Te:DCY=>LdYbCW=:LTHa@S16<D8@AS5244Ha?Nm243g_>LhecCW>=LdmkcW>>Nhe[CW>=JdicP<e
[@08<J`aS3F]<H`]JbUZ;F/YJP<YBP069BTYBRDY:DXQ22DY124800<M1b487>L00R4800dU:B489BTL
ib487>LQ21SV7>LHiQd77>LQ20027>L021SV7>LM1b487@LQ21S67>L56<H01QcW6>HLiaC56<HDa@8H
aP037>LHaQcW00@HaP0=5<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D00aS600/DaAS65:DDaABU6<HD
aAS64:@DYA2T008DY@096<HDYAS65<DHaQC56<HDYAS600HDY@0C4:@HaQC56<HDaAS65:DHaQBU6<HD
YAC5JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX011S65<DDaAC50aS600<DaAS65<D0
0QC500<HaQC55<D01QS600<DaAS66<H00QS600<HiQS66>H00acW00@HiQS66>HHaP@DY@LHaP<Li`04
6>HHaQSV6<H37>L00ad78@PQ20058@P00ad78@PQ20048@P01B4X8@PQ:24X9BT00R4X0bDY0R4X00<U
:B4X:DX00RU:12DY12U:00LYJRe[;F/]JbUZ;F/YJP03;F/01c6/=Jda[36/=LdicSG=008n3`05@Q0n
3dIAAU5:DP04CW<4F]H3E]D4F]H00ekgF_INm`04F]H3E[D2D[@00eJeD[AF]@03E[D2E]D00e[FE[EF
]@05E[D00e[fF]IJmP02F]H2E]D2F]H00eKEF]INm`02G_L4HaP4G_L00e[fF]IJmP04E[D3DY@4BU83
CW<014jCDYA>Te:D0Tic00A:LTicAS56<@92<009@Q12<48@?Plmkch?=LdicSG=00DicP<ec@<e[@@a
S08]J`03:FX]JbU:008YBP8UB@05:DXUBB4X8BPU:@048@P01QcW7@LLib4X8@PQ:08Q20<M1`037>LH
iQcW00LLi`078@PM1b487@LHiQS66>H011S600<HiQcW7>L011S600@LiaSV6>HHiPTHaP035<DHaQS6
00<HaP035<DHaQS600<HaP@DY@055<DHaQS65:D@Y0025:D02AS65<DDaAC56<HDaAS65:DDa@085:D0
11C56<HDaAS60aC500@HaQBU5:DDY@I[FP40005[FP00039_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DH
aQC56<HDaAS65:DHaQBU0QC51QS600<LiaS67>L01AS6010LiaS67>LDaAS65<DHaQC56<HDaAcW6<HH
iQS67>LHaP8HiPHLi`0K8@PLib488@PQ:2489BTQ224X8@PU:B489BTQ22DY8BPU:B4X9BTQ:2DY8BPU
:B4X9BTUBBU:008UB@0=9BTYBRDY:DXU:BU:9BT]JbU:;F/YJRe[:FX00be[00dYJSF]<H`e[C6<>Lhe
[CW>>NhmkcW^?Nm2<002AS44CW<01U[FE]EJeUKEG_MJmP9Nm`05HaQNmf<HF]INm`02F]H03EKEF]IF
eE[FE[EJeUJeF]IF]EKEF]INme[F009JmP0LG_MJmU[fF]INme[FE]EF]E[FD[AF]E:dE[EBU5JeDYAJ
eUJeF]IF]F<HG_MO5f<hIcUS>6LiF_H3G_L02U:dE[EF]DicBW9>LdYbCW=:DTic0THa00Q2<4HaCW=>
TdicDYA>LdjC0Tic01A:LTicBW96<D8@AS4n3d8@?Pm243W>?NlicSF]<J`e[C6/>Lhe[CW>13F]00/a
[3F]<J`aS2f;<H`]Rbe[:FX]JbUZ00<YBP068BPU:B4X9BTQ22DY0R48010M1b487@LQ21d78@PLib48
7>LQ21cW7@LLib487>LM1`8Li`0C6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQS65<D0
0QS600HHiQS67>LDaAS65<D26<H07QcW6<HHaQC56<HDaAS65:DHaQC56<HDaAS65:DHaQBU6<HDYAS6
5:DHaQC56<HDaAS65:DHaQ2T5:D@Y08DY@0<5<DDYAS65<DHaQC56<HDaAS65:DHaQBU0QC500@HaQC5
5:D@Y08DY@0>6<HDYAS65:DHaQBU6<I[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J00HD
aABU5<DDYAC55:D25<D011S65<DHaQC50aS600<DaAS66<H011BU21S600<HiQS66<H00aS600<HiQS6
6>H011S600<DaAS66<H021S61acW00<M1b487@L0224800PQ:2488BPQ:2DY8BPU:B4X1RDY00<YBRE9
:DX012DY00LUBBU:9DTUBBU::FXYBP07;F/3<J`023F]=LdecCG=?NlikSg_>Nh3AS44CW<2F]H00eKE
F]INm`02G_L015lGG_MO5ekg0U[f00=NmeKEF]H00U[F0eKE15Je15[F00=Nme[fG_L015[F1UJe0e:d
0UJe00=FeEJeE]D00ekg16Li15kg0eJe14ic00=:LTicBW8014Ha1dic00A:LTicCW=>L`=2<0A240<i
cP@e[@03=LdicSG=00Le[@05;H/aS36<<H`YJP03;F/3:DX59BT28BP28@P00ad78@PQ20028@P27@L0
0b487@LLi`067>L03ASV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV00<Li`036>HLiaS600LHaP065<DH
aQC56>HHaQSV11S600DDYAC55:DHaQC5008HaP@DY@095<DDYAC55<DHaQC56<HDYAC500PDY@075<DH
aQC56<HDaAS65<D011BU00<HaQC56<H011BU00LDaAS65<DDaABU5<DDY@06JeX10001JeX0000bJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HD
YAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQBU5<DDY@<HaP035<DHaQS6008HaP037>LHaQcW
00DHaP0@7>LHaQcW5<DHaQC56<HDaAS65<DHiQS67>LHaQSV6<H37>L00aSV7>LLi`027>L0;2487>LQ
22489BTQ224X8@PU:B488BPQ22DY8@PU:B4X9BTQ:2DY8BPU:B4X9BTQ:2DY9DTYBRE9:DXU:BE99BTY
BRDY:DXU:Be[:DX]JbUZ;F/YJRe[:FX2;F/02cF]<H`e[C6<>Lhe[CW>>NhmkcW^?Nl00THa00=2<4ic
CW<00Tic00AJeUKEF]IFe@ANm`05HaQNmf<HF]INm`02F]H02EKEF]IFeE[FE[EJeUJeE]EF]@02F]H0
2E[fF]INme[fG_MJmUkgF]IJmP02F]H055JeE]EB]5JeD[AF]E:DE[EBU5[FE[EJeUJeGaMNmf<HHcQW
>F<hIcT2G_L03E[fG_MF]EJeD[A>LdYbCW=:LTicBU9>Ld8`00=6<@06CW=BU4icCY=>Le:D0Tic01A:
LTicBW96<D8@AS4n3d8@?Pm243W>>NhicSF]<J`e[C6/>Lhe[CW>13F]01@a[3F]<J`aS2f;<H`]Rbe[
:FX]JbUZ:DXUBBU:8BPU:B4X9BTQ22DY0R4800<M1b487@L00b4800XLib487>LQ21cW8@PLiad77>LQ
208Li`0A6>HLiaSV7>LHiQcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHiP046<H01QcW6<HHiQC56<HDa@8H
aP0N7>LHaQS65<DHaQC56<HDYAS65<DHaQC56<HDYAC55:DHaQBU6<HDYAS65<DHaQC56<HDYAS64:@D
YA2T0QBU010HaQBU6<HDaAS65<DHaQC56<HDYAC55:DHaQC56<HDa@<DY@0?4:@HaQBU6<HDYAS65:DH
aV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00HDaABU5<DDYAC55:D25<D00aS65<DH
aP026<H01AC56<HHaQS65<D011BU21S600<HiQS66<H00aS600<HiQS66>H01QS600<DaAS66<H01QS6
1acW0ad7224800HQ:2488BPQ:2DY8BP39BT00b4X9BTU:@029BT00bU:9DTYBP049BT39DT2:DX00bUZ
:DX]J`06;F/02c6/=Jda[36/=LdicSG=?NlikSg_>Nh00dHa14ic00AFeE[FF]IJeP=Nm`03GaMNmelG
009Nm`9JmP9JeP05E]EJeUKEF]IFe@04E[D4F]H00ekgF_INm`04F]H4E[D015:dE[EF]EJe0U:d00AF
]EKEE[EFe@=Nm`AW>@ANm`=F]@A>L`=:LPA6<@U>L`05BW9>Ld8`AS52<004@Q03>Lh4=Jd3=Ld7=Jd2
<H`01Rf;<H`]Jbe[:FX]J`<YBP@U:@8Q:0039BTM1b4800HQ20<M1`LLi`0;6>HLiaSV7>LHiQcW6>HL
iaSV7>LHiP037>L011SV7>LLiacW21S600HDaAS65<DHiQS66>H46<H00aBU5<DDY@036<H00aC55:DD
Y@025:D02AC55:DDaAC56<HDaAS65:DDa@085:D01aC56<HDaAS65<DHaQC500@DY@036<HDaAS600@D
Y@<Da@046<HDYAC55:D6JeX10001JeX0000MKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`026<H031C56<HDYAS65:DHaQBU6<HDYAS65:DHaP@DY@09
5<DHaQS66<HDaAS65<DHaQC5008HaP0A6>HHaQcW6>HLiaSV7>LHaQcW6<HLiaS67>LHaR487>LM1`02
7>L01ASV7>LHiQcW6>H00QcW00PQ21cW8@PLib487>LQ21d71b4800lM1b488@PU:B489BTQ:2DY8BPU
:B4X9BTQ:2DY8BP00RDY00dYBRDY:DXUBBU:9DTYBRE9:DXUBBU:9DTYBP04;F/00bUZ;F/YJP04<H`0
23W>=JdecCW>?NlicSg_@S02AS4024icBW9>LdYbF]IF]E[FE[D4HaP00f<hHaQW>@02G_L04U[fF]IF
eE[FE]EJeUKEF]IF]E[FE[EJeUKEF]IFeE[FE]EJeP9Fe@0:E[EJeUJeF]IFeE[FDYAF]E:DE[D3DY@0
65kgF]INme[FIcUS>6LiHaQW>F<HIcUF]E[FE[EFeDjCDYA>TdYBAS5:DTHaAU56<@9:DP0>BW9:DTic
CY=BU4jCDYA>Te:DCY=:DTHaBU96<@=2400=?Nm243g_?PlecCW>=Lde[C6/=Jda[3F]<H`00SF]00di
cSF]>Lh]Rc6<;H/aS2f;<H`]Rbe[:DX]J`04:DX01b4X9BTU:BDY8BPU:B4X00DQ20067@LQ21cW8@PL
ib481AcW00DM1acW8@PLib48008Li`086<HHiQS67>LHaQcW6<HLi`8HaP035<DHaQS600@HaP076>HH
aQcW6<HHiQS67>L00aS600<LiaS66>H00QS601<DYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DH
aQC55:D@Y0025:D01QS65:DHaQBU6<HDY@<HaP0M5<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQBU
6<HDYAS65:DHaQBU6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP@HaP8Da@056<HD
aAC55<DHaP035<D011S65:DDYABU3QS600<LiaSV7>L00aSV00@LiaSV6>HHiPDLi`036>HLiaSV00LL
i`047@LQ21d77@L;8@P0124X8@PQ:24X2RDY00<UBBDY9DT00RU:00<UBBU::DX01RU:1be[136<0cF]
00@ikSW>>NhicP=6<@A>L`04E]EJeUKEE]D7HaP3G_L8F]H3E]D5F]H00eKEF]IF]@03E[D00eKEF]IF
e@02D[@00eJeD[ABU002DY@025[FF_IJeU[fHcQW>F<hIcT3HcP4E[D01e:DCY=BU4HaAU56<DIA00=6
<@A:DP04DYA>Te:DCY<3DY@3AU4014YB@Q12448@13g_00<icSG=>Lh013F]00La[3F]<J`ecCF]=Lde
[@03<H`022f;<H`]Rc6<:DXYJRU::FX3:DX79BT68@P27@L00acW7@LLi`087>L00ad77>LM1`036<H0
1aSV6<HHiQS66>HHaQSV01HHaP<Da@056<HDaAC55<DHaP025<D00aBU5<DDY@025<D00aS65<DHaP03
5:D25<D01AS65<DDaABU5<D011S600DDaAS66<HHaQC500<HaP8Da@035:DDaABU008Da@066<HDaAC5
5<DHaQC51V]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]J001[FV]jJeY_Nf]JJgXDa@<HaP0:5:DHaQBU6<HDYAS65:DHaQBU6<H45:D26<H01AC56<HDaAS6
5<D00aS600<DaAcW6<H00QSV00dLiaSV7>LHaQcW6<HLiaS67>LHaQd77>LQ20027>L01ASV7>LHiQcW
6>H00QcW00PM1acW8@PLib487>LQ21d71B4800<M1b488@P00R4800`U:B489BTQ:2DY8BPU:B4X9BTQ
:2DY8BP29BT03BU:9BTYBRE9:DXUBBU:9DTYBRE9:DXUBBU:00@]J`03:FX]JbUZ00@aS007=Lde[CW>
>LhmkcW>?Nl00THa00U2<4icBW9>LdYbF]IF]E[FE[D016<H00AW>F<HHcQJmP9Nm`0MF]IFeE[FE]EJ
eUKEF]IF]E[FE[EJeUKEF]IFeE[FE]EJeUKEF]IF]EKEE[EJeUKEF]IBU5JeDYAF]@03DY@0:5kgF]IN
me[FIcUS>6LiHaQW>F<HIcUF]EKEE[EJeTjCDYA>TdYBAS5:DTHaBU96<DIABU9>LdYBBW9>Te:DCY=B
U4jCDYA>TdYBAS5:DTHa0d8@00dmkch??Nm243G=>LhecCF]<J`e[C6/=JdaS002=Jd03CW>=JdicRf;
<H`]Rc6<;H/aS2f;;F/YBRe[008YBP099DTYBRDY9BTQ:2DY8BPU:B4X00DQ20067@LQ21cW8@PLib48
1AcW00lQ21cW8@PLib487>LHiQS67>LHaQcW6<HLiaS67>L011S600<DaAS66<H00QS600LLiaS66>HH
aQcW6<HHiP036<H00aSV6<HLi`026<H04aBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DD
YA2T008DY@086<HDYAS65:DHaQBU6<HDa@<HaP0K5<DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU
6<HDYAS65:DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00XHaQC55<DDaAS65<DH
aQC56<HDa@DHaP035<DHaQC500LHaP<Da@076<HHiQSV6>HLiaSV7>L00QSV00DHaQSV6<HHiQS6008H
iP037>LHiQcW00HLi`036>HM1acW00@M1`PQ20038BPQ224X00@Q208U:@038BPU:BDY00HU:@<YBP07
9DTYBRE9:DXUBBU:9DT022U:00<]JbUZ;F/00S6<00D]Rc6<<J`e[C6/00<mk`04>Ni6<DHaAS44BU84
E[D00elGHaQO5`05IcT2HcP2HaP015lGHaQJmUkg0e[f00INme[fF_IJmUkgF_H7F]H3E]D02E:dDYAB
]5:DE]EF]EKEGaMS6002GaL00f<hIcUS>003G_L01E[fDYABU5:DCY<00dYB0dYb00Q>LdYBBU9:DU:D
CY=BU4jC0e:D14YB14Ha0T8@00<n3cg_?Nl00Sg_0cW>00LecCW>>LhicS6<<J`aS005=Jd0136/=Jd]
Rc6<0Rf;12e[00<UBBU:9DT01BDY00<Q:2DY8BP00R4X00PU:AcW7@LLib487@LQ21d711cW00<Q21d7
8@P011cW00<HiQcW6>H01AcW00<HiQcW6<H03aS600DDaAS65<DHaQC500<HaP<Da@056<HDaAC55:DD
a@086<H00aC56<HDa@045:D00aC55:DDa@045:D00aC56<HDa@046<H01aC55:DDaABU5<DDYAC500LD
Y@I[FP40005[FP0002i_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00
JeY_Nf]JJgY[FVmk5:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDY@8HaP0L7>LHaQSV5<DH
aQC56<HDYAS65:DLiaS67>LHiQcW6>HLiaS67>LHaQcW6<HLiaS67>LHiQcW6>H37>L00aSV7>LHiP02
7>L022487>LQ21cW8@PLib487@L28@P01b4X8@PU:B489BTQ22DY008Q20057@LU:B4X9BTQ:0039BT0
3b4X9BTU:BDY9DTYBRE9:DXUBBU:9DTYBRE9:DXUB@04:DX06RUZ:DX]JbUZ;F/YJS6<;H/aS2f;=Jda
[3F]>NhmkcW^?Nm6<DYBAS5:LTYBCW=:DUJeD[@2E[D00f<HGaMS6002IcT016<hIcUS66Li0V<H011O
5f<HGaMNme[FG_MJeUkgF]INme[FG_MJeUkgF]INm`IJeP05E[EJeU:DE[EBU002E[D04U[FE[ES65kg
HaQNmfLiHcQW>E[fG_MJmUkgCY=BU4jCDYA6D@9:DP03CW=:DTic00A:DP03CY=BU4jC00ABU0A:DP0D
AS52<4Ha@S1243h?@Q0ikSg_>NhmkcG=>LhecCW>=LdicSG==JdaS08e[@09=Lde[CW><J`e[C6/<H`]
Jc6<008]J`0@:FX]JbUZ:DXUBBU:8BPU:B4X9BTQ:2DY8BPU:B489BT28@P01QcW8@PM1b487@LQ208L
i`0=6>HLiad78@PM1acW6>HLiaSV7>LHaQcW6>H00acW00HHiQcW6>HHiQS67>L26<H01QC56<HHaQSV
6<HLi`@HaP0G5<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDa@036<H0
1AC56<HDYABU4:@00QBU00<HaQBU5:D00aBU00dHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaP085:D0
1f]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J0QC500<HaQC56<H00QS600<DaAS65<D0
1AS60aC51aS600XDaAS65<DDaASV7>LHiQcW6>HLi`8HiP056<HHiQS66>HHaP026>H00acW6>HLi`04
7>L01ASV7>LLiad77>L00Qd700<Q21d77@L01b4800<Q:2488BP0124800<Q:2DY9BT022DY0bU:00LU
BBU:9DTYBRE9:DXUB@08:DX3;F/02Bf;<H`aS36<<J`e[C6/?NlikP02?Nl3AS44BU84E[D00elGHaQO
5`04IcT2HcP00fLiGaMS6002HaP3F_H01EkgF_IJmU[fG_L00e[f1e[F00YFeE[FE]EB]5:DD[ABU5KE
E[EFe@=O5`06HaQS>6LiHcQNme[f0Ukg00ABU4jCDYABU0=:DP04BW9>LdYbBW83BU8015:DCY=BU4jC
0e:D14YB14Ha00<n3d8@@Q0013g_1CW>00DecCW><H`a[36<00Le[@<]R`03<H`]Jbe[008]J`039DTY
BRE900DU:@048BPU:B4X9BT28BP01acW7@LLib487@LQ21d700@Li`038@PM1b4800@Li`<HiPDLi`03
6>HLiaS6014HaP055<DHaQC56<HDa@036<H45<D00aBU5<DHaP076<H00aC56<HDa@045:D00aC55:DD
a@045:D00aC56<HDa@046<H01aC55:DDaABU5<DDYAC500LDY@I[FP40005[FP0002i[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU6<HDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDY@8HaP0R6>HHaQcW5<DHaQC56<HDYAS65:DLiaS67>LHiQcW6>HL
iaS67>LHaQcW6<HLiaS67>LHiQcW6>HLiaSV7>LHiQcW6>H27>L022487>LQ21cW8@PLib487@L28@P0
22DY8@PQ:2489BTQ22DY7@L28@P01RDY8BPU:B4X9BTQ:0<U:@0=8BPU:BE9:DXUBBU:9DTYBRE9:DXU
BBU:9DT012U:01P]JbU::FXYJRe[:FXaS2f;<H`]RcF]<J`e[CW^?NlikSg_AS56DDHaCW=:DTYbBU83
E[D01E:dHaQO5f<HHcP00fLi01AS66LiHaQS65lGHaQO5ekgF]INme[FG_MJeUkgF]INme[FG_MJeU[f
1U[F00EF]E[FDYAF]E:D009F]@0AF]IF]F<HG_MS65kgIcUS>6LiF_INme[fG_M>Te:DCY=BU002BU80
14IACW=:DTic14YB00=>Te:DCY<015:D14YB01A6<D8`AS52<48@?Pm243W^?NlikSg_=LdicSG=>Lhe
cCW>=Lde[C6<0SF]00TicSF]=Lda[3F]<J`aS2e[<H`00Re[010YJRe[:FXYBRE9:DXQ:2DY8BPU:B4X
9BTQ:2DY8@PU:@8Q20077>LQ21d78@PM1b486>H00acW00TM1b487@LLiaSV7>LHiQcW6<H00acW00PH
iQcW6>HLiaSV7>LHaQSV0QS600HDaAS66<HLiaS66>H46<H05AC56<HDaAS65<DHaQC56<HDaAS65:DH
aQBU6<HDYAS65:DHaQC56<HDa@036<H01AC56<HDaAS64:@011BU00<HaQBU5:D00aBU00dHaQC56<HD
aAS65<DHaQBU6<HDYAS65:DHaP085:D01f]JKg][FV]jJeY_N`00005[NP005V]J0`0016]J00HHaQC5
5<DDaAS65<D56<H01aC56<HDaAS65<DHaQC500LHaP@DY@/HaP037>LHiQSV008Li`036>HLiacW008L
i`8M1`048@PM1ad77@L58@P00ad78@PQ20048@P01BDY8BPU:B4X9BT00R4X4RDY1bU:00T]JbUZ:FXY
JRe[:FX]Jbf;<H`00Rf;00<a[3F]<J`00cg_00DikTHaAS56<DYb00=>L`=Fe@06F]IO5elGGaMW>F<h
0VLi00E[FVMIIeUS>6Li009S>003F_INme[f00AJePMNm`AJeP=B]007E[EB]5JeD[AF]E:dE[D00Ukg
00EJmUkgGaMS65lG00AF]@A>L`03AU56<DIA00A:DP=>L`AW>@=cW0ABU0924003?Pm243g_00Hmk`<i
cP<ec@03>LhaS36<00`aS004:FX]JbUZ;F/3:DX49BT00b4X8@PQ:0048@P011d78@PQ22480ad71AcW
00@HiQcW6>HLi`8HiP036<HHiQS600/HaP<HiP@HaP@DY@<HaP0;5<DHaQC56<HDaAS65<DHaQC56<HD
a@036<H00aC56<HDa@056<H35:D01aS65<DDaAC56<HDaAS6008DY@054:@DYAS66<HDa@046<H011C5
5:DDaAC50aS600LDaAS65<DHaQBU5<DDY@05JeX20001JeX0000eKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65<DHaQC56<HDaAS65<DH
aQC56<HDaAS65:DHaQC56<HDaAS65<DHaP025<D01QBU6<HDYASV6<HLi`8HaP035<DHaQS600@HaP06
7>LHaQcW6>HLiaSV11cW00HQ21cW8@PLib487>L58@P00ad78@PQ20038@P02b4X9BTQ:2DY8@PU:B48
9BTQ:2DY8BP00bDY00DQ:2DY8BPU:B4X008U:@0N9DTU:BU:9BTYBRE9:DXUBBUZ:DX]JbU:;F/YBRe[
:FX]JbUZ<H`]Jc6<;F/e[C6/=JdikSg_>Nhmkd8`0THa00i>LdYbCW=:LU[FE[EJeUJeHaQNmf<HHcQW
>F<h0VLi00U[FVLiIcUS66LiHaQNme[fG_L015[F1ekg01EJeUKEF]IFeEJeDYAF]E:dE[EB]5JeD[AF
]E:dG_MJmUkgF_IS65kgHaP015Je00U>Le:DCW=>TdHaBU96<DYBAU400TYB00M>LdYbCW=S>6LiHcQW
>@03Li`3DY@01DjC@Q0n3d8@?Pl01cg_13W>00Xe[CW>=Jda[36<=JdaS36/<H`e[@8aS00<;H/aS2f;
<H`]Rbe[:FX]JbUZ:DXUBBU:12DY00DQ22DY8@PQ21d700<Q20067@LQ21cW8@PLib480QcW014HiQcW
6>HLiaSV7>LHaQcW6<HLiaS67>LDaAS65<DHaQC500<HaP0:5<DHaQS67>LHaQcW5<DHaQC56<H45:D2
6<H03AC56<HDaAS65<DHaQBU6<HDaAS65<DHaQC500<HaP035<DHaQC500@HaP@DY@0<6<HDYAS65<DH
aQC55:D@Y1BU4:@HaQC51AS601DDYAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS6JeY[NV]JKg][FV]j
00000Fmk000FJeX00`00JeY[FP04JeX25<D00aS65<DHaP066<H011C56<HDaAS60aC51AS600<DaAS6
5:D00aBU2aS60QSV00<LiaSV7>L01AcW00DQ21d77@LM1b48008M1`HQ20037@LQ2248008Q20039BTQ
:2DY00<Q:1<U:@LYBP8YJP03;F/YJRe[008]J`<]R`06<H`a[3F]<J`mkcW^0Sg_0dHa0Tic00A:LTic
E]EJeP9Fe@03GaMS65lG00=W>@04HcQWFFMIJeX3HcP016LiF_INme[f15[F1ekg15[F00]B]5JeD[AF
]E:dE[EB]5JeD[AF]E[f00=Nm`=O5`AF]@A>L`03AU56<DIA00A:DP=>L`AW>@=cW0ABU004?Pm2448@
@Q07?Nl3>Lh013G=>LhecCG=3S6<00@YJRe[:FX]J`<YBP@U:@038BPQ224X00HQ20<M1`038@PM1acW
00@Li`036>HLiaSV008HiP037>LHaQSV00`HaP036>HLiaSV00@HaP@DY@<HaP045<DHaQC56<H35<D0
11S65<DHaQC50aS600<DaAS65<D01AS60aBU0QC500HHaQC56<HDaAS64:@35:D00aC56<HHaP046<H0
0aC55:DDa@026<H02AC56<HDaAS65<DHaQBU5<DDY@06JeX10001JeX0000nJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDYAS65<DHaQC56<HD
aAS65<DHaQC56<HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDYAC55:DLiaS66>HDa@PHaP087>LHaQcW
6>HLiaSV7>LHiP8Li`068@PLib487>LQ21cW1B4800<M1b488@P00b48014Q:2DY8BPU:B489BTQ22DY
8BPU:B4X9BTQ:2DY8BPU:B4X00<U:@0N8BPYBRDY9DTU:BU:9DTYBRE9;F/YBRUZ:DX]JbU:;F/YJRe[
:FXaS2e[<H`]JcF]<J`e[CW^?NlikSg_0THa00m2<4icBW9>LdYbF]IF]E[FE[ES65kgHaQS>6LiHcP0
0VLi00U[FVLiIcUS66LiHaQNme[fG_L015[F1ekg01EJeUKEF]IFeEJeDYAF]E:dE[EB]5JeD[AF]E:d
G_MJmUkgF_IS65kgHaP015Je00M>LdjCCW=BU4HaBU96<@03BU8024IACW=:LTicHcQW>F<hIcT3Li`0
2U:DCY=BU5:D@Q0n3d8@?PlmkcW^1Cg_13W>00/e[CW>=Jde[C6<<J`aS3F]<H`a[2f;00<aS00:;H/a
S2f;;F/YJRe[:FXYBRE9:DX49BT00b489BTQ20038@P02Ad78@PM1b487>LQ21cW8@PHiP037>L03ASV
7>LHiQcW6<HLiaS67>LHaQcW5<DHaQC500<HaP0<5<DHaQS66<HDaAcW6<HLiaC56<HDaAS611BU00dD
aAS66<HHaQC56<HDaAS65:DHaQC56<HDa@036<H01AC56<HDaAS65<D011S611BU00`HaQBU6<HDaAS6
5<DDYA2T5:D@Y1S65<D56<H05ABU6<HDYAS65<DHaQC56<HDaAS65<DHaQBU6<I[FVmkJeY[NV]JKg/0
0001JgX001I[FP03001[FV]J00A[FP045<DDYAC55:D;6<H00aC56<HDa@046<H25<D01AS66>HHiQSV
7>L02aS62AcW00<HiQcW7>L00QcW0Qd71B48124X00<Q224X8@P0124800XU:B4X8BPQ:2DY8BPU:B48
8BPQ20@U:@0:9DTU:BE99DTYBRE9:DXUBBU:9DT4:DX012e[:FXYJRUZ1be[0cF]13g_0T8`00=6<Dic
CW<00Tic15Je00=O5f<HGaL016Li00=WFFLiIeT016Li1V<H00=O5ekgG_L01Ekg15[F25Je0U:d15[F
0eJe14ic1DHa00=2<4HaAS400dHa00E:DTYbBU9cW6nK009cW0=oo`0;D[AF]E:dD[@n3d8@?Pm243h?
@Q0n3`03?Nl023W^?NlikSg_<J`e[C6/=Jd4<H`012f;<H`]Rc6<0Rf;12e[00LYBRUZ:DXYJRU::FXY
BP049BT78@P00acW7@LLi`0;7>L>6<H36>H00acW5<DHaP026<H01QC56<HHaQC55:DDa@DDY@035<DD
YAC500<HaP@DY@036<HDaAS600`HaP095<DHaQC56<HDaAS65:DDaABU008Da@036<HDaAS600<HaP8D
a@036<HDYABU00<DY@035<DDYAC500<DY@I[FP40005[FP0002Q_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU6<HDaAS65<DHaQC56<HDa@DH
aP035<DHaQS600<HaP065:DHaQBU7>LHaQcW0QS600HDaAS66<HHiQS67>L46<H27>L01aSV7>LHiQcW
6>HLiaSV008Li`068@PLiad77>LQ21cW124801@U:B489BTQ22DY8@PU:B488BPQ22DY8@PU:B489BTQ
:2DY8BPU:B481RDY00`YBRDY:DXUBBU:9DTYBRE9:DXUBBU:9DT2:DX03Be[:DX]JbUZ;F/YJS6<;F/]
Rbe[=Jda[3F]00@mk`07@Q16<D8@CY=>Le:DCW<015Je00=S65lGHaP01FLi019[FVLiIcUS>6LiHcQW
>F<HHcQO5f<HGaMS65kgHaQNmelGF_H2G_L05U[FE]EJeU[FE[EB]5JeD[AF]E:dE[EBU5JeDYAJeUKE
F]IFeEJeD[AF]DYb0dic14Ha00E2<4Ha@S16<D8`0096<@07CW=:DTicKi]cW6nKLi`00goo01IF]E:D
E[EBU48@?Pm243h?@Q0mkd8@>NhmkcW^?NlikSg_>Nhe[C6/=Jda[0<aS006;H/aS2f;<H`]Jc6<1Re[
010YBRe[:DX]JbU:;F/Q:2DY8BPU:B489BTQ224X8@PU:@8Q20037>LQ21cW00DLi`056>HLiacW7>LH
iP036<H021C57>LHaQSV6<HLiaS66>H36<H00acW6<HLi`026<H05aC56<HDaAS65<DHaQBU6<HDYAS6
5:DDaABU6<HDYAS65:DHaQC56<HDYAS65:D00QC500<HaQC56<H011S601DDaAS66<HHiQS67>LDaAS6
5<DHaQC56<HDaAS65:DHaQBU6<HDYAS65<D00aS601DDYAS65:DDYA2T5:D@Y1S65:DHaQBU5<DDYAS6
JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeX011C55:DDaABU2aS600<DaAS65<D01AS6
0QC500@HiQcW6>HHiP/HaPLLi`036>HLiacW00@Li`038@PM1ad700@Q20068BPU:B4X8BPQ224X1B48
0R4X00PU:B4X9BTQ:2DY8@PQ:24812DY00XUBBDY9DTUBBU:9DTYBRE9:DXUB@@YBP8YJP03;F/YJRe[
00H]J`<e[@@mk`03AS52<48`00A>L`AF]@03GaMS65lG00AW>@03IeUW>FMI00AW>@AS6003GaMS66<H
00MNm`AJePMF]@9B]003E[EJeU[F009JeP=F]@A>L`E6<@03@S16<DHa00=6<@03BU9:LTYB00=cW004
Ki]oogooOol3D[@01EJe?Pm243h?@Q000ch?00<mkcW^?Nl00Sg_00HikSg_<J`e[C6/=Jd4<H`00bf;
<H`]R`02;H/00c6<;F/]J`02;F/01bU::FXYBRUZ:DXYJRU:00@U:@LQ20037>LM1acW00/Li`hHaP04
6>HLiaSV6>H26<H011C56<HHaQS60QC500<DYAC55:D011BU00<DaABU5<D00aS611BU00<HaQC56<H0
31S600<DaAS65<D00aS600<DYAC55:D011C51AS60QC51ABU00<DaABU5<D00aBU1V]J0@000F]J0000
8F]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXD
YAS65:DDa@046<H00aBU6<HDY@056<H03ABU6<HHaQS65<DHaQC56<HDaAS65<DHaQC500<HaP035<DH
aQC5008HaP067>LHaQSV6>HLiaSV11cW00HQ21d78@PLib487>L28@P02BDY8@PU:B489BTQ22DY8@PU
:@058@P04BDY8@PU:B489BTQ22DY8BPU:B4X:DXU:BU:8BPU:B4X9BT00RU:00/UBBU:9BTUBBDY:DXU
:BE99BTYBRDY008YBP0D;F/YBRUZ:FX]JbUZ<H`]Rc6<;H/e[C6/=JdikSg_>Nhmkd8@AS5240=:DP09
AU5BU4jCDYA>TekgF]IJmUlG00=S600>IcU[FVLiJeYWFF]JIeU_Nf]jKg]cW7>lLiag_@9[FP03IeU[
FV<H009S60=JeP03E]EF]EJe00IF]@04F]IF]EJeD[@2E[D06TjCCW=BU4HaBU96<DYB@Q16<D8@AS52
<4Ha@S16<D8`AS52<4Ha@S16<DjCDYA>Te:DD[@2E[D4AS400cg_>Nhmk`02?Nl023W>?NlicSg_>Lhm
kcW>?Nl2>Lh01cF]=Lde[C6<;H/aS2f;00<aS003;F/aS2e[00@]J`0=:DXUBBU:9DTYBRDY:DXQ:2DY
8BPU:B488BP00R4800<Lib487>L00b48018Lib487>LM1aSV7>LHiQcW6<HLiaS67>LHiQcW6>HLiaS6
7>L26<H011C56<HHaQcW0QS600lDaAS65<DHaQBU6<HDaAS65<DHaQC56<HDaAS65<D00aS600dDYAS6
5:DHaQC56<HDYAS65:DHaQC56<HDa@036<H02AC56<HDaAS65<DHaQBU5<D@Y0035:D01QC56<HDaAC5
5:DHaP8DY@084:@DYABU5<DDYAS65:DHaP8DY@034:@DYA2T00<DY@0;4:@HaQBU5<E[FVmkJeY[NV]J
Kg/00001JgX001I[FP03001[FV]J00A[FP@DY@<HaP045:DDaABU5<D46<H00aC56<HDa@056<H01AC5
6<HDaAS65<D031S600<LiaSV7>L011cW0R480ad700DQ21d78BPQ:2DY00<Q:0049BTQ:24X8BP48@P0
1R4X8@PQ:2488BPQ208Q:0059BTQ:2DY9BTUB@049BT0124X:DXYBRU:22DY0bE912U:00L]JbUZ;F/]
Rc6<;H/aS003=Jd01cg_>NhmkcW^@S12448`00A:DP04CY=BU4jCDY@3F]H4HaP00f]JIeUWF@02JeX0
1FMIJeY[NVmkJgX017>L16]J0f<H15[F1eJe00=FeEJeE]D015Je0dic0116DDHaAU56<D8`@Q12<48@
AS52<4Ha@S16<D8`AS52<096<@ABU0=F]@A6<@@mk`0:>LhikSW>>NhicSW^>LhikSW>>Nh4=Jd7<H`0
0bf;;F/]R`04;F/2:DX01BE9:DXUBBU:9DT00bDY00@Q:2488@PQ20<M1`@Q20DLi`036>HLiacW00<H
aP@Li`036<HHiQS600@HaP076>HHaQSV5<DHaQC56<H00aC51AS600<DaAS65<D00aS600<DYAC55:D0
0QC500PHaQC55<DDYAC55:DHaQC52AS61aBU00<HaQC56<H02aBU0QC500<HaQBU5:D02ABU1V]J0@00
0F]J000086mkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[
NV]JKg/DYAC55:D56<H00aBU6<HDY@056<H00aBU6<HDa@036<H01aC56<HDaAS65<DHaQC500<HaP03
5<DHaQC5008HaP066>HHaQcW6>HLiaSV11cW00HQ21d78@PLib487>L28@P02BDY8@PU:B489BTQ22DY
8@PU:@058@P04RDY8@PU:B489BTQ22DY8BPU:B4X:DXU:BU:8BPU:B4X9BTUB@8YBP0:9DTU:BU:9BTU
BBDY:DXU:BU:9BT2:DX02RUZ:DX]JbUZ;F/YJS6<;H/aS2f;0cF]00TikSg_>Nhmkd8@AS5244YBAU40
0TYB00MBU4jCDYA>Te[fF]INm`02HaP04ElGHaQW>F]JIcU[FVMIJeYWFFmkJgY_Ng>LMkecW7>lIeT0
0f]J0f<H00IJeUKEF]IJeUJeD[@6E[D00e[FE[EF]@02E[D06U:dDYA>LdjCAS5:DTHaBU9244Ha@Q16
<D8`AS52<4Ha@S16<D8`AS52<4HaCY=BU4jCDY@2E[D00e:dAS56<@02AS43?Nl04CW^?NlicSg_>Lhm
kcW>?NlicSg_>LhecCF]>Lhe[C6<;H/00c6<00@]Rc6<;F/aS0D]J`0=:DXUBBU:9DTYBRDY:DXQ:2DY
8BPU:B489BT00R4801TLib487>LQ21d78@PLiad77>LQ21SV7>LHiQcW6<HHiQS67>LHiQcW6>HLiaS6
7>LDa@046<H041cW6<HHaQC56<HDaAS65:DHaQC56<HDaAS65<DHaQC50aS6014DaAS65:DHaQBU6<HD
aAS65:DHaQBU6<HDaAS65<DHaQC500<HaP065<DHaQC55<DDYAS60QBU00P@Y1BU5<DHaQC56<HDYAC5
1ABU00DHaQBU5<DDYAS6008DY@054:@DYA2T5:D@Y0025:D02QC55:DHaV]JJgY[FVmkJeY[NP000Fmk
000FJeX00`00JeY[FP04JeX45:D36<H011BU5<DDYAC511S60aC51AS600DDaAS65<DHaQC500`HaPLL
i`047@LQ22488@P37@L01BDY8BPQ:24X9BT00b4X00<U:B4X8@P00b4800HQ:2488BPQ224X8@P28BP0
1BDY8BPU:BDY9DT00RDY00<Q:2DY9BT00bU:22DY00<UBBU:9DT012U:0be[00@]Rc6<;H/aS0<e[@07
?NlikSg_>Ni2<48@@S0014YB00A>Te:DCY=BU0=JePAS609WF@03JeYWFF]J009[FP03JgY_Nf]j00Ac
W0A[FP=S60AJePMF]@03E]EF]EKE00AF]@=>L`0>AU56<DIAAS52<48@@S1244Ha@S16<D8`AS52<0=6
<@03@S1BU5:D009BU0=F]@A6<@@mk`0:>LhikSW>>NhicSW^>LhikSW>>Nh4=Jd7<H`00bf;;F/]R`04
;F/012E9:DXYBRU:0bE900@U:B4X9BTU:@<Q20047@LQ21d77@L38@P77>L011SV6<HHaQS611cW00<H
aQSV6<H011S600XHiQS66>HDaAS65<DHaQC56<HDa@DHaP035<DHaQC500<HaP035:DDaABU008Da@08
6<HDaAC55:DDaABU6<HDa@THaPLDY@<HaP/DY@036<HDaAC500/DY@I[FP40005[FP0002i[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU6<HD
aAS65<DHaQC56<HDaAS65:DHaQBU6<HDa@8HaP036>HHaQcW00<HaP067>LHaQcW6<HLiaS60QSV00<L
iaSV7>L011S6014HiQcW6>HQ21cW8@PLib487>LQ21d78@PM1b487@LQ21d700DQ20077@LQ22489BTQ
22DY7@L00b48010Q:2DY8BPU:B4X9BTQ:2E99BTYBRDY:DXU:BU:9BTYBPDU:@07:DXUBBU:9DTYBRE9
:DX012e[02XYJRe[:FXe[C6<=JdaS3F]<H`e[CW>?NlicSg_?Pm243h?BU96DDYBAU5>LdYbCW=:LUJe
D[AF]E[FG_MJeUkgGaMS65lGJeYWFF]JIeUkgWNmNmh4Ool03G_nOomkoWooHcQW>F<hF]IFeE[FE]EF
]E:d009F]@05F]IF]EKED[AF]@02D[@03E:DE[EBU4icBU9>LdHaAU56<DYB?Nm243g_00924005AS52
44Ha@Q12<002@Q005ch?@Q12<4Ha@S16<D8`AS52<48@?Pm243h??NlikSg_>NhmkcW>?NlecCW>=Ld0
1CW>00`e[CG==Jde[C6<=Jd]Jc6<;F/aS2e[<H`2;F/01RUZ;F/YJRU:9DTYBP8UB@089BTYBRDY9BTQ
:2DY8@PQ:08Q200D7@LQ21d78@PLib486>HLiaSV7>LHiQcW6>HLiaS67>LHaQcW6<HLi`8HaP055<DH
aQC56<HDa@036<H011C57>LHaQcW0QS600DDaAS65:DHaQBU008Da@0H6<HDaAS65<DHaQC56<HDaAS6
5<DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC50aS6020DaAS65:DHaQC56<HDaAS65<DHaQC56<HD
YAS65:DDaABU6<HDYAC55:DHaQC56<HDaAS65:DHaQBU6<HDYAS611BU00M[FVmkJeY[NV]JKg/00001
JgX001I[FP03001[FV]J00A[FP046<HDaAC55<D56<H01QC56<HDYAC55:DDa@XHaP<HiP067>LHaQS6
6<HLiaSV0QcW11S600hLiaSV7>LLiad77>LM1acW7@LLib487@LQ21d71b4800HM1b488@PQ:2488BP5
8@P01RDY8BPU:B4X9BTQ:0@U:@049DTU:BE99BT29DT00bU:9BTU:@029BT00bE9:DXUB@04:DX7;F/0
0c6/=Jda[002<J`03SF]<J`ikSW^?NlikT8@?Pm244IABU96DDYBBW83CW<02U:dE[EF]E[fF]IJmU[F
HaQO5f<H0V]J00EWFF]JMkeggGNm00Eoo`03Noioog_n00=W>@05E]EJeU[FF]IB]009E[D4DY@00dYb
CW=:LP04AS42?Pl01D8@?Pm2<48@@S00148@00Dn3d8@?Pm6<D8`00=6<@06@S16<Ch?@Q0n3d8@0Sg_
00@ikSg_>LhikPPicP@e[@04<H`a[36<<H`4;H/01Re[;H/YJRe[:FX]J`8YBP039DTYBRDY00HU:@<Q
20067@LQ21d78@PLiad70QcW00LHiQcW6>HLiaSV7>LHiP036<H011SV7>LHiQSV2aS600@HiQS66>HD
a@8HaP@DY@DHaP035<DHaQS600<HaP095:DDaABU6<HDaAS65<DHaQC500THaP055:DDaABU6<HDa@05
6<H011C56<HDaAC51aBU00/HaQC56<HDYAC55:DDaABU5<DDYAC500<DY@I[FP40005[FP0002i_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU
6<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDa@8HaP037>LHaQSV00<HaP0;7>LHaQcW6<HHiQS67>LH
iQcW6>HLi`046<H04ASV7>LHiR487>LQ21cW8@PLib487@LQ21d78@PM1b487@L01B4800HM1b488@PU
:B489BT28@P04Qd78@PQ:2DY8BPU:B4X9BTQ:2U:9BTUBBDY:DXU:BU:9BTYBPDU:@07:DXUBBU:9DTY
BRE9:DX012e[02XYJRe[:FXe[C6<=JdaS3F]<H`e[CW>?NlicSg_?Pm243h?BU96DDYBAU5>LdYbCW=:
LUJeD[AF]E[FG_MJeUkgGaMS65lGJeYWFF]JIeUkgWNmNmh4Ool03G_nOomkoWooHcQW>F<hF]IFeE[F
E]EF]E:d009F]@0DE]EF]E[FD[AF]E:dE[EBU5:dDYA>LdYBCW=6<DYBAS56DCg_@Q0mk`924005AS52
448`@Q16<@02@Q005ch?@Q12<4Ha@S16<D8`AS52<48@?Pm243h??NlikSg_>NhmkcW>?NlecCW>=Ld0
13W>00<ecCF]>Lh00SF]00PaS3F];F/aS2e[<H`]Jc6<0Re[00/YJRe[:FXYBRE9:DXUBBU:9BTUBB4X
00<U:@0H8@PU:B488@PM1b487@LQ21cW8@PHiQcW6>HLiaSV7>LHiQcW6<HHiQS67>LHaQcW11S600DD
aAS65<DHaQC5008HaP037>LHaQcW008HaP0Q5<DHaQBU5<DDYAS65<DHaQC56<HDaAS65<DHaQC56<HD
aAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500<HaP0C5:DHaQC56<HDaAS65<DHaQC56<HD
YAS65:DHaQBU5<DDYAS65:D00QC500THaQC56<HDYAS65:DHaQBU6<H011BU00M[FV]jJeY_Nf]JJgX0
0001Kg/001I[FP03001[FV]J00A[FP046<HDaAS65<DB6<H01ASV6<HHiQC56<H00QC51aS60QSV00<L
iaSV7>L02AcW00<Q:2488BP0124800<Q:2DY8BP012DY00@Q:2488BPQ20<U:@048BPU:BDY9BT38BP;
9BT8:DX2;F/2:FX00bU::FXYBP04;F/012f;;F/]Rc6/0SF]00<mkcW^?Nl013g_148@00A6DDYBAU5:
DP=>L`ABU0=JePAS6003MmekgW_N00Qoo`=W>@]F]@=BU005BW9>LdYbCW=6D@02BU88@Q04?Nl00ch?
?Nln3`06@Q03?Pl00d8@?Nlmk`02?Nl00cG=>Lhec@09>Lh00cG=>Lhe[@06=Jd2;H/02C6<;H/]Rbe[
;H/YBRUZ:DXYJP039DT00bU:9BTU:@029BT00b4X9BTU:@039BT48@P01acW7@LLiad77>LM1acW008H
iP037>LHiQcW008Li`046>HLiaSV7>LI6<H25<D56<H00aC56<HDa@056<H25<D26<H011C56<HDaAS6
0QC50aS62aBU00LDaAS65<DDaABU5<DDY@036<H00aBU5<DDY@025<D01QS65<DDaABU5<DDY@I[FP40
005[FP0002A[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]J
Kg][FV]j5<DHaQC56<HDaAS65<D56<H01AC56<HDaAS65<D01QS600HLiaS66<HDYAS65:D36<H00aC5
6<HHaP026<H031cW6<HLiaSV7>LHiQcW6>HLiaSV7@LLi`8Q20039BTQ22DY00<Q200F9BTQ:2DY8BPU
:B4X9BTQ22DY8@PU:B4X9BTQ:2DY8BPU:B4X9BTQ22DY8@P59BT01B4X9BTU:BE99BT00bU:01TUBBe[
:DXYJRU:;F/YJRe[:DX]JbU:;F/YJRe[:FXaS2e[<H`]JcF]<J`e[CW^?NlikP02?Nl00ch??Nm24002
@Q001ch?BU96DDYBAU5BU4ic009>T`=BU00:E]EJeUKEHaQO5f<HGaMkgWOMNmh8Ool00f<hIcUS>007
E[D01E:dE[EB]5JeCY<00U:D00m>LdYbCW=:LTYBAU5:DT8@@S1244Ha?Pm243h?@Q000cg_00A243g_
@Q0mk`=24006?Pm243h?@Q0mkd8@0Sg_00TikSg_>NhicSG=>LhecCW>=Ld00cW>00PecCW>=LdicSG=
>Lhe[CG=13F]00H]Jc6<;F/aS2e[<H`2;F/03BU:;F/YBRU:9BTYBRDY9DTU:BU:8BPU:B4X00@U:@DQ
200A7>LQ21cW8@PLib486<HLiaS67>LHiQcW6>HLiaSV7>LHiP046<H01QSV6<HLiaC56<HDa@<HaP03
5<DHaQC5008HaP0:7>LHaQSV5<DHaQC56<HDYAS65:D56<H051C56<HHaQS65<DHaQBU6<HDYAS65<DH
aQC56<HDYAS65:DHaQC56<H55:D00aS65:DDY@035:D071S65<DHaQBU6<HDYAS65<DHaQC56<HDYAS6
5:DHaQBU6<HDYAS65:DHaV]JKg][FV]jJeY_N`000F]j000FJeX00`00JeY[FP04JeX011S65<DHaQC5
4QS600<HiQS66>H00aC521S600@LiaSV6>HHiPXLi`038BPQ224X00@Q20038BPU:B4X00@U:@068BPQ
224X8@PU:B4X1BDY00@Q:2DY8BPQ:0XU:@PYBP09:FX]Jbe[:FXYBRUZ:DX]JbUZ008]J`03;H/]Jbf;
008e[@05<J`mkcg_?NlikP03?Nl4@Q0014IABU96DDYB0dic15:D00=JeUKEF]H016<H0W_N00=ggGoo
Ool01Woo0fLi2eJe0e:D00A:LTicBW9>L`9:DP03AU52448@00I240@mk`03?Plmkch?00A24004?Pm2
448@@Q02?Pl4?Nl00cG=>Lhec@0;>Lh7=Jd00c6<;H/]R`02;H/022e[;H/YBRUZ:DXYJRE9:DX29DT6
9BT0124X9BTU:BDY124800HLiad77>LM1acW7@L27>L36>H01acW6>HLiaSV7>LHiQcW01XHaP8Da@@H
aP035<DHaQC500@HaP8Da@036<HDaAS6008HaP<Da@@HaP/DY@075<DHaQC55<DDYAC55:D00aS600<D
YAC55:D011C500@HaQBU5<DDY@I[FP40005[FP0001I_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FP<0000;JeY[NV]JKg/DaAS65<DHaQC56<HDa@056<H01AC56<HDaAS65<D0
1QS600PLiaS66<HDYAS65:DHaQC50aS601@DaAS66<HLiaS67>LHiQcW6>HLiaSV7>LHiR487>LM1b48
9BTQ22DY0b4801HU:B4X9BTQ:2DY8BPU:B489BTQ22DY8BPU:B4X9BTQ:2DY8BPU:B489BTQ20<U:@03
8BPU:BDY008U:@04:DXU:BU:9DT2:DX062UZ:DX]JbU:;F/YJRe[:DX]JbU:;F/YJRe[:FXaS2e[<H`]
JcF]<J`e[CW^?NlikP8mk`04@Q0mkd8@?Pl2@Q001TYBAU5:DTIACY=>L`=BU00<CY=BU5KEF]IFeF<H
GaMS65lGNmiggG_N27oo00=S>6LiHcP01eJe00AB]5JeD[AF]@9BU00@CY=>LdYbCW=:LTYBAU5:DT8@
AS52448`?Pm243h?@Q03?Nl03D8@?Nm243g_@Q0n3d8@?Pm243h?@Q0mkd8@008mk`07>NhmkcW^>Lhe
cCW>=Ld00cW>00HecCW>>LhicSG=>Lh2=Ld033F]>Lhe[CF]<J`e[Be[<H`]Jc6<;F/aS08]J`0=:DX]
JbU::DXU:BU:9BTYBRDY9DTQ:2DY8BP012DY1B48014Lib487>LQ21cW8@PHaQcW6<HLiaSV7>LHiQcW
6>HLiaSV00@HaP087>LHaQSV5<DHaQC56<HDa@<HaP0=5<DHaQS66>HHaQcW5<DHaQC56<HDYAS65:D0
1AS600<DaAS65<D00aS600hDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDaAS61ABU00<DaABU5:D00aBU
01XHaQC56<HDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<I[FV]jJeY_Nf]J0P000Fmk
000FJeX00`00JeY[FP04JeX00aS65<DHaP025<D00aBU5<DHaP0B6<H00aSV6<HHiP036<H26>H01QcW
6>HHiQS66>HHaP8HiP057>LHiQSV6>HLi`078@P00bDY8BPQ:0079BT012487@LQ21d70bDY00@Q:2DY
9BTU:@<UB@05:DXU:BDY9BT]J`03:FX3:DX012UZ;F/YJRUZ12U:0be[12U:0be[0bf;00@aS36/=Jde
[@@icP<mk`A24096<@05@S16<DIABU96D@02CY<00e:DCY=BU002DY@015[fG_MNmekg0f]j17_N00Ac
W6nKLiacW0=Nm`07F_INme[fG_MF]EKEE[D00e:D00A>TdYBBU9:DP=6D@04BU92<4Ha@S09?Nl00cW^
?NlicP0E>Lh7=Jd01SG==JdecC6/=Jda[0@e[@L]J`<YJP05;F/UBBU:9DTYBP038BP48@P01B4X8@PQ
:1d78@P00ad700TQ21d77>LHiQcW6>HM1acW7@L011cW0aSV00@LiaSV7>LHiP/HaP<DY@8Da@036<HD
aAS6008HaP0=5<DHaQS66<HDaAS65<DHaQC56<HDaAS65<D00QS600DHiQS66>HDaAS6008Da@035:DD
aABU008Da@036<HDaABU00<DY@035<DDYAC500@HaP<Da@LDY@045<DHaQC55<D45:D00aC56<HDa@06
JeX10001JeX0000VJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]J
JgY[FVmkJeY[NQC56<HDaAS65:DHaQBU6<HDa@<HaP035<DHaQC500THaP045<DLiaS67>L26<H04aC5
6<HHaQcW6<HLiaS67>LHaQcW6<HLiaS67>LHaQcW6<HQ21d700<Q20057@LQ22489BTQ20039BT02b4X
9BTU:BDY7@LQ21d78@PQ:2DY8BP00RDY00DYBRDY:DXU:BU:00@U:@0::DX]JbU:;F/YBRUZ:DX]JbU:
;F/2:DX02RE9:DXUBBe[:FX]JbU:;F/YBRUZ0be[01haS2e[<H`]JcF]<J`e[CW>>NhicSg_>NhmkcW^
@Q0n3d8@?Pm6<D8`AS52<4YBAU5:DTicDYA>Le:DCY<2DY@01ekgF_INme[fKg][FVmk00AkgP04Ki]c
W6nKLi`2G_L035[fG_MJmUkgF_IJeUJeF]I>Te:DCY=BU0A:DP0AAS5:DTHaAS52<4Ha?Nln3cg_@Q0i
kSg_>NhmkcW^?NlikP04>Lh01cg_>LhikSW>?NlicSW^00@icP03=LdicSW>00@icP09=Jda[3F]=Jde
cCF]>Lhe[CW>008e[@03<J`e[C6/00<e[@P]J`0@:DX]JbU::DXUBBU:9DTU:B489BTQ224X8@PU:B48
9BT28@P02AcW8@PLib487>LQ21SV7>LHiP027>L042487>LQ21SV7>LHiQcW6<HLiaS67>LHiQcW5<DH
aQC51aS600/DaAS65:DDaABU6<HDYAS65<DHaQC500DHaP095<DHaQC56<HDaAS65<DHaQC5008HaP0=
7>LHaQS65:DHaQBU6<HDYAS65:DHaQC56<H01ABU00THaQBU6<HDaAS65<DHaQBU6<H01QBU00@@Y1S6
5:DHaP@DY@0;4:@HaQC56<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP036<HDaAS6
008Da@035:DDaAS6018HaP036>HHaQSV00<HaP037>LHiQSV008HiP046<HHiQS67>L36>H00acW6>HH
iP078@P28BP89BT01R487@LQ21d79BTQ:0DU:@049DTYBRE99DT39BT2:FX00be[:FXYBP02:DX3:FX0
0be[:DXYBP02:DX3;F/4:DX3;F/012f;<H`]Rbf;0SF]00<a[3W>>Lh00SW>0cg_148@00Q2<4HaAS56
<DIABU96DE:D0djC0e:D0Ukg00EJmUkgJgY_Nf]j00AkgP=cW00=Ki]NmekgG_MJmUkgF_INmeJeE]EF
]E:DCY<00U:D0dYB00M6DDYBAU56DD8`AS52<00;?NlF>Lh7=Jd01SG==JdecC6/=Jda[0@e[@L]J`0;
:FX]JbUZ:FXUBBU:9DTYBR4X9BTQ:0048@P00b4X8@PQ:0037@L02b487@LM1ad77>LHiQcW6>HM1acW
7@L00QcW00@HiQcW6>HLi`<HiP037>LHiQS600XHaP<DY@066<HDaAC55<DHaQC51AS600LDaAS65<DH
aQC56<HDa@036<H011C56>HHaQSV0aC500@HaQBU5<DDY@8Da@036<HDaABU00<DY@035<DDYAC500@H
aP035<DHaQC500LDY@<Da@036<HDYABU008DY@035<DHaQC500I[FP40005[FP0002Q_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU6<HDaAS6
5<DHaQC56<HDa@DHaP075<DHaQC56<HDaAS65<D00aS600<DaAS66<H01aS600/HiQcW6>HLiaSV7>LH
iQcW6>HLiaSV00<Li`047@LQ21d78@P39BT00bE99BTYBP029BT01B4X9BTQ:2DY8BP00RDY00`YBRDY
:DXU:BU:9BTYBRE9:DXUBBU:9DT2:DX072e[:DXYJRU:;F/YBRe[:DX]JbU:;F/YBRe[:DX]JbU:;F/Y
JRe[:FX]JbU:;F/YBS6<;H/aS2f;0c6<13F]00XecCW>=Le6<D8@AS5244YbBU9>L`9:DP09AS5:DTIA
BU96DDYBBW9>LdYb00ABU00;IcUS66<hJeY_Nf]JKg]S>6LiHcQW>@03HaP03UkgF]INme[FE[EB]5Je
DYAB]5:DE[E6<DYBAS45@Q0043h?@Q0mkd8@?Nln3cW^?NlikSg_>LhmkcW>>NhicSg_13W>0Sg_01@i
kSg_?NlmkcW^>LhecCW>=LdicSF]>Lhe[CW>=JdecC6/=Jda[3F]0c6<00<e[C6<=Jd0136<018]Jc6<
;F/aS2UZ;F/YJRU:9DTYBRE9:DXUBBU:9DTU:B4X9BT88@P041cW8@PLib487>LQ21SV7>LHiQcW6>HL
iaSV7>LHaQcW1QS600lDaAS66>HLiaSV7>LDaAS65<DHaQC56<HDaAS65<D00aS6010DaAS65<DHaQC5
6<HDaAS65<DHaQC56<HDaAS65:DHaP@DY@0>5<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaP8DY@08
4:@DYABU6<HDYAS65:DHaPDDY@0H6<HDYAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaV]JJgY[
FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04JeX25<D011S65<DHaQC52AS600<DaAS66<H04QS60acW
00<HiQcW7>L021cW12483RDY00HUBBDY9DTU:BE99BT29DT00bU:9DTYBP0;:DX02RUZ:DXYJRU::FXY
BRUZ:DXYJRU:0be[0bUZ00@]Jbf;<H`]R`@aS0@e[@<icP04@S16<D8`@S04BU800dHaAU56<@03BU80
14IACW=>Ldic15:D0f<H00Q_Nf]jJgY[NVLiHcQW>F<h0f<H00AJeU[fF]IJmP=F]@ABU003AU56<DIA
00A24003?Pm243h?00Pmk`03>NhicSW^00LicPLmk`04=LdicSW>>Lh3=Ld00cW>=Jde[@05=Jd4<H`0
0c6/<H`a[003<H`2;H/00c6<;H/]J`02;F/022E9:DXYBRU:9DTYBRE9:DX39BT78@P01QcW7@LLiad7
7>LM1`@Li`036>HLiacW008Li`8HiP037>LHaQS600DHaP@Li`036<HDaAS6008HaP035<DHaQS600LH
aP055<DHaQC56<HDa@056<H35<D45:D00aS65<DHaP026<H02AC56<HDaAS65<DHaQC56<HDa@045:D2
5<D01AS65<DDaAC56<H00aBU00DHaQC55<DDaAS6008Da@035:DDaABU008Da@036<HDaAS600<HaPI[
FP40005[FP0002Q[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[
NV]JKg][FV]j5:DHaQBU6<HDaAS65<DHaQC56<HDa@DHaP055<DHaQC56<HDa@036<H01AC56<HHaQS6
5<D01aS600/HiQcW6>HLiaSV7>LHiQcW6>HLiaSV00<Li`047@LQ21d78@P39BT00bU:9BTUB@029BT0
1B4X9BTQ:2DY8BP00RDY00`YBRDY:DXU:BU:9BTYBRE9:DXUBBU:9DT2:DX07bUZ:DX]JbU::FXYBRe[
:DX]JbU:;F/YBRe[:DX]JbU:;F/YJRe[:FX]JbU:;F/YBS6<;H/aS2f;<H`]Rc6<00@e[@0:=LdicSG=
AS5244Ha@Q1>LdYBBW82BU802DHaBU96DDYBAU5:DTYbCW=:LP04DY@02f<hHaQW>F]JKg][FVmkHcQW
>F<hIcT00f<H00ANme[FG_MJeP=F]@07DYAF]E:DD[A6<DYBAS401D8@010n3d8@?Nln3cg_@Q0ikSg_
>NhmkcW>?NlicSg_>LhikP@icP05>Nhmkcg_?NlikP02?Nl03cW>=LdicSG=>Lhe[CW>=JdecCF]>Lha
[3F]<J`e[@03<H`00cF]<H`e[@04<H`04Re[<H`]Jc6<:FX]JbUZ:DXUBBU:9DTYBRE9:DXUBBDY8BPU
:@PQ200@7>LQ21cW8@PLib486>HLiaSV7>LHiQcW6>HLiaS67>L66<H04AC56<HHiQcW6>HLiaC56<HD
aAS65<DHaQC56<HDaAS65<D00aS600hDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS611BU00lDaAS6
5<DHaQC56<HDaAS65<DHaQC56<HDYAS64:@011BU00DHaQBU6<HDYAS6008DY@0A4:@DYABU6<HDYAS6
5:DHaQBU6<HDYAS65:DHaQC56<HDa@036<H01f]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX0
16]J00HHaQC55<DDaAS65<D;6<H00aC56<HHaP0@6<H00acW6>HLi`027>L00aSV7>LLi`067>L48@P>
9BT01RE99BTUBBDY9DTU:@8UB@03:DXUBBU:00/YBP0<:FXYBRUZ:DXYJRU::FXYBRUZ:DX]JbUZ0be[
0RUZ00<]Rc6<;H/0136<13F]0cW>0d8`00=6<DYBBU800TYB00E6<DIAAS5:DTIA009:DP03CW=:LTic
00ABU0=S609[NP06Kg][NVLiHcQW>F<h0f<H00AJeU[fF]IJmP=F]@ABU003AU56<DIA00A24003?Pm2
43h?00Pmk`03>NhicSW^00LicPLmk`8icP04=LdicSG=>Lh2=Ld7=Jd4<H`00c6/<H`a[004<H`3;H/0
0be[:FX]J`02:DX01RE9:DXUBBU:9DTYBP<U:@LQ20067>LM1acW7@LLiad70QcW00<HiQcW7>L01AcW
0QSV1aS611cW0aS600<DaAS66<H02AS600DDaAS65<DHaQC500DHaP035<DHaQC500@DY@046<HDaAS6
5<D36<H011C56<HDaAS60aC511BU00DHaQC55<DDaAS6008Da@<DY@8Da@086<HDaAC55<DHaQBU5<DD
Y@8Da@036<HDaAS600<HaPI[FP40005[FP0004i_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HD
aAS65<DHaQC56<HDaAS65<DHiQS67>LHaQcW6>HLiaSV7>LHiQcW6<HHiQS67>LHiQcW6>HLiaSV7>LH
iR487>L27@L38@P00b4X9BTQ:0059BT03B4X9BTU:BU:9BTYBRDY9DTU:BU:9DTYBRE900LYBP039DTY
BRE9008YBP0J;F/YBRe[:FX]JbUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbUZ;F/YJS6<;H/aS2f;<H`]
R`HaS004=JdaS3g_>Nh2?Nl02THa@S16<D8`AU56<DYB@S16<D8`0THa00i:DTHaCW=:LTicBW9F]E:d
E[EB]5JeD[AF]E:d0eJe00=B]5JeD[@015Je00=JeUKEF]H00UJe00UB]5JeBW9>LdYbAS52<4Ha@S00
0cg_00DicSg_>LhmkcW^00<mk`09>LhmkcW>?NlicSg_>LhmkcW^008mk`05@Q0mkd8@?Nln3`02?Nl0
2cW>>NhicSW>=JdicSF]>Lhe[CW><J`00cF]136<00X]Rc6<;H/aS2e[<H`YJRe[:FX]J`@YBP069BTY
BRDY:DXU:BU:12DY00HM1b488@PQ21cW8@P57>L01R487>LQ21SV7>LHiP8Li`067@LLib486>HLiaSV
1AS600dDYAS65<DHaQC56<HDaAS65<DHaQC56<HDa@036<H03aC56<HDaAS65:DHaQBU6<HDYAS65:DL
iaS66>HHaP035:D46<H01QBU6<HDYAS65<DHaP8Da@065:DHaQBU6<HDYAS60QBU018@Y1BU5:DHaQBU
6<HDaAS65<DHaQC56<HDYAS65:DHaQC56<H25<D04ABU6<HDYAS65:DHaQBU6<HDYAS6JeY[NV]JKg][
FV]j00000Fmk000FJeX00`00JeY[FP04JeX25<D011S65<DDaABU11C500<HaQC56<H00QS600TDaAS6
6<HHaQC56<HDaAS65<D01AS600LHiQcW6>HLiaSV7>LHiP046<H37>L00aSV7>LHiP047>L48@P00bDY
8BPU:@049BT00b4X9BTQ:0029DT00bU:9DTU:@039BT;:DX012E9:DXUBBe[0RUZ0Re[00DYJRe[:FX]
JbUZ00D]J`03:FX]JbUZ00<]J`07;H/aS2f;<H`]Rc6<;H/0136<00<e[C6/<J`013g_0THa00=2<4Ha
AS401THa00A6DDHaAU5:LP=>L`05D[AF]E:dE[EB]007E[D00e:dE[EF]@03E[D00eKEF]IFe@04E[D0
0dicBW9>L`04AS44?Nl3>Nh4?Nl3>Lh00cW^?NlikP02>Nh01Cg_>Nhn3cg_?Pl013g_13W>00<e[CG=
=Jd013G=13F]0c6<00H]Rc6<;H/aS2e[;H/4;F/012UZ:DXYBRU:0bE900DYBRE99DTUBBU:00<U:@@Q
20037@LLiad700DLi`037@LLiacW00XLi`@HaP<Da@036<HDaAS6008HaP035<DHaQS600XHaP8Da@04
6<HDaAC55<D56<H35:D46<H00aC55:DDa@026<H00aC56<HDY@035:D00aC56<HDa@045:D25<D56<H0
2QC56<HHaQC55:DDaABU6<HDaAS611BU00<DaAS65<D00QC500<HaQC5JeX01F]J0@000F]J0000A6]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDYAS6
5:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAcW6<HHiQS67>LHiQcW
6>HLiaSV7>LHaQcW6<H26>H021cW6>HLiaSV7>LHiQd77>L38@P01Ad78@PQ:2DY8BP01BDY00TQ:2DY
9BTYBRDY:DXU:BU:9BT00RE900<YBRE9:DX01RU:00<UBBU:9DT00RU:01X]JbU:;F/YJRe[:FX]JbUZ
;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbUZ<H`]Rc6<;H/aS2f;1S6<00<e[C6<?Nl00Sg_00/ikTHa@S16
<D8`BU96<DIA@S16<D8`0096<@0<BU96<DicBW9>LdYbE[EB]5JeD[AF]E:d0eJe00EB]5JeD[AF]E:d
00AF]@04F]IFeE[FD[@3E[D01dYbCW=:LTHa@S16<D8`00<mk`03>LhmkcW>00<mk`0;>NhmkcW>>Nhi
cSg_>LhmkcW>?NlikP02?Nl02D8@?Nm243g_@Q0mkcW^>Lhmk`02>Lh01SF]>Lhe[CW>=JdicP8e[@03
<J`e[C6<00<aS00:;H/aS2f;<H`]Jc6<:FX]JbUZ;F/4:DX01RDY:DXU:BU:9BTYBP8U:@088BPU:B48
8@PM1b487>LQ20DLi`068@PLib486>HLiaSV0QcW00HQ21cW7@LHiQcW6>H56<H03ABU6<HDaAS65<DH
aQC56<HDaAS65<DHaQC500<HaP0?5<DHaQC56<HDYAS65:DHaQBU6<HDYASV6<HLiaS600<DY@@HaP0?
5:DHaQBU6<HDaAS65<DHaQBU5<DDYAS65:DHaQ2T00@DY@0R6<HDYAS65<DHaQC56<HDaAS65:DHaQBU
6<HDaAS65<DHaQBU5<DDYAS65:DHaQBU6<HDYAS6JeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J
00A[FP046<HDaAC55<D56<H02AC56<HDaAS65<DHaQC56<HDa@0;6<H46>H011S66>HHaQcW0aSV00<L
iaSV7>L01acW12480bDY1bU:00<UBBDY9DT022DY00<UBBU:9DT00bU:00LUBBUZ:FX]JbUZ;F/YJP05
;F/01BUZ;F/]Jbe[:FX00be[00<YJRe[;F/01Be[00DYJRe[<H`aS2f;00LaS0@e[@@icP<mk`04@Q0n
3d8@?Pl3AS400dIABU96D@02AU401TYBAU56DDHaAU56<@=:DP03AU5>Ldic009>L`06CY=>LdjCCY=B
U4jC15:D0dYB1d8@0Sh?00E243h??Plmkch?008mk`03>NhmkcW^008ikPDmk`04>NhmkcW^?Nl2>Nh4
>Lh4=Jd01S6/=Jda[36/<H`a[08aS006;H/aS2e[;H/]Jbf;12e[0bUZ00L]JbUZ;F/YBRUZ:DXYJP04
:DX29BT00b4X9BTU:@029BT38@P00acW7@LLi`027@L28@P87>L00aSV6<HHiP066<H00aC56<HHaP03
6<H00aC56<HHaP036<H011BU5<DDYAS60aC500XHaQC56<HDaAS65<DHaQBU5<DDY@8Da@066<HDaAC5
5<DHaQC50aBU0aSV00DLiaC55<DDaAS600<DY@036<HDaAC5008Da@035:DDaABU00@DY@065<DDYAC5
5:DDaABU0QC500<HaQBU5:D00aBU00<DaABU5<D00aBU1V]J0@000F]J000086mkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DYAS65:D26<H031SV6<HH
aQC56<HDaAS65<DHaQC56<HDa@THaP0D5<DHaQS67>LHaQcW6<HLiaS67>LHaQcW6<HLiaSV7>LHiR48
7>LM1`@Li`057@LQ22488@PQ:0029BT7:DX04BDY:DXU:BU:9BTUBBDY:DXU:BE99BTYBRE9:DXUBBU:
9DT00RU:00T]JbU:;F/YJRe[:FX]Rbe[<H`00Re[00TYJRe[:FX]JbUZ;F/YJRe[:FX00be[00dYJRe[
:FX]JbUZ<H`]Rc6<;H/aS2f;<H`]R`02<H`4=Jd4>Lh3?Nl013h?@Q0n3d8@0dHa01Q:DTHaBU96<DYB
AU5:DTHaBU96<DYBAU5:DTIABU9:LTicBW9BU4icDYA>Le:DCY<3DY@00djCDYA:DP02BU87@Q0063g_
@Q0mkd8@?Nm243g_@Q0ikSg_>NhmkcW>?NlicSg_>NhmkcW^?NlikSg_>Lhmk`8icP06=LdicSG=>Lhe
[CG=0SF]00`a[3F]<H`e[C6<=Jd]Rc6<;H/aS2e[<H`2;F/032UZ;F/YBRe[:DX]JbUZ;F/YJRe[:DX]
J`8YBP0:9DTYBRE99BTQ:2DY8BPU:B4X9BT48@P041cW8@PLib487@LQ21SV7>LHiQcW6>HLiaSV7>LH
aQcW0QS602XDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDaAS6
5<DHaQC56<HDaAS65:DHaQBU6<HDaAS65:DHaQBU6<H35:D00acW6<HLi`026<H0;ABU6<HDYAC55:DH
aQBU6<HDYAS65:DHaQBU6<HDYAC55:DHaQBU6<HDYAS65:DHaQBU6<HDYAC55:DHaQBU6<HDYAS65:DD
aABU6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FP8Da@036<HDaAS6008HaP0;5<DH
aQS66<HDaAS65<DHaQC56<HDa@0;6<H01aSV7>LHiQSV6<HHiQS6008HiP047>LHiQcW6>H87>L48@P3
9BT7:DX00bE99BTUB@089BT01BE9:DXUBBU:9DT00RU:00H]JbUZ:FXYJRe[:FX7;F/01BUZ;F/]Jbe[
:FX00be[00DYJRe[;F/]JbUZ00<]J`03;H/aS36<00LaS0@e[@@icP<mk`04@Q0n3d8@?Pl3AS43AU40
0dYBAU5:DP02AU401DHaAU56<DYBAU400TYB14ic00I>TdicCY=>Te:DCY<4DY@3BU88@Q04?Pl023g_
?Plmkcg_>NhmkcW^?Nl2>Nh4?Nl00cW^?NlikP02>Nh00cg_>LhicP02>Lh4=Jd01S6/=Jda[36/<H`a
[08aS006;H/aS2e[;H/]Jbf;0be[0RUZ00T]JbUZ;F/YJRe[:DXYJRU::FX012U:00<Q:2DY9BT012DY
0b4800@Liad77>LM1`8Q20037@LLiacW00HLi`036>HHaQSV00@HaP035<DHaQS600DHaP035<DHaQS6
00<HaP035:DDaABU008Da@0<6<HDaAS65<DHaQC56<HDaAS65:DDaABU0QC500<HaQC56<H00aC50aBU
00HHiQcW6>HHiQC56<H25<D35:D25<D01AS65<DDaABU5<D01ABU00LDaABU5<DDYAC55:DHaP025<D5
5:D00aC55:DDa@035:D6JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQBU6<HDY@8HaP0<7>LHaQS65<DHaQC56<HDaAS65<DHaQC5
2aS6018DaAcW6<HLiaS67>LHaQcW6<HLiaS67>LHiQcW6>HM1acW8@P27>L01QSV7>LQ22487@LQ208U
:@038BPYBRU:00DYBP0A9BTYBRDY:DXU:BU:9BTUBBDY:DXU:BU:9DTYBRE9:DXUB@02:DX02Be[:DX]
JbUZ;F/YJS6<;F/]R`02;F/02bUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX00be[00TYJRe[:FXaS2f;<H`]
Rc6<;H/00c6<00<]RcF]=Jd00SF]13W>0cg_00@n3d8@?Pm240=6<@0JBU96<DYBAS5:DTIABU96<DYB
AS5:DTIABU96DDYBBW9>LdYbDYA>Le:DCW=BU4jCDYA>T`=BU0=:DPE2400J?Pm243g_@Q0mkd8@?Nm2
43g_@Q0ikSg_>NhmkcW>?NlicSg_>NhmkcW^?NlikSg_>Lhmk`8icP06=LdicSG==Lde[CW>0SF]00`a
[3F]<H`e[C6<=Jd]Rc6<;H/aS2e[<H`4;F/02RU:;F/YBRe[:FX]JbUZ;F/YBRe[0RU:00XUBBU:9DTU
:B4X9BTQ:2DY8BPU:@@Q200@7>LQ21cW8@PM1b486>HLiaSV7>LHiQcW6>HLiaS67>L26<H0:QC56<HD
aAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaQC56<HD
YAS65:DHaQC56<HDYAS65:DHaP<DY@037>LHaQcW008HaP0]5:DHaQBU6<HDYAC55:DHaQBU6<HDYAS6
5:DDaABU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAC55:DHaQBU6<HDYAS65:DDaF]JKg][FV]j
JeY_N`00005[NP005V]J00<006]JJeX016]J1aS600@DaAS65<DDa@HHaP055<DHaQS66<HDa@0:6<H4
7>L46<H57>L00aSV7>LLi`027>L012487@LQ21d70b480bE900XYBRDY9DTU:BE99BTUBBDY:DXUB@8Y
BP059DTU:BE99DTYBP039DT01BU:9DTYJRU::FX012U:00@YJRe[:FX]J`@YBP<]R`h]J`8]R`03<H`]
Rc6<008aS0@e[@04=LdicSG=>Lh3=Jd01cg_>NhikSW^@S12448`0096<@03@S16<D8@00=24003?Pm2
43h?00A240=6<@09AU5:DTYBBU9:LTicBW9:DTIA009:DPY240@mk`8ikP05?NlikSW^>LhikP04?Nl3
>Lh013g_>NhikSW^0cW>00@e[CG==Jdec@LaS0<e[@04<J`aS2f;<H`4:DX4;F/00bUZ;F/]J`03:DX5
;F/01Rf;;F/]RbUZ;F/YJP@U:@LQ20047>LM1acW8@P37@L011cW6>HHiQSV4AS600@DYAC55:DDa@@H
aP095<DHaQS65<DDYAC55:DDaABU008Da@036<HDaAC5008HaP035<DDYABU00TDY@<Da@036<HDYABU
00TDY@035<DDYAC500hDY@045<DHaQC56<H35:D6JeX10001JeX00008Kg][FV]jJeY_Nf]JJgY[FP80
0004JgY[FVmkJeX200003FmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk00@HaP075<DHaQC56<HDYAS6
5:D01AS600LDaAS65<DHaQC56<HDa@096<H27>L03aSV7>LHaQS65<DHaQSV7>LHiQcW6>HLiaSV8@PL
i`027@L07R487@LQ22489BTQ22U:9BTYBRDY:DXU:BU:9BTYBRDY:DXUBBU:9DTYBRDY:DXU:BU:9BTY
BRDY:DXUB@8YBP0:;F/YBRe[9DTYBRE9;F/YJRe[:FX4:DX01S6<;F/aS2UZ;F/YJP<]J`05:FX]JbUZ
;F/YJP04;F/0236<;F/aS2f;<H`]RcF]<J`2=Jd00cW>=LdicP02=Ld04cF]>LhicSg_>Lhmkd8@AS52
44Ha@S16<D8`@Q0n3d8@?Pm243h?00924004AS52448`@S02AS402dYBAU5:DTIACW=:LTicAU5:DTIA
BU801T8@01<n3d8@@Q1243W^?NlikSg_>LhmkcW>?NlicSg_>LhmkcW^?NlikP04>Lh02Cg_>LhmkcG=
>LhecCW>=JdicP02=Jd01C6<<J`aS3F]<H`00S6/00He[C6/=Jd]Rc6<;H/4:DX3;F/012UZ;F/YJRe[
0bU:00L]JbUZ;F/YJS6<;F/aS002;F/01bU:;F/Q:2DY8BPU:Ad700<Q200@7@LQ22488@PLib487>LQ
21cW8@PHaQcW6<HLiaS66>H56<H00acW6<HHiP046<H0:QC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC5
6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65:DHaQBU5<DDYAS65:DDaABU6<HDYAS65:DHaP8D
Y@034:@DYABU00<DY@0:4:@DYA2T6<HDYAS65:DHaQBU5<D55:D02QS65:DDaABU6<HDYAS65<DHaQC5
0aBU00M[FV]jJeY_Nf]JJgX00001Kg/000M[FP04001[FV]J0002JeX01000JeY[FP001F]J00<006]J
JeX016]J1aS60aC51AS600DDaAS66<HHaQC500`HaP@Li`@HaP<Li`036>HLiacW00@Li`048@PM1b48
7@L38@P02bE9:DXUBBE99BTUBBDY9DTU:BE99BT00bU:0RE900<U:BE99DT00RE900LYBRE9:DXUBBUZ
:DXYJP02:DX01RE9:DXYJRe[:FX]J`@YBP03;H/aS2f;00h]J`04<H`]Rbf;;H/3<H`4=Jd013G=>Lhe
cCW>0cF]0SW^00@mkcW^@S124092<0=6<@A24003?Pm243h?00A240=6<@9:DP05AU5:DTYbCW=:LP03
BU800dIA@Q124008@Q05?Nl4>Nh00cW>>Nhmk`03?Nl3>Lh2>Nh00cg_>NhicP02>Lh013F]=Lde[CG=
1c6<00Le[C6/=Jde[C6<;H/aS004:DX6;F/012UZ:DXYBRU:1Be[00<]Rbe[;H/00bUZ12DY1b4800<L
iad77>L00Qd700HQ21d76>HHiQcW6>HA6<H011BU5<DDYAC51QS60QC500DDYAC55:DDaABU00@Da@04
6<HDaAS66<H;5:D011C56<HDaAC52aBU00<DaABU5<D03QBU00@DaAS65<DHaP<DY@I[FP40005[FP00
01I[NV]JKg][FV]jJeY_N`00JgY[FP00JeY[NP00Kg][FP00JeY_Nf]JJgY[FP@0000<Kg][FV]j6>HL
iaSV7>LHiQcW6>HLiaSV0acW00DHaQcW6<HHaQC500HHaP0:7>LHaQcW6<HHiQS67>LHaQcW6>H37>L0
11S67>LHaQSV0aS601XLiaSV7>LHiR487@LQ21d78@PM1b488BPU:B4X9BTQ:2DY8BPYBRDY9DTQ:2DY
8BPU:BE90bU:01@UBBU::DXYBRE9:DXUBBU:9BTYBRE9:DXUBBU:9DTYBRE9;F/YBRUZ12U:00LUBBe[
:DXYJRU:;F/YBP02:FX022e[:FX]JbUZ;F/YJRe[:FX2;F/03S6<;F/aS2f;<H`]Rc6<;H/aS2f;=Jda
S36/<H`4=Jd02CW>=JdicSW^?NlikT8@?Nm24002?Nl07SW^?NlicSg_>LhmkcW>?NlicSg_?Pm243h?
AS52<4Ha@S1:DTHaBU9244Ha@Q16<Ch?@Q0n3d8@?Pm2408n3`0:?Nm243W>>NhicSg_>LhikSW>?Nl4
>Lh00cG=>Lhec@03>Lh01cG=>LhicSW>=LdicSG=00@e[@03<H`]Rc6<008aS00D;H/aS2UZ;F/YJRe[
:DX]JbU:;F/YJRe[:FX]JbUZ;F/UBBU:9DTYBP@]J`0H:FX]JbUZ;F/YJRe[:FX]JbU:;F/Q:2DY8BPU
:B4X9BTQ:2487@LQ21d78@PLiad71AS600TLiaS67>LDaAS65<DLiaS67>L00QS600DDaAS65:DHaQBU
00<HaP0F5<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<H45:D03aC56<HD
aAS65<DHaQC56<HDaAS65<DHaQC56<HDa@035:D011C56<HDaAS61aBU01THaQC56<HDYAS65:DHaQBU
6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<I[FVmkJeY[NP030001JgX000Q[FP8000=[FP04001[FV]J
0005JeX00`00JeY[FP04JeX01QcW6>HLiaSV7>LHiPDLi`046<HHiQS66>H76<H26>H00acW6<HHaP03
6<H00aSV6<HLi`037>L76<H47>L68@P01ad79BTU:BDY8BPU:B4X00PU:@/YBP039DTYBRE900DYBP03
9DTYBRU:00hYBP<]J`03:FX]JbUZ00@]J`04;H/]Jbf;;F/;<H`3=Jd013W>=LdecCG=0cg_00@n3d8@
?Pln3`<mk`8ikP0<?NlikSW^>NhmkcW^@Q0n3d8@@S16<D8`0THa00=6DDHaAS400d8`00=243h?@Q00
148@0cg_2cW>00<ecCW>=Ld02cW>13F]136<00<]Rc6<;H/012e[0RUZ1Be[00LYJRe[:FXYBRE9:DXU
B@0;;F/01bU::FXYBRDY8BPU:B4X00<U:@047@LQ22488@P37>L46<H02AcW6>HHiQSV6<HDaAS66>HL
i`026>H36<H011C55:DDaABU0aS600LDaAS66<HHaQC56<HDa@046<H01AC55:DDaABU6<H00QC500@D
YAC55:DDa@<DY@046<HDaAS65<D76<H011C56<HDaAS60aBU11S61aBU00<DaAS65<D00QC500@HaQC5
5<DDY@@Da@056<HDaAS65<DHaP035<D6JeX10001JeX0000TKg][FV]jJeY_Nf]JJgX006mkJeX006]J
Kg/006]jJeX006]JJgY[FVmkJeX006]JKg][FV]jJeY_NaSV7>LHiQcW6>HLiaSV0acW00@HiQcW6<HL
i`8HaP035<DHaQS600@HaP087>LHaQSV6<HLiaS67>LHaP<Li`066>HLiaS66>HHaQcW0aS600@LiaSV
7>LHiP<Q200B7@LQ21d78@PQ:2DY8BPU:B4X9BTQ:2E99BTYBR4X9BTQ:2DY0RU:01HUBBU::DXYBRE9
:DXUBBU:9DTYBRDY:DXUBBU:9DTYBRE9:DXUBBUZ:DX]J`8YBP0C9DTYBRU::FXYBRe[:DXYJRU:;F/Y
JRe[:FX]JbUZ;F/YJRe[:FX00Re[00haS2e[<H`]Rc6<;H/aS2f;<H`]Rc6/<H`e[C6<13F]00TicSF]
>LhikSg_>Ni243g_@Q000Sg_02XikSg_>LhmkcW>?NlicSg_>Lhmkch?@Q0n3dHa@S16<D8`BU96<DYB
@Q16<D8@AS4n3d8@?Pm243h?@Q0n3d8@?Nln3cW>?NlicSW^>LhmkcW>>Nh4>Lh00cG=>Lhec@05>Lh0
1CG=>LhecCW>=Ld013F]0c6<01H]Rc6<;H/aS2UZ;F/YJRe[:DX]JbU:;F/YJRe[:FX]JbUZ;F/UBBU:
9DTYBP@]J`0D:FX]JbUZ;F/YJRe[:FX]JbU:;F/Q:2DY8BPU:B4X9BTQ:2487@LQ208M1`037>LQ21S6
00@HaP097>LHaQcW5<DHaQC57>LHaQcW008HaP075<DHaQBU6<HDYAS65<D00aS601@DaAS65<DHaQC5
6<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAS611BU00lDaAS65<DHaQC56<HDaAS65<DHaQC56<HD
aAS65<D00aBU00@DaAS65<DHaPLDY@0L6<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC5
6<HDYAS6JeY[NV]JKg][FV]j0001Kg/000Q[FP8000A[FP8000I[FP03001[FV]J00A[FP067>LHiQcW
6>HLiaSV1AcW00@HaQSV6<HHiPLHaP037>LHiQSV00DHaP036>HHaQcW00<Li`LHaP@Li`@Q20037@LQ
224800<U:@038BPU:B4X00PU:@/YBP<UB@DYBP039DTYBRU:00hYBP<]J`03:FX]JbUZ00@]J`04;H/]
Jbf;;F/;<H`3=Jd2=Ld00cW>=Ldmk`02?Nl3?Pl00d8@?Nlmk`02?Nl3>Nh02Sg_>NhikSW^@Q0n3d8@
@S16<D8`0THa00I6DDHa@S12<4Ha@S07@Q03?Nl;>Lh00cG=>Lhec@0;>Lh4=Jd4<H`00bf;<H`]R`05
;F/2:FX4;F/01bUZ;F/YJRU:9DTYBRE900/]J`0::DXYJRU:9BTQ:2DY8BPU:B4X9BT28@P00ad78@PL
i`027>L46<H26>H01AcW6>HHaQC56<H00aSV00PLiaS66<HHaQC55:DDaABU1AS600DDaAS65<DHaQC5
00@HaP045<DDYAC55:D25<D01AS65:DDaABU5<D00aBU00HHaQC56<HDaAS65<D56<H011C56<HDaAS6
0aBU11S61aBU00@DaAS65<DHaP@Da@045:DDaAC56<H35<D01QS65<DHaQC56<HDa@I[FP40005[FP00
01i[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j
5<D36<H01QSV7>LHiR487>LQ208Li`056<HLiaS66<HDa@026<H00acW6<HLi`036<H04QcW6<HHiQS6
7>LHiQcW6<HLiaS67>LHaQcW6<HLiaS67>LHaP@Li`068@PLib487>LQ21cW0R4800PQ:2489BTQ:2DY
8BPU:B4X1BDY00HUBBU:9DTYBRE9:DX29DT01BDY:DXU:BU:9DT00bU:00`UBBU:9DTYBRE9:DXUBBU:
9DT]JbU:;F/4:DX02RUZ;F/YJRe[:FX]JbUZ;F/YJRe[0RUZ00DYBRe[:FX]JbUZ008]J`0:;H/]Jc6<
;F/aS2e[<H`]Rc6<;H/3<H`4=Jd03CG=>LhecCF]<J`e[C6/=Lde[CW><J`e[C6/00<e[@05<J`e[CW>
?NlicP03AS401d8`AS5244Ha>NhmkcW^008mk`03@Q0mkcg_00<mk`03@Q0mkd8@00@icP03=JdecCF]
008icP03?NlicSW>00@icP0;=JdicSF]>Lhe[CW><J`e[C6/<H`]R`03<H`042f;<H`aS36<;F/aS2e[
<H`]Jc6<:DXYJRU:;F/YBRe[0RU:00DUBBU::DX]JbU:008YJP<]J`05:DX]JbU::DXUB@03:DX022E9
:DXU:BU:9BTYBR489BT28@P00acW8@PLi`047>L022487>LQ21SV7>LHiQcW5<D26<H02QcW6<HLiaS6
7>LHaQcW5<DHaQC50aS600DDaAS65<DHaQC500<DY@0=5<DHaQC56<HDYAS65:DHaQC56<HDaAS65<D0
0aS600<DYAS66<H00aS600HDaAS65<DHaQBU6<H55:D01QS65:DHaQ2T5:D@Y08DY@0R6<HDYAS65:DH
aQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS6JeY_Nf]JJgY[FVmk0001
JgX001I[FP03001[FV]J00A[FP@HaP057>LHiQcW7@LQ20027@L01aSV7>LHiQSV5<DHaQC5008HiP03
7>LHiQS600HHaP<Li`056>HHaQSV6<HLi`036>H00acW6>HHiP057>L01Qd77>LM1ad78@PM1`<Q20/U
:@03:DXUBBU:008YBP039DTYBRDY00<U:@039DTYBRE900DYBP039DTYBRU:00<YBP<YJP06;F/YBRU:
:DX]JbUZ2Be[0bU:0be[00@YJRe[;F/]J`<]R`PaS0@e[@03>LhecCW>00Xe[@03<J`e[CF]008e[@8i
kP03?Nm6<DHa0096<@=2<0/mk`03?Pm243h?00@icP@e[@03>NhicSW^00@icP<ec@07>LhecCG==Lde
[C6/=Jd0236<00L]Jbf;;F/]Rbe[;H/]J`03:DX3:FX00be[:DXYBP05:DX3;F/012UZ;F/YJRUZ1bU:
0RE900DYBRE98BPQ:2DY00<M1`058@PLiacW7>LQ20037@L47>L46<H01ASV6<HHiQS66>H00QS600<D
aAS65<D00aS600@DaAS65<DHaP<DY@036<HDaAS6008Da@055:DDaABU6<HDa@056<H00aC56<HDa@07
6<H35<D00aS65:DDY@025:D35<D75:D03QC56<HDaAC55:DDaABU5<DDYAC55:DHaQC56<H35<D011S6
5:DDaABU0QC500<HaQC5JeX01F]J0@000F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QS600PDaAS66>HLiaSV8@PLib480QcW00DHaQcW
6<HHaQC5008HaP037>LHaQcW00<HaP0B6>HHaQcW6<HLiaSV7>LHaQcW6<HLiaS67>LHaQcW6<HLiaS6
11cW00HQ21cW8@PLib487>L28@P022DY8@PU:B4X9BTQ:2DY8BP59BT03bE9:DXUBBU:9DTYBRE9:DXU
:BE99BTYBRE9:DXUB@03:DX02RE9:DXUBBU:9DTYBRE9;F/YBRe[0RU:00lUBBU::FX]JbUZ;F/YJRe[
:FX]JbUZ;F/YJRe[:DX00RUZ014]JbUZ;F/]Jc6<;F/aS2e[<H`]Jc6<;H/aS2f;<H`]Rc6<00@e[@0?
=LdicSG==Jda[3F]<J`icSF]=Lda[3F]<J`e[C6/00<e[@05>LhmkcW>AS52<003AS401D8@AS4ikSg_
>Nh00Sg_00<n3cg_?Nl00cg_00=243g_@Q0013W>00He[CW>=JdecCW>?Nl6>Lh02CF]>Lhe[CW>=Jdi
cS6/=Jda[003<H`04Rf;<H`aS36<;H/aS2e[<H`]Jc6<;F/aS2U:;F/YBRe[:DX]J`8YBP059DTYBRU:
:FXYBP03;F/012UZ;F/YBRe[12U:00XUBBU::DXYBRDY:DXU:BU:8@PU:@8Q20037>LQ21cW00@Li`07
8@PLib486>HLiaSV7>L00QS600dDaAcW6<HLiaS67>LHaQcW5<DHaQC56<HDa@036<H00aC56<HDa@03
5:D02aC56<HDaAS65:DHaQBU6<HDaAS65<D00aS600<DaAS65:D01AS600HDaAS65<DHaQBU6<H55:D0
1QS65:DHaQ2T5:D@Y08DY@0R5<DDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS6
5:DHaQBU6<HDaAS6JeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FPPHaP036>HHaQSV00@H
aP035<DHaQS600@HaP057>LHiQcW6>HLi`036>H02AcW6>HLiaSV7>LHiQd77>LM1`087>L00ad78@PM
1`058@P27@L012488BPQ224X0bDY0bU:00<UBBDY9BT00RDY1bU:00<UBBU::DX01bU:00<UBBU:9DT0
4bU:0bUZ00<]JbUZ;F/00Re[0RUZ12U:12e[00<]Rc6<;H/00c6<00@]RcF]=Jde[@/aS0@e[@03=Lde
[CG=00LicP04=Lde[CG==Jd3?Nl01CW^?NlikSg_>Nh00Sg_00/icSG==LdecCW>=LdicSG=>LhecCW>
00Pe[@03<J`e[C6/00<aS0<]R`05<H`]Rbf;;H/aS004;F/00bUZ;F/]J`08:DX00bUZ:DXYBP03:DX2
:FX00be[:DXYBP02:DX49BT01R4X9BTU:BDY8BPU:@8Q:0038@PQ:24800<Q20066>HLiaSV6>HHaQSV
1AS600<HiQS66>H011S60aBU1AS600<DaAS65:D00aBU3QS600@DaAS65<DHaP<DY@LHaP085:DDaABU
5<DDYAC55:DDa@LDY@<Li`LDY@066<HDaAS66<HDaABU0QC500HHaQC56<HDYAC55:D25<D00aS65<E[
FP05JeX10001JeX0000QJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX0
06]JJgY[FVmkJeY[NQS67>LHaQSV00<HaP037>LHaQcW008HaP0G5<DHaQS66<HDaAS65<DHaQC56<HH
iQcW6>HLiaS67>LHaQcW6>HLiaSV7>LHiP027>L032487>LQ21SV7>LHiQd77>LQ21cW8@PM1`<Q2009
7@LQ21cW8@PLibDY8@PU:B4800<U:@049DTYBRE9:DX49BT05BE9:DXUBBU:9DTYBRE9:DXUBBU:9DTY
BRE9:DXUBBU:9DTYBRE9:DXUB@08:DX01Be[:DXYJRU:;F/01BU:00T]JbU:;F/YJRe[:FX]JbU:;F/0
1BU:00haS2e[;H/]Jc6<;F/aS2f;<H`]Rc6<<J`e[C6/136<00Ha[36<=JdaS36/<H`3=Jd01C6/=Jde
[CW>=Jd013W>01lmkcW>>Nhe[CW>=JdicSW^?NlikSg_>NhmkcW^?NlikSg_=JdicSF]>LhecCW>=Ldi
cSG=>LhecCW>=Jdec@04=Jd0136/=Jda[3F]136<00H]Jc6<;F/aS2e[<H`2;F/01RUZ;F/YJRe[:FX]
J`8YBP0>9DT]JbU::FXYBRe[:DX]JbU:;F/YBRUZ:DX]J`DYBP039DTU:BU:008U:@088BPU:B4X9BTQ
:2DY8@PU:@@Q20087@LLiaSV7>LHaQcW6<HLi`DHaP087>LHaQSV6<HLiaS65<DDY@DHaP0;5<DHaQC5
6<HDYAC55:DHaQC56<HDa@076<H01aC56<HDaAS65<DHaQC500<DY@0>5<DHaQC56<HDaAS65<DHaQBU
6<HDYAS65:DHaP@DY@094:@DYA2T5:DHiQcW6>HDYA2T00<DY@0K4:@DYAC56<HDaAS65:DHaQBU6<HD
aAS65<DHaQBU6<HDYAS65:DHaV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J21S600<H
iQS66>H01QS600<DaAS66<H00QS600<LiaSV7>L00aSV00/LiaSV7>LHiQcW6>HLiaSV7@LLiad700PL
i`037@LQ21d700@Q208M1`8Q20038BPQ224X00<U:@04:DXUBBU::DX49BT00bU:9DTYBP06:DX00bE9
:DXYBP05:DX00bE9:DXUB@0C:DX00bUZ;F/YJP03;F/3:FX00be[:DXYBP02:DX4;F/3;H/0136<;H/a
S36<0cF]2c6<13F]00<ecCF]=Ld01cW>00XecCF]=Lde[Cg_>NhmkcW^?NlikP<mk`0<>NhecCG=>Lhe
cCW>=LdicSG=>LhecCW>23F]00<a[3F]<J`00c6<00<]Rc6<;H/00Rf;00<aS2f;;H/01Re[00<YJRU:
:DX01RU:00<YJRU::DX00bU:00<]JbUZ:FX012U:1RDY00@Q:2DY8BPU:@8Q:0038@PQ:24800<Q2006
6>HLiaSV6>HHaQSV1AS600<HiQS66>H011S60aBU1AS600<DaAS65:D00aBU31S600HDaAS65<DHaQC5
6<H35:D76<H021BU5<DDYAC55:DDaABU5<D75:D00acW6>HLi`075:D36<H25<D02ABU5<DDaAS65<DH
aQBU5<DDY@045<D6JeX10001JeX0000QKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_NaS66>HHaQcW00<HaP037>LHaQcW00@HaP0E5<DHaQC56<HDaAS6
5<DHaQSV7>LHiQcW6<HLiaS67>LHiQcW6>HLiaSV008Li`0>8@PLib486>HLiaSV8@PLiad77>LQ21d7
8@PM1`<Q200>7>LQ21cW9BTQ22DY8@PU:B4X9BTUBBU:9DTYBP@U:@0E9DTYBRE9:DXUBBU:9DTYBRE9
:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRE900PYBP05:FXYBRe[:DXYJP05:DX02Be[:DX]JbUZ;F/YJRe[
:DX]J`05:DX03Rf;;F/aS2e[<H`]Jc6<;H/aS2f;<H`a[3F]<J`4<H`01SF]<H`a[36<=JdaS08a[0@e
[@03>Lhe[CW>00<icP0K>NhicSg_=JdicSF]>LhikSg_>NhmkcW^?NlikSg_>NhmkcF]>Lhe[CW>=Ldi
cSG=>LhecCW>008ec@03=JdicSF]00<e[@04<J`e[C6/=Jd4<H`01Re[<H`]Jc6<;F/aS0@]J`05:FX]
JbUZ;F/UB@02:DX03BUZ:DX]JbU:;F/YBRe[:DXYJRU:;F/YBRe[00HYBP0<9BTUBBDY9BTQ:2DY8BPU
:B4X9BTQ22DY0R4800XM1b488@PLiaSV7>LHaQcW6<HLi`DHaP057>LHaQcW6<HHiP026<H00aBU5<DH
aP036<H02AC56<HDaAC55:DHaQBU6<HDa@036<H00aC56<HHaP036<H01aC56<HDaAS65<DHaQC500<D
Y@0>5<DHaQC56<HDaAS65<DHaQBU6<HDYAS65:DHaP@DY@074:@DYA2T5:DHiQcW6>H00aBU01d@Y1BU
5:DDYAC56<HDaAS65:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65:DHaV]JJgY[FVmkJeY[NP00005_N`00
5V]J00<006]JJeX016]J0aS600<DaAS66<H01AS600<DaAS66<H02aS62QcW00<HiQcW7>L031cW00HQ
21d78@PM1b487@L88@P49BT01bE99BTUBBDY9DTU:BE900@U:@PYBP8UBAHYBP06:FX]Jbe[:FXYBRUZ
1BU:00<YJRU::FX01Be[00<]Rbe[<H`00S6<0bf;1C6<00<]Rc6<;H/0236<00<a[36<<J`013F]00<e
cCW>=Ld01CW>00TecCW>=LdicSG=>LhaS36/<H`013F]00<ecCF]=Ld00SF]0S6/13F]1S6<00<]Rbe[
;F/02Re[00<]Rbe[;F/00be[00<YJRU::FX02bU:0bDY00<Q:2DY9BT01BDY0b4800LQ:2488BPQ224X
8@PQ:0077>L27@L01R487@LLiaSV7>LHiPPHaP036>HHaQS600@HaP035<DHaQS600LHaP035<DHaQC5
00@DY@<HaP0:5<DHaQS66<HDYAC55:DDaABU5<DC5:D25<D01AS65:DDaABU5<D02QBU00@DaAS65<DD
a@HHaP035<E[FV]J00A[FP40005[FP0002A[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDaAS65<D56<H01AC56<HDaAS65<D00aS600<D
aAS66<H00QS600@LiaSV7>LHiP<Li`036>HLiaSV00DLi`036>HLiaSV00@Li`0@7@LLib487@LQ21d7
8@PM1b487@LU:B488BPQ22DY8@P28BP49BT01BU:9BTYBRDY:DX01BDY00XYJRU:;F/YBRUZ:DX]JbDY
:DXU:@<YBP059DTYBRU::DXUB@03:DX01BE9:DXYBRU:9DT00RU:00/YJRU:;F/YBRe[:FX]JbU:;F/Y
BRe[00<YBP0D;F/YBRe[:DX]JbUZ;F/YJS6<;F/aS2f;<H`]Rc6<;F/aS2e[<H`]R`<aS006;H/aS36<
<J`aS3F]1C6<00Xe[C6<>Lhe[CG==JdicSF]>Lhec@<icP09=LdicSG=>LhecCW>=Lde[C6<008e[@0>
=Lde[CW>=JdicSF]>LhaS3F]<H`e[C6/=Jda[0<aS004;H/aS2f;<H`4;F/00bUZ;F/]J`04;F/02C6<
;F/aS2UZ;F/YJRe[:DX]J`02:DX01bE9:DXUBBe[:DXYJRE900<YBP0=9BTYBRDY9BTQ:2DY8BPU:B4X
9BTQ:2487@L00R4800HU:B489BTQ22DY8@P87>L02B487>LQ21SV7>LHiQcW6<HLi`026<H04AC56<HD
aAcW6<HLiaC56<HDaAS65<DHaQC56<HDaAS65<D00aS600TDaAS65<DHaQ2T5:D@Y1BU5<D00aS600XD
aAS65<DHaQBU6<HDYAS65:DHaP8DY@034:@DYABU00<DY@0]4:@DYA2T6<HDYAC55:DHaQBU5<DDYAS6
5:DHaQBU6<HDYAC55:DHaQBU5<DDYAS64:@DYA2T6<HDYAS65:DHaQC56<HDaAS65<DHaV]JKg][FV]j
JeY_N`00005[NP005V]J00<006]JJeX016]J00<HaQC56<H00QS600<DaAS66<H01AS600<DaAS66<H0
2AS621cW00<HiQcW7>L01AcW00<HiQcW7>L01QcW00HQ21d78@PM1b487@L88@P49BT01bE99BTUBBDY
9DTU:BE900@U:@LYBP8UBALYBP8]J`8YJP03:DXYJRU:00@YBP03:FXYBRUZ00D]J`04;H/]Jc6<;H/3
<H`2;H/4<H`00bf;<H`]R`08<H`00c6/<H`a[004=Jd3=Ld5>Lh02CG=>LhecCW>=LdicS6<<J`aS004
=Jd013G==JdecCF]0S6/1CF]136<00<]Rc6<<H`032e[00<]Rbe[;F/00be[00<YJRU::FX02bU:1BDY
00<Q:2DY9BT00bDY0b4800LQ:2488BPQ224X8@PQ:0077>L022487@LM1ad77>LHiQcW6>H86<H00aSV
6<HHaP0>6<H00aC56<HDa@045:D56<H021C56<HDYAC55:DDaABU5<DC5:D01aS65<DDaABU5<DDYAC5
00XDY@<Da@DHaP035<DHaQS600I[FP40005[FP0002A_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5<DHaQC56<HDaAS65<D56<H00aC56<HDa@036<H0
0aC56<HHaP046<H011cW6>HLiaSV0acW00<HiQcW6>H01AcW00<HiQcW6>H011cW010Q21cW7@LM1b48
7@LQ21d78@PM1b4X8@PU:B488BPQ20<U:@088BPU:BDY:DXU:BU:9BTYBPDU:@0<;F/YBRUZ:DX]JbU:
:FXU:BU:9BTYBRE90bU:00<UBBU::DX01BU:010UBBU::DXYBRE9;F/YBRUZ:DX]JbUZ;F/YBRe[:DX]
J`<YBP0B;F/YBRe[:DX]JbUZ;F/YJS6<;F/aS2f;<H`]Rc6<;F/aS2e[0c6<00<]Rc6<;H/00S6<00<e
[C6<<J`01C6<00Pe[C6<=Lde[CW>=JdicSF]0cW>00/ecCW>=LdicSG=>LhecCW>=Lde[C6<008e[@0>
>Lhe[CG==JdicSF]>LhaS3F]<H`e[C6/=Jda[0<aS004;H/aS2f;<H`6;F/00bUZ;F/]J`02;F/02C6<
;F/aS2UZ;F/YJRe[:DX]J`02:DX01RE9:DXUBBUZ:DX]J`8YBP049DTYBRDY9DT29BT01b4X9BTQ:2DY
8BPU:B4X00@Q20069BTQ22DY8@PU:B481AcW00`HiQcW7>LQ21cW8@PHiQcW6>HLiaS66>H26<H071C5
6<HDaAcW6<HLiaC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQ2T5:D@Y1BU0QS600dD
aAS65<DHaQC56<HDYAS65:DHaQBU6<H@Y0075:D0;A2T5:D@Y1C55:DHaQBU5<DDYAS65:DHaQBU6<HD
YAS65:DHaQBU5<DDYAS65:DDaA2T5:D@Y1S65:DHaQBU6<HDaAS65<DHaQC56<I[FV]jJeY_Nf]JJgX0
0001Kg/001I[FP03001[FV]J00A[FP<HaP035<DHaQC5008HaP036>HHaQSV00@HaP035<DHaQC500DH
aP035<DHaQS600HHaP<Li`056>HLiaSV7>LHiP037>L011d78@PM1b480acW00dQ21d77@LM1b487@LQ
224X9BTQ:2DY8BPU:@038BP01RDY8BPUBBDY9DTU:@8UB@03:DXU:BDY008U:@039DTYBRE900@U:@0;
9DTU:BE99BTUBBDY9DTYBRUZ:DXYJP0;:DX00bUZ:DXYJP03:FX7:DX8;F/00bUZ;F/YJP07;F/01Rf;
<H`]Rc6<;F/]R`D]J`06<H`]Rc6<<H`e[C6/1CF]00<ecCW>=Ld00cW>00/ecCF]=Jde[C6<<J`aS36/
;H/aS2f;00@aS0@e[@03<J`aS36/00HaS003;H/]Jbe[00<]J`03:FX]JbUZ00<]J`06:FX]JbUZ;F/Y
BRUZ32U:0bDY12U:0RE900<YBRDY9BT00RDY00DQ:2DY8BPU:B4X008U:@DQ20067@LQ21cW7@LLiad7
0QcW00<HiQcW7>L00QcW00HDaABU5<DDYAS65<D56<H011C56<HHaQS60aC50aS61aBU1aS60aC51AS6
1aBU0aS600@DaAS65<DDa@DDY@035<DDYABU008DY@034:@DaABU00@Da@076<HDYAC55:DDaABU5<D0
11BU00@DaAS65<DDa@LDY@I[FP40005[FP0002][NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC56<HDaAS65<DLiaS67>LHaQSV6<HLi`026<H0
31C56<HHaQS65<DHaQC56<HDaAcW6<HHiP8HaP0;5<DHaQSV7>LHiQcW6>HLiaSV7>LHiP027>L00b48
7@LQ20027@L04acW8@PLib487>LQ21d78@PM1bDY8BPU:B4X9BTQ:2DY8@PU:B48008U:@05:DXU:BU:
9BTYBP059BT00bU:9DTYBP059BT02BU:9BTYBRDY:DXU:Be[:DX]J`04:DX00bE9:DXYBP02:DX02Be[
:DX]JbU:;F/YBRe[:DX]J`04:DX01bE9:DXUBBe[:FX]JbUZ00<]J`07:FX]JbUZ;F/YJRe[:FX00Re[
01@aS2e[<H`]Rc6<;H/aS2e[<H`]Jbf;;F/aS2f;<H`]Rc6<<J`e[C6/13F]00TicSF]>LhecCW>=Ldi
cSF]>Lh00SF]00DaS3F]<H`aS2f;008aS003<J`aS3F]008e[@04<J`e[C6<=Jd6<H`08Bf;<H`]Jc6<
;F/]RbUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbU:;F/YBRUZ:DX]JbE9:DXUBBU:9DTYBRE9:DXU:@02
9DT01BU:9DTYBRDY:DX00RDY01@Q:2DY9BTU:B4X9BTQ:2DY8BPU:Ad78@PM1b487@LQ21d78@PLib48
0QcW00<HiQcW7>L00acW014DYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU00<HaP0F5<DH
aQBU5<DDYAS65:DDaAS67>LHaQSV5<DHaQC56<HDYAS65:DHaQC56<H25<D00aBU6<HDY@035:D46<H0
3ABU6<HDYABU4:@DYA2T6<HDYAS64:@DYA2T008DY@0C6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU6<HD
YAC55:DHaQBU6<H021BU00M[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP036<HDaAS6
008HaP065<DHaQS66>HHaQSV11S600<DaAS65<D01AS600<DaAS66<H01QS600<LiaSV7>L00QcW00<H
iQcW6>H00acW00@M1b487@LQ20<Li`8M1`038@PM1b48008Q20088BPU:B4X9BTQ:2DY8BPU:@<Q:005
9DTU:BE99BTYBP029DT49BT00bE9:DXUB@049BT02bE99BTUBBDY9DTU:BE9:DXYJRU::FX02bU:00HY
JRU::FXYJRe[:FX7:DX8;F/00bUZ;F/YJP07;F/01Rf;<H`]Rc6<;F/]R`D]J`<aS003;H/e[C6/00De
[@<ec@04>LhecCW>>Lh3=Jd01c6<<J`aS36/;H/aS2f;00@aS0@e[@03<J`aS36/00@aS003;H/aS36<
00D]J`0<:FX]Jbe[;F/YJRe[:FX]JbUZ;F/YBRUZ32U:0bDY1BU:0RE912DY00<Q:2DY8BP00bDY00<Q
:2488@P00b4800LM1b487>LM1acW7@LHiP067>L01QC55:DDaABU6<HDa@LHaP<Da@036<HDaAS6008H
aPLDY@DHaP045<DHaQS66<H25<D46<H75:D36<H35<D00aS65:DDY@035:D011C55:DDYA2T0QBU00DD
aABU5<DDaAS6008Da@065:DDaABU5<DDYAC511BU0aC500<HaQBU5:D01ABU1V]J0@000F]J00005Vmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J0`00019[FV]jJeY_NaC56<HD
aAS65:DHaQBU6<HDaAS65<DLiaS66>H26<H01aC56<HHaQcW6<HLiaC500HHaP067>LHiQcW6>HLiaSV
0QcW00<Q21cW8@P011cW00THaQSV6<HQ21cW8@PM1b487@L0124800@Q:2489BTQ20<U:@038BPU:B4X
008U:@03:DXU:BU:00<U:@0B:DXU:BE99BTYBRDY:DXU:BU:9BTYBRDY:DXU:BU:9DTYBRE912U:00<Y
JRU:;F/00RU:024UBBU::DXYJRU:<H`]Jc6<;F/aS2f;<H`YJRe[:FX]JbU:;F/YBRe[:FX]JbUZ;F/Y
JRe[:FX]JbU:;F/YJRe[:FX00Re[00DaS2e[<H`]Jbf;00D]J`08<H`]Jc6<;H/aS2f;<H`]R`8aS003
>Lhe[CW>008e[@06<J`e[CF]>Lhe[CW>0cF]00HaS2f;<H`]Rc6<;F/2<H`00cF]<H`e[@08<H`03Bf;
<H`]Rbe[:FX]JbUZ;F/YJRe[:FX]JbUZ00<]J`04:FX]JbU::FX4:DX00bE9:DXYBP02:DX00be[:DXY
BP04:DX00bDY:DXU:@039BT02R4X9BTQ:2DY8@PQ:2489BTQ22DY0R4800PM1b487>LQ21cW8@PLib48
1QcW00<HiQcW6<H00aS600@4@@Q21448@P8HaP055<DDYA2T5:D@Y0036<H00aBU6<HDY@025<D36<H0
9aBU6<HDYAcW6>HLiaC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS65:DH
aQBU6<HDaAS65<DDYA2T5:D@Y0035:D011C56<HDaAS60QBU00D@Y1BU5:DDYA2T00@DY@046<HDYAC5
5<D36<H03aC56<HDaAS65<DHaQC56<HDYAS6JeY[NV]JKg][FP020001Kg/001I[FP03001[FV]J00A[
FP036<HDaAS6008Da@035:DDaAS600XHaP036>HHaQSV00PHaP036>HLiaSV00@Li`8M1`038@PM1acW
008Li`@HaP<M1`/Q20LU:@8UB@03:DXUBBDY00HU:@@UB@049BTUBBDY:DX39DT00bU:9DTYBP0>:DX0
0be[;H/]J`02;H/00c6<;H/]J`03;F/01RUZ:DXYJRUZ;F/YJP<]J`05:FX]JbUZ;F/YJP03;F/012UZ
;H/]Jbf;1be[00H]Rbe[;H/]Jc6<;H/5<H`013G=>LhecCG=0cF]00<ecCF]=Ld013F]136<0bf;00<a
[36<<J`0236<00@]Rc6<;H/aS0h]J`/YBP03;F/YJRUZ00@YBP039BTUBBDY00LU:@@Q20<Q:0039BTQ
2248008Q20<M1`XLi`@HaP<8@P08144HaQS66<H@Y1BU4:@DY@<HaP@DY@<HaP075<DHaQC55<DHiQcW
6>H01AS600<DaAS65<D00QC500DHaQC55<DDaAS600<Da@066<HDaAS66<HDaABU0QC500<HaQC56<H0
0QBU00<@Y1BU5:D00QBU00@HaQC56<HDa@hDY@<HaP035<DHaQC500DHaP035<DHaQC500I[FP40005[
FP0002][NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][
FV]j5<DHaQC56<HDYAS65:DHaQC56<HDaASV6<HLi`026<H01QC56<HHaQcW6<HLi`8HaP035<DHaQS6
008HaP067>LHiQcW6>HLiaSV0QcW00<Q21cW8@P00acW00XHiQS67>LHaR487>LQ21d78@PM1`@Q2004
9BTQ224X8@P39BT00b4X9BTQ:0029BT00bU:9BTYBP039BT04RE99BTYBRDY:DXU:BU:9BTYBRDY:DXU
:BU:9BTYBRE9:DXUB@@YBP04;F/YBRUZ9DT4:DX07Re[:DXaS2e[<H`]Jc6<;H/aS2UZ;F/YJRe[:DX]
JbU:;F/YJRe[:FX]JbUZ;F/YJRe[:DX]JbUZ;F/YJP8]J`05<H`]Jbf;;F/aS005;F/01S6<;F/aS2f;
<H`]R`<aS004;H/icSF]>Lh2=Jd01S6/=Jde[CW>=JdicP<e[@06<H`]Rc6<;H/aS2e[0S6<00<e[C6<
=Jd0236<00l]Rc6<;H/]JbUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX00Re[00<YJRU:;F/01RU:00@UBBU:
:DX]J`HYBP039BTYBRDY00<U:@0:8BPU:B4X8BPQ22DY8@PU:B489BT28@P021d78@PLib487>LQ21cW
7@L67>L00aSV7>LHaP036<H01@A12484@@Q25<D00QS600XDYA2T5:D@Y1S65<DHaQBU5<DDY@<HaP0Y
5<DHaQBU6<HDYAcW6>HLiaC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS6
5:DHaQBU6<HDaAS65<DDYA2T5:D@Y0035:D01AC56<HDaAS64:@00aBU00D@Y1BU5:DDYA2T008DY@03
5<DDYAS6008HaP0C5<DHaQC56<HDaAS65<DHaQC56<HDYAS6JeY_Nf]JJgY[FVmk00000F]j000FJeX0
0`00JeY[FP04JeX00aS65<DHaP025<D00aBU5<DHaP0:6<H00aSV6<HHiP086<H00aSV7>LHiP047>L0
12487@LM1ad70acW11S600<M1b487@L02b481bDY00@YBRE99DTUB@LU:@079DTYBRE99DTU:BE99BT0
0RE900@YBRE9:DXUB@lYBP03;F/]Rbe[008]R`03<H`]Rbe[00<]J`08:FXYBRUZ:FX]JbUZ;F/YJP<]
J`<YJP07;F/YJRe[;F/]Rbe[;H/01be[00@]Rbe[;H/]J`<aS004;H/aS36<<H`3=Ld01cW>=Jde[CF]
=Lde[CG=00@e[@@aS006;H/aS2f;<J`aS36/236<00@]Rc6<;H/aS0P]J`03:FX]Jbe[00<]J`/YBP8Y
JP03;F/YBRU:008YBP039BTUBBDY00LU:@@Q20048BPU:B4X8BP38@P27@L00b487@LLi`097>L46<H0
10Q21448@PQ20aS600@@Y1BU4:@DY@<HaP@DY@<HaP<Da@046<HHiQcW6>H56<H011C56<HDaAS60aC5
00HHaQC55<DDaAS65<D36<H25<D01aBU5<DDaAS65<DHaQ2T00HDY@046<HDaAS65<D>5:D36<H00aC5
6<HDa@056<H35<D6JeX10001JeX0000XKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeX006]JKg][FV]jJeY_NaBU6<HDYAS65:DHaQBU6<HDaAS65<D36<H00aC56<HDa@026<H0
1AcW6<HLiaS67>L01AS600XLiaS67>LHiQcW6>HLiaSV7>LHiPPLi`068@PM1b487@LQ21d70R48010U
:B489BTQ:2DY8BPU:B489BTQ:2DY8BPU:B4X9BTQ:08U:@039DTU:BDY00<U:@0O:DXU:BU:9BTYBRDY
9DTU:BU:9BTYBRE9:DXUBBU:9BTYBRDY:DXU:BU:9DTYBRE9:DXUBBU:9DTaS2e[<H`00Re[010YJRe[
:FX]JbUZ;F/YBRe[:DXYJRU:;F/YBRe[:FX]J`8YJP05:DX]Jbe[;F/YJP03;F/01BUZ<H`]Rc6<;H/0
12e[00<aS2e[<H`01C6<010e[C6<=Jda[3F]<J`e[C6<=Jda[3F]<J`e[C6<<J`aS0@]J`06<H`]Rc6<
;H/aS2f;0S6<00De[C6<=JdaS3F]008aS00@;F/aS2e[;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbU::FX2
:DX062DY:DXU:BU:9BTYBRDY:DXU:BU:9BTYBRDY:DXU:BU:9BTYBRDY:DXQ:2DY8BPU:@@Q200:7@LQ
21d78@PLib487>LU:B488BP48@P05AcW8@PLiacW6>HLiaSV7>LHiQcW6<HLiaS67>LHaQcW6<HLiaS6
6>HHaP035:D0812T6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaQBU6<HDYAS65<DHaQC5
6<HDYAC55:DHaQBU6<H55:D061S65:DHaQ2T5:D@Y1S65:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DH
aQ2T5:D@Y0@DY@036<HDYAS600DDY@066<HDYAS65<DHaQC511BU00XDaABU6<I[FV]jJeY_Nf]JJgX0
005_N`005V]J00<006]JJeX016]J00@DaABU5<DDY@8Da@THaP065<DHaQC56>HHaQSV21S600/HiQcW
6>HLiaSV7>LHiQcW6>HLiaSV00LLi`077@LQ21d78@PM1b487@L00b4812DY00<Q:2DY8BP01bDY00<Q
:2DY9BT01BDY0bE912DY00TUBBDY9DTUBBU:9DTYBRE9:DX00bE900HYBRE9:DXUBBU:9DT3:DX3;H/0
0c6<;F/]J`05;F/00bUZ:DXYJP04:DX2;F/00bUZ;F/YBP02:DX7;F/012f;<H`aS36<0be[00@]Rbe[
;H/]J`@aS00><J`aS36/<J`e[C6/=Jda[3F]<J`e[C6/=Jda[0<aS0@]J`@aS006;H/aS2f;<J`aS36/
136<00@]Rc6<;H/]R`8]J`05:FX]JbUZ;F/YJP03;F/4:DX00bDY9DTU:@059DT012DY9DTU:BU:0RE9
00<U:BE99BT00RE900<YBRE99BT00bDY1b480ad7224800<M1acW7@L00QcW00@HiQcW7>LLi`8HiP03
6<HHiQS6008HiP037>LHiQS6008HaP@DY@PHaP045<DHaQC56<H35<D01QS65<DDaAC56<HDa@@HaP@D
Y@035<DDYAC500<DY@066<HDaAC55<DDYA2T0QBU00XDaABU5<DDaAS65<DHaQC56<HDa@<HaP035<DD
YABU00HDY@035<DDYABU00HDY@@HaPLDY@I[FP40005[FP0002Y[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5:DHaQBU6<HDYAS65:DHaQC56<HDaAS6
5<D36<H021C56<HHaQcW6<HLiaS66>H56<H02QcW6<HLiaSV7>LHiQcW6>HLiaSV21cW00HQ21d78@PM
1b487@L28@P0424X8@PU:B4X9BTQ:2DY8@PU:B4X9BTQ:2DY8BPU:B4X0RDY00<YBRDY9BT00bDY01lY
BRDY:DXU:BE99BTYBRDY:DXU:BU:9DTYBRE9:DXU:BU:9BTYBRDY:DXUBBU:9DTYBRE9:DXUBC6<;F/a
S002;F/04bUZ;F/YJRe[:FX]JbU:;F/YBRe[:DXYJRU:;F/YJRe[:FX]JbU:008YJP<]J`09:FX]Jbe[
<H`]Rc6<;H/]JbUZ008]J`03<H`]Jc6<00DaS00@=JdaS3F]<J`e[C6/=JdaS3F]<J`e[C6/=JdaS3F]
<H`4;F/01S6<;H/aS2f;<H`]R`8aS005=JdaS3F]<H`a[002<H`032e[<H`]Jbe[:FX]JbUZ;F/YJRe[
:FX]J`8YJP0L:DX]JbU::DXU:BU:9BTYBRDY:DXU:BU:9BTYBRDY:DXU:BU:9BTYBRDY:DXU:BU:8BPU
:B4X9BT48@P02Qd78@PM1b487>LQ21cW8BPQ22DY124801LLib487>LLiaSV7>LHiQcW6>HLiaS67>LH
aQcW6<HLiaS67>LHaQcW6<HDYA2T008DY@0O6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DH
aQBU6<HDYAS65<DHaQC56<HDYAS65:DHaQBU6<H01ABU01PHaQBU6<H@Y1BU4:@HaQBU6<HDYAS65<DH
aQC56<HDaAS65<DHaQC56<H@Y1BU4:@45:D00aS65:DHaP055:D01QC55:DHaQC56<HDa@@DY@0:6<HD
YAC5JeY_Nf]JJgY[FVmk0001JgX001I[FP03001[FV]J00A[FPDHaP055<DHaQS66<HDa@056<H00aC5
6<HDa@076<H47>L01aS66>HHaQSV6<HHiQS600LLi`077@LQ21d77@LLiad77>L01b4800@M1b488@PQ
20<Q:0@Q20069BTQ:24X8BPU:B4X22DY00<YBRE9:DX00RE900<U:BE9:DX02RU:00D]JbUZ:FXYJRe[
00<YJP0:;F/YJRUZ:FX]JbUZ;F/YJRe[:FX>:DX012e[:FX]JbUZ0be[00<YJRe[:FX012e[00@]Rbe[
;H/]J`/aS003;H/aS2f;00DaS08]R`D]J`03;H/]Jc6<00<aS08a[003=Jda[3F]008e[@@aS0<]R`07
;F/YJRUZ:FX]JbUZ;F/01RU:00DUBBU:9DTYBRE900<U:@038BPU:BDY00TU:@<Q:0<Q20077@LQ21d7
7@LLiad77>L00Qd700HQ21d77>LHiQcW6>H37>L36>H00acW6<HHaP026<H35<D01acW6>HLiaSV5<DD
YAC500@HaP<DY@@HaP<DY@045<DHaQS66<H35:D00aS65<DHaP025<D01QBU5<DDYAC55:DDa@LDY@04
5<DDYAC55:D36<H02QC56<HHaQS65<DHaQC56<HDYAC51ABU00<DaABU5<D01aBU00<DaABU5<D01ABU
00@DaABU5<DDa@<HaP035:DDaABU00I[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk008HaP0?5<DHaQC56<HDaAS65<DHaQC56<HD
aAS65<DHaQBU00DHaP0=5<DHaQS67>LHiQcW6>HLiaS67>LHaQcW6<H00acW00lHiQcW7>LM1acW8@PL
ib487>LQ21cW8@PM1b487@L00R4801XU:B488@PM1b487@LU:B489BTQ224X8@PU:B489BTQ22DY8BPU
:B4X:DXU:BE99BTYBRDY0RE900dYBRE9:DXU:BU:9BTYBRE9:DXUBBUZ:DX]J`05:DX04be[:DX]JbU:
;F/YBRe[:DX]JbU:;F/YBRe[:FX]JbUZ;F/YBRe[00HYBP0A9DTYBRU::DXUBBU::FX]JbUZ;F/YJRe[
:FX]JbUZ;F/YJP04;F/00c6<;F/aS005<H`023F]<H`aS2f;<H`]Rc6<;H/3<H`012f;<H`]Jc6<0Re[
00DYJRe[:FXaS2e[008aS00H=JdaS36/<H`e[C6<=Jda[3F]<J`aS2f;<H`]Rc6<;F/aS2U:;F/YBRe[
:FX]JbUZ1BU:00/UBBU:9DTYBRDY:DXQ:2DY8BPU:B4X00HU:@079DTU:BU:8@PU:B489BT00R48014M
1b487>LQ21cW8@PLib487>LQ21d78@PHiQcW6>HLiaSV00<Li`0A6<HLiaS66<HDaAS65:DHaQBU6<HH
iQcW6>HLiaBU6<HDY@036<H021C55:DDYABU6<HLiaS66>H35:D04AS65<DHaQC55:D@Y1BU5<DHaQC5
6<HDYAS65:DHaQBU6<H01QBU01X@Y1BU5:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS6
5:DDaABU6<HDYAS61ABU00LHaQBU5<DDYAS65:DHaP035:D04QS65:DHaQBU6<HDaAS65<DHaQBU6<I[
FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J1AS600<DaAS65<D01aS60aC51aS611cW00LH
aQSV6<HHiQS66>HHaP077>L37@L012487>LM1acW2B4800DM1b488BPU:B4X00@Q208Q:0049BTQ:2DY
8BP89BT00bU:9DTYBP029DT00bDY9DTYBP0::DX2:FX01Be[:FXYJRUZ;F/00bUZ00P]JbUZ:FXYJRe[
:FX]JbUZ3RU:00@]JbUZ;F/YJP<]J`03:FX]JbUZ00@]J`04;H/]Jbf;;F/;<H`00bf;<H`]R`04<H`2
;H/00c6<;F/]J`03;F/00bf;;F/aS003<H`013F]<J`a[36/0cF]136<00<]Rc6<;H/00RUZ00<]JbUZ
;F/00Re[12U:00@UBBU::DXYBP<UB@039BTQ:2DY00/U:@8Q:0039BTQ:248008Q20<M1`048@PLiad7
7>L27@L01R487@LLiaSV7>LHiP<Li`046>HLiaSV6>H36<H25<D02AS65<DLiaSV7>LHiQC55:DDa@04
6<H35:D46<H35:D26<H00aC56<HDY@025:D00aS65<DHaP025<D01QBU5<DDYAC55:DDa@LDY@045<DD
YAC55:D56<H021C56<HDaAS65<DHaQBU5<D55:D00aC55:DDa@075:D00aC55:DDa@055:D00aC55:DD
a@026<H01AC56<HDYAC55:D01V]J0@000F]J00007V]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDa@<HaP0=5<DHaQC56<HDaAS65<DHaQC56<HDaAS6
5:D01aS600dDaAcW6>HLiaSV7>LHaQcW6<HLiaS67>LHiP047>L032487>LQ21cW8@PLib487>LQ21d7
8@PM1`8Q200Y8BPQ22487@LQ21d79BTQ22DY8@PU:B488BPQ22DY8@PU:B4X9BTQ:2E99BTYBRDY9DTU
:BU:9DTYBRE9:DXU:BU:9BTYBRE9:DXUBBe[:DXYJP05:DX04be[:DX]JbU:;F/YBRe[:DX]JbU:;F/Y
BRe[:FX]JbUZ;F/YBRUZ00PYBP0?9DTYBRU::DXYJRe[:FX]JbUZ;F/YJRe[:FX]JbUZ00@]J`03<H`]
Jc6<00DaS00:<J`aS36<;H/aS2f;<H`]Rc6<;H/3<H`02Be[<H`]Jbe[:FX]JbUZ<H`]J`02<H`0636/
<H`e[C6<=JdaS3F]<J`e[C6/<H`]Rc6<;H/aS2e[<H`YBRe[:DX]JbUZ;F/YJPDYBP099DTYBRE9:DXU
:BU:8BPU:B4X00<U:@038BPU:BDY008U:@08:DXU:BE98@PU:B489BTM1`<Q200=7>LQ21cW8@PLib48
7>LQ21d78@PHiQcW6>H00acW00@HiQcW6<HLi`8HaP0?5<DHaQBU6<HDYAS66>HLiaSV7>LDYAS65:DH
aQC5008HaP<DY@046<HHiQS67>L35:D04AS65<DHaQC55:D@Y1BU5<DHaQC56<HDYAS65:DHaQBU6<H0
1QBU01X@Y1BU5:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU5<DDYAS61ABU
00LDaABU6<HDYAS65:DHaP035:D04QS65:DHaQBU6<HDaAS65<DHaQBU6<I[FVmkJeY[NV]JKg/0005[
NP005V]J00<006]JJeX016]J00LHaQC56<HDaASV6<HHiP026<H00aC56<HHaP066<H00aSV6<HHiP08
6<H01ASV7>LLiacW6>H051cW00<M1acW7@L02b4800<U:B4X9BT022DY2RU:12DY00@UBBU::DXYBP<U
B@DYBP039DTYBRU:00<YBP<]J`@YBP07;F/YJRe[:FX]JbUZ;F/012U:0bUZ00@]JbUZ;F/YJP<YBP03
:FX]JbUZ00<]J`03:FX]Jbe[008]J`04<H`]Rc6<;H/3<H`4;F/012UZ;F/]Jc6<0bf;0be[00@]Rc6<
;H/]R`<]J`<aS003;H/aS36<00DaS006;F/]Rbe[;H/]Jbf;0Re[0bUZ00<]JbUZ;F/012DY12U:2RDY
1b4800<Q:2488BP01R4800@M1b487@LQ208M1`LLi`@HaP036>HHaQSV00@HaP035<DHaQC500/HaP03
5<DHaQC500<HaP045<DHaQS66<H35<D56<H00aC56<HHaP045:D01AS65<DDaAC56<H00QC500<DYAC5
5:D02aBU0aC500@HaQBU5<DDY@@HaP<DY@8HaP035<DHaQBU008DY@046<HDaAS65<D35:D35<D021S6
5<DDaAC56<HDYAC55:D6JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_NaC56<HDa@8HaP067>LHaQS65<DHaQC50aS600<DaAS65<D0
0QS600@LiaS67>LDa@HHaP0:7>LHiQcW6>HLiaSV7>LHiQcW6>H37>L04aSV7>LLiacW6>HQ21cW7@LL
ib487>LM1acW8@PLib487@LQ21d700DQ20057@LQ224X9BTQ:0039BT01b4X9BTQ:2DY8BPYBRE900DY
BP039DTYBRE900@U:@0>:DXUBBU:9DTYBRDY:DXUBBU:9DTYBRE9:DXUB@@YBP04;F/YJRe[9DT3:DX0
1RUZ;F/YJRe[:FX]J`8YJP0C:DX]JbU:;F/YBRe[:FX]JbUZ;F/YBRUZ:DX]JbUZ;F/YJRe[:FX00Re[
010aS2e[;H/]Rc6<;H/aS2f;<H`]Rbe[:FX]JbUZ;F/YJP8]J`0<<H`]Jc6<:FX]JbUZ<H`]Jc6<;F/a
S2e[0Rf;00<aS2f;<H`0136<01D]Rc6<;H/aS2e[<H`]Jc6<;F/aS2U:;F/YBRe[:FX]JbUZ:DXU:BE9
9BT00bU:00DUBBDY8BPU:B4X00<U:@068BPU:B4X8BPQ22DY0R4800HM1b488@PU:B489BT48@P01Qd7
8@PM1b487>LQ20@Li`046>HLiaSV7>L36<H00acW6<HLi`026<H01AC56<HDaAS65<D01AS600HDaAS6
5<DLiaS66>H26<H051BU6<HDaAS65<DHaQC56<HDaAS65:DHaQBU6<HDaAS65<DHaQC56<H55:D02AS6
5:DHaQBU6<HDYAS65:DHaP045:D01A2T5:DDYABU4:@00QBU00/HaQBU6<HDYAS65:DHaQC56<HDaAS6
008DY@054:@HaQC56<HDa@035:D06AC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaV]J
JgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX016]J00PHaQC56<HDaASV6<HHiQC52QS600<HiQS6
6>H021S60QcW00DHiQcW7>LLiaSV018Li`037@LLiad700/Q20<U:@038BPU:BDY00DU:@XYBP@U:@8Y
BP059DTYBRE9:DXUB@05:DX00bE9:DXYBP03:DX3;F/4:DX01be[:FX]JbUZ;F/YJRe[00@YBP07:FX]
JbUZ;F/YJRe[:FX00bU:00DYJRe[:FX]JbUZ00H]J`07<H`]Rc6<;H/aS2f;<H`01Re[00DYJRf;;H/a
S2f;00<]J`<]R`06<H`]Jbe[;F/aS2f;1c6<00T]Rc6<;F/]Rbe[;H/]Jbf;;F/00RUZ00D]JbUZ;F/Y
JRe[00@U:@@YBPXU:@LQ20038BPQ224X00@Q20047@LQ22488@P37@L00b487>LLi`057>L46<H00aSV
6<HHiP046<H00aC56<HDa@0;6<H35<D00aS65<DHaP046<H011C56<HDaAC51QS600<DaABU5:D00QBU
0QC500LHaQC55<DDaAS65:DDa@0<5:D01aC56<HDaAC55:DDaABU00@HaP<DY@045<DHaQS66<H35:D0
11S65<DHaQC50aBU00<DaAS65<D00QC500HHaQC55<DDYAC55:D6JeX10001JeX0000VJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQC56<HDaAS65<DH
aQC56<HDa@<HaP095<DHaQC56<HDaAS65<DHaQC5008HaP0`7>LHaQcW6<HLiaS68@PLib486>HLiaSV
7>LHiQcW6>HLiad78@PM1b487>LQ21cW8@PM1b487@LQ21d78@PM1b487@LQ21d78@PM1bDY8@PU:B48
9BTQ22DY8@PU:B480RDY02TYBRDY9DTU:BU:9BTYJRU:;F/YBRUZ:DX]JbUZ;F/YJRU:9DTYBRE9:DXU
BBU:9DTYBRDY:DXUBBU:9DTYBRE9:DXUBBe[:DX]JbU:;F/YBRe[008YBP049DTYBRU::FX5:DX00be[
:DX]J`02:DX02BE9:DXYJRe[:FX]JbUZ;F/YJP05;F/02RUZ;F/]Jc6<;F/aS2e[<H`]Jc6<0be[00<a
S2e[<H`00Re[00/YJRe[;F/aS2e[<H`]Jc6<;F/aS2f;00<aS00@;F/aS2e[<H`]Jc6<;H/aS2f;<H`]
Rc6<;H/aS2e[;H/4;F/4:DX02BE9:DXYBRU:9DTYBRE99BTQ:0069BT00bE99BTYBP029BT00b4X9BTU
:@039BT032489BTQ22DY8@PQ:1cW8@PLiad77>LQ20HLi`056>HLiaSV7>LHiP037>L011SV8@PLiad7
0QcW01/HaQSV5<DHaQC56<HHiQcW6>HLiaSV7>LHiQS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65<D0
0aS601HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAC55:DHaPDDY@056<HDYAS6
5:DDa@055:D00aS65:DHaP045:D08aC56<HDaAS65:DHaQBU6<HDaAS65:DDaABU6<HDYAS65:DHaQBU
6<HDYAS65<DHaQC56<HDYAC5JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeX00aS65<DH
aP0:6<H02AC56<HHaQS65<DHaQC56<HDa@026>H01AcW6>HHiQSV7>L00ad700<LiaSV7>L00QcW00PH
iQcW7>LQ21d78@PM1b480Qd70R4800DM1b488@PQ21d700<Q20037@LQ21d7008Q200:8BPQ224X8@PQ
:2488BPQ224X8@P79BT7:DX3;F/00bE9:DXUB@05:DX39DT02BU:9DTYBRE9:DXUBBU::FX]J`02:FX0
0bU::FXYBP0<:DX00bUZ:DXYJP03:DX5;F/00bUZ;F/]J`03;F/00bUZ;F/YJP02;H/01C6<;H/]Rbe[
;H/012e[0bf;00DaS2UZ;F/]Jc6<00<]R`03<H`]Rbf;00@aS003;H/aS2f;008]R`09<H`]Rc6<;H/a
S2f;<H`]Rc6<00L]J`TYBP039DTYBRDY014U:@<Q:0@Q21TLi`@HaP035<DHaQC500LLi`PHaP055<DH
aQC56<HDa@056<H35<D021S65<DHaQC56<HDaAS65<D45:D00aC55:DDa@075:D25<D00aS65:DDY@06
5:D35<D45:D36<H00aC56<HDa@025<D00aS65<DDY@035:D00aC55:DDa@035<D56<H35:D6JeX10001
JeX0000TKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]j
JeY_NaC56<HDaAS65<DHaQC50aS600/DaAS65<DHaQC56<HDaAS65<DHaQC5008HaP0`7>LHaQcW6<HL
iaS68@PLib486>HLiaSV7>LHiQcW6>HLiad78@PM1b487>LQ21cW8@PM1b487@LQ21d78@PM1b487@LQ
21d78@PM1bDY8@PU:B489BTQ22DY8@PU:B480RDY00`UBBDY:DXU:BE99BT]JbU::FXYBRe[:DX2:FX0
72e[:FXYBRE9:DXUBBU:9DTYBRE9:DXU:BU:9DTYBRE9:DXUBBU:9DT]JbU:;F/YBRe[:DX]JbE912U:
00<]JbU::DX00bU:00<]JbU:;F/00RU:00TUBBU::FX]JbUZ;F/YJRe[:FX01Be[00XYJRe[;F/aS2e[
<H`]Jc6<;F/aS0<]J`03<H`]Jc6<008]J`09:FX]Jbe[<H`]Jc6<;F/aS2e[00<aS00>;H/aS2e[<H`]
Jc6<;F/aS2f;<H`]Rc6<;H/aS08]R`03;F/aS2e[00<]J`HYBP059DTYBRE9:DXUB@039BT00b4X9BTU
:@029BT012U:9BTUBB4X1bDY00`Q22DY8@PQ:2489BTLiad77>LQ21cW7@L87>L01ASV7>LHiQcW6>H0
0QcW028M1acW8@PLiaSV6<HLiaC56<HDaAS66>HLiaSV7>LHiQcW6>HHaQC56<HDaAS65<DHaQC56<HD
aAS65<DHaQC56<HDa@<HaP0D5:DHaQC56<HDaAS65<DHaQC56<HDYAC55:DHaQBU6<HDYAS65:DDa@DD
Y@056<HDYAC55:DHaP055:D00aS65:DHaP045:D061C56<HDaAS65:DHaQBU6<HDaAS65:DHaQBU5<DD
YAS65:DHaQBU6<HDYAS65<DHaP8Da@095:DHaV]JJgY[FVmkJeY[NP00005_N`005V]J00<006]JJeX0
16]J0aS600<DaAS66<H01AS600DDaAS66<HHaQC500<HaP045<DHaQC57>L36>H01QcW6>HHiQd78@PM
1`<Li`086>HLiacW7>LHiR487@LQ20<M1`038@PM1b48008Q20097@LQ22488@PM1b487@LQ21d7008Q
200:8BPQ224X8@PQ:2488BPQ224X8@P79BT7:DX01Re[:FX]JbE9:DXUB@DYBP0:9DTYBRE9:DXUBBU:
9DTYBRE9:DX3:FX00be[:DXYJP0=:DX00bUZ:DXYJP03:DX;;F/012UZ;F/YJS6<12f;00<]Jbf;;F/0
0be[00@]Rc6<;H/]R`8]J`04:FX]Rbf;<H`3;H/5<H`3;H/02c6<;H/]Rbf;<H`]Rc6<;H/aS2f;<H`0
1be[1bU:00@UBBU::DXYBQ8U:@038BPU:B4X00@Q21TLi`@HaP035<DHaQC500LLi`PHaP055<DHaQC5
6<HDa@056<H02aC56<HDaAS65<DHaQC56<HDaAS65<D011BU00<DaABU5<D01aBU00<HaQC55<D021BU
00<DaAS65<D011BU0aS60aC500@HaQC56<HDa@@DY@055<DDYAC55<DHaP025<D46<H35:D6JeX10001
JeX0000FJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX300001f]JKg][
FV]j5<DHaQC5008HaP067>LHaQS65<DHaQC51AS603<DaAS65:DHaQBU6<HHiQcW6>HLiaS67>LHaR48
7>LM1acW8@PLiad77>LQ21cW7@LLib487>LM1acW8@PLib487@LQ21d78@PM1b487>LQ21cW9BTQ224X
8@PU:B4X9BTQ22DY8@P00RDY00XYBRDY9DTU:BU:9BTYBRE9:DXUB@<YBP03:FX]JbUZ00<]J`08:FX]
JbUZ;F/YJRe[:DXYJPPYBP069DTYBRE9;F/YJRe[0RUZ00TYBRe[:DXYJRU:;F/YJRe[:FX012U:014]
JbU:;F/YBRe[:DX]JbU:;F/YBRe[:FX]JbUZ;H/]Jc6<008]J`06:FX]Jbf;<H`]Rc6<1Be[00<aS2e[
;F/012e[00`YJRe[;H/aS2f;<H`]Jc6<;F/aS2e[<H`2;F/01RUZ;F/]Rc6<;H/aS0@]J`03:FX]JbUZ
00D]J`0k:FX]JbU:;F/YBRe[9BTYBRDY:DXUBBU:9DTU:B4X9BTQ:2E99BTYBR4X9BTQ:2DY8BPU:B4X
9BTQ:2DY8BPU:B489BTM1b487@LQ21d78@PM1acW6>HLiaSV7@LLib487>LHiQS67>LHaQcW6<HLiaSV
7>LHiP046<H01QcW6<HHiQC56<HDa@8HaP067>LHaQSV5<DHaQC511BU00hHaQBU6<HDaAS65<DHaQBU
6<HDYAS65:DHaQBU0aS602<DaAS66<HHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DH
aQBU6<HDYAS65:DHaQBU6<HDaAS65<DDYA2T008DY@0@6<HDYAS65<DHaQC56<HDYAS65:DHaQBU6<H@
Y1BU4:@25:D01QC55:DHaQC56<HDa@@DY@086<HDYAS6JeY_Nf]JJgY[FP80005[NP005V]J00<006]J
JeX016]J00@HaQC56<HDa@<HaP035<DHaQC500PHaP8Da@056<HDaAcW6>HLi`036>H<7>L00b487@LM
1`047>L48@P00ad78@PM1`027@L58@P0124X9BTU:BDY0b4X12DY00HYBRE99DTUBBU:9DT4:DX;;F/>
:DX012UZ;F/YJRe[1bU:0be[12U:0bUZ00D]JbU::FXYBRe[00<YJP03;F/YJRe[00L]J`@aS0@]J`03
;H/]Jbf;00@]J`05:FX]Jbe[<H`]R`03<H`3;H/00c6<;H/]R`03;F/0136<;H/aS2f;2be[00@YJRe[
;F/]J`<YJP<U:@049DTYBRE9:DX29BT00b4X9BTU:@079BT01R4X9BTQ:2DY8BPU:@<Q:0038@PM1b48
00@Q20PLi`<HaP076>HHaQSV6<HLiaSV7>L021S600<DaAS66<H01aS61ABU00DDaABU6<HDaAS600<D
a@056<HDYAC55:DDa@086<H01AC56<HDaAS65<D00aS600<DaAS66<H00QS600@DaABU5<DDY@8Da@05
6<HDYAC55:DDa@046<H35:D02QC55:DDaABU6<HDaAS65:DDaABU0QC500<HaQC55:D01QBU00@DaAS6
6<HHaPDDY@035<DDYF]J00E[FP40005[FP00021_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FP00JeY_Nf]JJgY[FVmk5<DHaQC50QS600HHiQS66<HDaAS65<D76<H0<ABU6<HD
YAS66>HLiaSV7>LHaQcW6<HM1acW8@PLiad77>LQ21cW7@LLib487>LQ21cW8@PLiad77>LQ21d78@PM
1b487@LQ21cW8@PLib4X8@PU:B489BTQ:2DY8@PU:B48008U:@0:9DTU:BU:9BTYBRDY:DXUBBU:9DT3
:DX01BUZ;F/YJRe[:FX00be[00HYJRe[:FXYJRU:;F/8:DX03BE9:DXUBBe[:FX]JbUZ;F/YBRUZ:DX]
JbU:008YJP03;F/YJRU:00<YBP0A;F/YBRe[:DX]JbU:;F/YBRe[:DX]JbUZ;F/YJS6<;F/]R`02;F/0
1RUZ;F/]Rc6<;H/aS0D]J`03<H`]Jbe[00@]J`0<:FX]Jbf;<H`]Rc6<;F/aS2e[<H`]Jc6<12e[00@]
Rc6<;H/aS0@]J`03:FX]JbUZ00D]J`0`:FX]JbU:;F/YBRe[9BTUBBDY:DXUBBU:9DTU:B4X9BTQ:2U:
9BTUBB4X9BTQ:2DY8BPU:B4X9BTQ:2DY8BPU:B489BTM1b487@LQ21d78@PM1acW6>HLiaSV8@PLiad7
0QcW00THaQSV6<HLiaS67>LHiQcW6>H011S600HHiQS67>LDaAS65<D26<H01QSV6<HLiaC56<HDa@@D
Y@0>6<HDYAS65<DHaQC56<HDYAS65:DHaQBU6<HDY@DHaP0=5<DHaQC56<HDaAS65<DHaQC56<HDaAS6
5<D00aS6014DYAS65:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DDYA2T008DY@0@6<HDYAS65<DHaQC5
6<HDYAS65:DHaQBU6<H@Y1BU4:@25:D01QS65:DHaQC56<HDa@@DY@0:6<HDYAS6JeY[NV]JKg][FV]j
0001Kg/001I[FP03001[FV]J00A[FP@HaP8DaA0HaQ8Li`8M1`088@PM1b487@LQ21d78@PM1`@Q2006
9BTQ:2DY7@LQ21d7124800@Q:2488BPQ208Q:0049BTQ:2E99DT8:DX012e[:FXYJRUZ1Be[00<YJRe[
;F/00be[00<YJRe[:FX012e[0bU:0bUZ00<]JbUZ;F/00Re[00PYJRe[;F/]JbUZ;F/YJRe[0bUZ00<]
JbUZ:FX00bU:00H]JbUZ;F/YJRe[:FX5;F/3;H/9;F/012UZ;F/YJRe[0RUZ0bU:12e[00<aS2f;<H`0
0bf;00<aS2e[;F/01Be[00<]Rbe[;H/00Re[00<YJRe[:DX012U:00HYJRU::FXYBRUZ:DX79BT01B4X
9BTU:BDY8BP01bDY00@Q:2DY8BPU:@8Q:08Q20037@LQ2248008Q20<Li`<M1`038@PHiQcW008Li`<M
1`DLi`036>HLiaS600HHaP8HiP037>LHiQS600<HaP<DY@/HaP<DY@8HaP035<DHaQS600THaP<Da@03
6<HDYABU008DY@<HaP@DY@055<DDYAC55<DHaP025<D00aBU5<DDY@025<D00aS65<DDY@0:5:D00aC5
6<HDa@0B5:D6JeX10001JeX0000WJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX006]JJgY[FVmkJeY[NQC56<HDaAS65:DHaQBU6>HHaQcW008HaP055<DHaQC56<HDa@026<H0
0aSV6<HLi`027>L021SV7>LHiQcW6>HQ21cW7@L97>L022487>LQ21d78@PM1b487@L28@P0524X8@PU
:B4X9BTQ:2487@LQ21d79BTQ224X8@PU:B489BTQ22DY8@P29BT012U:9BTYBRE91RU:02d]JbU:;F/Y
JRe[:FX]JbUZ;F/YJRe[:FX]JbUZ;F/YBRe[:FX]JbUZ;F/YBRe[:DX]JbU:;F/YBRe[:FX]JbUZ;F/Y
JRe[:FX]JbUZ;F/YBRe[:DX]JbU:;F/012U:00TYJRe[:FX]JbUZ;F/YJRe[:FX00Re[00HaS2e[<H`Y
JRe[:FX3;F/022UZ;F/YJRe[:FX]JbU:;F/4:DX00be[<H`]J`02;H/01C6<;H/aS2e[<H`02Be[00Ha
S2e[<H`YJRe[:FX3:DX022E9;F/YBRe[:DX]JbU:;F/49BT0524X9BTQ:2DY8BPU:B4X9BTQ:2DY8BPU
:B4X9BTQ:2DY8BPU:B489BT28@P031d78@PM1b487@LQ21cW8@PLib487>LQ208Li`066>HLiaSV8@PL
ib480QcW00PHiQcW6>HLiaSV7>LHaQSV1AS600LLiaS67>LHaQcW6<HHiP035:D46<H08aSV6<HLiaS6
6>HHaQcW5:DDaABU6<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU5<DDYAS65:DHaQC5
6<H01ABU01@HaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQBU6<HDYAC54:@DYA2T11BU00LHaQC56<HD
YAC55:DHaP025:D01A2T5:DDYABU4:@011BU00XHaQBU5<E[FVmkJeY[NV]JKg/0005[NP005V]J00<0
06]JJeX016]J1AS60QC53aS64QcW00XQ21d77@LM1b487@LQ21d78@PM1`@Q20069BTQ:2DY7@LQ21d7
124800DQ:2488BPQ22DY00<Q:003:DXUBBE900LYBP8YJP03;F/YJRe[008]J`03:FX]Jbe[00D]J`<Y
JP@]J`<YBP04:FX]JbUZ:FX2;F/022UZ;F/]Jbe[:FX]JbUZ;F/3:FX01Be[:FXYJRUZ;F/00bU:00H]
JbUZ;F/YJRe[:FX5;F/00bf;<H`]R`09;F/00bUZ;F/YJP02:FX012e[:DXYBRU:12e[00DaS2f;<H`]
Rc6<008]R`L]J`03;H/]Jbf;00@]J`DYBP06:FXYBRUZ:DXYJRU:2BDY00DQ:2DY9BTU:B4X00LU:@<Q
:0039BTM1b4800DQ20<Li`047@LQ21d77@L27>L01ASV7>LM1b487@L01acW1aS600@LiaSV6>HHiP@H
aP<DY@/HaP<DY@035<DHaQS600THaP045<DHaQS66<H25<D45:D36<H45:D00aC55:DDa@035<D011S6
5:DDaABU0QC500<HaQC55:D011BU00<@Y1BU5:D00aBU00<DaAS65<D04QBU1V]J0@000F]J00009fmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/DaAS6
5<DHaQBU6<HDYAcW6<HHiP026<H00aC56<HDa@046<H011cW6<HHiQSV0acW00HHiQcW6>HM1acW8@P9
7>L022487>LQ21d78@PM1b487@L28@P052DY8@PQ:24X9BTQ:2487@LQ21d78BPQ22DY8@PU:B489BTQ
22DY8@P29BT00bU:9BTYBP02:DX00bE9:DXYBP02:DX0;Be[:DX]JbUZ;F/YJRe[:FX]JbUZ;F/YJRe[
:FX]JbU:;F/YJRe[:FX]JbU::FXYBRe[:DX]JbU:;F/YJRe[:FX]JbUZ;F/YJRe[:FX]JbU:;F/YBRe[
:DX]J`02:DX02BE9:DXYJRe[:FX]JbUZ;F/YJP03;F/02BUZ<H`]Jc6<:FX]JbUZ;F/YJP03;F/01RUZ
;F/YJRe[:DX]J`8YBP0<9DTYBRe[;H/]Jc6<;H/aS2f;<H`]Jc6<2Be[00PaS2e[<H`YJRe[:FXYBRE9
0RU:00L]JbU:;F/YBRe[:DX]J`049BT0524X9BTQ:2DY8BPU:B4X9BTQ:2DY8BPU:B4X9BTQ:2DY8BPU
:B489BT28@P031d78@PM1b487@LQ21cW7@LLib487>LQ208Li`076>HLiaSV8@PLib486>H00acW00HH
iQcW6>HHiQS67>L56<H01acW6<HLiaS66>HHaQcW00<DY@@HaP0P7>LHaQSV6<HLiaS66>HDYAS65:DH
aQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQBU5<DDY@<HaPDDY@0D6<HDYAS65:DH
aQBU6<HDYAS65:DHaQC56<HDYAC55:DHaQ2T5:D@Y0@DY@086<HDaAS65:DHaQBU5<D@Y0<DY@034:@D
YABU00@DY@0:5<DDYAS6JeY[NV]JKg][FV]j0001Kg/001I[FP03001[FV]J00A[FP046<HDaAS65<D3
5:D46<H02aC56<HHaQS66>HLiaSV7>LHiQcW6>H01acW0aS611cW12480Qd700<Q21d78@P02B4800HQ
:2488BPQ224X8@P28BP00bDY8BPU:@0:9BT7:DX012UZ:DXYJRU:1be[00@YJRe[:FXYJPLYBP04:FXY
BRUZ:FX3;F/3;H/4:DX3;F/012UZ;F/YJRUZ136<00<YJRe[:FX012U:00<]Rbe[;H/032e[0bUZ0bU:
0Re[00<YJRe[;F/01Re[0c6<00L]Jbf;;F/]RbU::FXYBP05;F/02BUZ;F/]JbU:9DTYBRE9:DXUB@08
:DX79BT0124X9BTQ:2DY0b4X00LU:B4X8BPQ:2DY8BPU:@048@P00ad78@PM1`0?7>L36<H47>L36<H2
7>L01ASV7>LHiQcW6>H011S600LHiQS66>HHaQcW6>HLi`045:D>6<H00aC56<HDa@085:D01aS65<DH
aQBU5<DDYAC500<DY@046<HDaAS65<D35:D00aC56<HDa@036<H01AC56<HHaQS65<D011BU0aS600<D
YAC55:D01AC500@DYAC55:DHaP8Da@085:DDaABU5<DDYAC55:DDa@<DY@I[FP40005[FP0003U[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]j5<DHaQC5
6<HDYAS65:DHaQC56<HDaAS65<DHaQC57>LHiQcW6>HLiaSV7>LHiQcW6>HQ21cW7@L00QcW00<HaQSV
6>H00acW014M1b487@LQ21cW8@PLib487@LQ21d78BPQ22DY7@LQ21d7008Q20089BTQ22DY8@PU:B48
9BTQ20DU:@058BPU:BDY9BTQ:0029BT01RU:9DTYBRE9:DXUB@8YBP03;F/YBRe[008]J`03:FX]Jbe[
00<]J`03:DX]JbU:00@YBP0@;F/YBRUZ:DX]JbU:;F/YJRe[:FXaS2e[<H`YBRUZ:DX3;F/04RUZ;F/Y
BRe[:DXaS2f;<H`]Rbe[:DX]JbU:;F/YBRUZ;F/aS0H]J`0A:FX]JbUZ;F/YJRe[:DX]JbU:;F/YBRUZ
:DX]JbUZ;F/YJP07;F/01Rf;<H`]Rc6<;F/aS08]J`0N:DX]Jbe[;H/]Jc6<:FX]JbUZ;F/UBBU:9DTY
BRE9:DXUBBU:9DTYBRE9:DXUBBU:8BPU:B4X9DTU:BU:0RDY00dQ:2DY8BPU:B489BTQ22DY8@PU:B4X
9BTQ:0038@P021d78@PLib487>LQ21cW7@L47>L01ASV7>LLiacW6>H00QcW00<HaQC56<H011cW0aS6
00PLiaSV7>LHiQcW6>HLiaC511S600`LiaS67>LHiQcW6>HDaABU6<HDYAS65<D36<H01AC56<HDaAS6
5<D00aS600PDaAS65<DHaQBU6<HDYAC511BU00hDaAS65<DHaQBU6<HDYAC55:DHaQC56<HDaAS60aBU
00hHaQC56<HDaAS65<DHaQC56<HDaAS65:DHaQBU0QC5028HaQC56<HDYAS65:DHaQBU6<HDYAS65:DH
aQBU6<HDYAS65:DHaQBU6<HDYAS65:DHaQBU5<E[FVmkJeY[NV]JKg/0005[NP005V]J00<006]JJeX0
16]J00@HaQC56<HDa@<DY@HHaP095<DHaQSV7>LHiQcW6>HLiaSV00LLi`<HaP@Li`DQ208M1`8Q2003
7@LQ224800HQ20068BPQ224X8@PQ:248124X2bDY1bU:00@YJRU::FXYBPL]J`<YJP03;F/YBRU:00DY
BP03:FXYBRUZ008]J`05:FX]Jbf;<H`]R`04:DX3;F/3:FX00be[<H`aS002<H`3:FX4:DX00bf;;F/]
R`0;;F/2:FX00be[:FXYBP02:DX00bUZ;F/]J`08;F/02S6<;H/aS2e[;H/]Jbf;:DXYJRU:1be[00LY
JRU:9DTYBRE9:DXUB@08:DX79BT01R4X9BTQ:2DY8BPU:@<Q:0039BTQ:2DY008U:@@Q20<M1`lLi`<H
aP@Li`<HaP076>HLiacW7>LHiQcW6>H011S600LHiQS66>HHaQcW6>HLi`045:D>6<H00aC56<HDa@08
5:D01aS65<DHaQBU5<DDYAC500<DY@046<HDaAS65<D35:D01AC56<HDaAS65<D00aS600<DaAS66<H0
11BU0aS600<DYAC55:D00QC500HHaQC55<DDYAC55:D25<D02AS65:DDaABU5<DDYAC55:DDa@035:D6
JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]J
Kg][FV]jJeY_NaC56<HDa@/HaP0?5<DHaQS66<HHiQcW6>HLiaS67>LHaQcW6>HLiaSV00<Li`036>HL
iaSV008Li`068@PLib487@LQ21d7124800HU:B489BTQ:2DY8BP29BT032U:9BTU:B489BTQ22DY8@PU
:B489BTQ208U:@0A:DXU:BU:9BTYBRDY:DXUBBU:9DTYBRE9:DXYJRe[:FX]J`03:DX00bf;;F/aS002
;F/052UZ;F/]Jbe[:DX]JbUZ;F/YJRe[:DXYJRU:;F/YBRe[:DX]JbUZ;F/4:DX01BE9:DXUBBU:9DT0
0RU:00HYJRU:;F/YBRUZ:DX5;F/02RUZ;F/YJS6<;H/aS2f;<H`]Rc6<12e[00dYJRe[:FX]JbUZ;F/Y
JRe[:FX]JbUZ;F/YJP03;F/012UZ;F/]Jc6<0Re[00<YJRe[:FX00be[12U:00@U:BU:9BTYBP<U:@0=
:DXU:BU:9BTYBRE9:DXUBBU:9DTYBRDY:DX012DY00@Q:2DY8@PQ:0DQ20059BTQ22DY8@PU:@028@P0
1QcW8@PLib487@LQ208Li`056>HLiacW7>LHiP057>L05aS66>HHaQcW6>HLiaS67>LHaQcW6<HLiaS6
7>LHiQcW6>HDaABU6<HDYAS65:D01AS600<HiQcW6>H00b4803HM1acW6<HHiQC56<HDaAS65<DHaQC5
6<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65:DDaABU6<HDaAS65<DHaQC56<HDaAS65<DHaQBU
6<HDYAC55:DHaQBU6<HDYAS65:DHaQC56<H25:D00a2T5:DDY@025:D01QS65:DHaQBU6<HDY@8Da@03
6<HDaAS600<DY@035<DDYAS600PDY@07JeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP04JeXB
6<H00acW6>HLi`027>L26>H;7>L011d77>LM1acW224800<Q:2489BT01RDY00<Q224X8@P00R4X00HU
:B4X8BPQ224X8@P29DT012U:9DTUBBDY0RE90bU:00DUBBU:9DT]JbUZ008]J`<YBPP]J`07:FX]JbUZ
;F/YJRe[:FX012U:00HYJRU::FXYJRe[:FX5:DX00bE9:DXYBP0::DX7;F/01bf;<H`]Rc6<;H/aS2f;
00D]J`05:FX]Jbe[;F/YJP03;F/01BUZ;F/YJRe[:FX012e[00@]Rbe[;H/YJPH]J`@YBP039DTU:BE9
00@U:@049DTYBRE99DT2:DX012E9:DXYBRU:0bE900<YBRDY9BT00RDY1b4800TQ:2488BPQ224X8@PQ
:1d78@P00Qd70b482acW11S600@HiQcW6>HLi`<HiP<HaP046>HLiaSV7>L35:D011C55:DDaABU11S6
00<LiaSV7>L0124811S600DDaAS66<HHaQC500<HaP045<DHaQS66<H35<D01QS65<DHaQC56<HDa@<D
Y@035<DHaQC500DHaP045<DHaQC56<H25<D45:D011C56<HDaAC50aS61aBU0aC500HHaQBU5:DDYAS6
5<D26<H>5:D6JeX10001JeX0000PJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeX006]JJgY[FVmkJeY[NQC56<HDa@THaP035<DHaQS600<HaP0;6>HLiaSV7>LHaQcW6<HLiaSV
7>LHiP037>L00aSV7>LHiP027>L01R487>LQ21d78@PM1`@Q20069BTQ22DY8BPU:B4X0RDY00`UBBDY
9BTQ22DY8@PU:B489BTQ22DY8@P29BT04BU:9BTYBRDY:DXU:BU:9DTYBRE9:DXUBBU::FX]JbUZ;F/0
0bU:00<aS2e[;H/012e[018YJRe[:DX]JbUZ;F/YJRe[:DX]JbU:;F/YBRe[:DX]JbUZ;F/4:DX00bE9
:DXUB@03:DX022E9;F/YBRUZ:DX]JbU::FX4;F/02RUZ;F/YJS6<;H/aS2f;<H`]Rc6<12e[00lYJRe[
:FX]JbUZ;F/YJRe[:FX]JbUZ;F/YJRe[:FX012e[00HaS2e[;F/YJRe[:FX3;F/4:DX012DY:DXU:BU:
0bDY00dYBRDY:DXU:BU:9DTYBRE9:DXUBBU:9BTYBP029BT01R4X9BTU:B4X8@PU:@DQ20059BTQ22DY
8@PU:@028@P01acW8@PLib487@LQ21SV00<Li`036>HLiacW00@Li`0H6>HHaQcW6<HLiaSV7>LHaQcW
6<HLiaS66>HHaQcW6>HLiaSV6<HDYAC55:DHaQBU1AS600DHiQcW6>HQ21d7008Q200f6>HHaQcW5<DH
aQC56<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65<DH
aQC56<HDaAS65:DHaQBU6<HDYAC55:DHaQBU6<HDYAS65<DHaQ2T1QBU00/HaQBU6<HDYAC55:DHaQC5
6<HDaAS600<DY@036<HDYAC500PDY@07JeY_Nf]JJgY[FVmk00000F]j000FJeX00`00JeY[FP04JeXB
6<H37>L36>H<7>L01Qd77>LM1acW8@PM1`HQ20038BPQ22DY00HU:@038@PQ:24800DQ:0048@PQ:248
:DX49DT02BDY9DTYBRU:9DTYBRE9:DXUB@03;F/012UZ:DXYBRU:22e[0bUZ00@]JbUZ;F/YJP@YBP06
:FXYBRUZ:FX]JbUZ1BU:00<UBBU::DX02RU:1Be[00TYJRe[;H/aS2f;<H`]Rc6<;H/01be[00DYJRe[
:FX]JbUZ00<]J`03:FX]JbUZ00@]J`03;H/]Jbf;008]J`03:FX]Jbe[008]J`@YBP039DTU:BE900@U
:@<UB@03:DXUBBU:008YBP049DTYBRU::DX29DT49BT78@P01b4X8@PQ:2488BPQ224X00<M1`@Q20/L
i`@HaP036>HLiaSV008HiP037>LHiQS6008HaP046>HLiaSV7>L35:D011C55:DDaABU11S600<LiaSV
7>L012481QS600DDaAS66<HHaQC500<HaP045<DHaQS66<H35<D011S65<DHaQC50aBU00<DaAS65<D0
1AS600<DaAS65<D00QC500<HaQBU5:D00QBU0aC511S61aBU00@DaAS65<DDa@<DY@<HaP035<DDYABU
00`DY@I[FP40005[FP0001e_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FP00JeY_Nf]JJgY[FVmk008HaP095<DHaQC56<HDaAcW6>HLiaSV00<Li`0?6>HLiaSV7>LHaQcW6<HL
iaS67>LHaQcW6<HLiaS600LLi`8Q20047@LQ22DY9DT69BT00b4X9BTQ:0049BT01BU:9BTUBBDY:DX0
2BDY010YBRDY:DXU:BU:9BTYBRE9:DXUBBU:9DTYBRE9:DXUB@8YBP0=;F/YBRe[:FX]JbUZ;F/YJRe[
:FX]JbU:;F/00RU:00`UBBU::FX]JbUZ:DXUBBU:9DT]JbU::FX4:DX39BT03RU:9DTYBRU:;F/YBRe[
:DX]JbUZ;F/YBRe[:DX2;F/042f;;F/aS2e[;H/]Jc6<;F/]RbU:;F/YBRe[:DX]JbU:12e[00LYJRU:
;F/YBRe[:FX]J`04:DX2;F/032UZ;F/YJRe[:FX]JbU:;F/UBBU:9DTYBPLU:@07:DXUBBU:9DTYBRE9
:DX022DY00<Q22DY8@P00bDY00XQ:2DY8BPU:B4X9BTQ:2DY8@PU:@8Q200:7@LQ22488@PM1b487>LM
1acW8@P36<H05AcW6>HLiaSV7>LHiQcW6>HLiaSV7>LHaQcW6<HHiQd78@PM1acW6<HLi`026<H01QC5
6<HHaQcW6<HLi`@HaP076>HLiaSV6<HDYAS65:D00aS600XDYAS65:DHaQC56<HDaASV6<HLi`8HaP0A
5<DHaQS66<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDa@036<H01QC56<HHaQS65:DHaPDDY@0F6<HD
YAS65:DHaQBU6<HDYAC55:DHaQBU6<HDYAS65:DHaQBU6<HDYAS64:@45:D02AS65:DHaQC56<HDaAS6
5:DHaP025:D02A2T5:E[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FPLHaP036>HLiaSV
00DLi`046>HLiaSV7>L36>H36<H011SV7>LHiQSV1acW12481bDY00<Q:2DY9BT012DY00<UBBDY9DT0
2bDY00@UBBDY9DTU:@8UB@03:DXUBBU:00HYBP049DTYBRE9;F/2:FX2;F/02BUZ;F/]Jbe[:FX]JbUZ
;F/YJP04:DX3;F/00bE9:DXUB@08:DX39BT5:DX00bUZ:DXYJP03;F/012UZ:DXYJRU:2be[0bUZ00@]
JbUZ:FXYJP<]J`@YBP03:FX]JbUZ00@YBPL]J`<YJP04:DXUBBU::DX79BT00bE9:DXUB@03:DX00bE9
9BTU:@059BT012488BPQ224X12DY00LQ:2DY8BPU:B4X9BTQ:00:8@P47>L36<H02ASV7>LHiQcW6>HL
iaSV7>LHiP027>L46<H01b487@LQ21S66>HHaQSV008HaP045<DHiQS66>H56<H37>L011BU5<DDYAC5
11S60aC500<HaQC56<H011S600<DaAS65<D021S600@DaABU5<DDa@XHaPPDY@055<DDYAC55:DDa@05
5:D25<D01aS65:DDaABU5<DDYAC500PDY@8HaP065<DHaQBU5<DDYAC50aBU1V]J0@000F]J00007V]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgXDa@<H
aP095<DHaQC57>LHiQcW6>HLiaSV00<Li`0=6>HLiaS67>LHaQcW6<HHiQS67>LHaQcW6<H01acW00HM
1b488@PQ22DY:DX69BT01B4X9BTU:BDY8BP00RDY00DYBRDY:DXU:BE900TU:@0@:DXU:BU:9BTYBRDY
:DXUBBU:9DTYBRE9:DXUBBU:9DT2:DX03Re[:DX]JbUZ;F/YJRe[:FX]JbUZ;F/YBRe[9DT3:DX02RUZ
;F/YJRU:9DTYBRE9:FXYBRe[12U:0bDY0bU:00/UBBe[:DX]JbU:;F/YJRe[:DX]JbU:008]J`0@<H`]
Jbf;;F/aS2e[;H/]Jc6<:DX]JbU:;F/YBRe[:DX5;F/01RU::FXYBRe[:FX]J`@YBP0>:FX]Jbe[;F/Y
JRe[:FX]JbU:;F/UBBU:9DTYBPLU:@07:DXUBBU:9DTYBRE9:DX022DY00DQ22DY8@PU:B4X00<U:@08
8BPU:B4X9BTQ:2DY8@PQ:0@Q20087@LQ21d78@PLib487>LM1`<HaP0E7>LHiQcW6>HLiaSV7>LHiQcW
6>HLiaS66>HHaQcW7@LQ21d77>LHaQcW008HaP065<DHaQS67>LHaQcW11S600LHiQcW6>HHaQBU6<HD
Y@036<H02QBU6<HDYAS65<DHaQC57>LHaQSV0QS600<DaAS65<D00aS600dDaAS65<DHaQBU6<HDYAS6
5<DHaQC56<HDa@036<H011C56<HDYAC51ABU01DHaQBU6<HDYAS65:DHaQBU6<HDYAC55:DHaQBU6<HD
YAS65:DHaQBU6<H00QBU00`@Y1BU5:DDaABU6<HDaAS65<DHaQBU6<H25:D02A2T5:E[FVmkJeY[NV]J
Kg/00001JgX001I[FP<000A[FPLHaP036>HLiaSV00DLi`036>HLiaSV008HiP037>LHiQS6008HaP<H
iPPLi`@Q20TU:@038BPU:BDY008U:@039DTU:BE900/U:@059DTU:BE99BTYBP029DT2:DX00bE9:DXY
BP03:DX00bE9:DXUB@02:FX00be[:FX]J`02;F/012UZ;F/]Jbe[0bUZ12U:00H]JbUZ;F/UBBU:9DT8
:DX39BT5:DX00bUZ:DXYJP03;F/012UZ:DXYJRU:2Re[0RUZ00H]JbUZ:FXYJRe[:FX3;F/4:DX00bUZ
;F/YJP04:DX7;F/00bUZ;F/YJP03:DX00bE99BTU:@059BT01BE9:DXUBBU:9DT00RU:1bDY00@Q224X
8@PQ:0@U:@078BPU:B4X9BTQ:2DY8BP02R4811cW0aS600LHiQcW6>HLiaSV7>LHiP037>L00aSV6<HH
aP026<H022487@LQ21S66>HHaQSV5<D26<H00aSV6<HHiP056<H01acW6>HLiaBU5<DDYAC500<HaP8D
a@046<HDaAS65<D56<H00aC56<HDa@086<H00aC55:DDa@026<H00aC56<HHaP066<H85:D01AC55:DD
aABU5<D01ABU00THaQC55<DDYAC55:DDaABU5<D021BU00PDaAS66<HHaQBU5<DDYAC50aBU1F]J0P00
0F]J00007FmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[
NV]JKg/01AS6010HiQS68@PLib487>LQ21d78@PM1acW6>HLiaS67>LHaP8HiP0@7>LHiQcW6>HLiaSV
7@LLib487@LQ21d78@PQ:2DY8BP29BT01RU:9BTYBRDY9DTU:@<YBP049DTYBRE9:DX39BT00bU:9DTY
BP029DT01BDY:DXQ:2DY8BP00bDY00<Q:2DY9BT00RDY014YBRE9:DXUBBU:9DTYBRDY:DXU:BU:9DTY
BRE9:FXYBRe[008YBP039DTYBRU:00@YBP03;F/YBRe[008YBP0I9DTYBRE9:DXUBBe[:DX]JbDY:DXU
:BU:9BTYBRDY:FXYBRe[:DX]JbUZ;F/YJRe[:FX00be[01hYJRe[:FX]JbUZ;F/YJRe[:FX]JbU:;F/Y
JRe[:FX]JbE9:DXUBBU:9DTYBRE9;F/YJRe[:FX]JbUZ;F/2:DX04bE9:DX]Jbe[:FX]JbUZ;F/YJRe[
:DX]JbDY:DXU:BU:9BTYBRDY008UB@03:DXUBBDY00<U:@0::DXUBBU:9BTUBBDY:DXQ:2DY8BP39BT0
324X9BTQ22DY8@PU:B489BTQ22DY8@PU:@@Q20057>LQ21cW7>LHiP027>L01B487>LQ21cW7@L00QcW
00<HiQcW6>H01AS600LDaAS65<DHaQC56<HDa@026<H00aSV6<HLi`026<H00aBU6<HHaP046<H04ASV
6<HLiaC56<HDaAS65<DHaQC56<HDYAC55:DHaQBU5<D00aBU00HHaQC56<HDaAS65:D36<H09QC56<HD
YAC55:DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HD
aAS65:DHaQBU6<H25:D01A2T5:D@Y1BU4:@00aBU01TDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAS6
5:DDaABU6<I[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J00A[FPLHaP047>LM1acW7@L48@P0
0aSV7>LLi`046<H00acW6>HLi`077>L48@P01bDY8BPU:B4X9DTU:BE900@U:@@YBP039DTYBRU:00<U
:@049DTYBRE9:DX49BT00b4X9BTQ:0079BT01BE9:DXUBBU:9DT00bU:0bE900<YBRE9:DX02bU:0RUZ
00<]JbUZ:DX00RU:00@UBBU:9DTYBP<YJP03:DXUBBE9008UB@039BTUBBU:00<YBP03:FX]JbUZ00l]
J`05:DXYJRU:;F/YJP02;F/00bU:9DTYBP02:DX02BE9:DX]Jbe[:FX]JbUZ;F/YJP04:DX7;F/3:FX0
0bE99BTUB@059BT00bU:9DTYBP049BT2:DX00bE99BTU:@099BT022488BPQ224X8@PQ:2489BT38BP3
8@P02Qd78@PM1ad76>HLiaSV7@LLiad711cW00@HiQcW7>LLia8HaP035:DDaABU00/HaP045<DHaQC5
6<H:5:D01aC56<HDaAS65:DDaABU00@HaP@DY@8Da@046<HDaAC55<D56<H011C56<HDaAS60aC500HH
aQC55<DDaAS65<D46<H25<D00aS65<DDY@025:D00a2T5:DDY@045:D01QS65<DHaQS65<DDY@8Da@05
6<HDaAC55<DHaP025<D35:D6JeX10001JeX0000MJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NP056<H06QcW6<HQ21cW8@PLib487@LQ21d77>LHiQcW
6<HHiQS67>LHiQcW6>HLiaSV7>LHiR487>L27@L01R487@LQ224X9BTQ:08U:@0=:DXU:BE99BTYBRDY
:DXUBBU:9DTYBRE9:DX00bDY00`YBRE9:DXUBBU:9BTUBB4X9BTQ:2DY8BP69BT04BU:9DTYBRE9:DXU
BBU:9BTYBRDY:DXUBBU:9DT]JbU::FX012U:00<UBBU::DX00RU:00@]JbU:;F/UB@<YBP0E9DTYBRE9
;F/YBRe[9BTYBRDY:DXU:BU:9BT]JbU::FXYBRe[:FX]JbUZ00<]J`0Q:FX]Jbe[;F/YJRe[:FX]JbUZ
;F/YJRe[:DX]JbUZ;F/YJRe[9DTYBRE9:DXUBBU:9DT]JbUZ;F/YJRe[:FX]JbE900<YBP0E:FX]Jbe[
;F/YJRe[:FX]JbU:;F/U:BU:9BTYBRDY9DTU:BU:9DTYBRE900@U:@0<:DXUBBU:9BTYBRDY9DTQ:2DY
8BPU:B4X0bDY00XQ22DY8@PU:B489BTQ22DY8@PU:@@Q20057>LQ21cW7>LHiP027>L01B487>LQ21cW
8@P00QcW00<HiQcW6>H01AS600DDaAS65<DHaQC500<HaP045<DLiaS66>H26<H00aBU6<HHaP046<H0
2AcW6<HHiQC56<HDaAS65<DHaP025<D01QBU6<HDYAC55:DHaP<DY@086<HDaAS65<DHaQBU6<HDa@<H
aP0M5:DHaQBU5<DDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65:DHaQBU6<HDYAS65:DHaQC56<HD
a@036<H01ABU6<HDYAS64:@00aBU01l@Y1BU4:@DYA2T5:DDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HD
YAS65:DHaQBU5<E[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[FP046<HDaAS65<D36<H0
0aSV7>LHiP026>H01QcW6>HHiQSV7>LHiP@Li`8HiP037>LHiQcW00<Li`0:7@LLib487@LQ21d79DTU
:BE99BT9:DX01BE9:DXUBBU:9DT00bU:0bE900<YBRE9:DX00RU:00PUBBU::DXYBRE9:DXUBBU:0bE9
00<YBRE9:DX012U:00<UBBU:9DT01bU:12e[22U:00<UBBU::DX00bU:00<YJRe[:FX01bU:00@YJRe[
;F/]J`@YBP<YJP03;F/YJRe[008]J`03:FX]JbU:00HYBP@U:@@YBP03;F/YJRe[00@YBP@]J`03:FX]
Jbe[00<]J`LYBP039DTU:BE900DU:@049DTU:BE98BP=9BT58@P01R4X8@PQ:24X9BTQ:0@Q208M1`03
8@PLiacW008Li`056>HLiacW7>LHiP027>L36<H27>L00aSV7>LHaP036<H00aSV7>LHiP046<H26>H0
0acW5<DHaP0=6<H01AC55:DDaAC56<H00QC50QS600@DaAS65<DHaP8Da@065:DDaABU5<DDYAC50aS6
11BU00DHaQC56<HDaAS600<Da@066<HDaAC55<DHaQC511S600@DaABU5<DDa@<HaP085<DHaQC56<HD
aAS65<DHaP<Da@036<HDaAS600@DY@075<DHaQC55<DDYAC55:D00aS61ABU00HDaABU5<DDYAC55:D6
JeX10001JeX0000PKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]J
Kg][FV]jJeY_NaC56<HDa@8HaP0>6>HHaQcW6<HLiaS67>LHaQcW6<HLiaS67>LHiP<Li`0A6<HLiaS6
7>LHiQcW6>HQ21cW8@PM1b487@LQ22DY:DXU:@04:DX00be[:DXYJP02:DX0<BE9:DXUBBU:9DTYBRE9
:DXUBBU:9BTYBRDY:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRDY:DXU:BU:9DTYBRE9;F/YBRUZ9DTYBRE9
:DXUBBU:9DTYBRE9:DXUBBe[:FX]JbUZ00LYBP039DTYBRE900<YBP049DT]JbU:;F/5:DX022e[:DX]
JbUZ;F/YJRU:9DT2:DX04Be[:DX]JbUZ;F/YJRe[:FX]JbUZ;F/YBRUZ:DX]JbU::FX012DY00LUBBU:
9DTYBRUZ;F/YJP04:DX05Be[:FX]JbUZ;F/YJRe[:FX]JbUZ:DXUBBU:9DT]JbU::FXU:BU:9BTYBP05
9BT02BU:9BTU:B4X9BTQ:2E99BTYBP049BT03b4X9BTQ:2487@LQ21d79BTQ22DY8@PU:B489BTM1`03
8@P00acW8@PLi`057>L01QSV7>LHiQcW6>HLi`8HaP0<5<DLiaSV7>LHiQcW6<HHiQS67>LHaQcW1AS6
00HLiaS66<HDaAS65<D76<H06AC56<HDaAS65:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaQBU6<HD
YAS65:DHaQBU00<HaP8DY@0A4:@DYAC56<HDaAS65:DHaQBU6<HDYAS65:DHaQBU6<HDa@036<H07aBU
6<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HDYAS65:DHaQC56<HDaAS65:DDaABU6<HDYAS65:DHaQBU
00@HaP0B5<DDYAS65:DHaQBU6<HDYAS65:DHaV]JJgY[FVmkJeY[NP000Fmk000FJeX00`00JeY[FP04
JeX011S65<DHaQC50aS60aSV00DLiaSV6>HHiQcW00<HiPDLi`8HiP8Li`0=6>HLiacW7@LLib487@LQ
21d79DTU:BE99BT01bU:00`UBBU::DXYBRE9:DXUBBU:9DTYBRE9:DX29DT2:DX022E9:DXYBRU:9DTY
BRE9:DX39DT012U:9DTYBRE91BU:00DUBBU:9DTYBRE900DYBP@]J`PYBP039DTYBRU:00<YBP<YJPLY
BP8]J`03:FX]JbU:00<YBP03:FX]JbUZ00<]J`04:FX]Jbe[;F/7:DX49BT4:DX00be[:FX]J`04:DX6
;F/012UZ;F/]Jbe[1bU:00<UBBDY9DT01BDY00<UBBDY9DT00RDY00<Q:2DY9BT02BDY1B4800<Q:248
8BP00b4X1B480Qd71QcW00DHiQcW7>LLiaSV00<HaP046>HLiacW7>L46<H36>H46<H00acW6>HHiP02
6<H00aC56<HHaP0:6<H00aC55:DDa@035<D00aS65<DHaP026<H02QC56<HDaAC55:DDaABU5<DDYAC5
0aS611BU00<HaQC56<H00aC500DHaQC55<DDaAS600<Da@@HaP035<DDYAC5008HaP085<DHaQC56<HD
aAS65<DHaP<Da@056<HDaAS65<DHaP045:D45<D00aBU5<DDY@036<H55:D01QC55:DDaABU5<DDY@I[
FP40005[FP00021[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[
NV]JKg][FV]j5<DHaQC50QS600`LiaS67>LHaQcW6<HLiaS67>LHaQcW6<H37>L04aSV7>LHaQcW6<HL
iaSV7>LHiR487>LQ21d78@PM1b489BTYBRDY00@YBP03:FXYBRe[008YBP0a9DTYBRE9:DXUBBU:9DTY
BRE9:DXU:BU:9BTYBRE9:DXUBBU:9DTYBRE9:DXUBBU:9BTYBRDY:DXUBBU:9DTYJRU:;F/UBBU:9DTY
BRE9:DXUBBU:9DTYBRE9;F/YJRe[:FX01bU:00DUBBU:9DTYBRE9008YBP03;F/YBRe[00DYBP06:FXY
BRe[:FX]JbUZ0bU:00XUBBe[:DX]JbUZ;F/YJRe[:FX]J`8YJP06:DX]JbU::FXYBRe[12DY00LUBBU:
9DTYBRUZ;F/YJP04:DX05Be[:FX]JbUZ;F/YJRe[:FX]JbUZ:DXUBBU:9DTYJRU:;F/U:BU:9BTYBP05
9BT02BU:9BTU:B4X9BTQ:2U:9BTUB@049BT03R4X9BTQ:2487@LQ21d79BTQ22DY8@PU:B489BT28@P0
11d78@PLib481QcW00LHiQcW6>HLiaSV7>LDa@026<H00acW6>HLi`026>H01QS67>LHaQcW6<HLi`DH
aP067>LHaQS65<DHaQC51aS601TDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDaAS65<DHaQC56<HDYAS6
5:DHaQBU6<HDY@036<H04A2T5:DDYABU5<DHaQC56<HDYAS65:DHaQBU6<HDYAS65:D00aS601HDaAS6
5:DHaQBU6<HDaAS65<DHaQC56<HDaAS65<DHaQBU6<HDYAS65<DHaP8Da@095:DHaQBU6<HDYAS65:DH
aQBU00DHaP0A5:DDaABU6<HDYAS65:DHaQBU6<I[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J
00A[FP`HaP0<6>HHaQSV6>HLiaSV7>LHiQcW6>HLiaSV1QcW00@M1acW9BTQ:08U:@049DTU:BE99BT3
;F/012UZ;F/YJRUZ0RU:0bE900TYBRE99DTUBBU:9DTYBRE9:DX00bDY00<YBRE9:DX00RU:0bE95BU:
00DYJRe[;F/]JbUZ00<]J`05:FX]JbUZ:DXUB@0C:DX00bE9:DXUB@08:DX00be[:FXYJP0;:DX3;F/0
1bUZ;F/]Jbe[9DTYBRE9008YBP03:FXYBRe[00@]J`03:FX]JbU:00HYBPlU:@038BPU:B4X00TU:@08
8BPU:B488BPQ224X8@PQ:0HQ20077@LQ21d78@PM1b486>H01acW0QSV00LHaQSV6<HHiQS66>HHaP02
6>H00acW6>HHaP0=6<H01AC55:DDaABU6<H00aC500LDYA2T5:DHaQSV6<HHiP036<H75:D01AC56<HD
aAS65<D00QS611BU11S600<DaABU5<D03QBU00PDaAS65<DHaQC56<HDaAS60aBU00<HaQC56<H00QC5
00TDYAC55:DDaABU5<DDYAC55:D00aS60QC500<DYAC55:D02QBU1V]J0@000F]J00007FmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J001[FVmkJeY[NV]JKg/00QS601LDaAS6
6<HHaQC57>LHaQSV6<HLiaS67>LHaQcW6<HLiaSV7>LHiQcW6>HLiaSV00@Li`068@PLib488BPU:B4X
0RDY00TYBRDY:DXYJRe[:FX]JbU:;F/00RU:00dUBBU:9BTYBRDY:DXU:BU:9BTYBRE9:DXUB@039BT0
1bE9:DXUBBU:9BTYBRDY00<YBP059DTYBRE9:DXUB@03:DX01BE9:DXUBBU:9DT01RU:018]JbUZ;F/Y
JRe[:FX]JbUZ;F/YJRe[9DTYBRE9:DXUBBU:9DT3:DX00bE9:DXUB@02:DX00be[:DXYBP04:DX00bE9
:DXUB@05:DX012E9:DXYBRe[0RU:00<UBBU:9DT012U:010]JbU::FXYJRe[:FX]JbUZ;F/YJRU:9DTY
BRE9;F/YBP<]J`05:FX]JbUZ;F/YJP03:DX01BE9:DXUBBU:8BP00bDY00DQ:2DY8BPU:B4X00<U:@05
8BPU:B4X9BTQ:0069BT03RE99BTU:B4X9BTQ:2DY8@PU:B489BTQ22DY7@L38@P02Ad78@PM1b487@LQ
21d77>LHiP027>L05Qd77>LQ21S67>LHaQcW6<HLiaS67>LHaQcW6<HLiaSV7>LHaQcW6<HHiQC50QS6
00<HiQS67>L00QS6010DaAS65:DHaQBU6<HDYAS65:DHaQ2T5:D@Y1cW6<HLi`8HaP035<DHaQBU00HD
Y@076<HDaAS65<DHaQC56<H00QBU00P@Y1BU5<DHaQC56<HDYAS611BU00<@Y1BU5:D00QBU00dHaQBU
5<DDYAS65:DHaQC56<HDaAS65<DHaP025<D05QBU6<HDaAS65<DHaQBU6<HDYAS65:DHaQBU6<HDYAS6
5<DHaQC56<HDYAS61QBU00<@Y1BU5:D00aBU00M[FV]jJeY_Nf]JJgX00001Kg/001I[FP03001[FV]J
00A[FP`HaP036>HHaQSV00<HiP067>LHiQcW6>HLiaSV1QcW00<M1acW9BT00RDY00DQ:2E99BTUBBDY
00<]J`<YJP03;F/UBBU:008YBP<UB@07:DXUBBE99DTYBRE9:DX00bDY0bU:0bE900<YBRE9:DX052U:
0Re[00TYJRe[;F/]JbUZ;F/YJRe[:FX05BU:00<UBBU:9DT022U:0RUZ00<]JbU::DX02BU:00<]JbUZ
;F/00Re[00DYJRe[9DTYBRE9008YBP03:FXYBRe[00@]J`03:FX]JbU:00HYBPlU:@038BPU:B4X00LU
:@0:8BPU:BDY9BTQ224X8@PQ:2488BP68@P01Qd78@PM1b487@LQ208Li`036>HLiacW008Li`8HiP08
7>LHaQSV6<HHiQS66>HHaP8HiP037>LHiQS600dHaP045<DDYAC55:D25<D00aS65<DDY@025:D011S6
6>HHaQSV0aS61aBU00<DaAS65<D00aS600<DaABU5:D00QBU11S600<DaABU5<D03QBU00PDaAS65<DH
aQC56<HDaAS60aBU00<HaQC56<H00QC500/DYAC55:DDaABU5<DDYAC55:DHaQC5008HaP035<DDYAC5
00/DY@I[FP40005[FP0002A[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FP00JeY[NV]JKg][FV]j5<DHaQC56<HDaAS65<D46<H09acW6>HLiaSV7>LHaQcW6<HLiaS67>LHaQcW
6<HLiaSV7>LHiR487@LQ21d78@PM1b489DTYBRE9:DXYJRe[:FX]JbUZ;F/YJRe[:DX]J`02:DX01BE9
:DXUBBU:9DT01BU:00lU:BU:9DTYBRE9:DXU:BU:9BTYBRE9:DXUBBU:9DT00bU:00HU:BU:8BPU:B4X
9BT3:DX0136<;F/aS2e[136<00<]JbUZ;F/012U:01TUBBU:9DTYBRE9:DXUBBU:9DTYBRDY:DXU:BU:
9DTYBRE9:DXU:BU:9BT]JbU:;F/YBP039BT4:DX03BDY:DXU:BU:9DTYBRE9:DXUBBU:9DTYBRE900HY
BP0N;F/YBRe[:DX]JbU:;F/YJRe[:FX]JbUZ;F/UBBU:9DTYBRE9:DXUBBU:9BTYBR4X9BTQ:2DY8BPU
:B4X0bDY00@Q:2U:9BTYBP@U:@098BPU:B4X9BTQ22DY8@PU:B4X00<U:@0<8@PU:B488@PM1b487@LQ
21d78@PLib481AcW01TQ21cW7@LHaQcW6<HLiaS67>LHaQcW6<HLiaS67>LHiQcW6>HLiaS67>LHaQSV
6<HLi`046<H00aBU6<HDY@036<H03AC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500<HaP0<5:DHaQBU
6<HDYAS65:DDaABU6<HDYAS62ABU00@HaQBU6<H@Y08DY@036<HDYAS600DDY@056<HDYAS65:DDa@04
5:D0112T6<HDYAC50QBU01@@Y1BU5:DHaQBU6<HDaAS65<DHaQBU6<HDYAS65<DHaQBU6<HDYAS60QBU
00T@Y1S65<DHaQC56<HDaAS65<D00aBU00M[FVmkJeY[NV]JKg/00001JgX001I[FP03001[FV]J00A[
FP066<HDaAS65<DHaQC51AS600HHiQcW6>HLiaSV7>L36>H011cW6>HHiQSV1AcW00DM1b487@LQ21d7
008Q2004:DXUBBU::DX3;F/01RUZ;F/YJRe[:DXYJP`YBP<UB@@YBP069DTU:BE99BTYBRE91BU:00<U
BBU:9DT00bDY00PQ:2U::DXYBRe[;H/]Jbf;136<0be[1bU:00<UBBU:9DT012U:0RE900DYBRE9:DXU
BBU:00<UB@8YBP03:FXYBRUZ00<U:@DYBP8UB@8YBP059DTYBRE9:DXUB@03:DX00bE9:DXYBP02:DX2
:FX01Re[:FXYJRUZ;F/YJP<]J`03:FX]Jbe[00LYBP039DTYBRE900`U:@039DTU:BE900@U:@048BPU
:B4X9BT28BP49BT0124X9BTQ:24X1b4800<M1acW7@L021cW0aSV11S600THiQS66>HHiQcW6>HLiaS6
6>H021S60aC51AS600<DaAS65<D01AS600<DaAS66<H00aS600<DYAC55:D00QC500<HaQC55:D00QBU
0aC500<HaQBU5:D01ABU00@HaQC55<DDa@<DY@045<DHaQC55<D35:D00aC55:DDa@0<5:D0112T5:DD
YAS60aC50aS600@DYAC55:DDa@<HaP8Da@036<HDaABU008DY@085<DHaQC56<HDaAS65<DHaP<DY@I[
FP40005[FP0002A_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FP00JeY_
Nf]JJgY[FVmk5<DHaQC56<HDaAS65<D46<H0:1cW6>HLiaSV7>LHaQcW6<HLiaS67>LHaQcW6<HLiaSV
7>LHiR487@LQ21d78@PM1b489DTYBRE9:DXYJRe[:FX]JbUZ;F/YJRe[:DX]JbE90bU:00<UBBU:9DT0
1BU:00dU:BU:9DTYBRE9:DXU:BU:9BTYBRE9:DXUB@03:DX022E9:DXU:BU:8BPU:B4X9BT3:DX0136<
;F/aS2e[136<00<]JbUZ;F/012U:00LUBBU:9DTYBRE9:DXUB@03:DX03bDY:DXU:BU:9DTYBRE9:DXU
:BU:9BT]JbU:;F/YBP039BT4:DX03BDY:DXU:BU:9DTYBRE9:DXUBBU:9DTYBRE900HYBP0P;F/YBRe[
:DX]JbU:;F/YJRe[:FX]JbUZ;F/UBBU:9DTYBRE9:DXUBBU:9BTYBR4X9BTQ:2DY8BPU:B4X9BTQ:08U
:@03:DXU:BU:00@U:@078BPU:B4X9BTQ22DY8@P00bDY00@Q:2DY8@PU:@8Q20087@LQ21d78@PM1b48
7>LQ20DLi`0I7@LLib486<HLiaS67>LHaQSV6<HLiaS67>LHaQcW6>HLiaSV7>LHaQcW6<HLiaS66>H0
11S6014DYAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<DHaQC500<HaP0>5<DHaQBU6<HDYAS65:DH
aQBU6<HDYAS65:DHaPTDY@036<HDYAS6008DY@044:@HaQBU6<H55:D01AS65:DHaQBU6<H00QBU00H@
Y1BU5:DDaABU6<H25:D05A2T5:DDYAS65:DHaQC56<HDaAS65:DHaQBU6<HDaAS65:DHaQBU6<H@Y002
5:D04QS65<DHaQC56<HDaAS65<DDYA2T5:E[FV]jJeY_Nf]JJgX0005_N`005V]J00<006]JJeX016]J
00HHaQC56<HDaAS65<D56<H011SV7>LHiQcW0aSV00DLiaSV6>HHiQcW008HiP@Li`037@LQ21d700<Q
20057@LYBRU::DXUB@03;F/01RUZ;F/YJRe[:DXYJP`YBP039DTYBRE900@YBP069DTU:BE99BTYBRE9
1BU:0bE900@U:B4X9BTU:@<YBP04;F/]Rbe[;H/4<H`3;F/7:DX00bE9:DXUB@05:DX39DT01BU:9DTY
BRE9:DX00RE900@YBRUZ:DXYJP<U:@@YBP8UB@03:DXUBBU:008YBP059DTYBRE9:DXUB@06:DX01Be[
:FXYJRUZ;F/00RUZ0Re[00DYJRe[;F/]JbUZ00LYBP<UB@`U:@039DTU:BE900@U:@038BPU:B4X008Q
:0DU:@<Q:0039BTQ224800DQ20037@LLiad700LLi`8HiP037>LHiQS600<HaP096>HHaQSV6>HLiaSV
7>LHaQSV00PHaP045<DHaQC55<D46<H00aC56<HDa@0;6<H011BU5<DDYAS60aC50aBU00@DaAS65<DD
a@LDY@8Da@036<HDaABU008DY@<Da@076<HDYABU5:DDaABU5<D03QBU00D@Y1C55<DHaQC500<HaP04
5:DDaABU5<D46<H35<D35:D021C56<HDaAS65<DHaQC56<H35:D6JeX10001JeX0000RJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]JJgY[FVmkJeY[NQC56<HDaAS65<D6
6<H02QcW6<HLiaS66>HHaQcW6>HLiaSV11cW00HQ21cW7@LLib487@L28@P00bDY8@PQ:0049BT03BE9
:DXYBRe[:FX]JbUZ;F/YBRe[9DTYBRE900<YBP0G9DTYBRE9:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRE9
:DXUBBU:9DTU:B489BTQ20059BT01B4X9BTYJRe[:FX012e[00Pe[C6<<J`aS2e[:FX]JbE90bU:00HU
:BU:9BTYBRE9:DX29DT01RDY:DXU:BU:9BTYBP8U:@0=8BPYBRE9:DXUBBU:9DTYBRE9:DXUBBU:9DT0
0bU:00TUBBU:9DTYBRE9:DXUBBU:9DT00bU:00PUBBU::DX]JbU:;F/YBRUZ12U:00@UBBe[:FX]J`@Y
BP0=9BTYBRDY:DXUBBU:9DTYBRDY:DXQ:2DY8BP00RDY00TUBBDY:DXU:BU:9BTYBRDY:DX012DY00<Q
:2DY9BT012DY00LQ:2489BTQ224X8@PU:@028@P01Qd78@PQ22487@LQ208Li`0B6>HLiaSV7>LHiQcW
6>HLiaS67>LHaQcW6>HLiaSV7>LHaQcW0QS600HDaAS66<HLiaS67>L46<H00aC56<HDa@056<H031C5
6<HDaAS65<DHaQC56<HDaAS65<DHaP8Da@0:5:DHaQBU6<HDYAS65:DHaQBU6<H25:D01A2T5:DDYABU
4:@00aBU00XDaAS65<DHaQC56<HDaAS65:DHaPDDY@0>6<HDaAS64:@DYA2T5:D@Y1BU4:@DYA2T5:D@
Y0<DY@054:@HaQBU6<H@Y0035:D01QC56<HDaAC55:DHaP8DY@034:@DYABU00<DY@0E5<DHaQC56<HD
YAS65:DHaQBU5<DDYAS65:DHaV]JKg][FV]jJeY_N`00005[NP0000=[FP0000000`000V]J0P0016]J
0P001V]J00<006]JJeX016]J2aS600@HiQcW6>HHiP<HaP/Li`037@LQ224800@Q20@U:@<YBP07:FX]
JbUZ;F/YJRe[:FX00bU:00<UBBU::DX02BU:00`UBBU::DXYBRE9:DXUBBU:9DTYBR4X9BT28BP79BT0
0be[:FX]J`04;F/4<H`00bUZ;F/]J`04:DX39BT012E9:DXUBBU:0bDY0RE900<YBRE99BT00RDY12U:
00DUBBU:9DTYBRE900LYBP059DTYBRE9:DXUB@08:DX00bUZ:DXYJP08:DX00bUZ;F/YJP04:DX00bE9
9BTUB@04:DX00bDY9DTU:@089BT01bE99BTUBBDY9DTU:BE900XU:@lQ20<Li`036>HLiaSV00DLi`<H
iP077>LHiQcW6<HHiQS66>H01AS600<HiQS66<H02QS600DDaAS65<DHaQC500<HaP065<DHaQS66<HD
aAS611BU00<DaAS65<D00QC500<HaQC55:D02ABU00HHaQC56<HDaAS65<D26<H00aC55:DDa@045:D0
0aC56<HDa@075:D00a2T5:DDY@055:D35<D45:D00aS65<DHaP0;5:D01aS65<DHaQBU5<DDYAC500DD
Y@035<DDYF]J00E[FP40005[FP00021_Nf]JJgX006mkJeY[NP00Kg][FP00JeY_N`00JgY[FP00JeY[
NV]JKg][FP00JeY_Nf]JJgY[FVmk5<DHaQC50aS600<DaAS66<H00QS600HLiaS67>LHaQcW6<H26>H0
0acW6>HLi`037>L01Qd77>LQ21cW8@PM1`8Q20038BPQ22DY00@U:@8YBP0=9DT]JbUZ;F/YJRe[:DX]
JbE9:DXUBBU:9DT00bU:01DUBBU:9DTYBRE9:DXUBBU:9DTYBRE9:DXUBBU:9DTYBRE99BTQ22DY8@P0
0bDY00LQ:2DY9BTU:BUZ;F/YJP04;F/01c6/<H`e[C6<;F/YJRe[008YBP0A9DTYBRDY9DTU:BU:9DTY
BRE9:DXU:BE99BTYBRDY:DXQ:0029BT02RU:9DTYBRE9:DXUBBU:9DTYBRE90bU:00dUBBU:9DTYBRE9
:DXUBBU:9DTYBRE9:DXUB@04:DX01Be[:DX]JbU:;F/00RU:00HUBBU::DX]JbUZ;F/4:DX03BDY:DXU
:BU:9DTYBRE9:DXU:BU:8BPU:B4X008U:@09:DXU:BE99BTYBRDY:DXU:BU:008U:@058BPU:BDY9BTQ
:0059BT01R488BPQ22DY8@PQ:0@Q20057@LQ22488@PHiP037>L041SV7>LHiQcW6>HLiaS67>LHaQcW
6>HLiaSV7>LHaQcW11S600@DaAcW6<HLi`@HaP035<DHaQC500DHaP0I5<DHaQC56<HDaAS65<DHaQC5
6<HDaAS65<DHaQBU5<DDYAS65:DHaQBU6<HDYAS64:@00aBU010@Y1BU5:DDYA2T5:DDaAS65<DHaQC5
6<HDaAS65:DHaPDDY@0@6<HDaAS64:@DYA2T5:D@Y1BU4:@DYA2T5:D@Y1BU4:@25:D00aS65:DHaP02
5:D0212T5:DDaAS65<DHaQBU5<D85:D05AC56<HDaAS65:DHaQBU5<DDYAS65:DHaQBU6<I[FV]jJeY_
Nf]JJgX00001Kg/000=[FP05001[FV]JJeX00002JeX01000JeY[FP000V]J00<006]JJeX00f]J1000
0f]J11S600<HiQS66>H011S600DDaAS66<HHaQC5008HaP046>HHaQSV6<H37>L011d78@PM1ad70acW
0b4800DM1b4X8BPU:B4X00@YBP03:FXYBRUZ00<YBP03:FXYBRUZ008YBP049DTYBRE9:DX29DT00bDY
9DTU:@029DT02BU:9DTUBBDY9DTU:BE99BTUB@049BT01B4X9BTQ:2U:9DT00RU:00<]JbUZ;F/022e[
00LYJRe[:FXYBRE9:DXUB@039BT00b4X9BTU:@029BT01RE99BTUBBDY9DTU:@<YBP089DTYBRE99DTU
:BE99BTUB@LU:@07:DXUBBU:9BTUBBDY9DT00bU:00XYJRU::FXYBRUZ:DXYJRU::FXYBP8YJP03;F/Y
JRe[00LYBP079DTU:BE99BTUBBDY9DT01bDY00@YBRE9:DXYBP<U:@049DTYBRE9:DX:9BT012488BPQ
224X1B4800HQ:2488@PM1b487@L97>L26>H01QS66>HHaQSV6<HHiP@Li`035<DHaQC500@HaP047>LH
iQcW6>H35:D26<H02AC56<HDYAC55:DHaQC56<HDa@036<H011C56<HHaQS61ABU00TDaABU5<DDYAC5
5:DHaQC56<H011BU0QS600<DaABU5:D051BU00<HaQC56<H011S611BU00<DaAS65<D00aBU00<@Y1C5
5:D011C500XHaQBU5<DDYAC55:DDaABU6<HDa@<HaP035<DHaQBU00@DY@035<DDYF]J00=[FP<0005[
FP0001e[NV]JKg/006]jJeY_N`00JgY[FP00JeY[NP00Kg][FP00JeY_Nf]JJgY[FP00JeY[NV]JKg][
FV]j00DHaP057>LHaQSV6<HLi`026<H01AC56<HDaAS65<D00QS600<LiaS67>L00QcW00@HiR487>LQ
20@Li`037@LQ21d7008Q20099BTQ22DY9DTYBRE9;F/YBRe[008YBP0N9DTYBRU:;F/YBRe[9DTYBRE9
:DXU:BU:9BTYBRDY:DXU:BU:9BTYBRDY:DXU:BU:9BTYBRDY9DTU:BU:0RDY010Q:2DY9DTYBRE9:DXY
JRe[:FXaS2e[;H/]Jc6<;F/]R`8]J`0R:FX]JbE9:DXUBBU:8BPU:B4X9BTQ:2DY8BPYBRDY:DXU:BU:
9BTYBRE9:DXUBBU:9BTYBRDY:DXU:BU:9BTYBRDY9DT49BT01bE9:DXUBBU:9BTYBRDY00@YBP0W;F/Y
BRe[:DX]JbU:;F/YBRe[:DX]JbUZ;F/YJRU:9DTYBRE9:DXUBBU:9BTYBRDY:DXU:BU:9BTUBBDY:DXU
:BE99BTYBRE9:DXUBBU:00<U:@0<:DXUBBU:9DTYBRDY9DTU:BU:9BTUBB4X0bDY010Q22DY8@PQ:248
9BTQ22DY8@PU:Ad78@PM1b487>LM1`@Li`0a6>HLiaSV7>LHaQcW6<HLiaS67>LHaQcW6>HLiaSV6<HD
aAS65<DHiQS67>LHiQcW6>HLiaBU5<DDYAS65<DHaQC56<HDYAS65<DHaQC56<HDaAS65<DHaQC56<HD
aABU4:@00QBU00XHaQBU6<HDYAS65:DHaQC56<HDa@<DY@054:@HaQC56<H@Y0045:D02AS65:DDaABU
6<HDYAC55:DHaP045:D02a2T5:D@Y1BU5<DHaQC56<HDaAS65<D011BU00HHaQBU6<H@Y1BU4:@25:D0
8QS65:DHaQBU6<HDYAS65:DHaQBU6<HDYAS65<DHaQC56<HDaAS65<DHaQBU5<DDYAS65:DHaV]JKg][
FV]jJeY_N`000F]j0003JeX01@00JeY[FV]J00000V]J00@006]JJeX0009[FP03001[FV]J00=[FP03
001[FV]J00A[FP@HaP036>HHaQSV00HHaP095<DHaQS66<HDaASV6<HHiQS600<Li`<M1`068@PLiacW
7>LQ21d70R4800HU:B4X8BPQ:2U:9DT2:DX00bUZ:DXYJP03:DX00bUZ:DXYJP04:DX39DT01BU:9BTU
BBDY:DX012E900HU:BE99BTUBBDY9DT49BT00b4X9BTQ:003:DX00bE9;F/YJP09;F/01bUZ;F/YJRU:
9DTYBRE900DU:@098BPU:BDY9DTU:BE99BTUBBDY00<YBP<UB@05:DXU:BE99BTUB@079BT01bU:9DTY
BRDY9DTU:BE900<YBP0::FXYBRUZ:DXYJRU::FXYBRUZ:DX2:FX00be[:FX]J`07:DX01bE99BTUBBDY
9DTU:BE900LU:@<YBP089DTU:BDY9BTUBBU:9DTYBPXU:@048@PQ:2488BP58@P01R4X8@PQ21d78@PM
1`LLi`0:6>HLiacW6>HHaQSV6<HHiQS66>H47>L00aC56<HDa@046<H011cW6>HLiaSV0aBU00dDaAS6
6<HHaQBU5<DDYAS65<DHaQC56<HDa@036<H00aC56<HDY@045:D01QC55:DDaABU5<DDY@<HaP@DY@03
5<DHaQS601HDY@LHaP@DY@<Da@065:D@Y1BU5:DDaABU0QC500hHaQC55<DDYAC55:DDaABU5<DDYAS6
5<DHaQC50aS61ABU00<DaABUJeX01F]J0@000F]J000026mkJeX00000Kg][FV]jJeX2000016]jJeY_
Nf]J0P0000e_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_N`056<H01AcW6<HLiaS66>H00QS600DDaAS6
5<DHaQC5008HaP047>LHaQcW6>H27>L00b487>LQ20027>L01ASV7>LM1b487@L00R4800TU:B489BTU
BBU:9DT]JbU:;F/00RU:01hUBBU::DX]JbU:;F/UBBU:9DTYBRDY:DXU:BU:9BTYBRDY:DXU:BU:9BTY
BRDY:DXU:BU:9BTYBRDY9DT29BT0424X9BTUBBU:9DTYBRUZ;F/YJRf;;F/aS2e[;H/]Jc6<0Re[028Y
JRe[9DTYBRE9:DXQ:2DY8BPU:B4X9BTQ:2U:9BTYBRDY:DXU:BU:9DTYBRE9:DXU:BU:9BTYBRDY:DXU
:BE99BTYBP@U:@079DTYBRE9:DXU:BU:9BT012U:028]JbU:;F/YBRe[:DX]JbU:;F/YBRe[:FX]JbUZ
:DXUBBU:9DTYBRE9:DXU:BU:9BTYBRDY:DXU:BU:9BTUBBDY:DXU:@8UB@03:DXUBBU:00<U:@03:DXU
BBU:008UB@069BTYBRDY9DTU:BU:0RDY018Q:2DY8@PU:B489BTQ224X8@PU:B489BTM1b487@LQ21cW
8@P27>L05aSV7>LLiacW6>HLiaS67>LHaQcW6<HLiaS67>LHiQcW6>HHaQC56<HDaAcW6<H00QSV01PL
iaSV7>LDYAS65:DHaQC56<HDaAS65:DHaQC56<HDaAS65<DHaQC56<HDaAS65<D35:D03A2T6<HDYAS6
5:DHaQBU6<HDaAS65<DDYA2T008DY@036<HDaAS6008DY@0<4:@DYABU5<DDYAS65:DDaABU6<HDYAC5
11BU00/@Y1BU4:@DYAC56<HDaAS65<DHaQC500@DY@066<HDYAS64:@DYA2T0QBU01<HaQBU6<HDYAS6
5:DHaQBU6<HDYAS65:DHaQC56<HDaAS65<DHaP025<D03ABU6<HDYAS65:DHaV]JJgY[FVmkJeY[NP00
005_N`005V]J00<006]JJeX0of]JLf]J0@000F]J0000of]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]J001[FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_N`00SF]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg/00001JgX001I[FP03
001[FV]J0?m[FW=[FP40005[FP000?m_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FP00JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgX008e[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j00000Fmk000FJeX00`00JeY[FP3oJeYc
JeX10001JeX0003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeX006]J
JgY[FVmkJeY[NP00Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeX006]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk001[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J
JgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]J
Kg][FV]jJeY_Nf]JJgY[FVmk001[NV]JKg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FP00JeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgX006mkJeY[NV]JKg][FV]jJeY_N`00005[NP005V]J00<006]JJeX016]J00<006]JJeX03f]J
00<006]JJeX03f]J00<006]JJeX03f]J00<006]JJeX03V]J00<006]JJeX03f]J00<006]JJeX03f]J
00<006]JJeX03f]J00<006]JJeX03V]J00<006]JJeX03f]J00<006]JJeX03f]J00<006]JJeX03f]J
00<006]JJeX03f]J00<006]JJeX03V]J00<006]JJeX03f]J00<006]JJeX03f]J00<006]JJeX03f]J
00<006]JJeX03V]J00<006]JJeX03f]J00<006]JJeX03f]J00<006]JJeX03f]J00<006]JJeX01f]J
0@000F]J00005VmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]Jo`00M`00
0Fmk003oJeZ>JeX0003oJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_
Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[
NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk002=JeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]J005[NP00of]JSV]J0000ofmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmk
JeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]j
JeY_Nf]JJgY[FVmkJeY[NP00SF]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][
FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[FVmkJeY[NV]JKg][FV]jJeY_Nf]JJgY[
FP01Kg/00001\
\>"],
  ImageRangeCache->{{{0, 396}, {396, 0}} -> {-8.39969, -7.15074, 0.281453, \
0.281453}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  DensityGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[5]="]
}, Closed]],

Cell["\<\
Read in the FITS header. Note that only FITS cards with keywords are \
converted (eg. comments are omitted). \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(h\)\(=\)\(FitsHeader["\<C:\\Windows\\Desktop\\N2715_lr.fit\>"]\)\(\ \
\)\)\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \("SIMPLE  =                    T                                         \
         \nBITPIX  =                   16                                     \
             \nNAXIS   =                    2                                 \
                 \nNAXIS1  =                  313                             \
                     \nNAXIS2  =                  313                         \
                         \nOBJECT  = 'NGC 2715  '                             \
                             \nBSCALE  =         1.000000E+00                 \
                                 \nBZERO   =         0.000000E+00             \
                                     \nEXPOSURE=         8.000000E+02         \
                                         \nRA      = '09:06:01.3'             \
                                             \nDEC     = '+78:06:18 '         \
                                                 \nTIME    = '04:23:14  '     \
                                                     \nAIRMASS =         \
1.370000E+00                                                  \nHISTORY   \
'Flat-fielded, star-subtracted, calibrated, header-corrected '        \n\
TELESCOP= 'Lowell 1.1'                                                        \
  \nINSTRUME= 'RCA CCD 320'                                                   \
      \nOBSERVER= 'J. A. Tyson'                                               \
          \nAUTHOR  = 'Z. Frei et al. '                                       \
              \nREFERENC= 'AJ, 1995, in press '                               \
                  \nDATE    = '12/07/95  '                                    \
                      \nEQUINOX =         1.950000E+03                        \
                          \nBUNIT   = 'COUNTS    '                            \
                              \nFILTER1 = 'R         '                        \
                                  \nFILENAME= 'n2715_lR.fits'                 \
                                      \nOBSTAPE = 'Tape_1    '                \
                                          \nSKY     =         1.384150E+03    \
                                              \nSKYSIG  =         \
1.891504E+01                                                  \nIMAGE   = \
'n2715r.00 '                                                          \n\
DISKDIR = 'Tape_1/27mar'                                                      \
  \nDECORDER=                    F                                            \
      \nCRVAL1  =         1.365042E+02                                        \
          \nCRVAL2  =         7.810500E+01                                    \
              \nCRPIX1  =         1.640000E+02                                \
                  \nCRPIX2  =         1.570000E+02                            \
                      \nCDELT1  =        -3.750000E-04                        \
                          \nCDELT2  =         3.750000E-04                    \
                              \nCTYPE1  = 'RA---TAN  '                        \
                                  \nCTYPE2  = 'DEC--TAN  '                    \
                                      \nDATAMAX =         5.608000E+03        \
                                          \nDATAMIN =         3.910000E+02    \
                                              \nCCDNUM  =                    \
1                                                  \nXOFFSET =                \
   22                                                  \nYOFFSET =            \
      107                                                  \nG_CENT_X=        \
          163                                                  \nG_CENT_Y=    \
              156                                                  \nSATURAT \
=                    1                                                  \n\
DNAT0_ST=         0.000000E+00                                                \
  \nDNAT0_BV=         9.029303E+10                                            \
      \nB_RC3   =         1.179000E+01                                        \
          \nB-V_RC3 =         5.600000E-01                                    \
              \nPSF_FWHM=         2.819098E+00                                \
                  \nPA_RC3  =         2.200000E+01                            \
                      \nSIZE_RC3=         1.690000E+00                        \
                          \nBOA_RC3 =         4.700000E-01                    \
                              \nVELO_RC3=         1.317833E+03                \
                                  \nTYPE_RC3= '.SXT5..   '                    \
                                      \nDATE-OBS= '28/03/89  '                \
                                          "\)], "Output",
  CellLabel->"Out[6]="]
}, Closed]],

Cell[TextData[{
  "Let's manipulate some header information by converting the header to a \
list of rules. Note that FITS keywords with underscores and minus signs are \
convted to valid ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " symbol names by dropping the offending characters. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(hr = \ FitsHeaderToRules[h]\)], "Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    \({SIMPLE \[Rule] "T", BITPIX \[Rule] 16, NAXIS \[Rule] 2, 
      NAXIS1 \[Rule] 313, NAXIS2 \[Rule] 313, OBJECT \[Rule] "\"NGC 2715 \"", 
      BSCALE \[Rule] 1.`, BZERO \[Rule] 0.`, EXPOSURE \[Rule] 800.`, 
      RA \[Rule] "\"09:06:01.3\"", DEC \[Rule] "\"+78:06:18 \"", 
      TIME \[Rule] "\"04:23:14 \"", AIRMASS \[Rule] 1.37`, 
      TELESCOP \[Rule] "\"Lowell 1.1\"", INSTRUME \[Rule] "\"RCA CCD 320\"", 
      OBSERVER \[Rule] "\"J. A. Tyson\"", 
      AUTHOR \[Rule] "\"Z. Frei et al. \"", 
      REFERENC \[Rule] "\"AJ, 1995, in press \"", 
      DATE \[Rule] "\"12/07/95 \"", EQUINOX \[Rule] 1950.`, 
      BUNIT \[Rule] "\"COUNTS \"", FILTER1 \[Rule] "\"R \"", 
      FILENAME \[Rule] "\"n2715_lR.fits\"", OBSTAPE \[Rule] "\"Tape_1 \"", 
      SKY \[Rule] 1384.15`, SKYSIG \[Rule] 18.91504`, 
      IMAGE \[Rule] "\"n2715r.00 \"", DISKDIR \[Rule] "\"Tape_1/27mar\"", 
      DECORDER \[Rule] "F", CRVAL1 \[Rule] 136.5042`, CRVAL2 \[Rule] 78.105`, 
      CRPIX1 \[Rule] 164.`, CRPIX2 \[Rule] 157.`, 
      CDELT1 \[Rule] \(-0.000375`\), CDELT2 \[Rule] 0.000375`, 
      CTYPE1 \[Rule] "\"RA---TAN \"", CTYPE2 \[Rule] "\"DEC--TAN \"", 
      DATAMAX \[Rule] 5608.`, DATAMIN \[Rule] 391.`, CCDNUM \[Rule] 1, 
      XOFFSET \[Rule] 22, YOFFSET \[Rule] 107, GCENTX \[Rule] 163, 
      GCENTY \[Rule] 156, SATURAT \[Rule] 1, DNAT0ST \[Rule] 0.`, 
      DNAT0BV \[Rule] 9.029303`*^10, BRC3 \[Rule] 11.790000000000001`, 
      BVRC3 \[Rule] 0.5599999999999999`, PSFFWHM \[Rule] 2.819098`, 
      PARC3 \[Rule] 22.`, SIZERC3 \[Rule] 1.69`, 
      BOARC3 \[Rule] 0.47000000000000003`, VELORC3 \[Rule] 1317.833`, 
      TYPERC3 \[Rule] "\".SXT5.. \"", 
      DATEOBS \[Rule] "\"28/03/89 \""}\)], "Output",
  CellLabel->"Out[7]="]
}, Closed]],

Cell["\<\
Extract the right ascension, declination, time, airmass, and datamax \
keywords:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({RA, DEC, DATE, \ TIME, AIRMASS, DATAMAX}\  /. \ hr\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \({"\"09:06:01.3\"", "\"+78:06:18 \"", "\"12/07/95 \"", "\"04:23:14 \"", 
      1.37`, 5608.`}\)], "Output",
  CellLabel->"Out[8]="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Appendix I: ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " source code"
}], "Section"],

Cell["\<\
The following routines need to linked against the freely available CFITSIO \
library. See Appendix II for information on downloading this library (which \
is, incidentally, extremely well designed and trivial to build on every \
machine I've ever tried it on).\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
  "Routines calling ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " "
}], "Subsection"],

Cell[TextData[{
  "All explicit ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " calls are isolated to the following routines."
}], "Text"],

Cell[CellGroupData[{

Cell["fitsread.c", "Subsubsection"],

Cell["\<\
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include \"mathlink.h\"
#include \"fitsread.h\"

/*--------------------------------------------------------------------------*/\

/* return FITS 2D image */

void fitsread( char *name)
{
\tdouble *arr;
\tlong nelem;
\tlong dims[2];
\tlong depth = 2;
\tint nr,nc;
\tint status;             /* non-zero indicates a CFITSIO error */ 
\tlong mlerr;             /* 0 indicates an error for MathLink */
\tchar *better_name;      /* \"\\\\\" converted to \"/\" in directory name */

\tstatus = 0;
\tmlerr = 1;

\t/* Deal with Mathematica's \"\\\\\" directory separator for Windows */
\tbetter_name = convert_to_single_backslash(name);

\tarr = readimage(better_name, &nr, &nc, &status); 
\tif (check_for_errors(status,mlerr)){
\t\tfree(better_name);
\t\treturn;
\t}

\t/* Send the array back to Mathematica */
\tdims[0] = (long) nc;
\tdims[1] = (long) nr;
\tnelem = (long) nr*nc;
\tmlerr = MLPutRealArray(stdlink, arr, dims, NULL, depth);
\tfree(arr);
\tfree(better_name);
\tcheck_for_errors(status,mlerr);
\treturn;
}


/*--------------------------------------------------------------------------*/\

/* return FITS file header */

void fitsheader( char *name)
{
\tchar *header;
\tint status;             /* non-zero indicates a CFITSIO error */ 
\tlong mlerr;             /* 0 indicates an error for MathLink */
\tchar *better_name;      /* \"\\\\\" converted to \"/\" in directory name */

\tstatus = 0;
\tmlerr = 1;

\t/* Deal with Mathematica's \"\\\\\" directory separator for Windows */
\tbetter_name = convert_to_single_backslash(name);

\theader = readheader(better_name, &status); 
\tif (check_for_errors(status,mlerr)){
\t\tfree(better_name);
\t\treturn;
\t}

\t/* Send the string back to Mathematica */
\tmlerr = MLPutString(stdlink, header);
\tfree(header);
\tfree(better_name);
\tcheck_for_errors(status,mlerr);
\treturn;
}



/*--------------------------------------------------------------------------*/\

/* extract filename from a string with region subset suffix attatched */

char *get_filename(char s[])
{
\tint i;
\tchar *new_s;

\tnew_s = malloc(MAX_STRING);
\ti=0;
\twhile (s[i] != '\\0' && s[i] != '[' && i < MAX_STRING){
\t\t\tnew_s[i]=s[i];
\t\t++i;
\t}
\tnew_s[i]='\\0';
\treturn(new_s);
};


/*--------------------------------------------------------------------------*/\

/* convert \"\\\\\" to \"\\\" in a string. */

char* convert_to_single_backslash(char s[])
{
\tint i,j;
\tchar *new_s;

\tnew_s = malloc(MAX_STRING);
\ti=0;
\tj=0;
\twhile (s[i] != '\\0'){
\t\tif ((s[i] == 92) && (s[i+1] == 92) && i < MAX_STRING)
\t\t\t++i;
\t\tnew_s[j]=s[i];
\t\t++i;
\t\t++j;
\t}
\tnew_s[j]='\\0';
\treturn(new_s);
};



/*--------------------------------------------------------------------------*/\

/* General error routine that reports both CFITSIO and MathLink errors and
tries to recover from the latter. If an error was found return 1. */

int check_for_errors(int status, long mlerr)
{
\tchar err_msg[100];
\tint found;
\tfound = 0;
\tif (status>0){
\t\tsprintf(err_msg, \"%s\\\"%.76s\\\"%s\",
\t\t\t\"Message[fits::cfitsio,\",errormessage(status),\"]\");
\t\tfound = 1;
\t}
\tif (status==115){
\t\tsprintf(err_msg, \"%s\",
\t\t\t\"Message[fits::nofile]\");
\t}
\tif(mlerr==0){
\t\tsprintf(err_msg, \"%s\\\"%.76s\\\"%s\",
\t\t\t\"Message[fits::mlink,\",MLErrorMessage(stdlink),\"]\");
\t\tfound = 1;
\t}
\tif (found) {
\t\tMLClearError(stdlink);
\t\tMLNewPacket(stdlink);
\t\tMLEvaluate(stdlink, err_msg);         
\t\tMLNextPacket(stdlink);
\t\tMLNewPacket(stdlink);
\t\tMLPutSymbol(stdlink, \"$Failed\");
\t}
\treturn(found);
}



/*--------------------------------------------------------------------------*/\

#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
\t/* Due to a bug in some standard C libraries that have shipped with
\t * MPW, zero is passed to MLMain below.  (If you build this program
\t * as an MPW tool, you can change the zero to argc.)
\t */
\targc = argc; /* suppress warning */
\treturn MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR \
lpszCmdLine, int nCmdShow)
{
\tchar  buff[512];
\tchar FAR * buff_start = buff;
\tchar FAR * argv[32];
\tchar FAR * FAR * argv_end = argv + 32;

\thinstPrevious = hinstPrevious; /* suppress warning */

\tif( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
\tMLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
\treturn MLMain( argv_end - argv, argv);
}

#else

int main(argc, argv)
\tint argc; char* argv[];
{
\treturn MLMain(argc, argv);
}

#endif
\
\>", "Program"]
}, Closed]],

Cell[CellGroupData[{

Cell["fitsread.h", "Subsubsection"],

Cell["\<\
double *readimage(char *filename, int *nx, int *ny, int *status);
char *readheader(char *filename, int *error_status);
char *errormessage(int status);
char *convert_to_single_backslash(char *s);
char *get_filename(char *s);
int check_for_errors(int status, long mlerr);


#define MAX_STRING 1000
#define CFITSIO_ERROR 100
#define MATHLINK_ERROR 200\
\>", "Program"]
}, Closed]],

Cell[CellGroupData[{

Cell["fitsread.tm", "Subsubsection"],

Cell["\<\


void fitsread P(( char *));

:Begin:
:Function:       fitsread
:Pattern:        FitsRead[file_String]
:Arguments:      { file }
:ArgumentTypes:  { String }
:ReturnType:     Manual
:End:


void fitsheader P(( char *));

:Begin:
:Function:       fitsheader
:Pattern:        FitsHeader[file_String]
:Arguments:      { file }
:ArgumentTypes:  { String }
:ReturnType:     Manual
:End:

\
\>", "Program"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Routines calling the CFITSIO library", "Subsection"],

Cell["\<\
For convenience, all CFITSIO calls are isolated in these two routines\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["readimage.c", "Subsubsection"]], "Text"],

Cell["\<\
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#include \"fitsio.h\"

double *readimage(char *filename, int *nx, int *ny, int *error_status)

{
\tdouble *arr;
    fitsfile *fptr;       /* pointer to the FITS file, defined in fitsio.h */
    long naxes[2], npixels, firstelem;
    int nfound, anynull, status;
    double nullval;
\tchar filestring[FLEN_FILENAME];
\tstrncpy(filestring,filename,FLEN_FILENAME);

    status = 0;
\t*nx = 0;
\t*ny = 0;

    if(fits_open_file(&fptr, filestring, READONLY, &status)){
\t\tfits_close_file(fptr, &status);
\t\t*error_status = status;\t\t
\t\treturn (arr);
\t}
    if(fits_read_keys_lng(fptr, \"NAXIS\", 1, 2, naxes, &nfound, &status)){
\t\tfits_close_file(fptr, &status);
\t\t*error_status = status;
\t\treturn(arr);
\t}

\tfirstelem = 1;
    npixels  = naxes[0] * naxes[1];     /* number of pixels in the image */
    nullval  = 0;         /* don't check for null values in the image */
\tarr = (double *) malloc(sizeof(double)*npixels);
\tif(fits_read_img(fptr,TDOUBLE,firstelem,npixels,&nullval,arr,&anynull,&\
status)){
\t\tfits_close_file(fptr, &status);
\t\t*error_status = status;
\t\treturn(arr);
\t}

    if(fits_close_file(fptr, &status)){
\t\t*error_status = status;
\t\treturn(arr);
\t}

\t*nx = naxes[0];
\t*ny = naxes[1];
\t*error_status = status;

    return(arr);
};


char *errormessage(int status)
{
\tchar *message;
\tmessage=malloc(FLEN_ERRMSG);
\tfits_get_errstatus(status,message);
\treturn(message);
}\
\>", "Program"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["readheader.c", "Subsubsection"]], "Text"],

Cell["\<\
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#include \"fitsio.h\"

char *readheader(char *filename, int *error_status)
{
    fitsfile *fptr;       /* pointer to the FITS file, defined in fitsio.h */
    int status;
\tint nkey,nblank;
\tchar filestring[FLEN_FILENAME];
\tchar *header;
\tchar card[80];
\tint i,j;
\tint index;
\t
\tstrncpy(filestring,filename,FLEN_FILENAME);
    status = 0;
\t
    if(fits_open_file(&fptr, filestring, READONLY, &status)){
\t\tfits_close_file(fptr, &status);
\t\t*error_status = status;\t\t
\t\treturn (header);
\t}
\t
    if(fits_get_hdrspace(fptr, &nkey, &nblank, &status)){
\t\tfits_close_file(fptr, &status);
\t\t*error_status = status;
\t\treturn(header);
\t}
\t
\theader = malloc(nkey*81+1);   /* A FITS line is 80 characters by \
definition. Add 1 for newlines 
\t                                 and the final '\\0' on the last line. */
\tindex = 0;
\tfor(i=0;i<nkey;i++){
\t\tif(fits_read_record(fptr, i+1, card, &status)){
\t\t\tfits_close_file(fptr, &status);
\t\t\t*error_status = status;\t\t
\t\t\treturn (header);
\t\t}
\t\t
\t\tfor(j=0;j<80;j++){
\t\t\tif (card[j]!='\\0'){
\t\t \t\theader[index++] = card[j];
\t\t\t}
\t\t\telse{
\t\t\t\theader[index++] = ' ';
\t\t\t}
\t\t}
\t\theader[index++] = '\\n';
\t}
\theader[81*nkey-1] = '\\0';
\t
    if(fits_close_file(fptr, &status)){
\t\t*error_status = status;
\t\treturn(header);
\t}
\t
\t*error_status = status;
\t
    return(header);
};

\
\>", "Program"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Appendix II: Useful FITS links", "Section"],

Cell["\<\
An excellent site for general FITS information is:  http://fits.cv.nrao.edu

A general introduction to FITS can be found at: \
http://fits.gsfc.nasa.gov/fits_home.html

The CFITSIO library exists for various flavours of Unix, VMS, Windows, and \
MacOS, and is freely available at \
http://legacy.gsfc.nasa.gov/docs/software/fitsio/fitsio.html\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Appendix III: Tips for image processing in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " "
}], "Section"],

Cell[TextData[{
  "Users of other software packages (such as IDL or MATLAB) should remember \
that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " keeps a precise record of th",
  StyleBox["e ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["results",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  " from each step in your calculations in the Out[] symbols. ",
  StyleBox["This means that even if you remove a variable holding an image \
from memory, you're still storing the results you obtained earlier from your \
calculations with that image. ",
    FontSlant->"Italic"],
  " If, as is often the case, the intermediate steps in a calculation are \
also images, they could be taking up a collosal amount of storage.  For \
example, if the variable \"a\" holds an image and you evaluate it a few times \
by hitting \"shift-return\" in succession,  even if you delete the image from \
memory you can always get it back because it's stored in the Out[] variables \
corresponding to your previous steps. \n\nHaving access to the results from \
intermediate steps in long calculations is often a very useful feature, but \
if you're running into memory limitations you may not appreciate it much.  If \
you're willing to forego this feature of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and would prefer to recover the memory being used to store the results \
from previous calculations, then you can use the ",
  StyleBox["CleanSlate",
    FontFamily->"Courier"],
  " package.  This is shipped with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " version 4 (on my system it lives in \"C:\\\\Program Files\\\\Wolfram \
Research\\\\Mathematica\\\\4.0\\\\AddOns\\\\ExtraPackages\\\\Utilities\\\\\
CleanSlate.m\"), although it is not documented in the on-line help.  This \
package has a ",
  StyleBox["ClearInOut[]",
    FontFamily->"Courier"],
  " function that deletes the intermediate values from your calculations \
stored in the Out[] symbols,  while preserving the current values of your \
variables."
}], "Text"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 695}},
CellGrouping->Manual,
WindowSize->{678, 607},
WindowMargins->{{59, Automatic}, {17, Automatic}},
StyleDefinitions -> "ArticleModern.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 110, 4, 113, "Title"],
Cell[1852, 57, 36, 0, 40, "Author"],
Cell[1891, 59, 123, 5, 72, "Address"],
Cell[2017, 66, 1190, 27, 170, "Abstract"],

Cell[CellGroupData[{
Cell[3232, 97, 36, 0, 77, "SectionFirst"],
Cell[3271, 99, 906, 17, 103, "Text"],

Cell[CellGroupData[{
Cell[4202, 120, 27, 0, 60, "Subsection"],
Cell[4232, 122, 1414, 31, 236, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5683, 158, 27, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[5735, 162, 34, 0, 53, "Subsubsection"],
Cell[5772, 164, 466, 11, 123, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6275, 180, 39, 0, 37, "Subsubsection"],
Cell[6317, 182, 1297, 29, 411, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[7651, 216, 65, 0, 37, "Subsubsection"],
Cell[7719, 218, 1712, 35, 390, "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[9492, 260, 29, 0, 43, "Section"],
Cell[9524, 262, 101, 2, 45, "Input"],
Cell[9628, 266, 145, 3, 27, "Text"],
Cell[9776, 271, 119, 3, 45, "Input"],

Cell[CellGroupData[{
Cell[9920, 278, 135, 3, 61, "Input"],
Cell[10058, 283, 470900, 6612, 413, 204640, 3320, "GraphicsData", \
"PostScript", "Graphics"],
Cell[480961, 6897, 161, 4, 44, "Output"]
}, Closed]],
Cell[481137, 6904, 193, 4, 46, "Text"],
Cell[481333, 6910, 145, 4, 45, "Input"],

Cell[CellGroupData[{
Cell[481503, 6918, 135, 3, 61, "Input"],
Cell[481641, 6923, 361161, 4822, 413, 25143, 668, "GraphicsData", \
"PostScript", "Graphics"],
Cell[842805, 11747, 161, 4, 44, "Output"]
}, Closed]],
Cell[842981, 11754, 133, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[843139, 11761, 135, 3, 61, "Input"],
Cell[843277, 11766, 4852, 62, 2191, "Output"]
}, Closed]],
Cell[848144, 11831, 310, 7, 46, "Text"],

Cell[CellGroupData[{
Cell[848479, 11842, 84, 2, 61, "Input"],
Cell[848566, 11846, 1748, 28, 291, "Output"]
}, Closed]],
Cell[850329, 11877, 104, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[850458, 11884, 108, 2, 61, "Input"],
Cell[850569, 11888, 150, 3, 44, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[850768, 11897, 114, 5, 43, "Section"],
Cell[850885, 11904, 284, 5, 46, "Text"],

Cell[CellGroupData[{
Cell[851194, 11913, 111, 5, 60, "Subsection"],
Cell[851308, 11920, 146, 5, 27, "Text"],

Cell[CellGroupData[{
Cell[851479, 11929, 35, 0, 53, "Subsubsection"],
Cell[851517, 11931, 4656, 206, 3393, "Program"]
}, Closed]],

Cell[CellGroupData[{
Cell[856210, 12142, 35, 0, 37, "Subsubsection"],
Cell[856248, 12144, 375, 12, 197, "Program"]
}, Closed]],

Cell[CellGroupData[{
Cell[856660, 12161, 36, 0, 37, "Subsubsection"],
Cell[856699, 12163, 410, 25, 418, "Program"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[857158, 12194, 58, 0, 38, "Subsection"],
Cell[857219, 12196, 93, 2, 27, "Text"],

Cell[CellGroupData[{
Cell[857337, 12202, 64, 0, 39, "Text"],
Cell[857404, 12204, 1506, 64, 1064, "Program"]
}, Closed]],

Cell[CellGroupData[{
Cell[858947, 12273, 65, 0, 39, "Text"],
Cell[859015, 12275, 1479, 67, 1132, "Program"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[860555, 12349, 49, 0, 43, "Section"],
Cell[860607, 12351, 366, 9, 122, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[861010, 12365, 137, 5, 43, "Section"],
Cell[861150, 12372, 2093, 43, 293, "Text"]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

